/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */

package edu.duke.igsp.app.algorithms;

import edu.mit.broad.genome.objects.Template;
import java.util.ArrayList;

/**
 * Specification of a Metric
 */

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: April, 2009
 *
 */
public interface CMetric {

    public static final Type CATEGORICAL = new Type("categorical");
    public static final Type CONTINUOUS = new Type("continuous");
    public static final Type CAT_AND_CONT = new Type("cat_and-cont");

    public String getName();

    public CMetric cloneCMetric();
    /**
     * Contract: small numbers should correspond to small distance between vectors
     * and hence close neighbors.
     * <p/>
     * keys of the params map correspond to one of the constants defined above.
     * Typical example of params are USE_MEDIAN and USE_BIASED
     */
    public double getScore(ArrayList profile, Template template, int code);

    public double getScore(ArrayList x, ArrayList y, int code);

    public Type getType();

    public boolean isCategorical();

    public boolean isContinuous();

    /**
     * Inner class struc
     */
    static class Type {

        private String name;

        Type(String name) {
            this.name = name;
        }

        public boolean equals(Object obj) {
            if (obj instanceof Type) {
                return (((Type) obj).name).equals(name);
            }

            return false;
        }

        public int hashCode() {
            return name.hashCode();
        }
    }
}    // End CMetric

