/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */
package edu.duke.igsp.app.algorithms;

import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.DefaultDataset;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.objects.ScoredDataset;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.SortMode;
import edu.mit.broad.genome.math.Order;
import edu.mit.broad.genome.math.LabelledVectorProcessor;
import edu.mit.broad.genome.alg.gsea.GeneSetCohort;
import edu.mit.broad.genome.alg.gsea.GeneSetCohortGenerator;
import edu.mit.broad.genome.alg.DatasetMetrics;
import edu.mit.broad.genome.alg.gsea.KSCore;
import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;
import edu.stanford.nlp.classify.LogisticClassifier;
import edu.stanford.nlp.ling.BasicDatum;
import edu.duke.igsp.app.tools.ArrayListTool;
import edu.duke.igsp.app.tools.GsaaTool;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;
import java.util.Iterator;
import java.io.*;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: November, 2009
 *
 */
public class SnpSetAlgorithms {

    private static final Logger log = XLogger.getLogger(SnpSetAlgorithms.class);

    public static ArrayList Maximum(final List scoredDataset, final GeneSet[] origGeneSets) {
        ArrayList scores = new ArrayList();

        for (int i = 0; i < origGeneSets.length; i++) {
            ArrayList geneScore = new ArrayList();
            String[] members = origGeneSets[i].getMembersArray();
            for (int j = 0; j < members.length; j++) {
                for (int k = 0; k < scoredDataset.size(); k++) {
                    String name = (String) ((List) scoredDataset.get(k)).get(0);
                    if (name.equalsIgnoreCase(members[j])) {
                        geneScore.add((List) scoredDataset.get(k));
                        break;
                    }
                }
            }

            ArrayList maxScores = maxList(geneScore);
            scores.add(maxScores);
        }

        return scores;

    }

    public static ArrayList Average(final List scoredDataset, final GeneSet[] origGeneSets) {
        ArrayList scores = new ArrayList();

        for (int i = 0; i < origGeneSets.length; i++) {
            ArrayList geneScore = new ArrayList();
            String[] members = origGeneSets[i].getMembersArray();
            for (int j = 0; j < members.length; j++) {
                for (int k = 0; k < scoredDataset.size(); k++) {
                    String name = (String) ((List) scoredDataset.get(k)).get(0);
                    if (name.equalsIgnoreCase(members[j])) {
                        geneScore.add(scoredDataset.get(k));
                        break;
                    }
                }
            }

            ArrayList meanScores = meanList(geneScore);
            scores.add(meanScores);
        }

        return scores;

    }

    public static void reportDataset(final String root, final HashMap snpStruc, final String[] rowNames, final Template template, final Template[] rndTemplates) {
        File f_label = new File(root, "labels.txt");
        if (f_label.exists()) {
            f_label.delete();
        }
        File f_geno = new File(root, "genos.txt");
        if (f_geno.exists()) {
            f_geno.delete();
        }

        File f_gene = new File(root, "genes.txt");
        if (f_gene.exists()) {
            f_gene.delete();
        }
        
        int numCols = ((ArrayList) (((ArrayList) snpStruc.get(rowNames[0])).get(1))).size();

        ArrayList ageno = new ArrayList();
        ArrayList alabel = new ArrayList();
        for (int i = 0; i < numCols + 1; i++) {
            ArrayList a = new ArrayList();
            ageno.add(a);
        }
        for (int i = 0; i < numCols + 1; i++) {
            ArrayList a = new ArrayList();
            alabel.add(a);
        }

        for (int i = 0; i < rowNames.length; i++) {
            String snp_name = (String) (((ArrayList) snpStruc.get(rowNames[i])).get(0));
            ((ArrayList) ageno.get(0)).add(snp_name);
            ArrayList genos = (ArrayList) (((ArrayList) snpStruc.get(rowNames[i])).get(1));
            for (int j = 1; j < numCols+1; j++) {
                String s = (String) genos.get(j - 1);
                ((ArrayList) ageno.get(j)).add(s);
            }
        }

        ArrayListTool.saveMatrixS(ageno, root + "genos.txt");

        for (int n = 0; n < rndTemplates.length + 1; n++) {
            Template currTemp = null;
            if (n == 0) {
                currTemp = template;
            } else {
                currTemp = rndTemplates[n - 1];
            }

            ((ArrayList) alabel.get(0)).add("Label");
            for (int i = 1; i < numCols+1; i++) {
                String label = currTemp.getClassByProfilePos(i - 1).getName();
                if (label.equalsIgnoreCase("case")) {
                    ((ArrayList) alabel.get(i)).add("0");
                } else {
                    ((ArrayList) alabel.get(i)).add("1");
                }
            }
        }

        ArrayListTool.saveMatrixS(alabel, root + "labels.txt");
        ArrayListTool.saveArrayListS(rowNames, root+"genes.txt");

    }

    public static void reportDataset1(final String root, final HashMap snpStruc, final String[] rowNames, final Template template, final Template[] rndTemplates) {
        File f_label = new File(root, "labels.txt");
        if (f_label.exists()) {
            f_label.delete();
        }
        File f_geno = new File(root, "genos.txt");
        if (f_geno.exists()) {
            f_geno.delete();
        }

        File f_gene = new File(root, "genes.txt");
        if (f_gene.exists()) {
            f_gene.delete();
        }
        
        int numCols = 0;

        try {
            File file = new File(root + "genos.txt");
            FileOutputStream fos = new FileOutputStream(file, true);
            PrintWriter pw = new PrintWriter(fos);
            for (int i = 0; i < rowNames.length; i++) {
                String snp_name = (String) (((ArrayList) snpStruc.get(rowNames[i])).get(0));
                pw.print(snp_name + "\t");
                ArrayList genos = (ArrayList) (((ArrayList) snpStruc.get(rowNames[i])).get(1));
                numCols = genos.size();
                for (int j = 0; j < numCols; j++) {
                    String s = (String) genos.get(j);
                    pw.print(s + "\t");
                }
                pw.println();
            }
            pw.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

        try {
            File file = new File(root + "labels.txt");
            FileOutputStream fos = new FileOutputStream(file, true);
            PrintWriter pw = new PrintWriter(fos);
            for (int n = 0; n < rndTemplates.length + 1; n++) {
                Template currTemp = null;
                if (n == 0) {
                    currTemp = template;
                } else {
                    currTemp = rndTemplates[n - 1];
                }

                pw.print("Label\t");
                for (int i = 0; i < numCols; i++) {
                    String label = currTemp.getClassByProfilePos(i).getName();
                    if (label.equalsIgnoreCase("case")) {
                        pw.print("0\t");
                    } else {
                        pw.print("1\t");
                    }
                }
                pw.println();
            }
            pw.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        ArrayListTool.saveArrayListS(rowNames, root+"genes.txt");
    }

    public static ArrayList calcLRCM(final HashMap snpStruc, final String[] rowNames, final Template template, final Template[] rndTemplates) {
        ArrayList lrc_matrix = new ArrayList();
        for (int i = 0; i < rowNames.length; i++) {
            ArrayList lrc = new ArrayList();
            lrc_matrix.add(lrc);
        }

        int numCols = template.getNumItems();
        List feature_list = new ArrayList();
        for (int i = 0; i < numCols; i++) {
            List<Double> features = new ArrayList<Double>();
            for (int j = 0; j < rowNames.length; j++) {
                String geno = (String) ((ArrayList) ((ArrayList) snpStruc.get(rowNames[j])).get(1)).get(i);
                features.add(Double.parseDouble(geno));
            }
            feature_list.add(features);
        }

        for (int n = 0; n < rndTemplates.length + 1; n++) {
            edu.stanford.nlp.classify.Dataset ds = new edu.stanford.nlp.classify.Dataset();
            Template currTemp = null;
            if (n == 0) {
                currTemp = template;
            } else {
                currTemp = rndTemplates[n - 1];
            }

            for (int i = 0; i < numCols; i++) {
                List features = (ArrayList) feature_list.get(i);
                String label = currTemp.getClassByProfilePos(i).getName();
                BasicDatum bd = new BasicDatum(features, label);
                ds.add(bd);
            }

            LogisticClassifier classifier = new LogisticClassifier();
            classifier.train(ds);
            Set keySet = classifier.weightsAsCounter().keySet();
            Iterator it = keySet.iterator();
            int count = 0;
            while (it.hasNext()) {
                Object key = it.next();
                double weight = classifier.weightsAsCounter().getCount(key);
                ((ArrayList) lrc_matrix.get(count)).add(new Float(weight));
                count++;
            }
        }

        return lrc_matrix;
    }

    public static ArrayList meanList(final ArrayList al) {
        int size = ((ArrayList) al.get(0)).size();
        ArrayList means = new ArrayList();

        for (int i = 1; i < size; i++) {
            ArrayList col = new ArrayList();
            for (int j = 0; j < al.size(); j++) {
                col.add(((ArrayList) al.get(j)).get(i));
            }

            means.add(mean(col));
        }

        return means;
    }

    public static ArrayList maxList(final ArrayList al) {
        int size = ((ArrayList) al.get(0)).size();
        ArrayList maxs = new ArrayList();

        for (int i = 1; i < size; i++) {
            ArrayList col = new ArrayList();
            for (int j = 0; j < al.size(); j++) {
                col.add(((ArrayList) al.get(j)).get(i));
            }

            maxs.add(max(col));
        }
        return maxs;
    }

    public static ArrayList sumList(final ArrayList al) {
        int size = ((ArrayList) al.get(0)).size();
        ArrayList sums = new ArrayList();

        for (int i = 1; i < size; i++) {
            ArrayList col = new ArrayList();
            for (int j = 0; j < al.size(); j++) {
                col.add(((ArrayList) al.get(j)).get(i));
            }

            sums.add(sum(col));
        }

        return sums;
    }

    public static float sum(final ArrayList a) {
        float sum = 0;
        for (int i = 0; i < a.size(); i++) {
            float f = Float.parseFloat((String) a.get(i));
            sum += f;
        }
        return sum;
    }

    public static float mean(final ArrayList a) {
        int size = a.size();
        return sum(a) / size;
    }

    public static float max(final ArrayList a) {
        float max = Float.parseFloat((String) a.get(0));
        for (int i = 1; i < a.size(); i++) {
            float f = Float.parseFloat((String) a.get(i));
            if (f > max) {
                max = f;
            }
        }
        return max;
    }
}
