/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.duke.igsp.app.objects;

import edu.mit.broad.genome.TraceUtils;
import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;
import edu.mit.broad.genome.objects.AbstractObject;
import edu.mit.broad.genome.utils.ImmutedException;
import java.util.Vector;
import java.util.ArrayList;

/**
 * all methods that chnage the values of teh matrix sgouldfirst check the imm/mutablity state
 * of the matrix.
 * <p/>
 * * Lost of code/ideas copied form Kenji Hiranabe, Eiwa System Management, Inc.
 * GMatrix part of the javax.vecmath library (unoffical (his) impl)
 * http://java.sun.com/products/java-media/3D/forDevelopers/j3dapi/index.html
 * <p/>
 * as note: made sevral of the row related methods unfinal as need to override
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class Matrix extends AbstractObject {

    /**
     * (javax.vecmath note)
     * Implementation note:
     * The size of the matrix does NOT automaticly grow.
     * It does only when setSize is called. I believe this is the spec
     * and makes less confusion, less user bugs.
     */
    /**
     * @maint IMP IMP IMP IMP IMP
     * If any fields are added to this class, make sure to update
     * the constructors and the set(Matrix m) method
     * Also, make sure to do an immutability check if needed.
     * <p/>
     * The data of the Matrix.(1-D array. The (i,j) element is stored in elementData[i*fColCnt + j])
     * Intentionally made protected so that alg methods and such in this package have direct (aka quiok) access
     * <p/>
     * <p/>
     * The data of the Matrix.(1-D array. The (i,j) element is stored in elementData[i*fColCnt + j])
     * Intentionally made protected so that alg methods and such in this package have direct (aka quiok) access
     */
    /**
     * The data of the Matrix.(1-D array. The (i,j) element is stored in elementData[i*fColCnt + j])
     * Intentionally made protected so that alg methods and such in this package have direct (aka quiok) access
     */
    protected String elementData[];
    /**
     * The number of rows in this matrix.
     */
    private int fRowCnt;
    /**
     * The number of columns in this matrix.
     */
    private int fColCnt;
    /**
     * Flag for whether this Matrix can have its values altered or not
     */
    private boolean fImmuted;
    /**
     * For logging support
     */
    private static final transient Logger klog = XLogger.getLogger(Matrix.class);

    /**
     * Class Constructor.
     * Must call set() after this form of the constructor.
     */
    public Matrix() {
    }

    /**
     * Class Constructor.
     * Constructs an fRowCnt by fColCnt all zero matrix. (as change)
     * Note that even though row and column numbering begins with
     * zero, fRowCnt and fColCnt will be one larger than the maximum
     * possible matrix index values.
     *
     * @param nrows number of rows in this matrix.
     * @param ncols number of columns in this matrix.
     */
    public Matrix(int nrows, int ncols) {
        init(nrows, ncols);
    }
    static int name_cnt;

    private void init(int nrows, int ncols) {
        super.initialize("matrix_" + name_cnt++);
        if (nrows < 0) {
            throw new NegativeArraySizeException(nrows + " < 0");
        }

        if (ncols < 0) {
            throw new NegativeArraySizeException(ncols + " < 0");
        }

        this.fRowCnt = nrows;
        this.fColCnt = ncols;
        this.elementData = new String[fRowCnt * fColCnt];

        setZero();
    }

    public String getQuickInfo() {
        return getNumRow() + " x " + getNumCol();
    }

    /**
     * Class Constructor.
     * Data is copied over from specified float array.
     * Constructs an fRowCnt by fColCnt matrix initialized to the values
     * in the matrix array.  The array values are copied in one row at
     * a time in row major fashion.  The array should be at least
     * fRowCnt*fColCnt in length.
     * Note that even though row and column numbering begins with
     * zero, fRowCnt and fColCnt will be one larger than the maximum
     * possible matrix index values.
     *
     * @param nrows       number of rows in this matrix.
     * @param ncols       number of columns in this matrix.
     * @param elementData a 1D array that specifies a matrix in row major fashion
     */
    public Matrix(int nrows, int ncols, String[] elementData) {

        if (nrows < 0) {
            throw new NegativeArraySizeException(nrows + " < 0");
        }

        if (ncols < 0) {
            throw new NegativeArraySizeException(ncols + " < 0");
        }

        this.fRowCnt = nrows;
        this.fColCnt = ncols;
        this.elementData = new String[fRowCnt * fColCnt];

        set(elementData);
    }

    /**
     * Class Constructor.
     * Constructs a new Matrix and copies the initial values
     * from the parameter matrix.
     *
     * @param matrix the source of the initial values of the new Matrix
     *               <p/>
     *               as added: if shared values, then this matrix shares the elements with the specified amtric
     *               else the array is systemarrycopied
     */
    public Matrix(Matrix matrix, boolean shareValues) {
        init(matrix, shareValues);
    }

    private void init(Matrix matrix, boolean shareValues) {

        if (matrix == null) {
            throw new IllegalArgumentException("Param matrix cannot be null");
        }

        this.fRowCnt = matrix.fRowCnt;
        this.fColCnt = matrix.fColCnt;

        // DONT thats the whole point! (often)
        //this.fImmutable = matrix.fImmutable;

        if (shareValues) {
            this.fImmuted = matrix.fImmuted;
        } else {
            this.fImmuted = false;
        }

        if (shareValues) {
            this.elementData = matrix.elementData;
        } else {
            int newSize = fRowCnt * fColCnt;

            this.elementData = new String[newSize];

            System.arraycopy(matrix.elementData, 0, elementData, 0, newSize);
        }
    }

    /**
     * The cloned matrix is NOT immutable even if the matrix it was cloned from was.
     */
    public Matrix cloneDeep() {
        klog.debug("Making cloneDeep of Matrix");
        TraceUtils.showTrace();
        return new Matrix(this, false);
    }

    /**
     * Fill the matrix with specified value
     *
     * @param val
     */
    public void fill(String val) {
        checkImmutable();
        for (int i = 0; i < elementData.length; i++) {
            elementData[i] = val;
        }
    }

    public ArrayList toList() {
        ArrayList dist=new ArrayList();
        for(int i=0; i<fRowCnt; i++){
            ArrayList row=new ArrayList();
            for(int j=0; j<fColCnt; j++){
                row.add(elementData[i * fColCnt + j]);
            }
            dist.add(row);
        }
        return dist;
    }

    /**
     * Copies a sub-matrix derived from this matrix into the target matrix.
     * The upper left of the sub-matrix is located at (rowSource, colSource);
     * the lower right of the sub-matrix is located at
     * (lastRowSource,lastColSource).  The sub-matrix is copied into the
     * the target matrix starting at (rowDest, colDest).
     *
     * @param rowSource the top-most row of the sub-matrix
     * @param colSource the left-most column of the sub-matrix
     * @param numRow    the number of rows in the sub-matrix
     * @param numCol    the number of columns in the sub-matrix
     * @param rowDest   the top-most row of the position of the copied sub-matrix
     *                  within the target matrix
     * @param colDest   the left-most column of the position of the copied sub-matrix
     *                  within the target matrix
     * @param target    the matrix into which the sub-matrix will be copied
     */
    public void copySubMatrix(int rowSource, int colSource, int numRow, int numCol,
            int rowDest, int colDest, Matrix target) {

        if ((rowSource < 0) || (colSource < 0) || (rowDest < 0) || (colDest < 0)) {
            throw new ArrayIndexOutOfBoundsException("rowSource,colSource,rowDest,colDest < 0.");
        } else if ((fRowCnt < numRow + rowSource) || (fColCnt < numCol + colSource)) {
            throw new ArrayIndexOutOfBoundsException("Source Matrix too small.");
        } else if ((target.fRowCnt < numRow + rowDest) || (target.fColCnt < numCol + colDest)) {
            throw new ArrayIndexOutOfBoundsException("Target Matrix too small.");
        }

        for (int i = 0; i < numRow; i++) {
            for (int j = 0; j < numCol; j++) {
                target.elementData[(i + rowDest) * fColCnt + (j + colDest)] =
                        elementData[(i + rowSource) * fColCnt + (j + colSource)];
            }
        }
    }

    /**
     * Changes the size of this matrix dynamically.  If the size is increased
     * no data values will be lost.  If the size is decreased, only those data
     * values whose matrix positions were eliminated will be lost.
     *
     * @param nrows number of desired rows in this matrix
     * @param ncols number of desired columns in this matrix
     */
    public void setSize(int nrows, int ncols) {

        checkImmutable();

        if ((fRowCnt < 0) || (fColCnt < 0)) {
            throw new NegativeArraySizeException("nrows or ncols < 0");
        }

        int oldfRowCnt = this.fRowCnt;
        int oldfColCnt = this.fColCnt;
        int oldSize = this.fRowCnt * this.fColCnt;

        this.fRowCnt = nrows;
        this.fColCnt = ncols;

        int newSize = fRowCnt * fColCnt;
        String[] oldData = elementData;

        if (oldfColCnt == fColCnt) {

            // no need to reload elements.
            if (fRowCnt <= oldfRowCnt) {
                return;
            }

            // have to allocate memory.
            elementData = new String[newSize];

            // copy the old data.
            System.arraycopy(oldData, 0, elementData, 0, oldSize);

        // no need to pad. 0.0 is automaticly padded.
        // for (int i = oldSize; i < newSize; i++)
        //     elementData[i] = 0.0;
        } else {
            elementData = new String[newSize];

            setZero();

            for (int i = 0; i < oldfRowCnt; i++) {
                System.arraycopy(oldData, i * oldfColCnt, elementData, i * fColCnt, oldfColCnt);
            }
        }
    }

    /**
     * Sets the value of this matrix to the values found in the array parameter.
     * The values are copied in one row at a time, in row major
     * fashion.  The array should be at least equal in length to
     * the number of matrix rows times the number of matrix columns
     * in this matrix.
     *
     * @param matrix the row major source array
     */
    public void set(String matrix[]) {

        checkImmutable();

        int size = fRowCnt * fColCnt;

        System.arraycopy(matrix, 0, elementData, 0, size);
    }

    /**
     * Sets the value of this matrix to the values found in matrix m1.
     *
     * @param m1 the source matrix
     */
    public void set(Matrix m1, boolean sharedvalues) {

        checkImmutable();

        // This implementation is in 'no automatic size grow' policy.
        // When size mismatch, exception will be thrown from the below.
        if ((m1.fRowCnt < fRowCnt) || (m1.fColCnt < fColCnt)) {
            throw new ArrayIndexOutOfBoundsException("m1 smaller than this matrix");
        }

        if (sharedvalues) {
            this.elementData = m1.elementData;
            this.fRowCnt = m1.fRowCnt;
            this.fColCnt = m1.fColCnt;
            this.fImmuted = m1.fImmuted;
        } else {
            System.arraycopy(m1.elementData, 0, elementData, 0, fRowCnt * fColCnt);
        }
    }

    /**
     * Returns the number of rows in this matrix.
     *
     * @return number of rows in this matrix
     */
    public int getNumRow() {
        return fRowCnt;
    }

    /**
     * Returns the number of colmuns in this matrix.
     *
     * @return number of columns in this matrix
     */
    public int getNumCol() {
        return fColCnt;
    }

    /**
     * Retrieves the value at the specified row and column of this matrix.
     *
     * @param row    the row number to be retrieved (zero indexed)
     * @param column the column number to be retrieved (zero indexed)
     * @return the value at the indexed element
     */
    public String getElement(int row, int column) {

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:" + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        if (fColCnt <= column) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " > matrix's fColCnt:" + fColCnt);
        }

        if (column < 0) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " < 0");
        }

        return elementData[row * fColCnt + column];
    }

    /**
     * When you dont care about rows and columns
     * A bit faster than doing the other way
     *
     * @param absoluteIndex
     * @return
     */
    public String getElement(int absoluteIndex) {
        return elementData[absoluteIndex];
    }

    public void setElement(final int absoluteIndex, final String value) {
        checkImmutable();
        elementData[absoluteIndex] = value;
    }

    public int getAbsoluteIndex(final int row, final int column) {
        return row * fColCnt + column;
    }

    public String[] toArray() {
        String[] dest = new String[elementData.length];
        System.arraycopy(elementData, 0, dest, 0, elementData.length);
        return dest;
    }

    public int getAbsoluteSize() {
        return elementData.length;
    }

    /**
     * Modifies the value at the specified row and column of this matrix.
     *
     * @param row    the row number to be modified (zero indexed)
     * @param column the column number to be modified (zero indexed)
     * @param value  the new matrix element value
     */
    public void setElement(int row, int column, String value) {

        checkImmutable();

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:" + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        if (fColCnt <= column) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " > matrix's fColCnt:" + fColCnt);
        }

        if (column < 0) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " < 0");
        }

        elementData[row * fColCnt + column] = value;
    }

    public void setElementByIndexPos(int indexPos, String value) {

        checkImmutable();

        if (elementData.length <= indexPos) {
            throw new ArrayIndexOutOfBoundsException("row:" + indexPos + " > matrix's length:" + elementData.length);
        }

        if (indexPos < 0) {
            throw new ArrayIndexOutOfBoundsException("indexPos:" + indexPos + " < 0");
        }

        elementData[indexPos] = value;
    }

    /**
     * A safe copy is returned.
     */
    public String[] getRow(int row) {

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:" + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        String[] ret = new String[fColCnt];

        System.arraycopy(elementData, row * fColCnt, ret, 0, fColCnt);

        return ret;
    }

    /**
     * Places the values of the specified column into the array parameter.
     *
     * @param col   the target column number
     * @param array the array into which the column values will be placed
     */
    private void _getColumnInPlace(int col, String array[]) {

        if (fColCnt <= col) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > matrix's fColCnt:" + fColCnt);
        }

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        if (array.length < fRowCnt) {
            throw new ArrayIndexOutOfBoundsException("array.length:" + array.length + " < matrix's fRowCnt=" + fRowCnt);
        }

        for (int i = 0; i < fRowCnt; i++) {
            array[i] = elementData[i * fColCnt + col];
        }
    }

    /**
     * Copy the values from the array into the specified row of this
     * matrix.
     *
     * @param row   the row of this matrix into which the array values
     *              will be copied.
     * @param array the source array
     */
    public void setRow(int row, String array[]) {

        checkImmutable();

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:" + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        if (array.length < fColCnt) {
            throw new ArrayIndexOutOfBoundsException("array length:" + array.length + " < matrix's fColCnt=" + fColCnt);
        }

        System.arraycopy(array, 0, elementData, row * fColCnt, fColCnt);
    }

    public void setRow(int row, String value_for_all) {
        for (int c = 0; c < getNumCol(); c++) {
            setElement(row, c, value_for_all);
        }
    }

    /**
     * Data is NOT shared
     * Copy the values from the array into the specified row of this
     * matrix.
     *
     * @param row    the row of this matrix into which the vector values
     *               will be copied.
     * @param vector the source vector
     */
    public void setRow(int row, Vector vector) {

        checkImmutable();

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:" + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        int vecSize = vector.size();

        if (vecSize < fColCnt) {
            throw new ArrayIndexOutOfBoundsException("vector's size:" + vecSize + " < matrix's fColCnt=" + fColCnt);
        }

        for (int i = 0; i < fColCnt; i++) {
            elementData[row * fColCnt + i] = (String) vector.get(i);

        // if may use package friendly accessibility, would do;
        // System.arraycopy(vector.elementData, 0, elementData, row*fColCnt, fColCnt);
        }
    }

    /**
     * Copy the values from the array into the specified column of this
     * matrix.
     *
     * @param col   the column of this matrix into which the array values
     *              will be copied.
     * @param array the source array
     */
    public void setColumn(int col, String array[]) {

        checkImmutable();

        if (fColCnt <= col) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > matrix's fColCnt=" + fColCnt);
        }

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        if (array.length < fRowCnt) {
            throw new ArrayIndexOutOfBoundsException("array length:" + array.length + " < matrix's fRowCnt:" + fRowCnt);
        }

        for (int i = 0; i < fRowCnt; i++) {
            elementData[i * fColCnt + col] = array[i];
        }
    }

    /**
     * Copy the values from the array into the specified column of this
     * matrix.
     *
     * @param col    the column of this matrix into which the vector values
     *               will be copied.
     * @param vector the source vector
     */
    public void setColumn(int col, Vector vector) {
        setColumn(col, vector, true);
    }

    private void setColumn(int col, Vector vector, boolean enforce_max_bound) {

        checkImmutable();

        if (enforce_max_bound && fColCnt <= col) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > matrix's fColCnt=" + fColCnt);
        }

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        int vecSize = vector.size();

        if (enforce_max_bound && vecSize < fRowCnt) {
            throw new ArrayIndexOutOfBoundsException("vector size:" + vecSize + " < matrix's fRowCnt=" + fRowCnt);
        }

        for (int i = 0; i < vector.size(); i++) {
            elementData[i * fColCnt + col] = (String) vector.get(i);
        }
    }

    /**
     * Returns a string that contains the values of this Matrix.
     *
     * @return the String representation
     */
    public String toString() {

        String nl = System.getProperty("line.separator");
        StringBuffer out = new StringBuffer("[");

        out.append(nl);

        for (int i = 0; i < fRowCnt; i++) {
            out.append("  [");

            for (int j = 0; j < fColCnt; j++) {
                if (0 < j) {
                    out.append("\t");
                }

                out.append(elementData[i * fColCnt + j]);
            }

            if (i + 1 < fRowCnt) {
                out.append("]");
                out.append(nl);
            } else {
                out.append("] ]");
            }
        }

        return out.toString();
    }

    /**
     * Returns true if all of the data members of Matrix4d m1 are
     * equal to the corresponding data members in this Matrix4d.
     *
     * @param m1 The matrix with which the comparison is made.
     * @return true or false
     */
    public boolean equals(Matrix m1) {

        if (m1 == null) {
            return false;
        }

        if (m1.fRowCnt != fRowCnt) {
            return false;
        }

        if (m1.fColCnt != fColCnt) {
            return false;
        }

        for (int i = 0; i < fRowCnt; i++) {
            for (int j = 0; j < fColCnt; j++) {
                if (!elementData[i * fColCnt + j].equalsIgnoreCase(m1.elementData[i * fColCnt + j])) {
                    return false;
                }
            }
        }

        return true;
    }

    public void setImmutable() {
        this.fImmuted = true;
    }

    public boolean isImmutable() {
        return fImmuted;
    }

    private void checkImmutable() {

        if (fImmuted) {
            throw new ImmutedException();
        }
    }

    private static void _enforceEqualDimensions(Matrix a, Matrix b) {
        if (a.getNumRow() != b.getNumRow()) {
            throw new IllegalArgumentException("Mismatched matrices: must be of equal row lengths a: " + a.getNumRow() + " b: " + b.getNumRow());
        }

        if (a.getNumCol() != b.getNumCol()) {
            throw new IllegalArgumentException("Mismatched matrices: must be of equal col lengths a: " + a.getNumCol() + " b: " + b.getNumCol());
        }
    }

    private static int enforceSameNumOfCols(Matrix[] matrices) {
        int ncols = matrices[0].getNumCol();

        for (int i = 0; i < matrices.length; i++) {
            if (matrices[i].getNumCol() != ncols) {
                throw new RuntimeException("Unequal ncols: " + ncols + " and " + matrices[i].getNumCol() + "at index: " + i);
            }
        }

        return ncols;
    }

    private static int enforceSameNumOfRows(Matrix[] matrices) {
        int nrows = matrices[0].getNumRow();

        for (int i = 0; i < matrices.length; i++) {
            if (matrices[i].getNumRow() != nrows) {
                throw new RuntimeException("Unequal nrows: " + nrows + " and " + matrices[i].getNumRow() + "at index: " + i);
            }
        }

        return nrows;
    }

    public static int getTotalNumRows(Matrix[] matrices) {
        int nrows = 0;
        for (int i = 0; i < matrices.length; i++) {
            nrows += matrices[i].getNumRow();
        }

        return nrows;
    }

    public static int getTotalNumCols(Matrix[] matrices) {
        int ncols = 0;
        for (int i = 0; i < matrices.length; i++) {
            ncols += matrices[i].getNumCol();
        }

        return ncols;
    }

    /**
     * Sets all the values in this matrix to zero.
     */
    public void setZero() {

        checkImmutable();

        for (int i = 0; i < fRowCnt * fColCnt; i++) {
            elementData[i] = "";
        }
    }

    /**
     * Represents a single matrix cell.
     */
    public static class Cell {

        public int row;
        public int col;
        public String value;
    }    // End Cell
}        // End Matrix
/**
 * Sets this Matrix to the identity matrix.
 */
/*
public void setIdentity() {
checkImmutable();
setZero();
int min = fRowCnt < fColCnt ? fRowCnt : fColCnt;
for (int i = 0; i < min; i++)
elementData[i*fColCnt + i] = (float)1.0;
}
 */

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/



