/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */

package edu.duke.igsp.app.params;

import xtools.api.param.*;
import edu.mit.broad.genome.swing.fields.GComboBoxField;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;
import edu.duke.igsp.app.algorithms.*;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Map;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: April, 2009
 *
 */
public class CMetricParam extends AbstractParam implements ActionListener {

    private GComboBoxField cbOptions;

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     */
    public CMetricParam(boolean reqd) {
        this(new CMetrics.ChiSquare_Allele(), reqd);
    }

    public CMetricParam(CMetric def, boolean reqd) {
        super(METRIC_SNP, METRIC_SNP_ENGLISH, CMetric.class, METRIC_SNP_DESC,
                def, CMetrics.createAllMetrics(), reqd);
    }

    protected CMetricParam(String name, String desc, CMetric def, boolean reqd) {
        super(name, CMetric.class, desc,
                def, CMetrics.createAllMetrics(), reqd);
    }

    public CMetricParam(final CMetric[] metrics, final boolean reqd) {
        this(metrics[0], metrics, reqd);
    }

    public CMetricParam(final CMetric def, final CMetric[] metrics, final boolean reqd) {
        super(METRIC_SNP, METRIC_SNP_ENGLISH, CMetric.class, METRIC_SNP_DESC, def, metrics, reqd);
    }

    public boolean isFileBased() {
        return false;
    }

    public void setValue(Object value) {

        if (value == null) {
            super.setValue(null);
        } else {
            super.setValue(CMetrics.lookupMetric(value));
        }
    }

    public void setValue(CMetric metric) {
        super.setValue(metric);
    }

    public CMetric getCMetric() {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        return (CMetric) val;
    }


    public GFieldPlusChooser getSelectionComponent() {

        if (cbOptions == null) {
            cbOptions = ParamHelper.createActionListenerBoundHintsComboBox(false, this, this);
            ParamHelper.safeSelectValueDefaultByString(cbOptions.getComboBox(), this);
        }

        return cbOptions;

    }

    public void actionPerformed(ActionEvent evt) {
        this.setValue((CMetric) ((JComboBox) cbOptions.getComponent()).getSelectedItem());
    }

    // result hack to allow setting mean / median
    public Map getMetricParams(BooleanParam medianParam) {
        return ParamFactory.getMetricParams(medianParam);
    }

    public Map getMetricParams(boolean useMedian) {
        return ParamFactory.getMetricParams(useMedian);
    }

}    // End class CMetricParam


