/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */
package edu.duke.igsp.app.params;

import java.io.File;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.objects.Template;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: July, 2009
 *
 */
public class CpnTemplateParam extends FileParam{

    public CpnTemplateParam(final boolean reqd) {
        this(CPN_TEMPLATE, CPN_TEMPLATE_ENGLISH, CPN_TEMPLATE_DESC, new File[]{}, reqd);
    }

    public CpnTemplateParam(final String name, final boolean reqd) {
        this(name, CPN_TEMPLATE_ENGLISH, CPN_TEMPLATE_DESC, new File[]{}, reqd);
    }

    public CpnTemplateParam(final String name, final String nameEnglish, final boolean reqd) {
        this(name, nameEnglish, CPN_TEMPLATE_DESC, new File[]{}, reqd);
    }

    public CpnTemplateParam(final String name, final String nameEnglish, final String desc, final boolean reqd) {
        this(name, nameEnglish, desc, new File[]{}, reqd);
    }

    public CpnTemplateParam(final String name, final String nameEnglish, final String desc, final File[] hints, final boolean reqd) {
        super(name, nameEnglish, desc, hints, reqd);
    }

    public Template getTemplate() throws Exception {
        Object val = getValue();

        if (val instanceof Object[] && ((Object[]) val).length == 1) {
            val = ((Object[]) val)[0];
        }

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        } else if (val instanceof Template) {
            return (Template) val;
        } else {
            String path = (String)val;
            int i=path.lastIndexOf(".");
            if(i==-1){
                throw new IllegalArgumentException("Filename extension cannot be null");
            }
            String ext = path.substring(i+1,path.length());
            if(!ext.equalsIgnoreCase("cls")){
                throw new IllegalArgumentException("Filename extension must be .cls. Got: ."+ext);
            }
            return ParserFactory.readTemplate(new File(path), false, false, true);
        }
    }

}


