/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */
package edu.duke.igsp.app.params;

import java.util.ArrayList;
import java.io.*;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: July, 2009
 *
 */
public class GeneListParam extends FileParam {

    public GeneListParam(final boolean reqd) {
        this(GENE_FILE, GENE_FILE_ENGLISH, GENE_FILE_DESC, new File[]{}, reqd);
    }

    public GeneListParam(final String name, final boolean reqd) {
        this(name, GENE_FILE_ENGLISH, GENE_FILE_DESC, new File[]{}, reqd);
    }

    public GeneListParam(final String name, final String nameEnglish, final boolean reqd) {
        this(name, nameEnglish, GENE_FILE_DESC, new File[]{}, reqd);
    }

    public GeneListParam(final String name, final String nameEnglish, final String desc, final boolean reqd) {
        this(name, nameEnglish, desc, new File[]{}, reqd);
    }

    public GeneListParam(final String name, final String nameEnglish, final String desc, final File[] hints, final boolean reqd) {
        super(name, nameEnglish, desc, hints, reqd);
    }

    public String[] getGeneList() throws Exception {

        Object val = getValue();

        if (val instanceof Object[] && ((Object[]) val).length == 1) {
            val = ((Object[]) val)[0];
        }

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        } else {
            String path = (String) val;
            return read(path);
        }
    }

    public String[] read(String path) {
        ArrayList al = new ArrayList();
        try {
            FileReader input = new FileReader(path);
            BufferedReader br = new BufferedReader(input);
            String s = br.readLine();
            while (s != null) {
                al.add(s.trim());
                s = br.readLine();
            }
            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        String[] genes = new String[al.size()];
        for (int i = 0; i < al.size(); i++) {
            genes[i]=(String)al.get(i);
        }
        return genes;
    }
}

