/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */
package edu.duke.igsp.app.params;

import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.duke.igsp.app.parsers.SnpGeneStrucParser;
import edu.duke.igsp.app.parsers.SnpStrucParser;
import edu.duke.igsp.app.struts.SnpStruc;
import edu.duke.igsp.app.algorithms.CMetric;
import java.util.ArrayList;
import java.util.HashMap;
import java.io.File;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: July, 2009
 *
 */
public class SnpDatasetParam extends FileParam {

    public SnpDatasetParam(final boolean reqd) {
        this(SNP_FILE, SNP_FILE_ENGLISH, SNP_FILE_DESC, new File[]{}, reqd);
    }

    public SnpDatasetParam(final String name, final boolean reqd) {
        this(name, SNP_FILE_ENGLISH, SNP_FILE_DESC, new File[]{}, reqd);
    }

    public SnpDatasetParam(final String name, final String nameEnglish, final boolean reqd) {
        this(name, nameEnglish, SNP_FILE_DESC, new File[]{}, reqd);
    }

    public SnpDatasetParam(final String name, final String nameEnglish, final String desc, final boolean reqd) {
        this(name, nameEnglish, desc, new File[]{}, reqd);
    }

    public SnpDatasetParam(final String name, final String nameEnglish, final String desc, final File[] hints, final boolean reqd) {
        super(name, nameEnglish, desc, hints, reqd);
    }

    public PersistentObject getDataset() throws Exception {
        Object val = getValue();

        if (val instanceof Object[] && ((Object[]) val).length == 1) {
            val = ((Object[]) val)[0];
        }

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        } else if (val instanceof PersistentObject) {
            return (PersistentObject) val;
        } else {
            String path = (String) val;
            int i = path.lastIndexOf(".");
            if (i == -1) {
                throw new IllegalArgumentException("Filename extension cannot be null");
            }
            String ext = path.substring(i + 1, path.length());
            if (!ext.equalsIgnoreCase("snp")) {
                throw new IllegalArgumentException("Filename extension must be .snp. Got: ." + ext);
            }
            return ParserFactory.read(new File(path), false);
        }
    }

    public HashMap getSnpGeneStruc(ArrayList genes, int before, int after, int itemsOfTemplate) throws Exception {

        Object val = getValue();

        if (val instanceof Object[] && ((Object[]) val).length == 1) {
            val = ((Object[]) val)[0];
        }

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        } else {
            String path = (String) val;
            int i = path.lastIndexOf(".");
            if (i == -1) {
                throw new IllegalArgumentException("Filename extension cannot be null");
            }
            String ext = path.substring(i + 1, path.length());
            if (!ext.equalsIgnoreCase("snp")) {
                throw new IllegalArgumentException("Filename extension must be .snp. Got: ." + ext);
            }
            SnpGeneStrucParser parser = new SnpGeneStrucParser();
            return parser.parse(genes, before, after, path, itemsOfTemplate);
        }
    }

    public SnpStruc getSnpStruc_phenotype(CMetric cmetric, Template template, Template[] rndTemplates, ArrayList genes, int before, int after, int itemsOfTemplate) throws Exception {
        Object val = getValue();

        if (val instanceof Object[] && ((Object[]) val).length == 1) {
            val = ((Object[]) val)[0];
        }

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        } else {
            String path = (String) val;
            int i = path.lastIndexOf(".");
            if (i == -1) {
                throw new IllegalArgumentException("Filename extension cannot be null");
            }
            String ext = path.substring(i + 1, path.length());
            if (!ext.equalsIgnoreCase("snp")) {
                throw new IllegalArgumentException("Filename extension must be .snp. Got: ." + ext);
            }
            SnpStrucParser parser = new SnpStrucParser();
            return parser.parse_phenotype(cmetric, template, rndTemplates, genes, before, after, path, itemsOfTemplate);
        }
    }

    public SnpStruc getSnpStruc_phenotype_maximum(CMetric cmetric, Template template, Template[] rndTemplates, ArrayList genes, int before, int after, int itemsOfTemplate) throws Exception {
        Object val = getValue();

        if (val instanceof Object[] && ((Object[]) val).length == 1) {
            val = ((Object[]) val)[0];
        }

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        } else {
            String path = (String) val;
            int i = path.lastIndexOf(".");
            if (i == -1) {
                throw new IllegalArgumentException("Filename extension cannot be null");
            }
            String ext = path.substring(i + 1, path.length());
            if (!ext.equalsIgnoreCase("snp")) {
                throw new IllegalArgumentException("Filename extension must be .snp. Got: ." + ext);
            }
            SnpStrucParser parser = new SnpStrucParser();
            return parser.parse_phenotype_maximum(cmetric, template, rndTemplates, genes, before, after, path, itemsOfTemplate);
        }
    }

    public SnpStruc getSnpStruc_phenotype_maximum_pvalue(CMetric cmetric, Template template, Template[] rndTemplates, ArrayList genes, int before, int after, int itemsOfTemplate) throws Exception {
        Object val = getValue();

        if (val instanceof Object[] && ((Object[]) val).length == 1) {
            val = ((Object[]) val)[0];
        }

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        } else {
            String path = (String) val;
            int i = path.lastIndexOf(".");
            if (i == -1) {
                throw new IllegalArgumentException("Filename extension cannot be null");
            }
            String ext = path.substring(i + 1, path.length());
            if (!ext.equalsIgnoreCase("snp")) {
                throw new IllegalArgumentException("Filename extension must be .snp. Got: ." + ext);
            }
            SnpStrucParser parser = new SnpStrucParser();
            return parser.parse_phenotype_maximum_pvalue(cmetric, template, rndTemplates, genes, before, after, path, itemsOfTemplate);
        }
    }

    public SnpStruc getSnpStruc_phenotype_average(CMetric cmetric, Template template, Template[] rndTemplates, ArrayList genes, int before, int after, int itemsOfTemplate) throws Exception {
        Object val = getValue();

        if (val instanceof Object[] && ((Object[]) val).length == 1) {
            val = ((Object[]) val)[0];
        }

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        } else {
            String path = (String) val;
            int i = path.lastIndexOf(".");
            if (i == -1) {
                throw new IllegalArgumentException("Filename extension cannot be null");
            }
            String ext = path.substring(i + 1, path.length());
            if (!ext.equalsIgnoreCase("snp")) {
                throw new IllegalArgumentException("Filename extension must be .snp. Got: ." + ext);
            }
            SnpStrucParser parser = new SnpStrucParser();
            return parser.parse_phenotype_average(cmetric, template, rndTemplates, genes, before, after, path, itemsOfTemplate);
        }
    }

    public SnpStruc getSnpStruc_geneset_maximum(CMetric cmetric, Template template, ArrayList genes, int before, int after, int itemsOfTemplate) throws Exception {
        Object val = getValue();

        if (val instanceof Object[] && ((Object[]) val).length == 1) {
            val = ((Object[]) val)[0];
        }

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        } else {
            String path = (String) val;
            int i = path.lastIndexOf(".");
            if (i == -1) {
                throw new IllegalArgumentException("Filename extension cannot be null");
            }
            String ext = path.substring(i + 1, path.length());
            if (!ext.equalsIgnoreCase("snp")) {
                throw new IllegalArgumentException("Filename extension must be .snp. Got: ." + ext);
            }
            SnpStrucParser parser = new SnpStrucParser();
            return parser.parse_geneset_maximum(cmetric, template, genes, before, after, path, itemsOfTemplate);
        }
    }

    public SnpStruc getSnpStruc_geneset_average(CMetric cmetric, Template template, ArrayList genes, int before, int after, int itemsOfTemplate) throws Exception {
        Object val = getValue();

        if (val instanceof Object[] && ((Object[]) val).length == 1) {
            val = ((Object[]) val)[0];
        }

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        } else {
            String path = (String) val;
            int i = path.lastIndexOf(".");
            if (i == -1) {
                throw new IllegalArgumentException("Filename extension cannot be null");
            }
            String ext = path.substring(i + 1, path.length());
            if (!ext.equalsIgnoreCase("snp")) {
                throw new IllegalArgumentException("Filename extension must be .snp. Got: ." + ext);
            }
            SnpStrucParser parser = new SnpStrucParser();
            return parser.parse_geneset_average(cmetric, template, genes, before, after, path, itemsOfTemplate);
        }
    }

    public String getDatasetName() throws Exception {
        Object val = getValue();

        if (val instanceof Object[] && ((Object[]) val).length == 1) {
            val = ((Object[]) val)[0];
        }

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        } else {
            String path = (String) val;
            int i = path.lastIndexOf(File.separator);
            String filename = "";
            if (i == -1) {
                filename = path;
            }else{
                filename = path.substring(i + 1, path.length());
            }
            return filename;
        }
    }
}
