/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */
package edu.duke.igsp.app.parsers;

import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.parsers.ParserException;
import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;

import edu.duke.igsp.app.objects.GeneBean;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.HashMap;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: July, 2009
 *
 */
/**
 *	#	some	comments
 *	10095	24
 *	rs#	Chr	Pos	Sample1	Sample2	Sample3	Sample4	Sample5	Sample6	Sample7
 *	rs1393064	1	3088998	46.05	82.17	16.87	98.6	141.02	114.32	134.34
 *	rs966321	1	4215064	108.34	59.04	25.61	33.11	42.53	9.12	9.36
 *	rs1599169	1	5034491	7.43	13.94	8.55	21.13	15.09	19.05	16.47
 *	rs580309	1	5211912	11.4	3	3.16	2.34	4.43	1.56	6.04
 *	rs1414379	1	5697261	148.09	317.17	316.66	147.23	125.78	261.39	268.41
 *	rs1890191	1	5809727	8.62	9.12	1572.53	5.91	5.31	11.98	128.77
 */
public class NumGeneStrucParser {

    private static final Logger xlog = XLogger.getLogger(NumGeneStrucParser.class);

    public HashMap parse(ArrayList genes, int before, int after, String mappingMode, String path, int itemsOfTemplate) {
        HashMap dist = new HashMap();
        String filename = "C://copy.txt";

        int total = 0;
        int valid = 0;
        ArrayList cache = new ArrayList();
        try {
            FileReader input = new FileReader(path);
            BufferedReader br = new BufferedReader(input);
            String currLine = nextLine(br);

            // 1st  non-empty, non-comment line is numrows and numcols
            int[] nstuff = ParseUtils.string2ints(currLine, "\t");
            if (nstuff.length != 2) {
                throw new ParserException("file with bad row/col info on line: " + currLine);
            }

            int nrows = nstuff[0];
            int ncols = nstuff[1];

            currLine = nextLine(br);
            List colnames = ParseUtils.string2stringsList(currLine, "\t"); // colnames can have spaces

            colnames.remove(0);            // remove id
            colnames.remove(0);            // remove chr
            colnames.remove(0);            // remove start
            colnames.remove(0);            // remove end

            dist.put("COLNAME", colnames);

            if (colnames.size() != ncols) {
                throw new ParserException("Bad format -- expected ncols from specification on header line: " + ncols + " but found in data: " + colnames.size());
            }

            if (colnames.size() != itemsOfTemplate) {
                throw new ParserException("Bad format -- expected ncols from phenotype labels: " + itemsOfTemplate + " but found in data: " + colnames.size());
            }

            currLine = nextLineTrimless(br);

            for (int i = 0; i < genes.size(); i++) {
                //ArrayList probes = new ArrayList();


                String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
                int geneChr = ((GeneBean) genes.get(i)).getChromosome();
                int start = ((GeneBean) genes.get(i)).getStart() - before;
                if (start < 0) {
                    start = 0;
                }
                int end = ((GeneBean) genes.get(i)).getEnd();
                if (end == 0) {
                    continue;
                } else {
                    end = end + after;
                }

                ArrayList probes = new ArrayList(cache);
                if (cache != null && cache.size() != 0) {
                    cache.clear();
                }
                if (probes != null && probes.size() != 0) {
                    for (int j = 0; j < probes.size(); j++) {
                        int start_c = Integer.parseInt(((ArrayList) probes.get(j)).get(2).toString().trim());
                        if (start <= start_c) {
                            ArrayList probe_c = (ArrayList)((ArrayList) probes.get(j)).clone();
                            cache.add(probe_c);
                            ((ArrayList) probes.get(j)).remove(0);
                            ((ArrayList) probes.get(j)).remove(0);
                            ((ArrayList) probes.get(j)).remove(0);
                            ((ArrayList) probes.get(j)).remove(0);
                        } else {
                            probes.remove(j);
                            j--;
                        }

                    }
                }

                /*int start_next = 0;
                if (i != genes.size() - 1) {
                start_next = ((GeneBean) genes.get(i + 1)).getStart() - before;
                if (start_next < 0) {
                start_next = 0;
                }
                }*/

                while ((currLine != null) && (currLine.length() != 0)) {
                    total++;
                    ArrayList fields = string2stringsV2(currLine, ncols + 4); // spaces allowed in name & desc field so DONT tokenize them

                    if (fields.size() != ncols + 4) {
                        //System.out.println(">> " + fields);
                        throw new ParserException("Bad format - expect ncols: " + (ncols + 4) + " but found: " + fields.size() + " on line >" + currLine + "<\nIf this dataset has missing values, use ImputeDataset to fill these in before importing as a Dataset");
                    }
                    int probeChr = Integer.parseInt(fields.get(1).toString().trim());
                    int probeStart = Integer.parseInt(fields.get(2).toString().trim());
                    if (geneChr != probeChr) {
                        if (geneChr < probeChr) {
                            if (cache != null && cache.size() != 0) {
                                cache.clear();
                            }
                            total--;
                            break;
                        } else {
                            if (cache != null && cache.size() != 0) {
                                cache.clear();
                            }
                            if (probes != null && probes.size() != 0) {
                                probes.clear();
                            }
                            currLine = nextLineTrimless(br);
                            continue;
                        }
                    }

                    if (start <= probeStart && probeStart <= end) {
                        cache.add(fields.clone());

                        fields.remove(0);
                        fields.remove(0);
                        fields.remove(0);
                        fields.remove(0);
                        probes.add(fields);
                        valid++;
                    }

                    if (end < probeStart) {
                        total--;
                        break;
                    }

                    currLine = nextLineTrimless(br);
                }

                if (probes != null && probes.size() != 0) {
                    //ArrayListTool.saveStringA(geneSymbol+": "+start+"-"+end, filename);
                    //ArrayListTool.saveMatrixSA(cache, filename);
                    //ArrayListTool.saveMatrixSA(probes, filename);
                    if (mappingMode.equalsIgnoreCase("Sum")) {
                        dist.put(geneSymbol, sumList(probes));
                    } else if (mappingMode.equalsIgnoreCase("Mean")) {
                        dist.put(geneSymbol, meanList(probes));
                    } else {
                        throw new IllegalArgumentException("Mapping mode not supported. Got: " + mappingMode);
                    }
                }
            }

            xlog.info("There are " + total + " probes in the original dataset");
            xlog.info("Mapping resulted in filtering out " + (total - valid) + "/" + total + " probes, the remainning " + valid + " probes were used in the analysis");
            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return dist;
    }

    protected String nextLine(final BufferedReader bin) throws IOException {
        String currLine = bin.readLine();
        if (currLine == null) {
            return null;
        }

        currLine = currLine.trim();

        while ((currLine != null) && ((currLine.length() == 0) || (currLine.startsWith(Constants.COMMENT_CHAR)))) {
            // System.out.println(">>> " + currLine);
            currLine = bin.readLine();
            if (currLine != null) {
                currLine = currLine.trim();
            }
        }
        return currLine;
    }

    protected String nextLineTrimless(BufferedReader bin) throws IOException {
        String currLine = bin.readLine();
        if (currLine == null) {
            return null;
        }

        //currLine = currLine.trim();

        while ((currLine != null) && ((currLine.length() == 0) || (currLine.startsWith(Constants.COMMENT_CHAR)))) {
            currLine = bin.readLine();
        }

        return currLine;
    }

    public static ArrayList meanList(final ArrayList al) {
        enforceEqualSize(al);
        int size = ((ArrayList) al.get(0)).size();
        ArrayList means = new ArrayList();

        for (int i = 0; i < size; i++) {
            ArrayList col = new ArrayList();
            for (int j = 0; j < al.size(); j++) {
                col.add(((ArrayList) al.get(j)).get(i));
            }

            means.add(mean(col));
        }

        return means;
    }

    public static ArrayList sumList(final ArrayList al) {
        enforceEqualSize(al);
        int size = ((ArrayList) al.get(0)).size();
        ArrayList sums = new ArrayList();

        for (int i = 0; i < size; i++) {
            ArrayList col = new ArrayList();
            for (int j = 0; j < al.size(); j++) {
                col.add(((ArrayList) al.get(j)).get(i));
            }

            sums.add(sum(col));
        }

        return sums;
    }

    public static float sum(final ArrayList a) {
        float sum = 0;
        for (int i = 0; i < a.size(); i++) {
            float f = Float.parseFloat((String)a.get(i));
            sum += f;
        }
        return sum;
    }

    public static float mean(final ArrayList a) {
        int size = a.size();
        return sum(a) / size;
    }

    private static void enforceEqualSize(final ArrayList al) {
        if (al.size() == 0) {
            return;
        }

        int size = ((ArrayList) al.get(0)).size();
        for (int i = 0; i < al.size(); i++) {
            if (((ArrayList) al.get(i)).size() != size) {
                throw new IllegalArgumentException("Vector lengths not equal first=" + size + " and y=" + ((ArrayList) al.get(i)).size() + " at index " + i);
            }
        }
    }

    protected ArrayList string2stringsV2(String s, int expectedLen) throws IllegalArgumentException {

        if (null == s) {
            throw new NullPointerException("Cannot work on null String");
        }

        String delim = "\t"; // @note, always
        StringTokenizer tok = new StringTokenizer(s, delim, true); // note including the delim in rets
        ArrayList ret = new ArrayList();
        String prev = null;
        //System.out.println("PARSING>" + s + "<"  + tok.countTokens());

        int cnt = 0;
        while (tok.hasMoreTokens()) {
            final String curr = tok.nextToken(); // dont trim as curr might be a tab!

            if (cnt == 0) { // the first field
                ret.add(curr.trim()); // always add it, empty or not
            } else {
                if (curr.equals(delim)) {
                    if (prev.equals(delim)) { // 2 consecutive tabs
                        ret.add(""); //empty field
                    } else { // omit because its _the delim_
                    }
                } else {
                    ret.add(curr.trim()); // a real word, ok to trim. Then add
                }
            }

            prev = curr;
            cnt++;
        }

        if (ret.size() == expectedLen) {
            return ret;
        } else if (ret.size() < expectedLen) {
            /// fill out whatever's left with empty
            for (int i = ret.size(); i < expectedLen; i++) {
                ret.add("");
            }
            return ret;

        } else {
            // @note added Nov 28, 2005
            // delete any extra tabs (ret.size() > expectedLen
            ArrayList real_ret = new ArrayList();
            for (int i = 0; i < ret.size(); i++) {
                if (i < expectedLen) {
                    real_ret.add(ret.get(i));
                } else {
                    // dont add empty
                    Object obj = ret.get(i);
                    if (obj == null || obj.toString().trim().length() == 0) {
                        // dont add
                    } else {
                        real_ret.add(obj); // cant do anything  might be a genuine format error
                    }
                }
            }

            return real_ret;
        }
    }
}
