/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */
package edu.duke.igsp.app.parsers;

import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.FSet;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.parsers.ParserException;
import edu.mit.broad.genome.XLogger;
import org.apache.commons.math.stat.correlation.PearsonsCorrelation;
import org.apache.log4j.Logger;

import edu.duke.igsp.app.objects.GeneBean;
import edu.duke.igsp.app.algorithms.*;
import edu.duke.igsp.app.struts.SnpStruc;
import edu.duke.igsp.app.algorithms.SnpSetAlgorithms;
import edu.duke.igsp.app.tools.GsaaTool;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.HashMap;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: July, 2009
 *
 */
/**
 * # some comments
 * 10095	24
 * rs#	       Chr	Pos	    073-127_Call	001-101_Call	023-117_Call	023-117re_Call	090-323_Call	109-319_Call	095--325_Call	121-329_Call	127-137_Call	143-339_Call	142-343_Call	148-341_Call	146-139_Call	072-128_Call	113-332_Call	040-302_Call	046-123_Call	097-320_Call	039-303_Call	050-308_Call	058-309_Call	061-311_Call	155-348_Call	014-109_Call
 * rs1393064   1	3088998	CC	GG	CC	CC	CC	CC	CC	CC	NN	CC	CG	CC	CC	CC	CC	CG	NN	CG	CC	CC	CG	CG	CG	CC
 * rs966321	   1	4215064	AC	AA	CC	CC	AC	AA	CC	AC	AA	CC	AA	AC	AC	AC	AC	CC	NN	AA	AA	AA	AA	AC	AC	AC
 * rs1599169   1	5034491	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG	GG
 * rs580309	   1	5211912	CT	CT	NN	CC	CC	NN	NN	NN	NN	CT	NN	NN	NN	CC	CT	CT	NN	CT	TT	CC	CC	CC	CC	NN
 * rs1414379   1	5697261	CT	TT	TT	TT	TT	CT	CC	TT	TT	CT	TT	CT	CT	TT	TT	CT	CT	CT	CT	TT	CT	TT	TT	NN
 * rs1890191   1	5809727	AA	GG	AG	AG	AA	AG	AA	AG	AG	AA	AA	AA	AA	AG	AA	AA	AG	AA	AG	GG	AG	AG	AG	GG
 */
public class SnpStrucParser {

    private static final Logger xlog = XLogger.getLogger(NumGeneStrucParser.class);

    //for printing the mapping information
    public SnpStruc parse_phenotype_map(CMetric cmetric, Template template, Template[] rndTemplates, ArrayList genes, int before, int after, String path, int itemsOfTemplate) {
        int code = 0;
        int flag = 0;

        List snpSets = new ArrayList();
        List dels = new ArrayList();
        //DoubleElement[] datasetSynchedDels = new DoubleElement[snpGeneStruc.size()];

        int total = 0;
        int valid = 0;
        ArrayList cache = new ArrayList();
        try {
            FileReader input = new FileReader(path);
            BufferedReader br = new BufferedReader(input);
            String currLine = nextLine(br);

            // 1st  non-empty, non-comment line is numrows and numcols
            int[] nstuff = ParseUtils.string2ints(currLine, "\t");
            if (nstuff.length != 2) {
                throw new ParserException("file with bad row/col info on line: " + currLine);
            }

            int nrows = nstuff[0];
            int ncols = nstuff[1];

            currLine = nextLine(br);
            List colnames = ParseUtils.string2stringsList(currLine, "\t"); // colnames can have spaces

            colnames.remove(0);            // remove rs#
            colnames.remove(0);            // remove chr
            colnames.remove(0);            // remove pos

            if (colnames.size() != ncols) {
                throw new ParserException("Bad format -- expected ncols from specification on header line: " + ncols + " but found in data: " + colnames.size());
            }

            if (colnames.size() != itemsOfTemplate) {
                throw new ParserException("Bad format -- expected ncols from phenotype labels: " + itemsOfTemplate + " but found in data: " + colnames.size());
            }

            currLine = nextLineTrimless(br);

            //*********************************************
            String filename = "//nfs//furey_sata//fureylab//result_qing//wtccc//mapping.txt";
            FileWriter output = new FileWriter(filename);
            BufferedWriter bw = new BufferedWriter(output);
            //*********************************************

            //String[] atp = new String[]{"ATP6V1A", "ATP5O", "ATP6V1C1", "ATP6V1E1", "ATP6V1B1", "ATP6V1H", "ATP6V0B", "ATP6V1F", "ATP6V1D", "ATP6V0A1", "ATP6V1B2", "ATP6V0D1", "ATP6V0C", "ATP6V1G2", "ATP6AP1", "ATP6V0E", "ATP6V1G1", "ATP6V0A4"};
            for (int i = 0; i < genes.size(); i++) {
                ArrayList duplicates = new ArrayList();
                ArrayList snpNames = new ArrayList();
                String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
                int geneChr = ((GeneBean) genes.get(i)).getChromosome();
                String geneDes = ((GeneBean) genes.get(i)).getDescription();
                int start = ((GeneBean) genes.get(i)).getStart() - before;
                if (start < 0) {
                    start = 0;
                }
                int end = ((GeneBean) genes.get(i)).getEnd();
                if (end == 0) {
                    continue;
                } else {
                    end = end + after;
                }

                //*********************************************
                StringTokenizer st = new StringTokenizer(geneSymbol, "///");
                bw.write((st.nextToken()).trim() + "\t");
                //bw.write(geneSymbol + ": " + start + "-" + end);
                //bw.newLine();
                //*********************************************

                ArrayList probes = new ArrayList(cache);
                if (cache != null && cache.size() != 0) {
                    cache.clear();
                }
                if (probes != null && probes.size() != 0) {
                    for (int j = 0; j < probes.size(); j++) {
                        int pos_c = Integer.parseInt(((ArrayList) probes.get(j)).get(2).toString().trim());
                        if (start <= pos_c) {
                            ArrayList probe_c = (ArrayList) ((ArrayList) probes.get(j)).clone();
                            cache.add(probe_c);
                            String s = (String) ((ArrayList) probes.get(j)).remove(0);
                            snpNames.add(s);
                            duplicates.add(s);
                            ((ArrayList) probes.get(j)).remove(0);
                            ((ArrayList) probes.get(j)).remove(0);
                        } else {
                            probes.remove(j);
                            j--;
                        }

                    }
                }

                while ((currLine != null) && (currLine.length() != 0)) {
                    total++;
                    ArrayList fields = string2stringsV2(currLine, ncols + 3); // spaces allowed in name & desc field so DONT tokenize them

                    if (fields.size() != ncols + 3) {
                        //System.out.println(">> " + fields);
                        throw new ParserException("Bad format - expect ncols: " + (ncols + 3) + " but found: " + fields.size() + " on line >" + currLine + "<\nIf this dataset has missing values, use ImputeDataset to fill these in before importing as a Dataset");
                    }
                    int probeChr = Integer.parseInt(fields.get(1).toString().trim());
                    int probePos = Integer.parseInt(fields.get(2).toString().trim());
                    if (geneChr != probeChr) {
                        if (geneChr < probeChr) {
                            if (cache != null && cache.size() != 0) {
                                //*********************************************
                                for (int j = 0; j < cache.size(); j++) {
                                    bw.write((String) ((ArrayList) cache.get(j)).get(0) + "\t");
                                    /*for (int k = 0; k < ((ArrayList) cache.get(j)).size(); k++) {
                                    bw.write((String) ((ArrayList) cache.get(j)).get(k) + "\t");
                                    }
                                    bw.newLine();*/
                                }
                                bw.newLine();
                                //*********************************************
                                cache.clear();
                            }
                            total--;
                            break;
                        } else {
                            if (cache != null && cache.size() != 0) {
                                cache.clear();
                            }
                            if (probes != null && probes.size() != 0) {
                                probes.clear();
                            }
                            if (snpNames != null && snpNames.size() != 0) {
                                snpNames.clear();
                            }
                            if (duplicates != null && duplicates.size() != 0) {
                                duplicates.clear();
                            }
                            currLine = nextLineTrimless(br);
                            continue;
                        }
                    }

                    if (start <= probePos && probePos <= end) {
                        cache.add(fields.clone());

                        snpNames.add(fields.remove(0));
                        fields.remove(0);
                        fields.remove(0);
                        probes.add(fields);
                    }

                    if (end < probePos) {
                        total--;
                        break;
                    }

                    currLine = nextLineTrimless(br);
                }

                //removeHomo(probes, snpNames);

                if (probes != null && probes.size() != 0) {
                    enforceEqualSize(probes);
                    if (flag == 0) {
                        code = getCode(probes);
                        flag++;
                    }

                    //********************************************
                    if (probes.size() != cache.size()) {
                        //throw new IllegalArgumentException("Size of probes is not equal to the size of cache");
                        /*System.out.println(geneSymbol + ": " + start + "-" + end);
                        System.out.println("cache");
                        for (int j = 0; j < cache.size(); j++) {
                        for (int k = 0; k < ((ArrayList) cache.get(j)).size(); k++) {
                        System.out.print((String) ((ArrayList) cache.get(j)).get(k) + "\t");
                        }
                        System.out.println();
                        }
                        System.out.println("probes");
                        for (int j = 0; j < probes.size(); j++) {
                        for (int k = 0; k < ((ArrayList) probes.get(j)).size(); k++) {
                        System.out.print((String) ((ArrayList) probes.get(j)).get(k) + "\t");
                        }
                        System.out.println();
                        }*/
                    } else {
                        for (int j = 0; j < cache.size(); j++) {
                            bw.write((String) ((ArrayList) cache.get(j)).get(0) + "\t");
                            /*for (int k = 0; k < ((ArrayList) cache.get(j)).size(); k++) {
                            bw.write((String) ((ArrayList) cache.get(j)).get(k) + "\t");
                            }
                            bw.newLine();*/
                        }
                        bw.newLine();
                    }
                    //*********************************************

                    for (int j = 0; j < probes.size(); j++) {
                        ArrayList del = new ArrayList();
                        double dist_snp = cmetric.getScore((ArrayList) probes.get(j), template, code);
                        String snpName = (String) snpNames.get(j);
                        int f = 0;
                        for (int k = 0; k < duplicates.size(); k++) {
                            if (snpName.equalsIgnoreCase((String) duplicates.get(k))) {
                                f++;
                                break;
                            }
                        }
                        if (f == 0) {
                            del.add(snpNames.get(j));
                            del.add((new Double(dist_snp)).toString());
                            for (int k = 0; k < rndTemplates.length; k++) {
                                double dist_snp_rnd = cmetric.getScore((ArrayList) probes.get(j), rndTemplates[k], code);
                                del.add((new Double(dist_snp_rnd)).toString());
                            }
                            dels.add(del);
                            valid++;
                        }
                    }
                } else {
                    bw.newLine();
                }

                if (snpNames != null && snpNames.size() != 0) {
                    //GeneSet snpSet = new FSet(geneSymbol.toUpperCase(), geneDes, snpNames, true);
                    GeneSet snpSet = new FSet(geneSymbol, geneDes, snpNames, true);
                    snpSets.add(snpSet);
                }
            }

            //*********************************************
            bw.close();
            //*********************************************

            xlog.info("There are " + total + " probes in the original dataset");
            xlog.info("Mapping resulted in filtering out " + (total - valid) + "/" + total + " probes, the remainning " + valid + " probes were used in the analysis");
            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

        SnpStruc sstruc = new SnpStruc(dels, (GeneSet[]) snpSets.toArray(new GeneSet[snpSets.size()]));
        return sstruc;
    }

    //apply to ssea, without duplicates, phenotype permutation, not consider covariance structure.
    public SnpStruc parse_phenotype(CMetric cmetric, Template template, Template[] rndTemplates, ArrayList genes, int before, int after, String path, int itemsOfTemplate) {
        int code = 0;
        int flag = 0;

        List snpSets = new ArrayList();
        List dels = new ArrayList();
        //DoubleElement[] datasetSynchedDels = new DoubleElement[snpGeneStruc.size()];
        //String filename = "C://data//mapping.txt";

        int total = 0;
        int valid = 0;
        ArrayList cache = new ArrayList();
        try {
            FileReader input = new FileReader(path);
            BufferedReader br = new BufferedReader(input);
            String currLine = nextLine(br);

            // 1st  non-empty, non-comment line is numrows and numcols
            int[] nstuff = ParseUtils.string2ints(currLine, "\t");
            if (nstuff.length != 2) {
                throw new ParserException("file with bad row/col info on line: " + currLine);
            }

            int nrows = nstuff[0];
            int ncols = nstuff[1];

            currLine = nextLine(br);
            List colnames = ParseUtils.string2stringsList(currLine, "\t"); // colnames can have spaces

            colnames.remove(0);            // remove rs#
            colnames.remove(0);            // remove chr
            colnames.remove(0);            // remove pos

            if (colnames.size() != ncols) {
                throw new ParserException("Bad format -- expected ncols from specification on header line: " + ncols + " but found in data: " + colnames.size());
            }

            if (colnames.size() != itemsOfTemplate) {
                throw new ParserException("Bad format -- expected ncols from phenotype labels: " + itemsOfTemplate + " but found in data: " + colnames.size());
            }

            currLine = nextLineTrimless(br);

            //String[] atp = new String[]{"ATP6V1A", "ATP5O", "ATP6V1C1", "ATP6V1E1", "ATP6V1B1", "ATP6V1H", "ATP6V0B", "ATP6V1F", "ATP6V1D", "ATP6V0A1", "ATP6V1B2", "ATP6V0D1", "ATP6V0C", "ATP6V1G2", "ATP6AP1", "ATP6V0E", "ATP6V1G1", "ATP6V0A4"};
            for (int i = 0; i < genes.size(); i++) {
                ArrayList duplicates = new ArrayList();
                ArrayList snpNames = new ArrayList();
                String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
                int geneChr = ((GeneBean) genes.get(i)).getChromosome();
                String geneDes = ((GeneBean) genes.get(i)).getDescription();
                int start = ((GeneBean) genes.get(i)).getStart() - before;
                if (start < 0) {
                    start = 0;
                }
                int end = ((GeneBean) genes.get(i)).getEnd();
                if (end == 0) {
                    continue;
                } else {
                    end = end + after;
                }



                ArrayList probes = new ArrayList(cache);
                if (cache != null && cache.size() != 0) {
                    cache.clear();
                }
                if (probes != null && probes.size() != 0) {
                    for (int j = 0; j < probes.size(); j++) {
                        int pos_c = Integer.parseInt(((ArrayList) probes.get(j)).get(2).toString().trim());
                        if (start <= pos_c) {
                            ArrayList probe_c = (ArrayList) ((ArrayList) probes.get(j)).clone();
                            cache.add(probe_c);
                            String s = (String) ((ArrayList) probes.get(j)).remove(0);
                            snpNames.add(s);
                            duplicates.add(s);
                            ((ArrayList) probes.get(j)).remove(0);
                            ((ArrayList) probes.get(j)).remove(0);
                        } else {
                            probes.remove(j);
                            j--;
                        }

                    }
                }

                while ((currLine != null) && (currLine.length() != 0)) {
                    total++;
                    ArrayList fields = string2stringsV2(currLine, ncols + 3); // spaces allowed in name & desc field so DONT tokenize them

                    if (fields.size() != ncols + 3) {
                        //System.out.println(">> " + fields);
                        throw new ParserException("Bad format - expect ncols: " + (ncols + 3) + " but found: " + fields.size() + " on line >" + currLine + "<\nIf this dataset has missing values, use ImputeDataset to fill these in before importing as a Dataset");
                    }
                    int probeChr = Integer.parseInt(fields.get(1).toString().trim());
                    int probePos = Integer.parseInt(fields.get(2).toString().trim());
                    if (geneChr != probeChr) {
                        if (geneChr < probeChr) {
                            if (cache != null && cache.size() != 0) {
                                cache.clear();
                            }
                            total--;
                            break;
                        } else {
                            if (cache != null && cache.size() != 0) {
                                cache.clear();
                            }
                            if (probes != null && probes.size() != 0) {
                                probes.clear();
                            }
                            if (snpNames != null && snpNames.size() != 0) {
                                snpNames.clear();
                            }
                            if (duplicates != null && duplicates.size() != 0) {
                                duplicates.clear();
                            }
                            currLine = nextLineTrimless(br);
                            continue;
                        }
                    }

                    if (start <= probePos && probePos <= end) {
                        cache.add(fields.clone());

                        snpNames.add(fields.remove(0));
                        fields.remove(0);
                        fields.remove(0);
                        probes.add(fields);
                    }

                    if (end < probePos) {
                        total--;
                        break;
                    }

                    currLine = nextLineTrimless(br);
                }

                //removeHomo(probes, snpNames);

                if (probes != null && probes.size() != 0) {
                    enforceEqualSize(probes);
                    if (flag == 0) {
                        code = getCode(probes);
                        flag++;
                    }

                    for (int j = 0; j < probes.size(); j++) {
                        ArrayList del = new ArrayList();
                        double dist_snp = cmetric.getScore((ArrayList) probes.get(j), template, code);
                        String snpName = (String) snpNames.get(j);

                        int f = 0;
                        for (int k = 0; k < duplicates.size(); k++) {
                            if (snpName.equalsIgnoreCase((String) duplicates.get(k))) {
                                f++;
                                break;
                            }
                        }
                        if (f == 0) {
                            del.add(snpNames.get(j));
                            //System.out.print(snpName+"\t");
                            del.add((new Double(dist_snp)).toString());
                            //System.out.print((new Double(dist_snp)).toString()+"\t");
                            for (int k = 0; k < rndTemplates.length; k++) {
                                double dist_snp_rnd = cmetric.getScore((ArrayList) probes.get(j), rndTemplates[k], code);
                                del.add((new Double(dist_snp_rnd)).toString());
                                //System.out.print((new Double(dist_snp_rnd)).toString()+"\t");
                            }
                            //System.out.println();
                            dels.add(del);
                            valid++;
                        }
                    }
                }

                if (snpNames != null && snpNames.size() != 0) {
                    //GeneSet snpSet = new FSet(geneSymbol.toUpperCase(), geneDes, snpNames, true);
                    GeneSet snpSet = new FSet(geneSymbol, geneDes, snpNames, true);
                    snpSets.add(snpSet);
                }
            }
            xlog.info("There are " + total + " probes in the original dataset");
            xlog.info("Mapping resulted in filtering out " + (total - valid) + "/" + total + " probes, the remainning " + valid + " probes were used in the analysis");
            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

        SnpStruc sstruc = new SnpStruc(dels, (GeneSet[]) snpSets.toArray(new GeneSet[snpSets.size()]));
        return sstruc;
    }

    //final version - mapping - read entire file
    /*public SnpStruc parse_phenotype_maximum2(CMetric cmetric, Template template, Template[] rndTemplates, ArrayList genes, int before, int after, String path, int itemsOfTemplate) {
    int code = 0;
    int code_flag = 0;
    List snpSets = new ArrayList();
    List dels = new ArrayList();
    //DoubleElement[] datasetSynchedDels = new DoubleElement[snpGeneStruc.size()];
    //String filename = "C://data//mapping.txt";
    ArrayList snps = readData(path);
    String firstrow = (String) ((ArrayList) snps.get(0)).get(0);
    int flag = 1;
    if (firstrow.startsWith("#")) {
    snps.remove(0);
    flag++;
    }
    ArrayList nstuff = (ArrayList) snps.remove(0);

    try {
    if (nstuff.size() != 2) {
    throw new ParserException("file with bad row/col info on line: " + flag);
    }

    int nrows = Integer.parseInt((String) nstuff.get(0));
    int ncols = Integer.parseInt((String) nstuff.get(1));

    ArrayList headers = (ArrayList) snps.remove(0);
    headers.remove(0);            // remove rs#
    headers.remove(0);            // remove chr
    headers.remove(0);            // remove pos
    if (headers.size() != ncols) {
    throw new ParserException("Bad format -- expected ncols from specification on header line: " + ncols + " but found in data: " + headers.size());
    }

    if (headers.size() != itemsOfTemplate) {
    throw new ParserException("Bad format -- expected ncols from phenotype labels: " + itemsOfTemplate + " but found in data: " + headers.size());
    }

    //*********************************************
    String filename = "//nfs//igsp//furey_fc//qing//breast//mapping.txt";
    FileWriter output = new FileWriter(filename);
    BufferedWriter bw = new BufferedWriter(output);
    //*********************************************

    int pointer = 0;
    for (int i = 0; i < genes.size(); i++) {
    ArrayList snpNames = new ArrayList();
    ArrayList snpChrs = new ArrayList();
    ArrayList snpLocs = new ArrayList();
    String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
    int geneChr = ((GeneBean) genes.get(i)).getChromosome();
    String geneStrand = ((GeneBean) genes.get(i)).getStrand();
    String geneDes = ((GeneBean) genes.get(i)).getDescription();
    int start = 0;
    int end = 0;
    if (geneStrand.equalsIgnoreCase("1")) {
    start = ((GeneBean) genes.get(i)).getStart() - before;
    if (start < 0) {
    start = 0;
    }
    end = ((GeneBean) genes.get(i)).getEnd();
    if (end == 0) {
    continue;
    } else {
    end = end + after;
    }
    } else {
    start = ((GeneBean) genes.get(i)).getStart() - after;
    if (start < 0) {
    start = 0;
    }
    end = ((GeneBean) genes.get(i)).getEnd();
    if (end == 0) {
    continue;
    } else {
    end = end + before;
    }
    }

    //*********************************************
    StringTokenizer st = new StringTokenizer(geneSymbol, "///");
    bw.write("Gene symbol: " + (st.nextToken()).trim() + ";\tChr: " + geneChr + ";\tStrand: " + geneStrand + ";\tPosition: " + start + "-" + end);
    bw.newLine();
    //*********************************************

    ArrayList probes = new ArrayList();
    ArrayList pointer_list = (ArrayList) snps.get(pointer);
    int pointer_chr = Integer.parseInt((String) pointer_list.get(1));
    if (geneChr != pointer_chr) {
    for (int j = pointer + 1; j < snps.size(); j++) {
    pointer_list = (ArrayList) snps.get(j);
    pointer_chr = Integer.parseInt((String) pointer_list.get(1));
    if ((geneChr == pointer_chr)) {
    pointer = j;
    break;
    }
    }
    }

    pointer_list = (ArrayList) snps.get(pointer);
    pointer_chr = Integer.parseInt((String) pointer_list.get(1));
    int pointer_pos = Integer.parseInt((String) pointer_list.get(2));
    if (start >= pointer_pos || pointer == 0) {
    for (int j = pointer; j < snps.size(); j++) {
    ArrayList currlist = (ArrayList) snps.get(j);
    int curr_chr = Integer.parseInt((String) currlist.get(1));
    int currpos = Integer.parseInt((String) currlist.get(2));
    if (geneChr != curr_chr) {
    pointer = j;
    break;
    }
    if (start <= currpos && currpos <= end) {
    ArrayList al = (ArrayList) currlist.clone();
    snpNames.add(al.remove(0));
    snpChrs.add(al.remove(0));
    snpLocs.add(al.remove(0));
    probes.add(al);
    }
    if (end < currpos) {
    pointer = j;
    break;
    }
    }
    } else {
    for (int j = pointer; j >= 0; j--) {
    ArrayList currlist = (ArrayList) snps.get(j);
    int curr_chr = Integer.parseInt((String) currlist.get(1));
    if (geneChr != curr_chr) {
    pointer = j + 1;
    break;
    }
    int currpos = Integer.parseInt((String) currlist.get(2));
    if (currpos <= start) {
    pointer = j;
    break;
    }
    }
    for (int j = pointer; j < snps.size(); j++) {
    ArrayList currlist = (ArrayList) snps.get(j);
    int curr_chr = Integer.parseInt((String) currlist.get(1));
    int currpos = Integer.parseInt((String) currlist.get(2));
    if (geneChr != curr_chr) {
    pointer = j;
    break;
    }
    if (start <= currpos && currpos <= end) {
    ArrayList al = (ArrayList) currlist.clone();
    snpNames.add(al.remove(0));
    snpChrs.add(al.remove(0));
    snpLocs.add(al.remove(0));
    probes.add(al);
    }
    if (end < currpos) {
    pointer = j;
    break;
    }
    }
    }

    //removeHomo(probes, snpNames);

    if (probes != null && probes.size() != 0) {
    enforceEqualSize(probes);
    if (code_flag == 0) {
    code = getCode(probes);
    code_flag++;
    }

    ArrayList scores = new ArrayList();
    double maxScore = 0;
    int maxIndex = 0;
    for (int j = 0; j < probes.size(); j++) {
    double dist_snp = cmetric.getScore((ArrayList) probes.get(j), template, code);
    if (maxScore < dist_snp) {
    maxScore = dist_snp;
    maxIndex = j;
    }

    //*********************************************
    String snpName = (String) snpNames.get(j);
    String snpChr = (String) snpChrs.get(j);
    String snpLoc = (String) snpLocs.get(j);
    ArrayList probe = (ArrayList) probes.get(j);
    bw.write(snpName + "\t" + snpChr + "\t" + snpLoc + "\t" + dist_snp + "\t");
    for (int k = 0; k < probe.size(); k++) {
    bw.write((String) probe.get(k) + "\t");
    }
    bw.newLine();
    //*********************************************
    }
    scores.add(new Float(maxScore));
    String maxSnpName = (String) snpNames.get(maxIndex);
    ArrayList maxSnp = new ArrayList();
    maxSnp.add(maxSnpName);
    for (int k = 0; k < rndTemplates.length; k++) {
    double dist_snp_rnd = cmetric.getScore((ArrayList) probes.get(maxIndex), rndTemplates[k], code);
    scores.add(new Float(dist_snp_rnd));
    }
    dels.add(scores);
    GeneSet snpSet = new FSet(geneSymbol, geneDes, maxSnp, true);
    snpSets.add(snpSet);

    }

    //xlog.info("There are " + total + " probes in the original dataset");
    //xlog.info("Mapping resulted in filtering out " + (total - valid) + "/" + total + " probes, the remainning " + valid + " probes were used in the analysis");

    }
    //*********************************************
    bw.close();
    //*********************************************
    } catch (Exception e) {
    e.printStackTrace();
    }

    SnpStruc sstruc = new SnpStruc(dels, (GeneSet[]) snpSets.toArray(new GeneSet[snpSets.size()]));
    return sstruc;
    }*/
    //final version - mapping
    public SnpStruc parse_phenotype_maximum_mapping(CMetric cmetric, Template template, Template[] rndTemplates, ArrayList genes, int before, int after, String path, int itemsOfTemplate) {
        int code = 0;
        int code_flag = 0;
        int chr_number = 1;
        List snpSets = new ArrayList();
        List dels = new ArrayList();
        //DoubleElement[] datasetSynchedDels = new DoubleElement[snpGeneStruc.size()];
        ArrayList snps = readChrData(chr_number, itemsOfTemplate, path);

        try {

            //*********************************************
            String filename = "//nfs//igsp//furey_fc//qing//breast//mapping.txt";
            //String filename = "C://data//mapping.txt";
            FileWriter output = new FileWriter(filename);
            BufferedWriter bw = new BufferedWriter(output);
            //*********************************************

            int pointer = 0;
            for (int i = 0; i < genes.size(); i++) {
                ArrayList snpNames = new ArrayList();
                ArrayList snpChrs = new ArrayList();
                ArrayList snpLocs = new ArrayList();
                String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
                int geneChr = ((GeneBean) genes.get(i)).getChromosome();
                String geneStrand = ((GeneBean) genes.get(i)).getStrand();
                String geneDes = ((GeneBean) genes.get(i)).getDescription();
                int start = 0;
                int end = 0;
                if (geneStrand.equalsIgnoreCase("1")) {
                    start = ((GeneBean) genes.get(i)).getStart() - before;
                    if (start < 0) {
                        start = 0;
                    }
                    end = ((GeneBean) genes.get(i)).getEnd();
                    if (end == 0) {
                        continue;
                    } else {
                        end = end + after;
                    }
                } else {
                    start = ((GeneBean) genes.get(i)).getStart() - after;
                    if (start < 0) {
                        start = 0;
                    }
                    end = ((GeneBean) genes.get(i)).getEnd();
                    if (end == 0) {
                        continue;
                    } else {
                        end = end + before;
                    }
                }

                //*********************************************
                StringTokenizer st = new StringTokenizer(geneSymbol, "///");
                bw.write("Gene symbol: " + (st.nextToken()).trim() + ";\tChr: " + geneChr + ";\tStrand: " + geneStrand + ";\tPosition: " + start + "-" + end);
                bw.newLine();
                //*********************************************

                ArrayList probes = new ArrayList();
                ArrayList pointer_list = (ArrayList) snps.get(pointer);
                int pointer_chr = Integer.parseInt((String) pointer_list.get(1));
                if (geneChr != pointer_chr) {
                    chr_number++;
                    snps = readChrData(chr_number, itemsOfTemplate, path);
                    if(snps==null||snps.size()==0){
                        break;
                    }                    
                    pointer = 0;
                    pointer_list = (ArrayList) snps.get(pointer);
                    pointer_chr = Integer.parseInt((String) pointer_list.get(1));
                }

                int pointer_pos = Integer.parseInt((String) pointer_list.get(2));
                if (start >= pointer_pos) {
                    for (int j = pointer; j < snps.size(); j++) {
                        ArrayList currlist = (ArrayList) snps.get(j);
                        //int curr_chr = Integer.parseInt((String) currlist.get(1));
                        int currpos = Integer.parseInt((String) currlist.get(2));
                        if (start <= currpos && currpos <= end) {
                            ArrayList al = (ArrayList) currlist.clone();
                            snpNames.add(al.remove(0));
                            snpChrs.add(al.remove(0));
                            snpLocs.add(al.remove(0));
                            probes.add(al);
                        }
                        if (end < currpos) {
                            pointer = j;
                            break;
                        }
                        if (j == snps.size() - 1) {
                            pointer = j;
                        }
                    }
                } else {
                    for (int j = pointer; j >= 0; j--) {
                        if (j == 0) {
                            pointer = j;
                        }
                        ArrayList currlist = (ArrayList) snps.get(j);
                        //int curr_chr = Integer.parseInt((String) currlist.get(1));
                        int currpos = Integer.parseInt((String) currlist.get(2));
                        if (currpos <= start) {
                            pointer = j;
                            break;
                        }
                    }
                    for (int j = pointer; j < snps.size(); j++) {
                        ArrayList currlist = (ArrayList) snps.get(j);
                        //int curr_chr = Integer.parseInt((String) currlist.get(1));
                        int currpos = Integer.parseInt((String) currlist.get(2));
                        if (start <= currpos && currpos <= end) {
                            ArrayList al = (ArrayList) currlist.clone();
                            snpNames.add(al.remove(0));
                            snpChrs.add(al.remove(0));
                            snpLocs.add(al.remove(0));
                            probes.add(al);
                        }
                        if (end < currpos) {
                            pointer = j;
                            break;
                        }
                        if (j == snps.size() - 1) {
                            pointer = j;
                        }
                    }
                }

                //removeHomo(probes, snpNames);

                if (probes != null && probes.size() != 0) {
                    enforceEqualSize(probes);
                    if (code_flag == 0) {
                        code = getCode(probes);
                        code_flag++;
                    }

                    ArrayList scores = new ArrayList();
                    double maxScore = 0;
                    int maxIndex = 0;
                    for (int j = 0; j < probes.size(); j++) {
                        double dist_snp = cmetric.getScore((ArrayList) probes.get(j), template, code);
                        if (maxScore < dist_snp) {
                            maxScore = dist_snp;
                            maxIndex = j;
                        }

                        //*********************************************
                        String snpName = (String) snpNames.get(j);
                        String snpChr = (String) snpChrs.get(j);
                        String snpLoc = (String) snpLocs.get(j);
                        ArrayList probe = (ArrayList) probes.get(j);
                        bw.write(snpName + "\t" + snpChr + "\t" + snpLoc + "\t" + dist_snp + "\t");
                        for (int k = 0; k < probe.size(); k++) {
                            bw.write((String) probe.get(k) + "\t");
                        }
                        bw.newLine();
                        //*********************************************
                    }
                    scores.add(new Float(maxScore));
                    String maxSnpName = (String) snpNames.get(maxIndex);
                    ArrayList maxSnp = new ArrayList();
                    maxSnp.add(maxSnpName);
                    for (int k = 0; k < rndTemplates.length; k++) {
                        double dist_snp_rnd = cmetric.getScore((ArrayList) probes.get(maxIndex), rndTemplates[k], code);
                        scores.add(new Float(dist_snp_rnd));
                    }
                    dels.add(scores);
                    GeneSet snpSet = new FSet(geneSymbol, geneDes, maxSnp, true);
                    snpSets.add(snpSet);

                }

            }
            //*********************************************
            bw.close();
            //*********************************************
        } catch (Exception e) {
            e.printStackTrace();
        }

        SnpStruc sstruc = new SnpStruc(dels, (GeneSet[]) snpSets.toArray(new GeneSet[snpSets.size()]));
        return sstruc;
    }

    //final version
    public SnpStruc parse_phenotype_maximum(CMetric cmetric, Template template, Template[] rndTemplates, ArrayList genes, int before, int after, String path, int itemsOfTemplate) {
        int code = 0;
        int code_flag = 0;
        int chr_number = 1;
        List snpSets = new ArrayList();
        List dels = new ArrayList();
        //DoubleElement[] datasetSynchedDels = new DoubleElement[snpGeneStruc.size()];
        ArrayList snps = readChrData(chr_number, itemsOfTemplate, path);

        try {
            int pointer = 0;
            for (int i = 0; i < genes.size(); i++) {
                ArrayList snpNames = new ArrayList();
                String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
                int geneChr = ((GeneBean) genes.get(i)).getChromosome();
                String geneStrand = ((GeneBean) genes.get(i)).getStrand();
                String geneDes = ((GeneBean) genes.get(i)).getDescription();
                int start = 0;
                int end = 0;
                if (geneStrand.equalsIgnoreCase("1")) {
                    start = ((GeneBean) genes.get(i)).getStart() - before;
                    if (start < 0) {
                        start = 0;
                    }
                    end = ((GeneBean) genes.get(i)).getEnd();
                    if (end == 0) {
                        continue;
                    } else {
                        end = end + after;
                    }
                } else {
                    start = ((GeneBean) genes.get(i)).getStart() - after;
                    if (start < 0) {
                        start = 0;
                    }
                    end = ((GeneBean) genes.get(i)).getEnd();
                    if (end == 0) {
                        continue;
                    } else {
                        end = end + before;
                    }
                }

                ArrayList probes = new ArrayList();
                ArrayList pointer_list = (ArrayList) snps.get(pointer);
                int pointer_chr = Integer.parseInt((String) pointer_list.get(1));
                if (geneChr != pointer_chr) {
                    chr_number++;
                    snps = readChrData(chr_number, itemsOfTemplate, path);
                    if(snps==null||snps.size()==0){
                        break;
                    }
                    pointer = 0;
                    pointer_list = (ArrayList) snps.get(pointer);
                    pointer_chr = Integer.parseInt((String) pointer_list.get(1));
                }

                int pointer_pos = Integer.parseInt((String) pointer_list.get(2));
                if (start >= pointer_pos) {
                    for (int j = pointer; j < snps.size(); j++) {
                        ArrayList currlist = (ArrayList) snps.get(j);
                        //int curr_chr = Integer.parseInt((String) currlist.get(1));
                        int currpos = Integer.parseInt((String) currlist.get(2));
                        if (start <= currpos && currpos <= end) {
                            ArrayList al = (ArrayList) currlist.clone();
                            snpNames.add(al.remove(0));
                            al.remove(0);
                            al.remove(0);
                            probes.add(al);
                        }
                        if (end < currpos) {
                            pointer = j;
                            break;
                        }
                        if (j == snps.size() - 1) {
                            pointer = j;
                        }
                    }
                } else {
                    for (int j = pointer; j >= 0; j--) {
                        if (j == 0) {
                            pointer = j;
                        }
                        ArrayList currlist = (ArrayList) snps.get(j);
                        //int curr_chr = Integer.parseInt((String) currlist.get(1));
                        int currpos = Integer.parseInt((String) currlist.get(2));
                        if (currpos <= start) {
                            pointer = j;
                            break;
                        }
                    }
                    for (int j = pointer; j < snps.size(); j++) {
                        ArrayList currlist = (ArrayList) snps.get(j);
                        //int curr_chr = Integer.parseInt((String) currlist.get(1));
                        int currpos = Integer.parseInt((String) currlist.get(2));
                        if (start <= currpos && currpos <= end) {
                            ArrayList al = (ArrayList) currlist.clone();
                            snpNames.add(al.remove(0));
                            al.remove(0);
                            al.remove(0);
                            probes.add(al);
                        }
                        if (end < currpos) {
                            pointer = j;
                            break;
                        }
                        if (j == snps.size() - 1) {
                            pointer = j;
                        }
                    }
                }

                removeHomo(probes, snpNames);

                if (probes != null && probes.size() != 0) {
                    enforceEqualSize(probes);
                    if (code_flag == 0) {
                        code = getCode(probes);
                        code_flag++;
                    }

                    ArrayList scores = new ArrayList();
                    double maxScore = 0;
                    int maxIndex = 0;
                    for (int j = 0; j < probes.size(); j++) {
                        double dist_snp = cmetric.getScore((ArrayList) probes.get(j), template, code);
                        if (maxScore < dist_snp) {
                            maxScore = dist_snp;
                            maxIndex = j;
                        }
                    }
                    scores.add(new Float(maxScore));
                    String maxSnpName = (String) snpNames.get(maxIndex);
                    ArrayList maxSnp = new ArrayList();
                    maxSnp.add(maxSnpName);
                    for (int k = 0; k < rndTemplates.length; k++) {
                        double dist_snp_rnd = cmetric.getScore((ArrayList) probes.get(maxIndex), rndTemplates[k], code);
                        scores.add(new Float(dist_snp_rnd));
                    }
                    dels.add(scores);
                    GeneSet snpSet = new FSet(geneSymbol, geneDes, maxSnp, true);
                    snpSets.add(snpSet);

                }

            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        SnpStruc sstruc = new SnpStruc(dels, (GeneSet[]) snpSets.toArray(new GeneSet[snpSets.size()]));
        return sstruc;
    }

    //final version - read entire file
    /*public SnpStruc parse_phenotype_maximum(CMetric cmetric, Template template, Template[] rndTemplates, ArrayList genes, int before, int after, String path, int itemsOfTemplate) {
    int code = 0;
    int code_flag = 0;
    List snpSets = new ArrayList();
    List dels = new ArrayList();
    //DoubleElement[] datasetSynchedDels = new DoubleElement[snpGeneStruc.size()];
    ArrayList snps = readData(path);
    String firstrow = (String) ((ArrayList) snps.get(0)).get(0);
    int flag = 1;
    if (firstrow.startsWith("#")) {
    snps.remove(0);
    flag++;
    }
    ArrayList nstuff = (ArrayList) snps.remove(0);

    try {
    if (nstuff.size() != 2) {
    throw new ParserException("file with bad row/col info on line: " + flag);
    }

    int nrows = Integer.parseInt((String) nstuff.get(0));
    int ncols = Integer.parseInt((String) nstuff.get(1));

    ArrayList headers = (ArrayList) snps.remove(0);
    headers.remove(0);            // remove rs#
    headers.remove(0);            // remove chr
    headers.remove(0);            // remove pos
    if (headers.size() != ncols) {
    throw new ParserException("Bad format -- expected ncols from specification on header line: " + ncols + " but found in data: " + headers.size());
    }

    if (headers.size() != itemsOfTemplate) {
    throw new ParserException("Bad format -- expected ncols from phenotype labels: " + itemsOfTemplate + " but found in data: " + headers.size());
    }

    int pointer = 0;
    for (int i = 0; i < genes.size(); i++) {
    ArrayList snpNames = new ArrayList();
    String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
    int geneChr = ((GeneBean) genes.get(i)).getChromosome();
    String geneStrand = ((GeneBean) genes.get(i)).getStrand();
    String geneDes = ((GeneBean) genes.get(i)).getDescription();
    int start = 0;
    int end = 0;
    if (geneStrand.equalsIgnoreCase("1")) {
    start = ((GeneBean) genes.get(i)).getStart() - before;
    if (start < 0) {
    start = 0;
    }
    end = ((GeneBean) genes.get(i)).getEnd();
    if (end == 0) {
    continue;
    } else {
    end = end + after;
    }
    } else {
    start = ((GeneBean) genes.get(i)).getStart() - after;
    if (start < 0) {
    start = 0;
    }
    end = ((GeneBean) genes.get(i)).getEnd();
    if (end == 0) {
    continue;
    } else {
    end = end + before;
    }
    }

    ArrayList probes = new ArrayList();
    ArrayList pointer_list = (ArrayList) snps.get(pointer);
    int pointer_chr = Integer.parseInt((String) pointer_list.get(1));
    if (geneChr != pointer_chr) {
    for (int j = pointer + 1; j < snps.size(); j++) {
    pointer_list = (ArrayList) snps.get(j);
    pointer_chr = Integer.parseInt((String) pointer_list.get(1));
    if ((geneChr == pointer_chr)) {
    pointer = j;
    break;
    }
    }
    }

    pointer_list = (ArrayList) snps.get(pointer);
    pointer_chr = Integer.parseInt((String) pointer_list.get(1));
    int pointer_pos = Integer.parseInt((String) pointer_list.get(2));
    if (start >= pointer_pos || pointer == 0) {
    for (int j = pointer; j < snps.size(); j++) {
    ArrayList currlist = (ArrayList) snps.get(j);
    int curr_chr = Integer.parseInt((String) currlist.get(1));
    int currpos = Integer.parseInt((String) currlist.get(2));
    if (geneChr != curr_chr) {
    pointer = j;
    break;
    }
    if (start <= currpos && currpos <= end) {
    ArrayList al = (ArrayList) currlist.clone();
    snpNames.add(al.remove(0));
    al.remove(0);
    al.remove(0);
    probes.add(al);
    }
    if (end < currpos) {
    pointer = j;
    break;
    }
    }
    } else {
    for (int j = pointer; j >= 0; j--) {
    ArrayList currlist = (ArrayList) snps.get(j);
    int curr_chr = Integer.parseInt((String) currlist.get(1));
    if (geneChr != curr_chr) {
    pointer = j + 1;
    break;
    }
    int currpos = Integer.parseInt((String) currlist.get(2));
    if (currpos <= start) {
    pointer = j;
    break;
    }
    }
    for (int j = pointer; j < snps.size(); j++) {
    ArrayList currlist = (ArrayList) snps.get(j);
    int curr_chr = Integer.parseInt((String) currlist.get(1));
    int currpos = Integer.parseInt((String) currlist.get(2));
    if (geneChr != curr_chr) {
    pointer = j;
    break;
    }
    if (start <= currpos && currpos <= end) {
    ArrayList al = (ArrayList) currlist.clone();
    snpNames.add(al.remove(0));
    al.remove(0);
    al.remove(0);
    probes.add(al);
    }
    if (end < currpos) {
    pointer = j;
    break;
    }
    }
    }

    removeHomo(probes, snpNames);

    if (probes != null && probes.size() != 0) {
    enforceEqualSize(probes);
    if (code_flag == 0) {
    code = getCode(probes);
    code_flag++;
    }

    ArrayList scores = new ArrayList();
    double maxScore = 0;
    int maxIndex = 0;
    for (int j = 0; j < probes.size(); j++) {
    double dist_snp = cmetric.getScore((ArrayList) probes.get(j), template, code);
    if (maxScore < dist_snp) {
    maxScore = dist_snp;
    maxIndex = j;
    }
    }
    scores.add(new Float(maxScore));
    String maxSnpName = (String) snpNames.get(maxIndex);
    ArrayList maxSnp = new ArrayList();
    maxSnp.add(maxSnpName);
    for (int k = 0; k < rndTemplates.length; k++) {
    double dist_snp_rnd = cmetric.getScore((ArrayList) probes.get(maxIndex), rndTemplates[k], code);
    scores.add(new Float(dist_snp_rnd));
    }
    dels.add(scores);
    GeneSet snpSet = new FSet(geneSymbol, geneDes, maxSnp, true);
    snpSets.add(snpSet);

    }
    //xlog.info("There are " + total + " probes in the original dataset");
    //xlog.info("Mapping resulted in filtering out " + (total - valid) + "/" + total + " probes, the remainning " + valid + " probes were used in the analysis");

    }
    } catch (Exception e) {
    e.printStackTrace();
    }

    SnpStruc sstruc = new SnpStruc(dels, (GeneSet[]) snpSets.toArray(new GeneSet[snpSets.size()]));
    return sstruc;
    } */
    //apply to maximum (snp)
    /*public SnpStruc parse_phenotype_maximum(CMetric cmetric, Template template, Template[] rndTemplates, ArrayList genes, int before, int after, String path, int itemsOfTemplate) {
    int code = 0;
    int flag = 0;

    List snpSets = new ArrayList();
    List dels = new ArrayList();
    //DoubleElement[] datasetSynchedDels = new DoubleElement[snpGeneStruc.size()];
    //String filename = "C://data//mapping.txt";

    int total = 0;
    int valid = 0;
    ArrayList cache = new ArrayList();
    try {
    FileReader input = new FileReader(path);
    BufferedReader br = new BufferedReader(input);
    String currLine = nextLine(br);

    // 1st  non-empty, non-comment line is numrows and numcols
    int[] nstuff = ParseUtils.string2ints(currLine, "\t");
    if (nstuff.length != 2) {
    throw new ParserException("file with bad row/col info on line: " + currLine);
    }

    int nrows = nstuff[0];
    int ncols = nstuff[1];

    currLine = nextLine(br);
    List colnames = ParseUtils.string2stringsList(currLine, "\t"); // colnames can have spaces

    colnames.remove(0);            // remove rs#
    colnames.remove(0);            // remove chr
    colnames.remove(0);            // remove pos

    if (colnames.size() != ncols) {
    throw new ParserException("Bad format -- expected ncols from specification on header line: " + ncols + " but found in data: " + colnames.size());
    }

    if (colnames.size() != itemsOfTemplate) {
    throw new ParserException("Bad format -- expected ncols from phenotype labels: " + itemsOfTemplate + " but found in data: " + colnames.size());
    }

    currLine = nextLineTrimless(br);

    //String[] atp = new String[]{"ATP6V1A", "ATP5O", "ATP6V1C1", "ATP6V1E1", "ATP6V1B1", "ATP6V1H", "ATP6V0B", "ATP6V1F", "ATP6V1D", "ATP6V0A1", "ATP6V1B2", "ATP6V0D1", "ATP6V0C", "ATP6V1G2", "ATP6AP1", "ATP6V0E", "ATP6V1G1", "ATP6V0A4"};
    for (int i = 0; i < genes.size(); i++) {
    ArrayList duplicates = new ArrayList();
    ArrayList snpNames = new ArrayList();
    String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
    int geneChr = ((GeneBean) genes.get(i)).getChromosome();
    String geneDes = ((GeneBean) genes.get(i)).getDescription();
    int start = ((GeneBean) genes.get(i)).getStart() - before;
    if (start < 0) {
    start = 0;
    }
    int end = ((GeneBean) genes.get(i)).getEnd();
    if (end == 0) {
    continue;
    } else {
    end = end + after;
    }



    ArrayList probes = new ArrayList(cache);
    if (cache != null && cache.size() != 0) {
    cache.clear();
    }
    if (probes != null && probes.size() != 0) {
    for (int j = 0; j < probes.size(); j++) {
    int pos_c = Integer.parseInt(((ArrayList) probes.get(j)).get(2).toString().trim());
    if (start <= pos_c) {
    ArrayList probe_c = (ArrayList) ((ArrayList) probes.get(j)).clone();
    cache.add(probe_c);
    String s = (String) ((ArrayList) probes.get(j)).remove(0);
    snpNames.add(s);
    duplicates.add(s);
    ((ArrayList) probes.get(j)).remove(0);
    ((ArrayList) probes.get(j)).remove(0);
    } else {
    probes.remove(j);
    j--;
    }

    }
    }

    while ((currLine != null) && (currLine.length() != 0)) {
    total++;
    ArrayList fields = string2stringsV2(currLine, ncols + 3); // spaces allowed in name & desc field so DONT tokenize them

    if (fields.size() != ncols + 3) {
    //System.out.println(">> " + fields);
    throw new ParserException("Bad format - expect ncols: " + (ncols + 3) + " but found: " + fields.size() + " on line >" + currLine + "<\nIf this dataset has missing values, use ImputeDataset to fill these in before importing as a Dataset");
    }
    int probeChr = Integer.parseInt(fields.get(1).toString().trim());
    int probePos = Integer.parseInt(fields.get(2).toString().trim());
    if (geneChr != probeChr) {
    if (geneChr < probeChr) {
    if (cache != null && cache.size() != 0) {
    cache.clear();
    }
    total--;
    break;
    } else {
    if (cache != null && cache.size() != 0) {
    cache.clear();
    }
    if (probes != null && probes.size() != 0) {
    probes.clear();
    }
    if (snpNames != null && snpNames.size() != 0) {
    snpNames.clear();
    }
    if (duplicates != null && duplicates.size() != 0) {
    duplicates.clear();
    }
    currLine = nextLineTrimless(br);
    continue;
    }
    }

    if (start <= probePos && probePos <= end) {
    cache.add(fields.clone());

    snpNames.add(fields.remove(0));
    fields.remove(0);
    fields.remove(0);
    probes.add(fields);
    }

    if (end < probePos) {
    total--;
    break;
    }

    currLine = nextLineTrimless(br);
    }

    removeHomo(probes, snpNames);

    if (probes != null && probes.size() != 0) {
    enforceEqualSize(probes);
    if (flag == 0) {
    code = getCode(probes);
    flag++;
    }

    ArrayList scores = new ArrayList();
    double maxScore = 0;
    int maxIndex = 0;
    for (int j = 0; j < probes.size(); j++) {
    double dist_snp = cmetric.getScore((ArrayList) probes.get(j), template, code);
    if (maxScore < dist_snp) {
    maxScore = dist_snp;
    maxIndex = j;
    }
    }
    scores.add(new Float(maxScore));

    String maxSnpName = (String) snpNames.get(maxIndex);
    ArrayList maxSnp = new ArrayList();
    maxSnp.add(maxSnpName);
    for (int k = 0; k < rndTemplates.length; k++) {
    double dist_snp_rnd = cmetric.getScore((ArrayList) probes.get(maxIndex), rndTemplates[k], code);
    scores.add(new Float(dist_snp_rnd));
    }
    dels.add(scores);
    GeneSet snpSet = new FSet(geneSymbol, geneDes, maxSnp, true);
    snpSets.add(snpSet);

    }
    }
    //xlog.info("There are " + total + " probes in the original dataset");
    //xlog.info("Mapping resulted in filtering out " + (total - valid) + "/" + total + " probes, the remainning " + valid + " probes were used in the analysis");
    br.close();
    } catch (Exception e) {
    e.printStackTrace();
    }

    SnpStruc sstruc = new SnpStruc(dels, (GeneSet[]) snpSets.toArray(new GeneSet[snpSets.size()]));
    return sstruc;
    }*/
    //apply to maximum (snp)
    /*public SnpStruc parse_phenotype_maximum_pvalue(CMetric cmetric, Template template, Template[] rndTemplates, ArrayList genes, int before, int after, String path, int itemsOfTemplate) {
    int code = 0;
    int flag = 0;

    List snpSets = new ArrayList();
    List dels = new ArrayList();
    //DoubleElement[] datasetSynchedDels = new DoubleElement[snpGeneStruc.size()];
    //String filename = "C://data//mapping.txt";

    int total = 0;
    int valid = 0;
    ArrayList cache = new ArrayList();
    try {
    FileReader input = new FileReader(path);
    BufferedReader br = new BufferedReader(input);
    String currLine = nextLine(br);

    // 1st  non-empty, non-comment line is numrows and numcols
    int[] nstuff = ParseUtils.string2ints(currLine, "\t");
    if (nstuff.length != 2) {
    throw new ParserException("file with bad row/col info on line: " + currLine);
    }

    int nrows = nstuff[0];
    int ncols = nstuff[1];

    currLine = nextLine(br);
    List colnames = ParseUtils.string2stringsList(currLine, "\t"); // colnames can have spaces

    colnames.remove(0);            // remove rs#
    colnames.remove(0);            // remove chr
    colnames.remove(0);            // remove pos

    if (colnames.size() != ncols) {
    throw new ParserException("Bad format -- expected ncols from specification on header line: " + ncols + " but found in data: " + colnames.size());
    }

    if (colnames.size() != itemsOfTemplate) {
    throw new ParserException("Bad format -- expected ncols from phenotype labels: " + itemsOfTemplate + " but found in data: " + colnames.size());
    }

    currLine = nextLineTrimless(br);

    //String[] atp = new String[]{"ATP6V1A", "ATP5O", "ATP6V1C1", "ATP6V1E1", "ATP6V1B1", "ATP6V1H", "ATP6V0B", "ATP6V1F", "ATP6V1D", "ATP6V0A1", "ATP6V1B2", "ATP6V0D1", "ATP6V0C", "ATP6V1G2", "ATP6AP1", "ATP6V0E", "ATP6V1G1", "ATP6V0A4"};
    for (int i = 0; i < genes.size(); i++) {
    ArrayList duplicates = new ArrayList();
    ArrayList snpNames = new ArrayList();
    String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
    int geneChr = ((GeneBean) genes.get(i)).getChromosome();
    String geneDes = ((GeneBean) genes.get(i)).getDescription();
    int start = ((GeneBean) genes.get(i)).getStart() - before;
    if (start < 0) {
    start = 0;
    }
    int end = ((GeneBean) genes.get(i)).getEnd();
    if (end == 0) {
    continue;
    } else {
    end = end + after;
    }



    ArrayList probes = new ArrayList(cache);
    if (cache != null && cache.size() != 0) {
    cache.clear();
    }
    if (probes != null && probes.size() != 0) {
    for (int j = 0; j < probes.size(); j++) {
    int pos_c = Integer.parseInt(((ArrayList) probes.get(j)).get(2).toString().trim());
    if (start <= pos_c) {
    ArrayList probe_c = (ArrayList) ((ArrayList) probes.get(j)).clone();
    cache.add(probe_c);
    String s = (String) ((ArrayList) probes.get(j)).remove(0);
    snpNames.add(s);
    duplicates.add(s);
    ((ArrayList) probes.get(j)).remove(0);
    ((ArrayList) probes.get(j)).remove(0);
    } else {
    probes.remove(j);
    j--;
    }

    }
    }

    while ((currLine != null) && (currLine.length() != 0)) {
    total++;
    ArrayList fields = string2stringsV2(currLine, ncols + 3); // spaces allowed in name & desc field so DONT tokenize them

    if (fields.size() != ncols + 3) {
    //System.out.println(">> " + fields);
    throw new ParserException("Bad format - expect ncols: " + (ncols + 3) + " but found: " + fields.size() + " on line >" + currLine + "<\nIf this dataset has missing values, use ImputeDataset to fill these in before importing as a Dataset");
    }
    int probeChr = Integer.parseInt(fields.get(1).toString().trim());
    int probePos = Integer.parseInt(fields.get(2).toString().trim());
    if (geneChr != probeChr) {
    if (geneChr < probeChr) {
    if (cache != null && cache.size() != 0) {
    cache.clear();
    }
    total--;
    break;
    } else {
    if (cache != null && cache.size() != 0) {
    cache.clear();
    }
    if (probes != null && probes.size() != 0) {
    probes.clear();
    }
    if (snpNames != null && snpNames.size() != 0) {
    snpNames.clear();
    }
    if (duplicates != null && duplicates.size() != 0) {
    duplicates.clear();
    }
    currLine = nextLineTrimless(br);
    continue;
    }
    }

    if (start <= probePos && probePos <= end) {
    cache.add(fields.clone());

    snpNames.add(fields.remove(0));
    fields.remove(0);
    fields.remove(0);
    probes.add(fields);
    }

    if (end < probePos) {
    total--;
    break;
    }

    currLine = nextLineTrimless(br);
    }

    //removeHomo(probes, snpNames);
    if (probes != null && probes.size() != 0) {
    enforceEqualSize(probes);
    if (flag == 0) {
    code = getCode(probes);
    flag++;
    }

    ArrayList rndScores = new ArrayList();
    double maxScore = 0;
    int maxIndex = 0;
    for (int j = 0; j < probes.size(); j++) {
    double dist_snp = cmetric.getScore((ArrayList) probes.get(j), template, code);
    if (maxScore < dist_snp) {
    maxScore = dist_snp;
    maxIndex = j;
    }
    }
    //scores.add(new Float(maxScore));

    String maxSnpName = (String) snpNames.get(maxIndex);
    ArrayList maxSnp = new ArrayList();
    maxSnp.add(maxSnpName);
    for (int k = 0; k < rndTemplates.length; k++) {
    double dist_snp_rnd = cmetric.getScore((ArrayList) probes.get(maxIndex), rndTemplates[k], code);
    rndScores.add(new Float(dist_snp_rnd));
    }

    ArrayList pvalue_scores = new ArrayList();
    double realP = XMath.getPValueSnp(maxScore, rndScores);
    pvalue_scores.add(realP);

    for (int j = 0; j < rndScores.size(); j++) {
    float score = ((Float) rndScores.get(j)).floatValue();
    double p = XMath.getPValueSnp(score, rndScores);
    pvalue_scores.add(new Float(p));
    }

    dels.add(pvalue_scores);
    GeneSet snpSet = new FSet(geneSymbol, geneDes, maxSnp, true);
    snpSets.add(snpSet);

    }
    }
    //xlog.info("There are " + total + " probes in the original dataset");
    //xlog.info("Mapping resulted in filtering out " + (total - valid) + "/" + total + " probes, the remainning " + valid + " probes were used in the analysis");
    br.close();
    } catch (Exception e) {
    e.printStackTrace();
    }

    SnpStruc sstruc = new SnpStruc(dels, (GeneSet[]) snpSets.toArray(new GeneSet[snpSets.size()]));
    return sstruc;
    }*/
    //apply to maximum (snp)
    public SnpStruc parse_phenotype_maximum_pvalue(CMetric cmetric, Template template, Template[] rndTemplates, ArrayList genes, int before, int after, String path, int itemsOfTemplate) {
        int code = 0;
        int flag = 0;

        List snpSets = new ArrayList();
        List dels = new ArrayList();
        //DoubleElement[] datasetSynchedDels = new DoubleElement[snpGeneStruc.size()];
        //String filename = "C://data//mapping.txt";

        int total = 0;
        int valid = 0;
        ArrayList cache = new ArrayList();
        try {
            FileReader input = new FileReader(path);
            BufferedReader br = new BufferedReader(input);
            String currLine = nextLine(br);

            // 1st  non-empty, non-comment line is numrows and numcols
            int[] nstuff = ParseUtils.string2ints(currLine, "\t");
            if (nstuff.length != 2) {
                throw new ParserException("file with bad row/col info on line: " + currLine);
            }

            int nrows = nstuff[0];
            int ncols = nstuff[1];

            currLine = nextLine(br);
            List colnames = ParseUtils.string2stringsList(currLine, "\t"); // colnames can have spaces

            colnames.remove(0);            // remove rs#
            colnames.remove(0);            // remove chr
            colnames.remove(0);            // remove pos

            if (colnames.size() != ncols) {
                throw new ParserException("Bad format -- expected ncols from specification on header line: " + ncols + " but found in data: " + colnames.size());
            }

            if (colnames.size() != itemsOfTemplate) {
                throw new ParserException("Bad format -- expected ncols from phenotype labels: " + itemsOfTemplate + " but found in data: " + colnames.size());
            }

            currLine = nextLineTrimless(br);

            //String[] atp = new String[]{"ATP6V1A", "ATP5O", "ATP6V1C1", "ATP6V1E1", "ATP6V1B1", "ATP6V1H", "ATP6V0B", "ATP6V1F", "ATP6V1D", "ATP6V0A1", "ATP6V1B2", "ATP6V0D1", "ATP6V0C", "ATP6V1G2", "ATP6AP1", "ATP6V0E", "ATP6V1G1", "ATP6V0A4"};
            for (int i = 0; i < genes.size(); i++) {
                ArrayList duplicates = new ArrayList();
                ArrayList snpNames = new ArrayList();
                String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
                int geneChr = ((GeneBean) genes.get(i)).getChromosome();
                String geneDes = ((GeneBean) genes.get(i)).getDescription();
                int start = ((GeneBean) genes.get(i)).getStart() - before;
                if (start < 0) {
                    start = 0;
                }
                int end = ((GeneBean) genes.get(i)).getEnd();
                if (end == 0) {
                    continue;
                } else {
                    end = end + after;
                }



                ArrayList probes = new ArrayList(cache);
                if (cache != null && cache.size() != 0) {
                    cache.clear();
                }
                if (probes != null && probes.size() != 0) {
                    for (int j = 0; j < probes.size(); j++) {
                        int pos_c = Integer.parseInt(((ArrayList) probes.get(j)).get(2).toString().trim());
                        if (start <= pos_c) {
                            ArrayList probe_c = (ArrayList) ((ArrayList) probes.get(j)).clone();
                            cache.add(probe_c);
                            String s = (String) ((ArrayList) probes.get(j)).remove(0);
                            snpNames.add(s);
                            duplicates.add(s);
                            ((ArrayList) probes.get(j)).remove(0);
                            ((ArrayList) probes.get(j)).remove(0);
                        } else {
                            probes.remove(j);
                            j--;
                        }

                    }
                }

                while ((currLine != null) && (currLine.length() != 0)) {
                    total++;
                    ArrayList fields = string2stringsV2(currLine, ncols + 3); // spaces allowed in name & desc field so DONT tokenize them

                    if (fields.size() != ncols + 3) {
                        //System.out.println(">> " + fields);
                        throw new ParserException("Bad format - expect ncols: " + (ncols + 3) + " but found: " + fields.size() + " on line >" + currLine + "<\nIf this dataset has missing values, use ImputeDataset to fill these in before importing as a Dataset");
                    }
                    int probeChr = Integer.parseInt(fields.get(1).toString().trim());
                    int probePos = Integer.parseInt(fields.get(2).toString().trim());
                    if (geneChr != probeChr) {
                        if (geneChr < probeChr) {
                            if (cache != null && cache.size() != 0) {
                                cache.clear();
                            }
                            total--;
                            break;
                        } else {
                            if (cache != null && cache.size() != 0) {
                                cache.clear();
                            }
                            if (probes != null && probes.size() != 0) {
                                probes.clear();
                            }
                            if (snpNames != null && snpNames.size() != 0) {
                                snpNames.clear();
                            }
                            if (duplicates != null && duplicates.size() != 0) {
                                duplicates.clear();
                            }
                            currLine = nextLineTrimless(br);
                            continue;
                        }
                    }

                    if (start <= probePos && probePos <= end) {
                        cache.add(fields.clone());

                        snpNames.add(fields.remove(0));
                        fields.remove(0);
                        fields.remove(0);
                        probes.add(fields);
                    }

                    if (end < probePos) {
                        total--;
                        break;
                    }

                    currLine = nextLineTrimless(br);
                }

                //removeHomo(probes, snpNames);
                if (probes != null && probes.size() != 0) {
                    enforceEqualSize(probes);
                    if (flag == 0) {
                        code = getCode(probes);
                        flag++;
                    }

                    ArrayList scores = new ArrayList();
                    double maxScore = 0;
                    int maxIndex = 0;
                    for (int j = 0; j < probes.size(); j++) {
                        double dist_snp = cmetric.getScore((ArrayList) probes.get(j), template, code);
                        if (maxScore < dist_snp) {
                            maxScore = dist_snp;
                            maxIndex = j;
                        }
                    }

                    for (int k = 0; k < rndTemplates.length; k++) {
                        double dist_snp_rnd = cmetric.getScore((ArrayList) probes.get(maxIndex), rndTemplates[k], code);
                        scores.add(new Float(dist_snp_rnd));
                    }
                    scores.add(0, new Float(maxScore));

                    ArrayList pvalue_scores = new ArrayList();
                    double realP = GsaaTool.getPValue((float) maxScore, scores);
                    pvalue_scores.add(new Float(realP));

                    for (int j = 1; j < scores.size(); j++) {
                        float score = ((Float) scores.get(j)).floatValue();
                        double p = GsaaTool.getPValue(score, scores);
                        pvalue_scores.add(new Float(p));
                    }

                    String maxSnpName = (String) snpNames.get(maxIndex);
                    ArrayList maxSnp = new ArrayList();
                    maxSnp.add(maxSnpName);
                    dels.add(pvalue_scores);
                    GeneSet snpSet = new FSet(geneSymbol, geneDes, maxSnp, true);
                    snpSets.add(snpSet);

                }
            }
            //xlog.info("There are " + total + " probes in the original dataset");
            //xlog.info("Mapping resulted in filtering out " + (total - valid) + "/" + total + " probes, the remainning " + valid + " probes were used in the analysis");
            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

        SnpStruc sstruc = new SnpStruc(dels, (GeneSet[]) snpSets.toArray(new GeneSet[snpSets.size()]));
        return sstruc;
    }

    //apply to maximum
    /*public SnpStruc parse_phenotype_maximum(CMetric cmetric, Template template, Template[] rndTemplates, ArrayList genes, int before, int after, String path, int itemsOfTemplate) {
    int code = 0;
    int flag = 0;

    List snpSets = new ArrayList();
    List dels = new ArrayList();
    //DoubleElement[] datasetSynchedDels = new DoubleElement[snpGeneStruc.size()];
    //String filename = "C://data//mapping.txt";

    int total = 0;
    int valid = 0;
    ArrayList cache = new ArrayList();
    try {
    FileReader input = new FileReader(path);
    BufferedReader br = new BufferedReader(input);
    String currLine = nextLine(br);

    // 1st  non-empty, non-comment line is numrows and numcols
    int[] nstuff = ParseUtils.string2ints(currLine, "\t");
    if (nstuff.length != 2) {
    throw new ParserException("file with bad row/col info on line: " + currLine);
    }

    int nrows = nstuff[0];
    int ncols = nstuff[1];

    currLine = nextLine(br);
    List colnames = ParseUtils.string2stringsList(currLine, "\t"); // colnames can have spaces

    colnames.remove(0);            // remove rs#
    colnames.remove(0);            // remove chr
    colnames.remove(0);            // remove pos

    if (colnames.size() != ncols) {
    throw new ParserException("Bad format -- expected ncols from specification on header line: " + ncols + " but found in data: " + colnames.size());
    }

    if (colnames.size() != itemsOfTemplate) {
    throw new ParserException("Bad format -- expected ncols from phenotype labels: " + itemsOfTemplate + " but found in data: " + colnames.size());
    }

    currLine = nextLineTrimless(br);

    for (int i = 0; i < genes.size(); i++) {
    ArrayList duplicates = new ArrayList();
    ArrayList snpNames = new ArrayList();
    String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
    int geneChr = ((GeneBean) genes.get(i)).getChromosome();
    String geneDes = ((GeneBean) genes.get(i)).getDescription();
    int start = ((GeneBean) genes.get(i)).getStart() - before;
    if (start < 0) {
    start = 0;
    }
    int end = ((GeneBean) genes.get(i)).getEnd();
    if (end == 0) {
    continue;
    } else {
    end = end + after;
    }



    ArrayList probes = new ArrayList(cache);
    if (cache != null && cache.size() != 0) {
    cache.clear();
    }
    if (probes != null && probes.size() != 0) {
    for (int j = 0; j < probes.size(); j++) {
    int pos_c = Integer.parseInt(((ArrayList) probes.get(j)).get(2).toString().trim());
    if (start <= pos_c) {
    ArrayList probe_c = (ArrayList) ((ArrayList) probes.get(j)).clone();
    cache.add(probe_c);
    String s = (String) ((ArrayList) probes.get(j)).remove(0);
    snpNames.add(s);
    duplicates.add(s);
    ((ArrayList) probes.get(j)).remove(0);
    ((ArrayList) probes.get(j)).remove(0);
    } else {
    probes.remove(j);
    j--;
    }

    }
    }

    while ((currLine != null) && (currLine.length() != 0)) {
    total++;
    ArrayList fields = string2stringsV2(currLine, ncols + 3); // spaces allowed in name & desc field so DONT tokenize them

    if (fields.size() != ncols + 3) {
    //System.out.println(">> " + fields);
    throw new ParserException("Bad format - expect ncols: " + (ncols + 3) + " but found: " + fields.size() + " on line >" + currLine + "<\nIf this dataset has missing values, use ImputeDataset to fill these in before importing as a Dataset");
    }
    int probeChr = Integer.parseInt(fields.get(1).toString().trim());
    int probePos = Integer.parseInt(fields.get(2).toString().trim());
    if (geneChr != probeChr) {
    if (geneChr < probeChr) {
    if (cache != null && cache.size() != 0) {
    cache.clear();
    }
    total--;
    break;
    } else {
    if (cache != null && cache.size() != 0) {
    cache.clear();
    }
    if (probes != null && probes.size() != 0) {
    probes.clear();
    }
    if (snpNames != null && snpNames.size() != 0) {
    snpNames.clear();
    }
    if (duplicates != null && duplicates.size() != 0) {
    duplicates.clear();
    }
    currLine = nextLineTrimless(br);
    continue;
    }
    }

    if (start <= probePos && probePos <= end) {
    cache.add(fields.clone());

    snpNames.add(fields.remove(0));
    fields.remove(0);
    fields.remove(0);
    probes.add(fields);
    }

    if (end < probePos) {
    total--;
    break;
    }

    currLine = nextLineTrimless(br);
    }

    //removeHomo(probes, snpNames);

    if (probes != null && probes.size() != 0) {
    enforceEqualSize(probes);
    if (flag == 0) {
    code = getCode(probes);
    flag++;
    }

    ArrayList geneSnps = new ArrayList();
    for (int j = 0; j < probes.size(); j++) {
    ArrayList del = new ArrayList();
    double dist_snp = cmetric.getScore((ArrayList) probes.get(j), template, code);
    String snpName = (String) snpNames.get(j);

    del.add(snpName);
    //System.out.print(snpName+"\t");
    del.add((new Double(dist_snp)).toString());
    //System.out.print((new Double(dist_snp)).toString()+"\t");
    for (int k = 0; k < rndTemplates.length; k++) {
    double dist_snp_rnd = cmetric.getScore((ArrayList) probes.get(j), rndTemplates[k], code);
    del.add((new Double(dist_snp_rnd)).toString());
    //System.out.print((new Double(dist_snp_rnd)).toString()+"\t");
    }
    //System.out.println();
    geneSnps.add(del);
    //valid++;
    }

    if (geneSnps != null && geneSnps.size() != 0) {
    dels.add((SnpSetAlgorithms.maxList(geneSnps)));
    GeneSet snpSet = new FSet(geneSymbol, geneDes, snpNames, true);
    snpSets.add(snpSet);
    }

    }
    }
    //xlog.info("There are " + total + " probes in the original dataset");
    //xlog.info("Mapping resulted in filtering out " + (total - valid) + "/" + total + " probes, the remainning " + valid + " probes were used in the analysis");
    br.close();
    } catch (Exception e) {
    e.printStackTrace();
    }

    SnpStruc sstruc = new SnpStruc(dels, (GeneSet[]) snpSets.toArray(new GeneSet[snpSets.size()]));
    return sstruc;
    }*/
    //apply to maximum (gene_set)
    public SnpStruc parse_geneset_maximum(CMetric cmetric, Template template, ArrayList genes, int before, int after, String path, int itemsOfTemplate) {
        int code = 0;
        int code_flag = 0;
        int chr_number = 1;
        List snpSets = new ArrayList();
        List dels = new ArrayList();
        //DoubleElement[] datasetSynchedDels = new DoubleElement[snpGeneStruc.size()];
        ArrayList snps = readChrData(chr_number, itemsOfTemplate, path);

        try {
            int pointer = 0;
            for (int i = 0; i < genes.size(); i++) {
                ArrayList snpNames = new ArrayList();
                String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
                int geneChr = ((GeneBean) genes.get(i)).getChromosome();
                String geneStrand = ((GeneBean) genes.get(i)).getStrand();
                String geneDes = ((GeneBean) genes.get(i)).getDescription();
                int start = 0;
                int end = 0;
                if (geneStrand.equalsIgnoreCase("1")) {
                    start = ((GeneBean) genes.get(i)).getStart() - before;
                    if (start < 0) {
                        start = 0;
                    }
                    end = ((GeneBean) genes.get(i)).getEnd();
                    if (end == 0) {
                        continue;
                    } else {
                        end = end + after;
                    }
                } else {
                    start = ((GeneBean) genes.get(i)).getStart() - after;
                    if (start < 0) {
                        start = 0;
                    }
                    end = ((GeneBean) genes.get(i)).getEnd();
                    if (end == 0) {
                        continue;
                    } else {
                        end = end + before;
                    }
                }

                ArrayList probes = new ArrayList();
                ArrayList pointer_list = (ArrayList) snps.get(pointer);
                int pointer_chr = Integer.parseInt((String) pointer_list.get(1));
                if (geneChr != pointer_chr) {
                    chr_number++;
                    snps = readChrData(chr_number, itemsOfTemplate, path);
                    if(snps==null||snps.size()==0){
                        break;
                    }
                    pointer = 0;
                    pointer_list = (ArrayList) snps.get(pointer);
                    pointer_chr = Integer.parseInt((String) pointer_list.get(1));
                }

                int pointer_pos = Integer.parseInt((String) pointer_list.get(2));
                if (start >= pointer_pos) {
                    for (int j = pointer; j < snps.size(); j++) {
                        ArrayList currlist = (ArrayList) snps.get(j);
                        //int curr_chr = Integer.parseInt((String) currlist.get(1));
                        int currpos = Integer.parseInt((String) currlist.get(2));
                        if (start <= currpos && currpos <= end) {
                            ArrayList al = (ArrayList) currlist.clone();
                            snpNames.add(al.remove(0));
                            al.remove(0);
                            al.remove(0);
                            probes.add(al);
                        }
                        if (end < currpos) {
                            pointer = j;
                            break;
                        }
                        if (j == snps.size() - 1) {
                            pointer = j;
                        }
                    }
                } else {
                    for (int j = pointer; j >= 0; j--) {
                        if (j == 0) {
                            pointer = j;
                        }
                        ArrayList currlist = (ArrayList) snps.get(j);
                        //int curr_chr = Integer.parseInt((String) currlist.get(1));
                        int currpos = Integer.parseInt((String) currlist.get(2));
                        if (currpos <= start) {
                            pointer = j;
                            break;
                        }
                    }
                    for (int j = pointer; j < snps.size(); j++) {
                        ArrayList currlist = (ArrayList) snps.get(j);
                        //int curr_chr = Integer.parseInt((String) currlist.get(1));
                        int currpos = Integer.parseInt((String) currlist.get(2));
                        if (start <= currpos && currpos <= end) {
                            ArrayList al = (ArrayList) currlist.clone();
                            snpNames.add(al.remove(0));
                            al.remove(0);
                            al.remove(0);
                            probes.add(al);
                        }
                        if (end < currpos) {
                            pointer = j;
                            break;
                        }
                        if (j == snps.size() - 1) {
                            pointer = j;
                        }
                    }
                }

                removeHomo(probes, snpNames);

                if (probes != null && probes.size() != 0) {
                    enforceEqualSize(probes);
                    if (code_flag == 0) {
                        code = getCode(probes);
                        code_flag++;
                    }

                    ArrayList scores = new ArrayList();
                    double maxScore = 0;
                    int maxIndex = 0;
                    for (int j = 0; j < probes.size(); j++) {
                        double dist_snp = cmetric.getScore((ArrayList) probes.get(j), template, code);
                        if (maxScore < dist_snp) {
                            maxScore = dist_snp;
                            maxIndex = j;
                        }
                    }
                    scores.add(new Float(maxScore));
                    String maxSnpName = (String) snpNames.get(maxIndex);
                    ArrayList maxSnp = new ArrayList();
                    maxSnp.add(maxSnpName);
                    dels.add(scores);
                    GeneSet snpSet = new FSet(geneSymbol, geneDes, maxSnp, true);
                    snpSets.add(snpSet);

                }

            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        SnpStruc sstruc = new SnpStruc(dels, (GeneSet[]) snpSets.toArray(new GeneSet[snpSets.size()]));
        return sstruc;
    }

    //apply to average
    public SnpStruc parse_phenotype_average(CMetric cmetric, Template template, Template[] rndTemplates, ArrayList genes, int before, int after, String path, int itemsOfTemplate) {
        int code = 0;
        int flag = 0;

        List snpSets = new ArrayList();
        List dels = new ArrayList();
        //DoubleElement[] datasetSynchedDels = new DoubleElement[snpGeneStruc.size()];
        //String filename = "C://data//mapping.txt";

        int total = 0;
        int valid = 0;
        ArrayList cache = new ArrayList();
        try {
            FileReader input = new FileReader(path);
            BufferedReader br = new BufferedReader(input);
            String currLine = nextLine(br);

            // 1st  non-empty, non-comment line is numrows and numcols
            int[] nstuff = ParseUtils.string2ints(currLine, "\t");
            if (nstuff.length != 2) {
                throw new ParserException("file with bad row/col info on line: " + currLine);
            }

            int nrows = nstuff[0];
            int ncols = nstuff[1];

            currLine = nextLine(br);
            List colnames = ParseUtils.string2stringsList(currLine, "\t"); // colnames can have spaces

            colnames.remove(0);            // remove rs#
            colnames.remove(0);            // remove chr
            colnames.remove(0);            // remove pos

            if (colnames.size() != ncols) {
                throw new ParserException("Bad format -- expected ncols from specification on header line: " + ncols + " but found in data: " + colnames.size());
            }

            if (colnames.size() != itemsOfTemplate) {
                throw new ParserException("Bad format -- expected ncols from phenotype labels: " + itemsOfTemplate + " but found in data: " + colnames.size());
            }

            currLine = nextLineTrimless(br);

            //String[] atp = new String[]{"ATP6V1A", "ATP5O", "ATP6V1C1", "ATP6V1E1", "ATP6V1B1", "ATP6V1H", "ATP6V0B", "ATP6V1F", "ATP6V1D", "ATP6V0A1", "ATP6V1B2", "ATP6V0D1", "ATP6V0C", "ATP6V1G2", "ATP6AP1", "ATP6V0E", "ATP6V1G1", "ATP6V0A4"};
            for (int i = 0; i < genes.size(); i++) {
                ArrayList duplicates = new ArrayList();
                ArrayList snpNames = new ArrayList();
                String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
                int geneChr = ((GeneBean) genes.get(i)).getChromosome();
                String geneDes = ((GeneBean) genes.get(i)).getDescription();
                int start = ((GeneBean) genes.get(i)).getStart() - before;
                if (start < 0) {
                    start = 0;
                }
                int end = ((GeneBean) genes.get(i)).getEnd();
                if (end == 0) {
                    continue;
                } else {
                    end = end + after;
                }



                ArrayList probes = new ArrayList(cache);
                if (cache != null && cache.size() != 0) {
                    cache.clear();
                }
                if (probes != null && probes.size() != 0) {
                    for (int j = 0; j < probes.size(); j++) {
                        int pos_c = Integer.parseInt(((ArrayList) probes.get(j)).get(2).toString().trim());
                        if (start <= pos_c) {
                            ArrayList probe_c = (ArrayList) ((ArrayList) probes.get(j)).clone();
                            cache.add(probe_c);
                            String s = (String) ((ArrayList) probes.get(j)).remove(0);
                            snpNames.add(s);
                            duplicates.add(s);
                            ((ArrayList) probes.get(j)).remove(0);
                            ((ArrayList) probes.get(j)).remove(0);
                        } else {
                            probes.remove(j);
                            j--;
                        }

                    }
                }

                while ((currLine != null) && (currLine.length() != 0)) {
                    total++;
                    ArrayList fields = string2stringsV2(currLine, ncols + 3); // spaces allowed in name & desc field so DONT tokenize them

                    if (fields.size() != ncols + 3) {
                        //System.out.println(">> " + fields);
                        throw new ParserException("Bad format - expect ncols: " + (ncols + 3) + " but found: " + fields.size() + " on line >" + currLine + "<\nIf this dataset has missing values, use ImputeDataset to fill these in before importing as a Dataset");
                    }
                    int probeChr = Integer.parseInt(fields.get(1).toString().trim());
                    int probePos = Integer.parseInt(fields.get(2).toString().trim());
                    if (geneChr != probeChr) {
                        if (geneChr < probeChr) {
                            if (cache != null && cache.size() != 0) {
                                cache.clear();
                            }
                            total--;
                            break;
                        } else {
                            if (cache != null && cache.size() != 0) {
                                cache.clear();
                            }
                            if (probes != null && probes.size() != 0) {
                                probes.clear();
                            }
                            if (snpNames != null && snpNames.size() != 0) {
                                snpNames.clear();
                            }
                            if (duplicates != null && duplicates.size() != 0) {
                                duplicates.clear();
                            }
                            currLine = nextLineTrimless(br);
                            continue;
                        }
                    }

                    if (start <= probePos && probePos <= end) {
                        cache.add(fields.clone());

                        snpNames.add(fields.remove(0));
                        fields.remove(0);
                        fields.remove(0);
                        probes.add(fields);
                    }

                    if (end < probePos) {
                        total--;
                        break;
                    }

                    currLine = nextLineTrimless(br);
                }

                //removeHomo(probes, snpNames);

                if (probes != null && probes.size() != 0) {
                    enforceEqualSize(probes);
                    if (flag == 0) {
                        code = getCode(probes);
                        flag++;
                    }

                    ArrayList geneSnps = new ArrayList();
                    for (int j = 0; j < probes.size(); j++) {
                        ArrayList del = new ArrayList();
                        double dist_snp = cmetric.getScore((ArrayList) probes.get(j), template, code);
                        String snpName = (String) snpNames.get(j);

                        del.add(snpName);
                        //System.out.print(snpName+"\t");
                        del.add((new Double(dist_snp)).toString());
                        //System.out.print((new Double(dist_snp)).toString()+"\t");
                        for (int k = 0; k < rndTemplates.length; k++) {
                            double dist_snp_rnd = cmetric.getScore((ArrayList) probes.get(j), rndTemplates[k], code);
                            del.add((new Double(dist_snp_rnd)).toString());
                            //System.out.print((new Double(dist_snp_rnd)).toString()+"\t");
                        }
                        //System.out.println();
                        geneSnps.add(del);
                        //valid++;
                    }

                    if (geneSnps != null && geneSnps.size() != 0) {
                        dels.add((SnpSetAlgorithms.meanList(geneSnps)));
                        GeneSet snpSet = new FSet(geneSymbol, geneDes, snpNames, true);
                        snpSets.add(snpSet);
                    }

                }
            }
            //xlog.info("There are " + total + " probes in the original dataset");
            //xlog.info("Mapping resulted in filtering out " + (total - valid) + "/" + total + " probes, the remainning " + valid + " probes were used in the analysis");
            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

        SnpStruc sstruc = new SnpStruc(dels, (GeneSet[]) snpSets.toArray(new GeneSet[snpSets.size()]));
        return sstruc;
    }

    //apply to average (gene_set)
    public SnpStruc parse_geneset_average(CMetric cmetric, Template template, ArrayList genes, int before, int after, String path, int itemsOfTemplate) {
        int code = 0;
        int flag = 0;

        List snpSets = new ArrayList();
        List dels = new ArrayList();
        //DoubleElement[] datasetSynchedDels = new DoubleElement[snpGeneStruc.size()];
        //String filename = "C://data//mapping.txt";

        int total = 0;
        int valid = 0;
        ArrayList cache = new ArrayList();
        try {
            FileReader input = new FileReader(path);
            BufferedReader br = new BufferedReader(input);
            String currLine = nextLine(br);

            // 1st  non-empty, non-comment line is numrows and numcols
            int[] nstuff = ParseUtils.string2ints(currLine, "\t");
            if (nstuff.length != 2) {
                throw new ParserException("file with bad row/col info on line: " + currLine);
            }

            int nrows = nstuff[0];
            int ncols = nstuff[1];

            currLine = nextLine(br);
            List colnames = ParseUtils.string2stringsList(currLine, "\t"); // colnames can have spaces

            colnames.remove(0);            // remove rs#
            colnames.remove(0);            // remove chr
            colnames.remove(0);            // remove pos

            if (colnames.size() != ncols) {
                throw new ParserException("Bad format -- expected ncols from specification on header line: " + ncols + " but found in data: " + colnames.size());
            }

            if (colnames.size() != itemsOfTemplate) {
                throw new ParserException("Bad format -- expected ncols from phenotype labels: " + itemsOfTemplate + " but found in data: " + colnames.size());
            }

            currLine = nextLineTrimless(br);

            for (int i = 0; i < genes.size(); i++) {
                ArrayList duplicates = new ArrayList();
                ArrayList snpNames = new ArrayList();
                String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
                int geneChr = ((GeneBean) genes.get(i)).getChromosome();
                String geneDes = ((GeneBean) genes.get(i)).getDescription();
                int start = ((GeneBean) genes.get(i)).getStart() - before;
                if (start < 0) {
                    start = 0;
                }
                int end = ((GeneBean) genes.get(i)).getEnd();
                if (end == 0) {
                    continue;
                } else {
                    end = end + after;
                }



                ArrayList probes = new ArrayList(cache);
                if (cache != null && cache.size() != 0) {
                    cache.clear();
                }
                if (probes != null && probes.size() != 0) {
                    for (int j = 0; j < probes.size(); j++) {
                        int pos_c = Integer.parseInt(((ArrayList) probes.get(j)).get(2).toString().trim());
                        if (start <= pos_c) {
                            ArrayList probe_c = (ArrayList) ((ArrayList) probes.get(j)).clone();
                            cache.add(probe_c);
                            String s = (String) ((ArrayList) probes.get(j)).remove(0);
                            snpNames.add(s);
                            duplicates.add(s);
                            ((ArrayList) probes.get(j)).remove(0);
                            ((ArrayList) probes.get(j)).remove(0);
                        } else {
                            probes.remove(j);
                            j--;
                        }

                    }
                }

                while ((currLine != null) && (currLine.length() != 0)) {
                    total++;
                    ArrayList fields = string2stringsV2(currLine, ncols + 3); // spaces allowed in name & desc field so DONT tokenize them

                    if (fields.size() != ncols + 3) {
                        //System.out.println(">> " + fields);
                        throw new ParserException("Bad format - expect ncols: " + (ncols + 3) + " but found: " + fields.size() + " on line >" + currLine + "<\nIf this dataset has missing values, use ImputeDataset to fill these in before importing as a Dataset");
                    }
                    int probeChr = Integer.parseInt(fields.get(1).toString().trim());
                    int probePos = Integer.parseInt(fields.get(2).toString().trim());
                    if (geneChr != probeChr) {
                        if (geneChr < probeChr) {
                            if (cache != null && cache.size() != 0) {
                                cache.clear();
                            }
                            total--;
                            break;
                        } else {
                            if (cache != null && cache.size() != 0) {
                                cache.clear();
                            }
                            if (probes != null && probes.size() != 0) {
                                probes.clear();
                            }
                            if (snpNames != null && snpNames.size() != 0) {
                                snpNames.clear();
                            }
                            if (duplicates != null && duplicates.size() != 0) {
                                duplicates.clear();
                            }
                            currLine = nextLineTrimless(br);
                            continue;
                        }
                    }

                    if (start <= probePos && probePos <= end) {
                        cache.add(fields.clone());

                        snpNames.add(fields.remove(0));
                        fields.remove(0);
                        fields.remove(0);
                        probes.add(fields);
                    }

                    if (end < probePos) {
                        total--;
                        break;
                    }

                    currLine = nextLineTrimless(br);
                }

                //removeHomo(probes, snpNames);

                if (probes != null && probes.size() != 0) {
                    enforceEqualSize(probes);
                    if (flag == 0) {
                        code = getCode(probes);
                        flag++;
                    }

                    ArrayList geneSnps = new ArrayList();
                    for (int j = 0; j < probes.size(); j++) {
                        ArrayList del = new ArrayList();
                        double dist_snp = cmetric.getScore((ArrayList) probes.get(j), template, code);
                        String snpName = (String) snpNames.get(j);

                        del.add(snpName);
                        del.add((new Double(dist_snp)).toString());
                        geneSnps.add(del);
                    }

                    if (geneSnps != null && geneSnps.size() != 0) {
                        dels.add((SnpSetAlgorithms.meanList(geneSnps)));
                        GeneSet snpSet = new FSet(geneSymbol, geneDes, snpNames, true);
                        snpSets.add(snpSet);
                    }

                }
            }
            //xlog.info("There are " + total + " probes in the original dataset");
            //xlog.info("Mapping resulted in filtering out " + (total - valid) + "/" + total + " probes, the remainning " + valid + " probes were used in the analysis");
            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

        SnpStruc sstruc = new SnpStruc(dels, (GeneSet[]) snpSets.toArray(new GeneSet[snpSets.size()]));
        return sstruc;
    }

    protected String nextLine(final BufferedReader bin) throws IOException {
        String currLine = bin.readLine();
        if (currLine == null) {
            return null;
        }

        currLine = currLine.trim();

        while ((currLine != null) && ((currLine.length() == 0) || (currLine.startsWith(Constants.COMMENT_CHAR)))) {
            // System.out.println(">>> " + currLine);
            currLine = bin.readLine();
            if (currLine != null) {
                currLine = currLine.trim();
            }
        }
        return currLine;
    }

    protected String nextLineTrimless(BufferedReader bin) throws IOException {
        String currLine = bin.readLine();
        if (currLine == null) {
            return null;
        }

        //currLine = currLine.trim();

        while ((currLine != null) && ((currLine.length() == 0) || (currLine.startsWith(Constants.COMMENT_CHAR)))) {
            currLine = bin.readLine();
        }

        return currLine;
    }

    //only for A,C,G,T mode
    /*public static void homoRemove(ArrayList profile, ArrayList names) {
    if (profile != null && profile.size() != 0) {
    for (int i = 0; i < profile.size(); i++) {
    ArrayList bases = new ArrayList();
    for (int j = 0; j < ((ArrayList) profile.get(i)).size(); j++) {
    String s = (String) ((ArrayList) profile.get(i)).get(j);
    for (int k = 0; k < s.length(); k++) {
    bases.add(new Character(s.charAt(k)));
    }
    }

    //A, C, G, T, N
    int[] freq = new int[5];
    for (int j = 0; j < bases.size(); j++) {
    char c = ((Character) (bases.get(j))).charValue();
    switch (c) {
    case 'A':
    freq[0]++;
    break;
    case 'C':
    freq[1]++;
    break;
    case 'G':
    freq[2]++;
    break;
    case 'T':
    freq[3]++;
    break;
    case 'N':
    freq[4]++;
    break;
    default:
    break;
    }
    }

    int flag = 0;
    for (int j = 0; j < freq.length - 1; j++) {
    if (freq[j] > 0) {
    flag++;
    }
    }
    if (flag == 1) {
    profile.remove(i);
    names.remove(i);
    i--;
    }
    }
    }
    }*/
    //for genotype code 0,1,2
    public static void removeHomo(ArrayList profile, ArrayList names) {
        if (profile != null && profile.size() != 0) {
            for (int i = 0; i < profile.size(); i++) {
                int flag = 0;
                double first_score = (Double.parseDouble((String) ((ArrayList) profile.get(i)).get(0)));
                for (int j = 1; j < ((ArrayList) profile.get(i)).size(); j++) {
                    double score = (Double.parseDouble((String) ((ArrayList) profile.get(i)).get(j)));
                    if (first_score != score) {
                        flag = 1;
                        break;
                    }
                }
                if (flag == 0) {
                    profile.remove(i);
                    names.remove(i);
                    i--;
                }
            }
        }
    }

    private static void enforceEqualSize(final ArrayList al) {
        if (al.size() == 0) {
            return;
        }

        int size = ((ArrayList) al.get(0)).size();
        for (int i = 0; i < al.size(); i++) {
            if (((ArrayList) al.get(i)).size() != size) {
                throw new IllegalArgumentException("Vector lengths not equal first=" + size + " and y=" + ((ArrayList) al.get(i)).size() + " at index " + i);
            }
        }
    }

    protected ArrayList string2stringsV2(String s, int expectedLen) throws IllegalArgumentException {

        if (null == s) {
            throw new NullPointerException("Cannot work on null String");
        }

        String delim = "\t"; // @note, always
        StringTokenizer tok = new StringTokenizer(s, delim, true); // note including the delim in rets
        ArrayList ret = new ArrayList();
        String prev = null;
        //System.out.println("PARSING>" + s + "<"  + tok.countTokens());

        int cnt = 0;
        while (tok.hasMoreTokens()) {
            final String curr = tok.nextToken(); // dont trim as curr might be a tab!

            if (cnt == 0) { // the first field
                ret.add(curr.trim()); // always add it, empty or not
            } else {
                if (curr.equals(delim)) {
                    if (prev.equals(delim)) { // 2 consecutive tabs
                        ret.add(""); //empty field
                    } else { // omit because its _the delim_
                    }
                } else {
                    ret.add(curr.trim()); // a real word, ok to trim. Then add
                }
            }

            prev = curr;
            cnt++;
        }

        if (ret.size() == expectedLen) {
            return ret;
        } else if (ret.size() < expectedLen) {
            /// fill out whatever's left with empty
            for (int i = ret.size(); i < expectedLen; i++) {
                ret.add("");
            }
            return ret;

        } else {
            // @note added Nov 28, 2005
            // delete any extra tabs (ret.size() > expectedLen
            ArrayList real_ret = new ArrayList();
            for (int i = 0; i < ret.size(); i++) {
                if (i < expectedLen) {
                    real_ret.add(ret.get(i));
                } else {
                    // dont add empty
                    Object obj = ret.get(i);
                    if (obj == null || obj.toString().trim().length() == 0) {
                        // dont add
                    } else {
                        real_ret.add(obj); // cant do anything  might be a genuine format error
                    }
                }
            }

            return real_ret;
        }
    }

    public int getCode(ArrayList al) {
        ArrayList genoLabel = new ArrayList();
        for (int j = 0; j < al.size(); j++) {
            for (int k = 0; k < ((ArrayList) al.get(j)).size(); k++) {
                String s = (String) ((ArrayList) al.get(j)).get(k);
                for (int m = 0; m < s.length(); m++) {
                    String substr = s.substring(m, m + 1);
                    int flag = 0;
                    for (int l = 0; l < genoLabel.size(); l++) {
                        if (substr.equalsIgnoreCase((String) genoLabel.get(l))) {
                            flag++;
                        }
                    }
                    if (flag == 0) {
                        genoLabel.add(s);
                    }
                }

            }
        }

        int code = 0;
        int flag = 0;

        for (int j = 0; j <
                genoLabel.size(); j++) {
            String s = (String) genoLabel.get(j);
            if (s.equalsIgnoreCase("0") || s.equalsIgnoreCase("1") || s.equalsIgnoreCase("2")) {
                flag++;
                code = 1; //0-1-2-3
                break;

            }
            if (s.equalsIgnoreCase("B")) {
                flag++;
                code = 2; //AA-AB-BB
                break;

            }
        }
        if (flag == 0) {
            code = 3;
        }

        return code;
    }

    public double computeCor(ArrayList al1, ArrayList al2) {
        PearsonsCorrelation pc = new PearsonsCorrelation();
        double[] genos1 = new double[al1.size()];
        double[] genos2 = new double[al2.size()];
        for (int i = 0; i < al1.size(); i++) {
            genos1[i] = Double.parseDouble((String) al1.get(i));
            genos2[i] = Double.parseDouble((String) al2.get(i));
        }
        double cor = pc.correlation(genos1, genos2);
        return cor * cor;
    }

    public static ArrayList readData(
            String filename) {
        ArrayList result = new ArrayList();
        try {
            FileReader input = new FileReader(filename);
            BufferedReader br = new BufferedReader(input);
            String s = br.readLine();
            while (s != null) {
                StringTokenizer st = new StringTokenizer(s);
                ArrayList temp = new ArrayList();
                while (st.hasMoreTokens()) {
                    temp.add(st.nextToken().trim());
                }

                result.add(temp);
                s = br.readLine();
            }
            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return result;
    }

    public ArrayList readChrData(int chr, int itemsOfTemplate, String filename) {
        ArrayList snps = new ArrayList();
        int flag = 0;
        try {
            FileReader input = new FileReader(filename);
            BufferedReader br = new BufferedReader(input);
            String s = br.readLine();
            while (s != null) {
                StringTokenizer st = new StringTokenizer(s);
                ArrayList temp = new ArrayList();
                if (flag < 3) {
                    if (chr == 1) {
                        while (st.hasMoreTokens()) {
                            temp.add(st.nextToken().trim());
                        }
                        snps.add(temp);
                    }
                } else {
                    temp.add(st.nextToken().trim());
                    String chr_number = st.nextToken().trim();
                    int chr_number_i = Integer.parseInt(chr_number);
                    if (chr_number_i == chr) {
                        temp.add(chr_number);
                        while (st.hasMoreTokens()) {
                            temp.add(st.nextToken().trim());
                        }
                        snps.add(temp);
                    }
                    if (chr_number_i > chr) {
                        break;
                    }
                }
                flag++;
                s = br.readLine();
            }
            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

        if (chr == 1) {
            String firstrow = (String) ((ArrayList) snps.get(0)).get(0);
            flag = 1;
            if (firstrow.startsWith("#")) {
                snps.remove(0);
                flag++;
            }
            ArrayList nstuff = (ArrayList) snps.remove(0);

            try {
                if (nstuff.size() != 2) {
                    throw new ParserException("file with bad row/col info on line: " + flag);
                }

                int nrows = Integer.parseInt((String) nstuff.get(0));
                int ncols = Integer.parseInt((String) nstuff.get(1));

                ArrayList headers = (ArrayList) snps.remove(0);
                headers.remove(0);            // remove rs#
                headers.remove(0);            // remove chr
                headers.remove(0);            // remove pos
                if (headers.size() != ncols) {
                    throw new ParserException("Bad format -- expected ncols from specification on header line: " + ncols + " but found in data: " + headers.size());
                }

                if (headers.size() != itemsOfTemplate) {
                    throw new ParserException("Bad format -- expected ncols from phenotype labels: " + itemsOfTemplate + " but found in data: " + headers.size());
                }

            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return snps;
    }
}

