/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */

package edu.duke.igsp.app.services;


import org.ensembl.datamodel.*;
import org.ensembl.driver.*;
import org.ensembl.registry.Registry;
import edu.duke.igsp.app.tools.FileTool;
import edu.duke.igsp.app.objects.GeneBean;
import java.util.List;
import java.util.Collections;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.io.*;
import org.apache.commons.beanutils.BeanComparator;
import org.apache.commons.collections.comparators.ComparatorChain;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: April, 2009
 *
 */
public class EnsemblService {

    public EnsemblService() {
    }

    public static ArrayList fetchGenes(String species, String[] geneSymbols) {
        ArrayList genes = new ArrayList();

        try {

            Registry dr = Registry.createDefaultRegistry();
            CoreDriver driver = dr.getGroup(species).getCoreDriver();
            GeneAdaptor geneAdaptor = driver.getGeneAdaptor();

            for (int i = 0; i < geneSymbols.length; i++) {
                GeneBean gene = new GeneBean();
                StringTokenizer st = new StringTokenizer(geneSymbols[i], "///");
                String geneSymbol = st.nextToken().trim();
                List gene_ensembls = geneAdaptor.fetchBySynonym(geneSymbol);
                if (gene_ensembls != null && gene_ensembls.size() != 0) {
                    Gene gene_ensembl = (Gene) gene_ensembls.get(0);
                    gene.setSymbol(geneSymbol);
                    //gene.setAccessionID(gene_ensembl.getAccessionID());
                    String chr = gene_ensembl.getLocation().getSeqRegionName();
                    gene.setChromosome(filter(chr));
                    int start = gene_ensembl.getLocation().getStart();
                    int end = gene_ensembl.getLocation().getEnd();
                    gene.setStart(start);
                    gene.setEnd(end);
                    //gene.setDescription(gene_ensembl.getDescription());
                } else {
                    gene.setSymbol(geneSymbol);
                    gene.setChromosome(27);
                    gene.setStart(0);
                    gene.setEnd(0);
                }

                genes.add(gene);
            }

        } catch (AdaptorException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }

        ArrayList sortFields = new ArrayList();
        sortFields.add(new BeanComparator("chromosome"));
        sortFields.add(new BeanComparator("start"));
        sortFields.add(new BeanComparator("end"));

        ComparatorChain multiSort = new ComparatorChain(sortFields);

        Collections.sort(genes, multiSort);

        return genes;
    }

    public static int filter(String s) {
        String[] chrs = new String[]{"1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21", "22"};
        for (int i = 0; i < chrs.length; i++) {
            if (s.equalsIgnoreCase(chrs[i])) {
                return Integer.parseInt(chrs[i]);
            }
        }
        if (s.equalsIgnoreCase("X")) {
            return 23;
        }
        if (s.equalsIgnoreCase("Y")) {
            return 24;
        }
        if (s.equalsIgnoreCase("MT")) {
            return 25;
        }
        return 26;
    }

    public void fetchAllGenes(String species, String filename) {
        try {
            Registry dr = Registry.createDefaultRegistry();
            CoreDriver driver = dr.getGroup(species).getCoreDriver();
            GeneAdaptor geneAdaptor = driver.getGeneAdaptor();
            List genes = geneAdaptor.fetchAll();

            FileWriter output = new FileWriter(filename);
            BufferedWriter bw = new BufferedWriter(output);
            if (genes != null && genes.size() != 0) {
                for (int i = 0; i < genes.size(); i++) {
                    Gene gene = ((Gene) genes.get(i));
                    GeneBean geneBean = new GeneBean();

                    String accessionId = gene.getAccessionID();
                    if (accessionId != null) {
                        geneBean.setAccessionID(accessionId.trim());
                    } else {
                        geneBean.setAccessionID("NULL");
                    }

                    String displayName = gene.getDisplayName();
                    if (displayName != null) {
                        geneBean.setSymbol(displayName.trim());
                    } else {
                        geneBean.setSymbol("NULL");
                    }

                    String status = gene.getStatus();
                    if (status != null) {
                        geneBean.setStatus(status.trim());
                    } else {
                        geneBean.setStatus("NULL");
                    }

                    String chromosome = gene.getLocation().getSeqRegionName();
                    if (chromosome != null) {
                        geneBean.setChromosome(filter(chromosome.trim()));
                    } else {
                        geneBean.setChromosome(27);
                    }

                    int start = (gene.getLocation()).getStart();
                    geneBean.setStart(start);

                    int end = (gene.getLocation()).getEnd();
                    geneBean.setEnd(end);

                    String description = gene.getDescription();
                    if (description != null) {
                        description = description.substring(0,
                                description.lastIndexOf("["));
                        geneBean.setDescription(description.trim());
                    } else {
                        geneBean.setDescription("NULL");
                    }

                    bw.write(accessionId + "\t" + displayName + "\t" + status + "\t" + chromosome + "\t" + start + "\t" + end + "\t" + description);
                    bw.newLine();
                }
            }
        } catch (AdaptorException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void fetchAllGenesSort(String[] geneSymbols, String filename1, String filename2) {
        ArrayList genes = FileTool.readData(filename1);
        ArrayList dist = new ArrayList();
        try {
            FileWriter output = new FileWriter(filename2);
            BufferedWriter bw = new BufferedWriter(output);
            for (int i = 0; i < geneSymbols.length; i++) {
                String geneName = geneSymbols[i];
                GeneBean gb = new GeneBean();
                int flag = 0;
                for (int j = 0; j < genes.size(); j++) {
                    String gene = (String) ((ArrayList) genes.get(j)).get(2);
                    if (geneName.equalsIgnoreCase(gene)) {
                        flag++;
                        gb.setSymbol(geneName);
                        gb.setChromosome(Integer.parseInt((String) ((ArrayList) genes.get(j)).get(3)));
                        gb.setStart(Integer.parseInt((String) ((ArrayList) genes.get(j)).get(4)));
                        gb.setEnd(Integer.parseInt((String) ((ArrayList) genes.get(j)).get(5)));
                        break;
                    }
                }
                if (flag == 0) {
                    gb.setSymbol(geneName);
                    gb.setChromosome(27);
                    gb.setStart(0);
                    gb.setEnd(0);
                }
                dist.add(gb);
            }
            ArrayList sortFields = new ArrayList();
            sortFields.add(new BeanComparator("chromosome"));
            sortFields.add(new BeanComparator("start"));
            sortFields.add(new BeanComparator("end"));

            ComparatorChain multiSort = new ComparatorChain(sortFields);

            Collections.sort(dist, multiSort);

            for (int i = 0; i < dist.size(); i++) {
                bw.write(((GeneBean) dist.get(i)).toString());
                bw.newLine();
            }
            bw.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void fetchChrGenesSort(String species, String filename) {
        try {
            Registry dr = Registry.createDefaultRegistry();
            CoreDriver driver = dr.getGroup(species).getCoreDriver();
            GeneAdaptor geneAdaptor = driver.getGeneAdaptor();
            CoordinateSystem cs = new CoordinateSystem("chromosome");
            String seqRegName = "7";
            Location loc = new Location(cs, seqRegName);
            List genes = geneAdaptor.fetch(loc);
            System.out.println("gene size " + genes.size());

            ArrayList sortedGenes = new ArrayList();
            FileWriter output = new FileWriter(filename);
            BufferedWriter bw = new BufferedWriter(output);

            if (genes != null && genes.size() != 0) {
                for (int i = 0; i < genes.size(); i++) {
                    Gene gene = ((Gene) genes.get(i));
                    GeneBean geneBean = new GeneBean();
                    String status = gene.getStatus().trim();
                    if (status.equalsIgnoreCase("NOVEL")) {
                        continue;
                    } else {
                        geneBean.setStatus(status);
                    }

                    String accessionId = gene.getAccessionID().trim();
                    geneBean.setAccessionID(accessionId);

                    String displayName = gene.getDisplayName().trim();
                    if (displayName != null) {
                        StringTokenizer st = new StringTokenizer(displayName, "_");
                        geneBean.setSymbol(st.nextToken());
                    } else {
                        geneBean.setSymbol("NULL");
                    }

                    String chromosome = gene.getLocation().getSeqRegionName().trim();
                    geneBean.setChromosome(filter(chromosome));

                    int start = (gene.getLocation()).getStart();
                    geneBean.setStart(start);

                    int end = (gene.getLocation()).getEnd();
                    geneBean.setEnd(end);

                    String description = gene.getDescription();
                    if (description != null) {
                        description = description.substring(0,
                                description.lastIndexOf("["));
                        geneBean.setDescription(description.trim());
                    } else {
                        geneBean.setDescription("NULL");
                    }
                    sortedGenes.add(geneBean);
                }

                ArrayList sortFields = new ArrayList();
                //sortFields.add(new BeanComparator("chromosome"));
                sortFields.add(new BeanComparator("start"));
                //sortFields.add(new BeanComparator("end"));

                ComparatorChain multiSort = new ComparatorChain(sortFields);

                Collections.sort(sortedGenes, multiSort);

                for (int i = 0; i < sortedGenes.size(); i++) {
                    String accessionId = ((GeneBean) sortedGenes.get(i)).getAccessionID();
                    String displayName = ((GeneBean) sortedGenes.get(i)).getSymbol();
                    String status = ((GeneBean) sortedGenes.get(i)).getStatus();
                    int chromosome = ((GeneBean) sortedGenes.get(i)).getChromosome();
                    int start = ((GeneBean) sortedGenes.get(i)).getStart();
                    int end = ((GeneBean) sortedGenes.get(i)).getEnd();
                    String description = ((GeneBean) sortedGenes.get(i)).getDescription();
                    bw.write(accessionId + "\t" + displayName + "\t" + status + "\t" + chromosome + "\t" + start + "\t" + end + "\t" + description);
                    bw.newLine();
                }
            }
            bw.close();

        } catch (AdaptorException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
