/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */
package edu.duke.igsp.app.services;

import edu.duke.igsp.app.objects.GeneBean;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.Vector;
import java.io.*;
import java.util.*;
import org.apache.commons.beanutils.BeanComparator;
import org.apache.commons.collections.comparators.ComparatorChain;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: April, 2009
 *
 */
public class MappingService {

    public static HashMap Mapping(final ArrayList genes, final ArrayList genos, final ArrayList ann, final int before, final int after) {
        HashMap dist = new HashMap();
        int cnt = 0;
        for (int i = 0; i < genes.size(); i++) {
            ArrayList snps = new ArrayList();
            String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
            int geneChr = ((GeneBean) genes.get(i)).getChromosome();
            int start = ((GeneBean) genes.get(i)).getStart() - before;
            if (start < 0) {
                start = 0;
            }
            int end = ((GeneBean) genes.get(i)).getEnd();
            if (end == 0) {
                dist.put(geneSymbol, snps);
                continue;
            } else {
                end = end + after;
            }
            int start_next = 0;
            if (i != genes.size() - 1) {
                start_next = ((GeneBean) genes.get(i + 1)).getStart() - before;
                if (start_next < 0) {
                    start = 0;
                }
            }
            int start_next_index = 0;
            int flag = 0;
            for (int j = cnt; j < genos.size(); j++) {
                int snpChr = Integer.parseInt((String) ((ArrayList) ann.get(j)).get(0));
                int pos = Integer.parseInt((String) ((ArrayList) ann.get(j)).get(1));
                if (geneChr != snpChr) {
                    if (geneChr < snpChr) {
                        cnt = j;
                        break;
                    } else {
                        continue;
                    }
                }
                if (start_next <= pos) {
                    if (flag == 0) {
                        start_next_index = j;
                        flag++;
                    }
                }
                if (start <= pos && pos <= end) {
                    ArrayList snp = new ArrayList();
                    for (int k = 0; k < ((ArrayList) genos.get(j)).size(); k++) {
                        snp.add(((ArrayList) genos.get(j)).get(k));
                    }
                    snps.add(snp);
                }
                if (end < pos) {
                    if (start_next_index == 0) {
                        cnt = j;
                    } else {
                        cnt = start_next_index;
                    }
                    break;
                }
            }

            removeHomo(snps);
            dist.put(geneSymbol, snps);
        }
        return dist;
    }

    //remove genes without probes
    public static HashMap NoZeroMappingNum(ArrayList genes, final ArrayList data, final ArrayList ann, final int before, final int after, final String mappingMode) {
        HashMap dist = new HashMap();
        int cnt = 0;
        for (int i = 0; i < genes.size(); i++) {
            ArrayList probes = new ArrayList();
            String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
            int geneChr = ((GeneBean) genes.get(i)).getChromosome();
            int start = ((GeneBean) genes.get(i)).getStart() - before;
            if (start < 0) {
                start = 0;
            }
            int end = ((GeneBean) genes.get(i)).getEnd();
            if (end == 0) {
                continue;
            } else {
                end = end + after;
            }
            int start_next = 0;
            if (i != genes.size() - 1) {
                start_next = ((GeneBean) genes.get(i + 1)).getStart() - before;
                if (start_next < 0) {
                    start = 0;
                }
            }
            int start_next_index = 0;
            int flag = 0;
            for (int j = cnt; j < data.size(); j++) {
                int probeChr = Integer.parseInt((String) ((ArrayList) ann.get(j)).get(0));
                int probeStart = Integer.parseInt((String) ((ArrayList) ann.get(j)).get(1));
                if (geneChr != probeChr) {
                    if (geneChr < probeChr) {
                        cnt = j;
                        break;
                    } else {
                        continue;
                    }
                }
                if (start_next <= probeStart) {
                    if (flag == 0) {
                        start_next_index = j;
                        flag++;
                    }
                }
                if (start <= probeStart && probeStart <= end) {
                    ArrayList probe = new ArrayList();
                    for (int k = 0; k < ((ArrayList) data.get(j)).size(); k++) {
                        probe.add(((ArrayList) data.get(j)).get(k));
                    }
                    probes.add(probe);
                }
                if (end < probeStart) {
                    if (start_next_index == 0) {
                        cnt = j;
                    } else {
                        cnt = start_next_index;
                    }
                    break;
                }
            }

            if (probes != null && probes.size() != 0) {
                if(mappingMode.equalsIgnoreCase("Sum")){
                    dist.put(geneSymbol, sumList(probes));
                }else if(mappingMode.equalsIgnoreCase("Mean")){
                    dist.put(geneSymbol, meanList(probes));
                }else{
                    throw new IllegalArgumentException("Mapping mode not supported. Got: " + mappingMode);
                }
            }
        }
        return dist;
    }

    //remove genes without probes
    public static HashMap NoZeroMappingNumD(ArrayList genes, final ArrayList data, final ArrayList ann, final int before, final int after, final String mappingMode) {
        HashMap dist = new HashMap();
        int cnt = 0;
        for (int i = 0; i < genes.size(); i++) {
            ArrayList probes = new ArrayList();
            String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
            int geneChr = ((GeneBean) genes.get(i)).getChromosome();
            int start = ((GeneBean) genes.get(i)).getStart() - before;
            if (start < 0) {
                start = 0;
            }
            int end = ((GeneBean) genes.get(i)).getEnd();
            if (end == 0) {
                genes.remove(i);
                i--;
                continue;
            } else {
                end = end + after;
            }
            int start_next = 0;
            if (i != genes.size() - 1) {
                start_next = ((GeneBean) genes.get(i + 1)).getStart() - before;
                if (start_next < 0) {
                    start = 0;
                }
            }
            int start_next_index = 0;
            int flag = 0;
            for (int j = cnt; j < data.size(); j++) {
                int probeChr = Integer.parseInt((String) ((ArrayList) ann.get(j)).get(0));
                int probeStart = Integer.parseInt((String) ((ArrayList) ann.get(j)).get(1));
                if (geneChr != probeChr) {
                    if (geneChr < probeChr) {
                        cnt = j;
                        break;
                    } else {
                        continue;
                    }
                }
                if (start_next <= probeStart) {
                    if (flag == 0) {
                        start_next_index = j;
                        flag++;
                    }
                }
                if (start <= probeStart && probeStart <= end) {
                    ArrayList probe = new ArrayList();
                    for (int k = 0; k < ((ArrayList) data.get(j)).size(); k++) {
                        probe.add(((ArrayList) data.get(j)).get(k));
                    }
                    probes.add(probe);
                }
                if (end < probeStart) {
                    if (start_next_index == 0) {
                        cnt = j;
                    } else {
                        cnt = start_next_index;
                    }
                    break;
                }
            }

            if (probes != null && probes.size() != 0) {
                if(mappingMode.equalsIgnoreCase("Sum")){
                    dist.put(geneSymbol, sumList(probes));
                }else if(mappingMode.equalsIgnoreCase("Mean")){
                    dist.put(geneSymbol, meanList(probes));
                }else{
                    throw new IllegalArgumentException("Mapping mode not supported. Got: " + mappingMode);
                }
            }else{
                genes.remove(i);
                i--;
            }
        }
        return dist;
    }

    public static HashMap NoZeroMapping(final ArrayList genes, final ArrayList genos, final ArrayList ann, final int before, final int after) {
        HashMap dest = new HashMap();
        int cnt = 0;
        for (int i = 0; i < genes.size(); i++) {
            ArrayList snps = new ArrayList();
            String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
            int geneChr = ((GeneBean) genes.get(i)).getChromosome();
            int start = ((GeneBean) genes.get(i)).getStart() - before;
            if (start < 0) {
                start = 0;
            }
            int end = ((GeneBean) genes.get(i)).getEnd();
            if (end == 0) {
                continue;
            } else {
                end = end + after;
            }
            int start_next = 0;
            if (i != genes.size() - 1) {
                start_next = ((GeneBean) genes.get(i + 1)).getStart() - before;
                if (start_next < 0) {
                    start = 0;
                }
            }
            int start_next_index = 0;
            int flag = 0;
            for (int j = cnt; j < genos.size(); j++) {
                int snpChr = Integer.parseInt((String) ((ArrayList) ann.get(j)).get(0));
                int pos = Integer.parseInt((String) ((ArrayList) ann.get(j)).get(1));
                if (geneChr != snpChr) {
                    if (geneChr < snpChr) {
                        cnt = j;
                        break;
                    } else {
                        continue;
                    }
                }
                if (start_next <= pos) {
                    if (flag == 0) {
                        start_next_index = j;
                        flag++;
                    }
                }
                if (start <= pos && pos <= end) {
                    ArrayList snp = new ArrayList();
                    for (int k = 0; k < ((ArrayList) genos.get(j)).size(); k++) {
                        snp.add(((ArrayList) genos.get(j)).get(k));
                    }
                    snps.add(snp);
                }
                if (end < pos) {
                    if (start_next_index == 0) {
                        cnt = j;
                    } else {
                        cnt = start_next_index;
                    }
                    break;
                }
            }

            removeHomo(snps);
            if (snps != null && snps.size() != 0) {
                enforceEqualSize(snps);
                dest.put(geneSymbol, snps);
            }
        }
        return dest;
    }

    //remove genes without snps
    public static HashMap NoZeroMappingSnp(ArrayList genes, final ArrayList genos, final ArrayList ann, final int before, final int after) {
        HashMap dist = new HashMap();
        int cnt = 0;
        for (int i = 0; i < genes.size(); i++) {
            ArrayList snps = new ArrayList();
            String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
            int geneChr = ((GeneBean) genes.get(i)).getChromosome();
            int start = ((GeneBean) genes.get(i)).getStart() - before;
            if (start < 0) {
                start = 0;
            }
            int end = ((GeneBean) genes.get(i)).getEnd();
            if (end == 0) {
                continue;
            } else {
                end = end + after;
            }
            int start_next = 0;
            if (i != genes.size() - 1) {
                start_next = ((GeneBean) genes.get(i + 1)).getStart() - before;
                if (start_next < 0) {
                    start = 0;
                }
            }
            int start_next_index = 0;
            int flag = 0;
            for (int j = cnt; j < genos.size(); j++) {
                int snpChr = Integer.parseInt((String) ((ArrayList) ann.get(j)).get(0));
                int pos = Integer.parseInt((String) ((ArrayList) ann.get(j)).get(1));
                if (geneChr != snpChr) {
                    if (geneChr < snpChr) {
                        cnt = j;
                        break;
                    } else {
                        continue;
                    }
                }
                if (start_next <= pos) {
                    if (flag == 0) {
                        start_next_index = j;
                        flag++;
                    }
                }
                if (start <= pos && pos <= end) {
                    ArrayList snp = new ArrayList();
                    for (int k = 0; k < ((ArrayList) genos.get(j)).size(); k++) {
                        snp.add(((ArrayList) genos.get(j)).get(k));
                    }
                    snps.add(snp);
                }
                if (end < pos) {
                    if (start_next_index == 0) {
                        cnt = j;
                    } else {
                        cnt = start_next_index;
                    }
                    break;
                }
            }

            removeHomo(snps);
            if (snps != null && snps.size() != 0) {
                enforceEqualSize(snps);
                dist.put(geneSymbol, snps);
            }
        }
        return dist;
    }

    //remove genes without snps
    public static HashMap NoZeroMappingSnpD(ArrayList genes, final ArrayList genos, final ArrayList ann, final int before, final int after) {
        HashMap dist = new HashMap();
        int cnt = 0;
        for (int i = 0; i < genes.size(); i++) {
            ArrayList snps = new ArrayList();
            String geneSymbol = ((GeneBean) genes.get(i)).getSymbol();
            int geneChr = ((GeneBean) genes.get(i)).getChromosome();
            int start = ((GeneBean) genes.get(i)).getStart() - before;
            if (start < 0) {
                start = 0;
            }
            int end = ((GeneBean) genes.get(i)).getEnd();
            if (end == 0) {
                genes.remove(i);
                i--;
                continue;
            } else {
                end = end + after;
            }
            int start_next = 0;
            if (i != genes.size() - 1) {
                start_next = ((GeneBean) genes.get(i + 1)).getStart() - before;
                if (start_next < 0) {
                    start = 0;
                }
            }
            int start_next_index = 0;
            int flag = 0;
            for (int j = cnt; j < genos.size(); j++) {
                int snpChr = Integer.parseInt((String) ((ArrayList) ann.get(j)).get(0));
                int pos = Integer.parseInt((String) ((ArrayList) ann.get(j)).get(1));
                if (geneChr != snpChr) {
                    if (geneChr < snpChr) {
                        cnt = j;
                        break;
                    } else {
                        continue;
                    }
                }
                if (start_next <= pos) {
                    if (flag == 0) {
                        start_next_index = j;
                        flag++;
                    }
                }
                if (start <= pos && pos <= end) {
                    ArrayList snp = new ArrayList();
                    for (int k = 0; k < ((ArrayList) genos.get(j)).size(); k++) {
                        snp.add(((ArrayList) genos.get(j)).get(k));
                    }
                    snps.add(snp);
                }
                if (end < pos) {
                    if (start_next_index == 0) {
                        cnt = j;
                    } else {
                        cnt = start_next_index;
                    }
                    break;
                }
            }

            removeHomo(snps);
            if (snps != null && snps.size() != 0) {
                enforceEqualSize(snps);
                dist.put(geneSymbol, snps);
            }else{
                genes.remove(i);
                i--;
            }
        }
        return dist;
    }

    public static void removeHomo(ArrayList profile) {
        if (profile != null && profile.size() != 0) {
            for (int i = 0; i < profile.size(); i++) {
                ArrayList bases = new ArrayList();
                for (int j = 0; j < ((ArrayList) profile.get(i)).size(); j++) {
                    String s = (String) ((ArrayList) profile.get(i)).get(j);
                    for (int k = 0; k < s.length(); k++) {
                        bases.add(new Character(s.charAt(k)));
                    }
                }

                //A, C, G, T, N
                int[] freq = new int[5];
                for (int j = 0; j < bases.size(); j++) {
                    char c = ((Character) (bases.get(j))).charValue();
                    switch (c) {
                        case 'A':
                            freq[0]++;
                            break;
                        case 'C':
                            freq[1]++;
                            break;
                        case 'G':
                            freq[2]++;
                            break;
                        case 'T':
                            freq[3]++;
                            break;
                        case 'N':
                            freq[4]++;
                            break;
                        default:
                            break;
                    }
                }

                int flag = 0;
                for (int j = 0; j < freq.length - 1; j++) {
                    if (freq[j] > 0) {
                        flag++;
                    }
                }
                if (flag == 1) {
                    profile.remove(i);
                    i--;
                }
            }
        }
    }

    public static void lookUp(final String[] geneSymbols, final String filename1, final String filename2) {
        ArrayList genes = readData(filename1);
        ArrayList dist = new ArrayList();
        try {
            FileWriter output = new FileWriter(filename2);
            BufferedWriter bw = new BufferedWriter(output);
            for (int i = 0; i < geneSymbols.length; i++) {
                String geneName = geneSymbols[i];
                int flag = 0;
                for (int j = 0; j < genes.size(); j++) {
                    String gene = (String) ((ArrayList) genes.get(j)).get(0);
                    if (geneName.equalsIgnoreCase(gene)) {
                        flag++;
                        GeneBean gb = new GeneBean();
                        gb.setSymbol(geneName);
                        gb.setChromosome(Integer.parseInt((String) ((ArrayList) genes.get(j)).get(1)));
                        gb.setStart(Integer.parseInt((String) ((ArrayList) genes.get(j)).get(2)));
                        gb.setEnd(Integer.parseInt((String) ((ArrayList) genes.get(j)).get(3)));
                        gb.setStrand((String)((ArrayList) genes.get(j)).get(4));
                        dist.add(gb);
                        break;
                    }
                }
                if (flag == 0) {
                    GeneBean gb = new GeneBean();
                    gb.setSymbol(geneName);
                    gb.setChromosome(27);
                    gb.setStart(0);
                    gb.setEnd(0);
                    gb.setStrand("1");
                    dist.add(gb);
                }
            }

            ArrayList sortFields = new ArrayList();
            sortFields.add(new BeanComparator("chromosome"));
            sortFields.add(new BeanComparator("start"));
            sortFields.add(new BeanComparator("end"));

            ComparatorChain multiSort = new ComparatorChain(sortFields);

            Collections.sort(dist, multiSort);

            for (int i = 0; i < dist.size(); i++) {
                String displayName = ((GeneBean) dist.get(i)).getSymbol();
                int chromosome = ((GeneBean) dist.get(i)).getChromosome();
                int start = ((GeneBean) dist.get(i)).getStart();
                int end = ((GeneBean) dist.get(i)).getEnd();
                String strand = ((GeneBean) dist.get(i)).getStrand();
                bw.write(displayName + "\t" + chromosome + "\t" + start + "\t" + end + "\t" + strand);
                bw.newLine();
            }
            bw.close();

        } catch (IOException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static ArrayList meanList(final ArrayList al) {
        enforceEqualSize(al);
        int size = ((ArrayList)al.get(0)).size();
        ArrayList means = new ArrayList();

        for (int i = 0; i < size; i++) {
            ArrayList col = new ArrayList();
            for (int j = 0; j < al.size(); j++) {
                col.add(((ArrayList)al.get(j)).get(i));
            }

            means.add(mean(col));
        }

        return means;
    }

    public static ArrayList sumList(final ArrayList al) {
        enforceEqualSize(al);
        int size = ((ArrayList)al.get(0)).size();
        ArrayList sums = new ArrayList();

        for (int i = 0; i < size; i++) {
            ArrayList col = new ArrayList();
            for (int j = 0; j < al.size(); j++) {
                col.add(((ArrayList)al.get(j)).get(i));
            }

            sums.add(sum(col));
        }

        return sums;
    }

    public static float sum(final ArrayList a){
        float sum =0;
        for(int i=0; i<a.size(); i++){
            float f = ((Float)a.get(i)).floatValue();
            sum+=f;
        }
        return sum;
    }

    public static float mean(final ArrayList a){
        int size = a.size();
        return sum(a)/size;
    }

    private static void enforceEqualSize(final ArrayList al) {
        if (al.size() == 0) {
            return;
        }

        int size = ((ArrayList)al.get(0)).size();
        for (int i = 0; i < al.size(); i++) {
            if (((ArrayList)al.get(i)).size() != size) {
                throw new IllegalArgumentException("Vector lengths not equal first=" + size + " and y=" + ((ArrayList)al.get(i)).size() + " at index " + i);
            }
        }
    }

    public static ArrayList lookUp(final String[] geneSymbols, final String filename) {
        ArrayList genes = readData(filename);
        ArrayList dist = new ArrayList();
        for (int i = 0; i < geneSymbols.length; i++) {

            String geneName = geneSymbols[i];
            //delete "///"
            String gn = (new StringTokenizer(geneSymbols[i], "///")).nextToken().trim();

            int flag = 0;
            for (int j = 0; j < genes.size(); j++) {
                String gene = (String) ((ArrayList) genes.get(j)).get(0);
                if (gn.equalsIgnoreCase(gene)) {
                    flag++;
                    GeneBean gb = new GeneBean();
                    gb.setSymbol(geneName);
                    gb.setChromosome(Integer.parseInt((String) ((ArrayList) genes.get(j)).get(1)));
                    gb.setStart(Integer.parseInt((String) ((ArrayList) genes.get(j)).get(2)));
                    gb.setEnd(Integer.parseInt((String) ((ArrayList) genes.get(j)).get(3)));
                    gb.setStrand((String)((ArrayList) genes.get(j)).get(4));
                    dist.add(gb);
                    break;
                }
            }
            if (flag == 0) {
                GeneBean gb = new GeneBean();
                gb.setSymbol(geneName);
                gb.setChromosome(27);
                gb.setStart(0);
                gb.setEnd(0);
                gb.setStrand("1");
                dist.add(gb);
            }
        }

        ArrayList sortFields = new ArrayList();
        sortFields.add(new BeanComparator("chromosome"));
        sortFields.add(new BeanComparator("start"));
        sortFields.add(new BeanComparator("end"));

        ComparatorChain multiSort = new ComparatorChain(sortFields);

        Collections.sort(dist, multiSort);

        return dist;
    }

    public static ArrayList lookUp(final String[] geneSymbols, final InputStreamReader isr) {
        ArrayList genes = readData(isr);
        ArrayList dist = new ArrayList();
        for (int i = 0; i < geneSymbols.length; i++) {

            String geneName = geneSymbols[i];
            //delete "///"
            String gn = (new StringTokenizer(geneName, "///")).nextToken().trim();

            int flag = 0;
            for (int j = 0; j < genes.size(); j++) {
                String gene = (String) ((ArrayList) genes.get(j)).get(0);
                if (gn.equalsIgnoreCase(gene)) {
                    flag++;
                    GeneBean gb = new GeneBean();
                    gb.setSymbol(geneName);
                    gb.setChromosome(Integer.parseInt((String) ((ArrayList) genes.get(j)).get(1)));
                    gb.setStart(Integer.parseInt((String) ((ArrayList) genes.get(j)).get(2)));
                    gb.setEnd(Integer.parseInt((String) ((ArrayList) genes.get(j)).get(3)));
                    gb.setStrand((String)((ArrayList) genes.get(j)).get(4));
                    dist.add(gb);
                    break;
                }
            }
            if (flag == 0) {
                GeneBean gb = new GeneBean();
                gb.setSymbol(geneName);
                gb.setChromosome(27);
                gb.setStart(0);
                gb.setEnd(0);
                gb.setStrand("1");
                dist.add(gb);
            }
        }

        ArrayList sortFields = new ArrayList();
        sortFields.add(new BeanComparator("chromosome"));
        sortFields.add(new BeanComparator("start"));
        sortFields.add(new BeanComparator("end"));

        ComparatorChain multiSort = new ComparatorChain(sortFields);

        Collections.sort(dist, multiSort);

        return dist;
    }

    public static HashMap lookUpCqn(List geneSymbols, Matrix matrix, InputStreamReader isr) {
        ArrayList genes = readData(isr);
        ArrayList dist = new ArrayList();
        ArrayList counts = new ArrayList();
        List symbols = new ArrayList();
        for (int i = 0; i < geneSymbols.size(); i++) {
            String geneName = (String) geneSymbols.get(i);
            //delete "///"
            String gn = (new StringTokenizer(geneName, "///")).nextToken().trim();
            for (int j = 0; j < genes.size(); j++) {
                String gene = (String) ((ArrayList) genes.get(j)).get(6);
                if (gn.equalsIgnoreCase(gene)) {
                    ArrayList ginfo = new ArrayList();
                    ginfo.add(geneName);
                    int start = Integer.parseInt((String) ((ArrayList) genes.get(j)).get(3));
                    int end = Integer.parseInt((String) ((ArrayList) genes.get(j)).get(4));
                    int geneLength = Math.abs(end-start)+1;
                    ginfo.add((new Integer(geneLength)).toString());
                    ginfo.add((String)((ArrayList) genes.get(j)).get(7));
                    dist.add(ginfo);
                    Vector v = matrix.getRowV(i);
                    counts.add(v);
                    symbols.add(gn);
                    break;
                }
            }
        }

        Matrix newMatrix = new Matrix(counts.size(), matrix.getNumCol());
        for(int i=0; i<newMatrix.getNumRow(); i++){
            Vector v = (Vector)counts.get(i);
            newMatrix.setRow(i, v);
        }
        
        HashMap hm = new HashMap();
        hm.put("GENEINFO", dist);
        hm.put("GENESYMBOLS", symbols);
        hm.put("MATRIX", newMatrix);
        return hm;
    }

    public static ArrayList readData(
            String filename) {
        ArrayList result = new ArrayList();
        try {
            FileReader input = new FileReader(filename);
            BufferedReader br = new BufferedReader(input);
            String s = br.readLine();
            while (s != null) {
                StringTokenizer st = new StringTokenizer(s);
                ArrayList temp = new ArrayList();
                while (st.hasMoreTokens()) {
                    temp.add(st.nextToken());
                }

                result.add(temp);
                s = br.readLine();
            }

            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return result;
    }

    public static ArrayList readData(
            InputStreamReader isr) {
        ArrayList result = new ArrayList();
        try {
            BufferedReader br = new BufferedReader(isr);
            String s = br.readLine();
            while (s != null) {
                StringTokenizer st = new StringTokenizer(s, "\t");
                ArrayList temp = new ArrayList();
                while (st.hasMoreTokens()) {
                    temp.add(st.nextToken());
                }

                result.add(temp);
                s = br.readLine();
            }

            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return result;
    }
}
