/**
 * Department of Genetics SOFTWARE COPYRIGHT NOTICE AGREEMENT This software and
 * its documentation are copyright 2012 by the Department of Genetics/University
 * of North Carolina at Chapel Hill. All rights are reserved. This software is
 * supplied without any warranty or guaranteed support whatsoever. Neither
 * Department of Genetics nor University of North Carolina at Chapel Hill can be
 * responsible for its use, misuse, or functionality.
 */
package edu.duke.igsp.app.tools;

import java.util.*;
import java.math.BigDecimal;
import java.math.MathContext;
import org.apache.commons.math.analysis.interpolation.LoessInterpolator;

/**
 * @author Qing Xiong < qxiong@email.unc.edu >
 * Date: December, 2011
 *
 */
public class BigDecimalTool {

    public static void sortAsc(ArrayList al, final int sortby) {
        Collections.sort(al, new Comparator<List>() {
            public int compare(List o1, List o2) {
                BigDecimal value1 = (BigDecimal) ((ArrayList) o1).get(sortby);
                BigDecimal value2 = (BigDecimal) ((ArrayList) o2).get(sortby);
                int i = value1.compareTo(value2);
                if (i == 1) {
                    return 1;
                } else if (i == -1) {
                    return -1;
                } else {
                    return 0;
                }
            }
        });
    }

    public static void sortDesc(ArrayList al, final int sortby) {
        Collections.sort(al, new Comparator<List>() {
            public int compare(List o1, List o2) {
                BigDecimal value1 = (BigDecimal) ((ArrayList) o1).get(sortby);
                BigDecimal value2 = (BigDecimal) ((ArrayList) o2).get(sortby);
                int i = value1.compareTo(value2);
                if (i == 1) {
                    return -1;
                } else if (i == -1) {
                    return 1;
                } else {
                    return 0;
                }
            }
        });
    }

    public static ArrayList string2bigdecimal(ArrayList al) {
        ArrayList bd = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList temp = new ArrayList();
            for (int j = 0; j < ((ArrayList) al.get(i)).size(); j++) {
                String value_s = (String) ((ArrayList) al.get(i)).get(j);
                BigDecimal value_bd = new BigDecimal(value_s);
                temp.add(value_bd);
            }
            bd.add(temp);
        }
        return bd;
    }

    public static ArrayList calcpMoreThan(ArrayList al) {
        ArrayList ps = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList values = (ArrayList) al.get(i);
            ArrayList temp = new ArrayList();
            for (int j = 0; j < values.size(); j++) {
                BigDecimal bd = (BigDecimal) values.get(j);
                float p = getPValueMoreThan(bd, values);
                temp.add(new Float(p));
            }
            ps.add(temp);
        }
        return ps;
    }

    public static ArrayList calcpLessThan(ArrayList al) {
        ArrayList ps = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList values = (ArrayList) al.get(i);
            ArrayList temp = new ArrayList();
            for (int j = 0; j < values.size(); j++) {
                BigDecimal bd = (BigDecimal) values.get(j);
                float p = getPValueLessThan(bd, values);
                temp.add(new Float(p));
            }
            ps.add(temp);
        }
        return ps;
    }

    public static ArrayList calcpLessThanB(ArrayList al) {
        ArrayList ps = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList values = (ArrayList) al.get(i);
            ArrayList temp = new ArrayList();
            for (int j = 0; j < values.size(); j++) {
                BigDecimal bd = (BigDecimal) values.get(j);
                BigDecimal p = getPValueLessThanB(bd, values);
                temp.add(p);
            }
            ps.add(temp);
        }
        return ps;
    }

    public static float getPValueLessThan(BigDecimal score, ArrayList values) {
        int cntless = 0;
        int size = values.size();

        for (int i = 0; i < size; i++) {
            BigDecimal value = (BigDecimal) values.get(i);
            int sign = value.compareTo(score);
            if (sign == -1 || sign == 0) {
                cntless++;
            }
        }
        return ((float) cntless) / (float) size;
    }

    public static BigDecimal getPValueLessThanB(BigDecimal score, ArrayList values) {
        int cntless = 0;
        int size = values.size();

        for (int i = 0; i < size; i++) {
            BigDecimal value = (BigDecimal) values.get(i);
            int sign = value.compareTo(score);
            if (sign == -1 || sign == 0) {
                cntless++;
            }
        }
        return new BigDecimal(((float) cntless) / (float) size);
    }

    public static float getPValueMoreThan(BigDecimal score, ArrayList values) {
        int cntmore = 0;
        int size = values.size();

        for (int i = 0; i < size; i++) {
            BigDecimal value = (BigDecimal) values.get(i);
            int sign = value.compareTo(score);
            if (sign == 1 || sign == 0) {
                cntmore++;
            }
        }
        return ((float) cntmore) / (float) size;
    }

    public static float getPValueLessThan(BigDecimal score, BigDecimal[] values) {
        int cntless = 0;
        int size = values.length;

        for (int i = 0; i < size; i++) {
            int sign = values[i].compareTo(score);
            if (sign == -1 || sign == 0) {
                cntless++;
            }
        }
        return ((float) cntless) / (float) size;
    }

    public static float getPValueMoreThan(BigDecimal score, BigDecimal[] values) {
        int cntmore = 0;
        int size = values.length;

        for (int i = 0; i < size; i++) {
            int sign = values[i].compareTo(score);
            if (sign == 1 || sign == 0) {
                cntmore++;
            }
        }
        return ((float) cntmore) / (float) size;
    }

    public static float[] calcFdr(BigDecimal[][] scores) {
        float[] fdrs = new float[scores.length];
        for (int i = 0; i < scores.length; i++) {
            int lessRealThan = 0;
            int totalRealCnt = scores.length;
            int lessRndThan = 0;
            int totalRndCnt = scores.length * (scores[0].length - 1);
            for (int j = 0; j < scores[i].length; j++) {
                for (int k = 0; k < scores.length; k++) {
                    int sign = scores[k][j].compareTo(scores[i][0]);
                    if (j == 0) {
                        if (sign == -1 || sign == 0) {
                            lessRealThan++;
                        }
                    } else {
                        if (sign == -1 || sign == 0) {
                            lessRndThan++;
                        }
                    }
                }
            }
            float denr = (float) lessRealThan / (float) totalRealCnt;
            float numr = (float) lessRndThan / (float) totalRndCnt;
            fdrs[i] = numr / denr;

            if (fdrs[i] > 1.0f) {
                fdrs[i] = 1.0f;
            }
        }
        return fdrs;
    }

    public static BigDecimal[] calcFdrB(BigDecimal[][] scores) {
        BigDecimal[] fdrs = new BigDecimal[scores.length];
        for (int i = 0; i < scores.length; i++) {
            int lessRealThan = 0;
            int totalRealCnt = scores.length;
            int lessRndThan = 0;
            int totalRndCnt = scores.length * (scores[0].length - 1);
            for (int j = 0; j < scores[i].length; j++) {
                for (int k = 0; k < scores.length; k++) {
                    int sign = scores[k][j].compareTo(scores[i][0]);
                    if (j == 0) {
                        if (sign == -1 || sign == 0) {
                            lessRealThan++;
                        }
                    } else {
                        if (sign == -1 || sign == 0) {
                            lessRndThan++;
                        }
                    }
                }
            }

            //BigDecimal denr = (new BigDecimal(String.valueOf(lessRealThan))).divide(new BigDecimal(String.valueOf(totalRealCnt)), 6, BigDecimal.ROUND_HALF_EVEN);
            //BigDecimal numr = (new BigDecimal(String.valueOf(lessRndThan))).divide(new BigDecimal(String.valueOf(totalRndCnt)), 6, BigDecimal.ROUND_HALF_EVEN);            
            //fdrs[i] = numr.divide(denr, 6, BigDecimal.ROUND_HALF_EVEN);        

            double denr = (double) lessRealThan / (double) totalRealCnt;
            double numr = (double) lessRndThan / (double) totalRndCnt;
            fdrs[i] = new BigDecimal(String.valueOf(numr / denr));

            if (fdrs[i].compareTo(BigDecimal.ONE) == 1) {
                fdrs[i] = new BigDecimal(String.valueOf("1.0"));
            }
        }
        return fdrs;
    }

    public static BigDecimal[] calcFdrBDirec(BigDecimal[][] scores) {
        BigDecimal[] fdrs = new BigDecimal[scores.length];
        for (int i = 0; i < scores.length; i++) {
            int lessRealThan = 0;
            int totalRealCnt = 0;
            int lessRndThan = 0;
            int totalRndCnt = 0;
            int sign_real = scores[i][0].compareTo(BigDecimal.ZERO);
            if (sign_real == -1) {
                for (int j = 0; j < scores[i].length; j++) {
                    for (int k = 0; k < scores.length; k++) {
                        int sign_rand = scores[k][j].compareTo(BigDecimal.ZERO);
                        if (sign_rand == sign_real) {
                            int sign = scores[k][j].compareTo(scores[i][0]);
                            if (j == 0) {
                                totalRealCnt++;
                                if (sign == 1 || sign == 0) {
                                    lessRealThan++;
                                }
                            } else {
                                totalRndCnt++;
                                if (sign == 1 || sign == 0) {
                                    lessRndThan++;
                                }
                            }
                        }
                    }
                }
            } else {
                for (int j = 0; j < scores[i].length; j++) {
                    for (int k = 0; k < scores.length; k++) {
                        int sign_rand = scores[k][j].compareTo(BigDecimal.ZERO);
                        if (sign_rand == sign_real || sign_rand == 0) {
                            int sign = scores[k][j].compareTo(scores[i][0]);
                            if (j == 0) {
                                totalRealCnt++;
                                if (sign == -1 || sign == 0) {
                                    lessRealThan++;
                                }
                            } else {
                                totalRndCnt++;
                                if (sign == -1 || sign == 0) {
                                    lessRndThan++;
                                }
                            }
                        }
                    }
                }
            }

            double denr = (double) lessRealThan / (double) totalRealCnt;
            double numr = (double) lessRndThan / (double) totalRndCnt;
            fdrs[i] = new BigDecimal(String.valueOf(numr / denr));

            if (fdrs[i].compareTo(BigDecimal.ONE) == 1) {
                fdrs[i] = new BigDecimal(String.valueOf("1.0"));
            }
        }
        return fdrs;
    }

    public static BigDecimal[] calcFdrBMoreThan(BigDecimal[][] scores) {
        BigDecimal[] fdrs = new BigDecimal[scores.length];
        for (int i = 0; i < scores.length; i++) {
            int moreRealThan = 0;
            int totalRealCnt = scores.length;
            int moreRndThan = 0;
            int totalRndCnt = scores.length * (scores[0].length - 1);
            for (int j = 0; j < scores[i].length; j++) {
                for (int k = 0; k < scores.length; k++) {
                    int sign = scores[k][j].compareTo(scores[i][0]);
                    if (j == 0) {
                        if (sign == 1 || sign == 0) {
                            moreRealThan++;
                        }
                    } else {
                        if (sign == 1 || sign == 0) {
                            moreRndThan++;
                        }
                    }
                }
            }

            //BigDecimal denr = (new BigDecimal(String.valueOf(lessRealThan))).divide(new BigDecimal(String.valueOf(totalRealCnt)), 6, BigDecimal.ROUND_HALF_EVEN);
            //BigDecimal numr = (new BigDecimal(String.valueOf(lessRndThan))).divide(new BigDecimal(String.valueOf(totalRndCnt)), 6, BigDecimal.ROUND_HALF_EVEN);            
            //fdrs[i] = numr.divide(denr, 6, BigDecimal.ROUND_HALF_EVEN);        

            double denr = (double) moreRealThan / (double) totalRealCnt;
            double numr = (double) moreRndThan / (double) totalRndCnt;
            fdrs[i] = new BigDecimal(String.valueOf(numr / denr));

            if (fdrs[i].compareTo(BigDecimal.ONE) == 1) {
                fdrs[i] = new BigDecimal(String.valueOf("1.0"));
            }
        }
        return fdrs;
    }

    public static BigDecimal[] calcFdrBMoreThanDirec(BigDecimal[][] scores) {
        BigDecimal[] fdrs = new BigDecimal[scores.length];
        for (int i = 0; i < scores.length; i++) {
            int moreRealThan = 0;
            int totalRealCnt = 0;
            int moreRndThan = 0;
            int totalRndCnt = 0;

            int sign_real = scores[i][0].compareTo(BigDecimal.ZERO);
            if (sign_real == -1) {
                for (int j = 0; j < scores[i].length; j++) {
                    for (int k = 0; k < scores.length; k++) {
                        int sign_rand = scores[k][j].compareTo(BigDecimal.ZERO);
                        if (sign_rand == sign_real) {
                            int sign = scores[k][j].compareTo(scores[i][0]);
                            if (j == 0) {
                                totalRealCnt++;
                                if (sign == -1 || sign == 0) {
                                    moreRealThan++;
                                }
                            } else {
                                totalRndCnt++;
                                if (sign == -1 || sign == 0) {
                                    moreRndThan++;
                                }
                            }
                        }
                    }
                }
            } else {
                for (int j = 0; j < scores[i].length; j++) {
                    for (int k = 0; k < scores.length; k++) {
                        int sign_rand = scores[k][j].compareTo(BigDecimal.ZERO);
                        if (sign_rand == sign_real || sign_rand == 0) {
                            int sign = scores[k][j].compareTo(scores[i][0]);
                            if (j == 0) {
                                totalRealCnt++;
                                if (sign == 1 || sign == 0) {
                                    moreRealThan++;
                                }
                            } else {
                                totalRndCnt++;
                                if (sign == 1 || sign == 0) {
                                    moreRndThan++;
                                }
                            }
                        }
                    }
                }
            }

            double denr = (double) moreRealThan / (double) totalRealCnt;
            double numr = (double) moreRndThan / (double) totalRndCnt;
            fdrs[i] = new BigDecimal(String.valueOf(numr / denr));

            if (fdrs[i].compareTo(BigDecimal.ONE) == 1) {
                fdrs[i] = new BigDecimal(String.valueOf("1.0"));
            }
        }
        return fdrs;
    }

    public static float[] calcFwer(BigDecimal[][] scores) {
        float[] fwers = new float[scores.length];
        BigDecimal[] mins = new BigDecimal[scores[0].length];
        for (int i = 0; i < scores[0].length; i++) {
            BigDecimal min = scores[0][i];
            for (int j = 1; j < scores.length; j++) {
                int sign = scores[j][i].compareTo(min);
                if (sign == -1 || sign == 0) {
                    min = scores[j][i];
                }
            }
            mins[i] = min;
        }

        for (int i = 0; i < scores.length; i++) {
            int lessThan = 0;
            for (int j = 1; j < mins.length; j++) {
                int sign = scores[i][0].compareTo(mins[j]);
                if (sign == 1 || sign == 0) {
                    lessThan++;
                }
            }
            fwers[i] = (float) lessThan / (float) (mins.length - 1);
        }
        return fwers;
    }

    public static BigDecimal[] calcFwerBMoreThan(BigDecimal[][] scores) {
        BigDecimal[] fwers = new BigDecimal[scores.length];
        BigDecimal[] maxes = new BigDecimal[scores[0].length];
        for (int i = 0; i < scores[0].length; i++) {
            BigDecimal max = scores[0][i];
            for (int j = 1; j < scores.length; j++) {
                int sign = scores[j][i].compareTo(max);
                if (sign == 1 || sign == 0) {
                    max = scores[j][i];
                }
            }
            maxes[i] = max;
        }

        for (int i = 0; i < scores.length; i++) {
            int moreThan = 0;
            for (int j = 1; j < maxes.length; j++) {
                int sign = scores[i][0].compareTo(maxes[j]);
                if (sign == -1 || sign == 0) {
                    moreThan++;
                }
            }
            //fwers[i] = (new BigDecimal(String.valueOf(lessThan))).divide(new BigDecimal(String.valueOf(mins.length - 1)), 6, BigDecimal.ROUND_HALF_EVEN);            
            fwers[i] = new BigDecimal(String.valueOf((double) moreThan / (double) (maxes.length - 1)));
        }
        return fwers;
    }

    public static BigDecimal[] calcFwerBMoreThanDirec(BigDecimal[][] scores) {
        BigDecimal[] fwers = new BigDecimal[scores.length];
        BigDecimal[] maxes_pos = new BigDecimal[scores[0].length];
        BigDecimal[] maxes_neg = new BigDecimal[scores[0].length];

        for (int i = 0; i < scores[0].length; i++) {
            BigDecimal max_pos = BigDecimal.ZERO;
            BigDecimal max_neg = BigDecimal.ZERO;
            int index1 = 0;
            int index2 = 0;
            for (int j = 0; j < scores.length; j++) {
                int sign_rand = scores[j][i].compareTo(BigDecimal.ZERO);
                if (sign_rand == 1 || sign_rand == 0) {
                    if (index1 == 0) {
                        max_pos = scores[j][i];
                        index1++;
                    } else {
                        int sign = scores[j][i].compareTo(max_pos);
                        if (sign == 1) {
                            max_pos = scores[j][i];
                        }
                    }
                } else {
                    if (index2 == 0) {
                        max_neg = scores[j][i];
                        index2++;
                    } else {
                        int sign = scores[j][i].compareTo(max_neg);
                        if (sign == -1) {
                            max_neg = scores[j][i];
                        }
                    }
                }
            }
            maxes_pos[i] = max_pos;
            maxes_neg[i] = max_neg;
        }

        for (int i = 0; i < scores.length; i++) {
            int sign_real = scores[i][0].compareTo(BigDecimal.ZERO);
            int moreThan1 = 0;
            int moreThan2 = 0;
            if (sign_real == -1) {
                for (int j = 1; j < maxes_neg.length; j++) {
                    int sign = scores[i][0].compareTo(maxes_neg[j]);
                    if (sign == 1 || sign == 0) {
                        moreThan1++;
                    }
                }
                fwers[i] = new BigDecimal(String.valueOf((double) moreThan1 / (double) (maxes_neg.length - 1)));
            } else {
                for (int j = 1; j < maxes_pos.length; j++) {
                    int sign = scores[i][0].compareTo(maxes_pos[j]);
                    if (sign == -1 || sign == 0) {
                        moreThan2++;
                    }
                }
                fwers[i] = new BigDecimal(String.valueOf((double) moreThan2 / (double) (maxes_pos.length - 1)));
            }
        }

        return fwers;
    }

    public static BigDecimal[] calcFwerB(BigDecimal[][] scores) {
        BigDecimal[] fwers = new BigDecimal[scores.length];
        BigDecimal[] mins = new BigDecimal[scores[0].length];
        for (int i = 0; i < scores[0].length; i++) {
            BigDecimal min = scores[0][i];
            for (int j = 1; j < scores.length; j++) {
                int sign = scores[j][i].compareTo(min);
                if (sign == -1 || sign == 0) {
                    min = scores[j][i];
                }
            }
            mins[i] = min;
        }

        for (int i = 0; i < scores.length; i++) {
            int lessThan = 0;
            for (int j = 1; j < mins.length; j++) {
                int sign = scores[i][0].compareTo(mins[j]);
                if (sign == 1 || sign == 0) {
                    lessThan++;
                }
            }
            //fwers[i] = (new BigDecimal(String.valueOf(lessThan))).divide(new BigDecimal(String.valueOf(mins.length - 1)), 6, BigDecimal.ROUND_HALF_EVEN);            
            fwers[i] = new BigDecimal(String.valueOf((double) lessThan / (double) (mins.length - 1)));
        }
        return fwers;
    }

    public static BigDecimal[] calcFwerBDirec(BigDecimal[][] scores) {
        BigDecimal[] fwers = new BigDecimal[scores.length];
        BigDecimal[] mins_pos = new BigDecimal[scores[0].length];
        BigDecimal[] mins_neg = new BigDecimal[scores[0].length];

        for (int i = 0; i < scores[0].length; i++) {
            BigDecimal min_pos = BigDecimal.ZERO;
            BigDecimal min_neg = BigDecimal.ZERO;
            int index1 = 0;
            int index2 = 0;
            for (int j = 0; j < scores.length; j++) {
                int sign_rand = scores[j][i].compareTo(BigDecimal.ZERO);
                if (sign_rand == 1 || sign_rand == 0) {
                    if (index1 == 0) {
                        min_pos = scores[j][i];
                        index1++;
                    } else {
                        int sign = scores[j][i].compareTo(min_pos);
                        if (sign == -1) {
                            min_pos = scores[j][i];
                        }
                    }
                } else {
                    if (index2 == 0) {
                        min_neg = scores[j][i];
                        index2++;
                    } else {
                        int sign = scores[j][i].compareTo(min_neg);
                        if (sign == 1) {
                            min_neg = scores[j][i];
                        }
                    }
                }
            }
            mins_pos[i] = min_pos;
            mins_neg[i] = min_neg;
        }

        for (int i = 0; i < scores.length; i++) {
            int sign_real = scores[i][0].compareTo(BigDecimal.ZERO);
            int lessThan1 = 0;
            int lessThan2 = 0;
            if (sign_real == -1) {
                for (int j = 1; j < mins_neg.length; j++) {
                    int sign = scores[i][0].compareTo(mins_neg[j]);
                    if (sign == -1 || sign == 0) {
                        lessThan1++;
                    }
                }
                fwers[i] = new BigDecimal(String.valueOf((double) lessThan1 / (double) (mins_neg.length - 1)));
            } else {
                for (int j = 1; j < mins_pos.length; j++) {
                    int sign = scores[i][0].compareTo(mins_pos[j]);
                    if (sign == 1 || sign == 0) {
                        lessThan2++;
                    }
                }
                fwers[i] = new BigDecimal(String.valueOf((double) lessThan2 / (double) (mins_pos.length - 1)));
            }
        }

        return fwers;
    }

    public static float[] calcP(BigDecimal[][] scores) {
        float[] ps = new float[scores.length];

        for (int i = 0; i < scores.length; i++) {
            int lessThan = 0;
            for (int j = 1; j < scores[i].length; j++) {
                int sign = scores[i][0].compareTo(scores[i][j]);
                if (sign == 1 || sign == 0) {
                    lessThan++;
                }
            }
            ps[i] = (float) lessThan / (float) (scores[i].length - 1);
        }
        return ps;
    }

    public static BigDecimal[] calcPB(BigDecimal[][] scores) {
        BigDecimal[] ps = new BigDecimal[scores.length];

        for (int i = 0; i < scores.length; i++) {
            int lessThan = 0;
            for (int j = 1; j < scores[i].length; j++) {
                int sign = scores[i][0].compareTo(scores[i][j]);
                if (sign == 1 || sign == 0) {
                    lessThan++;
                }
            }
            //ps[i] = (new BigDecimal(String.valueOf(lessThan))).divide(new BigDecimal(String.valueOf(scores[i].length)), 6, BigDecimal.ROUND_HALF_EVEN);              
            ps[i] = new BigDecimal(String.valueOf((double) lessThan / (double) (scores[i].length - 1)));
        }
        return ps;
    }

    public static BigDecimal[] calcPBDirec(BigDecimal[][] scores) {
        BigDecimal[] ps = new BigDecimal[scores.length];

        for (int i = 0; i < scores.length; i++) {
            int lessThan = 0;
            int total = 0;
            int sign_real = scores[i][0].compareTo(BigDecimal.ZERO);
            if (sign_real == -1) {
                for (int j = 1; j < scores[i].length; j++) {
                    int sign_rand = scores[i][j].compareTo(BigDecimal.ZERO);
                    if (sign_rand == sign_real) {
                        total++;
                        int sign = scores[i][0].compareTo(scores[i][j]);
                        if (sign == -1 || sign == 0) {
                            lessThan++;
                        }
                    }
                }
            } else {
                for (int j = 1; j < scores[i].length; j++) {
                    int sign_rand = scores[i][j].compareTo(BigDecimal.ZERO);
                    if (sign_rand == sign_real || sign_rand == 0) {
                        total++;
                        int sign = scores[i][0].compareTo(scores[i][j]);
                        if (sign == 1 || sign == 0) {
                            lessThan++;
                        }
                    }
                }
            }
            ps[i] = new BigDecimal(String.valueOf((double) lessThan / (double) total));
        }
        return ps;
    }

    public static BigDecimal[] calcPBMoreThan(BigDecimal[][] scores) {
        BigDecimal[] ps = new BigDecimal[scores.length];

        for (int i = 0; i < scores.length; i++) {
            int moreThan = 0;
            for (int j = 1; j < scores[i].length; j++) {
                int sign = scores[i][0].compareTo(scores[i][j]);
                if (sign == -1 || sign == 0) {
                    moreThan++;
                }
            }
            //ps[i] = (new BigDecimal(String.valueOf(lessThan))).divide(new BigDecimal(String.valueOf(scores[i].length)), 6, BigDecimal.ROUND_HALF_EVEN);              
            ps[i] = new BigDecimal(String.valueOf((double) moreThan / (double) (scores[i].length - 1)));
        }
        return ps;
    }

    public static BigDecimal[] calcPBMoreThanDirec(BigDecimal[][] scores) {
        BigDecimal[] ps = new BigDecimal[scores.length];

        for (int i = 0; i < scores.length; i++) {
            int moreThan = 0;
            int total = 0;
            int sign_real = scores[i][0].compareTo(BigDecimal.ZERO);
            if (sign_real == -1) {
                for (int j = 1; j < scores[i].length; j++) {
                    int sign_rand = scores[i][j].compareTo(BigDecimal.ZERO);
                    if (sign_rand == sign_real) {
                        total++;
                        int sign = scores[i][0].compareTo(scores[i][j]);
                        if (sign == 1 || sign == 0) {
                            moreThan++;
                        }
                    }
                }
            } else {
                for (int j = 1; j < scores[i].length; j++) {
                    int sign_rand = scores[i][j].compareTo(BigDecimal.ZERO);
                    if (sign_rand == sign_real || sign_rand == 0) {
                        total++;
                        int sign = scores[i][0].compareTo(scores[i][j]);
                        if (sign == -1 || sign == 0) {
                            moreThan++;
                        }
                    }
                }
            }
            ps[i] = new BigDecimal(String.valueOf((double) moreThan / (double) total));
        }
        return ps;
    }

    public static ArrayList summary(ArrayList al) {
        ArrayList summaries = new ArrayList();
        ArrayList values = (ArrayList) al.get(0);
        ArrayList temp1 = new ArrayList();
        for (int i = 0; i < values.size(); i++) {
            double d = ((Float) values.get(i)).doubleValue();
            temp1.add(new BigDecimal(String.valueOf(d)));
        }
        summaries.add(temp1);
        for (int i = 1; i < al.size(); i++) {
            ArrayList values1 = (ArrayList) al.get(i);
            ArrayList values2 = (ArrayList) summaries.get(i - 1);
            ArrayList temp2 = new ArrayList();
            for (int j = 0; j < values1.size(); j++) {
                double d = ((Float) values1.get(j)).doubleValue();
                BigDecimal bd1 = new BigDecimal(String.valueOf(d));
                BigDecimal bd2 = (BigDecimal) values2.get(j);
                BigDecimal m = bd1.multiply(bd2);
                temp2.add(m);
                //temp2.add(m.divide(new BigDecimal(String.valueOf(i+1)),m.scale()+6,BigDecimal.ROUND_HALF_EVEN));
            }
            summaries.add(temp2);
        }
        return summaries;
    }

    public static ArrayList calcAvg(ArrayList al) {
        ArrayList gms = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList values = (ArrayList) al.get(i);
            ArrayList temp = new ArrayList();
            for (int j = 0; j < values.size(); j++) {
                BigDecimal bd = (BigDecimal) values.get(j);
                temp.add(bd.divide(new BigDecimal(String.valueOf(i + 1)), bd.scale() + 6, BigDecimal.ROUND_HALF_EVEN));
            }
            gms.add(temp);
        }
        return gms;
    }

    public static ArrayList findMin(ArrayList al) {
        ArrayList mins = new ArrayList();
        for (int i = 0; i < ((ArrayList) al.get(0)).size(); i++) {
            BigDecimal[] bds = new BigDecimal[al.size()];
            for (int j = 0; j < al.size(); j++) {
                bds[j] = (BigDecimal) ((ArrayList) al.get(j)).get(i);
            }
            BigDecimal min = min(bds);
            mins.add(min);
        }
        return mins;
    }

    public static float sum(ArrayList a) {
        float sum = 0;
        for (int i = 0; i < a.size(); i++) {
            float f = Float.parseFloat((String) a.get(i));
            sum += f;
        }
        return sum;
    }

    public static BigDecimal sum(BigDecimal[] bds) {
        BigDecimal sum = BigDecimal.ZERO;
        for (int i = 0; i < bds.length; i++) {
            sum = sum.add(bds[i]);
        }
        return sum;
    }

    public static float mean(ArrayList a) {
        int size = a.size();
        return sum(a) / size;
    }

    public static double nozeromin(final ArrayList a) {
        double min = ((Double) a.get(0)).doubleValue();
        for (int i = 1; i < a.size(); i++) {
            double f = ((Double) a.get(i)).doubleValue();
            if (f < min && f != 0) {
                min = f;
            }
        }
        return min;
    }

    public static BigDecimal mean(BigDecimal[] bds) {
        int size = bds.length;
        BigDecimal sum = sum(bds);
        return sum.divide(new BigDecimal(String.valueOf(size)), sum.scale() + 6, BigDecimal.ROUND_HALF_EVEN);
    }

    public static float max(final ArrayList a) {
        float max = ((Float) a.get(0)).floatValue();
        for (int i = 1; i < a.size(); i++) {
            float f = ((Float) a.get(i)).floatValue();
            if (f > max) {
                max = f;
            }
        }
        return max;
    }

    public static BigDecimal max(BigDecimal[] bds) {
        BigDecimal max = bds[0];
        for (int i = 0; i < bds.length; i++) {
            int sign = bds[i].compareTo(max);
            if (sign == 1 || sign == 0) {
                max = bds[i];
            }
        }
        return max;
    }

    public static float min(final ArrayList a) {
        float min = ((Float) a.get(0)).floatValue();
        for (int i = 1; i < a.size(); i++) {
            float f = ((Float) a.get(i)).floatValue();
            if (f < min) {
                min = f;
            }
        }
        return min;
    }

    public static BigDecimal min(BigDecimal[] bds) {
        BigDecimal min = bds[0];
        for (int i = 0; i < bds.length; i++) {
            int sign = bds[i].compareTo(min);
            if (sign == -1 || sign == 0) {
                min = bds[i];
            }
        }
        return min;
    }

    public static ArrayList shorten(ArrayList values, String threshold) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < values.size(); i++) {
            BigDecimal bd = (BigDecimal) ((ArrayList) values.get(i)).get(0);
            if (bd.compareTo(new BigDecimal(threshold)) == -1) {
                al.add(values.get(i));
            }
        }
        return al;
    }

    public static ArrayList calcTruncatedKSByReal(ArrayList values, ArrayList ranks, ArrayList ps, String threshold, int rankedListSize) {
        GsaaSeqTool.normalizeByTotalCol(values);
        ArrayList sigIndexes = new ArrayList();
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) values.get(0)).size(); i++) {
            //edu.mit.broad.genome.math.Vector dsList = ((RankedList)rlAll.get(i)).getScoresV(true);
            //double dsTotal = GsaaSeqTool.calcLnNorm(dsList, 1);
            //double dsUnit = dsTotal/(double)rankedListSize;            
            ArrayList setSigList = new ArrayList();
            ArrayList setList = new ArrayList();
            double setSigTotal = 0;
            double setTotal = 0;
            int maxSigRank = 0;
            if (i == 0) {
                for (int j = 0; j < values.size(); j++) {
                    double p = ((Float) ((ArrayList) ps.get(j)).get(i)).doubleValue();
                    double value = ((Double) ((ArrayList) values.get(j)).get(i)).doubleValue();
                    int rank = ((Integer) ((ArrayList) ranks.get(j)).get(i)).intValue();
                    setList.add(value);
                    if (p <= Double.parseDouble(threshold)) {
                        if (rank > maxSigRank) {
                            maxSigRank = rank;
                        }
                        setSigList.add(value);
                        sigIndexes.add(new Integer(j));
                    }
                }
                setSigTotal = GsaaSeqTool.calcLnNorm(setSigList, 1) - (maxSigRank + 1 - setSigList.size()) * ((double) 1 / (double) rankedListSize);
                //setSigTotal = GsaaSeqTool.calcLnNorm(setSigList, 1)-dsUnit*(maxSigRank+1-setSigList.size())*((double)1/(double)rankedListSize);                 
                //setTotal = GsaaSeqTool.calcLnNorm(setList, 1);             

            } else {
                for (int j = 0; j < sigIndexes.size(); j++) {
                    int sigIndex = ((Integer) sigIndexes.get(j)).intValue();
                    double value = ((Double) ((ArrayList) values.get(sigIndex)).get(i)).doubleValue();
                    int rank = ((Integer) ((ArrayList) ranks.get(sigIndex)).get(i)).intValue();
                    if (rank > maxSigRank) {
                        maxSigRank = rank;
                    }
                    setSigList.add(value);
                }
                for (int j = 0; j < values.size(); j++) {
                    Double value = (Double) ((ArrayList) values.get(j)).get(i);
                    setList.add(value);
                }
                setSigTotal = GsaaSeqTool.calcLnNorm(setSigList, 1) - (maxSigRank + 1 - setSigList.size()) * ((double) 1 / (double) rankedListSize);
                //setSigTotal = GsaaSeqTool.calcLnNorm(setSigList, 1)-dsUnit*(maxSigRank+1-setSigList.size())*((double)1/(double)rankedListSize);                 
                //setTotal = GsaaSeqTool.calcLnNorm(setList, 1);                 
            }

            if (setSigTotal <= 0) {
                setSigTotal = Math.pow(nozeromin(setList), 1);
            }

            //double setSigRatio = setSigTotal / setTotal;
            al.add(new BigDecimal(setTotal));
        }
        return al;
    }

    public static ArrayList calcTruncatedKS(ArrayList values, ArrayList ranks, ArrayList ps, String threshold, int rankedListSize) {
        GsaaSeqTool.normalizeByTotalCol(values);
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) values.get(0)).size(); i++) {
            ArrayList setSigList = new ArrayList();
            ArrayList setList = new ArrayList();
            int maxSigRank = 0;
            double setSigTotal = 0;
            for (int j = 0; j < values.size(); j++) {
                double p = ((Float) ((ArrayList) ps.get(j)).get(i)).doubleValue();
                double value = ((Double) ((ArrayList) values.get(j)).get(i)).doubleValue();
                int rank = ((Integer) ((ArrayList) ranks.get(j)).get(i)).intValue();
                setList.add(new Double(value));
                if (p <= Double.parseDouble(threshold)) {
                    if (rank > maxSigRank) {
                        maxSigRank = rank;
                    }
                    setSigList.add(new Double(value));
                }
            }

            if (!setSigList.isEmpty()) {
                setSigTotal = GsaaSeqTool.calcLnNorm(setSigList, 1) - (maxSigRank + 1 - setSigList.size()) * ((double) 1 / (double) rankedListSize);
            }

            if (setSigTotal <= 0) {
                setSigTotal = Math.pow(nozeromin(setList), 1);
            }

            al.add(new BigDecimal(setSigTotal));

        }
        return al;
    }

    public static ArrayList calcSigLnNormRatio(ArrayList values, ArrayList ps, String threshold) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) values.get(0)).size(); i++) {
            ArrayList setSigList = new ArrayList();
            ArrayList setList = new ArrayList();
            for (int j = 0; j < values.size(); j++) {
                double p = ((Float) ((ArrayList) ps.get(j)).get(i)).doubleValue();
                double value = ((Float) ((ArrayList) values.get(j)).get(i)).doubleValue();
                setList.add(new Double(value));
                if (p <= Double.parseDouble(threshold)) {
                    setSigList.add(new Double(value));
                }
            }

            if (!setSigList.isEmpty()) {
                setSigList.add(nozeromin(setList));
            }

            double setSigTotal = GsaaSeqTool.calcLnNorm(setSigList, 1);
            double setTotal = GsaaSeqTool.calcLnNorm(setList, 1);

            double setSigRatio = setSigTotal / setTotal;
            al.add(new BigDecimal(setSigRatio));
        }
        return al;
    }

    public static ArrayList calcSigLnNormRatioDirect(ArrayList values, ArrayList ps, String threshold) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) values.get(0)).size(); i++) {
            ArrayList setSigList1 = new ArrayList();
            ArrayList setSigList2 = new ArrayList();
            ArrayList setList1 = new ArrayList();
            ArrayList setList2 = new ArrayList();
            int count1 = 0;
            int count2 = 0;
            for (int j = 0; j < values.size(); j++) {
                double p = ((Float) ((ArrayList) ps.get(j)).get(i)).doubleValue();
                p = Math.abs(p);
                double value = ((Float) ((ArrayList) values.get(j)).get(i)).doubleValue();
                if (value >= 0) {
                    count1++;
                    setList1.add(new Double(value));
                    if (p <= Double.parseDouble(threshold)) {
                        setSigList1.add(new Double(value));
                    }
                } else {
                    count2++;
                    setList2.add(new Double(Math.abs(value)));
                    if (p <= Double.parseDouble(threshold)) {
                        setSigList2.add(new Double(Math.abs(value)));
                    }
                }
            }

            if (count1 == 0) {
                if (!setSigList2.isEmpty()) {
                    setSigList2.add(nozeromin(setList2));
                }
                
                double setSigTotal2 = GsaaSeqTool.calcLnNorm(setSigList2, 1);
                double setTotal2 = GsaaSeqTool.calcLnNorm(setList2, 1);
                double setSigRatio2 = setSigTotal2 / setTotal2;
                al.add(new BigDecimal(setSigRatio2 * (-1)));
            } else if (count2 == 0) {
                if (!setSigList1.isEmpty()) {
                    setSigList1.add(nozeromin(setList1));
                }

                double setSigTotal1 = GsaaSeqTool.calcLnNorm(setSigList1, 1);
                double setTotal1 = GsaaSeqTool.calcLnNorm(setList1, 1);
                double setSigRatio1 = setSigTotal1 / setTotal1;
                al.add(new BigDecimal(setSigRatio1));
            } else {
                if (!setSigList1.isEmpty()) {
                    setSigList1.add(nozeromin(setList1));
                }

                if (!setSigList2.isEmpty()) {
                    setSigList2.add(nozeromin(setList2));
                }

                double setSigTotal1 = GsaaSeqTool.calcLnNorm(setSigList1, 1);
                double setTotal1 = GsaaSeqTool.calcLnNorm(setList1, 1);
                double setSigRatio1 = setSigTotal1 / setTotal1;

                double setSigTotal2 = GsaaSeqTool.calcLnNorm(setSigList2, 1);
                double setTotal2 = GsaaSeqTool.calcLnNorm(setList2, 1);
                double setSigRatio2 = setSigTotal2 / setTotal2;

                if (setSigRatio1 >= setSigRatio2) {
                    al.add(new BigDecimal(setSigRatio1));
                } else {
                    al.add(new BigDecimal(setSigRatio2 * (-1)));
                }
            }
        }
        return al;
    }
    
    public static ArrayList calcSigLnNormRatioTDirect(ArrayList values, ArrayList ps, String threshold) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) values.get(0)).size(); i++) {
            ArrayList setSigList1 = new ArrayList();
            ArrayList setSigList2 = new ArrayList();
            ArrayList setSigList = new ArrayList();
            ArrayList setList1 = new ArrayList();
            ArrayList setList2 = new ArrayList();
            ArrayList setList = new ArrayList();
            int count1 = 0;
            int count2 = 0;
            for (int j = 0; j < values.size(); j++) {
                double p = ((Float) ((ArrayList) ps.get(j)).get(i)).doubleValue();
                p = Math.abs(p);
                double value = ((Float) ((ArrayList) values.get(j)).get(i)).doubleValue();
                setList.add(new Double(Math.abs(value)));
                if (p <= Double.parseDouble(threshold)) {
                    setSigList.add(new Double(Math.abs(value)));
                }
                if (value >= 0) {
                    count1++;
                    setList1.add(new Double(value));
                    if (p <= Double.parseDouble(threshold)) {
                        setSigList1.add(new Double(value));
                    }
                } else {
                    count2++;
                    setList2.add(new Double(Math.abs(value)));
                    if (p <= Double.parseDouble(threshold)) {
                        setSigList2.add(new Double(Math.abs(value)));
                    }
                }
            }

            if (count1 == 0) {
                if (!setSigList2.isEmpty()) {
                    setSigList2.add(nozeromin(setList2));
                }
                
                double setSigTotal2 = GsaaSeqTool.calcLnNorm(setSigList2, 1);
                double setTotal2 = GsaaSeqTool.calcLnNorm(setList2, 1);
                double setSigRatio2 = setSigTotal2 / setTotal2;
                al.add(new BigDecimal(setSigRatio2 * (-1)));
            } else if (count2 == 0) {
                if (!setSigList1.isEmpty()) {
                    setSigList1.add(nozeromin(setList1));
                }

                double setSigTotal1 = GsaaSeqTool.calcLnNorm(setSigList1, 1);
                double setTotal1 = GsaaSeqTool.calcLnNorm(setList1, 1);
                double setSigRatio1 = setSigTotal1 / setTotal1;
                al.add(new BigDecimal(setSigRatio1));
            } else {
                if (!setSigList1.isEmpty()) {
                    setSigList1.add(nozeromin(setList1));
                }

                if (!setSigList2.isEmpty()) {
                    setSigList2.add(nozeromin(setList2));
                }
                
                if (!setSigList.isEmpty()) {
                    setSigList.add(nozeromin(setList));
                }                

                double setSigTotal1 = GsaaSeqTool.calcLnNorm(setSigList1, 1);
                double setTotal1 = GsaaSeqTool.calcLnNorm(setList1, 1);
                double setSigRatio1 = setSigTotal1 / setTotal1;

                double setSigTotal2 = GsaaSeqTool.calcLnNorm(setSigList2, 1);
                double setTotal2 = GsaaSeqTool.calcLnNorm(setList2, 1);
                double setSigRatio2 = setSigTotal2 / setTotal2;
                
                double setSigTotal = GsaaSeqTool.calcLnNorm(setSigList, 1);
                double setTotal = GsaaSeqTool.calcLnNorm(setList, 1);
                double setSigRatio = setSigTotal / setTotal;                

                if (setSigRatio1 >= setSigRatio2) {
                    al.add(new BigDecimal(setSigRatio));
                } else {
                    al.add(new BigDecimal(setSigRatio * (-1)));
                }
            }
        }
        return al;
    }    

    public static ArrayList calcSigNumRatio(ArrayList ps, String threshold) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) ps.get(0)).size(); i++) {
            int setSigList = 0;
            int setTotal = 0;
            for (int j = 0; j < ps.size(); j++) {
                double d = ((Float) ((ArrayList) ps.get(j)).get(i)).doubleValue();
                setTotal++;
                if (d <= Double.parseDouble(threshold)) {
                    setSigList++;
                }
            }
            if (setSigList == 0) {
                setSigList = 1;
            }
            double setSigRatio = (double) setSigList / (double) setTotal;
            al.add(new BigDecimal(setSigRatio));
        }
        return al;
    }

    public static ArrayList calcSigNumRatioDirect(ArrayList ps, String threshold) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) ps.get(0)).size(); i++) {
            int setSigList1 = 0;
            int setTotal1 = 0;
            int setSigList2 = 0;
            int setTotal2 = 0;
            int count1 = 0;
            int count2 = 0;
            for (int j = 0; j < ps.size(); j++) {
                double d = ((Float) ((ArrayList) ps.get(j)).get(i)).doubleValue();
                if (d >= 0) {
                    count1++;
                    setTotal1++;
                    if (d <= Double.parseDouble(threshold)) {
                        setSigList1++;
                    }
                } else {
                    count2++;
                    setTotal2++;
                    if (Math.abs(d) <= Double.parseDouble(threshold)) {
                        setSigList2++;
                    }
                }
            }
            
            if (count1 == 0) {
                if (setSigList2 == 0) {
                    setSigList2 = 1;
                }
                double setSigRatio2 = (double) setSigList2 / (double) setTotal2;
                al.add(new BigDecimal(setSigRatio2 * (-1)));
            } else if (count2 == 0) {
                if (setSigList1 == 0) {
                    setSigList1 = 1;
                }
                double setSigRatio1 = (double) setSigList1 / (double) setTotal1;
                al.add(new BigDecimal(setSigRatio1));
            } else {
                if (setSigList1 == 0) {
                    setSigList1 = 1;
                }
                if (setSigList2 == 0) {
                    setSigList2 = 1;
                }
                double setSigRatio1 = (double) setSigList1 / (double) setTotal1;
                double setSigRatio2 = (double) setSigList2 / (double) setTotal2;
                if (setSigRatio1 >= setSigRatio2) {
                    al.add(new BigDecimal(setSigRatio1));
                } else {
                    al.add(new BigDecimal(setSigRatio2 * (-1)));
                }
            }
        }
        return al;
    }
    
    public static ArrayList calcSigNumRatioTDirect(ArrayList ps, String threshold) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) ps.get(0)).size(); i++) {
            int setSigList1 = 0;
            int setSigList2 = 0; 
            int setSigList = 0;   
            int setTotal1 = 0;
            int setTotal2 = 0;
            int setTotal = 0;
            int count1 = 0;
            int count2 = 0;
            for (int j = 0; j < ps.size(); j++) {
                double d = ((Float) ((ArrayList) ps.get(j)).get(i)).doubleValue();
                setTotal++;
                if (Math.abs(d) <= Double.parseDouble(threshold)) {
                    setSigList++;
                }
                if (d >= 0) {
                    count1++;
                    setTotal1++;
                    if (d <= Double.parseDouble(threshold)) {
                        setSigList1++;
                    }
                } else {
                    count2++;
                    setTotal2++;
                    if (Math.abs(d) <= Double.parseDouble(threshold)) {
                        setSigList2++;
                    }
                }
            }
            
            if (count1 == 0) {
                if (setSigList2 == 0) {
                    setSigList2 = 1;
                }
                double setSigRatio2 = (double) setSigList2 / (double) setTotal2;
                al.add(new BigDecimal(setSigRatio2 * (-1)));
            } else if (count2 == 0) {
                if (setSigList1 == 0) {
                    setSigList1 = 1;
                }
                double setSigRatio1 = (double) setSigList1 / (double) setTotal1;
                al.add(new BigDecimal(setSigRatio1));
            } else {
                if (setSigList1 == 0) {
                    setSigList1 = 1;
                }
                if (setSigList2 == 0) {
                    setSigList2 = 1;
                }
                if (setSigList == 0) {
                    setSigList = 1;
                }                
                double setSigRatio1 = (double) setSigList1 / (double) setTotal1;
                double setSigRatio2 = (double) setSigList2 / (double) setTotal2;
                double setSigRatio = (double) setSigList / (double) setTotal;                
                if (setSigRatio1 >= setSigRatio2) {
                    al.add(new BigDecimal(setSigRatio));
                } else {
                    al.add(new BigDecimal(setSigRatio * (-1)));
                }
            }
        }
        return al;
    }    

    public static ArrayList calcRankSum(ArrayList ranks) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) ranks.get(0)).size(); i++) {
            int sum = 1;
            for (int j = 0; j < ranks.size(); j++) {
                int rank = ((Integer) ((ArrayList) ranks.get(j)).get(i)).intValue();
                sum += rank;
            }
            al.add(new BigDecimal(sum));

        }
        return al;
    }

    public static ArrayList calcRankSumDirect(ArrayList values, ArrayList ranks) {       
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) values.get(0)).size(); i++) {
            int sum1 = 1;
            int sum2 = 1;
            int count1 = 0;
            int count2 = 0;
            for (int j = 0; j < values.size(); j++) {
                int rank = ((Integer) ((ArrayList) ranks.get(j)).get(i)).intValue();
                double value = ((Float) ((ArrayList) values.get(j)).get(i)).doubleValue();
                if (value >= 0) {
                    count1++;
                    sum1 += rank;
                } else {
                    count2++;
                    sum2 += rank;
                }
            }

            if (count1 == 0) {
                al.add(new BigDecimal(sum2 * (-1)));
            } else if (count2 == 0) {
                al.add(new BigDecimal(sum1));
            } else {
                if (sum1 <= sum2) {
                    al.add(new BigDecimal(sum1));
                } else {
                    al.add(new BigDecimal(sum2 * (-1)));
                }
            }
        }
        return al;
    }
    
    public static ArrayList calcRankSumTDirect(ArrayList values, ArrayList ranks) {       
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) values.get(0)).size(); i++) {
            int sum1 = 1;
            int sum2 = 1;
            int sum = 1;
            int count1 = 0;
            int count2 = 0;
            for (int j = 0; j < values.size(); j++) {
                int rank = ((Integer) ((ArrayList) ranks.get(j)).get(i)).intValue();
                double value = ((Float) ((ArrayList) values.get(j)).get(i)).doubleValue();
                sum += rank;
                if (value >= 0) {
                    count1++;
                    sum1 += rank;
                } else {
                    count2++;
                    sum2 += rank;
                }
            }

            if (count1 == 0) {
                al.add(new BigDecimal(sum2 * (-1)));
            } else if (count2 == 0) {
                al.add(new BigDecimal(sum1));
            } else {
                if (sum1 <= sum2) {
                    al.add(new BigDecimal(sum));
                } else {
                    al.add(new BigDecimal(sum * (-1)));
                }
            }
        }
        return al;
    }    

    public static ArrayList calcL2Norm(ArrayList values) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) values.get(0)).size(); i++) {
            ArrayList setList = new ArrayList();
            for (int j = 0; j < values.size(); j++) {
                double value = ((Float) ((ArrayList) values.get(j)).get(i)).doubleValue();
                setList.add(new Double(value));
            }

            double setTotal = GsaaSeqTool.calcLnNorm(setList, 2);
            al.add(new BigDecimal(setTotal));
        }
        return al;
    }

    public static ArrayList calcL2NormDirect(ArrayList values) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) values.get(0)).size(); i++) {
            ArrayList setList1 = new ArrayList();
            ArrayList setList2 = new ArrayList();
            int count1 = 0;
            int count2 = 0;
            for (int j = 0; j < values.size(); j++) {
                double value = ((Float) ((ArrayList) values.get(j)).get(i)).doubleValue();
                if (value >= 0) {
                    count1++;
                    setList1.add(new Double(value));
                } else {
                    count2++;
                    setList2.add(new Double(Math.abs(value)));
                }
            }

            if (count1 == 0) {
                double setTotal2 = GsaaSeqTool.calcLnNorm(setList2, 2);
                al.add(new BigDecimal(setTotal2 * (-1)));
            } else if (count2 == 0) {
                double setTotal1 = GsaaSeqTool.calcLnNorm(setList1, 2);
                al.add(new BigDecimal(setTotal1));
            } else {
                double setTotal1 = GsaaSeqTool.calcLnNorm(setList1, 2);
                double setTotal2 = GsaaSeqTool.calcLnNorm(setList2, 2);
                if (setTotal1 >= setTotal2) {
                    al.add(new BigDecimal(setTotal1));
                } else {
                    al.add(new BigDecimal(setTotal2 * (-1)));
                }
            }
        }
        return al;
    }
    
    public static ArrayList calcL2NormTDirect(ArrayList values) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) values.get(0)).size(); i++) {
            ArrayList setList1 = new ArrayList();
            ArrayList setList2 = new ArrayList();
            ArrayList setList = new ArrayList();
            int count1 = 0;
            int count2 = 0;
            for (int j = 0; j < values.size(); j++) {
                double value = ((Float) ((ArrayList) values.get(j)).get(i)).doubleValue();
                setList.add(new Double(Math.abs(value)));
                if (value >= 0) {
                    count1++;
                    setList1.add(new Double(value));
                } else {
                    count2++;
                    setList2.add(new Double(Math.abs(value)));
                }
            }

            if (count1 == 0) {
                double setTotal2 = GsaaSeqTool.calcLnNorm(setList2, 2);
                al.add(new BigDecimal(setTotal2 * (-1)));
            } else if (count2 == 0) {
                double setTotal1 = GsaaSeqTool.calcLnNorm(setList1, 2);
                al.add(new BigDecimal(setTotal1));
            } else {
                double setTotal1 = GsaaSeqTool.calcLnNorm(setList1, 2);
                double setTotal2 = GsaaSeqTool.calcLnNorm(setList2, 2);
                double setTotal = GsaaSeqTool.calcLnNorm(setList, 2);
                if (setTotal1 >= setTotal2) {
                    al.add(new BigDecimal(setTotal));
                } else {
                    al.add(new BigDecimal(setTotal * (-1)));
                }
            }
        }
        return al;
    }    

    public static ArrayList calcMean(ArrayList values) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) values.get(0)).size(); i++) {
            ArrayList setList = new ArrayList();
            for (int j = 0; j < values.size(); j++) {
                double value = ((Float) ((ArrayList) values.get(j)).get(i)).doubleValue();
                setList.add(new Double(value));
            }

            double setTotal = GsaaSeqTool.calcLnNorm(setList, 1);
            double setMean = setTotal / (double) setList.size();
            al.add(new BigDecimal(setMean));
        }
        return al;
    }

    public static ArrayList calcMeanDirect(ArrayList values) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) values.get(0)).size(); i++) {
            ArrayList setList1 = new ArrayList();
            ArrayList setList2 = new ArrayList();
            int count1 = 0;
            int count2 = 0;
            for (int j = 0; j < values.size(); j++) {
                double value = ((Float) ((ArrayList) values.get(j)).get(i)).doubleValue();
                if (value >= 0) {
                    count1++;
                    setList1.add(new Double(value));
                } else {
                    count2++;
                    setList2.add(new Double(Math.abs(value)));
                }
            }

            if (count1 == 0) {
                double setTotal2 = GsaaSeqTool.calcLnNorm(setList2, 1);
                double setMean2 = setTotal2 / (double) setList2.size();
                al.add(new BigDecimal((setMean2 * (-1))));
            } else if (count2 == 0) {
                double setTotal1 = GsaaSeqTool.calcLnNorm(setList1, 1);
                double setMean1 = setTotal1 / (double) setList1.size();
                al.add(new BigDecimal(setMean1));
            } else {
                double setTotal1 = GsaaSeqTool.calcLnNorm(setList1, 1);
                double setTotal2 = GsaaSeqTool.calcLnNorm(setList2, 1);
                double setMean1 = setTotal1 / (double) setList1.size();
                double setMean2 = setTotal2 / (double) setList2.size();
                if (setMean1 >= setMean2) {
                    al.add(new BigDecimal(setMean1));
                } else {
                    al.add(new BigDecimal((setMean2 * (-1))));
                }
            }
        }
        return al;
    }
    
    public static ArrayList calcMeanTDirect(ArrayList values) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) values.get(0)).size(); i++) {
            ArrayList setList1 = new ArrayList();
            ArrayList setList2 = new ArrayList();
            ArrayList setList = new ArrayList();
            int count1 = 0;
            int count2 = 0;
            for (int j = 0; j < values.size(); j++) {
                double value = ((Float) ((ArrayList) values.get(j)).get(i)).doubleValue();
                setList.add(new Double(Math.abs(value)));
                if (value >= 0) {
                    count1++;
                    setList1.add(new Double(value));
                } else {
                    count2++;
                    setList2.add(new Double(Math.abs(value)));
                }
            }

            if (count1 == 0) {
                double setTotal2 = GsaaSeqTool.calcLnNorm(setList2, 1);
                double setMean2 = setTotal2 / (double) setList2.size();
                al.add(new BigDecimal((setMean2 * (-1))));
            } else if (count2 == 0) {
                double setTotal1 = GsaaSeqTool.calcLnNorm(setList1, 1);
                double setMean1 = setTotal1 / (double) setList1.size();
                al.add(new BigDecimal(setMean1));
            } else {
                double setTotal1 = GsaaSeqTool.calcLnNorm(setList1, 1);
                double setTotal2 = GsaaSeqTool.calcLnNorm(setList2, 1);
                double setTotal = GsaaSeqTool.calcLnNorm(setList, 1);
                double setMean1 = setTotal1 / (double) setList1.size();
                double setMean2 = setTotal2 / (double) setList2.size();
                double setMean = setTotal / (double) setList.size();
                if (setMean1 >= setMean2) {
                    al.add(new BigDecimal(setMean));
                } else {
                    al.add(new BigDecimal((setMean * (-1))));
                }
            }
        }
        return al;
    }    

    public static ArrayList calcGM(ArrayList ps) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) ps.get(0)).size(); i++) {
            double setTotal = 0;
            for (int j = 0; j < ps.size(); j++) {
                double p = ((Float) ((ArrayList) ps.get(j)).get(i)).doubleValue();
                setTotal += Math.log(p);
            }

            double setGm = Math.exp(setTotal / (double) ps.size());
            al.add(new BigDecimal(setGm));
        }
        return al;
    }

    public static ArrayList calcGMDirect(ArrayList ps) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) ps.get(0)).size(); i++) {
            double setTotal1 = 0;
            double setTotal2 = 0;
            int count1 = 0;
            int count2 = 0;
            for (int j = 0; j < ps.size(); j++) {
                double p = ((Float) ((ArrayList) ps.get(j)).get(i)).doubleValue();
                if (p >= 0) {
                    count1++;
                    setTotal1 += Math.log(p);
                } else {
                    count2++;
                    setTotal2 += Math.log(Math.abs(p));
                }
            }

            if (count1 == 0) {
                double setGm2 = Math.exp(setTotal2 / (double) count2);
                al.add(new BigDecimal((setGm2 * (-1))));
            } else if (count2 == 0) {
                double setGm1 = Math.exp(setTotal1 / (double) count1);
                al.add(new BigDecimal(setGm1));
            } else {
                double setGm1 = Math.exp(setTotal1 / (double) count1);
                double setGm2 = Math.exp(setTotal2 / (double) count2);
                if (setGm1 <= setGm2) {
                    al.add(new BigDecimal(setGm1));
                } else {
                    al.add(new BigDecimal(setGm2 * (-1)));
                }
            }
        }
        return al;
    }
    
    public static ArrayList calcGMTDirect(ArrayList ps) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) ps.get(0)).size(); i++) {
            double setTotal1 = 0;
            double setTotal2 = 0;
            double setTotal = 0;
            int count1 = 0;
            int count2 = 0;
            for (int j = 0; j < ps.size(); j++) {
                double p = ((Float) ((ArrayList) ps.get(j)).get(i)).doubleValue();
                setTotal += Math.log(Math.abs(p));
                if (p >= 0) {
                    count1++;
                    setTotal1 += Math.log(p);
                } else {
                    count2++;
                    setTotal2 += Math.log(Math.abs(p));
                }
            }

            if (count1 == 0) {
                double setGm2 = Math.exp(setTotal2 / (double) count2);
                al.add(new BigDecimal((setGm2 * (-1))));
            } else if (count2 == 0) {
                double setGm1 = Math.exp(setTotal1 / (double) count1);
                al.add(new BigDecimal(setGm1));
            } else {
                double setGm1 = Math.exp(setTotal1 / (double) count1);
                double setGm2 = Math.exp(setTotal2 / (double) count2);
                double setGm = Math.exp(setTotal / (double) ps.size());
                if (setGm1 <= setGm2) {
                    al.add(new BigDecimal(setGm));
                } else {
                    al.add(new BigDecimal(setGm * (-1)));
                }
            }
        }
        return al;
    }    

    public static ArrayList calcMinPGM(ArrayList al) {
        ArrayList gms = new ArrayList();
        ArrayList values = (ArrayList) al.get(0);
        ArrayList temp1 = new ArrayList();
        for (int i = 0; i < values.size(); i++) {
            double d = ((Float) values.get(i)).doubleValue();
            double logd = Math.log(d);
            temp1.add(new Double(logd));
        }
        gms.add(temp1);
        for (int i = 1; i < al.size(); i++) {
            ArrayList values1 = (ArrayList) al.get(i);
            ArrayList values2 = (ArrayList) gms.get(i - 1);
            ArrayList temp2 = new ArrayList();
            for (int j = 0; j < values1.size(); j++) {
                double d1 = ((Float) values1.get(j)).doubleValue();
                double logd1 = Math.log(d1);
                double d2 = ((Double) values2.get(j)).doubleValue();
                double logd2 = logd1 + d2;
                temp2.add(new Double(logd2));
            }
            gms.add(temp2);
        }
        for (int i = 0; i < gms.size(); i++) {
            ArrayList values3 = (ArrayList) gms.get(i);
            for (int j = 0; j < values3.size(); j++) {
                double d = ((Double) values3.get(j)).doubleValue();
                d = Math.exp(d / (double) (i + 1));
                ((ArrayList) gms.get(i)).set(j, new BigDecimal(d));
            }
        }

        return gms;
    }

    public static ArrayList calcFisherScore(ArrayList ps) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) ps.get(0)).size(); i++) {
            ArrayList setList = new ArrayList();
            for (int j = 0; j < ps.size(); j++) {
                double p = ((Float) ((ArrayList) ps.get(j)).get(i)).doubleValue();
                setList.add(new Double(p));
            }

            double score = GsaaSeqTool.fisherTest(setList);
            al.add(new BigDecimal(score));
        }
        return al;
    }

    public static ArrayList calcFisherScoreDirect(ArrayList ps) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) ps.get(0)).size(); i++) {
            ArrayList setList1 = new ArrayList();
            ArrayList setList2 = new ArrayList();
            int count1 = 0;
            int count2 = 0;
            for (int j = 0; j < ps.size(); j++) {
                double p = ((Float) ((ArrayList) ps.get(j)).get(i)).doubleValue();
                if (p >= 0) {
                    count1++;
                    setList1.add(new Double(p));
                } else {
                    count2++;
                    setList2.add(new Double(Math.abs(p)));
                }
            }

            if (count1 == 0) {
                double score2 = GsaaSeqTool.fisherTest(setList2);
                al.add(new BigDecimal(score2 * (-1)));
            } else if (count2 == 0) {
                double score1 = GsaaSeqTool.fisherTest(setList1);
                al.add(new BigDecimal(score1));
            } else {
                double score1 = GsaaSeqTool.fisherTest(setList1);
                double score2 = GsaaSeqTool.fisherTest(setList2);
                if (score1 >= score2) {
                    al.add(new BigDecimal(score1));
                } else {
                    al.add(new BigDecimal(score2 * (-1)));
                }
            }
        }
        return al;
    }
    
    public static ArrayList calcFisherScoreTDirect(ArrayList ps) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < ((ArrayList) ps.get(0)).size(); i++) {
            ArrayList setList1 = new ArrayList();
            ArrayList setList2 = new ArrayList();
            ArrayList setList = new ArrayList();
            int count1 = 0;
            int count2 = 0;
            for (int j = 0; j < ps.size(); j++) {
                double p = ((Float) ((ArrayList) ps.get(j)).get(i)).doubleValue();
                setList.add(new Double(Math.abs(p)));
                if (p >= 0) {
                    count1++;
                    setList1.add(new Double(p));
                } else {
                    count2++;
                    setList2.add(new Double(Math.abs(p)));
                }
            }

            if (count1 == 0) {
                double score2 = GsaaSeqTool.fisherTest(setList2);
                al.add(new BigDecimal(score2 * (-1)));
            } else if (count2 == 0) {
                double score1 = GsaaSeqTool.fisherTest(setList1);
                al.add(new BigDecimal(score1));
            } else {
                double score1 = GsaaSeqTool.fisherTest(setList1);
                double score2 = GsaaSeqTool.fisherTest(setList2);
                double score = GsaaSeqTool.fisherTest(setList);
                if (score1 >= score2) {
                    al.add(new BigDecimal(score));
                } else {
                    al.add(new BigDecimal(score * (-1)));
                }
            }
        }
        return al;
    }    

    public static BigDecimal[][] normalizeMean(BigDecimal[][] bds) {
        BigDecimal[][] normScores = new BigDecimal[bds.length][bds[0].length];
        for (int i = 0; i < bds.length; i++) {
            BigDecimal realScore = bds[i][0];

            BigDecimal[] rndScores = new BigDecimal[bds[i].length - 1];
            for (int j = 1; j < bds[i].length; j++) {
                rndScores[j - 1] = bds[i][j];
            }
            BigDecimal mean = mean(rndScores);

            normScores[i][0] = realScore.divide(mean, realScore.scale() + 6, BigDecimal.ROUND_HALF_EVEN);
            for (int j = 1; j < bds[i].length; j++) {
                normScores[i][j] = bds[i][j].divide(mean, bds[i][j].scale() + 6, BigDecimal.ROUND_HALF_EVEN);
            }
        }
        return normScores;
    }

    public static BigDecimal[][] normalizeMeanDirect(BigDecimal[][] bds) {
        BigDecimal[][] normScores = new BigDecimal[bds.length][bds[0].length];
        for (int i = 0; i < bds.length; i++) {
            BigDecimal realScore = bds[i][0];

            ArrayList rndScores1 = new ArrayList();
            ArrayList rndScores2 = new ArrayList();
            int count1 = 0;
            int count2 = 0;
            for (int j = 1; j < bds[i].length; j++) {
                int sign = bds[i][j].compareTo(BigDecimal.ZERO);
                if (sign == 1 || sign == 0) {
                    count1++;
                    rndScores1.add(bds[i][j]);
                } else {
                    count2++;
                    rndScores2.add(bds[i][j].multiply(new BigDecimal(-1)));
                }
            }

            if (count1 == 0) {
                BigDecimal[] rndScoresNeg = new BigDecimal[rndScores2.size()];
                for (int k = 0; k < rndScores2.size(); k++) {
                    rndScoresNeg[k] = (BigDecimal) rndScores2.get(k);
                }
                BigDecimal meanNeg = mean(rndScoresNeg);
                int sign_real = realScore.compareTo(BigDecimal.ZERO);
                if (sign_real == -1) {
                    normScores[i][0] = realScore.divide(meanNeg, realScore.scale() + 6, BigDecimal.ROUND_HALF_EVEN);
                } else {
                    normScores[i][0] = BigDecimal.ONE;
                }
                for (int j = 1; j < bds[i].length; j++) {
                    normScores[i][j] = bds[i][j].divide(meanNeg, bds[i][j].scale() + 6, BigDecimal.ROUND_HALF_EVEN);
                }
            } else if (count2 == 0) {
                BigDecimal[] rndScoresPos = new BigDecimal[rndScores1.size()];
                for (int k = 0; k < rndScores1.size(); k++) {
                    rndScoresPos[k] = (BigDecimal) rndScores1.get(k);
                }
                BigDecimal meanPos = mean(rndScoresPos);
                int sign_real = realScore.compareTo(BigDecimal.ZERO);
                if (sign_real == 1 || sign_real == 0) {
                    normScores[i][0] = realScore.divide(meanPos, realScore.scale() + 6, BigDecimal.ROUND_HALF_EVEN);
                } else {
                    normScores[i][0] = BigDecimal.ONE;
                }
                for (int j = 1; j < bds[i].length; j++) {
                    normScores[i][j] = bds[i][j].divide(meanPos, bds[i][j].scale() + 6, BigDecimal.ROUND_HALF_EVEN);
                }
            } else {
                BigDecimal[] rndScoresPos = new BigDecimal[rndScores1.size()];
                for (int k = 0; k < rndScores1.size(); k++) {
                    rndScoresPos[k] = (BigDecimal) rndScores1.get(k);
                }
                BigDecimal[] rndScoresNeg = new BigDecimal[rndScores2.size()];
                for (int k = 0; k < rndScores2.size(); k++) {
                    rndScoresNeg[k] = (BigDecimal) rndScores2.get(k);
                }

                BigDecimal meanPos = mean(rndScoresPos);
                BigDecimal meanNeg = mean(rndScoresNeg);

                int sign_real = realScore.compareTo(BigDecimal.ZERO);
                if (sign_real == 1 || sign_real == 0) {
                    normScores[i][0] = realScore.divide(meanPos, realScore.scale() + 6, BigDecimal.ROUND_HALF_EVEN);
                } else {
                    normScores[i][0] = realScore.divide(meanNeg, realScore.scale() + 6, BigDecimal.ROUND_HALF_EVEN);
                }
                for (int j = 1; j < bds[i].length; j++) {
                    int sign_rnd = bds[i][j].compareTo(BigDecimal.ZERO);
                    if (sign_rnd == 1 || sign_rnd == 0) {
                        normScores[i][j] = bds[i][j].divide(meanPos, bds[i][j].scale() + 6, BigDecimal.ROUND_HALF_EVEN);
                    } else {
                        normScores[i][j] = bds[i][j].divide(meanNeg, bds[i][j].scale() + 6, BigDecimal.ROUND_HALF_EVEN);
                    }
                }
            }
        }
        return normScores;
    }

    public static BigDecimal[][] normalizeMeanCol(BigDecimal[][] bds) {
        BigDecimal[][] normScores = new BigDecimal[bds.length][bds[0].length];
        for (int i = 0; i < bds[0].length; i++) {
            BigDecimal[] scores = new BigDecimal[bds.length];
            for (int j = 0; j < bds.length; j++) {
                scores[j] = bds[j][i];
            }
            BigDecimal mean = mean(scores);
            for (int j = 0; j < bds.length; j++) {
                normScores[j][i] = bds[j][i].divide(mean, bds[j][i].scale() + 6, BigDecimal.ROUND_HALF_EVEN);
            }
        }
        return normScores;
    }

    public static BigDecimal[][] normalizeMeanSDMore(BigDecimal[][] bds) {
        BigDecimal[][] normScores = new BigDecimal[bds.length][bds[0].length];
        BigDecimal[] mins = new BigDecimal[bds.length];
        for (int i = 0; i < bds.length; i++) {
            BigDecimal realScore = bds[i][0];
            ArrayList rndScores = new ArrayList();
            for (int j = 1; j < bds[i].length; j++) {
                rndScores.add(bds[i][j]);
            }
            BigDecimal mean = BigDecimalMathUtils.mean(rndScores, new MathContext(realScore.precision() + 6));
            BigDecimal sd = BigDecimalMathUtils.stddev(rndScores, true, new MathContext(realScore.precision() + 6));
            if (sd.compareTo(BigDecimal.ZERO) == 0) {
                normScores[i][0] = BigDecimal.ONE;
            } else {
                normScores[i][0] = (realScore.subtract(mean)).divide(sd, realScore.scale() + 6, BigDecimal.ROUND_HALF_EVEN);
            }
            for (int j = 1; j < bds[i].length; j++) {
                if (sd.compareTo(BigDecimal.ZERO) == 0) {
                    normScores[i][j] = BigDecimal.ONE;
                } else {
                    normScores[i][j] = (bds[i][j].subtract(mean)).divide(sd, bds[i][j].scale() + 6, BigDecimal.ROUND_HALF_EVEN);
                }
            }
            mins[i] = min(normScores[i]);
        }
        BigDecimal min = min(mins);
        min = min.abs();
        for (int i = 0; i < normScores.length; i++) {
            for (int j = 0; j < normScores[0].length; j++) {
                normScores[i][j] = normScores[i][j].add(min);
            }
        }
        return normScores;
    }

    public static BigDecimal[][] normalizeMeanSDLess(BigDecimal[][] bds) {
        BigDecimal[][] normScores = new BigDecimal[bds.length][bds[0].length];
        BigDecimal[] mins = new BigDecimal[bds.length];
        for (int i = 0; i < bds.length; i++) {
            BigDecimal realScore = bds[i][0];
            ArrayList rndScores = new ArrayList();
            for (int j = 1; j < bds[i].length; j++) {
                rndScores.add(bds[i][j]);
            }
            BigDecimal mean = BigDecimalMathUtils.mean(rndScores, new MathContext(realScore.precision() + 6));
            BigDecimal sd = BigDecimalMathUtils.stddev(rndScores, true, new MathContext(realScore.precision() + 6));
            if (sd.compareTo(BigDecimal.ZERO) == 0) {
                normScores[i][0] = new BigDecimal(0.000001);
            } else {
                normScores[i][0] = (realScore.subtract(mean)).divide(sd, realScore.scale() + 6, BigDecimal.ROUND_HALF_EVEN);
            }
            for (int j = 1; j < bds[i].length; j++) {
                if (sd.compareTo(BigDecimal.ZERO) == 0) {
                    normScores[i][j] = new BigDecimal(0.000001);;
                } else {
                    normScores[i][j] = (bds[i][j].subtract(mean)).divide(sd, bds[i][j].scale() + 6, BigDecimal.ROUND_HALF_EVEN);
                }
            }
            mins[i] = min(normScores[i]);
        }
        BigDecimal min = min(mins);
        min = min.abs();
        for (int i = 0; i < normScores.length; i++) {
            for (int j = 0; j < normScores[0].length; j++) {
                normScores[i][j] = normScores[i][j].add(min);
            }
        }
        return normScores;
    }

    public static BigDecimal[][] normalizeMeanSDMoreCol(BigDecimal[][] bds) {
        BigDecimal[][] normScores = new BigDecimal[bds.length][bds[0].length];
        for (int i = 0; i < bds[0].length; i++) {
            ArrayList scores = new ArrayList();
            for (int j = 0; j < bds.length; j++) {
                scores.add(bds[j][i]);
            }
            BigDecimal min = BigDecimalMathUtils.min(scores);
            BigDecimal mean = BigDecimalMathUtils.mean(scores, new MathContext(min.precision() + 6));
            BigDecimal sd = BigDecimalMathUtils.stddev(scores, true, new MathContext(min.precision() + 6));
            for (int j = 0; j < bds.length; j++) {
                if (sd.compareTo(BigDecimal.ZERO) == 0) {
                    normScores[j][i] = BigDecimal.ONE;
                } else {
                    normScores[j][i] = (bds[j][i].subtract(mean)).divide(sd, bds[j][i].scale() + 6, BigDecimal.ROUND_HALF_EVEN);
                }
            }
        }
        BigDecimal[] mins = new BigDecimal[normScores.length];
        for (int i = 0; i < normScores.length; i++) {
            mins[i] = min(normScores[i]);
        }
        BigDecimal min = min(mins);
        min = min.abs();
        for (int i = 0; i < normScores.length; i++) {
            for (int j = 0; j < normScores[0].length; j++) {
                normScores[i][j] = normScores[i][j].add(min);
            }
        }
        return normScores;
    }

    public static BigDecimal[][] normalizeMeanSDLessCol(BigDecimal[][] bds) {
        BigDecimal[][] normScores = new BigDecimal[bds.length][bds[0].length];
        for (int i = 0; i < bds[0].length; i++) {
            ArrayList scores = new ArrayList();
            for (int j = 0; j < bds.length; j++) {
                scores.add(bds[j][i]);
            }
            BigDecimal min = BigDecimalMathUtils.min(scores);
            BigDecimal mean = BigDecimalMathUtils.mean(scores, new MathContext(min.precision() + 6));
            BigDecimal sd = BigDecimalMathUtils.stddev(scores, true, new MathContext(min.precision() + 6));
            for (int j = 0; j < bds.length; j++) {
                if (sd.compareTo(BigDecimal.ZERO) == 0) {
                    normScores[j][i] = new BigDecimal(0.000001);
                } else {
                    normScores[j][i] = (bds[j][i].subtract(mean)).divide(sd, bds[j][i].scale() + 6, BigDecimal.ROUND_HALF_EVEN);
                }
            }
        }
        BigDecimal[] mins = new BigDecimal[normScores.length];
        for (int i = 0; i < normScores.length; i++) {
            mins[i] = min(normScores[i]);
        }
        BigDecimal min = min(mins);
        min = min.abs();
        for (int i = 0; i < normScores.length; i++) {
            for (int j = 0; j < normScores[0].length; j++) {
                normScores[i][j] = normScores[i][j].add(min);
            }
        }
        return normScores;
    }

    public static BigDecimal[][] normalizeLoess(BigDecimal[][] bds, double[] setSizes) {
        BigDecimal[][] normScores = new BigDecimal[bds.length][bds[0].length];
        for (int i = 0; i < bds.length; i++) {
            BigDecimal realScore = bds[i][0];
            BigDecimal[] rndScores = new BigDecimal[bds[i].length - 1];
            for (int j = 1; j < bds[i].length; j++) {
                rndScores[j - 1] = bds[i][j];
            }
            BigDecimal mean = mean(rndScores);
            normScores[i][0] = realScore.divide(mean, realScore.scale() + 6, BigDecimal.ROUND_HALF_EVEN);
            for (int j = 1; j < bds[i].length; j++) {
                normScores[i][j] = bds[i][j].divide(mean, bds[i][j].scale() + 6, BigDecimal.ROUND_HALF_EVEN);
            }
        }
        double[][] normScoresLn = new double[bds.length][bds[0].length];
        for (int i = 0; i < normScores.length; i++) {
            for (int j = 0; j < normScores[0].length; j++) {
                normScoresLn[i][j] = BigFunctions.ln(normScores[i][j], normScores[i][j].scale()).doubleValue();
            }
        }
        LoessInterpolator loess = new LoessInterpolator();

        ArrayIndexComparator comparator = new ArrayIndexComparator(setSizes);
        Integer[] indexes = comparator.createIndexArray();
        Arrays.sort(indexes, comparator);
        double[] sortedSetSizes = new double[setSizes.length];
        double[][] sortedNormScoresLn = new double[normScoresLn.length][normScoresLn[0].length];
        for (int i = 0; i < sortedSetSizes.length; i++) {
            sortedSetSizes[i] = setSizes[indexes[i].intValue()];
        }
        for (int i = 0; i < sortedNormScoresLn.length; i++) {
            sortedNormScoresLn[i] = normScoresLn[indexes[i].intValue()];
        }

        for (int i = 0; i < sortedNormScoresLn[0].length; i++) {
            double[] col = new double[sortedNormScoresLn.length];
            for (int j = 0; j < sortedNormScoresLn.length; j++) {
                col[j] = sortedNormScoresLn[j][i];
            }
            double[] loessValues = new double[col.length];
            try {
                loessValues = loess.smooth(sortedSetSizes, col);
            } catch (Exception e) {
                e.printStackTrace();
            }
            for (int j = 0; j < normScoresLn.length; j++) {
                int index = 0;
                for (int k = 0; k < indexes.length; k++) {
                    if (indexes[k].intValue() == j) {
                        index = k;
                        break;
                    }
                }
                col[j] = normScoresLn[j][i] - loessValues[index];
                normScores[j][i] = BigFunctions.exp(new BigDecimal(col[j]), normScores[j][i].scale());
            }
        }

        return normScores;
    }
}

class ArrayIndexComparator implements Comparator<Integer> {

    private final double[] array;

    public ArrayIndexComparator(double[] array) {
        this.array = array;
    }

    public Integer[] createIndexArray() {
        Integer[] indexes = new Integer[array.length];
        for (int i = 0; i < array.length; i++) {
            indexes[i] = i;
        }
        return indexes;
    }

    public int compare(Integer index1, Integer index2) {
        return (new Double(array[index1])).compareTo(new Double(array[index2]));
    }
}
