/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */

package edu.duke.igsp.app.tools;

import java.io.*;
import java.util.*;
import edu.duke.igsp.app.objects.GeneBean;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: April, 2009
 *
 */
public class FileTool {
    public FileTool(){}

    public static ArrayList readData(
            String filename) {
        ArrayList result = new ArrayList();
        try {
            FileReader input = new FileReader(filename);
            BufferedReader br = new BufferedReader(input);
            String s = br.readLine();
            while (s != null) {
                StringTokenizer st = new StringTokenizer(s);
                ArrayList temp = new ArrayList();
                while (st.hasMoreTokens()) {
                    temp.add(st.nextToken());
                }

                result.add(temp);
                s =
                        br.readLine();
            }

            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return result;
    }

    public static ArrayList readData(InputStreamReader isr) {
        ArrayList result = new ArrayList();
        try {
            BufferedReader br = new BufferedReader(isr);
            String s = br.readLine();
            while (s != null) {
                StringTokenizer st = new StringTokenizer(s);
                ArrayList temp = new ArrayList();
                while (st.hasMoreTokens()) {
                    temp.add(st.nextToken());
                }

                result.add(temp);
                s = br.readLine();
            }

            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return result;
    }

    //for AA, BB, AB
    public static void toGenos(String filename1, String filename2) {
        ArrayList data = readData(filename1);

        ArrayList dist = new ArrayList();
        for (int i = 0; i < data.size(); i++) {
            ArrayList al = new ArrayList();
            al.add((String) ((ArrayList) data.get(i)).get(0));
            al.add((String) ((ArrayList) data.get(i)).get(1));
            al.add((String) ((ArrayList) data.get(i)).get(2));
            String[] alleles = new String[2];
            alleles[0] = (String) ((ArrayList) data.get(i)).get(3);
            alleles[1] = (String) ((ArrayList) data.get(i)).get(4);
            for (int j = 5; j < ((ArrayList) data.get(i)).size(); j++) {
                String geno = (String) ((ArrayList) data.get(i)).get(j);
                String a = "";
                String b = "";
                char c = geno.charAt(0);
                if (c == 'A') {
                    a = alleles[0];
                } else if (c == 'B') {
                    a = alleles[1];
                } else {
                    a = "N";
                }
                c = geno.charAt(1);
                if (c == 'A') {
                    b = alleles[0];
                } else if (c == 'B') {
                    b = alleles[1];
                } else {
                    b = "N";
                }
                String orig = a + b;
                al.add(orig);
            }
            dist.add(al);
        }
        ArrayListTool.saveMatrixS(dist, filename2);
    }

    public static ArrayList toBeans(String filename) {
        ArrayList dist = new ArrayList();
        ArrayList data = readData(filename);

        for (int i = 0; i < data.size(); i++) {
            GeneBean gb = new GeneBean();
            gb.setSymbol((String) ((ArrayList) data.get(i)).get(0));
            gb.setChromosome(Integer.parseInt((String) ((ArrayList) data.get(i)).get(1)));
            gb.setStart(Integer.parseInt((String) ((ArrayList) data.get(i)).get(2)));
            gb.setEnd(Integer.parseInt((String) ((ArrayList) data.get(i)).get(3)));
            dist.add(gb);
        }
        return dist;
    }

    public static ArrayList toBeans(InputStreamReader isr) {
        ArrayList dist = new ArrayList();
        ArrayList data = readData(isr);

        for (int i = 0; i < data.size(); i++) {
            GeneBean gb = new GeneBean();
            gb.setSymbol((String) ((ArrayList) data.get(i)).get(0));
            gb.setChromosome(Integer.parseInt((String) ((ArrayList) data.get(i)).get(1)));
            gb.setStart(Integer.parseInt((String) ((ArrayList) data.get(i)).get(2)));
            gb.setEnd(Integer.parseInt((String) ((ArrayList) data.get(i)).get(3)));
            gb.setStrand((String)((ArrayList) data.get(i)).get(4));
            dist.add(gb);
        }
        return dist;
    }
    
    public static ArrayList readGeneNames(int index, String filename) {
        ArrayList dist = new ArrayList();
        try {
            FileReader input = new FileReader(filename);
            BufferedReader br = new BufferedReader(input);
            String s = br.readLine();
            while (s != null) {
                StringTokenizer st = new StringTokenizer(s);
                for (int i = 0; i < index; i++) {
                    st.nextToken();
                }
                dist.add(st.nextToken());
                s = br.readLine();
            }
            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return dist;
    }

    public static ArrayList readHeaders(String filename) {
        ArrayList dist = new ArrayList();
        try {
            FileReader input = new FileReader(filename);
            BufferedReader br = new BufferedReader(input);
            String s = br.readLine();
            StringTokenizer st = new StringTokenizer(s);
            while(st.hasMoreTokens()){
                dist.add(st.nextToken());
            }
            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return dist;
    }

    public static void deleteAll(File path) {
        if (!path.exists()) {
            return;
        }
        if (path.isFile()) {
            path.delete();
            return;
        }
        File[] files = path.listFiles();
        for (int i = 0; i < files.length; i++) {
            deleteAll(files[i]);
        }
        path.delete();
    }
}
