/**
 * Department of Genetics
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2012 by the
 * Department of Genetics/University of North Carolina at Chapel Hill.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither Department of Genetics nor University of North Carolina at Chapel Hill can be responsible for
 * its use, misuse, or functionality.
 */
package edu.duke.igsp.app.tools;

import static edu.duke.igsp.app.tools.BigDecimalTool.min;
import edu.mit.broad.genome.alg.*;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.Vector;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.HashMap;

/**
 * @author Qing Xiong < qxiong@email.unc.edu >
 * Date: December, 2011
 *
 */
public class GsaaSeqTool {

    public static void sortAsc(ArrayList al, final int sortby) {
        Collections.sort(al, new Comparator<List>() {
            public int compare(List o1, List o2) {
                Float value1 = (Float) ((ArrayList) o1).get(sortby);
                Float value2 = (Float) ((ArrayList) o2).get(sortby);
                float i = value1.floatValue() - value2.floatValue();
                if (i > 0) {
                    return 1;
                } else if (i < 0) {
                    return -1;
                } else {
                    return 0;
                }
            }
        });
    }

    public static void sortDesc(ArrayList al, final int sortby) {
        Collections.sort(al, new Comparator<List>() {
            public int compare(List o1, List o2) {
                Float value1 = (Float) ((ArrayList) o1).get(sortby);
                Float value2 = (Float) ((ArrayList) o2).get(sortby);
                float i = value1.floatValue() - value2.floatValue();
                if (i > 0) {
                    return 1;
                } else if (i < 0) {
                    return -1;
                } else {
                    return 0;
                }
            }
        });
    }  

    public static ArrayList string2float(ArrayList al) {
        ArrayList bd = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList temp = new ArrayList();
            for (int j = 0; j < ((ArrayList) al.get(i)).size(); j++) {
                String value_s = (String) ((ArrayList) al.get(i)).get(j);
                Float value_bd = new Float(value_s);
                temp.add(value_bd);
            }
            bd.add(temp);
        }
        return bd;
    }

    public static ArrayList calcpMoreThan(ArrayList al) {
        ArrayList ps = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList values = (ArrayList) al.get(i);
            ArrayList temp = new ArrayList();
            for (int j = 0; j < values.size(); j++) {
                Float bd = (Float) values.get(j);
                float p = getPValueMoreThan(bd.floatValue(), values);
                temp.add(new Float(p));
            }
            ps.add(temp);
        }
        return ps;
    }

    public static ArrayList calcpMoreThanD(ArrayList al) {
        ArrayList ps = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList values = (ArrayList) al.get(i);
            ArrayList temp = new ArrayList();
            for (int j = 0; j < values.size(); j++) {
                Double bd = (Double) values.get(j);
                double p = getPValueMoreThan(bd.doubleValue(), values);
                temp.add(new Double(p));
            }
            ps.add(temp);
        }
        return ps;
    }

    public static ArrayList calcpLessThan(ArrayList al) {
        ArrayList ps = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList values = (ArrayList) al.get(i);
            ArrayList temp = new ArrayList();
            for (int j = 0; j < values.size(); j++) {
                Float bd = (Float) values.get(j);
                float p = getPValueLessThan(bd.floatValue(), values);
                temp.add(new Float(p));
            }
            ps.add(temp);
        }
        return ps;
    }
    
    public static ArrayList calcpLessThanD(ArrayList al) {
        ArrayList ps = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList values = (ArrayList) al.get(i);
            ArrayList temp = new ArrayList();
            for (int j = 0; j < values.size(); j++) {
                Double bd = (Double) values.get(j);
                double p = getPValueLessThan(bd.doubleValue(), values);
                temp.add(new Double(p));
            }
            ps.add(temp);
        }
        return ps;
    }    

    public static float getPValueLessThan(float score, ArrayList values) {
        int cntless = 0;
        int size = values.size();

        for (int i = 0; i < size; i++) {
            float value = ((Float) values.get(i)).floatValue();
            if (value <= score) {
                cntless++;
            }
        }
        return ((float) cntless) / (float) size;
    }

    public static float getPValueMoreThan(float score, ArrayList values) {
        int cntmore = 0;
        int size = values.size();

        for (int i = 0; i < size; i++) {
            float value = ((Float) values.get(i)).floatValue();
            if (score <= value) {
                cntmore++;
            }
        }
        return ((float) cntmore) / (float) size;
    }   
    
    public static double getPValueLessThan(double score, ArrayList values) {
        int cntless = 0;
        int size = values.size();

        for (int i = 0; i < size; i++) {
            double value = ((Double) values.get(i)).doubleValue();
            if (value <= score) {
                cntless++;
            }
        }
        return ((double) cntless) / (double) size;
    } 
    
    public static double getPValueMoreThan(double score, ArrayList values) {
        int cntmore = 0;
        int size = values.size();

        for (int i = 0; i < size; i++) {
            double value = ((Double) values.get(i)).doubleValue();
            if (value >= score) {
                cntmore++;
            }
        }
        return ((double) cntmore) / (double) size;
    }    

    public static ArrayList summary(ArrayList al) {
        ArrayList summaries = new ArrayList();
        ArrayList values = (ArrayList) al.get(0);
        summaries.add(values);
        for (int i = 1; i < al.size(); i++) {
            ArrayList values1 = (ArrayList) al.get(i);
            ArrayList values2 = (ArrayList) summaries.get(i - 1);
            ArrayList temp = new ArrayList();
            for (int j = 0; j < values1.size(); j++) {
                Float bd1 = (Float) values1.get(j);
                Float bd2 = (Float) values2.get(j);
                temp.add(bd1.floatValue() * bd2.floatValue());
            }
            summaries.add(temp);
        }
        return summaries;
    }

    public static ArrayList findMinP(ArrayList al) {
        ArrayList minps = new ArrayList();
        for (int i = 0; i < ((ArrayList) al.get(0)).size(); i++) {
            ArrayList temp = new ArrayList();
            for (int j = 0; j < al.size(); j++) {
                Float f = (Float) ((ArrayList) al.get(j)).get(i);
                temp.add(f);
            }
            float minp = min(temp);
            minps.add(new Float(minp));
        }
        return minps;
    }
    
    public static float findMin(ArrayList al) {
        ArrayList mins = new ArrayList();
        for (int i = 0; i < ((ArrayList) al.get(0)).size(); i++) {
            ArrayList temp = new ArrayList();
            for (int j = 0; j < al.size(); j++) {
                Float f = (Float) ((ArrayList) al.get(j)).get(i);
                temp.add(f);
            }
            float min = min(temp);
            mins.add(new Float(min));
        }
        float min = min(mins);
        return min;
    }  
    
    public static float findNoZeroMin(ArrayList al) {
        ArrayList mins = new ArrayList();
        for (int i = 0; i < ((ArrayList) al.get(0)).size(); i++) {
            ArrayList temp = new ArrayList();
            for (int j = 0; j < al.size(); j++) {
                Float f = (Float) ((ArrayList) al.get(j)).get(i);
                temp.add(f);
            }
            float min = nozeromin(temp);
            mins.add(new Float(min));
        }
        float min = nozeromin(mins);
        return min;
    }     
    
    public static float findMax(ArrayList al) {
        ArrayList maxs = new ArrayList();
        for (int i = 0; i < ((ArrayList) al.get(0)).size(); i++) {
            ArrayList temp = new ArrayList();
            for (int j = 0; j < al.size(); j++) {
                Float f = (Float) ((ArrayList) al.get(j)).get(i);
                temp.add(f);
            }
            float max = max(temp);
            maxs.add(new Float(max));
        }
        float max = max(maxs);
        return max;
    }      

    public static float sum(final ArrayList a) {
        float sum = 0;
        for (int i = 0; i < a.size(); i++) {
            float f = ((Float) a.get(i)).floatValue();
            sum += f;
        }
        return sum;
    }
    
    public static float sum(float[] a) {
        float sum = 0;
        for (int i = 0; i < a.length; i++) {
            float f = a[i];
            sum += f;
        }
        return sum;
    } 
    
    public static double sum(double[] a) {
        double sum = 0;
        for (int i = 0; i < a.length; i++) {
            double f = a[i];
            sum += f;
        }
        return sum;
    } 
    
    public static int sum(int[] a) {
        int sum = 0;
        for (int i = 0; i < a.length; i++) {
            int f = a[i];
            sum += f;
        }
        return sum;
    }    

    public static float mean(final ArrayList a) {
        int size = a.size();
        return sum(a) / (float)size;
    }

    public static float max(final ArrayList a) {
        float max = ((Float) a.get(0)).floatValue();
        for (int i = 1; i < a.size(); i++) {
            float f = ((Float) a.get(i)).floatValue();
            if (f > max) {
                max = f;
            }
        }
        return max;
    }

    public static float min(final ArrayList a) {
        float min = ((Float) a.get(0)).floatValue();
        for (int i = 1; i < a.size(); i++) {
            float f = ((Float) a.get(i)).floatValue();
            if (f < min) {
                min = f;
            }
        }
        return min;
    }
    
    public static double min(final double[] d) {
        double min = d[0];
        for (int i = 1; i < d.length; i++) {
            double value = d[i];
            if (value < min) {
                min = value;
            }
        }
        return min;
    }    
    
    public static float nozeromin(final ArrayList a) {
        float min = ((Float) a.get(0)).floatValue();
        for (int i = 1; i < a.size(); i++) {
            float f = ((Float) a.get(i)).floatValue();
            if (f<min&&f!=0) {
                min = f;
            }
        }
        return min;
    }       

    public static HashMap match(ArrayList genesets, String[] genes, ArrayList values) {
        HashMap hm = new HashMap();
        for (int i = 0; i < genesets.size(); i++) {
            ArrayList geneset = (ArrayList) genesets.get(i);
            String geneset_name = (String) geneset.get(0);
            ArrayList commonItems = new ArrayList();
            for (int j = 1; j < geneset.size(); j++) {
                String name1 = (String) geneset.get(j);
                for (int k = 0; k < genes.length; k++) {
                    String name2 = genes[k];
                    if (name1.equalsIgnoreCase(name2)) {
                        commonItems.add((ArrayList) values.get(k));
                        break;
                    }
                }
            }
            hm.put(geneset_name, commonItems);
        }
        return hm;
    }

    public static ArrayList shorten(ArrayList values, String threshold) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < values.size(); i++) {
            Float bd = (Float) ((ArrayList) values.get(i)).get(0);
            if (bd.floatValue() <= (new Float(threshold)).floatValue()) {
                al.add(values.get(i));
            }
        }
        return al;
    }

    public static double[] calcAS(double[] ratios, double[] diffs) {
        double[] scores = new double[ratios.length];
        for (int i = 0; i < ratios.length; i++) {
            double p = getPValueMoreThan(ratios[i], ratios, diffs[i], diffs);
            double score = (-1) * Math.log10(p);
            scores[i] = score;
        }
        return scores;
    }
    
    public static double[] calcAS(double[] values) {
        double[] scores = new double[values.length];
        for (int i = 0; i < values.length; i++) {
            double p = getPValueMoreThan(values[i], values);
            double score = (-1) * Math.log10(p);
            scores[i] = score;
        }
        return scores;
    }    

    public static double[] calcPMoreThan(double[] ratios, double[] diffs) {
        double[] scores = new double[ratios.length];
        for (int i = 0; i < ratios.length; i++) {
            double p = getPValueMoreThan(ratios[i], ratios, diffs[i], diffs);
            scores[i] = p;
        }
        return scores;
    } 
    

    public static double[] calcPMoreThan(double[] values) {
        double[] scores = new double[values.length];
        for (int i = 0; i < values.length; i++) {
            double p = getPValueMoreThan(values[i], values);
            scores[i] = p;
        }
        return scores;
    }
    
    public static double getPValueMoreThan(double score, double[] values) {
        int cntmore = 0;
        int size = values.length;

        for (int i = 0; i < size; i++) {
            if (values[i] >= score) {
                cntmore++;
            }
        }
        return ((double) cntmore) / (double) size;
    }    

    public static double getPValueMoreThan(final double ratio, final double[] ratios, final double diff, final double[] diffs) {
        int cntmore = 0;

        for (int i = 0; i < ratios.length; i++) {
            if (ratios[i] >= ratio && diffs[i] >= diff) {
                cntmore++;
            }
        }

        return ((double) cntmore) / (double) ratios.length;
    }
    

    public static double getPValueLessThan(final double score1, final ArrayList al1, final double score2, final ArrayList al2) {
        int cntless = 0;

        for (int i = 0; i < al1.size(); i++) {
            double value1 = ((Double) al1.get(i)).doubleValue();
            double value2 = ((Double) al2.get(i)).doubleValue();
            if (value1 <= score1 && value2 <= score2) {
                cntless++;
            }
        }

        return ((double) cntless) / (double) al1.size();
    }

    public static double getPValueMoreThan(final double score1, final ArrayList al1, final double score2, final ArrayList al2) {
        int cntmore = 0;

        for (int i = 0; i < al1.size(); i++) {
            double value1 = ((Double) al1.get(i)).doubleValue();
            double value2 = ((Double) al2.get(i)).doubleValue();
            if (value1 >= score1 && value2 >= score2) {
                cntmore++;
            }
        }

        return ((double) cntmore) / (double) al1.size();
    }    

    public static ArrayList calcScoresLog2RatioSeqAbs(final Dataset ds, final Template template, final Template[] rndTemplates) {
        ArrayList scores = new ArrayList();
        for (int i = 0; i < ds.getNumRow(); i++) {
            ArrayList al = new ArrayList();
            scores.add(al);
        }

        Metric metric = (Metrics.createClassMeansLog2RatioMetrics())[0];
        for (int i = 0; i < rndTemplates.length + 1; i++) {
            for (int j = 0; j < ds.getNumRow(); j++) {
                double dist_exp = 0;
                if (i == 0) {
                    dist_exp = metric.getScore(ds.getRow(j), template, null);
                } else {
                    dist_exp = metric.getScore(ds.getRow(j), rndTemplates[i - 1], null);
                }
                dist_exp = Math.abs(dist_exp);
                ((ArrayList) scores.get(j)).add(new Double(dist_exp));
            }
        }

        return scores;
    }  
    
    public static ArrayList calcScoresLog2RatioSeq(final Dataset ds, final Template template, final Template[] rndTemplates) {
        ArrayList scores = new ArrayList();
        for (int i = 0; i < ds.getNumRow(); i++) {
            ArrayList al = new ArrayList();
            scores.add(al);
        }

        Metric metric = (Metrics.createClassMeansLog2RatioMetrics())[0];
        for (int i = 0; i < rndTemplates.length + 1; i++) {
            for (int j = 0; j < ds.getNumRow(); j++) {
                double dist_exp = 0;
                if (i == 0) {
                    dist_exp = metric.getScore(ds.getRow(j), template, null);
                } else {
                    dist_exp = metric.getScore(ds.getRow(j), rndTemplates[i - 1], null);
                }
                ((ArrayList) scores.get(j)).add(new Double(dist_exp));
            }
        }

        return scores;
    }    

    public static ArrayList calcScoresSignal2NoiseSeqAbs(final Dataset ds, final Template template, final Template[] rndTemplates) {
        ArrayList scores = new ArrayList();
        for (int i = 0; i < ds.getNumRow(); i++) {
            ArrayList al = new ArrayList();
            scores.add(al);
        }

        Metric metric = (Metrics.createSignal2NoiseSeqMetrics())[0];
        for (int i = 0; i < rndTemplates.length + 1; i++) {
            for (int j = 0; j < ds.getNumRow(); j++) {
                double dist_exp = 0;
                if (i == 0) {
                    dist_exp = metric.getScore(ds.getRow(j), template, null);
                } else {
                    dist_exp = metric.getScore(ds.getRow(j), rndTemplates[i - 1], null);
                }

                dist_exp = Math.abs(dist_exp);
                ((ArrayList) scores.get(j)).add(new Double(dist_exp));
            }
        }
        return scores;
    } 
    
    public static ArrayList calcScoresSignal2NoiseSeq(final Dataset ds, final Template template, final Template[] rndTemplates) {
        ArrayList scores = new ArrayList();
        for (int i = 0; i < ds.getNumRow(); i++) {
            ArrayList al = new ArrayList();
            scores.add(al);
        }

        Metric metric = (Metrics.createSignal2NoiseSeqMetrics())[0];
        for (int i = 0; i < rndTemplates.length + 1; i++) {
            for (int j = 0; j < ds.getNumRow(); j++) {
                double dist_exp = 0;
                if (i == 0) {
                    dist_exp = metric.getScore(ds.getRow(j), template, null);
                } else {
                    dist_exp = metric.getScore(ds.getRow(j), rndTemplates[i - 1], null);
                }

                ((ArrayList) scores.get(j)).add(new Double(dist_exp));
            }
        }
        return scores;
    }     
    
    public static double[] calcScoresLog2RatioSeqAbs(final Dataset ds, final Template template) {
        double[] scores = new double[ds.getNumRow()];
        Metric metric = (Metrics.createClassMeansLog2RatioMetrics())[0];
        for (int i = 0; i < ds.getNumRow(); i++) {
            double dist = metric.getScore(ds.getRow(i), template, null);
            dist = Math.abs(dist);
            scores[i] = dist;
        }

        return scores;
    }   
    
    public static double[] calcScoresLog2RatioSeq(final Dataset ds, final Template template) {
        double[] scores = new double[ds.getNumRow()];
        Metric metric = (Metrics.createClassMeansLog2RatioMetrics())[0];
        for (int i = 0; i < ds.getNumRow(); i++) {
            double dist = metric.getScore(ds.getRow(i), template, null);
            scores[i] = dist;
        }

        return scores;
    }     

    public static double[] calcScoresSignal2NoiseSeqAbs(final Dataset ds, final Template template) {
        double[] scores = new double[ds.getNumRow()];
        Metric metric = (Metrics.createSignal2NoiseSeqMetrics())[0];
        for (int i = 0; i < ds.getNumRow(); i++) {
            Vector row = ds.getRow(i);
            double dist = metric.getScore(row, template, null);
            dist = Math.abs(dist);
            scores[i] = dist;
        }

        return scores;
    }
    
    public static double[] calcScoresSignal2NoiseSeq(final Dataset ds, final Template template) {
        double[] scores = new double[ds.getNumRow()];
        Metric metric = (Metrics.createSignal2NoiseSeqMetrics())[0];
        for (int i = 0; i < ds.getNumRow(); i++) {
            Vector row = ds.getRow(i);
            double dist = metric.getScore(row, template, null);
            scores[i] = dist;
        }

        return scores;
    }    

    public static ArrayList calcP(ArrayList dataset1, ArrayList dataset2) {
        ArrayList results = new ArrayList();
        for (int i = 0; i < dataset1.size(); i++) {
            ArrayList rowValues1 = (ArrayList) dataset1.get(i);
            ArrayList rowValues2 = (ArrayList) dataset2.get(i);
            ArrayList rowResults = new ArrayList();
            for (int j = 0; j < rowValues1.size(); j++) {
                double value1 = ((Double) rowValues1.get(j)).doubleValue();
                double value2 = ((Double) rowValues2.get(j)).doubleValue();
                double p = getPValueMoreThan(value1, rowValues1, value2, rowValues2);
                rowResults.add(new Double(p));
            }
            results.add(rowResults);
        }
        return results;
    }
    

    public static ArrayList calcAS(ArrayList dataset1, ArrayList dataset2) {
        ArrayList results = new ArrayList();
        for (int i = 0; i < dataset1.size(); i++) {
            ArrayList rowValues1 = (ArrayList) dataset1.get(i);
            ArrayList rowValues2 = (ArrayList) dataset2.get(i);
            ArrayList rowResults = new ArrayList();
            for (int j = 0; j < rowValues1.size(); j++) {
                double value1 = ((Double) rowValues1.get(j)).doubleValue();
                double value2 = ((Double) rowValues2.get(j)).doubleValue();
                double p = getPValueMoreThan(value1, rowValues1, value2, rowValues2);
                double score = (-1) * Math.log10(p);
                rowResults.add(new Double(score));
            }
            results.add(rowResults);
        }
        return results;
    }    

    public static ArrayList calcPCol(ArrayList dataset1, ArrayList dataset2) {
        ArrayList results = new ArrayList();
        for (int i = 0; i < dataset1.size(); i++) {
            results.add(new ArrayList());
        }
        for (int i = 0; i < ((ArrayList) dataset1.get(0)).size(); i++) {
            ArrayList colValues1 = new ArrayList();
            ArrayList colValues2 = new ArrayList();
            for (int j = 0; j < dataset1.size(); j++) {
                colValues1.add(((ArrayList) dataset1.get(j)).get(i));
                colValues2.add(((ArrayList) dataset2.get(j)).get(i));
            }

            for (int j = 0; j < colValues1.size(); j++) {
                double value1 = ((Double) colValues1.get(j)).doubleValue();
                double value2 = ((Double) colValues2.get(j)).doubleValue();
                double p = getPValueMoreThan(value1, colValues1, value2, colValues2);
                ((ArrayList) results.get(j)).add(new Double(p));
            }
        }
        return results;
    }

    public static ArrayList calcAsCol(ArrayList dataset1, ArrayList dataset2) {
        ArrayList results = new ArrayList();
        for (int i = 0; i < dataset1.size(); i++) {
            results.add(new ArrayList());
        }
        for (int i = 0; i < ((ArrayList) dataset1.get(0)).size(); i++) {
            ArrayList colValues1 = new ArrayList();
            ArrayList colValues2 = new ArrayList();
            for (int j = 0; j < dataset1.size(); j++) {
                colValues1.add(((ArrayList) dataset1.get(j)).get(i));
                colValues2.add(((ArrayList) dataset2.get(j)).get(i));
            }

            for (int j = 0; j < colValues1.size(); j++) {
                double value1 = ((Double) colValues1.get(j)).doubleValue();
                double value2 = ((Double) colValues2.get(j)).doubleValue();
                double p = getPValueMoreThan(value1, colValues1, value2, colValues2);
                double score = (-1) * Math.log10(p);
                ((ArrayList) results.get(j)).add(new Double(score));
            }
        }
        return results;
    }

    public static ArrayList calcAS(ArrayList al) {
        ArrayList ass = new ArrayList();
        for (int i = 0; i < al.size(); i++) {
            ArrayList values = (ArrayList) al.get(i);
            ArrayList temp = new ArrayList();
            for (int j = 0; j < values.size(); j++) {
                Double bd = (Double) values.get(j);
                double p = getPValueMoreThan(bd.doubleValue(), values);
                double as = (-1) * Math.log(p);
                temp.add(new Double(as));
            }
            ass.add(temp);
        }
        return ass;
    }   

    public static double[] estimateSizeFactors(final Matrix matrix) {
        double[] sizes = new double[matrix.getNumCol()];
        Vector gmeans = new Vector(matrix.getNumRow());
        for (int i = 0; i < matrix.getNumRow(); i++) {
            double[] cnts = matrix.getRowV(i).toArrayDouble();
            double sum = 0;
            for (int j = 0; j < cnts.length; j++) {
                sum += Math.log(cnts[j]);
            }
            double gmean = sum / (double) cnts.length;
            gmeans.setElement(i, gmean);
        }

        for (int i = 0; i < matrix.getNumCol(); i++) {
            Vector v = new Vector(matrix.getNumRow());
            for (int j = 0; j < matrix.getNumRow(); j++) {
                double cnt = Math.log(matrix.getElement(j, i));
                v.setElement(j, cnt - gmeans.getElement(j));
            }
            double median = v.median();
            sizes[i] = Math.exp(median);
        }

        return sizes;
    }   
    
    public static double[] estimateSizeFactorsTrim(final Matrix matrix, final Template template) {
        Matrix counts = normalizeTotal(matrix);
        ArrayList al = new ArrayList();
        Metric metric = (Metrics.createClassMeansLog2RatioMetrics())[0];
        for (int i = 0; i < counts.getNumRow(); i++) {
            Vector vector = counts.getRowV(i);
            double d = metric.getScore(vector, template, null);
            d=Math.abs(d);
            double mean = matrix.getRowV(i).mean();
            //a/b>0.85 (a<b)
            if(d<0.234&&mean>100){
                al.add(matrix.getRowV(i));
            }
        }
        int numRow = al.size();
        int numCol = matrix.getNumCol();
        Matrix m = new Matrix(numRow,numCol);
        for (int i = 0; i < numRow; i++) {
            m.setRow(i, (Vector)al.get(i));
        }
        
        double[] sizes = new double[m.getNumCol()];
        Vector gmeans = new Vector(m.getNumRow());
        for (int i = 0; i < m.getNumRow(); i++) {
            double[] cnts = m.getRowV(i).toArrayDouble();
            double sum = 0;
            for (int j = 0; j < cnts.length; j++) {
                sum += Math.log(cnts[j]);
            }
            double gmean = sum / (double) cnts.length;
            gmeans.setElement(i, gmean);
        }

        for (int i = 0; i < m.getNumCol(); i++) {
            Vector v = new Vector(m.getNumRow());
            for (int j = 0; j < m.getNumRow(); j++) {
                double cnt = Math.log(m.getElement(j, i));
                v.setElement(j, cnt - gmeans.getElement(j));
            }
            double median = v.median();
            sizes[i] = Math.exp(median);
        }

        return sizes;
    }    

    public static Matrix normalizeTM(Matrix matrix) {
        Matrix newMatrix = new Matrix(matrix.getNumRow(), matrix.getNumCol());
        for (int i = 0; i < matrix.getNumCol(); i++) {
            double sum = matrix.getColumnV(i).sum();
            for (int j = 0; j < matrix.getNumRow(); j++) {
                double value = (double) matrix.getElement(j, i);
                value = value / sum;
                newMatrix.setElement(j, i, (float) value);
            }
        }
        for (int i = 0; i < newMatrix.getNumRow(); i++) {
            double mean = newMatrix.getRowV(i).mean();
            for (int j = 0; j < newMatrix.getNumCol(); j++) {
                double value = (double) newMatrix.getElement(i, j);
                value = value / mean;
                newMatrix.setElement(i, j, (float) value);
            }
        }
        return newMatrix;
    }

    public static Matrix normalizeTotal(final Matrix matrix) {
        Matrix newMatrix = new Matrix(matrix.getNumRow(), matrix.getNumCol());
        for (int i = 0; i < matrix.getNumCol(); i++) {
            double sum = matrix.getColumnV(i).sum();
            for (int j = 0; j < matrix.getNumRow(); j++) {
                double value = (double) matrix.getElement(j, i);
                value = value / sum;
                newMatrix.setElement(j, i, (float) value);
            }
        }
        return newMatrix;
    }
    
    public static ArrayList normalizeTotal(final ArrayList matrix) {
        ArrayList newMatrix = new ArrayList();
        for (int i = 0; i < matrix.size(); i++) {
            ArrayList temp = new ArrayList();
            newMatrix.add(temp);
        }
        for (int i = 0; i < ((ArrayList)matrix.get(0)).size(); i++) {
            float sum = 0;
            for (int j = 0; j < matrix.size(); j++) {
                float value = ((Float)((ArrayList) matrix.get(j)).get(i)).floatValue();
                sum+=value;
            }            
            for (int j = 0; j < matrix.size(); j++) {
                float value = ((Float)((ArrayList) matrix.get(j)).get(i)).floatValue();
                value = value / sum;
                ((ArrayList)newMatrix.get(j)).add(new Float(value));
            }
        }
        return newMatrix;
    }    
    
    public static Matrix normalizeMedian(Matrix matrix) {
        Matrix newMatrix = new Matrix(matrix.getNumRow(), matrix.getNumCol());
        for (int i = 0; i < matrix.getNumCol(); i++) {
            double median = matrix.getColumnV(i).median();
            for (int j = 0; j < matrix.getNumRow(); j++) {
                double value = (double) matrix.getElement(j, i);
                value = value / median;
                newMatrix.setElement(j, i, (float) value);
            }
        }
        return newMatrix;
    }    

    public static Matrix normalizeDESeq(final Matrix matrix) {
        Matrix newMatrix = new Matrix(matrix.getNumRow(), matrix.getNumCol());
        double[] sf = estimateSizeFactors(matrix);
        for (int i = 0; i < matrix.getNumRow(); i++) {
            Vector row = matrix.getRowV(i);
            for (int j = 0; j < matrix.getNumCol(); j++) {
                newMatrix.setElement(i, j, (float) ((row.getElement(j)) / sf[j]));
            }
        }
        return newMatrix;
    }
    
    public static Matrix normalizeDESeqTrim(final Matrix matrix, final Template template) {
        Matrix newMatrix = new Matrix(matrix.getNumRow(), matrix.getNumCol());
        double[] sf = estimateSizeFactorsTrim(matrix, template);
        for (int i = 0; i < matrix.getNumRow(); i++) {
            Vector row = matrix.getRowV(i);
            for (int j = 0; j < matrix.getNumCol(); j++) {
                newMatrix.setElement(i, j, (float) ((row.getElement(j)) / sf[j]));
            }
        }
        return newMatrix;
    }    

    public static Matrix normalizeByPhenoDESeq(Matrix matrix, Template t) {
        int numCols = matrix.getNumCol();
        if (numCols == 2) {
            return normalizeDESeq(matrix);
        } else {
            Matrix newMatrix = new Matrix(matrix.getNumRow(), matrix.getNumCol());
            String class1 = t.getClassName(0);
            String class2 = t.getClassName(1);
            double[][] loggmeans = new double[matrix.getNumRow()][2];
            for (int i = 0; i < matrix.getNumRow(); i++) {
                Vector v = matrix.getRowV(i);
                Vector[] vs = t.splitByTemplateClass(v);
                double[] cnts1 = vs[0].toArrayDouble();
                double sum1 = 0;
                for (int j = 0; j < cnts1.length; j++) {
                    sum1 += Math.log(cnts1[j]);
                }
                loggmeans[i][0] = sum1 / (double) cnts1.length;
                double[] cnts2 = vs[1].toArrayDouble();
                double sum2 = 0;
                for (int j = 0; j < cnts2.length; j++) {
                    sum2 += Math.log(cnts2[j]);
                }
                loggmeans[i][1] = sum2 / (double) cnts2.length;
                for (int j = 0; j < matrix.getNumCol(); j++) {
                    String cname = t.getClassByProfilePos(j).getName();
                    if (cname.equalsIgnoreCase(class1)) {
                        double value = Math.log((double)v.getElement(j))-loggmeans[i][0];
                        newMatrix.setElement(i, j, (float) value);
                    }
                    if(cname.equalsIgnoreCase(class2)){
                        double value = Math.log((double)v.getElement(j))-loggmeans[i][1];
                        newMatrix.setElement(i, j, (float) value);
                    }
                }                
            }
            Matrix resultMatrix = new Matrix(matrix.getNumRow(), matrix.getNumCol());
            for (int i = 0; i < matrix.getNumCol(); i++) {
                double sf = Math.exp(newMatrix.getColumnV(i).median());
                for (int j = 0; j < matrix.getNumRow(); j++) {
                    double value = (double) matrix.getElement(j, i);
                    value = value / sf;
                    resultMatrix.setElement(j, i, (float) value);
                }
            }
            return resultMatrix;
        }
    }
    
    public static double calcL2Norm(Vector v){
        double l2norm = 0;
        for(int i=0; i<v.getSize(); i++){
            double d = ((Float)v.getElement(i)).doubleValue();
            l2norm+=d*d;
        }
        return l2norm;
    }
    
    public static double calcL1Norm(Vector v){
        double l1norm = 0;
        for(int i=0; i<v.getSize(); i++){
            double d = ((Float)v.getElement(i)).doubleValue();
            l1norm+=d;
        }
        return l1norm;
    }    
    
    public static double calcLnNorm(Vector v, float n){
        double lnnorm = 0;
        for(int i=0; i<v.getSize(); i++){
            double d = ((Float)v.getElement(i)).doubleValue();
            lnnorm+=Math.pow(d, (double)n);
        }
        return lnnorm;
    }      
    
    public static double calcLnNorm(ArrayList v, double n){
        double lnnorm = 0;
        for(int i=0; i<v.size(); i++){
            double d = ((Double)v.get(i)).doubleValue();
            lnnorm+=Math.pow(d, n);
        }
        return lnnorm;
    }      
    
    public static void normalizeByMeanSdRow(ArrayList scores1, ArrayList scores2) {
        Vector mins1 = new Vector(scores1.size());
        Vector mins2 = new Vector(scores2.size());
        for (int i = 0; i < scores1.size(); i++) {
            ArrayList al = (ArrayList) scores1.get(i);
            Vector v = new Vector(al.size() - 1);
            for (int j = 0; j < v.getSize(); j++) {
                v.setElement(j, ((Double) al.get(j + 1)).doubleValue());
            }
            double mean = v.mean();
            double sd = v.stddev(false, false);
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                al.set(j, new Double((d - mean) / sd));
            }
            double min = ((Double) Collections.min(al)).doubleValue();
            mins1.setElement(i, min);
        }

        for (int i = 0; i < scores2.size(); i++) {
            ArrayList al = (ArrayList) scores2.get(i);
            Vector v = new Vector(al.size() - 1);
            for (int j = 0; j < v.getSize(); j++) {
                v.setElement(j, ((Double) al.get(j + 1)).doubleValue());
            }
            double mean = v.mean();
            double sd = Math.sqrt(v.var(false, false));
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                al.set(j, new Double((d - mean) / sd));
            }
            double min = ((Double) Collections.min(al)).doubleValue();
            mins2.setElement(i, min);
        }

        double min1 = mins1.min();
        double min2 = mins2.min();
        double min = 0;
        if (min1 <= min2) {
            min = min1;
        } else {
            min = min2;
        }

        for (int i = 0; i < scores1.size(); i++) {
            ArrayList al = (ArrayList) scores1.get(i);
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                if (Double.isNaN(d) || Double.isInfinite(d)) {
                    d = 0.000001;
                } else {
                    d = d+Math.abs(min);
                }
                al.set(j, new Double(d));
            }
        }

        for (int i = 0; i < scores2.size(); i++) {
            ArrayList al = (ArrayList) scores2.get(i);
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                if (Double.isNaN(d) || Double.isInfinite(d)) {
                    d = 0.000001;
                } else {
                    d = d+Math.abs(min);
                }
                al.set(j, new Double(d));
            }
        }
    }    

    public static void normalizeByMeanSdRow(ArrayList scores) {
        Vector mins = new Vector(scores.size());
        for (int i = 0; i < scores.size(); i++) {
            ArrayList al = (ArrayList) scores.get(i);
            Vector v = new Vector(al.size() - 1);
            for (int j = 0; j < v.getSize(); j++) {
                v.setElement(j, ((Double) al.get(j + 1)).doubleValue());
            }
            double mean = v.mean();
            double sd = Math.sqrt(v.var(false, false));
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                al.set(j, new Double((d - mean) / sd));
            }
            double min = ((Double) Collections.min(al)).doubleValue();
            mins.setElement(i, min);
        }
        double min = (double) mins.min();
        for (int i = 0; i < scores.size(); i++) {
            ArrayList al = (ArrayList) scores.get(i);
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                if (Double.isNaN(d) || Double.isInfinite(d)) {
                    d = 0.000001;
                } else {
                    d = d+Math.abs(min);
                }
                al.set(j, new Double(d));
            }
        }
    }
    
    public static void normalizeByMeanSdRow(double[][] scores1, double[][] scores2) {
        Vector mins1 = new Vector(scores1.length);
        Vector mins2 = new Vector(scores2.length);
        for (int i = 0; i < scores1.length; i++) {
            Vector v = new Vector(scores1[i].length - 1);
            for (int j = 0; j < v.getSize(); j++) {
                v.setElement(j, scores1[i][j+1]);
            }
            double mean = v.mean();
            double sd = v.stddev(false, false);
            for (int j = 0; j < scores1[i].length; j++) {
                scores1[i][j] =(scores1[i][j] - mean) / sd;
            }
            double min = min(scores1[i]);
            mins1.setElement(i, min);
        }

        for (int i = 0; i < scores2.length; i++) {
            Vector v = new Vector(scores2[i].length - 1);
            for (int j = 0; j < v.getSize(); j++) {
                v.setElement(j, scores2[i][j+1]);
            }
            double mean = v.mean();
            double sd = Math.sqrt(v.var(false, false));
            for (int j = 0; j < scores2[i].length; j++) {
                scores2[i][j] =(scores2[i][j] - mean) / sd;
            }
            double min = min(scores2[i]);
            mins2.setElement(i, min);
        }

        double min1 = mins1.min();
        double min2 = mins2.min();
        double min = 0;
        if (min1 <= min2) {
            min = min1;
        } else {
            min = min2;
        }

        for (int i = 0; i < scores1.length; i++) {
            for (int j = 0; j < scores1[i].length; j++) {
                if (Double.isNaN(scores1[i][j]) || Double.isInfinite(scores1[i][j])) {
                    scores1[i][j] = 0.000001;
                } else {
                    scores1[i][j] = scores1[i][j]+Math.abs(min);
                }
            }
        }

        for (int i = 0; i < scores2.length; i++) {
            for (int j = 0; j < scores2[i].length; j++) {
                if (Double.isNaN(scores2[i][j]) || Double.isInfinite(scores2[i][j])) {
                    scores2[i][j] = 0.000001;
                } else {
                    scores2[i][j] = scores2[i][j]+Math.abs(min);
                }
            }
        }
    }    

    public static void normalizeByMeanSdRow(double[][] scores) {
        Vector mins = new Vector(scores.length);
        for (int i = 0; i < scores.length; i++) {
            Vector v = new Vector(scores[i].length - 1);
            for (int j = 0; j < v.getSize(); j++) {
                v.setElement(j, scores[i][j+1]);
            }
            double mean = v.mean();
            double sd = Math.sqrt(v.var(false, false));
            for (int j = 0; j < scores[i].length; j++) {
                scores[i][j]=(scores[i][j] - mean) / sd;
            }
            double min = min(scores[i]);
            mins.setElement(i, min);
        }
        double min = (double) mins.min();
        for (int i = 0; i < scores.length; i++) {
            for (int j = 0; j < scores[i].length; j++) {
                if (Double.isNaN(scores[i][j]) || Double.isInfinite(scores[i][j])) {
                    scores[i][j] = 0.000001;
                } else {
                    scores[i][j] = scores[i][j]+Math.abs(min);
                }
            }
        }
    }
    
    public static void normalizeByMeanSdCol(ArrayList scores1, ArrayList scores2) {
        Object firstElement1 = ((ArrayList) scores1.get(0)).get(0);
        double flag1 = 0;
        if (firstElement1.getClass().getSimpleName().equalsIgnoreCase("Double")) {
            flag1=1;
        }
        Object firstElement2 = ((ArrayList) scores2.get(0)).get(0);
        double flag2 = 0;
        if (firstElement2.getClass().getSimpleName().equalsIgnoreCase("Double")) {
            flag2=1;
        }

        int sizeCol1 = ((ArrayList)scores1.get(0)).size();
        Vector mins1 = new Vector(sizeCol1);
        for (int i = 0; i < sizeCol1; i++) {
            int sizeRow = scores1.size();
            Vector v = new Vector(sizeRow);
            for(int j = 0; j < v.getSize(); j++) {
                ArrayList al = (ArrayList)scores1.get(j);
                double score = 0;
                if (flag1 == 0) {
                    score = ((Float) (al).get(i)).doubleValue();
                } else {
                    score = ((Double) (al).get(i)).doubleValue();
                }
                v.setElement(j, score);
            }
            double mean = v.mean();
            double sd = Math.sqrt(v.var(false, false));
            for (int j = 0; j < v.getSize(); j++) {
                ArrayList al = (ArrayList)scores1.get(j);
                double score = 0;
                if (flag1 == 0) {
                    score = ((Float) (al).get(i)).doubleValue();
                } else {
                    score = ((Double) (al).get(i)).doubleValue();
                }
                al.set(i, new Double((score - mean) / sd));
                v.setElement(j, new Double((score - mean) / sd));
            }
            double min = (double)v.min();
            mins1.setElement(i, min);
        }

        int sizeCol2 = ((ArrayList)scores2.get(0)).size();
        Vector mins2 = new Vector(sizeCol2);
        for (int i = 0; i < sizeCol2; i++) {
            int sizeRow = scores2.size();
            Vector v = new Vector(sizeRow);
            for(int j = 0; j < v.getSize(); j++) {
                ArrayList al = (ArrayList)scores2.get(j);
                double score = 0;
                if (flag2 == 0) {
                    score = ((Float) (al).get(i)).doubleValue();
                } else {
                    score = ((Double) (al).get(i)).doubleValue();
                }
                v.setElement(j, score);
            }
            double mean = v.mean();
            double sd = Math.sqrt(v.var(false, false));
            for (int j = 0; j < v.getSize(); j++) {
                ArrayList al = (ArrayList)scores2.get(j);
                double score = 0;
                if (flag2 == 0) {
                    score = ((Float) (al).get(i)).doubleValue();
                } else {
                    score = ((Double) (al).get(i)).doubleValue();
                }
                al.set(i, new Double((score - mean) / sd));
                v.setElement(j, new Double((score - mean) / sd));
            }
            double min = (double)v.min();
            mins2.setElement(i, min);
        }

        double min1 = mins1.min();
        double min2 = mins2.min();
        double min = 0;
        if (min1 <= min2) {
            min = min1;
        } else {
            min = min2;
        }

        for (int i = 0; i < scores1.size(); i++) {
            ArrayList al = (ArrayList) scores1.get(i);
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                if (Double.isNaN(d) || Double.isInfinite(d)) {
                    d = 0.000001;
                } else {
                    d = d+Math.abs(min);
                }
                if (flag1 == 0) {
                    al.set(j, new Float(d));
                } else {
                    al.set(j, new Double(d));
                }                
            }
        }

        for (int i = 0; i < scores2.size(); i++) {
            ArrayList al = (ArrayList) scores2.get(i);
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                if (Double.isNaN(d) || Double.isInfinite(d)) {
                    d = 0.000001;
                } else {
                    d = d+Math.abs(min);
                }
                if (flag2 == 0) {
                    al.set(j, new Float(d));
                } else {
                    al.set(j, new Double(d));
                }
            }
        }
    }  
    
    public static void normalizeByMeanSdCol(double[] scores1, double[] scores2) {
        int size1 = scores1.length;
        Vector v1 = new Vector(size1);
        for (int i = 0; i < v1.getSize(); i++) {
            v1.setElement(i, scores1[i]);
        }
        double mean1 = v1.mean();
        double sd1 = Math.sqrt(v1.var(false, false));
        for (int i = 0; i < v1.getSize(); i++) {
            scores1[i] = (scores1[i] - mean1) / sd1;
        }
        double min1 = (double) v1.min();
        
        int size2 = scores2.length;
        Vector v2 = new Vector(size2);
        for (int i = 0; i < v2.getSize(); i++) {
            v2.setElement(i, scores2[i]);
        }
        double mean2 = v2.mean();
        double sd2 = Math.sqrt(v2.var(false, false));
        for (int i = 0; i < v2.getSize(); i++) {
            scores2[i] = (scores2[i] - mean2) / sd2;
        }
        double min2 = (double) v2.min();        
        double min = 0;
        if (min1 <= min2) {
            min = min1;
        } else {
            min = min2;
        }

        for (int i = 0; i < scores1.length; i++) {
            if (Double.isNaN(scores1[i]) || Double.isInfinite(scores1[i])) {
                scores1[i] = 0.000001;
            } else {
                scores1[i] = scores1[i] + Math.abs(min);
            }           
        }

        for (int i = 0; i < scores2.length; i++) {
            if (Double.isNaN(scores2[i]) || Double.isInfinite(scores2[i])) {
                scores2[i] = 0.000001;
            } else {
                scores2[i] = scores2[i] + Math.abs(min);
            }           
        }
    }     

    public static void normalizeByMeanSdCol(ArrayList scores) {
        Object firstElement = ((ArrayList) scores.get(0)).get(0);
        double flag = 0;
        if (firstElement.getClass().getSimpleName().equalsIgnoreCase("Double")) {
            flag=1;
        }

        int sizeCol = ((ArrayList)scores.get(0)).size();
        Vector mins = new Vector(sizeCol);
        for (int i = 0; i < sizeCol; i++) {
            int sizeRow = scores.size();
            Vector v = new Vector(sizeRow);
            for(int j = 0; j < v.getSize(); j++) {
                ArrayList al = (ArrayList)scores.get(j);
                double score = 0;
                if (flag == 0) {
                    score = ((Float) (al).get(i)).doubleValue();
                } else {
                    score = ((Double) (al).get(i)).doubleValue();
                }
                v.setElement(j, score);
            }
            double mean = v.mean();
            double sd = Math.sqrt(v.var(false, false));
            for (int j = 0; j < v.getSize(); j++) {
                ArrayList al = (ArrayList)scores.get(j);
                double score = 0;
                if (flag == 0) {
                    score = ((Float) (al).get(i)).doubleValue();
                } else {
                    score = ((Double) (al).get(i)).doubleValue();
                }
                al.set(i, new Double((score - mean) / sd));
                v.setElement(j, new Double((score - mean) / sd));
            }
            double min = (double)v.min();
            mins.setElement(i, min);
        }

        double min = mins.min();
        for (int i = 0; i < scores.size(); i++) {
            ArrayList al = (ArrayList) scores.get(i);
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                if (Double.isNaN(d) || Double.isInfinite(d)) {
                    d = 0.000001;
                } else {
                    d = d+Math.abs(min);
                }
                if (flag == 0) {
                    al.set(j, new Float(d));
                } else {
                    al.set(j, new Double(d));
                }
            }
        }
    }
    
    public static void normalizeByTotalCol(ArrayList scores) {
        Object firstElement = ((ArrayList) scores.get(0)).get(0);
        double flag = 0;
        if (firstElement.getClass().getSimpleName().equalsIgnoreCase("Double")) {
            flag=1;
        }

        int sizeCol = ((ArrayList)scores.get(0)).size();
        for (int i = 0; i < sizeCol; i++) {
            int sizeRow = scores.size();
            Vector v = new Vector(sizeRow);
            for(int j = 0; j < v.getSize(); j++) {
                ArrayList al = (ArrayList)scores.get(j);
                double score = 0;
                if (flag == 0) {
                    score = ((Float) (al).get(i)).doubleValue();
                } else {
                    score = ((Double) (al).get(i)).doubleValue();
                }
                v.setElement(j, score);
            }
            double sum = v.sum();
            for (int j = 0; j < v.getSize(); j++) {
                ArrayList al = (ArrayList)scores.get(j);
                double score = 0;
                if (flag == 0) {
                    score = ((Float) (al).get(i)).doubleValue();
                } else {
                    score = ((Double) (al).get(i)).doubleValue();
                }
                al.set(i, new Double(score/ sum));
            }
        }
    }    
    
    public static void normalizeByTotal(double[] scores) {
        double sum = sum(scores);
        for(int i=0; i<scores.length; i++){
            scores[i]=scores[i]/sum;
        }
    }    

    public static ArrayList combineBySumOfScores(ArrayList scores1, ArrayList scores2) {
        ArrayList combinedScores = new ArrayList();
        for (int i = 0; i < scores1.size(); i++) {
            ArrayList al = new ArrayList();
            combinedScores.add(al);
        }

        for (int i = 0; i < scores1.size(); i++) {
            ArrayList combinedScoresRow = ((ArrayList) combinedScores.get(i));
            for (int j = 0; j < ((ArrayList) scores1.get(i)).size(); j++) {
                double dist_exp = ((Double) ((ArrayList) scores1.get(i)).get(j)).doubleValue();
                double dist_snp = ((Double) ((ArrayList) scores2.get(i)).get(j)).doubleValue();
                double dist = dist_exp + dist_snp;
                combinedScoresRow.add(new Double(dist));
            }
        }
        return combinedScores;
    }  
    
    public static ArrayList combineByMeanOfScores(ArrayList scores1, ArrayList scores2) {
        ArrayList combinedScores = new ArrayList();
        for (int i = 0; i < scores1.size(); i++) {
            ArrayList al = new ArrayList();
            combinedScores.add(al);
        }

        for (int i = 0; i < scores1.size(); i++) {
            ArrayList combinedScoresRow = ((ArrayList) combinedScores.get(i));
            for (int j = 0; j < ((ArrayList) scores1.get(i)).size(); j++) {
                double dist_exp = ((Double) ((ArrayList) scores1.get(i)).get(j)).doubleValue();
                double dist_snp = ((Double) ((ArrayList) scores2.get(i)).get(j)).doubleValue();
                double dist = (dist_exp + dist_snp)/(double)2;
                combinedScoresRow.add(new Double(dist));
            }
        }
        return combinedScores;
    }     
    
    public static double[] combineBySumOfScores(double[] scores1, double[] scores2) {
        double[] combinedScores = new double[scores1.length];

        for (int i = 0; i < scores1.length; i++) {
            double dist_exp = scores1[i];
            double dist_snp = scores2[i];;
            combinedScores[i] = dist_exp + dist_snp;
        }
        return combinedScores;
    }  
    
    public static double[] combineByMeanOfScores(double[] scores1, double[] scores2) {
        double[] combinedScores = new double[scores1.length];

        for (int i = 0; i < scores1.length; i++) {
            double dist_exp = scores1[i];
            double dist_snp = scores2[i];;
            combinedScores[i] = (dist_exp + dist_snp)/(double)2;
        }
        return combinedScores;
    }     
    
    public static double fisherTest(ArrayList pvalues) {
        double x2 = 0;
        for (int i = 0; i < pvalues.size(); i++) {
            x2 += Math.log(((Double) pvalues.get(i)).doubleValue());
        }
        x2 = (-2) * x2;
        return x2;
    }    
}
