/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */
package edu.duke.igsp.app.tools;

import edu.mit.broad.genome.math.Vector;
import org.apache.commons.math.stat.ranking.NaturalRanking;
import org.apache.commons.math.stat.ranking.NaNStrategy;
import org.apache.commons.math.stat.ranking.TiesStrategy;
import java.util.*;
import java.text.NumberFormat;
import java.text.DecimalFormat;
import java.math.RoundingMode;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: October, 2010
 *
 */
public class GsaaTool {
    public static HashMap calcPValue(HashMap scores) {
        HashMap pvalue_scores = new HashMap();
        Set keys = scores.keySet();
        Iterator it = keys.iterator();
        while (it.hasNext()) {
            String key = (String) it.next();
            ArrayList pvalues = new ArrayList();
            for (int j = 0; j < ((ArrayList) scores.get(key)).size(); j++) {
                ArrayList allScores = (ArrayList) scores.get(key);
                float score = ((Float) allScores.get(j)).floatValue();
                double p = getPValue(score, allScores);
                pvalues.add(new Double(p));
            }
            pvalue_scores.put(key, pvalues);
        }
        return pvalue_scores;
    }
    
    public static ArrayList calcPValue(ArrayList scores) {
        ArrayList pvalue_scores = new ArrayList();
        for (int i = 0; i < scores.size(); i++) {
            ArrayList pvalues = new ArrayList();
            for (int j = 0; j < ((ArrayList) scores.get(i)).size(); j++) {
                ArrayList allScores = (ArrayList) scores.get(i);
                double score = ((Double) allScores.get(j)).doubleValue();
                double p = getPValue(score, allScores);
                pvalues.add(new Double(p));
            }
            pvalue_scores.add(pvalues);
        }
        return pvalue_scores;
    }  
    
    public static HashMap calcPValueLessThan(HashMap scores) {
        HashMap pvalue_scores = new HashMap();
        Set keys = scores.keySet();
        Iterator it = keys.iterator();
        while (it.hasNext()) {
            String key = (String) it.next();
            ArrayList pvalues = new ArrayList();
            for (int j = 0; j < ((ArrayList) scores.get(key)).size(); j++) {
                ArrayList allScores = (ArrayList) scores.get(key);
                float score = ((Float) allScores.get(j)).floatValue();
                double p = getPValueLessThan(score, allScores);
                pvalues.add(new Double(p));
            }
            pvalue_scores.put(key, pvalues);
        }
        return pvalue_scores;
    }
    
    public static ArrayList calcPValueLessThan(ArrayList scores) {
        ArrayList pvalue_scores = new ArrayList();
        for (int i = 0; i < scores.size(); i++) {
            ArrayList pvalues = new ArrayList();
            for (int j = 0; j < ((ArrayList) scores.get(i)).size(); j++) {
                ArrayList allScores = (ArrayList) scores.get(i);
                double score = ((Double) allScores.get(j)).doubleValue();
                double p = getPValueLessThan(score, allScores);
                pvalues.add(new Double(p));
            }
            pvalue_scores.add(pvalues);
        }
        return pvalue_scores;
    }    

    public static double getPValue(final float score, final ArrayList values) {

        int cntmore = 0;

        for (int i = 0; i < values.size(); i++) {
            if (((Float) values.get(i)).floatValue() >= score) {
                cntmore++;
            }
        }

        return ((double) cntmore) / (double) values.size();
    }

    public static double getPValue(final double score, final ArrayList values) {

        int cntmore = 0;

        for (int i = 0; i < values.size(); i++) {
            if (((Double) values.get(i)).doubleValue() >= score) {
                cntmore++;
            }
        }

        return ((double) cntmore) / (double) values.size();
    }
    
    public static float getPValue(final float score, final Vector values) {

        int cntmore = 0;

        for (int i = 0; i < values.getSize(); i++) {
            if (((Float) values.getElement(i)).floatValue() >= score) {
                cntmore++;
            }
        }

        return ((float) cntmore) / (float) values.getSize();
    }    

    public static double getPValueLessThan(final float score, final ArrayList values) {

        int cntless = 0;

        for (int i = 0; i < values.size(); i++) {
            if (((Float) values.get(i)).floatValue() <= score) {
                cntless++;
            }
        }

        return ((double) cntless) / (double) values.size();
    }

    public static double getPValueLessThan(final double score, final ArrayList values) {

        int cntless = 0;

        for (int i = 0; i < values.size(); i++) {
            if (((Double) values.get(i)).doubleValue() <= score) {
                cntless++;
            }
        }

        return ((double) cntless) / (double) values.size();
    }
    
    public static float getPValueLessThan(final float score, final Vector values) {

        int cntless = 0;

        for (int i = 0; i < values.getSize(); i++) {
            if (((Float) values.getElement(i)).floatValue() <= score) {
                cntless++;
            }
        }

        return ((float) cntless) / (float) values.getSize();
    }   
    
    public static ArrayList calcPTwoTailed_fm(ArrayList scores) {
        ArrayList pvalue_scores = new ArrayList();
        for (int i = 0; i < scores.size(); i++) {
            ArrayList pvalues = new ArrayList();
            for (int j = 0; j < ((ArrayList) scores.get(i)).size(); j++) {
                ArrayList allScores = (ArrayList) scores.get(i);
                double score = ((Double) allScores.get(j)).doubleValue();
                double p = getPValueTwoTailed(score, allScores);
                if (score >= 0) {
                    pvalues.add(new Double(p));
                } else {
                    pvalues.add(new Double(p * (-1)));
                }
            }
            pvalue_scores.add(pvalues);
        }
        return pvalue_scores;
    }


    public static double getPValueTwoTailed(final double score, final ArrayList values) {
        int cntmore = 0;
        int size = values.size();

        for (int i = 0; i < size; i++) {
            double value = ((Double) values.get(i)).doubleValue();
            if (Math.abs(value) >= Math.abs(score)) {
                cntmore++;
            }
        }

        return ((double) cntmore) / (double) size;
    }    

    public static int isHomo(ArrayList al) {
        int count = 0;
        float first = ((Float) al.get(0)).floatValue();
        count++;
        for (int i = 1; i < al.size(); i++) {
            float score = ((Float) al.get(i)).floatValue();
            if (first == score) {
                count++;
            }
        }
        if (count == al.size()) {
            return 1;
        }
        return 0;
    }

    public static ArrayList toRanks(ArrayList scores) {
        ArrayList rank_scores = new ArrayList();
        for (int i = 0; i < scores.size(); i++) {
            ArrayList ranks = new ArrayList();
            ArrayList allScoresOri = (ArrayList) scores.get(i);
            double[] allScores = new double[allScoresOri.size()];
            for (int j = 0; j < allScoresOri.size(); j++) {
                allScores[j] = (((Double) allScoresOri.get(j)).doubleValue());
            }
            NaturalRanking ranking = new NaturalRanking(NaNStrategy.MINIMAL, TiesStrategy.AVERAGE);
            double[] ranks_d = ranking.rank(allScores);
            for (int j = 0; j < allScoresOri.size(); j++) {
                ranks.add(new Double(ranks_d[j]));
            }
            rank_scores.add(ranks);
        }
        return rank_scores;
    }

    public static HashMap toRanks(HashMap scores) {
        HashMap rank_scores = new HashMap();
        Set keys = scores.keySet();
        Iterator it = keys.iterator();
        while (it.hasNext()) {
            ArrayList ranks = new ArrayList();
            String key = (String) it.next();
            ArrayList allScoresOri = (ArrayList) scores.get(key);
            double[] allScores = new double[allScoresOri.size()];
            for (int j = 0; j < allScoresOri.size(); j++) {
                allScores[j] = ((double) ((Float) allScoresOri.get(j)).floatValue());
            }
            NaturalRanking ranking = new NaturalRanking(NaNStrategy.MINIMAL, TiesStrategy.AVERAGE);
            double[] ranks_d = ranking.rank(allScores);
            for (int j = 0; j < allScoresOri.size(); j++) {
                ranks.add(new Double(ranks_d[j]));
            }
            rank_scores.put(key, ranks);
        }
        return rank_scores;
    }

    public static ArrayList combineByFisherMethod(String[] rowNames, ArrayList exp, HashMap snp) {
        ArrayList combinedScores = new ArrayList();
        for (int i = 0; i < exp.size(); i++) {
            ArrayList al = new ArrayList();
            combinedScores.add(al);
        }
        for (int i = 0; i < exp.size(); i++) {
            for (int j = 0; j < ((ArrayList) exp.get(i)).size(); j++) {
                ArrayList pvalues = new ArrayList();
                double dist_exp_ori = ((Double) ((ArrayList) exp.get(i)).get(j)).doubleValue();
                double dist_exp = Math.abs(dist_exp_ori);
                pvalues.add(new Double(dist_exp));
                double dist_snp = 1;
                ArrayList al_snp = (ArrayList) snp.get(rowNames[i]);
                if (al_snp != null) {
                    dist_snp = ((Double) al_snp.get(j)).doubleValue();
                }
                pvalues.add(new Double(dist_snp));
                double x2 = fisherTest(pvalues);
                if (dist_exp_ori < 0) {
                    x2 = (-1) * x2;
                }
                ((ArrayList) combinedScores.get(i)).add(new Double(x2));
            }
        }
        return combinedScores;
    }

    public static ArrayList combineBySumOfScores(String[] rowNames, ArrayList exp, HashMap snp) {
        ArrayList combinedScores = new ArrayList();
        for (int i = 0; i < exp.size(); i++) {
            ArrayList al = new ArrayList();
            combinedScores.add(al);
        }

        for (int i = 0; i < exp.size(); i++) {
            ArrayList al_snp = (ArrayList) snp.get(rowNames[i]);
            ArrayList combinedScoresRow = ((ArrayList) combinedScores.get(i));
            for (int j = 0; j < ((ArrayList) exp.get(i)).size(); j++) {
                double dist_exp = ((Double) ((ArrayList) exp.get(i)).get(j)).doubleValue();
                double dist_snp = 0;
                if (al_snp != null) {
                    dist_snp = ((Double) al_snp.get(j)).doubleValue();
                }
                double dist = dist_exp + dist_snp;
                combinedScoresRow.add(new Double(dist));
            }
        }
        return combinedScores;
    }

    public static double fisherTest(ArrayList pvalues) {
        double x2 = 0;
        for (int i = 0; i < pvalues.size(); i++) {
            x2 += Math.log(((Double) pvalues.get(i)).doubleValue());
        }
        x2 = (-2) * x2;
        return x2;
    }

    public static void normalizeByMeanRow(ArrayList scores) {
        for (int i = 0; i < scores.size(); i++) {
            ArrayList al = (ArrayList) scores.get(i);
            Vector v = new Vector(al.size() - 1);
            for (int j = 0; j < v.getSize(); j++) {
                v.setElement(j, Math.abs(((Double) al.get(j + 1)).doubleValue()));
            }
            double mean = v.mean();
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                d = d / mean;
                if (Double.isNaN(d) || Double.isInfinite(d)) {
                    d = 0.000001;
                }
                al.set(j, new Double(d));
            }
        }
    }

    public static void normalizeByMeanSdRow(ArrayList scores) {
        Vector mins = new Vector(scores.size());
        for (int i = 0; i < scores.size(); i++) {
            ArrayList al = (ArrayList) scores.get(i);
            Vector v = new Vector(al.size() - 1);
            for (int j = 0; j < v.getSize(); j++) {
                v.setElement(j, ((Double) al.get(j + 1)).doubleValue());
            }
            double mean = v.mean();
            double sd = Math.sqrt(v.var(false, false));
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                al.set(j, new Double((d - mean) / sd));
            }
            double min = ((Double) Collections.min(al)).doubleValue();
            mins.setElement(i, min);
        }
        double min = (double) mins.min();
        for (int i = 0; i < scores.size(); i++) {
            ArrayList al = (ArrayList) scores.get(i);
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                if (Double.isNaN(d) || Double.isInfinite(d)) {
                    d = 0.000001;
                } else {
                    d = d+Math.abs(min);
                }
                al.set(j, new Double(d));
            }
        }
    }

    public static void normalizeByMeanSdRow(ArrayList scores1, ArrayList scores2) {
        Vector mins1 = new Vector(scores1.size());
        Vector mins2 = new Vector(scores2.size());
        for (int i = 0; i < scores1.size(); i++) {
            ArrayList al = (ArrayList) scores1.get(i);
            Vector v = new Vector(al.size() - 1);
            for (int j = 0; j < v.getSize(); j++) {
                v.setElement(j, ((Double) al.get(j + 1)).doubleValue());
            }
            double mean = v.mean();
            double sd = Math.sqrt(v.var(false, false));
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                al.set(j, new Double((d - mean) / sd));
            }
            double min = ((Double) Collections.min(al)).doubleValue();
            mins1.setElement(i, min);
        }

        for (int i = 0; i < scores2.size(); i++) {
            ArrayList al = (ArrayList) scores2.get(i);
            Vector v = new Vector(al.size() - 1);
            for (int j = 0; j < v.getSize(); j++) {
                v.setElement(j, ((Double) al.get(j + 1)).doubleValue());
            }
            double mean = v.mean();
            double sd = Math.sqrt(v.var(false, false));
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                al.set(j, new Double((d - mean) / sd));
            }
            double min = ((Double) Collections.min(al)).doubleValue();
            mins2.setElement(i, min);
        }

        double min1 = mins1.min();
        double min2 = mins2.min();
        double min = 0;
        if (min1 <= min2) {
            min = min1;
        } else {
            min = min2;
        }

        for (int i = 0; i < scores1.size(); i++) {
            ArrayList al = (ArrayList) scores1.get(i);
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                if (Double.isNaN(d) || Double.isInfinite(d)) {
                    d = 0.000001;
                } else {
                    d = d+Math.abs(min);
                }
                al.set(j, new Double(d));
            }
        }

        for (int i = 0; i < scores2.size(); i++) {
            ArrayList al = (ArrayList) scores2.get(i);
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                if (Double.isNaN(d) || Double.isInfinite(d)) {
                    d = 0.000001;
                } else {
                    d = d+Math.abs(min);
                }
                al.set(j, new Double(d));
            }
        }
    }

    public static void normalizeByMeanSdRow(ArrayList scores1, HashMap scores2) {
        Vector mins1 = new Vector(scores1.size());
        Vector mins2 = new Vector(scores2.size());
        for (int i = 0; i < scores1.size(); i++) {
            ArrayList al = (ArrayList) scores1.get(i);
            Vector v = new Vector(al.size() - 1);
            for (int j = 0; j < v.getSize(); j++) {
                v.setElement(j, ((Double) al.get(j + 1)).doubleValue());
            }
            double mean = v.mean();
            double sd = Math.sqrt(v.var(false, false));
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                al.set(j, new Double((d - mean) / sd));
            }
            double min = ((Double) Collections.min(al)).doubleValue();
            mins1.setElement(i, min);
        }
        
        Set keys = scores2.keySet();
        Iterator it = keys.iterator();        
        int cnt = 0;
        while (it.hasNext()) {
            String key = (String) it.next();
            ArrayList al = (ArrayList) scores2.get(key);
            Vector v = new Vector(al.size() - 1);
            for (int j = 0; j < v.getSize(); j++) {
                v.setElement(j, (double) (((Float) al.get(j + 1)).floatValue()));
            }
            double mean = v.mean();
            double sd = Math.sqrt(v.var(false, false));
            for (int j = 0; j < al.size(); j++) {
                double d = (double) ((Float) al.get(j)).floatValue();
                al.set(j, new Double((d - mean) / sd));
            }
            double min = ((Double) Collections.min(al)).doubleValue();
            mins2.setElement(cnt, min);
            cnt++;
        }

        double min1 = mins1.min();
        double min2 = mins2.min();
        double min = 0;
        if (min1 <= min2) {
            min = min1;
        } else {
            min = min2;
        }

        for (int i = 0; i < scores1.size(); i++) {
            ArrayList al = (ArrayList) scores1.get(i);
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                if (Double.isNaN(d) || Double.isInfinite(d)) {
                    d = 0.000001;
                } else {
                    d = d+Math.abs(min);
                }
                al.set(j, new Double(d));
            }
        }

        it = keys.iterator();
        while (it.hasNext()) {
            String key = (String) it.next();
            ArrayList al = (ArrayList) scores2.get(key);
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                if (Double.isNaN(d) || Double.isInfinite(d)) {
                    d = 0.000001;
                } else {
                    d = d+Math.abs(min);
                }
                al.set(j, new Double(d));
            }
        }
    }

    public static void normalizeByMeanSdCol(ArrayList scores1, HashMap scores2) {
        Object firstElement1 = ((ArrayList) scores1.get(0)).get(0);
        double flag1 = 0;
        if (firstElement1.getClass().getSimpleName().equalsIgnoreCase("Double")) {
            flag1=1;
        }

        Set keys = scores2.keySet();
        Object firstElement2 = ((ArrayList) scores2.get((String)keys.iterator().next())).get(0);
        double flag2 = 0;
        if (firstElement2.getClass().getSimpleName().equalsIgnoreCase("Double")) {
            flag2=1;
        }
        
        int sizeCol1 = ((ArrayList)scores1.get(0)).size();
        Vector mins1 = new Vector(sizeCol1);
        for (int i = 0; i < sizeCol1; i++) {
            int sizeRow = scores1.size();
            Vector v = new Vector(sizeRow);
            for(int j = 0; j < v.getSize(); j++) {
                ArrayList al = (ArrayList)scores1.get(j);
                double score = 0;
                if (flag1 == 0) {
                    score = (double)((Float) (al).get(i)).floatValue();
                } else {
                    score = ((Double) (al).get(i)).doubleValue();
                }
                v.setElement(j, score);
            }
            double mean = v.mean();
            double sd = Math.sqrt(v.var(false, false));
            for (int j = 0; j < v.getSize(); j++) {
                ArrayList al = (ArrayList)scores1.get(j);
                double score = 0;
                if (flag1 == 0) {
                    score = (double)((Float) (al).get(i)).floatValue();
                } else {
                    score = ((Double) (al).get(i)).doubleValue();
                }
                al.set(i, new Double((score - mean) / sd));
                v.setElement(j, new Double((score - mean) / sd));
            }
            double min = (double)v.min();
            mins1.setElement(i, min);
        }
        

        Iterator it = keys.iterator();
        int sizeCol2 = ((ArrayList) scores2.get((String)keys.iterator().next())).size();
        Vector mins2 = new Vector(sizeCol2);
        for (int i = 0; i < sizeCol2; i++) {
            it = keys.iterator();
            int cnt = 0;
            Vector v = new Vector(scores2.size());
            while (it.hasNext()) {
                String key = (String) it.next();
                ArrayList al = (ArrayList) scores2.get(key);
                double score = 0;
                if (flag2==0) {
                    score = (double) ((Float) al.get(i)).floatValue();
                } else {
                    score = ((Double) al.get(i)).doubleValue();
                }
                v.setElement(cnt, score);
                cnt++;
            }
            double mean = v.mean();
            double sd = Math.sqrt(v.var(false, false));
            it = keys.iterator();
            cnt=0;
            while (it.hasNext()) {
                String key = (String) it.next();
                ArrayList al = (ArrayList) scores2.get(key);
                double score = 0;
                if (flag2==0) {
                    score = (double) ((Float) al.get(i)).floatValue();
                } else {
                    score = ((Double) al.get(i)).doubleValue();
                }
                al.set(i, new Double((score - mean) / sd));
                v.setElement(cnt, new Double((score - mean) / sd));
                cnt++;
            }
            double min = (double)v.min();
            mins2.setElement(i, min);
        }

        double min1 = mins1.min();
        double min2 = mins2.min();
        double min = 0;
        if (min1 <= min2) {
            min = min1;
        } else {
            min = min2;
        }

        for (int i = 0; i < scores1.size(); i++) {
            ArrayList al = (ArrayList) scores1.get(i);
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                if (Double.isNaN(d) || Double.isInfinite(d)) {
                    d = 0.000001;
                } else {
                    d = d+Math.abs(min);
                }
                if (flag1 == 0) {
                    al.set(j, new Float(d));
                } else {
                    al.set(j, new Double(d));
                }
            }
        }

        it = keys.iterator();
        while (it.hasNext()) {
            String key = (String) it.next();
            ArrayList al = (ArrayList) scores2.get(key);
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                if (Double.isNaN(d) || Double.isInfinite(d)) {
                    d = 0.000001;
                } else {
                    d = d+Math.abs(min);
                }
                if (flag2 == 0) {
                    al.set(j, new Float(d));
                } else {
                    al.set(j, new Double(d));
                }
            }
        }
    }

    public static void normalizeByMeanSdCol(ArrayList scores1, ArrayList scores2) {
        Object firstElement1 = ((ArrayList) scores1.get(0)).get(0);
        double flag1 = 0;
        if (firstElement1.getClass().getSimpleName().equalsIgnoreCase("Double")) {
            flag1=1;
        }
        Object firstElement2 = ((ArrayList) scores2.get(0)).get(0);
        double flag2 = 0;
        if (firstElement2.getClass().getSimpleName().equalsIgnoreCase("Double")) {
            flag2=1;
        }

        int sizeCol1 = ((ArrayList)scores1.get(0)).size();
        Vector mins1 = new Vector(sizeCol1);
        for (int i = 0; i < sizeCol1; i++) {
            int sizeRow = scores1.size();
            Vector v = new Vector(sizeRow);
            for(int j = 0; j < v.getSize(); j++) {
                ArrayList al = (ArrayList)scores1.get(j);
                double score = 0;
                if (flag1 == 0) {
                    score = (double)((Float) (al).get(i)).floatValue();
                } else {
                    score = ((Double) (al).get(i)).doubleValue();
                }
                v.setElement(j, score);
            }
            double mean = v.mean();
            double sd = Math.sqrt(v.var(false, false));
            for (int j = 0; j < v.getSize(); j++) {
                ArrayList al = (ArrayList)scores1.get(j);
                double score = 0;
                if (flag1 == 0) {
                    score = (double)((Float) (al).get(i)).floatValue();
                } else {
                    score = ((Double) (al).get(i)).doubleValue();
                }
                al.set(i, new Double((score - mean) / sd));
                v.setElement(j, new Double((score - mean) / sd));
            }
            double min = (double)v.min();
            mins1.setElement(i, min);
        }

        int sizeCol2 = ((ArrayList)scores2.get(0)).size();
        Vector mins2 = new Vector(sizeCol2);
        for (int i = 0; i < sizeCol2; i++) {
            int sizeRow = scores2.size();
            Vector v = new Vector(sizeRow);
            for(int j = 0; j < v.getSize(); j++) {
                ArrayList al = (ArrayList)scores2.get(j);
                double score = 0;
                if (flag2 == 0) {
                    score = (double)((Float) (al).get(i)).floatValue();
                } else {
                    score = ((Double) (al).get(i)).doubleValue();
                }
                v.setElement(j, score);
            }
            double mean = v.mean();
            double sd = Math.sqrt(v.var(false, false));
            for (int j = 0; j < v.getSize(); j++) {
                ArrayList al = (ArrayList)scores2.get(j);
                double score = 0;
                if (flag2 == 0) {
                    score = (double)((Float) (al).get(i)).floatValue();
                } else {
                    score = ((Double) (al).get(i)).doubleValue();
                }
                al.set(i, new Double((score - mean) / sd));
                v.setElement(j, new Double((score - mean) / sd));
            }
            double min = (double)v.min();
            mins2.setElement(i, min);
        }

        double min1 = mins1.min();
        double min2 = mins2.min();
        double min = 0;
        if (min1 <= min2) {
            min = min1;
        } else {
            min = min2;
        }

        for (int i = 0; i < scores1.size(); i++) {
            ArrayList al = (ArrayList) scores1.get(i);
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                if (Double.isNaN(d) || Double.isInfinite(d)) {
                    d = 0.000001;
                } else {
                    d = d+Math.abs(min);
                }
                if (flag1 == 0) {
                    al.set(j, new Float(d));
                } else {
                    al.set(j, new Double(d));
                }                
            }
        }

        for (int i = 0; i < scores2.size(); i++) {
            ArrayList al = (ArrayList) scores2.get(i);
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                if (Double.isNaN(d) || Double.isInfinite(d)) {
                    d = 0.000001;
                } else {
                    d = d+Math.abs(min);
                }
                if (flag2 == 0) {
                    al.set(j, new Float(d));
                } else {
                    al.set(j, new Double(d));
                }
            }
        }
    }

    public static void normalizeByMeanSdCol(ArrayList scores) {
        Object firstElement = ((ArrayList) scores.get(0)).get(0);
        double flag = 0;
        if (firstElement.getClass().getSimpleName().equalsIgnoreCase("Double")) {
            flag=1;
        }

        int sizeCol1 = ((ArrayList)scores.get(0)).size();
        Vector mins = new Vector(sizeCol1);
        for (int i = 0; i < sizeCol1; i++) {
            int sizeRow = scores.size();
            Vector v = new Vector(sizeRow);
            for(int j = 0; j < v.getSize(); j++) {
                ArrayList al = (ArrayList)scores.get(j);
                double score = 0;
                if (flag == 0) {
                    score = (double)((Float) (al).get(i)).floatValue();
                } else {
                    score = ((Double) (al).get(i)).doubleValue();
                }
                v.setElement(j, score);
            }
            double mean = v.mean();
            double sd = Math.sqrt(v.var(false, false));
            for (int j = 0; j < v.getSize(); j++) {
                ArrayList al = (ArrayList)scores.get(j);
                double score = 0;
                if (flag == 0) {
                    score = (double)((Float) (al).get(i)).floatValue();
                } else {
                    score = ((Double) (al).get(i)).doubleValue();
                }
                al.set(i, new Double((score - mean) / sd));
                v.setElement(j, new Double((score - mean) / sd));
            }
            double min = (double)v.min();
            mins.setElement(i, min);
        }

        double min = mins.min();
        for (int i = 0; i < scores.size(); i++) {
            ArrayList al = (ArrayList) scores.get(i);
            for (int j = 0; j < al.size(); j++) {
                double d = ((Double) al.get(j)).doubleValue();
                if (Double.isNaN(d) || Double.isInfinite(d)) {
                    d = 0.000001;
                } else {
                    d = d+Math.abs(min);
                }
                if (flag == 0) {
                    al.set(j, new Float(d));
                } else {
                    al.set(j, new Double(d));
                }
            }
        }
    }

    public static void homoRemove(ArrayList scores) {
        String objClass = ((ArrayList) scores.get(0)).get(0).getClass().getSimpleName();
        for (int i = 0; i < scores.size(); i++) {
            int flag = 0;
            double first_score = 0;
            if (objClass.equalsIgnoreCase("Float")) {
                first_score = (double) ((Float) ((ArrayList) scores.get(i)).get(0)).floatValue();
            } else {
                first_score = ((Double) ((ArrayList) scores.get(i)).get(0)).doubleValue();
            }
            for (int j = 1; j < ((ArrayList) scores.get(i)).size(); j++) {
                double score = 0;
                if (objClass.equalsIgnoreCase("Float")) {
                    score = (double) ((Float) ((ArrayList) scores.get(i)).get(j)).floatValue();
                } else {
                    score = ((Double) ((ArrayList) scores.get(i)).get(j)).doubleValue();
                }
                if (first_score != score) {
                    flag = 1;
                    break;
                }
            }
            if (flag == 0) {
                scores.remove(i);
                i--;
            }
        }
    }

    public static void homoRemove(HashMap scores) {
        Set keys = scores.keySet();
        Iterator it = keys.iterator();
        ArrayList keys_remove = new ArrayList();
        while (it.hasNext()) {
            String key = (String) it.next();
            ArrayList al = (ArrayList) scores.get(key);
            int flag = 0;
            double first_score = 0;
            String objClass = al.get(0).getClass().getSimpleName();
            if (objClass.equalsIgnoreCase("Float")) {
                first_score = (double) ((Float) al.get(0)).floatValue();
            } else {
                first_score = ((Double) al.get(0)).doubleValue();
            }
            for (int i = 1; i < al.size(); i++) {
                double score = 0;
                if (objClass.equalsIgnoreCase("Float")) {
                    first_score = (double) ((Float) al.get(i)).floatValue();
                } else {
                    first_score = ((Double) al.get(i)).doubleValue();
                }
                if (first_score != score) {
                    flag = 1;
                    break;
                }
            }
            if (flag == 0) {
                keys_remove.add(key);
            }
        }
        for (int i = 0; i < keys_remove.size(); i++) {
            String key = (String) keys_remove.get(i);
            scores.remove(key);
        }
    }

    public static double doubleFormat(double value, int decimalPlaces) {
        NumberFormat nf = NumberFormat.getNumberInstance();
        nf.setRoundingMode(RoundingMode.HALF_UP);
        nf.setMaximumFractionDigits(decimalPlaces);
        nf.setMinimumFractionDigits(decimalPlaces);
        return Double.valueOf(nf.format(value));
    }

    public static double doubleFormat(double value) {
        DecimalFormat formatter = new DecimalFormat("0.0000");
        double d = Double.valueOf(formatter.format(value));
        return d;
    }
}

