/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */
package edu.duke.igsp.app.tools;

import java.io.*;
import java.text.*;
import java.util.*;

/**
 * @author Qing Xiong < qing.xiong@duke.edu >
 * Date: April, 2009
 *
 */
public class MatrixTool {

    public MatrixTool() {
    }
    private static DecimalFormat fmt = new DecimalFormat("0.000000");

    //rowIndex & colIndex start from 0;
    public static double[][] delRowCol(double[][] matrix, int rowIndex,
            int colIndex) {
        int initsize_row = matrix.length;
        int initsize_col = matrix[0].length;
        double[][] resultMatrix = new double[initsize_row - 1][initsize_col - 1];
        double temp;
        for (int i = 0; i < initsize_row; i++) {
            for (int j = 0; j < initsize_col; j++) {
                if (i < rowIndex && j < colIndex) {
                    temp = getValue(matrix, i, j);
                    setValue(resultMatrix, i, j, temp);
                }
                if (i > rowIndex && j < colIndex) {
                    temp = getValue(matrix, i, j);
                    setValue(resultMatrix, i - 1, j, temp);
                }
                if (i < rowIndex && j > colIndex) {
                    temp = getValue(matrix, i, j);
                    setValue(resultMatrix, i, j - 1, temp);
                }
                if (i > rowIndex && j > colIndex) {
                    temp = getValue(matrix, i, j);
                    setValue(resultMatrix, i - 1, j - 1, temp);
                }
            }
        }
        return resultMatrix;
    }

    //rowIndex start from 0;
    public static double[][] delRow(double[][] matrix, int rowIndex) {
        int initsize_row = matrix.length;
        int initsize_col = matrix[0].length;
        double[][] resultMatrix = new double[initsize_row - 1][initsize_col];
        double temp = 0;
        for (int i = 0; i < initsize_row; i++) {
            for (int j = 0; j < initsize_col; j++) {
                if (i < rowIndex) {
                    temp = getValue(matrix, i, j);
                    setValue(resultMatrix, i, j, temp);
                } else {
                    if (i > rowIndex) {
                        temp = getValue(matrix, i, j);
                    }
                    setValue(resultMatrix, i - 1, 0, temp);
                }
            }
        }
        return resultMatrix;
    }

    //rowIndex and colIndex start from 0.
    public static void setValue(double[][] matrix, int rowIndex, int colIndex,
            double value) {
        if (rowIndex < matrix.length && colIndex < matrix[0].length) {
            matrix[rowIndex][colIndex] = value;
        } else {
            System.out.println("Error: OutOfArrayBoundary in method setValue()");
        }
    }

    public static double getValue(double[][] matrix, int rowIndex, int colIndex) {
        if (rowIndex < matrix.length && colIndex < matrix[0].length) {
            return matrix[rowIndex][colIndex];
        } else {
            System.out.println("Error: OutOfArrayBoundary in method getValue()");
            return -999999;
        }
    }

    public static void setValue(ArrayList matrix, int rowIndex, int colIndex,
            double value) {
        if (rowIndex < matrix.size() &&
                colIndex < ((ArrayList) (matrix.get(0))).size()) {
            ((ArrayList) (matrix.get(rowIndex))).set(colIndex,
                    new Double(value));
        } else {
            System.out.println("Error: OutOfArrayBoundary in method setValue()");
        }
    }

    public static double getValue(ArrayList matrix, int rowIndex, int colIndex) {
        if (rowIndex < matrix.size() &&
                colIndex < ((ArrayList) (matrix.get(0))).size()) {
            return ((Double) (((ArrayList) (matrix.get(rowIndex))).get(
                    colIndex))).doubleValue();
        } else {
            System.out.println("Error: OutOfArrayBoundary in method getValue()");
            return -999999;
        }
    }

    public void delRowCol(ArrayList matrix, int rowIndex, int colIndex) {
        if (rowIndex < matrix.size() &&
                colIndex < ((ArrayList) (matrix.get(0))).size()) {
            matrix.remove(rowIndex);
            for (int i = 0; i < matrix.size(); i++) {
                ((ArrayList) (matrix.get(i))).remove(colIndex);
            }
        } else {
            System.out.println("Error: OutOfArrayBoundary in method delRowCol()");
        }
    }

    public static ArrayList getMatrix(String filename) {
        ArrayList result = new ArrayList();
        try {
            FileReader input = new FileReader(filename);
            BufferedReader br = new BufferedReader(input);
            String s = br.readLine();
            while (s != null) {
                StringTokenizer st = new StringTokenizer(s);
                ArrayList temp = new ArrayList();
                while (st.hasMoreTokens()) {
                    temp.add(st.nextToken());
                }
                result.add(temp);
                s = br.readLine();
            }
            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    public static void getMatrix(String[][] matrix, String filename) {
        try {
            FileReader input = new FileReader(filename);
            BufferedReader br = new BufferedReader(input);
            String s = br.readLine();
            for (int i = 0; i < matrix.length; i++) {
                for (int j = 0; j < matrix[0].length; j++) {
                    StringTokenizer st = new StringTokenizer(s);
                    matrix[i][j] = st.nextToken();
                }
                s = br.readLine();
            }
            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static String[][] createMatrixString(ArrayList matrix) {
        int row = matrix.size();
        int col = ((ArrayList) matrix.get(0)).size();
        String[][] temp = new String[row][col];
        for (int i = 0; i < row; i++) {
            for (int j = 0; j < col; j++) {
                temp[i][j] = (String) (((ArrayList) (matrix.get(i))).get(j));
            }
        }
        return temp;
    }

    public static double[][] createMatrixDouble(ArrayList matrix) {
        int row = matrix.size();
        int col = ((ArrayList) matrix.get(0)).size();
        double[][] temp = new double[row][col];
        for (int i = 0; i < row; i++) {
            for (int j = 0; j < col; j++) {
                temp[i][j] = ((Double) (((ArrayList) (matrix.get(i))).get(j))).doubleValue();
            }
        }
        return temp;
    }

    public static int[][] createMatrixInt(ArrayList matrix) {
        int row = matrix.size();
        int col = ((ArrayList) matrix.get(0)).size();
        int[][] temp = new int[row][col];
        for (int i = 0; i < row; i++) {
            for (int j = 0; j < col; j++) {
                temp[i][j] = ((Integer) (((ArrayList) (matrix.get(i))).get(j))).intValue();
            }
        }
        return temp;
    }

    public static ArrayList createMatrixString(String[][] matrixInArray) {
        ArrayList temp = new ArrayList();
        for (int i = 0; i < matrixInArray.length; i++) {
            ArrayList temp1 = new ArrayList();
            for (int j = 0; j < matrixInArray[0].length; j++) {
                temp1.add(matrixInArray[i][j]);
            }
            temp.add(temp1);
        }
        return temp;
    }

    public static ArrayList createMatrixDouble(double[][] matrixInArray) {
        ArrayList temp = new ArrayList();
        for (int i = 0; i < matrixInArray.length; i++) {
            ArrayList temp1 = new ArrayList();
            for (int j = 0; j < matrixInArray[0].length; j++) {
                temp1.add(new Double(matrixInArray[i][j]));
            }
            temp.add(temp1);
        }
        return temp;
    }

    public static ArrayList createMatrixInt(int[][] matrixInArray) {
        ArrayList temp = new ArrayList();
        for (int i = 0; i < matrixInArray.length; i++) {
            ArrayList temp1 = new ArrayList();
            for (int j = 0; j < matrixInArray[0].length; j++) {
                temp1.add(new Integer(matrixInArray[i][j]));
            }
            temp.add(temp1);
        }
        return temp;
    }

    public static void saveMatrix(ArrayList matrix, String filename) {
        try {
            FileWriter output = new FileWriter(filename);
            BufferedWriter bw = new BufferedWriter(output);
            for (int i = 0; i < matrix.size(); i++) {
                for (int j = 0; j < ((ArrayList) matrix.get(i)).size();
                        j++) {
                    String s = (String) ((ArrayList) matrix.get(i)).get(j);
                    bw.write(s);
                    bw.write("\t");
                }
                bw.newLine();
            }
            bw.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void saveMatrix(HashMap matrix, String filename) {
        try {
            FileWriter output = new FileWriter(filename);
            BufferedWriter bw = new BufferedWriter(output);
            Set keys = matrix.keySet();
            Iterator it = keys.iterator();
            while (it.hasNext()) {
                String key = (String) it.next();
                ArrayList al = (ArrayList) matrix.get(key);
                for (int i = 0; i < al.size(); i++) {
                    String s = (String) al.get(i);
                    bw.write(s);
                    bw.write("\t");
                }
                bw.newLine();
            }
            bw.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void saveMatrix(String[][] matrix, String filename) {
        try {
            FileWriter output = new FileWriter(filename);
            BufferedWriter bw = new BufferedWriter(output);
            for (int i = 0; i < matrix.length; i++) {
                for (int j = 0; j < matrix[i].length; j++) {
                    String s = matrix[i][j];
                    bw.write(s);
                    bw.write(" ");
                }
                bw.newLine();
            }
            bw.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void saveMatrix(double[][] matrix, String filename) {
        try {
            FileWriter output = new FileWriter(filename);
            BufferedWriter bw = new BufferedWriter(output);
            for (int i = 0; i < matrix.length; i++) {
                for (int j = 0; j < matrix[i].length; j++) {
                    String s = Double.toString(matrix[i][j]);
                    bw.write(s);
                    bw.write(" ");
                }
                bw.newLine();
            }
            bw.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void saveMatrix(int[][] matrix, String filename) {
        try {
            FileWriter output = new FileWriter(filename);
            BufferedWriter bw = new BufferedWriter(output);
            for (int i = 0; i < matrix.length; i++) {
                for (int j = 0; j < matrix[i].length; j++) {
                    String s = Integer.toString(matrix[i][j]);
                    bw.write(s);
                    bw.write(" ");
                }
                bw.newLine();
            }
            bw.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void printMatrix(ArrayList matrix) {
        for (int i = 0; i < matrix.size(); i++) {
            for (int j = 0; j < ((ArrayList) matrix.get(i)).size(); j++) {
                System.out.print((String) ((ArrayList) matrix.get(i)).get(j) + " ");
            }
            System.out.println();
        }
    }

    public static void printMatrix(String[][] matrix) {
        for (int i = 0; i < matrix.length; i++) {
            for (int j = 0; j < matrix[i].length; j++) {
                System.out.print(matrix[i][j] + " ");
                if (j == (matrix[0].length - 1)) {
                    System.out.println();
                }
            }
        }
    }

    public static void printMatrix(double[][] matrix) {
        for (int i = 0; i < matrix.length; i++) {
            for (int j = 0; j < matrix[i].length; j++) {
                System.out.print(matrix[i][j] + " ");
                if (j == (matrix[0].length - 1)) {
                    System.out.println();
                }
            }
        }
    }

    public static void printMatrix(int[][] matrix) {
        for (int i = 0; i < matrix.length; i++) {
            for (int j = 0; j < matrix[i].length; j++) {
                System.out.print(matrix[i][j] + " ");
                if (j == (matrix[0].length - 1)) {
                    System.out.println();
                }
            }
        }
    }

    public static void resetMatrix(String[][] matrix) {
        for (int i = 0; i < matrix.length; i++) {
            for (int j = 0; j < matrix[i].length; j++) {
                matrix[i][j] = "";
            }
        }
    }

    public static void resetMatrix(double[][] matrix) {
        for (int i = 0; i < matrix.length; i++) {
            for (int j = 0; j < matrix[i].length; j++) {
                matrix[i][j] = 0;
            }
        }
    }

    public static void resetMatrix(int[][] matrix) {
        for (int i = 0; i < matrix.length; i++) {
            for (int j = 0; j < matrix[i].length; j++) {
                matrix[i][j] = 0;
            }
        }
    }

    public static void searchByArray(String[] names, int columnFlag,
            String filename, String filenameForResult) {
        String[][] matrix = createMatrixString(getMatrix(filename));

        String[][] resultMatrix = new String[names.length][matrix[0].length];
        int count = 0;
        for (int i = 0; i < names.length; i++) {
            for (int j = 0; j < matrix.length; j++) {
                if (matrix[j][columnFlag].equalsIgnoreCase(names[i])) {
                    if (resultMatrix[i][columnFlag] != null) {
                        count--;
                    }
                    for (int k = 0; k < matrix[0].length; k++) {
                        if (matrix[j][k] != null) {
                            resultMatrix[i][k] = matrix[j][k];
                        } else {
                            resultMatrix[i][k] = "NaN";
                        }
                    }
                    count++;
                }
            }
        }

        if (count < resultMatrix.length) {
            for (int i = count; i < resultMatrix.length; i++) {
                for (int j = 0; j < resultMatrix[0].length; j++) {
                    resultMatrix[i][j] = "NaN";
                }
            }
            //printMatrix(resultMatrix);

        } else {

            saveMatrix(resultMatrix, filenameForResult);

        }

        /*int count=0;
        for(int i=0; i<resultMatrix.length; i++){
        for(int j=0; j<resultMatrix[0].length; j++){
        if(resultMatrix[i][j]==null){
        resultMatrix[i][j]="NaN";
        //count++;
        }
        }
        }
        //if (count == 0) {
        //printMatrix(resultMatrix);
        saveMatrix(resultMatrix, filenameForResult);
        //}*/

    }

    public static void searchByMatrix(String filenameForMatrix,
            String filenameForFile,
            int columnFlag,
            String pathForResult) {

        ArrayList array = new ArrayList();
        try {
            FileReader input = new FileReader(filenameForMatrix);
            BufferedReader br = new BufferedReader(input);
            String s = br.readLine().trim();
            int item = 1;
            while (s != null) {
                StringTokenizer st = new StringTokenizer(s);
                while (st.hasMoreTokens()) {
                    array.add(st.nextToken());
                }

                if (array.size() != 1 && array.size() != 0) {
                    //if (array.size()>1) {
                    String[] names = new String[array.size()];
                    for (int i = 0; i < names.length; i++) {
                        names[i] = (String) array.get(i);
                    }
                    searchByArray(names, columnFlag, filenameForFile,
                            pathForResult + item + ".txt");
                }
                s = br.readLine();
                array.clear();
                item++;
            }
            br.close();
        } catch (Exception e) {
            System.out.println("Error---" + e.toString());
        }
    }

    public static double[][] createTranspose(double[][] matrix) {
        int size_row = matrix.length;
        int size_col = matrix[0].length;
        double[][] resultMatrix = new double[size_col][size_row];
        double temp = 0;

        for (int i = 0; i < size_row; i++) {
            for (int j = 0; j < size_col; j++) {
                temp = MatrixTool.getValue(matrix, i, j);
                MatrixTool.setValue(resultMatrix, j, i, temp);
            }
        }

        return resultMatrix;
    }

    //matrix must be square.
    public static double determinant(double[][] matrix) {
        double[][] smaller;
        double temp1 = 0;
        double temp2 = 0;
        double temp3 = 0;
        double temp4 = 0;
        double sum = 0;
        double topofmatrix = 0;
        if (matrix.length == 2) {
            temp1 = matrix[0][0];
            temp2 = matrix[1][1];
            temp3 = matrix[0][1];
            temp4 = matrix[1][0];
            sum = temp1 * temp2 - temp3 * temp4;
            return sum;
        } else {
            for (int i = 0; i < matrix.length; i++) {
                temp1 = matrix[0][i];
                smaller = MatrixTool.delRowCol(matrix, 0, i);
                temp2 = determinant(smaller);
                if (i % 2 == 1) {
                    temp1 = -1 * temp1;
                }
                sum = sum + temp1 * temp2;
            }
            if (Math.abs(sum) < .000000001) {
                sum = 0;
            }
            return sum;
        }
    }

    //numberofcolumns in matrix1 must equal numberofrows in matrix2.
    public static double[][] multiplyMatrix(double[][] matrix1,
            double[][] matrix2) {
        double[][] resultMatrix = new double[matrix1.length][matrix2[0].length];
        if (matrix1[0].length != matrix2.length) {
            System.out.println("Error: the number of columns in matrix1 must equal the number of rows in matrix2");
            return resultMatrix;
        } else {
            for (int i = 0; i < matrix1.length; i++) {
                for (int j = 0; j < matrix2[0].length; j++) {
                    for (int k = 0; k < matrix1[0].length; k++) {
                        resultMatrix[i][j] += matrix1[i][k] * matrix2[k][j];
                    }
                }
            }
        }
        return resultMatrix;
    }

    public static double[] multiplyMatrix(double[][] matrix1, double[] matrix2) {
        double[] resultMatrix = new double[matrix1.length];
        if (matrix1[0].length != matrix2.length) {
            System.out.println("Error: the number of columns in matrix1 must equal the number of rows in matrix2");
            return resultMatrix;
        } else {
            for (int i = 0; i < matrix1.length; i++) {
                for (int k = 0; k < matrix1[0].length; k++) {
                    resultMatrix[i] += matrix1[i][k] * matrix2[k];
                }
            }
        }
        return resultMatrix;
    }

    //matrix must be square.
    public static double[][] invertMatrix(double[][] matrix) {
        int size = matrix.length;
        double[][] resultMatrix = new double[size][size];
        double temp = 0;

        double determinant = determinant(matrix);

        if (determinant == 0) {
            System.out.println("Error: no inverse");
            return null;
        }

        if (size > 2) {
            for (int i = 0; i < size; i++) {
                for (int j = 0; j < size; j++) {
                    double[][] tempMatrix = MatrixTool.delRowCol(matrix, i, j);
                    temp = determinant(tempMatrix) / determinant;
                    if ((i + j) % 2 == 1) {
                        temp = -1 * temp;
                    }
                    MatrixTool.setValue(resultMatrix, i, j, temp);
                }
            }
        } else {
            temp = MatrixTool.getValue(matrix, 0, 0);
            MatrixTool.setValue(resultMatrix, 1, 1, temp / determinant);
            temp = MatrixTool.getValue(matrix, 0, 1);
            MatrixTool.setValue(resultMatrix, 0, 1, -1 * temp / determinant);
            temp = MatrixTool.getValue(matrix, 1, 0);
            MatrixTool.setValue(resultMatrix, 1, 0, -1 * temp / determinant);
            temp = MatrixTool.getValue(matrix, 1, 1);
            MatrixTool.setValue(resultMatrix, 0, 0, temp / determinant);
        }
        resultMatrix = MatrixTool.createTranspose(resultMatrix);

        return resultMatrix;
    }

    public static double[][] subMatrix_row(double[][] matrix, int colIndex1,
            int colIndex2) {
        double[][] resultMatrix = new double[2][matrix[0].length];
        double[] temp1 = matrix_row(matrix, colIndex1);
        double[] temp2 = matrix_row(matrix, colIndex2);
        for (int i = 0; i < matrix[0].length; i++) {
            resultMatrix[0][i] = temp1[i];
            resultMatrix[1][i] = temp2[i];
        }
        return resultMatrix;
    }

    public static double[][] subMatrix_row1(double[][] matrix, int rowIndex1,
            int rowIndex2) {
        double[][] resultMatrix = new double[rowIndex2 - rowIndex1 +
                1][matrix[0].length];
        for (int i = 0; i < rowIndex2 - rowIndex1 + 1; i++) {
            for (int j = 0; j < matrix[0].length; j++) {
                resultMatrix[i][j] = matrix[i + rowIndex1][j];
            }
        }
        return resultMatrix;
    }

    public static double[][] subMatrix_col(double[][] matrix, int colIndex1,
            int colIndex2) {
        double[][] resultMatrix = new double[matrix.length][2];
        double[] temp1 = matrix_col(matrix, colIndex1);
        double[] temp2 = matrix_col(matrix, colIndex2);
        for (int i = 0; i < matrix.length; i++) {
            resultMatrix[i][0] = temp1[i];
            resultMatrix[i][1] = temp2[i];
        }
        return resultMatrix;
    }

    public static double[][] subMatrix_col1(double[][] matrix, int colIndex1,
            int colIndex2) {
        double[][] resultMatrix = new double[matrix.length][colIndex2 - colIndex1 +
                1];
        for (int i = 0; i < matrix.length; i++) {
            for (int j = 0; j < colIndex2 - colIndex1 + 1; j++) {
                resultMatrix[i][j] = matrix[i][j + colIndex1];
            }
        }
        return resultMatrix;
    }

    public static String[][] subMatrix_row(String[][] matrix, int colIndex1,
            int colIndex2) {
        String[][] resultMatrix = new String[2][matrix[0].length];
        String[] temp1 = matrix_row(matrix, colIndex1);
        String[] temp2 = matrix_row(matrix, colIndex2);
        for (int i = 0; i < matrix[0].length; i++) {
            resultMatrix[0][i] = temp1[i];
            resultMatrix[1][i] = temp2[i];
        }
        return resultMatrix;
    }

    public static String[][] subMatrix_row1(String[][] matrix, int rowIndex1,
            int rowIndex2) {
        String[][] resultMatrix = new String[rowIndex2 - rowIndex1 +
                1][matrix[0].length];
        for (int i = 0; i < rowIndex2 - rowIndex1 + 1; i++) {
            for (int j = 0; j < matrix[0].length; j++) {
                resultMatrix[i][j] = matrix[i + rowIndex1][j];
            }
        }
        return resultMatrix;
    }

    public static String[][] subMatrix_col(String[][] matrix, int colIndex1,
            int colIndex2) {
        String[][] resultMatrix = new String[matrix.length][2];
        String[] temp1 = matrix_col(matrix, colIndex1);
        String[] temp2 = matrix_col(matrix, colIndex2);
        for (int i = 0; i < matrix.length; i++) {
            resultMatrix[i][0] = temp1[i];
            resultMatrix[i][1] = temp2[i];
        }
        return resultMatrix;
    }

    public static String[][] subMatrix_col1(String[][] matrix, int colIndex1,
            int colIndex2) {
        String[][] resultMatrix = new String[matrix.length][colIndex2 - colIndex1 +
                1];
        for (int i = 0; i < matrix.length; i++) {
            for (int j = 0; j < colIndex2 - colIndex1 + 1; j++) {
                resultMatrix[i][j] = matrix[i][j + colIndex1];
            }
        }
        return resultMatrix;
    }

    public static double[] matrix_col(double[][] matrix, int colIndex) {
        double[] resultArray = new double[matrix.length];
        for (int i = 0; i < matrix.length; i++) {
            resultArray[i] = matrix[i][colIndex];
        }
        return resultArray;
    }

    public static String[] matrix_col(String[][] matrix, int colIndex) {
        String[] resultArray = new String[matrix.length];
        for (int i = 0; i < matrix.length; i++) {
            resultArray[i] = matrix[i][colIndex];
        }
        return resultArray;
    }

    public static double[] matrix_row(double[][] matrix, int rowIndex) {
        double[] resultArray = new double[matrix[0].length];
        for (int i = 0; i < matrix[0].length; i++) {
            resultArray[i] = matrix[rowIndex][i];
        }
        return resultArray;
    }

    public static String[] matrix_row(String[][] matrix, int rowIndex) {
        String[] resultArray = new String[matrix[0].length];
        for (int i = 0; i < matrix[0].length; i++) {
            resultArray[i] = matrix[rowIndex][i];
        }
        return resultArray;
    }

    //matrix1[0].length=matrix2[0].length
    public static String[][] matrixMerge_row(String[][] matrix1,
            String[][] matrix2) {
        String[][] resultMatrix = new String[matrix1.length +
                matrix2.length][matrix1[0].length];
        for (int i = 0; i < matrix1.length; i++) {
            for (int j = 0; j < matrix1[0].length; j++) {
                resultMatrix[i][j] = matrix1[i][j];
            }
        }
        for (int i = 0; i < matrix2.length; i++) {
            for (int j = 0; j < matrix2[0].length; j++) {
                resultMatrix[i + matrix1.length][j] = matrix2[i][j];
            }
        }
        return resultMatrix;
    }

    //matrix1.length=matrix2.length
    public static String[][] matrixMerge_col(String[][] matrix1,
            String[][] matrix2) {
        String[][] resultMatrix = new String[matrix1.length][matrix1[0].length +
                matrix2[0].length];
        for (int i = 0; i < matrix1.length; i++) {
            for (int j = 0; j < matrix1[0].length; j++) {
                resultMatrix[i][j] = matrix1[i][j];
            }
        }
        for (int i = 0; i < matrix2.length; i++) {
            for (int j = 0; j < matrix2[0].length; j++) {
                resultMatrix[i][j + matrix1[0].length] = matrix2[i][j];
            }
        }
        return resultMatrix;
    }
}
