/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.vdb.chip;

import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.strucs.Hyperlink;
import edu.mit.broad.vdb.meg.Gene;
import gnu.trove.TObjectIntHashMap;

import java.util.List;
import java.util.Set;

/**
 * Capture a Chip object while enabling lazy loading of chip data
 * <p/>
 * Probe -> unique sequence feature used to measure a gene
 */
public interface Chip extends PersistentObject {

    public static final String COMBO_DUMMY_SOURCE = "combo_dummy_source";
    public static NullSymbolMode OMIT_NULLS = new NullSymbolModes.OmitNullSymbolMode();
    public static NullSymbolMode REPLACE_WITH_PROBEID = new NullSymbolModes.ReplaceWithProbeIdMode();

    public Chip cloneShallow(final String newName);

    public String getSourcePath();

    public int getNumProbes() throws Exception;

    public Hyperlink getHyperlink(final String probeName);

    // HUGO/GENE SYMBOL BASED -------------------------------------
    // will error out if no such probe
    public Gene getHugo(final String probeName) throws Exception;

    public Set getHugos(final GeneSet probeNames) throws Exception;

    public Set getSymbols(final NullSymbolMode nmode) throws Exception;

    public Set getSymbols(final GeneSet probeNames, final NullSymbolMode nmode) throws Exception;

    public GeneSet getSymbols_gs(final GeneSet probeNames, final NullSymbolMode nmode) throws Exception;

    public GeneSetPair getSymbols(final GeneSetPair probes, final NullSymbolMode nmode) throws Exception;

    public GeneSetMatrixPair getSymbols(final GeneSetMatrixPair probes, final NullSymbolMode nmode) throws Exception;

    public Set getSymbols(final List probeNames, final NullSymbolMode nmode) throws Exception;

    public List getSymbols_synched(final List probeNames, final NullSymbolMode nmode) throws Exception;

    public String getSymbol(final String probeName, final NullSymbolMode nmode);

    public String getTitle(final String probeName, final NullSymbolMode nmode);

    // -------------------------------------------------------------------------------------------- //

    public String[] probize(final String[] forTheseSymbols);

    public GeneSet probize_gs(final String[] forTheseSymbols);

    public GeneSet probize_gs(final GeneSet forTheseSymbols);

    public String[] probize(final GeneSet forTheseSymbols);

    // ANNOTATORS (i.e append symbol) ---------------------------------

    public StringDataframe annotate(final IDataframe sdf);

    public StringDataframe annotate(final RankedList rl);

    // SYMBOLIZERS (i.e convert to symbol) ---------------------------------

    public RankedList symbolize(final RankedList rl);

    public GeneSetMatrix symbolize(final GeneSetMatrix gm);

    public GeneSet symbolize(final GeneSet gset);

    public StringDataframe symbolize(final String name, final TObjectIntHashMap probeCntMap);

    // PROBE-SYMBOL NAME RELATED ---------------------------------
    public Set getProbeNames(final Set symbols) throws Exception;

    public Set getProbeNames(final String symbol) throws Exception;

    // PROBE NAME RELATED ---------------------------------
    public String getProbeName(final int i) throws Exception;

    public String[] getProbeNamesArr() throws Exception;

    public Set getProbeNames() throws Exception;

    public GeneSet getProbeNamesAsGeneSet() throws Exception;

    public GeneSet getProbeNamesAsGeneSet(final Dataset ds) throws Exception;

    public boolean isProbe(final String probeName) throws Exception;

    // PROBE RELATED ------------------------------------------

    public Probe getProbe(final int i) throws Exception;

    public Probe getProbe(final String probeName) throws Exception;

    public Probe[] getProbes() throws Exception;

    public Probe[] getProbes(final GeneSet probeNames) throws Exception;

    public Probe[] getProbes(final Dataset ds) throws Exception;


} // End Chip
