/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.vdb.chip;

import edu.mit.broad.genome.objects.AbstractObject;
import edu.mit.broad.genome.objects.ColorMap;
import edu.mit.broad.genome.objects.FeatureAnnot;
import edu.mit.broad.genome.objects.GenesOfInterest;
import edu.mit.broad.genome.objects.strucs.Linked;
import edu.mit.broad.genome.reports.web.LinkedFactory;

/**
 * Integrates chip and dataset annotation
 * <p/>
 * Chip is used to lookup the annotation.
 * Ann is the thing that determines how many features and their names
 * <p/>
 *
 * @author Aravind Subramanian
 *         Date: Jan 22, 2005
 *         Time: 8:41:14 PM
 */
public class FeatureAnnotChipImpl extends AbstractObject implements FeatureAnnot {

    private Chip fChip;

    private ColorMap.Rows fColorMap_opt;

    private Helper fHelper;

    /**
     * Class constructor
     *
     * @param chip
     * @param colorMap_opt
     */
    public FeatureAnnotChipImpl(final Chip chip,
                                final ColorMap.Rows colorMap_opt) {

        if (chip == null) {
            throw new IllegalArgumentException("Param chip cannot be null");
        }

        super.initialize(chip.getName());

        this.fChip = chip;
        this.fColorMap_opt = colorMap_opt;
        this.fHelper = new Helper();
    }

    /**
     * Class constructor
     *
     * @param chip
     */
    public FeatureAnnotChipImpl(final Chip chip) {
        this(chip, (GenesOfInterest) null);
    }

    /**
     * Class constructor
     *
     * @param chip
     * @param gin_opt
     */
    public FeatureAnnotChipImpl(final Chip chip,
                                final GenesOfInterest gin_opt) {

        if (chip == null) {
            throw new IllegalArgumentException("Param chip cannot be null");
        }

        super.initialize(chip.getName());

        if (gin_opt != null) {
            this.fColorMap_opt = gin_opt.getAsColorMap();
        }

        this.fChip = chip;
        this.fHelper = new Helper();
    }

    public Chip getChip() {
        return fChip;
    }

    public void setChip(final Chip chip, final ColorMap.Rows cmr) {
        if (chip != null) {
            Helper.checkChip(fChip, chip);
            this.fChip = chip;
            this.fColorMap_opt = cmr;
        }
    }

    public String getQuickInfo() {
        return fChip.getQuickInfo();
    }

    public String[] getNames() {
        try {
            return fChip.getProbeNamesArr();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public ColorMap.Rows getColorMap() {
        return fColorMap_opt;
    }

    public int getNumFeatures() {
        try {
            return fChip.getNumProbes();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    // No native as made from a chip
    public boolean hasNativeDescriptions() {
        return false;
    }

    public String getNativeDesc(final String featureName) {
        return null;
    }

    public String getGeneSymbol(final String featureName) {
        return fHelper.getGeneSymbol(featureName, fChip);
    }

    public String getGeneTitle(final String featureName) {
        return fHelper.getGeneTitle(featureName, fChip);
    }

    // Features are gene symbols
    public Linked getLink(final String featureName) {
        return LinkedFactory.createLinkedGeneSymbol(featureName);
    }

} // End class FeatureAnnotationChipImpl
