/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.vdb.map;


/**
 * enum construct
 */
public class ProbeMergeMode {

    public static final ProbeMergeMode AVERAGE_MULTI_PROBES = new ProbeMergeMode("Average One-to-Many Probes");
    public static final ProbeMergeMode OMIT_MULTI_PROBES = new ProbeMergeMode("Completely Omit One-to-Many Probes");
    public static final ProbeMergeMode CHOOSE_ANY_ONE_PROBE = new ProbeMergeMode("Arbitarily Pick One, if >1 Probe");

    /**
     * @maint manually keep array in sync with declarations above and below
     */
    public static final ProbeMergeMode[] ALL = new ProbeMergeMode[]{AVERAGE_MULTI_PROBES, OMIT_MULTI_PROBES, CHOOSE_ANY_ONE_PROBE};

    String fType;

    ProbeMergeMode(String type) {
        this.fType = type;
    }

    public int hashCode() {
        return fType.hashCode();
    }

    public String toString() {
        return fType;
    }

    public boolean equals(Object obj) {

        if (obj instanceof ProbeMergeMode) {
            if (((ProbeMergeMode) obj).fType.equals(this.fType)) {
                return true;
            }
        }

        return false;
    }

    /**
     * a lookup metod for dir
     */
    public static ProbeMergeMode lookup(Object obj) {

        if (obj == null) {
            throw new NullPointerException("Null mode not allowed");
        }

        if (obj instanceof ProbeMergeMode) {
            return (ProbeMergeMode) obj;
        }

        if (obj instanceof String) {
            for (int i = 0; i < ALL.length; i++) {
                if (ALL[i].fType.equalsIgnoreCase(obj.toString())) {
                    return ALL[i];
                }
            }
        }

        throw new IllegalArgumentException("Unable to lookup MappingMode for: " + obj);
    }

} // End MappingMode
