/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.vdb.meg;

import edu.mit.broad.genome.NamingConventions;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

/**
 * Created by IntelliJ IDEA.
 * User: aravind subramanian
 * Date: Jan 1, 2004
 * Time: 11:46:36 PM
 * To change this template use Options | File Templates.
 */
public class DefaultGene implements Gene {

    private String fSymbol;

    private String fTitle;

    protected String fCyto_opt;

    private Set fAliases;

    protected Set fSeqAccessions;

    private static final Set EMPTY_SET = Collections.unmodifiableSet(new HashSet());

    // db4o constructor
    public DefaultGene() {
    }

    public Gene cloneDeep() {
        DefaultGene gene = new DefaultGene();
        gene.fSymbol = this.fSymbol;
        gene.fTitle = this.fTitle;
        gene.fAliases = new HashSet(fAliases);
        gene.fSeqAccessions = new HashSet(fSeqAccessions);
        return gene;
    }

    /**
     * Class constructor
     *
     * @param symbol
     * @param title
     * @param seqAccessions
     * @param aliases
     */
    public DefaultGene(final String symbol,
                       String title,
                       final Set aliases,
                       final String cyto_opt,
                       final Set seqAccessions) {

        if (symbol == null) {
            throw new IllegalArgumentException("Parameter symbol cannot be null");
        }

        this.fTitle = NamingConventions.titleize(title);
        this.fSymbol = symbol.toUpperCase(); // @note

        if (aliases != null) {
            this.fAliases = Collections.unmodifiableSet(aliases);
        }

        if (seqAccessions != null) {
            this.fSeqAccessions = Collections.unmodifiableSet(seqAccessions);
        }

        if (cyto_opt != null) {
            this.fCyto_opt = NamingConventions.parseCyto(cyto_opt);

            /*
            if (this.fCyto_opt == null) {
                fCyto_opt = cyto_opt; // not in format but keep anyway
            }
            */
        }

    }

    public String getSymbol() {
        return fSymbol;
    }

    public String getTitle() {
        return fTitle;
    }

    public String getCyto() {
        return fCyto_opt;
    }

    public String getTitle_truncated() {
        return Helper.getTitle_truncated(fTitle);
    }

    public Set getAliases() {
        if (fAliases == null) {
            return EMPTY_SET;
        } else {
            return Collections.unmodifiableSet(fAliases);
        }
    }

    public String[] getAliasesArray() {
        Set set = getAliases();
        return (String[]) set.toArray(new String[set.size()]);
    }

    public Set getSeqAccessions() {
        if (fSeqAccessions == null) {
            return EMPTY_SET;
        } else {
            return Collections.unmodifiableSet(fSeqAccessions);
        }
    }

    public boolean containsAlias(String alias) {
        return fAliases != null && fAliases.contains(alias);
    }

    public boolean containsSeqAccession(String acc) {
        return fSeqAccessions != null && fSeqAccessions.contains(acc);
    }

    public boolean equals(Object obj) { // @note equals on just the symbol
        return obj instanceof Gene && ((Gene) obj).getSymbol().equalsIgnoreCase(getSymbol());
    }

    public String toString() {
        return getSymbol();
    }

    // @note equals on justthe symbol
    public int hashCode() {
        return getSymbol().hashCode();
    }


} // End class DefaultNugo
