package edu.mit.broad.vdb.meg;

import edu.mit.broad.vdb.Organism;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * Author: Aravind Subramanian
 * Date: Oct 21, 2006
 * Time: 8:52:01 PM
 */
public interface EntrezGene extends Gene {

    public EntrezGene cloneDeep(final Set useTheseAliases);

    public EntrezGene cloneShallow(final String cyto);

    public EntrezGene cloneShallow(final Set addTheseSeqAccessions);

    /**
     * The nomenclature status of the name, where
     * 0 = official from a nomenclature committee,
     * I = interim from a nomenclature committee,
     * - = NCBI-supplied.
     */

    public static final String STATUS_OFFICIAL = "0";
    public static final String STATUS_INTERIM = "I";
    public static final String STATUS_NCBI = "N";

    public long getEntrezID();

    public long getTaxonId();

    public String getStatus();

    public boolean isOfficial();

    public boolean isOrg(final Organism org);

    public boolean isHuman();

    public boolean isMouse();

    public boolean isRat();

    public static class GeneIdAccessionSet {

        private long ezid;

        private Set accessions;

        public GeneIdAccessionSet(final long entrezGeneId, final Set accessions) {

            if (accessions == null) {
                throw new IllegalArgumentException("Param accessions cannot be null");
            }

            this.ezid = entrezGeneId;
            this.accessions = accessions;
        }

        public EntrezGene toGene(final EntrezGeneDb ezdb) {
            return ezdb.getEntrezGene(ezid);
        }

    }


    public static class Helper {

        public static Map toMap(final EntrezGene.GeneIdAccessionSet[] sets) {

            Map map = new HashMap();

            for (int i = 0; i < sets.length; i++) {
                String id = sets[i].ezid + "";
                Object obj = map.get(id);
                if (obj == null) {
                    obj = new HashSet();
                }
                ((Set) obj).addAll(sets[i].accessions);
                map.put(id, obj);
            }

            return map;
        }

    }

} // End interface EntrezGene
