package edu.mit.broad.vdb.meg;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.vdb.Organism;
import edu.mit.broad.vdb.chip.Probe;

import java.util.HashSet;
import java.util.Set;

/**
 * Author: Aravind Subramanian
 * Date: Oct 21, 2006
 * Time: 8:57:32 PM
 */
public class EntrezGeneImpl extends DefaultGene implements EntrezGene, Probe {

    private long fGeneID;

    private long fTaxonID;

    private String fStatus;

    // -------------------------------------------------------------------------------------------- //

    public Probe cloneDeepProbe() {
        throw new NotImplementedException();
    }

    public String getName() {
        return getSymbol();
    }

    public Gene getGene() {
        return this;
    }

    // -------------------------------------------------------------------------------------------- //


    public EntrezGene cloneDeep(final Set useTheseAliases) {
        return new EntrezGeneImpl(getEntrezID(), getTaxonId(), getSymbol(), getTitle(), useTheseAliases, getCyto(), getSeqAccessions(), getStatus());
    }

    public EntrezGene cloneShallow(final String cyto) {
        super.fCyto_opt = cyto;
        return this;
    }

    public EntrezGene cloneShallow(final Set addTheseSeqAccessions) {
        Set set = new HashSet();
        set.addAll(addTheseSeqAccessions);

        if (super.fSeqAccessions != null) {
            set.addAll(super.fSeqAccessions);
        }

        super.fSeqAccessions = set;
        return this;
    }

    /**
     * Class constructor
     *
     * @param entrezGeneID
     * @param symbol
     * @param title
     * @param aliases
     * @param cyto_opt
     * @param seqAccessions
     */
    public EntrezGeneImpl(
            final long entrezGeneID,
            final long entrezTaxonID,
            final String symbol,
            final String title,
            final Set aliases,
            final String cyto_opt,
            final Set seqAccessions,
            final String status) {

        super(symbol, title, aliases, cyto_opt, seqAccessions);
        this.fGeneID = entrezGeneID;
        this.fTaxonID = entrezTaxonID;


        if (status == null) {
            throw new IllegalArgumentException("Param status cannot be null");
        }


        if (status.equalsIgnoreCase("O") || status.equalsIgnoreCase("I") || status.equalsIgnoreCase("-")) {

        } else {
            throw new IllegalArgumentException("Unknown status>" + status + "<");
        }

        this.fStatus = status;
    }

    public boolean isOrg(final Organism org) {
        return getTaxonId() == org.getTaxonID();
    }

    public boolean isHuman() {
        return getTaxonId() == Organism.HUMAN.getTaxonID();
    }

    public boolean isMouse() {
        return getTaxonId() == Organism.MOUSE.getTaxonID();
    }

    public boolean isRat() {
        return getTaxonId() == Organism.RAT.getTaxonID();
    }

    public long getEntrezID() {
        return fGeneID;
    }

    public long getTaxonId() {
        return fTaxonID;
    }

    /**
     * Nomenclature status:
     * when not '-', indicates the status of the name from the
     * nomenclature authority (O for official, I for interim)
     *
     * @return
     */
    public boolean isOfficial() {
        return fStatus.equalsIgnoreCase("O") == false;
    }

    public String getStatus() {
        return fStatus;
    }

} // End class DefaultEntrezGeneId
