/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.vdb.meg;

import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.NotImplementedException;

import java.util.Set;

/**
 * HugoEntry
 * <p/>
 * Special handling
 * <p/>
 * A Hugo is meant to represent ONE gene. There might be many accessions 9synonyms) that refer to a gene.
 * Sometimes, the gene has an assigned symbol (hugo) and symbol is not null
 * In other cases, it has accessions but not a symbol yet -> isNa -> true
 * (na -> not avaliable or not assigned)
 */
public class SimpleGene implements Gene {

    private String fSymbol;

    private String fTitle;


    public Gene cloneDeep() {
        return new SimpleGene(this.fSymbol, this.fTitle);
    }

    /**
     * SYMBOL cannot be null
     * DEFAULT_TITLE can be null
     * <p/>
     * Class constructor
     *
     * @param symbol
     * @param title
     */
    public SimpleGene(final String symbol, final String title) {
        init(symbol, title);
    }


    // all data must already be duplicate etc if needeed
    private void init(String symbol, String title) {
        if (symbol == null) {
            throw new IllegalArgumentException("Parameter symbol cannot be null");
        }

        /*
        if (title == null) {
            throw new IllegalArgumentException("Parameter title cannot be null");
        }
        */

        this.fSymbol = symbol;

        // Get rid of the annoying quotes in title names
        this.fTitle = NamingConventions.titleize(title);

    }

    public String getSymbol() {
        return fSymbol;
    }

    public String getTitle() {
        return fTitle;
    }

    public String getTitle_truncated() {
        return Helper.getTitle_truncated(fTitle);
    }

    // This is part of the magix
    public String toString() {
        return fSymbol;
    }

    public String getCyto() {
        return null;
    }

    public String toString(char delim) {
        StringBuffer buf = new StringBuffer();
        buf.append(getSymbol()).append(delim).append(getTitle()).append(delim);
        return buf.toString();
    }

    public int hashCode() {
        return fSymbol.hashCode();
    }

    // equality done only on the Symbol
    /// @note part of the magix
    public boolean equals(Object obj) {
        if (obj instanceof Gene) {
            return ((Gene) obj).getSymbol().equalsIgnoreCase(this.fSymbol);
        }

        return false;
    }

    public Set getAliases() {
        throw new NotImplementedException();
    }

    public String[] getAliasesArray() {
        Set set = getAliases();
        return (String[]) set.toArray(new String[set.size()]);
    }

    public boolean containsAlias(String alias) {
        throw new NotImplementedException();
    }

    public boolean containsSeqAccession(String acc) {
        throw new NotImplementedException();
    }

    public Set getSeqAccessions() {
        throw new NotImplementedException();
    }

} // End class SimpleHugo