/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.vdb.msigdb;

import edu.mit.broad.genome.objects.FeatureAnnot;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.GeneSetMatrix;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.vdb.sampledb.SampleAnnot;

import java.util.Set;


/**
 * @author Aravind Subramanian
 *         Date: Jun 1, 2004
 *         Time: 2:07:46 AM
 */
public interface MSigDB extends PersistentObject {

    public String getName();

    public String getVersion();

    public String getBuildDate();

    public int getNumGeneSets();

    public int getNumGenes();

    public String[] getStandardNames();

    public GeneSet getAllGenes(final boolean originalIds);

    public GeneSetAnnotation[] getGeneSetAnnotations();

    public GeneSetAnnotation getGeneSetAnnotation(int g);

    public GeneSetAnnotation getGeneSetAnnotation(final String gsStdName);

    public Set getGeneSetAnnotations_set();

    public MSigDB[] splitByTags();

    public MSigDB[] splitByCategory();

    public MSigDB[] split(boolean byCategory, boolean byTag);

    public GeneSetMatrix createGeneSetMatrix(final boolean originalIds);

    public GeneSetMatrix createGeneSetMatrix(final String[] gsetNames, final boolean originalIds);

    public FeatureAnnot getAsFeatureAnnot();

    public SampleAnnot getAsSampleAnnot();

} // End interface MSigDb
