package edu.mit.broad.vdb.sampledb;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.objects.StringDataframe;
import edu.mit.broad.genome.utils.Strings;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.vdb.chip.Chip;

/**
 * @author Aravind Subramanian
 *         Date: Jul 10, 2006
 *         Time: 10:33:25 PM
 */
public abstract class AbstractSample implements Sample {

    public String fCelFileName;

    protected String fSampleName;

    protected SampleDataSource fDataSource;

    protected String fChipName;

    protected String fOrganism;

    protected float fPCall;

    protected boolean fQuality;

    protected String fComments;

    // @maint bludgeon our way through
    public String getCategoryValue(final String categoryName) {
        if (categoryName == null) {
            throw new IllegalArgumentException("Param categoryName cannot be null");
        }

        if (categoryName.equalsIgnoreCase(SampleAnnot.HEADER_PLATFORM)) {
            return getChipName();
        } else if (categoryName.equalsIgnoreCase(SampleAnnot.HEADER_SAMPLE_NAME)) {
            return getName();
        } else if (categoryName.equalsIgnoreCase(SampleAnnot.HEADER_ORGANISM)) {
            return getOrganism();
        } else if (categoryName.equalsIgnoreCase(SampleAnnot.HEADER_COMMENTS)) {
            return getComments();
        } else if (categoryName.equalsIgnoreCase(SampleAnnot.HEADER_PCALLS)) {
            return getPCall() + "";
        } else if (categoryName.equalsIgnoreCase(SampleAnnot.HEADER_QUALITY)) {
            return getQuality() + "";
        } else if (categoryName.equalsIgnoreCase(SampleAnnot.HEADER_SOURCE_OF_DATA)) {
            return getDataSource().getName();
        } else {
            return null; // let sub class handle exceptioning etc
        }

    }

    public static void setCommonSampleProperties(final AbstractSample[] samples, final StringDataframe sdf) {

        final String[] sample_names = sdf.getColumn(SampleAnnot.HEADER_SAMPLE_NAME);
        for (int r = 0; r < samples.length; r++) {
            samples[r].fSampleName = sample_names[r].toUpperCase();
        }


        final String[] data_source = sdf.getColumn(SampleAnnot.HEADER_SOURCE_OF_DATA);
        for (int r = 0; r < samples.length; r++) {
            samples[r].fDataSource = SampleDataSource.lookup(data_source[r]);
        }

        final String[] chip_types = sdf.getColumn(SampleAnnot.HEADER_PLATFORM);
        for (int r = 0; r < samples.length; r++) {
            samples[r].fChipName = VdbRuntimeResources.geo2affy_chip_convert(chip_types[r]);
        }

        final String[] organisms = sdf.getColumn(SampleAnnot.HEADER_ORGANISM);
        for (int r = 0; r < samples.length; r++) {
            samples[r].fOrganism = organisms[r];
        }

        final String[] pcalls = sdf.getColumn(SampleAnnot.HEADER_PCALLS);
        for (int r = 0; r < samples.length; r++) {
            if (pcalls[r].equalsIgnoreCase("na")) {
                samples[r].fPCall = Float.NaN;
            } else {
                samples[r].fPCall = _float(pcalls[r]);
            }
        }

        final String[] quality = sdf.getColumn(SampleAnnot.HEADER_QUALITY);
        for (int r = 0; r < samples.length; r++) {
            samples[r].fQuality = lookupQuality(quality[r]);
        }

        final String[] comments = sdf.getColumn(SampleAnnot.HEADER_COMMENTS);
        for (int r = 0; r < samples.length; r++) {
            samples[r].fComments = comments[r];
        }

    }

    public static float _float(final String cellValue) {
        String s = Strings.replace(cellValue, COMMA, EMPTY);
        return Float.parseFloat(s); // excel likes to put commas
    }

    protected final static String COMMA = ",";
    protected final static String EMPTY = "";

    public static boolean lookupQuality(final String quality) {
        return !(quality != null && quality.toUpperCase().startsWith("N"));
    }

    public Chip getChip() {
        return VdbRuntimeResources.getChip(getChipName());
    }

    public String getCelFileName() {
        return fCelFileName;
    }

    public String getChipName() {
        return fChipName;
    }

    public String getName() {
        return fSampleName;
    }

    public SampleDataSource getDataSource() {
        return fDataSource;
    }

    public String getOrganism() {
        return fOrganism;
    }

    public float getPCall() {
        return fPCall;
    }

    public boolean getQuality() {
        return fQuality;
    }

    public String getComments() {
        return fComments;
    }

    public String getPCall_str() {
        float pc = getPCall();

        if (pc < 20) {
            return "<20";
        } else if (pc < 30) {
            return "20<=>30";
        } else if (pc < 40) {
            return "30<=>40";
        } else if (pc > 60) { // wierd on the other end
            return ">60";
        } else {
            return "40<=>60";
        }

    }

    public int hashCode() {
        return getName().hashCode();
    }

    public String toString() {
        return getName();
    }

    public boolean equals(Object obj) {
        throw new NotImplementedException();
    }

} // End class AbstractSample