package edu.mit.broad.vdb.sampledb;

import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.Printf;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.strucs.Linked;
import edu.mit.broad.genome.parsers.MiscParsers;
import edu.mit.broad.genome.reports.web.LinkedFactory;
import edu.mit.broad.genome.utils.ParseException;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.map.Chip2ChipMapperSymbol;
import gnu.trove.TObjectIntHashMap;
import org.apache.log4j.Logger;

import java.util.*;

/**
 * @author Aravind Subramanian
 *         Date: May 29, 2006
 *         Time: 1:03:47 PM
 */
public abstract class AbstractSampleAnnot extends AbstractObject implements SampleAnnot {

    protected Logger log = XLogger.getLogger(AbstractSampleAnnot.class);

    protected static final Logger klog = XLogger.getLogger(AbstractSampleAnnot.class);

    /// @Note IMP if you add here make sure to check the clone methods
    protected Sample[] fSamples;

    // Lazilly filled
    private Map fCelFileNameSampleMap;

    // Lazilly filled
    private Map fSampleNameSampleMap;

    /**
     * Class constructor
     *
     * @param samples
     */
    protected AbstractSampleAnnot() {
        // @note the implementor must fill samples
    }

    public SampleAnnot createRandomAnnot() {
        return createRandomAnnot(getSampleNames());
    }

    public SampleAnnot createRandomAnnot(final String[] useOnlyTheseSamples) {
        throw new NotImplementedException();
    }

    // -------------------------------------------------------------------------------------------- //

    protected static StringDataframe checkSdf_makeUC_and_remove_bad_qc_samples(final MiscParsers.ExcelSdfColorStruc exsr) throws ParseException {
        // too error prone to keep the bad qc sample in
        // Just remove them from the SDF

        StringDataframe sdf = exsr.sdf;
        // @note IMP make everything upper case
        sdf = sdf.cloneDeep_uppercase();

        // Do a full search to make sure there are no funny chars in any field
        List badValues = new ArrayList();

        Set checkOnlyTheseFields = new HashSet();
        checkOnlyTheseFields.add(SampleAnnot.HEADER_SAMPLE_NAME);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_SERIES_ID);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_SAMPLE_TYPE);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_POPULATION_NAME);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_LINEAGE);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_SEMISUPER);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_BASIC);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_POPULATION_NUMBER);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_SOURCE_OF_DATA);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_PLATFORM);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_ORGANISM);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_RNA_AMPLIFICATION);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_DIAGNOSIS);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_MOLECULAR_ABNORMALITY);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_CELL_SOURCE);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_DIFFERENTIATION_STATE);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_REPLICATE_NUMBER);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_SAMPLE_CODE);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_SORT_DATE);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_SORT_LOCATION);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_RNA_PREP_DATE);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_RNA_CONCENTRATION);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_SAMPLE_TYPE);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_PCALLS);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_QUALITY);
        checkOnlyTheseFields.add(SampleAnnot.HEADER_CELL_COUNT);

        for (int r = 0; r < sdf.getNumRow(); r++) {
            for (int c = 0; c < sdf.getNumCol(); c++) {
                String value = sdf.getElement(r, c);
                if (checkOnlyTheseFields.contains(sdf.getColumnName(c).toUpperCase())) {
                    if (NamingConventions.isBadCharacter(value, false, true, true)) { // dates are valid, allow hyphens .. some cel files have them and some sample names also
                        badValues.add(value);
                    }
                }
            }
        }

        if (!badValues.isEmpty()) {
            throw new ParseException("Bad values in the SampleAnnot Db: " + badValues.size() + "\n" + badValues);
        }

        List rowNames2Use = new ArrayList();
        for (int r = 0; r < sdf.getNumRow(); r++) {
            boolean goodQuality = AbstractSample.lookupQuality(sdf.getElement(r, SampleAnnot.HEADER_QUALITY));

            if (goodQuality) {
                if (sdf.hasColumn("LINEAGE") && sdf.getElement(r, "LINEAGE").equalsIgnoreCase("NA")) {
                    goodQuality = false; // @note hack to exclude samples with NA lineage
                }
            }

            if (goodQuality) {
                rowNames2Use.add(sdf.getRowName(r));
            }


        }

        klog.info("# of samples excluded by QC: " + (sdf.getNumRow() - rowNames2Use.size()));

        exsr.sdf = sdf.cloneDeep_by_row_names((String[]) rowNames2Use.toArray(new String[rowNames2Use.size()]), true);
        return exsr.sdf;
    }

    protected static void checkForDuplicates_SampleNames_and_SampleCelFiles(final Sample[] samples) throws ParseException {
        Set duplicateCelFileNames = new HashSet();
        Set duplicateCodeFileNames = new HashSet();
        Set names = new HashSet();
        for (int r = 0; r < samples.length; r++) {
            String cfn = samples[r].getCelFileName();

            if (cfn == null) {
                throw new IllegalStateException("sample cell file cfn is null: " + cfn);
            }

            if (names.contains(cfn)) {
                duplicateCelFileNames.add(cfn);
            }

            cfn = samples[r].getName();

            if (cfn == null) {
                throw new IllegalStateException("sample name cfn is null: " + cfn);
            }

            if (names.contains(cfn)) {
                duplicateCodeFileNames.add(cfn);
            }

            names.add(samples[r].getCelFileName());
            names.add(samples[r].getName());
        }

        if (duplicateCelFileNames.size() != 0) {
            throw new ParseException("Duplicate sample CEL file name: " + duplicateCelFileNames);
        }

        if (duplicateCodeFileNames.size() != 0) {
            throw new ParseException("Duplicate sample code name: " + duplicateCodeFileNames);
        }
    }

    // -------------------------------------------------------------------------------------------- //

    public SampleAnnot cloneDeep(final String[] useOnlyTheseSamples) {
        return cloneDeep(getName() + "_" + useOnlyTheseSamples.length, useOnlyTheseSamples);
    }

    public SampleAnnot cloneDeep(final Chip onlyFromThisChip) {
        List useSampleNames = new ArrayList();

        for (int i = 0; i < getNumSamples(); i++) {
            Sample sample = getSample(i);
            if (sample.getChip().equals(onlyFromThisChip)) {
                useSampleNames.add(sample.getName());
            }
        }

        return cloneDeep(getName() + "_" + onlyFromThisChip.getName(), (String[]) useSampleNames.toArray(new String[useSampleNames.size()]));
    }

    // -------------------------------------------------------------------------------------------- //

    public Map getCelFileSampleNameMap() {
        Map map = new HashMap();
        for (int i = 0; i < getNumSamples(); i++) {

            String cfn = getSample(i).getCelFileName();
            String sn = getSample(i).getName();
            map.put(cfn.toUpperCase(), sn);
            //map.put(cfn.toLowerCase(), sn);
            if (cfn.toUpperCase().endsWith(".CEL") == false) {
                map.put(cfn.toUpperCase() + ".CEL", sn);
                //map.put(cfn.toLowerCase() + ".cel", getSample(i).getName());
            }

        }

        log.debug(">>>> " + map);

        return map;
    }

    public int getNumSamples() {
        return fSamples.length;
    }

    public Sample[] getSamples() {
        return fSamples;
    }

    public Sample[] getSamples(final String categName, final String stateName) {
        Category categ = createCategory(categName, new String[]{stateName});
        return categ.getSamples();
    }

    public String[] getSampleNames(final String categName, final String stateName) {
        Category categ = createCategory(categName, new String[]{stateName});
        return categ.getSampleNames();
    }

    public String[] getSampleNames(final String categName, final String[] stateNames) {
        Category categ = createCategory(categName, stateNames);
        return categ.getSampleNames();
    }

    public String[] getSampleNames() {
        String[] names = new String[getNumSamples()];
        for (int i = 0; i < names.length; i++) {
            names[i] = getSample(i).getName();
        }

        return names;
    }

    public List getSampleNames_list() {
        String[] names = getSampleNames();
        List list = new ArrayList();
        for (int i = 0; i < names.length; i++) {
            list.add(names[i]);
        }

        return list;
    }

    public String[] getSampleNames(final String[] forTheseCelFileNames) {
        List names = getSampleNames_list(forTheseCelFileNames);
        return (String[]) names.toArray(new String[names.size()]);
    }

    public List getSampleNames_list(final String[] forTheseCelFileNames) {
        String[] names = new String[forTheseCelFileNames.length];
        List list = new ArrayList();
        for (int i = 0; i < names.length; i++) {
            list.add(getSampleName(forTheseCelFileNames[i]));
        }

        return list;
    }

    public Sample getSample(final int i) {
        return fSamples[i];
    }

    private Sample _getSample(String celFileNameOrSampleName) {

        celFileNameOrSampleName = make_name_proper(celFileNameOrSampleName);
        Sample sample = _get(celFileNameOrSampleName);

        if (sample != null) {
            return sample;
        }

        // Try harder -> RMA sometimes appends a leading X to sample names

        if (celFileNameOrSampleName.startsWith("X")) {
            celFileNameOrSampleName = celFileNameOrSampleName.substring(1, celFileNameOrSampleName.length());
        }

        return _get(celFileNameOrSampleName);
    }

    private Sample _get(final String name) {
        if (fCelFileNameSampleMap == null) {
            this.fCelFileNameSampleMap = new HashMap();
            this.fSampleNameSampleMap = new HashMap();
            for (int i = 0; i < getNumSamples(); i++) {
                Sample sample = getSample(i);
                fCelFileNameSampleMap.put(sample.getCelFileName(), sample);
                fSampleNameSampleMap.put(sample.getName(), sample);
            }
        }

        // First try by cel file name
        Object obj = fCelFileNameSampleMap.get(name);
        if (obj != null) {
            return (Sample) obj;
        }

        obj = fSampleNameSampleMap.get(name);
        if (obj != null) {
            return (Sample) obj;
        }

        /*
        System.out.println("?>>>> CHECKING: >" + name + "<");
        System.out.println(">>> " + fCelFileNameSampleMap);
        */

        return null;
    }

    public boolean isSample(final String cellFileOrSampleName) {
        return _getSample(cellFileOrSampleName) != null;
    }

    private TObjectIntHashMap sampleNameIndexMap;

    public int getSampleIndex(final Sample sample) {
        return getSampleIndex(sample.getName());
    }

    public int getSampleIndex(final String sampleName) {
        if (sampleNameIndexMap == null) {
            sampleNameIndexMap = new TObjectIntHashMap();
            for (int i = 0; i < getNumSamples(); i++) {
                sampleNameIndexMap.put(getSample(i).getName(), i);
            }
        }

        return sampleNameIndexMap.get(sampleName);
    }

    // -------------------------------------------------------------------------------------------- //

    // @note only works if the entire thing is 1 chip

    public Chip getChip() {
        return getChip(getSampleNames());
    }

    public Chip getChip(final Dataset ds) {
        return getChip(ds.getColumnNamesArray());
    }

    public Chip getChip(final String celFileOrSampleName) {
        if (celFileOrSampleName == null) {
            throw new IllegalArgumentException("Param celFileOrSampleName cannot be null");
        }

        Sample sample = getSample(celFileOrSampleName);
        return sample.getChip();
    }

    public Chip getChip(final TemplateFromAnnot dt) {
        return getChip(dt.getSampleNames_ordered());
    }

    // checks and gets
    public Chip getChip(final String[] celFileOrSampleName) {
        if (celFileOrSampleName == null) {
            throw new IllegalArgumentException("Param celFileOrSampleName cannot be null");
        }

        Chip chip = null;
        for (int i = 0; i < celFileOrSampleName.length; i++) {
            Sample sample = getSample(celFileOrSampleName[i]);
            Chip got = sample.getChip();
            if (chip == null) {
                chip = got;
            } else {
                if (got == null) {
                    throw new IllegalArgumentException("Chip was not null: " + chip.getName() +
                            " but got null: " + got + " for celFileOrSampleName: "
                            + celFileOrSampleName[i]);
                } else {
                    if (!chip.getName().equalsIgnoreCase(got.getName())) {
                        throw new IllegalArgumentException("Chip was: " + chip.getName() +
                                " but got was: " + got + " for celFileOrSampleName: "
                                + celFileOrSampleName[i]);
                    }
                }
            }
        }

        return chip;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public Dataset orderByDefaultCategory(final Dataset ds) {
        final String[] ordered = orderByDefaultCategory(ds.getColumnNamesArray());
        return new DatasetGenerators().orderDatasetColumns(ds.getName(), ordered, ds, true);
    }

    public Dataset orderByCategory(final Dataset ds, final String categName) {
        final String[] ordered = orderByCategory(ds.getColumnNamesArray(), categName);
        return new DatasetGenerators().orderDatasetColumns(ds.getName(), ordered, ds, true);
    }

    public String[] getSampleNames_OrderByCategory(final String categName) {

        List samples = new ArrayList();
        for (int i = 0; i < getNumSamples(); i++) {
            samples.add(getSample(i));
        }

        Collections.sort(samples, new CategoryComparator(categName));
        return _toSampleNames(samples);
    }

    public String[] orderByDefaultCategory(final String[] inputCelOrSampleNames) {
        List names = getSampleNames_list(inputCelOrSampleNames);
        Collections.sort(names, createDefaultSampleComparator());
        return (String[]) names.toArray(new String[names.size()]);
    }

    public String[] orderByCategory(final String[] inputSampleNames, final String categName) {
        List names = getSampleNames_list(inputSampleNames);
        Collections.sort(names, new CategoryComparator(categName));
        return (String[]) names.toArray(new String[names.size()]);
    }

    public void orderByCategory(final List inputSampleNames, final String categName) {
        Collections.sort(inputSampleNames, new CategoryComparator(categName));
    }

    public String[] orderStateNames(final String[] inputStateNames, final String categName) {
        List names = Arrays.asList(inputStateNames);
        Collections.sort(names, createDefaultStateComparator(categName));
        return (String[]) names.toArray(new String[names.size()]);
    }

    public Dataset orderStateNames(final Dataset dsWithStatesAsColNames, final String categName) {
        final String[] ordered = orderStateNames(dsWithStatesAsColNames.getColumnNamesArray(), categName);
        return new DatasetGenerators().orderDatasetColumns(dsWithStatesAsColNames.getName(), ordered, dsWithStatesAsColNames, true);
    }

    public String[] getSampleNames_OrderByDefaultCategory() {
        List names = getSampleNames_list();
        Collections.sort(names, createDefaultSampleComparator());
        return (String[]) names.toArray(new String[names.size()]);
    }

    class CategoryComparator implements Comparator {

        String fCategName;

        CategoryComparator(final String categName) {
            this.fCategName = categName;
        }

        // first check the diff state number and then the popn name if the number is ditto
        public int compare(Object pn1, Object pn2) {

            if (pn1 == null) {
                throw new IllegalArgumentException("Param pn1 cannot be null");
            }

            if (pn2 == null) {
                throw new IllegalArgumentException("Param pn2 cannot be null");
            }

            Sample sample1;
            Sample sample2;

            if (pn1 instanceof Sample) {
                sample1 = (Sample) pn1;
            } else {
                sample1 = getSample(pn1.toString());
            }

            if (pn2 instanceof Sample) {
                sample2 = (Sample) pn2;
            } else {
                sample2 = getSample(pn2.toString());
            }


            String value1 = sample1.getCategoryValue(fCategName);
            String value2 = sample2.getCategoryValue(fCategName);

            if (value1.equalsIgnoreCase(value2)) {
                return sample1.getName().compareToIgnoreCase(sample2.getName());
            } else {
                return value1.compareToIgnoreCase(value2);
            }

        }

        public boolean equals(Object o2) {
            return false;
        }
    }    // End CategoryComparator

    // -------------------------------------------------------------------------------------------- //
    // ----------------------------------------- MISC METHODS ----------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public Linked getLink(final String sampleName) {
        return LinkedFactory.createLinkedGeneSymbol(sampleName); // @todo
    }

    public String getQuickInfo() {
        return null;
    }

    // cel file names for ALL samples
    public String[] getCelFileNames() {
        String[] cfn = new String[getNumSamples()];
        for (int i = 0; i < cfn.length; i++) {
            cfn[i] = getSample(i).getCelFileName();
        }

        return cfn;
    }

    // Only support gene symbol to probe set mappings
    public GeneSet map(final GeneSet gset, final Dataset ds, final boolean getSourceChipFromDataset) throws Exception {
        Chip chip = getChip(ds);
        Chip2ChipMapperSymbol c2cs = VdbRuntimeResources.getGeneSymbolMapper(chip, getSourceChipFromDataset);
        GeneSet mgset = c2cs.map(gset, false).getMappedGeneSet(true);
        return mgset.cloneDeep(ds); // Finally qualify to the features actually in the ds
    }

    // -------------------------------------------------------------------------------------------- //

    public List getSamples_list(final int[] indices) {
        List list = new ArrayList();
        for (int i = 0; i < indices.length; i++) {
            list.add(getSample(indices[i]));
        }

        return list;
    }

    public List getSampleNames_list(final int[] indices) {
        List list = new ArrayList();
        for (int i = 0; i < indices.length; i++) {
            list.add(getSample(indices[i]).getName());
        }

        return list;
    }

    public Sample[] getSamples(final int[] indices) {
        Sample[] samples = new Sample[indices.length];
        for (int i = 0; i < indices.length; i++) {
            samples[i] = getSample(indices[i]);
        }

        return samples;
    }

    public String getSampleName(final String celFileName) {
        return getSample(celFileName).getName();
    }

    public Sample[] getSamples(final String[] celFileOrSampleNames) {
        final Sample[] samples = new Sample[celFileOrSampleNames.length];
        for (int i = 0; i < celFileOrSampleNames.length; i++) {
            samples[i] = getSample(celFileOrSampleNames[i]);
        }

        return samples;
    }

    public String[] getCelFileNames(final String[] celFileOrSampleNames) {
        final String[] names = new String[celFileOrSampleNames.length];
        for (int i = 0; i < celFileOrSampleNames.length; i++) {
            names[i] = getSample(celFileOrSampleNames[i]).getCelFileName();
        }

        return names;
    }

    // -------------------------------------------------------------------------------------------- //

    protected static String make_name_proper(String celFileOrSampleName) {
        //log.debug("Checking >" + celFileOrSampleName + "<");

        if (celFileOrSampleName == null) {
            throw new IllegalArgumentException("Param celFileOrSampleName cannot be null");
        }

        celFileOrSampleName = celFileOrSampleName.trim();

        if (celFileOrSampleName.endsWith(".CEL")) {
            celFileOrSampleName = celFileOrSampleName.substring(0, celFileOrSampleName.indexOf(".CEL"));
        }

        if (celFileOrSampleName.endsWith(".cel")) {
            celFileOrSampleName = celFileOrSampleName.substring(0, celFileOrSampleName.indexOf(".cel"));
        }

        if (celFileOrSampleName.endsWith(".TXT")) {
            celFileOrSampleName = celFileOrSampleName.substring(0, celFileOrSampleName.indexOf(".TXT"));
        }

        if (celFileOrSampleName.endsWith(".txt")) {
            celFileOrSampleName = celFileOrSampleName.substring(0, celFileOrSampleName.indexOf(".txt"));
        }

        celFileOrSampleName = celFileOrSampleName.trim();

        return celFileOrSampleName;
    }

    // ------------------------------------------------------------------------------- //
    // ------------------------------- TEMPLATE METHODS ------------------------------- //
    // ------------------------------------------------------------------------------- //


    public TemplateFromAnnot createTemplate(final String categoryName) throws Exception {
        return createCategory(categoryName).createTemplate();
    }

    // for ALL Categories
    public TemplateFromAnnot[] createAllTemplates() throws Exception {
        String[] names = getAllCategoricalCategoryNames();
        TemplateFromAnnot[] dts = new TemplateFromAnnot[names.length];
        for (int i = 0; i < names.length; i++) {
            dts[i] = createTemplate(names[i]);
        }

        return dts;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public String getStateName(final String categName, final Sample sample) {
        return sample.getCategoryValue(categName);
    }

    // .cel INsensitive
    public Sample getSample(final String celFileNameOrSampleName) {
        Sample sample = _getSample(celFileNameOrSampleName);
        if (sample == null) {
            throw new IllegalArgumentException("No such sample: " + celFileNameOrSampleName + " \nsamples are\n" + getSampleNames_list());
        } else {
            return sample;
        }
    }

    // Makes sure that all samples in the dataset are known to sindb
    public void ensureValidSampleNames(final Dataset ds, boolean enforceSameOrder) throws Exception {
        ensureValidSampleNames(ds.getColumnNamesArray(), enforceSameOrder);
    }

    public void ensureValidSampleNames(final String[] celFileOrSampleNames, boolean enforceSameOrder) throws Exception {

        if (enforceSameOrder) {
            if (celFileOrSampleNames.length != getNumSamples()) {
                throw new IllegalArgumentException("Mismatched sample numbers. celFileOrSampleNames: " + celFileOrSampleNames + " annot: " + getNumSamples());
            }
        }

        List unknownSampleNames = new ArrayList();

        for (int c = 0; c < celFileOrSampleNames.length; c++) {
            String sampleName = celFileOrSampleNames[c];
            try {

                Sample sample = getSample(sampleName);

                if (enforceSameOrder) {
                    Sample sa_c = getSample(c);
                    if (sample.equals(sa_c) == false) {
                        String str = "Mismatched sample orderd b/w ds and sample annotation: " + sample.getName() + " " + sa_c.getName();
                        klog.debug(str);
                        throw new Exception(str);
                    }

                }

            } catch (Throwable t) {
                if (unknownSampleNames.size() < 2) {
                    t.printStackTrace();
                }
                unknownSampleNames.add(sampleName);
            }
        }

        if (unknownSampleNames.size() != 0) {
            StringBuffer buf = new StringBuffer("Unknown sample names:  " + unknownSampleNames.size() + " \n");
            for (int i = 0; i < unknownSampleNames.size(); i++) {
                buf.append("[").append(unknownSampleNames.get(i)).append("] ");
            }
            throw new RuntimeException(buf.toString());
        } else {
            log.info("All samples passed checks: " + celFileOrSampleNames.length);
        }
    }

    public String ensureSameOrganism(final Dataset ds) throws Exception {
        return ensureSameOrganism(ds.getColumnNamesArray());
    }

    public String ensureSameOrganism(final String[] celFileOrSampleNames) throws Exception {

        List unknownSampleNames = new ArrayList();
        String organism = null;
        for (int c = 0; c < celFileOrSampleNames.length; c++) {
            String sampleName = celFileOrSampleNames[c];
            try {

                Sample sample = getSample(sampleName);
                if (organism == null) {
                    organism = sample.getOrganism();
                }

                if (!sample.getOrganism().equalsIgnoreCase(organism)) {
                    throw new IllegalArgumentException("Mismatched organisms: " + organism + " and " + sample.getOrganism() + " for sample: " + sample.getName());
                }

            } catch (Throwable t) {
                unknownSampleNames.add(sampleName);
            }
        }

        if (unknownSampleNames.size() != 0) {
            StringBuffer buf = new StringBuffer("Unknown sample names:  " + unknownSampleNames.size() + " \n");
            for (int i = 0; i < unknownSampleNames.size(); i++) {
                buf.append("[").append(unknownSampleNames.get(i)).append("] ");
            }

            throw new RuntimeException(buf.toString());
        }

        log.info("All samples passed sin checks: " + celFileOrSampleNames.length);
        return organism;
    }

    public Chip ensureSameChip(final Sample[] samples) {

        List mismatchedSampleNames = new ArrayList();
        Chip chip = null;
        for (int c = 0; c < samples.length; c++) {
            try {
                if (chip == null) {
                    chip = samples[c].getChip();
                }

                if (!samples[c].getChip().equals(chip)) {
                    mismatchedSampleNames.add(samples[c].getName());
                    //throw new IllegalArgumentException("Mismatched chips: " + chip + " and " + samples[c].getChip() + " for sample: " + samples[c].getName());
                }

            } catch (Throwable t) {
                throw new RuntimeException(t);
            }
        }

        if (mismatchedSampleNames.size() != 0) {
            StringBuffer buf = new StringBuffer("Mismatched chips for samples:  " + mismatchedSampleNames.size() + " \n");
            for (int i = 0; i < mismatchedSampleNames.size(); i++) {
                buf.append("[").append(mismatchedSampleNames.get(i)).append("] ");
            }

            throw new RuntimeException(buf.toString());
        }

        log.info("All samples passed checks: " + samples.length);

        return chip;
    }

    // -------------------------------------------------------------------------------------------- //
    // ----------------------------------- CATEGORY RELATED METHODS ------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public Category[] createAllCategories() {
        String[] names = getAllCategoricalCategoryNames();
        Category[] categs = new Category[names.length];
        for (int i = 0; i < names.length; i++) {
            categs[i] = createCategory(names[i]);
        }

        return categs;
    }

    // For ALL states
    public Category createCategory(final String categoryName) {

        final Map stateNameStateMap = new HashMap();

        final String[] celFileOrSampleNames = getCelFileNames();

        boolean match = false;
        for (int i = 0; i < celFileOrSampleNames.length; i++) {
            final Sample sample = getSample(celFileOrSampleNames[i]);
            final String stateName = sample.getCategoryValue(categoryName);
            match = true;
            Object curr = stateNameStateMap.get(stateName);
            if (curr == null) {
                curr = new ArrayList();
            }
            ((List) curr).add(sample);
            stateNameStateMap.put(stateName, curr);
        }

        //log.debug("For category: " + categoryName + " # of states: " + stateState.size());

        // sanity check
        if (!match) {
            throw new IllegalArgumentException("No matches for category: " + categoryName +
                    " for states: " + stateNameStateMap + " "
                    + " celFileOrSampleNames: " + celFileOrSampleNames.length + " \n" + Printf.outs(celFileOrSampleNames));
        }

        final State[] states = new State[stateNameStateMap.size()];
        int cnt = 0;
        Iterator it = stateNameStateMap.keySet().iterator();
        while (it.hasNext()) {
            String stateName = it.next().toString();
            //log.debug("Done value: " + value);
            List list = (List) stateNameStateMap.get(stateName);
            Sample[] samples = (Sample[]) list.toArray(new Sample[list.size()]);
            states[cnt++] = new State(stateName, samples);
        }

        return new Category(categoryName, states, this);
    }

    // for ALL samples
    public Category createCategory(final String categoryName, final Set forTheseStateNames) {
        return _createCategory(categoryName,
                (String[]) forTheseStateNames.toArray(new String[forTheseStateNames.size()]), getCelFileNames());
    }

    // for ALL samples
    public Category createCategory(final String categoryName, final String[] forTheseStateNames) {
        return _createCategory(categoryName, forTheseStateNames, getCelFileNames());
    }

    // this lookup is case INsensitive
    private Category _createCategory(final String categoryName,
                                     final String[] forTheseStateNames_in, // <- order needs to be preserved
                                     final String[] celFileOrSampleNames) {

        final String[] forTheseStateNames = new String[forTheseStateNames_in.length];
        for (int i = 0; i < forTheseStateNames.length; i++) {
            forTheseStateNames[i] = forTheseStateNames_in[i].toUpperCase();
        }

        // check that no duplicate states
        final Set set = checkForDuplicates(forTheseStateNames);
        final Map stateNameKeyStateMap = new HashMap();

        boolean atLeastOneMatch = false;
        for (int i = 0; i < celFileOrSampleNames.length; i++) {
            Sample sample = getSample(celFileOrSampleNames[i]);
            String stateName = sample.getCategoryValue(categoryName);// @note this is case INsensitive
            if (set.contains(stateName)) {
                atLeastOneMatch = true;
                Object curr = stateNameKeyStateMap.get(stateName);
                if (curr == null) {
                    curr = new ArrayList();
                }
                ((List) curr).add(sample);
                stateNameKeyStateMap.put(stateName, curr);
            }
        }

        if (stateNameKeyStateMap.size() != forTheseStateNames.length) {
            StringBuffer buf = new StringBuffer("No match in SampleDb for category: " + categoryName + " for some states(s): \n");
            for (int i = 0; i < forTheseStateNames.length; i++) {
                if (!stateNameKeyStateMap.containsKey(forTheseStateNames[i])) {
                    buf.append(forTheseStateNames[i]).append('\n');
                }
            }

            throw new IllegalArgumentException(buf.toString());
        }

        //log.debug("For category: " + categoryName + " # of states: " + stateStrKeystateObjectMap.size());

        // sanity check
        if (!atLeastOneMatch) {
            throw new IllegalArgumentException("No matches at in category: " + categoryName + " for states: " + set);
        }

        final State[] states = new State[forTheseStateNames.length];

        for (int i = 0; i < states.length; i++) {
            //log.debug("Done state: " + state);
            List list = (List) stateNameKeyStateMap.get(forTheseStateNames[i]);
            Sample[] samples = (Sample[]) list.toArray(new Sample[list.size()]);
            states[i] = new State(forTheseStateNames[i], samples);
        }

        /*
        // @note dont do this as we want to keep the input order
        int cnt = 0;
        Iterator it = stateStrKeystateObjectMap.keySet().iterator();
        while (it.hasNext()) {
            String stateName = it.next().toString();
            //log.debug("Done state: " + state);
            List list = (List) stateStrKeystateObjectMap.get(state);
            Sample[] samples = (Sample[]) list.toArray(new Sample[list.size()]);
            states[cnt++] = new state(state, samples);
        }
        */

        return new Category(categoryName, states, this);
    }

    private static Set checkForDuplicates(final String[] ss) {
        // check that no duplicate values
        Set set = new HashSet();
        for (int i = 0; i < ss.length; i++) {
            if (set.contains(ss[i])) {
                throw new IllegalArgumentException("Duplicate states not allowed: " + ss[i]);
            }
            set.add(ss[i]);

        }
        if (set.size() != ss.length) {
            throw new IllegalArgumentException("Duplicate states specified: " + set);
        }

        return set;
    }

    protected static String[] _toSampleNames(final Sample[] samples) {
        String[] names = new String[samples.length];
        for (int i = 0; i < samples.length; i++) {
            names[i] = samples[i].getName();
        }

        return names;
    }

    protected static String[] _toSampleNames(final List samples) {
        String[] ss = new String[samples.size()];
        for (int i = 0; i < samples.size(); i++) {
            ss[i] = ((Sample) samples.get(i)).getName();
        }

        return ss;
    }

} // End class SampleAnnotAbstract
