package edu.mit.broad.vdb.sampledb;

import edu.mit.broad.genome.objects.ColorMap;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.genome.objects.TemplateFromAnnot;
import edu.mit.broad.genome.objects.strucs.Linked;
import edu.mit.broad.vdb.chip.Chip;

import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Aravind Subramanian
 *         Date: Sep 10, 2005
 *         Time: 8:02:38 PM
 */
public interface SampleAnnot extends PersistentObject {

    // @note IMP the order here is useful for visualizers
    public static final String HEADER_SAMPLE_NAME = "SAMPLE_NAME";
    public static final String HEADER_SAMPLE_TITLE = "SAMPLE_TITLE";
    public static final String HEADER_SAMPLE_DESC = "SAMPLE_DESC";
    public static final String HEADER_CEL_FILE_NAME = "CEL_FILE_NAME";
    public static final String HEADER_SAMPLE_SOURCE = "SAMPLE_SOURCE";
    public static final String HEADER_SERIES_ID = "SERIES_ID";
    public static final String HEADER_SERIES_TITLE = "SERIES_TITLE";
    public static final String HEADER_SERIES_SUMMARY = "SERIES_SUMMARY";
    public static final String HEADER_SAMPLE_TYPE = "TYPE";
    public static final String HEADER_POPULATION_NAME = "POPULATION_NAME";
    public static final String HEADER_LINEAGE = "LINEAGE";
    public static final String HEADER_LINEAGE2 = "LINEAGE2";
    public static final String HEADER_SEMISUPER = "SEMISUPER";
    public static final String HEADER_BASIC = "BASIC";

    public static final String HEADER_POPULATION_NUMBER = "POPULATION_NUMBER";

    public static final String HEADER_PUBMEDID = "PUBMEDID";
    public static final String HEADER_AUTHOR = "AUTHOR";

    public static final String HEADER_SOURCE_OF_DATA = "SOURCE_OF_DATA";
    public static final String HEADER_PLATFORM = "PLATFORM";
    public static final String HEADER_ORGANISM = "ORGANISM";
    public static final String HEADER_SCHEMA = "ORGANISM";
    public static final String HEADER_RNA_AMPLIFICATION = "RNA_AMPLIFICATION";
    public static final String HEADER_DIAGNOSIS = "DIAGNOSIS";

    public static final String HEADER_SUBMISSION_DATE = "SUBMISSION_DATE";
    public static final String HEADER_RELEASE_DATE = "RELEASE_DATE";

    public static final String HEADER_MOLECULAR_ABNORMALITY = "MOLECULAR_ABNORMALITY";
    public static final String HEADER_CELL_SOURCE = "SOURCE_OF_CELLS";
    public static final String HEADER_DIFFERENTIATION_STATE = "DIFFERENTIATION_STATE";
    public static final String HEADER_REPLICATE_NUMBER = "REPLICATE_NUMBER";

    public static final String HEADER_SAMPLE_CODE = "SAMPLE_CODE";

    public static final String HEADER_SORT_DATE = "SORT_DATE";
    public static final String HEADER_SORT_LOCATION = "SORT_LOCATION";
    public static final String HEADER_RNA_PREP_DATE = "RNA_PREPARATION_DATE";
    public static final String HEADER_RNA_CONCENTRATION = "CONCENTRATION_OF_STARTING_RNA(NG/UL)";
    public static final String HEADER_CELL_COUNT = "CELL_COUNT_FROM_SORT";
    public static final String HEADER_PCALLS = "%_P_CALLS_ON_ARRAY";
    public static final String HEADER_QUALITY = "GOOD_QUALITY";
    public static final String HEADER_COMMENTS = "COMMENTS";

    // @note IMP the order here is useful for visualizers
    public static final String HEADER_DRUG_CLASS = "DRUG_CLASS";
    public static final String HEADER_DRUG_CLASS2 = "DRUG_CLASS2";
    public static final String HEADER_BATCH_ID = "BATCH_ID";
    public static final String HEADER_DRUG_DOSE = "DOSE";
    //public static final String HEADER_CEL_FILE_NAME_VEHICLE = "CELL_FILE_NAME_VEHICLE";
    public static final String HEADER_CEL_FILE_NAME_VEHICLE = "VEHICLE_SCAN_ID";
    public static final String HEADER_CEL_FILE_NAME_PERTURB = "PERTURB_SCAN_ID";
    public static final String HEADER_CELL_LINE = "CELL_LINE";
    public static final String HEADER_DRUG_NAME = "DRUG_NAME";


    public Map getCelFileSampleNameMap();

    // -------------------------------------------------------------------------------------------- //
    public SampleAnnot cloneDeep(final String newName, final String[] useOnlyTheseSamples);

    public SampleAnnot cloneDeep(final String[] useOnlyTheseSamples);

    public SampleAnnot cloneDeep(final Chip onlyFromThisChip);

    // -------------------------------------------------------------------------------------------- //

    public SampleAnnot createRandomAnnot();

    public SampleAnnot createRandomAnnot(final String[] useOnlyTheseSamples);

    // -------------------------------------------------------------------------------------------- //

    public boolean isSample(String sampleName);

    public Sample getSample(final int coln);

    public Sample getSample(final String celFileNameOrSampleName);

    public int getSampleIndex(final Sample sample);

    public int getSampleIndex(final String celFileNameOrSampleName);

    // -------------------------------------------------------------------------------------------- //

    public Comparator createDefaultStateComparator(final String categName);

    public Comparator createDefaultSampleComparator();

    public String[] getSampleNames_OrderByDefaultCategory();

    public String[] getSampleNames_OrderByCategory(final String categName);

    public String[] orderByDefaultCategory(final String[] inputSampleNames);

    public String[] orderByCategory(final String[] inputSampleNames, final String categName);

    public void orderByCategory(final List inputSampleNames, final String categName);

    public Dataset orderByDefaultCategory(final Dataset ds);

    public Dataset orderByCategory(final Dataset ds, final String categName);

    // by state
    public String[] orderStateNames(final String[] inputStateNames, final String categName);

    public Dataset orderStateNames(final Dataset dsWithStatesAsColNames, final String categName);

    // -------------------------------------------------------------------------------------------- //

    public List getSamples_list(final int[] indices);

    public Sample[] getSamples();

    public Sample[] getSamples(final int[] indices);

    public Sample[] getSamples(final String[] celFileNamesOrSampleNames);

    // -------------------------------------------------------------------------------------------- //

    public int getNumSamples();

    public String[] getSampleNames();

    public String[] getSampleNames(final String[] celFileOrSampleNames);

    public List getSampleNames_list();

    public List getSampleNames_list(final int[] indices);

    public String getSampleName(final String celFileName);

    public Sample[] getSamples(final String categName, final String stateName);

    public String[] getSampleNames(final String categName, final String stateName);

    public String[] getSampleNames(final String categName, final String[] stateNames);

    //public String[] getTrainTestSampleNames();    

    // -------------------------------------------------------------------------------------------- //

    public String[] getCelFileNames();

    public String[] getCelFileNames(final String[] celFileOrSampleNames);

    public ColorMap.Columns getColorMap();

    public Linked getLink(final String sampleName);

    // -------------------------------------------------------------------------------------------- //

    public Chip getChip(final Dataset ds);

    public Chip getChip(final String celFileNameOrSampleName);

    // @note only works if the entire thing is 1 chip
    public Chip getChip();

    public Chip getChip(final TemplateFromAnnot dt);

    public Chip getChip(final String[] celFileNameOrSampleName);

    // ------------------------------------------------------------------------------- //
    // ------------------------------- TEMPLATE METHODS ------------------------------- //
    // ------------------------------------------------------------------------------- //

    public TemplateFromAnnot createTemplate(final String categoryName) throws Exception;

    // for ALL Categories
    public TemplateFromAnnot[] createAllTemplates() throws Exception;

    // ------------------------------------------------------------------------------- //
    // ------------------------------- CATEGORY METHODS ------------------------------- //
    // ------------------------------------------------------------------------------- //

    public String[] getAllCategoricalCategoryNames();

    public boolean isValidCategoricalCategory(final String categoryName);

    //public String[] getAllCategoryNames();

    //public boolean isCategoryName(final String categName);

    public Category[] createAllCategories();

    // For ALL states
    public Category createCategory(final String categoryName);

    public String getStateName(final String categoryName, final Sample sample);

    // for ALL samples
    public Category createCategory(final String categoryName, final Set forTheseStateNames);

    // for ALL samples
    public Category createCategory(final String categoryName, final String[] forTheseStateNames);

} // End interface SampleAnnot
