package edu.mit.broad.vdb.sampledb;

/**
 * @author Aravind Subramanian
 * Date: Oct 14, 2006
 * Time: 4:09:45 PM
 */

/**
 * @author Aravind Subramanian
 * Date: Jun 2, 2006
 * Time: 8:21:09 PM
 */

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.alg.ComparatorFactory;
import edu.mit.broad.genome.objects.ColorMap;
import edu.mit.broad.genome.objects.StringDataframe;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.objects.strucs.NamedColor;
import edu.mit.broad.genome.parsers.MiscParsers;
import edu.mit.broad.genome.utils.ColorUtils;
import edu.mit.broad.genome.utils.FileUtils;

import java.awt.*;
import java.io.File;
import java.util.*;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Aug 18, 2005
 *         Time: 4:43:39 PM
 * @note intentionally very nci60 specific
 * <p/>
 * <p/>
 * Do deep sanity checks when parsing out the sin file:
 * 1) expected column headers exist
 * 2) fields are type ok (e.g date, float etc)
 * <p/>
 */

public class SampleAnnotNCI60 extends AbstractSampleAnnot {

    public static String[] getCommonCategoryNames() {
        return new String[]{};
    }

    private static List ALL_CATEGORY_NAMES = new ArrayList(); // like it as a list to control the order a bit
    private static Set CATEGORICAL_CATEGORY_NAMES = new HashSet();

    static {
        // @maint needs to be manually kept in synch with above
        ALL_CATEGORY_NAMES.add(HEADER_SAMPLE_NAME);
        ALL_CATEGORY_NAMES.add(HEADER_LINEAGE);
    }

    // -------------------------------------------------------------------------------------------- //
    // ------------------------------------- CLASS VARIABLES -------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    private ColorMap.Columns fColorMap;

    private Map fStateColorMap;

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    /**
     * PRIVATIZED Class constructor
     *
     * @param name
     */
    private SampleAnnotNCI60(final String name, final Map stateColorMap, final SampleNCI60[] samples) {
        initHere(name, stateColorMap, samples);
    }

    // The core init method
    private void initHere(final String name, final Map stateColorMap, final SampleNCI60[] samples) {
        super.initialize(name);
        this.fSamples = samples;
        this.fStateColorMap = stateColorMap;
        this.fColorMap = new ColorMapImpl(stateColorMap);
    }

    // @note IMP keep these in synch with the constructor and vars
    public SampleAnnot cloneDeep(final String newName, final String[] useOnlyTheseSamples) {
        final List list = new ArrayList();
        for (int i = 0; i < useOnlyTheseSamples.length; i++) {
            list.add(getSample(useOnlyTheseSamples[i]));
        }

        return new SampleAnnotNCI60(newName, fStateColorMap, (SampleNCI60[]) list.toArray(new SampleNCI60[list.size()]));
    }

    public String[] getAllCategoricalCategoryNames() {
        return (String[]) CATEGORICAL_CATEGORY_NAMES.toArray(new String[CATEGORICAL_CATEGORY_NAMES.size()]);
    }

    public boolean isValidCategoricalCategory(final String categoryName) {

        if (categoryName == null) {
            throw new IllegalArgumentException("Param categoryName cannot be null");
        }

        return CATEGORICAL_CATEGORY_NAMES.contains(categoryName.toUpperCase()); // @note always UC check
    }

    public static String[] getAllCategoryNames() {
        return (String[]) ALL_CATEGORY_NAMES.toArray(new String[ALL_CATEGORY_NAMES.size()]);
    }

    public static boolean isCategoryName(final String categName) {
        return ALL_CATEGORY_NAMES.contains(categName);
    }

    // -------------------------------------------------------------------------------------------- //
    // ----------------------------------------- SAMPLE METHODS ----------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public Comparator createDefaultSampleComparator() {
        return new LineageComparator();
    }

    public Comparator createDefaultStateComparator(String categName) {
        throw new NotImplementedException();
    }

    public String[] getSampleNames_OrderByLineage() {

        List samples = new ArrayList();
        for (int i = 0; i < getNumSamples(); i++) {
            samples.add(getSample(i));
        }

        Collections.sort(samples, new LineageComparator());

        return _toSampleNames(samples);
    }

    public ColorMap.Columns getColorMap() {
        return fColorMap;
    }

    public String[] getLineages() {
        List list = new ArrayList();
        for (int i = 0; i < getNumSamples(); i++) {
            String ln = _getSample(i).getLineage();
            if (list.contains(ln) == false) {
                list.add(ln);
            }
        }

        return (String[]) list.toArray(new String[list.size()]);
    }

    // -------------------------------------------------------------------------------------------- //

    private SampleNCI60 _getSample(final int i) {
        return (SampleNCI60) getSample(i);
    }

    // -------------------------------------------------------------------------------------------- //
    // ---------------------------------- PARSING METHODS ----------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public void saveBrief(final Template t_opt, final File file) throws Exception {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < getNumSamples(); i++) {
            final SampleNCI60 sample = (SampleNCI60) getSample(i);
            buf.append(sample.getName()).append('\t');
            buf.append(sample.getLineage()).append('\t');
            buf.append(sample.getDataSource()).append('\t');
            buf.append(sample.getChipName()).append('\t');
            if (t_opt != null) {
                buf.append(t_opt.getItemByProfilePos(i).getId()).append('\t');
            }
            if (t_opt != null) {
                buf.append(t_opt.getClassByProfilePos(i).getName());
            }

            buf.append('\n');
        }

        FileUtils.write(buf.toString(), file);
    }

    public static SampleAnnotNCI60 parse(final File file) throws Exception {
        return parse(MiscParsers.excel2sdf_and_colors(file));
    }


    public static SampleAnnotNCI60 parse(final MiscParsers.ExcelSdfColorStruc exsr) throws Exception {

        final StringDataframe sdf = AbstractSampleAnnot.checkSdf_makeUC_and_remove_bad_qc_samples(exsr);

        SampleNCI60[] samples = new SampleNCI60[sdf.getNumRow()];
        for (int r = 0; r < samples.length; r++) {
            String name = make_name_proper(sdf.getRowName(r));
            samples[r] = new SampleNCI60(name);
        }

        AbstractSample.setCommonSampleProperties(samples, sdf);

        String[] ss = sdf.getColumn(HEADER_LINEAGE);
        for (int r = 0; r < samples.length; r++) {
            samples[r].fLineage = ss[r];
        }

        checkForDuplicates_SampleNames_and_SampleCelFiles(samples);

        return new SampleAnnotNCI60(exsr.sdf.getName(), exsr.stateColorMap, samples);
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    /**
     * Inner class representing the color map
     * <p/>
     * Rather than using all categories as columns we only use the ones below
     */
    private class ColorMapImpl implements ColorMap.Columns {

        private Map fValueColorMap;

        private Map fBatchNameColorMap;

        // each category becomes a row
        private String[] ROW_NAMES = new String[]{ // HEADER_DRUG_CLASS2 for padding
                "",
                HEADER_BATCH_ID,
                HEADER_CELL_LINE,
                ""
        };

        /**
         * Class constructor
         *
         * @param name
         */
        public ColorMapImpl(final Map valueColorMap) {
            if (valueColorMap == null) {
                throw new IllegalArgumentException("Param nameColorMap cannot be null");
            }

            this.fValueColorMap = new HashMap();
            this.fBatchNameColorMap = new HashMap();

            // @note annot munges case to UC always thats whay we make a new map
            for (Iterator iterator = valueColorMap.keySet().iterator(); iterator.hasNext();) {
                Object key = iterator.next();
                Object value = valueColorMap.get(key);
                this.fValueColorMap.put(key.toString(), value);
                this.fValueColorMap.put(key.toString().toUpperCase(), value);
            }

            //System.out.println(">>> " + valueColorMap);

        }

        public String[] getRowNames() {
            return ROW_NAMES;
        }

        public Color getColor(final String rowName, final String sampleName) {

            if (rowName.equalsIgnoreCase(HEADER_LINEAGE)) {
                String batch = ((SampleNCI60) getSample(sampleName)).getLineage();
                Object obj = fBatchNameColorMap.get(batch);
                if (obj == null) {
                    obj = ColorUtils.generate();
                    fBatchNameColorMap.put(batch, obj);
                }

                return (Color) obj;
            }

            final String value = getValue(rowName, sampleName);
            final Object color = fValueColorMap.get(value);
            if (color != null) {
                return (Color) color;
            }

            return null;
        }

        // simply by value irrespective of category etc
        public Color getColor(final String value) {

            final Object color = fValueColorMap.get(value);
            if (color != null) {
                return (Color) color;
            }

            return null;
        }

        public NamedColor[] getLegend(final String categName) {
            List list = new ArrayList();
            for (int i = 0; i < getNumSamples(); i++) {
                Sample sample = getSample(i);
                String value = sample.getCategoryValue(categName);
                Color color = getColor(value);
                if (color != null) {
                    NamedColor nc = new NamedColor(value, color);
                    if (list.contains(nc) == false) {
                        list.add(nc);
                    }
                }
            }

            Collections.sort(list, new ComparatorFactory.NamedColorComparator());

            return (NamedColor[]) list.toArray(new NamedColor[list.size()]);
        }

        public String getRowName(final int row) {
            return ROW_NAMES[row];
        }

        // each row is a diff category
        // this returns the value in that categories column that corresps to the sample
        public String getValue(final String rowName, final String sampleName) {
            final Sample sample = getSample(sampleName);
            if (isCategoryName(rowName)) {
                return sample.getCategoryValue(rowName);
            }

            return null;
        }

        public int getNumRow() {
            return ROW_NAMES.length;
        }

    } // End inner class ColorMapImpl


    class LineageComparator implements Comparator {
        // first check the diff state number and then the popn name if the number is ditto
        public int compare(Object pn1, Object pn2) {

            if (pn1 == null) {
                throw new IllegalArgumentException("Param pn1 cannot be null");
            }

            if (pn2 == null) {
                throw new IllegalArgumentException("Param pn2 cannot be null");
            }

            SampleNCI60 sample1;
            SampleNCI60 sample2;

            if (pn1 instanceof SampleNCI60) {
                sample1 = (SampleNCI60) pn1;
            } else {
                sample1 = (SampleNCI60) getSample(pn1.toString());
            }

            if (pn2 instanceof SampleNCI60) {
                sample2 = (SampleNCI60) pn2;
            } else {
                sample2 = (SampleNCI60) getSample(pn2.toString());
            }

            return sample1.getLineage().compareToIgnoreCase(sample2.getLineage());

        }

        public boolean equals(Object o2) {
            return false;
        }
    }    // End CellLineComparator


} // End class SampleAnnotNCI60
