package edu.mit.broad.vdb.sampledb;

import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.APMMatrix;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.DefaultDataset;

/**
 * @author Aravind Subramanian
 *         Date: Jun 27, 2006
 *         Time: 1:18:38 AM
 */
public class StateExpressionImpl implements StateExpression {

    public State fState;

    public Dataset fStateDs;

    /**
     * Class constructor
     *
     * @param stateName
     * @param ds
     */
    public StateExpressionImpl(final State state, final Dataset ds) {
        this.fState = state;
        this.fStateDs = ds;
    }

    public State getState() {
        return fState;
    }

    public Dataset getDataset() {
        return fStateDs;
    }

    public double getMeanOrMedianValue(final String rowName, final boolean useMean) {
        return getDataset().getRow(rowName).meanOrMedian(useMean);
    }

    public float calcPCallFrac(final DefaultDataset ds_with_apm, final String probeName) {

        APMMatrix apm = ds_with_apm.getAPMMatrix();

        int rowIndex = ds_with_apm.getRowIndex(probeName);

        float numPCall = 0;
        for (int c = 0; c < fStateDs.getNumCol(); c++) {

            String colName = fStateDs.getColumnName(c);
            int colIndex = ds_with_apm.getColumnIndex(colName);

            if (apm.isPresent(rowIndex, colIndex)) {
                numPCall++;
            }

        }

        return numPCall / (float) fStateDs.getNumCol();

    }

    public float calcExpressionFrac(final float exprMoreThanThisValue, final String probeName) {
        Vector v = fStateDs.getRow(probeName);

        float numMoreCnt = 0;

        for (int i = 0; i < v.getSize(); i++) {
            if (v.getElement(i) >= exprMoreThanThisValue) {
                numMoreCnt++;
            }
        }

        return numMoreCnt / (float) v.getSize();
    }

    public float calcExpressionMean(final String probeName, final boolean useMeanGex_for_each_class_not_median) {
        return (float) fStateDs.getRow(probeName).meanOrMedian(useMeanGex_for_each_class_not_median);
    }

} // End class StateExpressionImpl
