/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench;


import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;
import xtools.api.Tool;
import xtools.api.ToolCategory;
import xtools.api.param.Param;

import java.util.Comparator;
import java.util.StringTokenizer;


/**
 * A collection of commonly useful comparators.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ComparatorFactory2 {

    private static final Logger klog = XLogger.getLogger(ComparatorFactory2.class);

    /**
     * Privatized class constructor.
     * Only static methods.
     */
    private ComparatorFactory2() {
    }


    public static class IntegerComparator implements Comparator {

        public int compare(Object pn1, Object pn2) {

            Integer s1 = new Integer(Integer.MIN_VALUE);

            try {
                if (pn1 != null) {
                    s1 = new Integer(pn1.toString());
                }
            } catch (Throwable t) {
            }

            Integer s2 = new Integer(Integer.MIN_VALUE);

            try {
                if (pn2 != null) {
                    s2 = new Integer(pn2.toString());
                }
            } catch (Throwable t) {
            }

            return s1.compareTo(s2);
        }

        public boolean equals(Object o2) {
            return false;
        }
    }    // End IntegerComparator

    public static class FloatComparator implements Comparator {

        public int compare(Object pn1, Object pn2) {

            Float s1 = new Float(Float.MIN_VALUE);

            try {
                if (pn1 != null) {
                    s1 = new Float(pn1.toString());
                }
            } catch (Throwable t) {
            }

            Float s2 = new Float(Float.MIN_VALUE);

            try {
                if (pn2 != null) {
                    s2 = new Float(pn2.toString());
                }
            } catch (Throwable t) {
            }

            return s1.compareTo(s2);
        }

        public boolean equals(Object o2) {
            return false;
        }
    }    // End FloatComparator


    /**
     * Alphabetical Comparator.
     * case sensitive
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    public static class ParamNameComparator implements Comparator {

        /**
         * Return -1 if o1 is less than o2, 0 if they're equal, +1 if o1 is greater than o2.
         */
        public int compare(Object pn1, Object pn2) {

            String s1 = ((Param) pn1).getName();
            String s2 = ((Param) pn2).getName();

            //klog.debug("Comparing: " + s1 + " and: " + s2);

            return s1.compareTo(s2);
        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End ParamNameComparator

    public static class ToolByNameComparator implements Comparator {

        /**
         * Return -1 if o1 is less than o2, 0 if they're equal, +1 if o1 is greater than o2.
         */
        public int compare(Object pn1, Object pn2) {

            // trouble is that getName is the full package qualified class name
            // so have to extract

            String s1 = getClasslessToolName((Tool) pn1);
            String s2 = getClasslessToolName((Tool) pn2);

            //klog.debug("Comparing: " + s1 + " and: " + s2);
            return s1.compareTo(s2);
            // return s1.substring(0, 1).compareTo(s2.substring(0, 1));
        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End ToolByNameComparator

    public static String getClasslessToolName(final Tool tool) {
        StringTokenizer tok = new StringTokenizer(tool.getName(), ".");
        String name = null;
        while (tok.hasMoreElements()) {
            name = tok.nextToken();
        }

        return name;
    }

    /**
     * Class ToolCategoryComparator
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    public static class ToolCategoryComparator implements Comparator {

        /**
         * Return -1 if o1 is less than o2, 0 if they're equal, +1 if o1 is greater than o2.
         */
        public int compare(Object pn1, Object pn2) {

            String s1 = pn1.toString();
            String s2 = pn2.toString();

            if (pn1 instanceof ToolCategory) {
                s1 = ((ToolCategory) pn1).getName();
            }

            if (pn2 instanceof ToolCategory) {
                s2 = ((ToolCategory) pn2).getName();
            }

            return s1.compareTo(s2);
        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End StringComparator

    /**
     * Alphabetical sorts classes by class NAME
     * <p/>
     * objexts must be Class'score or Strings
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    public static class ClassNameComparator implements Comparator {

        /**
         * Return -1 if o1 is less than o2, 0 if they're equal, +1 if o1 is greater than o2.
         */
        public int compare(Object pn1, Object pn2) {

            String s1 = pn1.toString();
            String s2 = pn2.toString();

            if (pn1 instanceof Class) {
                s1 = ((Class) pn1).getName();
            }

            if (pn2 instanceof Class) {
                s2 = ((Class) pn2).getName();
            }

            return s1.compareTo(s2);
        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End ClassNameComparator

}    // End ComparatorFactory2

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
