/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.actions;


import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.utils.SystemUtils;
import edu.mit.broad.xbench.core.api.Application;
import edu.mit.broad.xbench.prefs.FilePreference;
import edu.mit.broad.xbench.prefs.Preference;
import edu.mit.broad.xbench.prefs.XPreferencesFactory;
import org.apache.log4j.Logger;

import javax.swing.*;

/**
 * Base action for actions that triggers an external process on a File.
 * For example, opening a file in Microsoft Excel.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public abstract class ExtAction extends XAction {

    protected String fPath;
    protected final Logger log = XLogger.getLogger(ExtAction.class);

    /**
     * @param file The File to run the External Action on
     */
    public void setPath(String path) {
        this.fPath = path;
    }

    public String getPath() {
        return fPath;
    }

    protected void showHelpfulExtError(final Preference pref, final String appname, final Throwable t) {

        StringBuffer msg = new StringBuffer("Could not find Application: ").append(appname).append(' ');
        msg.append("\nUse Options->Preferences... to set the correct path to ").append(appname).append("\n\n");

        JButton showPref = XPreferencesFactory.createActionButton(pref);
        Application.getWindowManager().showError(msg.toString(), t, new JButton[]{showPref});

    }

    protected static String getExePath(FilePreference pref) {

        String exePath;
        if (SystemUtils.isMac()) {
            exePath = "/usr/bin/open";
        } else if (SystemUtils.isLinuxOrUnix()) {
            exePath = pref.getFile().getPath();
        } else { // assume windows
            if (pref.getFile().exists()) { // lets go with the pref if it exists
                exePath = pref.getFile().getPath();
            } else {
                exePath = "rundll32 url.dll,FileProtocolHandler"; // otherwise let windows do it
            }
        }

        return exePath;
    }

}    // End ExtAction

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
