/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.actions;


import edu.mit.broad.xbench.core.Widget;
import edu.mit.broad.xbench.core.api.Application;

import javax.swing.*;
import javax.swing.event.InternalFrameListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;


/**
 * for widget actions that take a while (more than several secs) to launch
 * Basicaly same as the WidgetAction except no cursor or app status stuff
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public abstract class LongWidgetAction extends WidgetAction {

    /*
     * WITHOUT FOXTROT - orig amber style impl.
     *  DONT DELETE YET
     * public void actionPerformed(ActionEvent evt) {
     *   //log.debug("actionPerformed: " + evt);
     *
     *   Runnable r = createTask(evt);
     *   Thread t = new Thread(r);
     *   t.start();
     *   //log.debug("started thread");
     * }
     */
    public Runnable createTask(final ActionEvent evt) {

        return new Runnable() {

            public void run() {

                try {
                    //log.debug("Waiting for action");
                    //waitCursor("");
                    //defaultCursor();
                    // this is just to indicate that something hapenned so a dummy timer
                    log.info("Starting: " + getName());
                    Widget widget = getWidget();

                    if (widget != null) {
                        if (fExplicitSize != null) {
                            fWindow = Application.getWindowManager().openWindow(widget, fExplicitSize);
                        } else {
                            fWindow = Application.getWindowManager().openWindow(widget);
                        }

                        //window.setIcon(true); // causes the window to launch and then instantly become an icon

                        // comm out after window interface
                        if (fWindow instanceof JInternalFrame) {
                            ((JInternalFrame) fWindow).setJMenuBar(widget.getJMenuBar());
                            InternalFrameListener ifl = getInternalFrameListener();
                            if (ifl != null) {
                                ((JInternalFrame) fWindow).addInternalFrameListener(ifl);
                            }
                        }

                        log.info("Opened widget: " + widget.getTitle());

                    } else {
                        log.info("Null widget - no window opened");
                    }

                } catch (Throwable t) {
                    Application.getWindowManager().showError("Trouble making widget", t);
                }
            }
        };
    }

    /**
     * Subclasses can override to install a real listener
     * Null here
     *
     * @return
     */
    protected InternalFrameListener getInternalFrameListener() {
        return null;
    }

    /**
     * Make this action only respond to double cliks
     * Obvioulsy not valid if used in a popup / menu but useful
     * for buttons / jlists
     *
     * @param onlydc if true then only responds to double clicks
     *               Default is OFF (i.e single click)
     */
    public void setDoubleClickOnly(boolean onlydc) {
        this.fOnlyDc = onlydc;
    }

    public void mouseClicked(MouseEvent e) {

        if (fOnlyDc == false) {
            e.consume();

            return;
        }

        if (e.getClickCount() == 2) {

            //log.debug("Doing double click");
            fOnlyDc = false;

            actionPerformed(new ActionEvent(e.getSource(), e.getID(), this.getId(),
                    e.getModifiers()));

            fOnlyDc = true;
        }
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }
}    // End LongWidgetAction

/*--- Formatted in Sun Java Convention Style on Fri, Nov 15, '02 ---*/
