/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.actions;

import edu.mit.broad.genome.parsers.AuxUtils;
import edu.mit.broad.xbench.core.Widget;

import javax.swing.*;
import java.io.File;

/**
 * Functions:
 * 1) Acts as a proxy for the real FileAction.
 * <p/>
 * 2) If the Widget action is an ObjectAction, and a File were specified, this class parses the File and gives the
 * resulting object to the ObjectAction.
 * Doesnt parse the File until the popup option is actually selected.
 * <p/>
 * Note that typically actions are used via ActionFactory, so client code may not need to explicitly ever use
 * this class
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ProxyFileAction extends WidgetAction {

    private FileAction fAction;
    private File[] fFiles;

    /**
     * Class Constructor.
     *
     * @param action
     */
    public ProxyFileAction(final FileAction action, File file) {

        if (action == null) {
            throw new IllegalArgumentException("Param action cannot be null");
        }

        if (file == null) {
            throw new IllegalArgumentException("Param file cannot be null");
        }

        this.fAction = action;

        // @note
        file = AuxUtils.getBaseFileFromAuxFile(file);

        this.fFiles = new File[]{file};

        this.putValue(Action.NAME, getName());
        this.putValue(Action.SMALL_ICON, getIcon());

        Object desc = fAction.getValue(Action.SHORT_DESCRIPTION);

        if (desc != null) {
            this.putValue(Action.SHORT_DESCRIPTION, desc);
        }
    }

    /**
     * Class constructor
     *
     * @param action
     * @param files
     */
    public ProxyFileAction(FilesAction action, File[] files) {

        if (action == null) {
            throw new IllegalArgumentException("Param action cannot be null");
        }

        if (files == null) {
            throw new IllegalArgumentException("Param files cannot be null");
        }

        this.fAction = action;
        this.fFiles = files;

        this.putValue(Action.NAME, getName());
        this.putValue(Action.SMALL_ICON, getIcon());

        Object desc = fAction.getValue(Action.SHORT_DESCRIPTION);

        if (desc != null) {
            this.putValue(Action.SHORT_DESCRIPTION, desc);
        }
    }

    public String getName() {
        return fAction.getName();
    }

    public Icon getIcon() {
        return fAction.getIcon();
    }

    public String getDescription() {
        return fAction.getDescription();
    }

    public Widget getWidget() throws Exception {

        if (fAction instanceof FilesAction) {
            ((FilesAction) fAction).setFiles(fFiles);
        } else {
            fAction.setFile(fFiles[0]);
        }

        return fAction.getWidget();
    }

    public String getId() {
        return fAction.getId();
    }

}    // End ProxyFileAction

