/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.actions;

import edu.mit.broad.xbench.core.Widget;

import javax.swing.*;
import java.io.File;

/**
 * proxy for real action
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ProxyFileObjectAction extends LongWidgetAction {

    private FileObjectAction fAction;
    private Object fFileOrObj;

    /**
     * Class Constructor.
     *
     * @param action
     */
    public ProxyFileObjectAction(FileObjectAction action, File file) {

        if (action == null) {
            throw new IllegalArgumentException("Param action cannot be null");
        }

        if (file == null) {
            throw new IllegalArgumentException("Param file cannot be null");
        }

        this.fAction = action;
        this.fFileOrObj = file;

        this.putValue(Action.NAME, getName());
        this.putValue(Action.SMALL_ICON, getIcon());

        Object desc = fAction.getValue(Action.SHORT_DESCRIPTION);

        if (desc != null) {
            this.putValue(Action.SHORT_DESCRIPTION, desc);
        }
    }

    /**
     * Class constructor
     *
     * @param action
     * @param obj
     */
    public ProxyFileObjectAction(FileObjectAction action, Object obj) {

        if (action == null) {
            throw new IllegalArgumentException("Param action cannot be null");
        }

        if (obj == null) {
            throw new IllegalArgumentException("Param obj cannot be null");
        }

        this.fAction = action;
        this.fFileOrObj = obj;

        this.putValue(Action.NAME, getName());
        this.putValue(Action.SMALL_ICON, getIcon());

        Object desc = fAction.getValue(Action.SHORT_DESCRIPTION);

        if (desc != null) {
            this.putValue(Action.SHORT_DESCRIPTION, desc);
        }
    }

    public String getName() {
        return fAction.getName();
    }

    public Icon getIcon() {
        return fAction.getIcon();
    }

    public String getDescription() {
        return fAction.getDescription();
    }

    public Widget getWidget() throws Exception {

        if (fFileOrObj instanceof File) {
            fAction.setFile((File) fFileOrObj);
        } else {
            fAction.setObject(fFileOrObj);
        }

        return fAction.getWidget();
    }

    public String getId() {
        return fAction.getId();
    }
}    // End ProxyFileObjectAction
