/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.actions;

import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.xbench.core.Widget;
import edu.mit.broad.xbench.core.api.Application;

import javax.swing.*;
import java.io.File;

/**
 * Functions:
 * 1) Acts as a proxy for the real ObjectAction.
 * <p/>
 * 2) If the Widget action is an ObjectAction, and a File were specified,
 * his class parses the File and gives the resulting object to the ObjectAction.
 * Lazy loading -> Doesnt parse the File until the popup option is actually selected.
 * <p/>
 * <p/>
 * Note that typically actions are used via ActionFactory, so client code may not need to explicitly ever use
 * this class
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ProxyObjectAction extends LongWidgetAction {

    private ObjectAction fAction;
    private Object fFileOrObj;

    /**
     * Class Constructor.
     *
     * @param action
     */
    public ProxyObjectAction(ObjectAction action, File file) {

        if (action == null) {
            throw new IllegalArgumentException("Param action cannot be null");
        }

        if (file == null) {
            throw new IllegalArgumentException("Param file cannot be null");
        }

        this.fAction = action;
        this.fFileOrObj = file;

        this.putValue(Action.NAME, getName());
        this.putValue(Action.SMALL_ICON, getIcon());

        Object desc = fAction.getValue(Action.SHORT_DESCRIPTION);

        if (desc != null) {
            this.putValue(Action.SHORT_DESCRIPTION, desc);
        }
    }

    /**
     * Class constructor
     *
     * @param action
     * @param obj
     */
    public ProxyObjectAction(ObjectAction action, Object obj) {

        if (action == null) {
            throw new IllegalArgumentException("Param action cannot be null");
        }

        if (obj == null) {
            throw new IllegalArgumentException("Param obj cannot be null");
        }

        this.fAction = action;
        this.fFileOrObj = obj;

        this.putValue(Action.NAME, getName());
        this.putValue(Action.SMALL_ICON, getIcon());

        Object desc = fAction.getValue(Action.SHORT_DESCRIPTION);

        if (desc != null) {
            this.putValue(Action.SHORT_DESCRIPTION, desc);
        }
    }

    public String getName() {
        return fAction.getName();
    }

    public Icon getIcon() {
        return fAction.getIcon();
    }

    public String getDescription() {
        return fAction.getDescription();
    }

    public Widget getWidget() {

        try {

            Object obj;

            if (fFileOrObj instanceof File) {
                File f = (File) fFileOrObj;
                log.debug("Reading file for action: " + f);
                obj = ParserFactory.read(f, true);
            } else {
                obj = fFileOrObj;
            }

            if (obj != null) {
                fAction.setObject(obj);
                return fAction.getWidget();
            } else {
                return null; // parsing cancelled
            }


        } catch (Exception e) {
            /*
            // somtimes cancelled is false even though it was cancelled
            if (e instanceof java.io.InterruptedIOException) {
                log.info("Data import was cancelled");
                return null;
            }
            */
            Application.getWindowManager().showError("Trouble making Proxy", e);
        }

        return null;
    }

    public String getId() {
        return fAction.getId();
    }
}    // End ProxyObjectAction
