/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.actions;

import edu.mit.broad.xbench.actions.ext.BrowserAction;
import edu.mit.broad.xbench.core.api.Application;
import edu.mit.broad.xbench.prefs.XPreferencesFactory;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.net.URI;
import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Feb 3, 2005
 *         Time: 3:19:07 PM
 */
public class ShowAppRuntimeHomeDirAction extends BrowserAction {

    /**
     * Class constructor
     *
     * @param name
     */
    public ShowAppRuntimeHomeDirAction(final String name) {
        super(name, "Show runtime home directory of this application", null, XPreferencesFactory.kAppRuntimeHomeDir);
    }

    public void actionPerformed(ActionEvent evt) {
        super.setPath(XPreferencesFactory.kAppRuntimeHomeDir.getPath()); // might have changed since init
        try {

            if (getPath() == null)
                throw new NullPointerException("null path associated with BrowserAction");

            URI homeDirURI = new File(getPath()).toURI();
            // we need to add an (empty) authority designator for compatibility with all platforms
            // (mac requires an authority field in file URIs, windows does not)
            // the resulting URI will have the form "file://<absolute path>"
            homeDirURI = new URI(homeDirURI.getScheme(), "", homeDirURI.getPath(), null, null);
            Desktop.getDesktop().browse(homeDirURI);
            Application.getFileManager().registerRecentlyOpenedURL(getPath());

        } catch (Exception e) {
            Application.getWindowManager().showError("Could not launch browser", e);
        }
    }

}    // End ShowAppRuntimeHomeDirAction