/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.actions;


import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.swing.SwingWorker;
import org.apache.log4j.Logger;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.util.Properties;


/**
 * An I18N-aware Action that self-configures when initialized
 * in the context of a UIRes container. It can then be attached
 * to toolbuttons, menu items, etc..
 *
 * @author <a href="mailto:kdowney@amberarcher.com">Kyle F. Downey</a>
 * @version $Revision: 1.3 $
 */
public abstract class UIResAction extends AbstractAction {

    protected final Logger log = XLogger.getLogger(UIResAction.class);
    private final Properties params = new Properties();

    public void actionPerformed(final ActionEvent evt) {

        SwingWorker swingWorker = new SwingWorker() {

            public Object construct() {

                try {
                    Runnable r = createTask(evt);
                    r.run();
                    return null;
                } catch (Throwable t) {
                    log.error("Error", t);
                    return null;
                }
            }
        };

        swingWorker.start();
    }

    protected Runnable createTask(ActionEvent evt) {
        return null;
    }

    public abstract String getId();

    public abstract String getName();

    public abstract String getDescription();

    public abstract Icon getIcon();

    public String getParam(String paramName) {

        if (paramName == null) {
            log.warn("paramName is null");
        }

        return params.getProperty(paramName);
    }

    public void setParam(String paramName, String paramValue) {

        if (paramName == null) {
            log.warn("paramName is null");
        }

        if (paramValue == null) {
            log.warn("paramValue is null");
        }

        params.setProperty(paramName, paramValue);
    }

    public static void waitCursor(String taskDesc) {
        //System.out.println(">>>> " + taskDesc);
        //Application.getWindowManager().getRootFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

        // @note disabled Jan 2006 - not surw why useful
        //Application.showWaitCursor(taskDesc);
    }

    public static void defaultCursor() {
        //System.out.println(">>> defaultCurso");
        //Application.getWindowManager().getRootFrame().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        //Application.showDefaultCursor();
    }

}    // End UIResAction

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
