/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.actions;


import edu.mit.broad.genome.XLogger;
import edu.mit.broad.xbench.core.WindowComponent;
import edu.mit.broad.xbench.core.api.Application;
import org.apache.log4j.Logger;

import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;


/**
 * Similar functionality as Xaction, except that here it is possible
 * to optionally only respond to double clicks
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 * @see XAction
 */
public abstract class XDCAction extends edu.mit.broad.xbench.actions.UIResAction implements MouseListener {

    protected final Logger log = XLogger.getLogger(WidgetAction.class);

    /**
     * Indicates whether or not to respond only to double clicks. Default false
     */
    protected boolean fOnlyDoubleClick;

    public XDCAction() {

        super();

        // dont -> class vars not inited yet
        //putValue(Action.NAME, getName());
        //putValue(Action.SMALL_ICON, getIcon());
    }

    public void open(WindowComponent comp) {
        Application.getWindowManager().openWindow(comp);
    }

    /**
     * Subclasses must impl.
     * <p/>
     * Example:<br><pre>
     * public void actionPerformed(final ActionEvent evt) {
     *   if (fOnlyDoubleClick) {
     *       return;
     *   }
     *   // do stuff
     * }
     * <p/>
     * </pre>
     *
     * @param evt    make this action only respond to double cliks
     *               Obvioulsy not valid if used in a popup / menu but useful
     *               for buttons / jlists
     * @param onlydc if true then only responds to double clicks
     *               Default is OFF (i.e single click)
     *               <p/>
     *               make this action only respond to double cliks
     *               Obvioulsy not valid if used in a popup / menu but useful
     *               for buttons / jlists
     * @param onlydc if true then only responds to double clicks
     *               Default is OFF (i.e single click)
     *               <p/>
     *               make this action only respond to double cliks
     *               Obvioulsy not valid if used in a popup / menu but useful
     *               for buttons / jlists
     * @param onlydc if true then only responds to double clicks
     *               Default is OFF (i.e single click)
     *               <p/>
     *               make this action only respond to double cliks
     *               Obvioulsy not valid if used in a popup / menu but useful
     *               for buttons / jlists
     * @param onlydc if true then only responds to double clicks
     *               Default is OFF (i.e single click)
     *               <p/>
     *               make this action only respond to double cliks
     *               Obvioulsy not valid if used in a popup / menu but useful
     *               for buttons / jlists
     * @param onlydc if true then only responds to double clicks
     *               Default is OFF (i.e single click)
     *               <p/>
     *               make this action only respond to double cliks
     *               Obvioulsy not valid if used in a popup / menu but useful
     *               for buttons / jlists
     * @param onlydc if true then only responds to double clicks
     *               Default is OFF (i.e single click)
     *               <p/>
     *               make this action only respond to double cliks
     *               Obvioulsy not valid if used in a popup / menu but useful
     *               for buttons / jlists
     * @param onlydc if true then only responds to double clicks
     *               Default is OFF (i.e single click)
     */

    /**
     * make this action only respond to double cliks
     * Obvioulsy not valid if used in a popup / menu but useful
     * for buttons / jlists
     *
     * @param onlydc if true then only responds to double clicks
     * Default is OFF (i.e single click)
     */
    public void setDoubleClickOnly(boolean onlydc) {
        this.fOnlyDoubleClick = onlydc;
    }

    public void mouseClicked(MouseEvent e) {

        if (fOnlyDoubleClick == false) {
            e.consume();

            return;
        }

        if (e.getClickCount() == 2) {
            fOnlyDoubleClick = false;

            //log.debug("Doing double click");
            actionPerformed(new ActionEvent(e.getSource(), e.getID(), this.getId(),
                    e.getModifiers()));

            fOnlyDoubleClick = true;
        }
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }
}    // End XDCAction

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
