/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.actions.ext;


import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.ext.SystemCall;
import edu.mit.broad.xbench.actions.ExtAction;
import edu.mit.broad.xbench.prefs.FilePreference;
import edu.mit.broad.xbench.prefs.XPreferencesFactory;

import javax.swing.*;
import java.awt.event.ActionEvent;


/**
 * Action to launch Adobe acrobat pdf viewer
 *
 * @author Aravind Subramanian
 */
public class AcrobatAction extends ExtAction {

    /**
     * Class Constructor.
     */
    public AcrobatAction() {

        this.putValue(Action.NAME, getName());
        this.putValue(Action.SMALL_ICON, getIcon());
        this.putValue(Action.SHORT_DESCRIPTION, getDescription());
    }

    /**
     * Class Constructor.
     *
     * @param file
     */
    public AcrobatAction(String path) {

        this();

        this.fPath = path;
    }

    public void actionPerformed(ActionEvent evt) {
        FilePreference pref = XPreferencesFactory.kAcrobat;
        try {
            if (fPath != null) {
                SystemCall.launchExternalApplication(fPath, pref.getFile().getPath());
            } else {
                SystemCall.launchExternalApplication(pref.getFile().getPath());
            }
        } catch (Exception e) {
            showHelpfulExtError(pref, "Adobe Acrobat Reader", e);
        }
    }

    public String getId() {
        return "AcrobatAction";
    }

    public String getName() {
        return "Adobe Acrobat";
    }

    public String getDescription() {
        return "Launch Adobe Acrobat in an External Process";
    }

    public Icon getIcon() {
        return JarResources.getIcon("Pdf.gif");
    }
}    // End AcrobatAction

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
