/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.actions.ext;


import edu.mit.broad.genome.JarResources;
import edu.mit.broad.xbench.actions.ExtAction;
import edu.mit.broad.xbench.core.api.Application;

import javax.swing.Icon;
import javax.swing.Action;
import java.awt.event.ActionEvent;
import java.awt.Desktop;
import java.io.File;
import java.net.URI;

/**
 * Action to launch a web browser
 *
 * @author Aravind Subramanian
 */
public class BrowserAction extends ExtAction {

    private static final String NAME = "Web Browser";
    private static final Icon ICON = JarResources.getIcon("Htm.gif");

    private String fName;
    private String fDesc;
    private Icon fIcon;

    /**
     * Class Constructor.
     */
    public BrowserAction() {
        init(NAME, null, ICON, null);
    }

    /**
     * Class Constructor.
     *
     * @param path
     */
    public BrowserAction(final String path) {
        this(NAME, null, ICON, path);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param icon
     * @param path
     */
    public BrowserAction(final String name, final String desc, final Icon icon, final String path) {
        init(name, desc, icon, path);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param icon
     * @param file
     */
    public BrowserAction(final String name, final String desc, final Icon icon, final File file) {
        String path;
        path = file.toString();
        init(name, desc, icon, path);
    }

    private void init(final String name, final String desc, final Icon icon, final String path) {
        this.fName = name;
        this.fDesc = desc;
        if (this.fDesc == null) {
            this.fDesc = "Launch a Web Browser in an External Process";
        }

        this.fPath = path;

        /*
        if (this.fPath == null) {
            TraceUtils.showTrace();
            log.warn("Null path for browser: " + fPath + " " + fName);
        }
        */

        this.fIcon = icon;
        this.putValue(Action.NAME, getName());
        this.putValue(Action.SMALL_ICON, getIcon());
        this.putValue(Action.SHORT_DESCRIPTION, getDescription());
    }

    /**
     * Action method.  Will launch browser and go to URI described in getPath.
     * Note that doesn't work if path is file path.  This method is overridden in
     * ShowAppRuntimeHomeDirAction and ShowDefaultOutputDirAction.
     * @param evt
     */
    public void actionPerformed(ActionEvent evt) {
        try {

            if (getPath() == null)
                throw new NullPointerException("null path associated with BrowserAction");

            Desktop.getDesktop().browse(new URI(getPath()));
            Application.getFileManager().registerRecentlyOpenedURL(getPath());

        } catch (Exception e) {
            Application.getWindowManager().showError("Could not launch browser", e);
        }
    }

    public String getId() {
        return "BrowserAction";
    }

    public String getName() {
        return fName;
    }

    public String getDescription() {
        return fDesc;
    }

    public Icon getIcon() {
        return fIcon;
    }
}    // End BrowserAction

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
