/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.actions.ext;


import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.ext.SystemCall;
import edu.mit.broad.genome.parsers.AuxUtils;
import edu.mit.broad.xbench.actions.ExtAction;
import edu.mit.broad.xbench.prefs.FilePreference;
import edu.mit.broad.xbench.prefs.XPreferencesFactory;

import javax.swing.*;
import java.awt.event.ActionEvent;

/**
 * Action to launch Microsoft Excel.
 *
 * @author Aravind Subramanian
 */
public class ExcelAction extends ExtAction {

    /**
     * Class Constructor.
     */
    public ExcelAction() {

        this.putValue(Action.NAME, getName());
        this.putValue(Action.SMALL_ICON, getIcon());
        this.putValue(Action.SHORT_DESCRIPTION, getDescription());
    }

    /**
     * Class Constructor.
     *
     * @param path
     */
    public ExcelAction(String path) {

        this();

        this.fPath = path;
    }

    public void actionPerformed(ActionEvent evt) {

        FilePreference pref = XPreferencesFactory.kExcel;
        String path2excel = getExePath(pref);

        try {
            if (fPath != null) {
                fPath = AuxUtils.getBasePathFromAuxPath(fPath);
                //SystemCall.launchExternalApplication(fPath, pref.getFile().getPath());
                SystemCall.launchExternalApplication(fPath, path2excel);
            } else {
                SystemCall.launchExternalApplication(pref.getFile().getPath());
            }
        } catch (Throwable t) {
            showHelpfulExtError(pref, "Microsoft Excel", t);
        }
    }

    public String getId() {
        return "ExcelAction";
    }

    public String getName() {
        return "Excel";
    }

    public String getDescription() {
        return "Launch Microsoft Excel in an External Process";
    }

    public Icon getIcon() {
        return JarResources.getIcon("Xls.gif");
    }
}    // End ExcelAction

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
