/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.actions.ext;


import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.ext.SystemCall;
import edu.mit.broad.xbench.actions.ExtAction;
import edu.mit.broad.xbench.prefs.FilePreference;
import edu.mit.broad.xbench.prefs.XPreferencesFactory;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.io.File;


/**
 * Action to launch native os explorer
 *
 * @author Aravind Subramanian
 */
public class OsExplorerAction extends ExtAction {

    /**
     * Class Constructor.
     */
    public OsExplorerAction() {

        this.putValue(Action.NAME, getName());
        this.putValue(Action.SMALL_ICON, getIcon());
        this.putValue(Action.SHORT_DESCRIPTION, getDescription());
    }


    /**
     * Class Constructor.
     *
     * @param path
     */
    public OsExplorerAction(String path) {

        this();

        this.fPath = path;
    }

    public void actionPerformed(ActionEvent evt) {

        FilePreference pref = XPreferencesFactory.kOsExplorer;
        try {

            // windows exp doesnt seem to work well for files -- launches the "browser etc"
            if (fPath != null) {
                File file = new File(fPath);
                File dir;

                if (file.isDirectory()) {
                    dir = file;
                } else {
                    dir = file.getParentFile();
                }

                SystemCall.launchExternalApplication(dir, pref.getFile().getPath());
            } else {
                SystemCall.launchExternalApplication(pref.getFile().getPath());
            }
        } catch (Throwable t) {
            showHelpfulExtError(pref, "File Explorer", t);
        }
    }

    public String getId() {
        return "OsExplorerAction";
    }

    public String getName() {
        return "File Explorer";
    }

    public Icon getIcon() {
        return JarResources.getIcon("OsExplorer16.gif");
        //return JarResources.getIcon("Open16.gif");
    }

    public String getDescription() {
        return "Launch FileExplorer in an External Process";
    }

}    // End OsExplorerAction

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
