/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.actions.misc_actions;

import edu.mit.broad.genome.swing.GuiHelper;
import edu.mit.broad.genome.swing.dnd.StringTransferable;

import javax.swing.*;
import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

/**
 * Class CopyFileNamesAction
 * <p/>
 * just copy the file names, not the paths. Useful when making ffns etc.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class CopyFileNamesAction extends AbstractAction {

    private FilesSelectable fSel;
    private final CopyFileNamesAction fAction = this;

    /**
     * Class constructor
     *
     * @param selectable
     */
    public CopyFileNamesAction(FilesSelectable selectable) {

        if (selectable == null) {
            throw new IllegalArgumentException("Param selectable cannot be null");
        }

        this.fSel = selectable;

        this.putValue(Action.NAME, "Copy File Name(s)");
        this.putValue(Action.SMALL_ICON, GuiHelper.ICON_COPY16);
        this.putValue(Action.SHORT_DESCRIPTION, "Copy Just the File Name(s) -- and NOTthe file path");
    }

    public void actionPerformed(ActionEvent evt) {

        Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();

        clip.setContents(new StringTransferable(fSel.getSelectedFileNames()), fSel.getSelectedFiles());
    }


    public MyKeyListener createCtrlZKeyListener() {

        return new MyKeyListener();

    }


    private class MyKeyListener implements KeyListener {
        public void keyTyped(KeyEvent e) {
        }

        public void keyReleased(KeyEvent e) {
        }

        /**
         * Needed - perhaps due to, java doesnt natively support the ctrl-copy thing
         */
        public void keyPressed(KeyEvent e) {

            if ((e.getKeyCode() == KeyEvent.VK_Z) && e.isControlDown()) {

                fAction.actionPerformed(null);
            }
        }
    }

}

