/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.actions.misc_actions;


import edu.mit.broad.genome.objects.GeneSetMatrix;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.xbench.actions.FileObjectAction;
import edu.mit.broad.xbench.core.Widget;
import edu.mit.broad.xbench.core.api.Application;

import javax.swing.*;
import java.io.File;

/**
 * Extract the row names of a dataset as a grp file
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */

public class GeneSetMatrix2GeneSetsAction extends FileObjectAction {

    private Object fFileOrObject;

    /**
     * Class constructor
     *
     * @param name
     * @param id
     * @param desc
     */
    public GeneSetMatrix2GeneSetsAction() {
        this.putValue(Action.NAME, getName());
        this.putValue(Action.SMALL_ICON, getIcon());
        this.putValue(Action.SHORT_DESCRIPTION, getDescription()
        );
    }

    /**
     * Only works on Template files
     *
     * @param file
     */
    public void setFile(File file) {
        this.fFileOrObject = file;
    }

    /**
     * Only works on Template objects
     *
     * @param obj
     */
    public void setObject(Object obj) {
        this.fFileOrObject = obj;

    }

    public Widget getWidget() {

        // the load is non-blocking so better to use a null widget mechanism
        Widget widget = null;

        try {

            if (fFileOrObject != null) {
                GeneSetMatrix gm;
                if (fFileOrObject instanceof File) {
                    gm = (GeneSetMatrix) ParserFactory.read((File) fFileOrObject);
                } else if (fFileOrObject instanceof GeneSetMatrix) {
                    gm = (GeneSetMatrix) fFileOrObject;
                } else {
                    throw new IllegalArgumentException("Only GeneSetMatrix or File Objects allowed. Got: " + fFileOrObject);
                }

                ParserFactory.extractGeneSets(gm);

                Application.getWindowManager().showMessage("Successfully created a GeneSet from the Dataset " + gm.getName() + " into: " + gm.getNumGeneSets() + " gene sets");

            } else {
                Application.getWindowManager().showMessage("No file or object to work on was specified");
            }

        } catch (Throwable t) {
            Application.getWindowManager().showError("Error creating GeneSets from GeneSetMatrix", t);
        }

        return widget;
    }

    public String getId() {
        return "GeneSetMatrix2GeneSetsAction";
    }

    public String getName() {
        return "=> Extract GeneSets from the GeneSetMatrix";
    }

    public Icon getIcon() {
        return null;
    }

    public String getDescription() {
        return "Extract the GeneSets from the GeneSetMatrix";
    }
}    // End GeneSetMatrix2GeneSetsAction

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
