/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.actions.misc_actions;


import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.xbench.actions.FileObjectAction;
import edu.mit.broad.xbench.core.Widget;
import edu.mit.broad.xbench.core.api.Application;

import javax.swing.*;
import java.io.File;

/**
 * force reloading a file, ignoring if it has been cached or not
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */

public class LoadAction extends FileObjectAction {

    private Object fFileOrObject;

    private boolean fUseCache;

    private static final Icon LOAD_ICON = JarResources.getIcon("Parser16.gif");
    private static final Icon RELOAD_ICON = JarResources.getIcon("Refresh16_2.gif");

    /**
     * Class constructor
     *
     * @param name
     * @param id
     * @param desc
     */
    public LoadAction(boolean useCache) {
        this.putValue(Action.NAME, getName());
        this.putValue(Action.SMALL_ICON, getIcon());
        this.putValue(Action.SHORT_DESCRIPTION, getDescription());

        this.fUseCache = useCache;
    }

    public void setFile(File file) {
        this.fFileOrObject = file;
    }

    public void setObject(Object obj) {
        this.fFileOrObject = obj;
    }

    public Widget getWidget() {

        // the load is non-blocking so better to use a null widget mechanism
        Widget widget = null;

        ProgressMonitorInputStream pis = null;

        try {

            if (fFileOrObject != null) {
                File sourceFile;

                if (fFileOrObject instanceof File) {
                    sourceFile = (File) fFileOrObject;
                } else {
                    sourceFile = ParserFactory.getCache().getSourceFile(fFileOrObject);
                }

                /*
                String msg;

                if (fUseCache) {
                    msg = "Loading data from: " + sourceFile.getPath();
                } else {
                    msg = "Reloading data from: " + sourceFile.getPath();
                }
                */

                //log.debug("loading data from: " + sourceFile);
                PersistentObject pob = ParserFactory.read(sourceFile, fUseCache);
                if (pob != null) {
                    if (fUseCache) {
                        log.info("Loaded: " + pob.getName());
                    } else {
                        Application.getWindowManager().showMessage("<html><body><b>Successfully reloaded: " + pob.getName() + "</b><br>From file: " + sourceFile + "</body></html>");
                    }
                } else {
                    log.info("Cancelled data import!!");
                }

            } else {
                Application.getWindowManager().showMessage("No file or object to load/reload was specified");
            }

        } catch (Throwable t) {
            if ((pis != null) && (t instanceof java.io.InterruptedIOException)) {
                // suppress --> pis cancelled
            } else {
                Application.getWindowManager().showError("Error loading data from file", t);
            }
        }

        return widget;
    }

    public String getId() {
        if (fUseCache) {
            return "LoadAction";
        } else {
            return "ForceReloadAction";
        }
    }

    public String getName() {
        if (fUseCache) {
            return "Import data";
        } else {
            return "Force data reload";
        }
    }

    public Icon getIcon() {
        if (fUseCache) {
            return LOAD_ICON;
        } else {
            return RELOAD_ICON;
        }
    }

    public String getDescription() {
        if (fUseCache) {
            return "Load data from source filee";
        } else {
            return "Reload data from source file overwritting the cache";
        }
    }

}    // End ForceReloadAction

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
