/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.core;


import edu.mit.broad.genome.TraceUtils;
import edu.mit.broad.genome.swing.GuiHelper;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * A NamingConventions Dialog window. Lays out components.
 * <p/>
 * What is the difference between this class and DialogDescriptor?
 * Mainly that this creates a window (i.e a JInternalFrame) that can
 * be around til user chooses to close, rather than a an option pane
 * based input which is usually modal and needs to be dismissed
 * after the message / inout is complete.
 * Example of use - stack trace displayed upon double-clicking the
 * log console.
 * <p/>
 * Only meant for use with JDesktop panes.
 * <p/>
 * A Widget implemntation that simply dumps data  in a comp
 * Why is this needed? Some methods need to return a Widget. If these
 * methods experience an error, sometimes easier to simply still return
 * a widget (i.e this error widget) with a description of the error
 * rather than poping up a option pane.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class DialogWidget extends JPanel implements Widget {

    private final String fTitle;
    private Icon fIcon = GuiHelper.ICON_OPTIONPANE_INFO16;    // default

    /**
     * Class Constructor.
     */
    public DialogWidget(String title, JComponent comp, JButton[] buttons, boolean addOkButton, boolean centralButtonLayout) {

        super();

        this.fTitle = title;

        this.setLayout(new BorderLayout());
        this.add(comp, BorderLayout.CENTER);

        if (addOkButton) {
            JButton bOk = createOkButton();
            JButton[] butts = new JButton[buttons.length + 1];
            int i = 0;

            for (i = 0; i < buttons.length; i++) {
                butts[i] = buttons[i];
            }

            butts[i] = bOk;
            if (centralButtonLayout) {
                this.add(GuiHelper.Button.createCentralButtonPanel(butts), BorderLayout.SOUTH);
            } else {
                this.add(GuiHelper.Button.createButtonPanel(butts), BorderLayout.SOUTH);
            }
        } else {
            if (centralButtonLayout) {
                this.add(GuiHelper.Button.createCentralButtonPanel(buttons), BorderLayout.SOUTH);
            } else {
                this.add(GuiHelper.Button.createButtonPanel(buttons), BorderLayout.SOUTH);
            }
        }

    }

    /**
     * Class Constructor.
     *
     * @param title
     * @param comp
     * @param button
     */
    public DialogWidget(String title, JComponent comp, JButton button, boolean addOkButton) {
        this(title, comp, new JButton[]{button}, addOkButton, false);
    }

    /**
     * Class Constructor.
     * JIF with specified component and NO buttons
     *
     * @param title
     * @param comp
     */
    public DialogWidget(String title, JComponent comp) {

        this.fTitle = title;

        this.setLayout(new BorderLayout());
        this.add(comp, BorderLayout.CENTER);
    }

    /**
     * Class Constructor.
     *
     * @param title
     * @param icon
     * @param comp
     */
    public DialogWidget(String title, Icon icon, JComponent comp) {

        this.fTitle = title;
        this.fIcon = icon;

        this.setLayout(new BorderLayout());
        this.add(comp, BorderLayout.CENTER);
    }

    /**
     * Class Constructor.
     * JIF with JTextArea in which the specified content is displayed
     *
     * @param title
     * @param content
     */
    public DialogWidget(String title, String content) {

        this.fTitle = title;

        JTextArea ta = new JTextArea(content);

        ta.setEditable(false);
        this.setLayout(new BorderLayout());
        this.add(new JScrollPane(ta), BorderLayout.CENTER);
        this.add(GuiHelper.Button.createButtonPanel(new JButton[]{createOkButton()}),
                BorderLayout.SOUTH);
    }

    /**
     * Class Constructor.
     * Ditto as above except appends specified throwables stack trace to the content
     *
     * @param title
     * @param content_start
     * @param t
     */
    public DialogWidget(String title, String content_start, Throwable t) {
        this(title, trace(content_start, t));
    }

    public boolean allowClose() {
        return true;
    }

    public JComponent getWindowUI() {
        return this;
    }

    public String getTitle() {
        return fTitle;
    }

    /*
    public void setCentralButtonPanelLayout() {
        JPanel panel = new JPanel();
        panel.add()
        this.add(GuiHelper.Button.createButtonPanel(new JButton[]{createOkButton()}),
                 BorderLayout.SOUTH);
    }
    */

    public Icon getIcon() {
        return fIcon;
    }

    public JMenuBar getJMenuBar() {
        return Widget.EMPTY_MENU_BAR;
    }

    /**
     * Button when clicked closes this window
     *
     * @return
     */
    private static JButton createOkButton() {

        JButton bOk = new JButton("OK");

        bOk.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                TraceUtils.showTrace();
                //Application.getCurrentWindow().close();
            }
        });

        return bOk;
    }

    private static String trace(String content_start, Throwable t) {

        StringBuffer buf = new StringBuffer(content_start);

        if (t != null) {
            buf.append("\n\n---- Stack Trace ----\n");
            buf.append(TraceUtils.getAsString(t));
        } else {
            buf.append("\n\n No stack trace available");
        }

        return buf.toString();
    }
}    // End ErrorWidget

/*--- Formatted in Sun Java Convention Style on Fri, Nov 15, '02 ---*/
