/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.core;


import edu.mit.broad.xbench.core.api.Application;

import javax.swing.*;
import java.awt.*;

/**
 * WindowComponent wrapper around a DialogWidget / ErrorWidget
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class SimpleWindow implements WindowComponent {

    private final Widget fWidget;

    /**
     * Class Constructor.
     * centralButtonlayout -> buttos are NOT right justified
     */
    public SimpleWindow(String title, JComponent comp, JButton[] buttons,
                        boolean addOkButton, boolean centralButtonLayout) {

        super();

        this.fWidget = new DialogWidget(title, comp, buttons, addOkButton, centralButtonLayout);
    }

    /**
     * Class Constructor.
     *
     * @param title
     * @param comp
     * @param button
     */
    public SimpleWindow(String title, JComponent comp, JButton button, boolean addOkButton) {
        this(title, comp, new JButton[]{button}, addOkButton, false);
    }

    /**
     * Class Constructor.
     * JIF with specified component and NO buttons
     *
     * @param title
     * @param comp
     */
    public SimpleWindow(String title, JComponent comp) {
        this.fWidget = new DialogWidget(title, comp);
    }

    /**
     * Class Constructor.
     *
     * @param title
     * @param icon
     * @param comp
     */
    public SimpleWindow(String title, Icon icon, JComponent comp) {
        this.fWidget = new DialogWidget(title, icon, comp);
    }

    /**
     * Class Constructor.
     * JIF with JTextArea in which the specified content is displayed
     *
     * @param title
     * @param content
     */
    public SimpleWindow(String title, String content) {
        this.fWidget = new DialogWidget(title, content);
    }

    /**
     * Open and display the dialog window
     */
    public void open() {
        Application.getWindowManager().openWindow(this);
    }

    public void open(Dimension d) {
        Application.getWindowManager().openWindow(this, d);
    }

    public JComponent getWindowUI() {
        return fWidget.getWindowUI();
    }

    public boolean allowClose() {
        return fWidget.allowClose();
    }

    public String getTitle() {
        return fWidget.getTitle();
    }

    public Icon getIcon() {
        return fWidget.getIcon();
    }

}    // End DialogWindow

/*--- Formatted in Sun Java Convention Style on Fri, Nov 15, '02 ---*/
