/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.core;


import javax.swing.*;


/**
 * A wrapper for a JComponent that can be hosted
 * as a document window in an MDI WindowManager.
 *
 * @author Kyle Downey (Amber Archer Consulting)
 * @version $Revision: 1.3 $
 */
public interface WindowComponent {

    /**
     * Gets the visual representation of the component.
     */
    public JComponent getWindowUI();

    /**
     * Gets a human-readable title for the component
     * contained in the window. May return null if none
     * makes sense, and the Window manager will provide
     * a default, such as Untitled3.
     */
    public String getTitle();

    /**
     * Callback made from the WindowManager to the component
     * when the user tries to close the window manually or
     * through a menu function like "Window|Close All." This
     * lets components do things like check their internal
     * state and prompt whether the user wants to save before
     * closing, not save, or cancel the close.
     *
     * @return false if the user intervenes and cancels the
     *         close operation
     */
    public boolean allowClose();

    /**
     * Whether or not to allow the jinternal frame to be
     * extracted and externalized in a JFrame
     * Dont see why its needed -> any harm in allowing all windows to be
     * externalized?
     */

    //public boolean allowExternalization();
    public Icon getIcon();
}

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
