package edu.mit.broad.xbench.core.api;

import com.jidesoft.dialog.BannerPanel;
import edu.mit.broad.genome.JarResources;

import javax.swing.*;
import javax.swing.border.BevelBorder;
import java.awt.*;

/**
 * @author Aravind Subramanian
 *         Date: Jan 7, 2006
 *         Time: 10:49:13 PM
 */
public interface DialogType {

    public static final DialogType FileNameRequest = new FileNameRequest();

    public static final DialogType ParamChooserPanel = new ParamChooserPanel();

    public static final DialogType ReadMessageAndProceeedPanel = new ReadMessageAndProceeedPanel();

    public JComponent createBannerPanel(String title);


    static abstract class AbstractDialogType implements DialogType {

        private String fName;

        /**
         * Class constructor
         *
         * @param name
         */
        AbstractDialogType(String name) {
            this.fName = name;
        }

        public String toString() {
            return fName;
        }

        public int hashCode() {
            return fName.hashCode();
        }

        public boolean equals(Object obj) {
            if (obj != null && obj instanceof DialogType) {
                return obj.toString().equals(fName);
            }

            return false;
        }

    }

    class FileNameRequest extends AbstractDialogType {

        FileNameRequest() {
            super("fnr");
        }

        public JComponent createBannerPanel(String title) {
            String desc = "Please specify the information requested below. " +
                    "Output files will be saved in the Project output folder " +
                    Application.getVdbManager().getDefaultOutputDir().getPath() +
                    "A listing of the current folder contents is displayed for reference below";

            BannerPanel headerPanel1 = new BannerPanel(title,
                    desc,
                    JarResources.getImageIcon("SaveDialogBannerBig.gif"));

            headerPanel1.setFont(new Font("Tahoma", Font.PLAIN, 11));
            headerPanel1.setBackground(Color.WHITE);
            headerPanel1.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
            return headerPanel1;
        }

    }

    class ReadMessageAndProceeedPanel extends AbstractDialogType {

        ReadMessageAndProceeedPanel() {
            super("rmp");
        }

        public JComponent createBannerPanel(String title) { // @todo
            String desc = "Heres a message";

            BannerPanel headerPanel1 = new BannerPanel(title,
                    desc,
                    JarResources.getImageIcon("param_chooser_dialog_big.gif"));

            headerPanel1.setFont(new Font("Tahoma", Font.PLAIN, 11));
            headerPanel1.setBackground(Color.WHITE);
            headerPanel1.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
            return headerPanel1;
        }

    }

    class ParamChooserPanel extends AbstractDialogType {

        ParamChooserPanel() {
            super("pcp");
        }

        public JComponent createBannerPanel(String title) {
            String desc = "Please enter all required (red) parameters below";

            BannerPanel headerPanel1 = new BannerPanel(title,
                    desc,
                    JarResources.getImageIcon("param_chooser_dialog_big.gif"));

            headerPanel1.setFont(new Font("Tahoma", Font.PLAIN, 11));
            headerPanel1.setBackground(Color.WHITE);
            headerPanel1.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
            return headerPanel1;
        }

    }

} // End class DialogType

