package edu.mit.broad.xbench.core.api;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.xbench.core.StatusBar;
import edu.mit.broad.xbench.explorer.filemgr.XDirChooser;
import edu.mit.broad.xbench.explorer.filemgr.XFileChooser;
import edu.mit.broad.xbench.explorer.filemgr.XTreeDirectoryPane;
import org.apache.log4j.AppenderSkeleton;
import org.apache.log4j.Logger;
import org.apache.log4j.spi.LoggingEvent;

import javax.swing.*;
import java.awt.*;
import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Jan 2, 2006
 *         Time: 8:57:27 PM
 */
public class HeadlessApplication implements Application.Handler {

    private static final Logger klog = XLogger.getLogger(HeadlessApplication.class);

    private final FileManager fFileManager = new HeadlessFileManagerImpl();

    private HeadlessStatusBar fStatusBar;

    private String fName;

    private static final VdbManager fVdbmanager = new VdbManagerImpl("foo");


    /**
     * Class constructor
     *
     * @param name
     */
    public HeadlessApplication(final String name) {
        this.fName = name;
    }

    public String getName() {
        return fName;
    }

    public boolean isFirstExecutionOfNewVersion() {
        return false;
    }

    public FileManager getFileManager() {
        return fFileManager;
    }

    public VdbManager getVdbManager() {
        return fVdbmanager;
    }

    public ToolManager getToolManager() {
        throw new HeadlessException();
    }

    public WindowManager getWindowManager() throws HeadlessException {
        throw new HeadlessException();
    }

    public StatusBar getStatusBar() {
        if (fStatusBar == null) {
            fStatusBar = new HeadlessStatusBar();
        }

        return fStatusBar;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //


    public static class HeadlessStatusBar extends AppenderSkeleton implements StatusBar {

        public JComponent getAsComponent() {
            throw new HeadlessException();
        }

        public void append(final LoggingEvent event) {
        }

        public void close() {
        }

        public boolean requiresLayout() {
            return false;
        }

    }

    /**
     * Class that implements a FileManager in a headless way
     */
    static class HeadlessFileManagerImpl implements FileManager {

        // we need these in addition to the automatic file chooser based mechanism because
        // files can be loaded in in other ways - for example double click of a jlist
        public void registerRecentlyOpenedDir(final File dir) {
        }

        public void registerRecentlyOpenedFile(final File addFile) {
        }

        public void registerRecentlyOpenedURL(final String url) {
        }


        public File createFile(final String name) {
            return new File(Application.getVdbManager().getDefaultOutputDir(), name);
        }

        public File[] getRecentDirs() {
            return new File[]{};
        }

        public XStore getRecentFilesStore() {
            return null;
        }

        public XStore getRecentDirsStore() {
            return null;
        }

        public File getLastDirAccesessed() {
            return null;
        }

        public File getSecondLastBookmarkDirAccesessed() {
            return null;
        }

        public File getDefaultOutputDir() {
            throw new NotImplementedException();
        }

        public XFileChooser getFileChooser() throws HeadlessException {
            throw new HeadlessException();
        }

        public XFileChooser getFileChooser(final javax.swing.filechooser.FileFilter filt) throws HeadlessException {
            throw new HeadlessException();
        }

        public XFileChooser getFileChooser(final javax.swing.filechooser.FileFilter[] filts) throws HeadlessException {
            throw new HeadlessException();
        }

        public XDirChooser getDirChooser(final String approveButtonTxt) {
            throw new HeadlessException();
        }

        public XTreeDirectoryPane createXTreeDirectoryPane(final String rootLocation) throws HeadlessException {
            throw new HeadlessException();
        }

        public XTreeDirectoryPane createXTreeDirectoryPane() throws HeadlessException {
            throw new HeadlessException();
        }

    } // End class HeadlessFileManagerImpl


} // End interface HeadlessAppHandler

// -------------------------------------------------------------------------------------------- //
// ----------------------------- DISPLAYS A MESSAGE INFO, WARNING ETC ------------------------- //
// -------------------------------------------------------------------------------------------- //

/*
public void showError(final String msg, final Throwable t) {
    klog.error(msg, t);
}

public void showError(final Throwable t) {
    klog.error(t);
}

public void showError(final String msg) {
    klog.error(msg);
}

public void showError(final String msg,
                      final Throwable t,
                      final JButton[] customRemedyOptions) {
    klog.error(msg, t);
}

public void showError(final String title,
                      final String contentInHtmlOrText,
                      final JButton[] customRemedyOptions) {
    klog.error(title + "\n" + contentInHtmlOrText);
}

public void showMessage(final String msg) {
    klog.info(msg);
}
*/