package edu.mit.broad.xbench.core.api;

import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.objects.GenesOfInterest;
import edu.mit.broad.genome.utils.SystemUtils;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import edu.mit.broad.xbench.prefs.XPreferencesFactory;
import org.apache.log4j.Logger;

import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Jan 5, 2006
 *         Time: 1:10:01 PM
 */
public class VdbManagerImpl implements VdbManager {

    private String fBuildDate;

    private static final Logger klog = XLogger.getLogger(VdbManagerImpl.class);

    /**
     * Class constructor
     */
    public VdbManagerImpl(final String buildDate) {
        //System.out.println("##### init VdbManagerImpl: " + buildDate);
        this.fBuildDate = buildDate;
    }

    // @note this is the key preference - everything else is relative to this
    // Except for the vdb tweak
    public File getRuntimeHomeDir() {
        return XPreferencesFactory.kAppRuntimeHomeDir;
    }

    public DescriptedFile[] getDescriptedFiles() {
        return new DescriptedFile[]{
                new DescriptedFile(getDefaultOutputDir(), "OUTPUT"),
                new DescriptedFile(getRuntimeHomeDir(), "HOME"),
                new DescriptedFile(getExamplesDir(), "EXAMPLES"),
                new DescriptedFile(getTmpDir(), "TMP")
        };
    }

    public File getTmpDir() {
        return _mkdir(new File(getRuntimeHomeDir(), "tmp"));
    }

    public File getExamplesDir() {
        return _mkdir(new File(getRuntimeHomeDir(), "xexamples"));
    }

    public File getTestDataDir() {
        return _mkdir(new File(getRuntimeHomeDir(), "test"));
    }

    public File getReportsCacheDir() { // @note that the reports cache dir is specific to the version.
        return _mkdir(new File(getRuntimeHomeDir(), "reports_cache_" + fBuildDate));
    }

    // This holds really large files such as unigene, hugo etc
    public File getDatabasesDir() {
        return _mkdir(new File(_tweakAppRuntimeDir_hack(), "databases"));
    }

    public File getChip2ChipDir() {
        return _mkdir(new File(_tweakAppRuntimeDir_hack(), "chip2chip"));
    }

    public File getDefaultOutputDir() {
        File pwd = SystemUtils.getPwd();
        //File out = _mkdir(new File(getRuntimeHomeDir(), "output"));
        String dn = NamingConventions.createNiceEnglishDate_for_dirs();
        return _mkdir(new File(pwd, dn));
    }

    private static File _mkdir(File dir) {
        if (dir.exists() == false) {
            boolean made = dir.mkdir();
            if (!made) {
                klog.fatal("Could not make dir: " + dir);
            } else {
                klog.info("Made Vdb dir JIT: " + dir);
            }
        }

        return dir;
    }

    // the one in the developers src dir if found and in debug mode.
    // else the one in runtime home
    // NO -> idea local vcs size becomes way too large as it slurps in things from there
    // so place elsewhere (this can be avoided by telling intellij to not cvs) but problem
    // is that jar becomes too big. So seperate out annots as an independent download
    // place
    // again no, can exclude for idea and better to place under home always for users -- trouble if you let
    // them change its location
    // again, no: i like the flex of keeping ti someplace meaningful
    // so do the debug thing
    private File _tweakAppRuntimeDir_hack() {

        File dir = new File("Y:/vdb/");
        if (dir.exists()) {
            return dir;
        }

        // @note making the assumption here that anyone running xtools on unix here prefers to get the universal location
        // of the annotation files

        dir = new File("/xchip/projects/vdb/");

        //System.out.println(">>>>>> " + dir + " exists: " + dir.exists());
        if (dir.exists()) {
            return dir;
        }

        return getRuntimeHomeDir();
    }

    public SampleAnnot getDefaultSampleAnnot() {
        throw new NotImplementedException(); // @todo
    }

    public GenesOfInterest getDefaultGenesOfInterest() {
        return null;
        //throw new NotImplementedException(); // @todo
    }

} // End class VdbManagerImpl

