package edu.mit.broad.xbench.core.api;

import edu.mit.broad.genome.Errors;
import edu.mit.broad.xbench.actions.XAction;
import edu.mit.broad.xbench.core.WindowComponent;

import javax.swing.*;
import java.awt.*;
import java.io.File;

/**
 * Class that defines windowing API's.
 * Most will trow a headless exception.
 */
public interface WindowManager {

    // global help for the entire app
    public JButton createHelpButton() throws HeadlessException;

    public void repaintRootWindow() throws HeadlessException;

    public JFrame getRootFrame() throws HeadlessException;

    public Dimension getExpectedWindowSize() throws HeadlessException;

    public double getDesktopWidth() throws HeadlessException;

    public edu.mit.broad.xbench.core.Window openWindow(final WindowComponent wc) throws HeadlessException;

    public edu.mit.broad.xbench.core.Window openWindow(final WindowComponent wc, final Dimension dim) throws HeadlessException;

    // -------------------------------------------------------------------------------------------- //
    // ----------------------------- DISPLAYS A MESSAGE INFO, WARNING ETC ------------------------- //
    // -------------------------------------------------------------------------------------------- //
    public void showError(final String msg) throws HeadlessException;

    public void showError(final Throwable t) throws HeadlessException;

    public void showError(final Errors errors) throws HeadlessException;

    public void showError(final String msg, final Throwable t) throws HeadlessException;

    public void showError(final String title,
                          final String contentInHtmlOrText) throws HeadlessException;

    public void showError(final String msg,
                          final Throwable t,
                          final JButton[] customRemedyOptions) throws HeadlessException;

    public void showError(final String title,
                          final String contentInHtmlOrText,
                          final JButton[] customRemedyOptions) throws HeadlessException;

    // -------------------------------------------------------------------------------------------- //


    public boolean showConfirm(final String msg) throws HeadlessException;

    public boolean showConfirm(final String title, final String msg) throws HeadlessException;

    public void showMessage(final String msg) throws HeadlessException;

    public void showMessage(final String msg, final Component comp);

    public void showMessage(final String msg, final Component comp, final JButton[] customButtons, final boolean addCloseButton);

    public void showMessage(final String title, final String msg) throws HeadlessException;

    public String showInput(final String inputDesc) throws HeadlessException;

    public String showInput(final String inputDesc, final Object initialSelectionValue) throws HeadlessException;

    public DialogDescriptor createDialogDescriptor(final String title, final Component comp, final Action helpAction_opt);

    public DialogDescriptor createDialogDescriptor(final String title, final Component comp);

    // -------------------------------------------------------------------------------------------- //
    // ----------------------------------ACTION RELATED------------------------------------ //
    // -------------------------------------------------------------------------------------------- //

    public JPopupMenu createPopupMenu(final Object obj) throws HeadlessException;

    public XAction createAction(final Class clazz, final File files[]) throws HeadlessException;

    public boolean runDefaultAction(final Object obj);

    public void runModalTool(final VTool tool, final DialogType dt);

} // End interface WindowManager
