/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.heatmap;

import edu.mit.broad.genome.math.ColorScheme;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.ScoredDataset;
import edu.mit.broad.genome.objects.Template;
import org.genepattern.heatmap.image.DisplaySettings;

import java.awt.*;
import java.awt.image.BufferedImage;

/**
 * Factory implementation so that can switch b/w my table and gp's heat maps easily
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */

// not statics so that things are auto-thread safe
public interface GramImager {

    public DisplayState getDisplayState();

    public BufferedImage createBpogImage(final Dataset ds);

    public BufferedImage createBpogImage(final Dataset ds, final DisplaySettings displaySettings);

    public BufferedImage createBpogImage(final Dataset ds, final ColorScheme cs);

    public BufferedImage createBpogImage(final Dataset ds, final Template t, final ColorScheme cs);

    public BufferedImage createBpogImage(final Dataset ds, final Template t, final DisplaySettings set);

    public BufferedImage createBpogImage(final Dataset ds, final Template t, final ColorScheme cs, final DisplaySettings set);

    public BufferedImage createBpogImage(final Dataset ds, final Template t);

    public BufferedImage createBpogImage(final Dataset ds, final Template t, final GeneSet gset);

    public BufferedImage createBpogImage(final Dataset ds, final Template t, final GeneSet gset, final DisplaySettings settings);

    public BufferedImage createBpogImage(final Dataset ds, final GeneSet gset);

    public BufferedImage createBpogImage(final Dataset ds, final GeneSet gset, final DisplaySettings settings);

    // maintains the order in the sds (i.e gset order ignored)
    public BufferedImage createBpogImage(final ScoredDataset sds, final Template t, final GeneSet gset);

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    /**
     * Static inner helper class
     */
    public static class Helper {

        public static DisplaySettings createDisplaySettings_for_gene_sets_heat_map() {
            DisplaySettings ds = new DisplaySettings();
            ds.rowSize = 8;
            ds.drawGrid = false;
            ds.showFeatureGridLines = true;
            ds.showSampleGridLines = true;
            ds.gridLinesColor = Color.LIGHT_GRAY;
            return ds;
        }

    }

} // End GramImager
