package edu.mit.broad.xbench.heatmap;

import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.math.ColorScheme;
import edu.mit.broad.genome.objects.*;
import org.genepattern.data.expr.IExpressionData;
import org.genepattern.heatmap.image.DisplaySettings;
import org.genepattern.heatmap.image.HeatMap;

import java.awt.image.BufferedImage;

/**
 * @author Aravind Subramanian
 *         Date: Dec 6, 2005
 *         Time: 2:37:42 PM
 */
public class GramImagerImpl implements GramImager {

    private DisplayState fDisplayState;

    /**
     * Class constructor
     */
    public GramImagerImpl() {
        this(null);
    }

    /**
     * Class constructor
     *
     * @param state
     */
    public GramImagerImpl(final DisplayState state) {
        if (state == null) {
            this.fDisplayState = new DisplayState(); // default
        } else {
            this.fDisplayState = state;
        }

    }

    public BufferedImage createBpogImage(final Dataset ds, final DisplaySettings displaySettings) {
        return _xcoreCreateBpogImage(ds, null, null, displaySettings);
    }

    public BufferedImage createBpogImage(final Dataset ds) {
        return _xcoreCreateBpogImage(ds, null, null, null);
    }

    public BufferedImage createBpogImage(final Dataset ds, final ColorScheme cs) {
        return _xcoreCreateBpogImage(ds, null, cs, null);
    }

    public BufferedImage createBpogImage(final Dataset ds, final Template t) {
        return _xcoreCreateBpogImage(ds, t, null, null);
    }

    public BufferedImage createBpogImage(final Dataset ds, final Template t, final ColorScheme cs, final DisplaySettings set) {
        return _xcoreCreateBpogImage(ds, t, cs, set);
    }

    public BufferedImage createBpogImage(final Dataset ds, final Template t, final DisplaySettings set) {
        return _xcoreCreateBpogImage(ds, t, null, set);
    }

    public BufferedImage createBpogImage(final Dataset ds, final Template t, final ColorScheme cs) {
        return _xcoreCreateBpogImage(ds, t, cs, null);
    }

    public BufferedImage createBpogImage(final Dataset ds, final Template t, final GeneSet gset) {
        return createBpogImage(ds, t, gset, null);
    }

    public BufferedImage createBpogImage(final Dataset ds, final Template t, final GeneSet gset, final DisplaySettings settings) {
        Dataset sub = new DatasetGenerators().extractRows(ds, gset);
        return _xcoreCreateBpogImage(sub, t, null, settings);
    }

    public BufferedImage createBpogImage(final Dataset ds, final GeneSet gset) {
        return createBpogImage(ds, null, gset, null);
    }

    public BufferedImage createBpogImage(final Dataset ds, final GeneSet gset, final DisplaySettings settings) {
        return createBpogImage(ds, null, gset, settings);
    }

    // maintains the order in the sds (i.e gset order ignored)
    public BufferedImage createBpogImage(final ScoredDataset sds, final Template t, final GeneSet gset) {
        return createBpogImage(new DatasetGenerators().extractRowsSorted(sds, gset), t);
    }

    public DisplayState getDisplayState() {
        return fDisplayState;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    private DisplaySettings _createDefaultDisplaySettings(final Dataset origDs,
                                                          final ColorScheme colorScheme_opt
    ) {


        final DisplaySettings set = new DisplaySettings();

        set.drawGrid = true;
        set.rowSize = DisplayState.DEFAULT_CELL_HEIGHT;
        set.columnSize = DisplayState.DEFAULT_CELL_HEIGHT;

        set.sampleAnnonationsHeight = 15;
        set.drawGrid = true; // i prefer this for gsea
        set.showFeatureGridLines = true; // i prefer this for gsea
        set.showSampleGridLines = true; // i prefer this for gsea

        set.drawRowDescriptions = origDs.getAnnot().getFeatureAnnot().hasNativeDescriptions();

        set.drawRowNames = true;

        if (fDisplayState != null && fDisplayState.getColorScheme() != null) {
            set.colorConverter = fDisplayState.getColorScheme();
        }

        if (colorScheme_opt != null) {
            set.colorConverter = GPWrappers.createColorScheme(origDs, colorScheme_opt);
        }

        return set;
    }

    private BufferedImage _xcoreCreateBpogImage(final Dataset origDs,
                                                final Template origT_opt,
                                                final ColorScheme colorScheme_opt,
                                                DisplaySettings displaySettings_opt
    ) {

        if (origDs == null) {
            throw new IllegalArgumentException("Param origDs cannot be null");
        }


        if (displaySettings_opt == null) {
            displaySettings_opt = _createDefaultDisplaySettings(origDs, colorScheme_opt);
        }

        final IExpressionData data = GPWrappers.createIExpressionData(origDs);

        return HeatMap.createImage(data, displaySettings_opt,
                GPWrappers.createSampleAnnotator(origDs, origT_opt), GPWrappers.createFeatureAnnotator(origDs));
    }


} // End GramImagerImpl

