/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.prefs;

import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;

/**
 * @author Aravind Subramanian
 *         Date: Mar 14, 2003
 *         Time: 4:36:08 PM
 */
abstract class AbstractPreference implements Preference {

    private String fName;
    private String fDesc;
    private Object fDefault;

    private boolean fDebug;

    private boolean fNeedsRestart;

    protected static final Logger klog = XLogger.getLogger(AbstractPreference.class);


    /**
     * Must manually call init() if this form of the protected constructor is used by
     * a subclass
     */
    protected AbstractPreference() {
    }

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     * @param def
     */
    protected AbstractPreference(final String name, final String desc, final Object def, final boolean isDebug, final boolean needsRestart) {
        init(name, desc, def, isDebug, needsRestart);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     * @param def
     * @param isDebug
     */
    protected void init(final String name, final String desc, final Object def, final boolean isDebug, final boolean needsRestart) {
        if (name == null) {
            throw new IllegalArgumentException("Parameter name cannot be null");
        }

        if (desc == null) {
            throw new IllegalArgumentException("Parameter desc cannot be null");
        }

        if (def == null) {
            throw new IllegalArgumentException("Parameter def cannot be null");
        }

        this.fName = name;
        this.fDesc = desc;
        this.fDefault = def;
        this.fDebug = isDebug;
        this.fNeedsRestart = needsRestart;
    }

    public String getName() {
        return fName;
    }

    public String getDesc() {
        return fDesc;
    }

    public Object getDefault() {
        return fDefault;
    }

    public boolean isDebugPreference() {
        return fDebug;
    }

    public boolean needsRestart() {
        return fNeedsRestart;
    }

    protected void _setValueOfPref2SelectionComponentValue(Object value) {
        // Object value = getValue(); <- this gets the prefs value and not the components!
        klog.debug("Saving pref: " + getName() + " getValue: " + value);
        if (value != null) {
            kPrefs.put(getName(), value.toString());
        }

    }

} // End class AbstractPreference
