package edu.mit.broad.xbench.project;

import edu.mit.broad.genome.XLogger;
import net.java.swingfx.waitwithstyle.CancelableAdaptee;
import net.java.swingfx.waitwithstyle.CancelableProgressPanel;
import net.java.swingfx.waitwithstyle.PerformanceInfiniteProgressPanel;

import javax.swing.*;

/**
 * @author Aravind Subramanian
 *         Date: Jan 7, 2006
 *         Time: 12:16:23 AM
 */
public class ProjectLoggers {

    private static final org.apache.log4j.Logger klog = XLogger.getLogger(ProjectLoggers.class);

    /**
     * Writer class impl
     */
    public static class JProgressBarProjectLogger implements ProjectLogger {

        private JProgressBar fBar;

        private CancelableAdaptee veil;

        /**
         * Class constructor
         *
         * @param bar
         */
        public JProgressBarProjectLogger() {

            this.fBar = new JProgressBar();
            fBar.setStringPainted(true);

            this.fBar.setBorderPainted(true);
            this.fBar.setEnabled(false);

            //Set up the veil
            this.veil = new CancelableProgressPanel("Loading project ...");
        }

        public void startVeil(final JRootPane rootPane) {
            //When using the glass pane approach, restore the old one (jgoodies or something elst may be using it)
            //Component oldGlassPane = rootPane.getGlassPane();
            //LookupWorkerRunnable runnable = new LookupWorkerRunnable(sym, veil, rootPane, oldGlassPane);
            rootPane.setGlassPane(veil.getComponent());
            rootPane.revalidate();

            veil.start();
        }

        public void stopVeil() {
            veil.stop();
        }

        public void write(final String msg) {
            veil.setText(msg);
            klog.debug(msg);
            fBar.setString(msg);
        }

        public void setEnabled(boolean enabled) {
            fBar.setEnabled(enabled);
        }

        public void setIndeterminate(boolean indet) {
            fBar.setIndeterminate(indet);
        }

    } // End class JProgressBarProjectLogger

    public static class RomainLogger implements ProjectLogger {

        private CancelableAdaptee veil;

        /**
         * Class constructor
         *
         * @param bar
         */
        public RomainLogger() {
            //Set up the veil

        }

        public void startVeil(final JRootPane rootPane) {
            this.veil = new CancelableProgressPanel("Loading project ...");

            // supposedly better perf - i havent had] a problem with the regular one
            //this.veil = new PerformanceInfiniteProgressPanel();

            rootPane.setGlassPane(veil.getComponent());
            rootPane.revalidate();

            veil.start();
        }

        public void stopVeil() {
            if (veil != null) {
                veil.stop();
            }
        }

        public void write(final String msg) {
            if (veil != null) {
                veil.setText(msg);
                klog.debug(msg);
            }
        }

        public void setEnabled(boolean enabled) {
        }

        public void setIndeterminate(boolean indet) {
        }

    } // End class JProgressBarProjectLogger


    public static class TerminalLogger implements ProjectLogger {

        /**
         * Class constructor
         *
         * @param bar
         */
        public TerminalLogger() {
            //Set up the veil
        }

        public void write(final String msg) {
            klog.debug(msg);
        }

        public void setEnabled(boolean enabled) {
        }

        public void setIndeterminate(boolean indet) {
        }

    } // End class JProgressBarProjectLogger

    public static class RomainLoggerLight implements ProjectLogger {

        private CancelableAdaptee veil;

        /**
         * Class constructor
         *
         * @param bar
         */
        public RomainLoggerLight() {
            //Set up the veil

        }

        public void startVeil(final String name, final JRootPane rootPane) {
            this.veil = new CancelableProgressPanel(name);

            // supposedly better perf - i havent had] a problem with the regular one
            this.veil = new PerformanceInfiniteProgressPanel();

            rootPane.setGlassPane(veil.getComponent());
            rootPane.revalidate();

            veil.start();
        }

        public void stopVeil() {
            if (veil != null) {
                veil.stop();
            }
        }

        public void write(final String msg) {
            if (veil != null) {
                veil.setText(msg);
                klog.debug(msg);
            }
        }

        public void setEnabled(boolean enabled) {
        }

        public void setIndeterminate(boolean indet) {
        }

    } // End class JProgressBarProjectLogger

}
