/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.tui;

import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.reports.api.Report;
import edu.mit.broad.genome.utils.DateUtils;

import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Mar 9, 2003
 *         Time: 9:20:41 PM
 */
public class ReportStub {

    private String fName;
    private long fTimestamp;

    private File fReportFile;

    private Report fReport;

    private String fTsLessName;

    /**
     * @param rptFile
     */
    public ReportStub(final File rptFile) {
        this.fReportFile = rptFile;
        this.fName = fReportFile.getName();

        final Object[] rets = NamingConventions.parseReportTimestampFromName(fName);
        this.fTimestamp = ((Long) rets[1]).longValue();

        this.fTsLessName = rets[0].toString();
    }

    public String getName() {
        return fName;
    }

    public String getName_without_ts() {
        return fTsLessName;
    }

    public long getTimestamp() {
        return fTimestamp;
    }

    public File getReportFile() {
        return fReportFile;
    }

    public Report getReport(boolean useCache) throws Exception {
        if (fReport == null) {
            fReport = ParserFactory.readReport(fReportFile, useCache);
        }

        return fReport;
    }

    public String getHourMin() {
        return DateUtils.toHourMin(fTimestamp) + "min";
    }


} // End class ReportStub
