/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.tui;


import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.xbench.actions.WidgetAction;
import edu.mit.broad.xbench.core.Widget;
import xtools.api.Tool;
import xtools.api.param.ParamSet;

import javax.swing.*;


/**
 * @author Aravind Subramanian
 */
public class SingleToolLauncherAction extends WidgetAction {

    private Tool fTool;
    private ParamSet fPSet;
    private String fTitle;
    private Icon fIcon;

    /**
     * Class Constructor.
     *
     * @param tool
     */
    public SingleToolLauncherAction(Tool tool, ParamSet pset, String titleOpt) {
        this(tool, pset, titleOpt, ToolLauncher.ICON);
    }

    /**
     * Class constructor
     *
     * @param tool
     * @param pset
     * @param iconOpt
     */
    public SingleToolLauncherAction(final Tool tool, final ParamSet pset, final String titleOpt, final Icon iconOpt) {

        if (tool == null) {
            throw new IllegalArgumentException("Param tool cannot be null");
        }

        if (pset == null) {
            throw new IllegalArgumentException("Param pset cannot be null");
        }

        this.fTool = tool;
        this.fPSet = pset;
        this.fTitle = titleOpt;
        this.fIcon = iconOpt;

        this.putValue(Action.NAME, getName());
        this.putValue(Action.SMALL_ICON, fIcon);
        this.putValue(Action.SHORT_DESCRIPTION, getDescription());
    }


    public Widget getWidget() {
        // IMP dont place this in the class init are -- cause the app to recursively loop
        setSize(ToolLauncherAction.DEFAULT_DIM.width, ToolLauncherAction.DEFAULT_DIM.height, true);
        return new SingleToolLauncher(fTool, fPSet, true, true, true, true, true, fTitle, SingleToolLauncher.ICON);
    }

    public String getId() {
        return "SingleToolLauncherAction";
    }

    public String getName() {
        return ParseUtils.getLastToken(fTool.getName(), ".");
    }

    public Icon getIcon() {
        return fIcon;
    }

    public String getDescription() {
        return "Set Parameters and Launch Analysis Tools";
    }

}    // End ToolAction

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
