package edu.mit.broad.xbench.tui;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.swing.AutoCompletion;
import edu.mit.broad.genome.swing.GuiHelper;
import org.apache.log4j.Logger;
import xtools.api.Tool;

import javax.swing.*;
import java.awt.*;
import java.util.StringTokenizer;

/**
 * @author Aravind Subramanian
 *         Date: Nov 7, 2005
 *         Time: 5:19:14 PM
 */
public class ToolSelectorComboBox extends JComboBox implements ToolSelectorUI {

    private final Tool[] fTools;
    private final Logger log = XLogger.getLogger(ToolSelectorComboBox.class);

    /**
     * Class Constructor.
     */
    public ToolSelectorComboBox(final Tool[] tools) {
        this.fTools = tools;
        this.setModel(new DefaultComboBoxModel(fTools));
        this.setRenderer(new Renderer());
        AutoCompletion.enable(this);
    }

    public Tool selectTool(final String toolName) {
        if (toolName == null) {
            return null;
        }

        for (int i = 0; i < fTools.length; i++) {
            if (fTools[i].getName().equalsIgnoreCase(toolName)) {
                super.setSelectedItem(fTools[i]);
                return fTools[i];
            }
        }

        //TraceUtils.showTrace(); // happens when last tool is changed
        log.warn("Unknown tool name, and could not select it: " + toolName);

        return null;
    }

    public Component getComponent() {
        return this;
    }

    /**
     *
     */
    public static class Renderer extends DefaultListCellRenderer {


        public Component getListCellRendererComponent(JList list, Object value, int index,
                                                      boolean isSelected, boolean cellHasFocus) {

            // doesnt work properly unless called
            super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);

            //log.debug("value: " + value + " class: " + value.getClass());

            this.setBorder(BorderFactory.createLineBorder(Color.BLACK));
            this.setFont(GuiHelper.FONT_DEFAULT);

            //log.debug("user object: " + node);
            if (value instanceof Tool) {
                Tool tool = (Tool) value;
                StringTokenizer tok = new StringTokenizer(tool.getName(), ".");
                String name = null;
                while (tok.hasMoreTokens()) {
                    name = tok.nextToken();
                }
                this.setText(name);
                this.setIcon(tool.getCategory().getIcon());
                this.setToolTipText(((Tool) value).getDesc());
            } else {
                this.setBorder(null);
                this.setText(value.toString());
            }

            return this;
        }

    }    // End Renderer
}        // End ToolSelectorComboBox
