/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.xbench.xchoosers;

import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.objects.TemplateMode;

import javax.swing.*;
import javax.swing.event.ListDataListener;

/**
 *
 */
public class TemplateComboBoxModel implements ComboBoxModel {

    private DefaultComboBoxModel fModel;

    /**
     * Thios is the no-selection sharing magic
     */
    private int fSelIndex = 0;

    /**
     * Class Constructor.
     *
     * @param model
     */
    public TemplateComboBoxModel(ComboBoxModel model, TemplateMode mode) {
        this.fModel = new DefaultComboBoxModel();

        if (mode == TemplateMode.CONTINUOUS_ONLY) {
            for (int i = 0; i < model.getSize(); i++) {
                Template t = (Template) model.getElementAt(i);
                if (t.isContinuous()) {
                    fModel.addElement(t);
                }
            }
        } else if (mode == TemplateMode.CATEGORICAL_2_CLASS_ONLY) {
            for (int i = 0; i < model.getSize(); i++) {
                Template t = (Template) model.getElementAt(i);
                if ((t.isContinuous() == false) && (t.getNumClasses() == 2)) {
                    fModel.addElement(t);
                }
            }
        } else if (mode == TemplateMode.CATEGORICAL_ONLY) {
            for (int i = 0; i < model.getSize(); i++) {
                Template t = (Template) model.getElementAt(i);
                if (t.isContinuous() == false) {
                    fModel.addElement(t);
                }
            }
        } else {
            for (int i = 0; i < model.getSize(); i++) {
                Template t = (Template) model.getElementAt(i);
                fModel.addElement(t);
            }
        }
    }

    public void addListDataListener(ListDataListener l) {
        fModel.addListDataListener(l);
    }

    public Object getElementAt(int index) {
        return fModel.getElementAt(index);
    }

    public int getSize() {
        return fModel.getSize();
    }

    public void removeListDataListener(ListDataListener l) {
        fModel.removeListDataListener(l);
    }

    /**
     * ComboBoxModel impl.
     *
     * @return
     */
    public Object getSelectedItem() {
        return fModel.getElementAt(fSelIndex);
    }

    /**
     * ComboBoxModel implementation
     */
    public void setSelectedItem(Object obj) {
        this.fSelIndex = fModel.getIndexOf(obj);
    }

} // End class TemplateComboBoxModel
