/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute FeatureAnnotatorPanelle for its
 use, misuse, or functionality.
 */

package org.genepattern.annotation;

import org.genepattern.data.expr.IExpressionData;
import org.genepattern.heatmap.PixelConverter;
import org.genepattern.heatmap.image.FeatureAnnotator;

import javax.swing.*;
import java.awt.*;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * @author Joshua Gould
 */
public class FeatureAnnotatorPanel extends JPanel {

    private SparseClassVector classVector;

    private int widthPerClass = 6;

    private PixelConverter pixelConverter;

    private IExpressionData data;

    private int rowSize;

    private int numAnnotations = 0;

    public FeatureAnnotatorPanel(SparseClassVector _classVector,
                                 IExpressionData _data, int rowSize, final JComponent c) {
        setBackground(Color.WHITE);
        this.classVector = _classVector;
        this.data = _data;
        this.rowSize = rowSize;

        classVector.addListener(new SparseClassVectorListener() {

            public void classChanged() {
                numAnnotations = 0;

                for (int i = 0; i < data.getRowCount(); i++) {
                    List classNumbers = classVector.getClassNumbers(i);

                    if (classNumbers != null) {
                        numAnnotations = Math.max(classNumbers.size(),
                                numAnnotations);
                    }
                }

                c.invalidate();
                c.validate();
                c.doLayout();

                Component a = c.getTopLevelAncestor();
                a.invalidate();
                a.validate();
                a.doLayout();
                repaint();
            }

        });
        pixelConverter = new PixelConverter(this);
        pixelConverter.setRowSize(rowSize);

    }

    public void setRowSize(int i) {
        rowSize = i;
    }

    public Dimension getPreferredSize() {
        return new Dimension(numAnnotations * widthPerClass
                + (numAnnotations + 1) * 2, data.getRowCount() * rowSize);
    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Rectangle bounds = g.getClipBounds();

        int top = pixelConverter.getTopIndex(bounds.y);
        int bottom = pixelConverter.getBottomIndex(bounds.y + bounds.height,
                data.getRowCount());

        for (int i = top; i < bottom; i++) {
            List classNumbers = classVector.getClassNumbers(i);
            if (classNumbers != null) {
                int xStart = 2;
                for (int j = 0; j < classNumbers.size(); j++) {
                    Integer classNumber = (Integer) classNumbers.get(j);
                    g.setColor(classVector.getColor(classNumber));
                    g.fillRect(xStart, i * rowSize, widthPerClass, rowSize);

                    xStart += widthPerClass + 2;
                }
            }
        }

    }

    public void setFeatureAnnotator(FeatureAnnotator fa) {

        Map color2ClassNumberMap = new HashMap();
        int counter = 0;
        for (int i = 0; i < data.getRowCount(); i++) {
            List colors = fa.getColors(data.getRowName(i));
            for (int j = 0; j < colors.size(); j++) {
                Color c = (Color) colors.get(j);
                if (c != null) {
                    Integer classNumber = (Integer) color2ClassNumberMap.get(c);
                    if (classNumber == null) {
                        classNumber = new Integer(counter++);
                        color2ClassNumberMap.put(c, classNumber);
                    }
                }
            }
        }
        SparseClassVector cv = new SparseClassVector();
        for (Iterator it = color2ClassNumberMap.keySet().iterator(); it
                .hasNext();) {
            Color c = (Color) it.next();
            Integer number = (Integer) color2ClassNumberMap.get(c);
            cv.setClass(number, "", c);
        }
        for (int i = 0; i < data.getRowCount(); i++) {
            List colors = fa.getColors(data.getRowName(i));
            for (int j = 0; j < colors.size(); j++) {
                Color c = (Color) colors.get(j);
                if (c != null) {
                    cv.addClass(i, (Integer) color2ClassNumberMap.get(c));
                }
            }
        }
        classVector = cv;
    }

}
