/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute FeatureAnnotatorPanelle for its
 use, misuse, or functionality.
 */

package org.genepattern.annotation;

import org.genepattern.data.expr.IExpressionData;
import org.genepattern.heatmap.PixelConverter;
import org.genepattern.heatmap.image.SampleAnnotator;

import javax.swing.*;
import java.awt.*;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * @author Joshua Gould
 */
public class SampleAnnotatorPanel extends JPanel {

    private SparseClassVector classVector;

    private int heightPerClass = 6;

    private PixelConverter pixelConverter;

    private IExpressionData data;

    private int columnSize;

    private int numAnnotations = 0;

    private int spacer = 2;

    public SampleAnnotatorPanel(SparseClassVector _classVector,
                                IExpressionData _data, int columnSize, final JComponent c) {
        setBackground(Color.WHITE);
        this.classVector = _classVector;
        this.data = _data;
        this.columnSize = columnSize;

        classVector.addListener(new SparseClassVectorListener() {

            public void classChanged() {
                numAnnotations = 0;
                for (int i = 0; i < data.getColumnCount(); i++) {
                    List classNumbers = classVector.getClassNumbers(i);
                    if (classNumbers != null) {
                        numAnnotations = Math.max(classNumbers.size(),
                                numAnnotations);
                    }
                }

                c.invalidate();
                c.validate();
                c.doLayout();

                Component a = c.getTopLevelAncestor();
                a.invalidate();
                a.validate();
                a.doLayout();
                repaint();
            }

        });
        pixelConverter = new PixelConverter(this);
        pixelConverter.setColumnSize(columnSize);

    }

    public void setColumnSize(int i) {
        columnSize = i;
    }

    public Dimension getPreferredSize() {
        return new Dimension(data.getColumnCount() * columnSize, numAnnotations
                * heightPerClass + numAnnotations * spacer);
    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Rectangle bounds = g.getClipBounds();

        int left = pixelConverter.getLeftIndex(bounds.x);
        int right = pixelConverter.getRightIndex(bounds.x + bounds.width, data
                .getColumnCount());

        for (int col = left; col < right; col++) {
            List classNumbers = classVector.getClassNumbers(col);
            if (classNumbers != null) {
                int y = (numAnnotations - 1) * heightPerClass
                        + (numAnnotations - 1) * spacer;
                for (int j = 0; j < classNumbers.size(); j++) {
                    Integer classNumber = (Integer) classNumbers.get(j);
                    g.setColor(classVector.getColor(classNumber));
                    g.fillRect(col * columnSize, y, columnSize, heightPerClass);
                    y = y - heightPerClass - spacer;
                }
            }
        }

    }

    public void setSampleAnnotator(SampleAnnotator fa) {
        Map color2ClassNumberMap = new HashMap();
        int counter = 0;
        for (int i = 0; i < data.getRowCount(); i++) {
            List colors = fa.getColors(data.getRowName(i));
            for (int j = 0; j < colors.size(); j++) {
                Color c = (Color) colors.get(j);
                if (c != null) {
                    Integer classNumber = (Integer) color2ClassNumberMap.get(c);
                    if (classNumber == null) {
                        classNumber = new Integer(counter++);
                        color2ClassNumberMap.put(c, classNumber);
                    }
                }
            }
        }
        SparseClassVector cv = new SparseClassVector();
        for (Iterator it = color2ClassNumberMap.keySet().iterator(); it
                .hasNext();) {
            Color c = (Color) it.next();
            Integer number = (Integer) color2ClassNumberMap.get(c);
            cv.setClass(number, "", c);
        }
        for (int i = 0; i < data.getRowCount(); i++) {
            List colors = fa.getColors(data.getRowName(i));
            for (int j = 0; j < colors.size(); j++) {
                Color c = (Color) colors.get(j);
                if (c != null) {
                    cv.addClass(i, (Integer) color2ClassNumberMap.get(c));
                }
            }
        }
        classVector = cv;
    }

}
