/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

package org.genepattern.clustering.hierarchical;

import java.awt.*;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

public class AtrGtrReader {
    private double maxArrayCorr;

    private Map nodes = new HashMap();

    private double minArrayCorr;

    private String rootArrayNodeId;

    private HashMap id2Index;

    private String[] ids;

    /**
     * @param ids      the ids of the leaf nodes
     * @param filename atr or gtr file name
     * @throws IOException
     */
    public AtrGtrReader(String[] ids, String filename) throws IOException {
        this.ids = ids;
        BufferedReader br = null;
        try {
            br = new BufferedReader(new FileReader(filename));
            parse(br, ids);
        } finally {
            if (br != null) {
                br.close();
            }
        }
    }

    public String[] getIds() {
        return ids;
    }

    private int getIndex(String id) throws IOException {
        Integer i = (Integer) id2Index.get(id);
        if (i == null) {
            throw new IOException("id " + id + " not found");
        }
        return i.intValue();
    }

    void parse(BufferedReader br, String[] ids) throws IOException {
        String line;
        boolean isFirstLine = true;
        Node node = null;
        id2Index = new HashMap();
        for (int i = 0; i < ids.length; i++) {
            id2Index.put(ids[i], new Integer(i));
        }
        while ((line = br.readLine()) != null) {
            StringTokenizer tokens = new StringTokenizer(line, "\t");
            // first token is node id
            String id = tokens.nextToken();
            // next token is left id
            String left = tokens.nextToken();
            // next token is right id
            String right = tokens.nextToken();
            // next token is correlatation
            double corr = Double.parseDouble(tokens.nextToken());

            Color color = null;
            if (tokens.hasMoreTokens()) {
                color = Color.decode(tokens.nextToken());
            }
            // The first node will have the highest correlation
            if (isFirstLine) {
                maxArrayCorr = corr;
                isFirstLine = false;
            }
            Node lnode = null;
            Node rnode = null;
            // check if left is arrayNodes
            // if not then it is a leaf
            if (nodes.containsKey(left)) {
                lnode = (Node) nodes.get(left);
            } else {
                lnode = new Node(left);

                // set index for leaf... all leaves are arrays
                int index = getIndex(left);
                // System.out.println("array leaf: " + a.getId());
                lnode.setIndex(index);
                // System.out.println("array index " + a.getIndex());
                lnode.setMinIndex(index);
                lnode.setMaxIndex(index);
                // put into arrayNodes hash
                nodes.put(left, lnode);
            }

            // check if right is arrayNodes
            // if not then it is a leaf
            if (nodes.containsKey(right)) {
                rnode = (Node) nodes.get(right);
            } else {
                rnode = new Node(right);
                // set index
                // set index for leaf... all leaves are arrays
                int index = getIndex(right);
                rnode.setIndex(index);
                rnode.setMinIndex(index);
                rnode.setMaxIndex(index);
                // put into arrayNodes hash
                nodes.put(right, rnode);
            }

            node = new Node(id, lnode, rnode, corr);
            if (color != null) {
                node.setColor(color);
            }
            // set min and max indexes
            node
                    .setMinIndex(Math.min(rnode.getMinIndex(), lnode
                            .getMinIndex()));
            node
                    .setMaxIndex(Math.max(rnode.getMaxIndex(), lnode
                            .getMaxIndex()));

            // set parents of l and r nodes
            lnode.setParent(node);
            rnode.setParent(node);

            // put node into arrayNodes
            nodes.put(id, node);

        }

        // set min correlation - will be last node parsed
        minArrayCorr = node.getCorrelation();
        // set root node id - last node parsed
        rootArrayNodeId = node.getId();
    }

    public double getMaxArrayCorr() {
        return maxArrayCorr;
    }

    public Map getNodes() {
        return nodes;
    }

    public double getMinArrayCorr() {
        return minArrayCorr;
    }

    public String getRootNodeId() {
        return rootArrayNodeId;
    }
}
