/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

package org.genepattern.clustering.hierarchical;

import java.awt.*;

/**
 * @author Lisa Simirenko
 * @author Joshua Gould
 */
public class Node {

    protected String id = null;

    protected Node left = null;

    protected Node right = null;

    protected Node parent = null;

    protected double index;

    protected double correlation = 1;

    protected Color color = Color.black;

    protected boolean selected = false;

    protected int minIndex;

    protected int maxIndex;

    /**
     * Creates a new instance of Node
     */
    public Node(String name) {
        id = name;
    }

    /**
     * Creates a new instance of Node that is not a leaf (has left and right
     * nodes that are not null). The index will be calculated as the mean of the
     * left and right indexes.
     */
    public Node(String name, Node first, Node last, double corr) {
        id = name;
        left = first;
        right = last;
        correlation = corr;
        setIndex();
    }

    public String getId() {
        return id;
    }

    public void setId(String newid) {
        id = newid;
    }

    public void setParent(Node node) {
        parent = node;
    }

    public void setLeft(Node node) {
        left = node;
    }

    public void setRight(Node node) {
        right = node;
    }

    public Node getParent() {
        if (this.isRoot()) {
            return this;
        }
        return parent;
    }

    public Node getLeft() {
        if (this.isLeaf()) {
            return this;
        }
        return left;
    }

    public Node getRight() {
        if (this.isLeaf()) {
            return this;
        }
        return right;
    }

    public void setCorrelation(double corr) {
        correlation = corr;
    }

    public double getCorrelation() {
        return correlation;
    }

    public void setIndex(double number) {
        index = number;
    }

    public void setIndex() {
        if ((left != null) && (right != null)) {
            index = (right.getIndex() + left.getIndex()) / 2;
        }
    }

    public double getIndex() {
        return index;
    }

    public void setColor(Color c) {
        color = c;
    }

    public Color getColor() {
        return color;
    }

    public void setSelected(boolean b) {
        selected = b;
        if (left != null) {
            left.setSelected(b);
        }
        if (right != null) {
            right.setSelected(b);
        }
    }

    public void setRootColor(Color color) {
        this.color = color;
        if (left != null) {
            left.setRootColor(color);
        }
        if (right != null) {
            right.setRootColor(color);
        }
    }

    public boolean isSelected() {
        return selected;
    }

    public boolean isLeaf() {
        return ((left == null) && (right == null));
    }

    public boolean isRoot() {
        return (parent == null);
    }

    public String toString() {
        return id;
    }

    public int getMaxIndex() {
        return maxIndex;
    }

    public void setMaxIndex(int maxIndex) {
        this.maxIndex = maxIndex;
    }

    public int getMinIndex() {
        return minIndex;
    }

    public void setMinIndex(int minIndex) {
        this.minIndex = minIndex;
    }

    public void printSubtree() {
        if (!isLeaf()) {
            System.out.println(getId() + ", corr " + getCorrelation()
                    + ", index " + getIndex() + " min " + minIndex + " max "
                    + maxIndex);
            System.out.print("Left: ");
            getLeft().printSubtree();
            System.out.print("Right: ");
            getRight().printSubtree();
        } else {
            System.out.println("LEAF " + getId() + " corr " + getCorrelation()
                    + ", index " + getIndex());
        }
    }

    protected void recomputeIndices() {
        if (!isLeaf()) {
            left.recomputeIndices();
            right.recomputeIndices();
            setIndex();
            maxIndex = Math.max(left.maxIndex, right.maxIndex);
            minIndex = Math.min(left.minIndex, right.minIndex);
        }
    }

}
