package org.genepattern.gsea;

import org.genepattern.clustering.hierarchical.ArrayTreePanel;
import org.genepattern.clustering.hierarchical.GeneTreePanel;
import org.genepattern.clustering.hierarchical.Node;
import org.genepattern.data.expr.ExpressionData;
import org.genepattern.data.expr.IExpressionData;
import org.genepattern.data.matrix.IExpressionDataUtil;
import org.genepattern.heatmap.HeatMapComponent;
import org.genepattern.module.AnalysisUtil;
import org.tigr.microarray.mev.cluster.algorithm.AlgorithmException;

import javax.swing.*;
import java.awt.*;
import java.util.HashMap;
import java.util.Map;

public class HCLViewer {
    HCLAlgorithm alg;

    ArrayTreePanel arrayTreePanel;

    GeneTreePanel geneTreePanel;

    private IExpressionData expressionData;

    private IExpressionData reorderedData;

    public HCLViewer(IExpressionData expressionData, int mergeType,
                     int distanceMeasure, int clusterBy) {
        this.expressionData = expressionData;
        try {
            alg = new HCLAlgorithm(expressionData, mergeType, distanceMeasure,
                    clusterBy);
            createDisplayObjects();
        } catch (AlgorithmException e) {
            e.printStackTrace();
        }
    }

    private void createDisplayObjects() {
        int[] samplesOrder = null;
        int[] genesOrder = null;
        if (alg.getClusterBy() == HCLAlgorithm.BOTH
                || alg.getClusterBy() == HCLAlgorithm.COLUMNS) {
            samplesOrder = alg.getSamples_order();
            arrayTreePanel = (ArrayTreePanel) createTree(expressionData, alg
                    .getSamples_node_order(), alg.getSamples_child_1_array(),
                    alg.getSamples_child_2_array(), alg.getSamples_order(), alg
                    .getSamples_height().getRowPackedCopy(), true);
        }

        if (alg.getClusterBy() == HCLAlgorithm.BOTH
                || alg.getClusterBy() == HCLAlgorithm.ROWS) {
            genesOrder = alg.getGenes_order();
            geneTreePanel = (GeneTreePanel) createTree(expressionData, alg
                    .getGenes_node_order(), alg.getGenes_child_1_array(), alg
                    .getGenes_child_2_array(), alg.getGenes_order(), alg
                    .getGenes_height().getRowPackedCopy(), false);
        }
        this.reorderedData = IExpressionDataUtil.sliceView(expressionData,
                genesOrder, samplesOrder);
    }

    public ArrayTreePanel getArrayTreePanel() {
        return arrayTreePanel;
    }

    public GeneTreePanel getGeneTreePanel() {
        return geneTreePanel;
    }

    public IExpressionData getReorderedData() {
        return reorderedData;
    }

    public static void main(String[] args) {
        ExpressionData expressionData = AnalysisUtil
                .readExpressionData(args[0]);
        expressionData = new ExpressionData(expressionData
                .getExpressionMatrix().transpose(), null, null);
        int mergeType = HCLAlgorithm.AVERAGE_LINKAGE;
        int distanceMeasure = HCLAlgorithm.PEARSONUNCENTERED;
        int clusterBy = HCLAlgorithm.ROWS;

        HCLViewer vis = new HCLViewer(expressionData, mergeType,
                distanceMeasure, clusterBy);
        JFrame frame = new JFrame();
        HeatMapComponent c = new HeatMapComponent(frame,
                vis.getReorderedData(), vis.getGeneTreePanel(), vis
                .getArrayTreePanel());
        frame.setJMenuBar(c.createMenuBar(true));
        frame.setContentPane(c);
        c.showFrame();

    }

    private static JPanel createTree(IExpressionData expressionData,
                                     int[] node_order, int[] child1_array, int[] child_2_array,
                                     int[] order, float[] rowPackedHeight, boolean samples) {
        Map id2Index = new HashMap();
        int size = samples ? expressionData.getColumnCount() : expressionData
                .getRowCount();
        String[] ids = new String[size];

        for (int i = 0; i < order.length; i++) {
            if (samples) {
                id2Index.put(expressionData.getColumnName(order[i]),
                        new Integer(i));
                ids[i] = expressionData.getColumnName(order[i]);
            } else {
                id2Index.put(expressionData.getRowName(order[i]),
                        new Integer(i));
                ids[i] = expressionData.getRowName(order[i]);
            }
        }
        Map nodes = new HashMap();

        Node node = null;

        double minHeight = Double.POSITIVE_INFINITY;
        double leafCorrelation = -1;
        for (int i = 0; i < node_order.length - 1; i++) {
            String id = "" + node_order[i];
            String left = "" + child1_array[node_order[i]];
            if (Integer.parseInt(left) < size) {
                left = samples ? expressionData.getColumnName(Integer
                        .parseInt(left)) : expressionData.getRowName(Integer
                        .parseInt(left));
            }
            String right = "" + child_2_array[node_order[i]];
            if (Integer.parseInt(right) < size) {
                right = samples ? expressionData.getColumnName(Integer
                        .parseInt(right)) : expressionData.getRowName(Integer
                        .parseInt(right));
            }
            Node lnode = null;
            Node rnode = null;
            // check if left is arrayNodes
            // if not then it is a leaf
            if (nodes.containsKey(left)) {
                lnode = (Node) nodes.get(left);
            } else {
                lnode = new Node(left);
                lnode.setCorrelation(leafCorrelation);
                // set index for leaf... all leaves are arrays
                int index = ((Integer) id2Index.get(left)).intValue();
                // System.out.println("array leaf: " + a.getId());
                lnode.setIndex(index);
                // System.out.println("array index " + a.getIndex());
                lnode.setMinIndex(index);
                lnode.setMaxIndex(index);
                // put into arrayNodes hash
                nodes.put(left, lnode);
            }

            // check if right is arrayNodes
            // if not then it is a leaf
            if (nodes.containsKey(right)) {
                rnode = (Node) nodes.get(right);
            } else {
                rnode = new Node(right);
                rnode.setCorrelation(leafCorrelation);
                // set index
                // set index for leaf... all leaves are arrays
                int index = ((Integer) id2Index.get(right)).intValue();
                rnode.setIndex(index);
                rnode.setMinIndex(index);
                rnode.setMaxIndex(index);
                // put into arrayNodes hash
                nodes.put(right, rnode);
            }

            double corr = rowPackedHeight[node_order[i]];
            minHeight = Math.min(minHeight, corr);
            node = new Node(id, lnode, rnode, corr);

            // set min and max indexes
            node
                    .setMinIndex(Math.min(rnode.getMinIndex(), lnode
                            .getMinIndex()));
            node
                    .setMaxIndex(Math.max(rnode.getMaxIndex(), lnode
                            .getMaxIndex()));

            // set parents of l and r nodes
            lnode.setParent(node);
            rnode.setParent(node);

            // put node into arrayNodes
            nodes.put(id, node);
        }

        double maxHeight = node.getCorrelation(); // last node is root,
        // has max height
        String rootId = node.getId();
        if (samples) {
            ArrayTreePanel arrayTree = new ArrayTreePanel(nodes, rootId, ids);
            arrayTree.setYMax(maxHeight);
            arrayTree.setYMin(minHeight);
            arrayTree.setPreferredSize(new Dimension(300, 150));
            arrayTree.setElementWidth(8);
            return arrayTree;
        } else {
            GeneTreePanel geneTree = new GeneTreePanel(nodes, rootId, ids);
            geneTree.setXMax(minHeight);
            geneTree.setXMin(maxHeight);
            geneTree.setPreferredSize(new Dimension(300, 150));
            geneTree.setElementHeight(8);
            return geneTree;
        }

    }
}
