/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

package org.genepattern.gsea;

import org.genepattern.data.expr.IExpressionData;
import org.genepattern.data.matrix.IExpressionDataUtil;
import org.genepattern.heatmap.ColorScheme;
import org.genepattern.heatmap.HeatMapComponent;
import org.genepattern.table.BrowserLauncher;
import org.tigr.microarray.mev.cluster.algorithm.AlgorithmException;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;

/**
 * @author Joshua Gould
 */
public class LeadingEdgePanel {

    private HeatMapComponent heatMap;

    private File resultDirectory;

    /**
     * @param parent         the parent for the leading edge panel
     * @param accessoryPanel the panel to display above the leading edge view or
     *                       <code>null</code>
     */
    public LeadingEdgePanel(Frame parent, JPanel accessoryPanel) {
        heatMap = new HeatMapComponent(parent, IExpressionDataUtil
                .createRandomData(1, 1), // XXX set dummy data initially
                null, null, accessoryPanel);
        heatMap.setFeatureUIString("Gene Set");
        heatMap.setSampleUIString("Gene");
        heatMap.setShowFeatureAnnotator(false);
        heatMap.setShowRowDescriptions(false);
        heatMap.setShowSampleAnnotator(false);
        heatMap.setShowColorSchemeOptions(false);

        final JTable featureTable = heatMap.getFeatureTable();
        featureTable.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (resultDirectory == null) {
                    return;
                }
                int row = featureTable.rowAtPoint(e.getPoint());
                int column = featureTable.columnAtPoint(e.getPoint());
                if (row != -1 && column == 0) {
                    String value = (String) featureTable
                            .getValueAt(row, column);
                    value = value.replaceAll("_signal", "");
                    File gsetGseaFile = new File(resultDirectory, value
                            + ".html");
                    try {
                        BrowserLauncher
                                .openURL(gsetGseaFile.toURI().toString());
                    } catch (Exception x) {
                    }
                }
            }
        });

    }

    public void setResultDirectory(File resultDirectory) {
        this.resultDirectory = resultDirectory;
    }

    public void setData(IExpressionData data, boolean hierarchicalCluster,
                        ColorScheme cs) {
        heatMap.setColorConverter(cs);
        if (hierarchicalCluster) {
            int mergeType = HCLAlgorithm.SINGLE_LINKAGE;
            int distanceMeasure = HCLAlgorithm.EUCLIDEAN;
            int clusterBy = HCLAlgorithm.BOTH;
            try {
                if (data.getRowCount() > 0 && data.getColumnCount() > 0) {
                    HCLAlgorithm alg = new HCLAlgorithm(data, mergeType,
                            distanceMeasure, clusterBy);
                    data = IExpressionDataUtil.sliceView(data,
                            alg.getGenes_order(), alg.getSamples_order());
                }
            } catch (AlgorithmException ae) {
                ae.printStackTrace();
            }
        }
        heatMap.setExpressionData(data);
    }

    public JMenuBar getMenuBar() {
        heatMap.setOptionsDialogOptions(false, false, false);
        return heatMap.createMenuBar(false, false, false, false);
    }

    public HeatMapComponent getHeatMapComponent() {
        return heatMap;
    }

    public void showLegend() {
        heatMap.showLegend();
    }

    public void showSaveDatasetDialog() {
        heatMap.showSaveDatasetDialog();
    }

    public void showSaveImageDialog(String format) {
        heatMap.showSaveImageDialog(format);
    }

    public void showOptionsDialog() {
        heatMap.showOptionsDialog();
    }

}
