/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

package org.genepattern.io;

import gnu.trove.TIntArrayList;

import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashSet;
import java.util.Set;

/**
 * @author Joshua Gould
 */
public class Util {
    private Util() {
    }

    public static double[] toPrimitive(Double[] data) {
        double[] temp = new double[data.length];
        for (int i = 0; i < temp.length; i++) {
            temp[i] = data[i].doubleValue();
        }
        return temp;
    }

    public static boolean[] toPrimitive(Boolean[] data) {
        boolean[] temp = new boolean[data.length];
        for (int i = 0; i < temp.length; i++) {
            temp[i] = data[i].booleanValue();
        }
        return temp;
    }

    public static int[] toPrimitive(Integer[] data) {
        int[] temp = new int[data.length];
        for (int i = 0; i < temp.length; i++) {
            temp[i] = data[i].intValue();
        }
        return temp;
    }

    public static String toString(Throwable t) {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        t.printStackTrace(pw);
        pw.close();
        return sw.toString();
    }

    public static String toString(File file) throws IOException {
        StringBuffer sb = new StringBuffer();
        BufferedReader br = null;
        try {
            br = new BufferedReader(new FileReader(file));
            String s = null;
            while ((s = br.readLine()) != null) {
                sb.append(s);
                sb.append("\n");
            }
        } finally {
            if (br != null) {
                br.close();
            }
        }
        return sb.toString();
    }

    /**
     * Makes all elements in the <tt>names</tt> arrray unique
     *
     * @param names
     */
    public static void makeUnique(String[] names) {
        Set nameSet = new HashSet(names.length);
        for (int i = 0; i < names.length; i++) {
            String name = names[i];
            if (nameSet.contains(name)) {
                int j = 1;
                String newName = name + "-" + j;
                while (nameSet.contains(newName)) {
                    newName = name + "-" + j;
                    j++;
                }
                name = newName;
                names[i] = name;
            }
            nameSet.add(name);
        }
    }

    public static String toString(double[] d) {
        StringBuffer sb = new StringBuffer();
        if (d == null) {
            return "";
        }
        for (int i = 0; i < d.length; i++) {
            if (i > 0) {
                sb.append(" ");
            }
            sb.append(d[i]);
        }
        return sb.toString();
    }

    /**
     * @param rangeString string of form 1-5, 7, 9-11
     * @return indices
     */
    public static int[] rangeToIntArray(String rangeString) {
        TIntArrayList indices = new TIntArrayList();
        String[] tokens = rangeString.split(",");
        for (int j = 0, length = tokens.length; j < length; j++) {
            String s = tokens[j].trim();
            int dashIndex = -1;
            if ((dashIndex = s.indexOf("-")) != -1) {
                int first = 0;
                try {
                    first = Integer.parseInt(s.substring(0, dashIndex).trim());
                } catch (NumberFormatException nfe) {
                    throw new NumberFormatException(s.substring(0, dashIndex)
                            .trim()
                            + " is not an integer.");
                }
                int last = 0;
                try {
                    last = Integer.parseInt(s.substring(dashIndex + 1,
                            s.length()).trim());
                } catch (NumberFormatException nfe) {
                    throw new NumberFormatException(s.substring(dashIndex + 1,
                            s.length()).trim()
                            + " is not an integer.");
                }
                if (first <= last) {
                    for (int i = first; i <= last; i++) {
                        indices.add(i);
                    }
                } else {
                    for (int i = first; i >= last; i--) {
                        indices.add(i);
                    }
                }
            } else {
                try {
                    int index = Integer.parseInt(s.trim());
                    indices.add(index);
                } catch (NumberFormatException nfe) {
                    throw new NumberFormatException(s.trim()
                            + " is not an integer.");
                }
            }
        }
        return indices.toNativeArray();
    }

    /**
     * @param rangeString string of form 1:2:5, 7, 9:11
     * @return indices
     */
    public static int[] semicolonRangeToIntArray(String rangeString) {
        TIntArrayList indices = new TIntArrayList();
        String[] tokens = rangeString.split(",");
        for (int j = 0, length = tokens.length; j < length; j++) {
            String s = tokens[j].trim();
            String[] range = s.split(":");
            if (range.length == 1) {
                int index = parseInt(s.trim());
                indices.add(index);
            } else if (range.length == 2 || range.length == 3) {
                addToArray(range, indices);
            } else {
                throw new NumberFormatException(s
                        + " does not specify a valid range.");
            }
        }
        return indices.toNativeArray();
    }

    static int parseInt(String s) {
        try {
            return Integer.parseInt(s);
        } catch (NumberFormatException nfe) {
            throw new NumberFormatException(s + " is not an integer.");
        }
    }

    private static void addToArray(String[] range, TIntArrayList indices) {
        int first = parseInt(range[0].trim());
        int by = 1;
        int lastIndex = 1;
        if (range.length == 3) {
            by = parseInt(range[2].trim());
            lastIndex = 2;
        }

        int last = parseInt(range[lastIndex].trim());
        if (first <= last) {
            for (int i = first; i <= last; i += by) {
                indices.add(i);
            }
        } else {
            for (int i = first; i >= last; i -= by) {
                indices.add(i);
            }
        }
    }

    public static String toString(int[] d) {
        StringBuffer sb = new StringBuffer();
        if (d == null) {
            return "";
        }
        for (int i = 0; i < d.length; i++) {
            if (i > 0) {
                sb.append(" ");
            }
            sb.append(d[i]);
        }
        return sb.toString();
    }

    public static String getBaseFileName(String filepath) {
        String name = new File(filepath).getName();
        int dotIndex = name.lastIndexOf(".");
        if (dotIndex > 0) {
            return name.substring(0, dotIndex);
        }
        return name;
    }

    /**
     * @param file
     * @return the file extension, e.g. txt, gct
     */
    public static String getExtension(File file) {
        String name = file.getName();
        int dotIndex = name.lastIndexOf(".");
        if (dotIndex > 0) {
            return name.substring(dotIndex + 1, name.length());
        }
        return null;
    }


    public static void download(URL url, File file) throws IOException {
        long lastModifiedDate = System.currentTimeMillis();
        InputStream is = null;
        FileOutputStream fos = null;
        try {
            URLConnection connection = url.openConnection();
            lastModifiedDate = connection.getHeaderFieldDate("X-lastModified",
                    lastModifiedDate);
            is = connection.getInputStream();
            byte[] b = new byte[100000];
            int bytesRead = 0;
            fos = new FileOutputStream(file);
            while ((bytesRead = is.read(b)) != -1) {
                fos.write(b, 0, bytesRead);
            }
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException x) {
                }
            }
            if (fos != null) {
                try {
                    fos.close();
                    file.setLastModified(lastModifiedDate);
                } catch (IOException x) {
                }
            }
        }
    }

}
