/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

package org.genepattern.menu;

import com.sun.media.jai.codec.ImageEncodeParam;
import com.sun.media.jai.codec.JPEGEncodeParam;
import org.genepattern.uiutil.FileChooser;
import org.genepattern.uiutil.UIUtil;

import javax.media.jai.JAI;
import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;

/**
 * @author Joshua Gould
 */
public abstract class AbstractSaveImageAction extends PlotAction {
    Frame parent;

    public AbstractSaveImageAction(Frame parent) {
        if (parent == null) {
            throw new NullPointerException("Null parent not allowed");
        }
        if (false) { // !FileChooser.RUNNING_ON_MAC
            putValue(AbstractAction.NAME, "Save Image...");
            KeyStroke ks = KeyStroke.getKeyStroke('S', Toolkit
                    .getDefaultToolkit().getMenuShortcutKeyMask());
            this.putValue(AbstractAction.ACCELERATOR_KEY, ks);
        } else {
            putValue(AbstractAction.NAME, "Save Image");
        }

        this.parent = parent;
    }

    protected abstract BufferedImage createImage();

    public void actionPerformed(ActionEvent e) {
        try {
            if (!FileChooser.RUNNING_ON_MAC) {
                saveImage();
            }
        } catch (Exception x) {
            showError(x, "An error occurred while saving the plot.");
        }
    }

    public JMenuItem[] getSubMenuItems() {
        if (!FileChooser.RUNNING_ON_MAC) {
            return null;
        }
        final JMenuItem bmpItem = new JMenuItem("bmp...");
        final JMenuItem jpegItem = new JMenuItem("jpeg...");
        final JMenuItem pngItem = new JMenuItem("png...");
        final JMenuItem tiffItem = new JMenuItem("tiff...");
        ActionListener l = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String fileFormat = null;
                if (e.getSource() == bmpItem) {
                    fileFormat = "bmp";
                } else if (e.getSource() == jpegItem) {
                    fileFormat = "jpeg";
                } else if (e.getSource() == pngItem) {
                    fileFormat = "png";
                } else if (e.getSource() == tiffItem) {
                    fileFormat = "tiff";
                }
                File outputFile = FileChooser.showSaveDialog(parent);
                if (outputFile != null) {
                    save(outputFile, fileFormat);
                }
            }
        };
        bmpItem.addActionListener(l);
        jpegItem.addActionListener(l);
        pngItem.addActionListener(l);
        tiffItem.addActionListener(l);
        return new JMenuItem[]{bmpItem, jpegItem, pngItem, tiffItem};
    }

    void saveImage() throws Exception {
        if ((Double.parseDouble(System
                .getProperty("java.specification.version"))) < 1.4) {
            UIUtil.showMessageDialog(getPlot().getTopLevelAncestor(),
                    "Java 1.4 is required to save an image.");
            return;
        }

        File outputFile = null;
        String outputFileFormat = null;
        if (!FileChooser.RUNNING_ON_MAC) {
            JFileChooser fc = new JFileChooser();
            fc.setAcceptAllFileFilterUsed(false);
            fc.addChoosableFileFilter(new MyFileFilter(new String[]{"bmp"},
                    "BMP image", "BMP"));
            fc.addChoosableFileFilter(new MyFileFilter(new String[]{"jpeg",
                    "jpg"}, "JPEG image", "JPEG"));
            fc.addChoosableFileFilter(new MyFileFilter(new String[]{"png"},
                    "PNG image", "PNG"));
            fc.addChoosableFileFilter(new MyFileFilter(new String[]{"tiff"},
                    "TIFF image", "TIFF"));
            if (fc.showSaveDialog(getPlot().getTopLevelAncestor()) == JFileChooser.APPROVE_OPTION) {
                outputFile = fc.getSelectedFile();
                outputFileFormat = ((MyFileFilter) fc.getFileFilter())
                        .getFileFormat();
                outputFile = createFileWithCorrectExtension(outputFile,
                        outputFileFormat);

                if (!FileChooser.overwriteFile(getPlot().getTopLevelAncestor(),
                        outputFile)) {
                    return;
                }

            }
        }
        if (outputFile != null) {
            save(outputFile, outputFileFormat);
        }
    }

    private File createFileWithCorrectExtension(File outputFile,
                                                String outputFileFormat) {
        String name = outputFile.getName();
        String extension = "";
        int index = name.lastIndexOf(".");
        if (index > 0) {
            extension = name.substring(index, name.length());
            name = name.substring(0, index);
        }

        if (outputFileFormat.equalsIgnoreCase("jpeg")) {
            if (!extension.equalsIgnoreCase(".jpg")
                    && !extension.equalsIgnoreCase(".jpeg")) {
                extension += ".jpeg";
            }
        } else if (outputFileFormat.equalsIgnoreCase("png")) {
            if (!extension.equalsIgnoreCase(".png")) {
                extension += ".png";
            }

        } else if (outputFileFormat.equalsIgnoreCase("tiff")) {
            if (!extension.equalsIgnoreCase(".tiff")) {
                extension += ".tiff";
            }

        } else if (outputFileFormat.equalsIgnoreCase("bmp")) {
            if (!extension.equalsIgnoreCase(".bmp")) {
                extension += ".bmp";
            }

        }
        return new File(outputFile.getParent(), name + extension);
    }

    private void save(File outputFile, final String outputFileFormat) {
        final File f = outputFile;
        new Thread() {
            public void run() {
                BufferedImage bufferedImage = createImage();
                ImageEncodeParam fParam = null;
                if (outputFileFormat.equalsIgnoreCase("jpeg")) {
                    JPEGEncodeParam jpegParam = new JPEGEncodeParam();
                    jpegParam.setQuality(1.0f);
                    fParam = jpegParam;
                } else if (outputFileFormat.equalsIgnoreCase("png")) {
                    fParam = new com.sun.media.jai.codec.PNGEncodeParam.RGB();
                } else if (outputFileFormat.equalsIgnoreCase("tiff")) {
                    com.sun.media.jai.codec.TIFFEncodeParam param = new com.sun.media.jai.codec.TIFFEncodeParam();
                    param
                            .setCompression(com.sun.media.jai.codec.TIFFEncodeParam.COMPRESSION_NONE);
                    fParam = param;
                } else if (outputFileFormat.equalsIgnoreCase("bmp")) {
                    fParam = new com.sun.media.jai.codec.BMPEncodeParam();
                }
                try {
                    JAI.create("filestore", bufferedImage,
                            f.getCanonicalPath(), outputFileFormat, fParam);
                } catch (Throwable x) {
                    UIUtil.showErrorDialog(getPlot().getTopLevelAncestor(),
                            "An error occurred while saving the image.");
                }
            }
        }.start();
    }

    private void showError(Exception e, String msg) {
        if (e.getMessage() != null) {
            msg += "\nCause: " + e.getMessage();
        }
        UIUtil.showErrorDialog(getPlot().getTopLevelAncestor(), msg);
    }

    /**
     * @author Joshua Gould
     */
    static class MyFileFilter extends javax.swing.filechooser.FileFilter {
        java.util.List extensions;

        String fileFormat, description;

        public MyFileFilter(String[] extensions, String description,
                            String fileFormat) {
            this.extensions = java.util.Arrays.asList(extensions);
            this.description = description;
            this.fileFormat = fileFormat;

        }

        public boolean accept(File f) {
            if (f.isDirectory()) {
                return true;
            }
            String name = f.getName();
            int dotIndex = name.lastIndexOf(".");
            if (dotIndex > 0) {
                String ext = name.substring(dotIndex + 1, name.length());
                return extensions.contains(ext.toLowerCase());
            }
            return false;
        }

        public String getDescription() {
            return description;
        }

        public String getFileFormat() {
            return fileFormat;
        }
    }

}
