/*
  The Broad Institute
  SOFTWARE COPYRIGHT NOTICE AGREEMENT
  This software and its documentation are copyright (2003-2006) by the
  Broad Institute/Massachusetts Institute of Technology. All rights are
  reserved.

  This software is supplied without any warranty or guaranteed support
  whatsoever. Neither the Broad Institute nor MIT can be responsible for its
  use, misuse, or functionality.
*/

/*
  The Broad Institute
  SOFTWARE COPYRIGHT NOTICE AGREEMENT
  This software and its documentation are copyright (2003-2006) by the
  Broad Institute/Massachusetts Institute of Technology. All rights are
  reserved.

  This software is supplied without any warranty or guaranteed support
  whatsoever. Neither the Broad Institute nor MIT can be responsible for its
  use, misuse, or functionality.
*/


package org.genepattern.menu;

import org.genepattern.table.GPTable;

import javax.swing.*;
import java.awt.*;

public abstract class GPMenuBar extends JMenuBar {
    protected FileMenu fileMenu;

    protected EditMenu editMenu;

    protected ViewMenu viewMenu;

    protected Frame parent;

    protected abstract FileMenu createFileMenu(JComponent plot);

    protected abstract ViewMenu createViewMenu(JComponent plot);

    protected boolean showExitMenu = false;

    public GPMenuBar(JComponent plot, Frame parent) {
        if (parent == null) {
            throw new NullPointerException("Null parent not allowed");
        }
        this.parent = parent;
        fileMenu = createFileMenu(plot);
        viewMenu = createViewMenu(plot);
        add(fileMenu);
        add(viewMenu);
    }

    public GPMenuBar(GPTable table, JComponent plot, Frame parent,
                     int featureColumnIndex) {
        if (parent == null) {
            throw new NullPointerException("Null parent not allowed");
        }
        this.parent = parent;
        fileMenu = createFileMenu(plot);
        viewMenu = createViewMenu(plot);
        editMenu = new EditMenu(parent, table, featureColumnIndex);
        add(fileMenu);
        add(editMenu);
        add(viewMenu);
    }

    public GPMenuBar(GPTable table, JComponent plot, Frame parent,
                     int featureColumnIndex, boolean showExitMenu) {
        if (parent == null) {
            throw new NullPointerException("Null parent not allowed");
        }
        this.parent = parent;
        this.showExitMenu = showExitMenu;
        fileMenu = createFileMenu(plot);
        editMenu = new EditMenu(parent, table, featureColumnIndex);
        viewMenu = createViewMenu(plot);
        add(fileMenu);
        add(editMenu);
        add(viewMenu);
    }

    public void setPlot(JComponent plot) {
        viewMenu.setPlot(plot);
        fileMenu.setPlot(plot);
    }

    public JMenu getFileMenu() {
        return fileMenu;
    }

    public JMenu getEditMenu() {
        return editMenu;
    }

    public JMenu getViewMenu() {
        return viewMenu;
    }
}