/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

package org.genepattern.plot;

import org.genepattern.data.expr.IExpressionData;
import org.genepattern.menu.jfree.JFreeMenuBar;
import org.genepattern.menu.jfree.JFreeUtil;
import org.genepattern.uiutil.CenteredDialog;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.SymbolAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

import javax.swing.*;
import java.awt.*;

public class ProfilePlot {
    IExpressionData expressionData;

    JFreeChart chart;

    ChartPanel chartPanel;

    JDialog dialog;

    /**
     * Creates a new instance
     *
     * @param parent         the dialog parent
     * @param expressionData the expression data
     */
    public ProfilePlot(Frame parent, IExpressionData expressionData) {
        this.expressionData = expressionData;
        chart = ChartFactory.createScatterPlot("",
                // title
                "Column",
                // x-axis label
                "Value",
                // y-axis label
                null,
                // data
                PlotOrientation.VERTICAL, false,
                // create legend?
                false,
                // generate tooltips?
                false
                // generate URLs?
        );

        chartPanel = new ChartPanel(chart, false, false, false, false, false);
        chartPanel.setMouseZoomable(true, false);
        XYLineAndShapeRenderer lineRenderer = (XYLineAndShapeRenderer) chart
                .getXYPlot().getRenderer();
        lineRenderer.setLinesVisible(true);
        String[] columnNames = new String[expressionData.getColumnCount()];
        for (int j = 0, columns = expressionData.getColumnCount(); j < columns; j++) {
            columnNames[j] = expressionData.getColumnName(j);
        }
        SymbolAxis xAxis = new SymbolAxis("Column", columnNames);
        xAxis.setVerticalTickLabels(true);
        chart.getXYPlot().setDomainAxis(xAxis);
        dialog = new CenteredDialog(parent);
        dialog.setTitle("Profile");
        dialog.getContentPane().add(chartPanel);
        dialog.setJMenuBar(new JFreeMenuBar(chartPanel, parent));
    }

    /**
     * Plots the given feature names in a dialog. If featureNames is null, plots
     * profile for all features
     *
     * @param featureNames the feature names to plot
     * @throws IllegalArgumentException if a feature is not found.
     */
    public void plot(String[] featureNames) {
        int[] rowIndices = null;
        if (featureNames != null) {
            rowIndices = new int[featureNames.length];
            for (int i = 0; i < rowIndices.length; i++) {
                rowIndices[i] = expressionData.getRowIndex(featureNames[i]);
                if (rowIndices[i] == -1) {
                    throw new IllegalArgumentException("Feature "
                            + featureNames[i]
                            + " was not found in the loaded dataset.");
                }
            }
        }
        plot(rowIndices);
    }

    public void plot(int[] rowIndices) {
        XYSeriesCollection coll = new XYSeriesCollection();

        int rows = rowIndices != null ? rowIndices.length : expressionData
                .getRowCount();
        for (int i = 0; i < rows; i++) {
            int index = rowIndices != null ? rowIndices[i] : i;
            XYSeries series = new XYSeries(expressionData.getRowName(index));
            for (int j = 0; j < expressionData.getColumnCount(); j++) {
                series.add(j, expressionData.getValue(index, j));
            }
            coll.addSeries(series);
        }

        chart.getXYPlot().setDataset(coll);
        if (rowIndices != null && rowIndices.length <= 5) { // only show
            // legend if
            // showing 5 genes or less
            JFreeUtil.createLegend(chart);
        } else {
            chart.clearSubtitles();

        }

        Thread t = new Thread() {
            public void run() {
                dialog.pack();
                dialog.setVisible(true);
            }
        };
        if (SwingUtilities.isEventDispatchThread()) {
            t.run();
        } else {
            SwingUtilities.invokeLater(t);
        }

    }

}
