/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

/*
 The Broad Institute
 SOFTWARE COPYRIGHT NOTICE AGREEMENT
 This software and its documentation are copyright (2003-2006) by the
 Broad Institute/Massachusetts Institute of Technology. All rights are
 reserved.

 This software is supplied without any warranty or guaranteed support
 whatsoever. Neither the Broad Institute nor MIT can be responsible for its
 use, misuse, or functionality.
 */

package org.genepattern.table;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableModel;
import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;

/**
 * @author Joshua Gould
 */
public class GPTable extends JTable {
    CopyAction copy = new CopyAction();

    SelectAllAction selectAllAction = new SelectAllAction();

    public GPTable(TableModel model) {
        super(model);
        this.setShowGrid(true);
        this.setGridColor(java.awt.Color.black);
        this.setColumnSelectionAllowed(true);
        this.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        this.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
        addMouseListener(new java.awt.event.MouseAdapter() {
            java.awt.Point p = new java.awt.Point();

            public void mouseClicked(java.awt.event.MouseEvent e) {
                p.x = e.getX();
                p.y = e.getY();
                int row = rowAtPoint(p);
                int column = columnAtPoint(p);
                if (row != -1 && column != -1) {
                    Object value = getValueAt(row, column);
                }
            }
        });
    }

    protected JTableHeader createDefaultTableHeader() {
        return new JTableHeader(columnModel) {
            public String getToolTipText(MouseEvent e) {
                java.awt.Point p = e.getPoint();
                int index = columnModel.getColumnIndexAtX(p.x);
                int realIndex = columnModel.getColumn(index).getModelIndex();
                return realIndex >= 0 ? getColumnName(realIndex) : null;
            }
        };
    }

    public Action getCopyAction() {
        return copy;
    }

    public Action getSelectAllAction() {
        return selectAllAction;
    }

    /**
     * Copies the data in the table to the system clipboard
     */
    public void copy() {
        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
        int[] selectedRows = this.getSelectedRows();
        int[] selectedColumns = this.getSelectedColumns();
        StringBuffer buf = getSelectedData(this, selectedRows, selectedColumns);
        StringSelection stringSelection = new StringSelection(buf.toString());
        clipboard.setContents(stringSelection, stringSelection);
    }

    /**
     * Gets the data selected by the user in the this.
     *
     * @param table           Description of the Parameter
     * @param selectedRows    Description of the Parameter
     * @param selectedColumns Description of the Parameter
     * @return the selected data.
     */
    public static StringBuffer getSelectedData(JTable table,
                                               int[] selectedRows, int[] selectedColumns) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < selectedRows.length; i++) {
            for (int j = 0; j < selectedColumns.length; j++) {
                Object value = table.getValueAt(selectedRows[i],
                        selectedColumns[j]); // column converted to model
                if (value != null) {
                    buf.append(value);
                }
                buf.append("\t");
            }
            buf.append("\n");
        }
        return buf;
    }

    public String getToolTipText(MouseEvent event) {
        String tip = null;
        Point p = event.getPoint();
        int hitColumnIndex = columnAtPoint(p);
        int hitRowIndex = rowAtPoint(p);
        if ((hitColumnIndex != -1) && (hitRowIndex != -1)) {
            Object obj = getValueAt(hitRowIndex, hitColumnIndex);
            if (obj != null) {
                tip = obj.toString();
            }
        }
        if (tip == null) {
            tip = getToolTipText();
        }
        return tip;
    }

    /**
     * @author Joshua Gould
     */
    static class LinkRenderer extends
            javax.swing.table.DefaultTableCellRenderer {
        protected void setValue(Object obj) {

            if (obj == null) {
                super.setValue(null);
                return;
            }
        }
    }

    /**
     * @author Joshua Gould
     */
    class CopyAction extends AbstractAction {
        CopyAction() {
            super("Copy");
            KeyStroke ks = KeyStroke.getKeyStroke('C', Toolkit
                    .getDefaultToolkit().getMenuShortcutKeyMask());
            String cmd = (String) this.getValue(Action.NAME);
            GPTable.this.getActionMap().put(cmd, this);
            GPTable.this.getInputMap().put(ks, cmd);
            this.putValue(AbstractAction.ACCELERATOR_KEY, ks);
            setEnabled(false);

            final ListSelectionListener listener = new ListSelectionListener() {
                public void valueChanged(ListSelectionEvent e) {
                    if (e.getFirstIndex() >= 0) {
                        setEnabled(true);
                        GPTable.this.getSelectionModel()
                                .removeListSelectionListener(this);
                    }
                }
            };
            GPTable.this.getSelectionModel().addListSelectionListener(listener);
        }

        public void actionPerformed(ActionEvent e) {
            copy();
        }
    }

    /**
     * @author Joshua Gould
     */
    class SelectAllAction extends AbstractAction {
        SelectAllAction() {
            super("Select All");
            KeyStroke selectAllKeyStroke = KeyStroke.getKeyStroke('A', Toolkit
                    .getDefaultToolkit().getMenuShortcutKeyMask());
            String selectAllCommand = (String) this.getValue(Action.NAME);
            GPTable.this.getActionMap().put(selectAllCommand, this);
            GPTable.this.getInputMap()
                    .put(selectAllKeyStroke, selectAllCommand);
            this.putValue(AbstractAction.ACCELERATOR_KEY, selectAllKeyStroke);
        }

        public void actionPerformed(ActionEvent e) {
            selectAll();
        }
    }

}
