package org.genepattern.uiutil;

import com.enterprisedt.net.ftp.FTPConnectMode;
import com.enterprisedt.net.ftp.FTPException;
import com.enterprisedt.net.ftp.FileTransferClient;
import java.io.BufferedReader;

import javax.swing.*;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import xtools.gsea.GsaaSeqSP;

public class FTPList extends JList {

    private FileTransferClient ftpClient;
    private String[] fFileNames;

    /**
     * Class constructor
     *
     * @param host
     * @param dir
     * @throws IOException
     */
    public FTPList(final String host, final String username, final String password, final String dir) throws IOException, FTPException, IllegalArgumentException {
        this(host, username, password, dir, null);
    }

    public FTPList(final String host, final String username, final String password, final String dir, final Comparator comp_opt) throws IOException, FTPException, IllegalArgumentException {

        ftpClient = new FileTransferClient();

        // set remote host
        ftpClient.setRemoteHost(host);
        ftpClient.setUserName(username);
        ftpClient.setPassword(password);

        // connect to the server
        ftpClient.connect();

        // set connect mode to Passive
        ftpClient.getAdvancedFTPSettings().setConnectMode(FTPConnectMode.PASV);

        ftpClient.changeDirectory(dir);   

        this.fFileNames = ftpClient.directoryNameList();

        DefaultListModel model = new DefaultListModel();
        if (fFileNames != null) {

            if (comp_opt != null) {
                List all = Arrays.asList(fFileNames);
                Collections.sort(all, comp_opt);
                this.fFileNames = (String[]) all.toArray(new String[all.size()]);
            }

            for (int i = 0, length = fFileNames.length; i < length; i++) {
                model.addElement(new org.genepattern.uiutil.FTPFile(host, dir, fFileNames[i]));
            }
        }
        setModel(model);
    }  

    public String[] getFileNames() {
        return fFileNames;
    }

    public void quit() throws IOException, FTPException {
        ftpClient.disconnect();
    }
}