/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xapps.api;

import com.jidesoft.swing.JideButton;
import com.jidesoft.swing.JideSplitPane;
import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.parsers.AuxUtils;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.swing.GuiHelper;
import edu.mit.broad.genome.swing.SRLayout;
import edu.mit.broad.genome.swing.SimpleInternalPanel;
import edu.mit.broad.genome.swing.VerticalFlowLayout;
import edu.mit.broad.genome.swing.dnd.DndTarget;
import edu.mit.broad.genome.swing.dnd.DropTargetDecorator;
import edu.mit.broad.xbench.actions.XAction;
import edu.mit.broad.xbench.actions.ext.BrowserAction;
import edu.mit.broad.xbench.core.Widget;
import edu.mit.broad.xbench.core.api.Application;
import edu.mit.broad.xbench.explorer.filemgr.JRecentFilesList;
import edu.mit.broad.xbench.explorer.filemgr.XFileChooser;
import edu.mit.broad.xbench.explorer.objmgr.ObjectTree;
import edu.mit.broad.xbench.tui.ReportStub;
import edu.mit.broad.xbench.tui.TaskManager;
import edu.mit.broad.xbench.tui.ToolRunnerControl;
import foxtrot.Job;
import foxtrot.Worker;
import org.apache.log4j.Logger;
import xapps.gsea.GseaAppConf;
import xapps.gsea.GseaWebResources;
import xtools.api.Tool;
import xtools.gsea.Gsea;

import javax.swing.*;
import java.awt.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;

/**
 * Widget that builds the in a JTabbedPane:
 * 1) file system view along with a few easy access buttons
 * 2) a panel for the recently accessed file list
 * 3) opens up in last folder disp if possible
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class AppDataLoaderWidget extends SimpleInternalPanel implements Widget {

    public static final String TITLE = "Load data";
    public static final Icon ICON = JarResources.getIcon("LocalFileExplorerWidget_16_v2.jpg");

    private static final Logger klog = XLogger.getLogger(AppDataLoaderWidget.class);

    private JComponent previousFilesPanel;

    private AppDataLoaderWidget fInstance = this;

    // private JTextField tfLoadingStatus;

    /**
     * Class Constructor.
     */
    public AppDataLoaderWidget() {
        super(null, "<html><body><b>Load data</b>: Import data into the application</body></html>");

        init();
    }

    private void init() {

        jbInit();

    }

    // does the GUI building
    private void jbInit() {

        JPanel loadPanel = createLoadPanel();

        this.previousFilesPanel = JRecentFilesList.createComponent("<html><body><b>Recently used files</b> &nbsp&nbsp&nbsp&nbsp " +
                "(double click to load, right click for more options)</body></html>");

        // this.tfLoadingStatus = new JTextField();
        // this.tfLoadingStatus.setEditable(false);
        //this.tfLoadingStatus.setBackground(Color.decode("#FFFF66"));
        // this.tfLoadingStatus.setBorder(BorderFactory.createTitledBorder("Status"));

        //this.setLayout(new BorderLayout());

        //this.setLayout(new VerticalFlowLayout(true, true));


        JideSplitPane split = new JideSplitPane(JideSplitPane.HORIZONTAL_SPLIT);
        split.setShowGripper(true);
        split.add(previousFilesPanel);

        ObjectTree tree = new ObjectTree();
        tree.setBorder(null);
        SimpleInternalPanel sip = new SimpleInternalPanel("<html><body><b>Object cache</b> &nbsp&nbsp&nbsp " +
                "(objects already loaded & ready for use, right click for more options)</body></html>");
        sip.add(new JScrollPane(tree), BorderLayout.CENTER);
        split.add(sip);
        split.setInitiallyEven(true);


        this.add(loadPanel, BorderLayout.CENTER);
        split.setBorder(BorderFactory.createEmptyBorder(10, 0, 0, 0));
        // this.add(tfLoadingStatus);
        this.add(split, BorderLayout.SOUTH);

        //this.add(previousFilesPanel);

        //this.add(filePanel, BorderLayout.NORTH);
        //this.add(previousFilesPanel, BorderLayout.CENTER);

    }

    /*
    public void setLoadedMessage(final File file, final boolean error) {
        if (error) {
            tfLoadingStatus.setForeground(Color.RED);
        } else {
            tfLoadingStatus.setForeground(Color.decode("#008000"));
        }

        tfLoadingStatus.setFont(GuiHelper.FONT_FIXED_TEXT_BOLD);
        tfLoadingStatus.setText("Loaded file: " + file);
    }
    */

    private JPanel createLoadPanel() {

        //JButton bBrowse = new JButton(new FileOpenAction());
        JideButton bBrowse = new JideButton(new AppDataLoaderWidget.FileOpenAction());
        bBrowse.setRolloverEnabled(true);
        bBrowse.setButtonStyle(JideButton.TOOLBAR_STYLE);
        bBrowse.setBorder(BorderFactory.createTitledBorder("Method 1:"));
        Dimension d = new Dimension(150, 60);
        //bBrowse.setSize(d);
        //bBrowse.setMaximumSize(d);
        //bBrowse.setPreferredSize(d);

        //JButton bLoadLast = new JButton(new LoadLastAnalysisFilesAction());
        JideButton bLoadLast = new JideButton(new AppDataLoaderWidget.LoadLastAnalysisFilesAction());
        bLoadLast.setRolloverEnabled(true);
        bLoadLast.setButtonStyle(JideButton.TOOLBAR_STYLE);
        bLoadLast.setBorder(BorderFactory.createTitledBorder("Method 2:"));
        //bLoadLast.setSize(d);
        //bLoadLast.setMaximumSize(d);
        //bLoadLast.setPreferredSize(d);


        JLabel label = new JLabel("<html>\n" +
                "<body>\n" +
                "\n" +
                // "<p>&nbsp;</p>\n" +
                //  "<p>&nbsp;</p>\n" +
                // "<blockquote>\n" +
                // "  <b>   </b> <blockquote>\n" +
                "    <b>Dataset</b>:\n" +
                "    <font color=\"#800000\"><i><b><font size=\"4\">res</font></b></i></font><font size=\"4\">\n" +
                "    </font> or <font color=\"#800000\" size=\"4\"><i><b>gct</b></i></font> \n" +
                "    (Broad/MIT)," +
                "<p>&nbsp;&nbsp&nbsp;&nbsp&nbsp;&nbsp&nbsp;&nbsp&nbsp;&nbsp&nbsp;&nbsp;&nbsp&nbsp;&nbsp;&nbsp&nbsp;&nbsp" +
                "<font color=\"#800000\" size=\"4\"><i><b>pcl</b></i></font> (Stanford)</p>\n" +
                "<p>&nbsp;&nbsp&nbsp;&nbsp&nbsp;&nbsp&nbsp;&nbsp&nbsp;&nbsp&nbsp;&nbsp;&nbsp&nbsp;&nbsp;&nbsp&nbsp;&nbsp" +
                "<font color=\"#800000\" size=\"4\"><i><b>txt</b></i></font> (tab-delim text)</p>\n" +
                "<p>&nbsp;&nbsp&nbsp;&nbsp&nbsp;&nbsp&nbsp;&nbsp&nbsp;&nbsp&nbsp;&nbsp;&nbsp&nbsp;&nbsp;&nbsp&nbsp;&nbsp" +
                "<font color=\"#800000\"><i><b><font size=\"4\">snp</font></b></i></font><font size=\"4\">\n" +
                "</font> or <font color=\"#800000\" size=\"4\"><i><b>num</b></i></font> \n" +
                "(Duke)," +
                // "<font color=\"#800000\" size=\"4\"><i><b>mage-ml</b></i></font> (crap)</p>\n" +
                "    <p>&nbsp;&nbsp<b>Phenotype</b> <b>labels</b>:\n" +
                "    <font color=\"#800000\" size=\"4\"><i><b>cls</b></i></font></p>\n" +
                "    <p>&nbsp;&nbsp<b>Gene sets</b>:\n" +
                "    <font color=\"#800000\" size=\"4\"><i><b>gmx</b></i></font> or\n" +
                "    <font color=\"#800000\" size=\"4\"><i><b>gmt</b></i></font>\n" +
                //  "  </blockquote>\n" +
                //"</blockquote>\n" +
                "\n" +
                "</body>\n" +
                "\n" +
                "</html>");

        label.setBorder(BorderFactory.createTitledBorder("Supported file formats"));

        JButton bFormatHelp = new JButton(new BrowserAction("More on GSEA file formats ...",
                "Online documentation on supported data formats", GuiHelper.ICON_HELP16,
                GseaWebResources.getGseaDataFormatsHelpURL()));


        /*
        FormLayout layout = new FormLayout("110dlu, 4dlu, 175dlu, 2dlu, 200dlu", "40dlu, 40dlu 20dlu");
        PanelBuilder builder = new PanelBuilder(layout);
        builder.setDefaultDialogBorder();
        CellConstraints cc = new CellConstraints();
        builder.add(bBrowse, cc.xy(1, 1));
        builder.add(createDndPanel(), cc.xywh(3, 1, 1, 3));
        builder.add(label, cc.xywh(5, 1, 1, 2));
        builder.add(bFormatHelp, cc.xy(5, 3));
        return builder.getPanel();
        */

        JPanel panel = new JPanel(new SRLayout(3, 10));

        JPanel pan = new JPanel(new VerticalFlowLayout(true, false));
        pan.add(bBrowse);
        pan.add(Box.createVerticalStrut(20));
        pan.add(bLoadLast);
        pan.add(Box.createGlue());

        panel.add(pan);
        panel.add(createDndPanel());

        JPanel sub = new JPanel(new BorderLayout());
        sub.add(label, BorderLayout.NORTH);
        JPanel format = new JPanel(new BorderLayout());
        format.add(bFormatHelp, BorderLayout.SOUTH);
        sub.add(format, BorderLayout.SOUTH);
        panel.add(sub);
        return panel;
    }

    private JPanel createDndPanel() {
        final AppDataLoaderWidget.MyTextArea ta = new AppDataLoaderWidget.MyTextArea();
        ta.setBackground(Color.LIGHT_GRAY);

        final JButton bClear = new JButton("Clear");
        bClear.addActionListener(new ActionListener() {
            public void actionPerformed(final ActionEvent evt) {
                ta.clear();
            }
        });

        final JButton bLoad = new JButton("Load these files!", JarResources.getIcon("Dnd2.gif"));
        bLoad.addActionListener(new ActionListener() {
            public void actionPerformed(final ActionEvent evt) {
                File[] files = ta.getFiles();
                if (files.length == 0) {
                    Application.getWindowManager().showMessage("No files to import!\nDrag and drop files into the box and try again");
                } else {
                    ParserFactory.createImportTask(files).run();
                }
            }
        });

        JPanel panel = new JPanel(new BorderLayout());
        panel.setBorder(BorderFactory.createTitledBorder("Method 3: drag and drop files here"));
        panel.add(new JScrollPane(ta), BorderLayout.CENTER);

        JPanel bp = new JPanel(new SRLayout(2, 10));
        bp.add(bClear);
        bp.add(bLoad);
        panel.add(bp, BorderLayout.SOUTH);
        return panel;
    }

    static class MyTextArea extends JTextArea implements DndTarget {
        private java.util.List fFiles;

        MyTextArea() {
            super();
            super.setEditable(false);
            new DropTargetDecorator(this);
            this.fFiles = new ArrayList();
        }

        public Component getDroppableIntoComponent() {
            return this;
        }

        public File[] getFiles() {
            Set files = new HashSet();
            for (int i = 0; i < fFiles.size(); i++) {
                Object it = fFiles.get(i);
                if (it instanceof File) {
                    files.add(it);
                }
            }

            return (File[]) files.toArray(new File[files.size()]);
        }

        void clear() {
            this.fFiles.clear();
            this.setText("");
        }

        public void setDropData(Object obj) {

            //log.debug("Drop data: " + obj + " class: " + obj.getClass());
            if (obj == null) {
                return;
            }

            final java.util.List list = (java.util.List) obj;

            for (int i = 0; i < list.size(); i++) {
                if (list.get(i) instanceof File) {
                    fFiles.add(list.get(i));
                }
            }

            //System.out.println(">>> " + obj);
            StringBuffer buf = new StringBuffer();
            for (int i = 0; i < fFiles.size(); i++) {
                Object it = fFiles.get(i);
                if (it instanceof File) {
                    buf.append(((File) it).getName()).append('\n');
                }
            }

            //System.out.println(">> " + buf);
            this.setText(buf.toString());
        }

        // accepts one or mroe files dnd into it
        public DataFlavor[] getDroppableFlavors() {
            return new DataFlavor[]{DataFlavor.javaFileListFlavor};
        }

    }

    public JComponent getWindowUI() {
        return this;
    }

    public String getTitle() {
        return TITLE;
    }

    public Icon getIcon() {
        return ICON;
    }

    public boolean allowClose() {
        return true;
    }

    public JMenuBar getJMenuBar() {
        return Widget.EMPTY_MENU_BAR;
    }

    /**
     * Inner class for opening a file chooser and loading data from files
     */

    class FileOpenAction extends XAction {

        /**
         * Class constructor
         *
         * @param name
         */
        FileOpenAction() {
            this.putValue(Action.NAME, getName());
            this.putValue(Action.SMALL_ICON, getIcon());
            this.putValue(Action.SHORT_DESCRIPTION, getDescription());
        }

        public void actionPerformed(final ActionEvent evt) {

            final XFileChooser fcd = Application.getFileManager().getFileChooser(GseaAppConf.createAllFileFilters());
            final boolean proceed = fcd.showOpenDialog();
            if (proceed) {
                final File[] files = fcd.getSelectedFiles();
                //Printf.outl(files);
                Worker.post(new Job() {
                    public Object run() {
                        Runnable r = ParserFactory.createImportTask(files);
                        Thread t = new Thread(r);
                        // @note else the action makes the vdbgui hang
                        t.setPriority(Thread.MIN_PRIORITY);
                        t.start();
                        return null;
                    }
                });

                // @todo is this needed??
                for (int i = 0; i < files.length; i++) {
                    if (!AuxUtils.isAuxFile(files[i])) {
                        Application.getFileManager().registerRecentlyOpenedFile(files[i]);
                    }
                }
            }
        }

        public String getId() {
            return "FileOpenAction";
        }

        public String getName() {
            return "Browse for files ...";
        }

        public Icon getIcon() {
            return JarResources.getIcon("Open16.gif");
        }

        public String getDescription() {
            return "Open a File and Load its Data into the Application";
        }


    }    // End FileOpenAction

    class LoadLastAnalysisFilesAction extends XAction {

        /**
         * Class constructor
         *
         * @param name
         */
        LoadLastAnalysisFilesAction() {
            this.putValue(Action.NAME, getName());
            this.putValue(Action.SMALL_ICON, getIcon());
            this.putValue(Action.SHORT_DESCRIPTION, getDescription());
        }

        public void actionPerformed(ActionEvent e) {

            Worker.post(new Job() {
                public Object run() {

                    final ReportStub rs = Application.getToolManager().getLastReportStub(new Gsea().getName());

                    if (rs == null) {
                        Application.getWindowManager().showMessage("No history available, nothing loaded!");
                        return null;
                    }

                    // OK, go for it
                    try {
                        final Tool tool = TaskManager.createTool(rs.getReport(true).getProducer().getName());
                        final Properties source_params = rs.getReport(true).getParametersUsed();

                        Runnable runnable = ToolRunnerControl.createLoadToolTask(tool,
                                rs.getName(), true, source_params, fInstance, false);

                        Thread t = new Thread(runnable);
                        t.setPriority(Thread.MIN_PRIORITY);
                        t.start();

                        Application.getWindowManager().showMessage("Data from the last run of this tool is being automagically loaded. They will soon be available as parameter options");

                    } catch (Throwable t) {
                        Application.getWindowManager().showError(t);
                    }

                    return null;
                }
            });
        }

        public String getId() {
            return "LoadLastAction";
        }

        public String getName() {
            return "Load last dataset used";
        }

        public Icon getIcon() {
            return JarResources.getIcon("History16_v2.gif");
        }

        public String getDescription() {
            return "Load datasets used the last time GSAA was run";
        }

    }    // End FileOpenAction


}    // End AppFileExplorerWidget

/*
private Runnable createImportTask(final File file) {
    return new Runnable() {
        public void run() {
            if (file.isDirectory()) {
                Application.getWindowManager().showError("Only files can be choosen (a directory was specified)");
            } else {
                try {
                    final FileInputStream in = new FileInputStream(file);
                    final ProgressMonitorInputStream2 pis = new ProgressMonitorInputStream2(in);
                    ParserFactory.read(file.getPath(), pis, true);    // just gets placed in memory, no other thing
                    log.info("Loaded file: " + file.getPath());
                } catch (Throwable t) {
                    log.error("Parsing trouble", t);
                    Application.getWindowManager().showError(t);
                }
            }
        }
    };
}

*/
