/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xapps.api;

import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.xbench.actions.WidgetAction;
import edu.mit.broad.xbench.core.Widget;
import edu.mit.broad.xbench.tui.SingleToolLauncher;
import xtools.api.Tool;
import xtools.api.param.ParamSet;

import javax.swing.*;


/**
 * @author Aravind Subramanian
 */
public class AppToolLauncherAction extends WidgetAction {

    private Tool fTool;
    private ParamSet fPSet;
    private Icon fIcon;

    private String fOptTitle;

    /**
     * Class constructor
     *
     * @param tool
     * @param pset
     * @param icon
     */
    public AppToolLauncherAction(final Tool tool, final ParamSet pset, final String optTitle, final Icon icon) {

        if (tool == null) {
            throw new IllegalArgumentException("Param tool cannot be null");
        }

        if (pset == null) {
            throw new IllegalArgumentException("Param pset cannot be null");
        }

        this.fTool = tool;
        this.fPSet = pset;
        this.fIcon = icon;
        this.fOptTitle = optTitle;

        this.putValue(Action.NAME, getName());
        this.putValue(Action.SMALL_ICON, fIcon);
        this.putValue(Action.SHORT_DESCRIPTION, getDescription());
    }


    private SingleToolLauncher fStl;

    public Widget getWidget() {
        // IMP dont place this in the class init are -- cause the app to recursively loop
        //setSize(ToolLauncherAction.DEFAULT_DIM.width, ToolLauncherAction.DEFAULT_DIM.height, true);

        if (fStl == null) {
            fStl = new SingleToolLauncher(fTool, fPSet, false, false, false, false, true, fOptTitle, fIcon);
        }

        fStl.revalidate();
        fStl.repaint();
        return fStl;
    }

    public String getId() {
        return "AppToolLauncherAction";
    }

    public String getName() {
        return ParseUtils.getLastToken(fTool.getName(), ".");
    }

    public Icon getIcon() {
        return fIcon;
    }

    public String getDescription() {
        return "Set parameters and perform a new task";
    }

}    // End ToolAction


