package xapps.api;

import com.jidesoft.action.CommandBar;
import com.jidesoft.action.CommandMenuBar;
import com.jidesoft.action.DefaultDockableBarHolder;
import com.jidesoft.action.DockableBarContext;
import com.jidesoft.swing.JideMenu;
import com.jidesoft.swing.JideTabbedPane;
import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.xbench.actions.XAction;
import edu.mit.broad.xbench.core.StatusBar;
import edu.mit.broad.xbench.core.WindowComponent;
import edu.mit.broad.xbench.core.api.*;
import org.apache.log4j.Logger;
import xapps.api.frameworks.fiji.JideWindow;
import xapps.api.vtools.VToolRunner;

import javax.swing.*;
import java.awt.*;
import java.io.File;

/**
 *
 */
//public class BareBonesGUIApplication extends DefaultDockableBarHolder implements DockableHolder, Application.Handler {
public class BareBonesApplication extends DefaultDockableBarHolder implements Application.Handler {

    public static final String BUILD_DATE = "January5_2006_build";

    private static final String FRAME_TITLE = "bare_bones";

    private static final Logger klog = XLogger.getLogger(BareBonesApplication.class);

    // @note IMP IMP: this is the name under which docking prefs etc are stored
    public static final String PROFILE_NAME = "bare_bones";

    // -------------------------------------------------------------------------------------------- //

    private BareBonesApplication fFrame = this;

    private Holder fHolder;

    private final VdbManager fVdbmanager = new VdbManagerImpl("foo");

    private WindowManager fWindowManager = new WindowManagerImpl();

    private FileManager fFileManager;


    /**
     * Class constructor
     *
     * @param meta
     * @param globalMenu
     * @param logger
     * @throws Exception
     */
    // Keep this lean and mean
    public BareBonesApplication() {
        super.setTitle(FRAME_TITLE);

        System.out.println("$$$$$$$$$$$$$$$$$$ BAREBONES APP");

        klog.debug("Started bare ...");

        this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE); // dont exit - there might be other frames

        this.fHolder = new Holder();
        klog.debug("Made holder ...");

        //this.getContentPane().setLayout(new BorderLayout());
        //this.getContentPane().add(fHolder, BorderLayout.CENTER);

        // this.pack(); // <- imp to call before the sizing calls This messes up refresh (remember the junit testing thing)

        int initWidth = 610;
        int initHeight = 600; // @note actual image sizes + a bit

        this.setSize(initWidth, initHeight);
        this.getContentPane().setLayout(new BorderLayout());
        this.getContentPane().add(fHolder, BorderLayout.CENTER);

        this.setJMenuBar(createOnlyFileMenuBar()); // so that open and quit work
        // always want to launch, even if errors

        this.setVisible(true);
    }

    // -------------------------------------------------------------------------------------------- //
    // ------------------------ IMPLEMENT THE APP HANDLER  (ost things delegated to the heper) ---- //
    // -------------------------------------------------------------------------------------------- //

    public String getName() {
        return FRAME_TITLE;
    }

    public boolean isFirstExecutionOfNewVersion() {
        return false;
    }

    public ToolManager getToolManager() {
        throw new NotImplementedException();
    }

    public FileManager getFileManager() {
        if (fFileManager == null) { // cant do at init because frame would be null
            fFileManager = new FileManagerImpl();
        }
        return fFileManager;
    }

    public VdbManager getVdbManager() {
        return fVdbmanager;
    }

    public WindowManager getWindowManager() throws HeadlessException {
        return fWindowManager;
    }

    public StatusBar getStatusBar() {
        return new HeadlessApplication.HeadlessStatusBar();
        //return new StatusBarJideImpl();
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------- IMPLEMENT THE APPLICATION HANDLING DUTIES ----------------------- //
    // -------------------------------------------------------------------------------------------- //

    private CommandBar createOnlyFileMenuBar() {
        CommandBar cb = new CommandMenuBar("Menu Bar");
        cb.setInitSide(DockableBarContext.DOCK_SIDE_NORTH);
        cb.setInitIndex(0);
        cb.setPaintBackground(false);
        cb.setStretch(true);
        cb.setFloatable(true);
        cb.setHidable(false);
        cb.add(createFileMenu());
        return cb;
    }

    public JMenu createFileMenu() {
        JMenuItem item;

        JMenu fileMenu = new JideMenu("File");
        fileMenu.setMnemonic('F');
        fileMenu.addSeparator();

        item = new JMenuItem("Exit", 'x');
        fileMenu.add(item);
        return fileMenu;
    }


    //class Holder extends DockableHolderPanel {
    class Holder extends JideTabbedPane {

        /**
         * Class constructor
         *
         * @param meta
         * @param globalMenu
         * @param logger
         * @throws Exception
         */
        public Holder() {
            //super(fFrame);

            // first split vertically then, horiz
            //getDockingManager().setInitSplitPriority(DockingManager.SPLIT_SOUTH_NORTH_EAST_WEST);
            // add all dockable frames
            //getDockingManager().beginLoadLayoutData();

        }

        public edu.mit.broad.xbench.core.Window openWindow(final WindowComponent wc) {
            //wc.getWindowUI().revalidate();
            this.addTab(wc.getTitle(), wc.getIcon(), wc.getWindowUI());
            return new JideWindow(wc.getTitle(), wc.getWindowUI());
        }

        public void clearUp() {
        }

    } // End class Holder


    class WindowManagerImpl extends AbstractWindowManager {

        private VToolRunner fRunner;

        WindowManagerImpl() {
            super(fFrame);
        }

        public edu.mit.broad.xbench.core.Window openWindow(final WindowComponent wc) {
            return fHolder.openWindow(wc);
        }

        public JPopupMenu createPopupMenu(final Object obj) {
            klog.debug("Doing nothing for createPopupMenu: " + obj);
            return null;
        }

        public XAction createAction(final Class clazz, final File files[]) {
            throw new NotImplementedException();
        }

        public boolean runDefaultAction(final Object obj) {
            throw new NotImplementedException();
        }

        public void runModalTool(final VTool tool, final DialogType dt) {
            if (fRunner == null) {
                fRunner = new VToolRunner();
            }

            fRunner.showRunner(tool, dt);
        }

    } // End class WindowManagerImpl

} // End class BareBonesApplication
