package xapps.api.frameworks;

import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.xbench.actions.*;
import edu.mit.broad.xbench.actions.ext.BrowserAction;
import edu.mit.broad.xbench.actions.ext.ExcelAction;
import edu.mit.broad.xbench.actions.ext.OsExplorerAction;
import edu.mit.broad.xbench.actions.ext.TextpadAction;
import edu.mit.broad.xbench.dataio.ParserAction;
import org.apache.log4j.Logger;

import javax.swing.*;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Aravind Subramanian
 *         Date: Dec 23, 2005
 *         Time: 7:51:56 PM
 */
public abstract class AbstractActionLookup {


    private static final Logger klog = XLogger.getLogger(AbstractActionLookup.class);

    /**
     * hack to fix bug when multiple files are right clicked in explorer and
     * quickly dnd'ed into the desktop
     * java's desktop pane cant keep up and synch issues occur
     * no harm, no fowl, so just ignore --- the frame still gets added
     * just not positioned great
     */
    public static final int HACKNUM = 1;

    /**
     * key -> ext, value -> Actions that are possible on the ext
     */
    protected final Map fExtActionsMap = new HashMap();

    protected AbstractActionLookup() {

    }

    /**
     * Note sub classes must implement this method
     *
     * @param obj
     * @return
     */
    protected abstract PobActions lookupActions(final Object obj);

    /**
     * @param file
     * @return
     * @maint file association - popup menu options are set here.
     */
    public JPopupMenu createPopup(final File file) {

        String ext = NamingConventions.getExtension(file);
        Object obj = fExtActionsMap.get(ext);

        if (obj == null) {

            // unknown files can at the least be opened in text editor / browser / excel
            JPopupMenu p = new JPopupMenu();

            if (file.isDirectory() == false) {
                p.add(new TextpadAction(file.getPath()));
                p.add(new ExcelAction(file.getPath()));
                p.add(new BrowserAction(file.getPath()));
            }

            p.add(new OsExplorerAction(file.getPath()));

            return p;
        } else {
            PobActions da = (PobActions) obj;
            return _createPopup(da.allActions, file);
        }
    }


    /**
     * @param obj
     * @return
     * @maint Object <-> actions mapping mainatined here
     */
    public JPopupMenu createPopup(final Object obj) {

        //klog.debug("createPopup Object: " + obj);

        if (obj instanceof File) {
            return createPopup((File) obj);
        } else {

            final PobActions pa = lookupActions(obj);
            final XAction[] actions = pa.allActions;

            if (actions != null && actions.length > 0) {
                return createPopupForObject(actions, obj);
            } else {
                return new JPopupMenu();
            }
        }
    }

    // ditto as file -> except no parser included as object already parsed!
    public JPopupMenu createPopupForObject(final XAction[] actions, final Object obj) {

        JPopupMenu menu = new JPopupMenu();
        List list = new ArrayList();

        for (int i = 0; i < actions.length; i++) {
            if (actions[i] == null) {
                list.add(new JSeparator());
            } else if (actions[i] instanceof ParserAction) {

            } else {
                try {
                    list.add(new JMenuItem(createAction(actions[i], obj)));
                } catch (Throwable e) {
                    klog.error("Error making popup", e);
                    list.add(new JMenuItem("Error making popup: " + e));
                }
            }
        }

        for (int i = 0; i < list.size(); i++) {
            menu.add((JComponent) list.get(i));
        }

        return menu;
    }

    private JPopupMenu _createPopup(XAction[] actions, File file) {

        JPopupMenu menu = new JPopupMenu();
        if (actions == null) {
            return menu;
        }

        List list = new ArrayList();

        for (int i = 0; i < actions.length; i++) {
            if (actions[i] == null) {
                list.add(new JSeparator());
            } else {
                try {
                    JMenuItem item = new JMenuItem(createAction(actions[i], file));

                    list.add(item);
                } catch (Throwable e) {
                    klog.error(e);
                    list.add(new JMenuItem("Error making popup: " + e));
                }
            }
        }

        for (int i = 0; i < list.size(); i++) {
            menu.add((JComponent) list.get(i));
        }

        return menu;
    }

    /**
     * data can be a File, an Object
     *
     * @param action
     * @param data
     * @return
     */
    public XAction createAction(final XAction action, final Object data) throws Exception {

        if (action instanceof FileObjectAction) {
            return (new ProxyFileObjectAction((FileObjectAction) action, data));
        } else if (action instanceof ObjectAction) {
            return (new ProxyObjectAction((ObjectAction) action, data));
        } else if ((action instanceof FileAction) && (data instanceof File)) { // @note file action and a file type
            return (new ProxyFileAction((FileAction) action, (File) data));
        } else if ((action instanceof FileAction) && data instanceof PersistentObject &&
                (ParserFactory.getCache().isCached((PersistentObject) data))) { // @note file action and a file type
            return (new ProxyFileAction((FileAction) action, ParserFactory.getCache().getSourceFile(data)));
        } else if ((action instanceof FilesAction) && (data instanceof File[])) {
            return (new ProxyFileAction((FilesAction) action, (File[]) data));
        } else if ((action instanceof ExtAction) && (data instanceof File)) {
            Class cl = action.getClass();
            ExtAction real = (ExtAction) cl.newInstance();
            real.setPath(((File) data).getPath());
            return real;
        } else if ((action instanceof ExtAction) && (data instanceof PersistentObject)) {
            Class cl = action.getClass();
            ExtAction real = (ExtAction) cl.newInstance();
            real.setPath(ParserFactory.getCache().getSourceFile(data).getPath());
            return real;
        } else if (action instanceof WidgetAction) {    // simple, data-less widget opening
            Class cl = action.getClass();
            return (WidgetAction) cl.newInstance();
        } else {
            throw new Exception("Unknown action type: " + action + " and object combo: " + data);
        }
    }


    /**
     * Only works for Actions that have parameter less constructors
     *
     * @param actionClass
     * @param data
     * @return
     * @throws Exception
     */
    public XAction createAction(Class actionClass, Object data) {

        try {

            XAction action = (XAction) actionClass.newInstance();

            return createAction(action, data);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}

/* IMP DONT DELETE THESE YET
DEPRECATED DEC 2005 with jide


public JPopupMenu createPopupMenu(FtpFile ffile) {
    JPopupMenu menu = new JPopupMenu();
    String ext = ffile.getExtension();

    Object obj = fExtActionsMap.get(ext);

    if (obj == null) {
        klog.warn("No actions for: " + ext);
        return menu;
    }

    PobActions da = (PobActions) obj;
    XAction[] actions = da.allActions;

    if (actions == null) {
        return menu;
    }

    List list = new ArrayList();

    for (int i = 0; i < actions.length; i++) {
        if (actions[i] == null) {
            list.add(new JSeparator());
        } else {
            try {
                Class cl = actions[i].getClass();
                XAction real = (XAction) cl.newInstance();
                XAction wrapper = createAction(real, ffile);
                JMenuItem item = new JMenuItem(wrapper);

                list.add(item);
            } catch (Throwable e) {
                klog.error("error making popup", e);
                list.add(new JMenuItem("Error making popup: " + e));
            }
        }
    }

    for (int i = 0; i < list.size(); i++) {
        menu.add((JComponent) list.get(i));
    }

    return menu;
}


public boolean runDefaultAction(final File file) {

    String ext = NamingConventions.getExtension(file);
    Object obj = fExtActionsMap.get(ext);

    if (obj == null) {
        klog.info("No explicit actions for: " + file + " ... running textpad anyway");
        TextpadAction action = new TextpadAction(file.getPath());
        action.actionPerformed(null);
        //Application.getWindowManager().showMessage("No known action for: " + file.getAbsolutePath());
    } else {
        try {
            PobActions xas = (PobActions) obj;
            XAction action = xas.getDefaultFileAction();
            action = createAction(action, file);
            action.actionPerformed(null);
        } catch (Exception e) {
            klog.error(e);
            //Application.getWindowManager().showError("Could not run Default action", e);
            return false;
        }
    }

    return true;
}


public boolean runDefaultAction(Object obj) {

    if (obj == null) {
        klog.debug("Null object -- not running any default action");
        return false;
    }

    boolean error = false;

    if (obj instanceof File) {
        return runDefaultAction((File) obj);
    } else if (obj instanceof List) {
        List objs = (List) obj;

        boolean sleep = false;
        if (objs.size() > HACKNUM) {
            sleep = true;
            //log.debug("Intentionallly slowing things down: " + objs.size());
        }

        for (int i = 0; i < objs.size(); i++) {
            try {
                runDefaultAction(objs.get(i));
                if ((sleep) && (i >= HACKNUM)) {
                    Thread.sleep(50);
                }
            } catch (InterruptedException ie) {
                error = true;
                klog.error("hack backfired", ie);
            }
        }

    } else {
        PobActions da = lookupActions(obj);
        XAction[] actions = da.allActions;

        if ((actions == null) || (actions.length == 0)) {
        } else {
            try {
                XAction action = createAction(actions[0], obj);
                action.actionPerformed(null);
            } catch (Exception e) {
                error = true;
            }
        }
    }

    return error;
}

*/