package xapps.api.vtools;

import com.jidesoft.swing.JideButton;
import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.reports.api.ReportIndexState;
import edu.mit.broad.genome.reports.api.ToolReport;
import edu.mit.broad.genome.utils.ClassUtils;
import edu.mit.broad.xbench.core.api.Application;
import edu.mit.broad.xbench.core.api.DialogType;
import edu.mit.broad.xbench.core.api.VTool;
import org.apache.log4j.Logger;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Jan 7, 2006
 *         Time: 10:22:04 PM
 */
public abstract class AbstractVTool extends AbstractAction implements VTool {

    private DialogType fDialogType;

    protected static final transient Logger klog = XLogger.getLogger(AbstractVTool.class);

    // @todo assess if static is ok?
    protected static ToolReport kReport;

    private String fName;

    /**
     * Class constructor
     *
     * @param name
     * @param icon
     * @param dt
     */
    public AbstractVTool(final String name, final Icon icon, final DialogType dt) {
        super(name, icon);
        this.fDialogType = dt;
        this.fName = name;

        if (kReport == null) {
            klog.debug("Making vtoolreport");
            final File dir = Application.getVdbManager().getDefaultOutputDir();
            kReport = new ToolReport(null, false, dir, new ReportIndexState(false, false, false, null));
        }

    }

    public String getName() {
        return fName;
    }

    public void actionPerformed(final ActionEvent evt) {
        Application.getWindowManager().runModalTool(this, fDialogType);
    }

    protected void showSavedMessage(File file) {
        Application.getWindowManager().showMessage("Saved result to : " + file.getPath());
    }

    protected void showSavedMessage() {
        Application.getWindowManager().showMessage("Saved result to : " + kReport.getReportDir());
    }

    protected void showError(final Throwable t) {
        Application.getWindowManager().showError(t);
    }

    protected void showError(final String msg) {
        Application.getWindowManager().showError(msg);
    }

    public String getHelpURL() {
        //return JarResources.getHelpURL(getName());
        return JarResources.getHelpURL(this.getClass().getName());
    }

    protected static JButton createButton(final VTool tool) {
        JideButton but = new JideButton(tool);
        String cn = ClassUtils.shorten(tool.getClass());
        but.setToolTipText(tool.getTitle() + " [" + cn + "]"); // for debugging
        but.setButtonStyle(JideButton.TOOLBOX_STYLE);
        but.setPreferredSize(new Dimension(100, 20));
        return but;
    }
    // -------------------------------------------------------------------------------------------- //
    // ------------------------------------ HELPER DIALOG COMPONENTS ------------------------------ //
    // -------------------------------------------------------------------------------------------- //

    public static class ShowMessageComponent extends JPanel {

        public JTextField tfName;

        private JTextArea taMessage;

        public ShowMessageComponent(final String objName,
                                    final boolean askForAName) {

            this.setBorder(BorderFactory.createEmptyBorder(10, 10, 0, 10));
            this.setLayout(new BorderLayout());

            taMessage = new JTextArea(10, 60);
            this.add(new JScrollPane(taMessage), BorderLayout.CENTER);

            if (askForAName) {
                this.tfName = new JTextField(60);
                tfName.setBorder(BorderFactory.createTitledBorder("Enter a short name for the " + objName));
                this.add(tfName, BorderLayout.SOUTH);
            }

        }

        public void setMessage(String msg) {
            taMessage.setText(msg);
        }

    } // End class GetFileNameComponent


} // End class AbstractModalAction

/*
static class MyTool implements Tool {

    public void execute() throws Exception {
    }

    public ToolCategory getCategory() {
        throw new NotImplementedException();
    }

    public ToolReport getReport() {
        throw new NotImplementedException();
    }

    public void declareParams() {
    }

    public String getHelpURL() {
        throw new NotImplementedException();
    }

    public ParamSet getParamSet() {
        throw new NotImplementedException();
    }

    public String getName() {
        throw new NotImplementedException();
    }

    public String getTitle() {
        throw new NotImplementedException();
    }

    public String getDesc() {
        throw new NotImplementedException();
    }

    public long getExecutionTime() {
        throw new NotImplementedException();
    }

    public long getStartTime() {
        throw new NotImplementedException();
    }

    public long getStopTime() {
        throw new NotImplementedException();
    }

    public void printfUsage() {
    }

    public void requestKill() {
    }

    public void setOutputStream(final PrintStream sout) {
    }

}
*/

/*
public File save(final PersistentObject pob, final boolean showSavedItDialog) throws Exception {
    File file = createFile(pob.getName(), DataFormat.getExtension(pob));
    ParserFactory.save(pob, file);
    if (showSavedItDialog) {
        Application.getWindowManager().showMessage("Saved result to : " + file.getPath());
    }
    return file;
}

public File[] save(final PersistentObject[] pobs, final boolean showSavedItDialog) throws Exception {

    File[] files = new File[pobs.length];
    for (int i = 0; i < pobs.length; i++) {
        files[i] = createFile(pobs[i].getName(), DataFormat.getExtension(pobs[i]));
        ParserFactory.save(pobs[i], files[i]);
    }

    if (files.length > 0 && showSavedItDialog) {
        Application.getWindowManager().showMessage("Saved " + pobs.length + " results to : " + files[0].getParent());
    }

    return files;
}

public File save(final String name, final BufferedImage buf) throws Exception {
    File file = createFile(name, "png");
    ImageUtils.savePng(buf, file);
    Application.getWindowManager().showMessage("Saved image to : " + file.getPath());
    return file;
}
*/