package xapps.api.vtools.stackables;

import com.jidesoft.popup.JidePopup;
import com.jidesoft.swing.JideButton;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.ColorScheme;
import edu.mit.broad.genome.math.ColorSchemes;
import edu.mit.broad.genome.math.ScaleMode;
import edu.mit.broad.genome.swing.GuiHelper;
import org.apache.log4j.Logger;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.image.BufferedImage;

/**
 * @author Aravind Subramanian
 *         Date: Jan 9, 2006
 *         Time: 1:12:33 PM
 */
public class ManagerHelper {

    protected static final Logger klog = XLogger.getLogger(StackablesManager.class);

    // images any bigger take too much memory
    protected static final int IMAGE_MAX = 5000;

    public static JComponent createColorMapChooser(final java.util.List stackables) {

        final ColorScheme[] cs = ColorSchemes.createAll();
        final JList list = new JList(cs);
        GuiHelper.safeSelect(list);

        list.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                final ColorScheme cs = (ColorScheme) list.getSelectedValue();
                for (int i = 0; i < stackables.size(); i++) {
                    Stackable st = (Stackable) stackables.get(i);
                    if (st instanceof HStackables.HeatMapComponent) {
                        ((HStackables.HeatMapComponent) st).setColorScheme(cs);
                    }
                }
            }
        });

        return list;
    }

    public static JComponent createScaleModeChooser(final java.util.List stackables) {

        final ScaleMode[] sm = ScaleMode.ALL;
        final JList list = new JList(sm);
        GuiHelper.safeSelect(list);

        list.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                final ScaleMode sm = (ScaleMode) list.getSelectedValue();
                for (int i = 0; i < stackables.size(); i++) {
                    Stackable st = (Stackable) stackables.get(i);
                    if (st instanceof HStackables.HeatMapComponent) {
                        ((HStackables.HeatMapComponent) st).setScaleMode(sm);
                    }
                }
            }
        });

        return list;
    }

    public static void setGridSize(final int size, final java.util.List stackables) {

        for (int i = 0; i < stackables.size(); i++) {
            Stackable stackable = (Stackable) stackables.get(i);

            if (stackable instanceof Stackable.Vertical) {
                ((Stackable.Vertical) stackable).setCellWidth(size);
            } else if (stackable instanceof Stackable.Horizontal) {
                ((Stackable.Horizontal) stackable).setCellHeight(size);
            }
        }
    }

    public static void showInPopup(final Component comp, final Component parent, boolean attachable) {
        showInPopup(comp, parent, attachable, null);
    }

    public static void showInPopup(final Component comp,
                                   final Component parent,
                                   boolean attachable,
                                   Dimension dim_opt) {

        final JidePopup popup = new JidePopup();
        popup.setAttachable(attachable);
        popup.getContentPane().setLayout(new BorderLayout());

        popup.getContentPane().add(comp);

        popup.setDefaultFocusComponent(comp);

        popup.setOwner(parent);
        popup.setResizable(true);
        popup.setMovable(true);

        if (dim_opt != null) {
            popup.setPreferredSize(dim_opt);
        }

        if (popup.isPopupVisible()) {
            popup.hidePopup();
        } else {
            popup.showPopup();
        }
    }

    public static JButton createButton(final Action action) {
        JideButton but = new JideButton(action);
        but.setButtonStyle(JideButton.TOOLBOX_STYLE);
        but.setPreferredSize(new Dimension(100, 20));
        return but;
    }


    public static BufferedImage createBufferedImage(final JComponent comp) {

        int height = comp.getHeight();
        int width = comp.getWidth();


        if (height > IMAGE_MAX) {
            height = IMAGE_MAX;
        }

        if (width > IMAGE_MAX) {
            width = IMAGE_MAX;
        }

        klog.debug(comp.getWidth() + " " + comp.getHeight() + " rh: " + height);

        BufferedImage bi = new BufferedImage(width, height, 1);
        //log.debug("Done image creation");
        Graphics2D g2d = (Graphics2D) bi.getGraphics();

        // Anti-aliasing to smoothen out the font
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

        comp.paint(g2d);

        g2d.dispose();

        return bi;
    }


} // End class ManagerHelper
