package xapps.api.vtools.stackables;

import com.jidesoft.swing.JideSplitPane;

import javax.swing.*;
import javax.swing.table.TableColumn;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * @author Aravind Subramanian
 *         Date: Jan 1, 2006
 *         Time: 9:45:40 PM
 */
public interface Stackable {

    public static final int MAX_FONT_SIZE = 14;
    public static final Font DEFAULT_FONT = new Font("Arial", Font.PLAIN, 10);

    public JComponent getComponent();

    public BufferedImage createImage();

    public interface Horizontal extends Stackable {

        public void setCellHeight(int cellHeight);

    }

    public interface Vertical extends Stackable {

        public void setCellWidth(int cellWidth);

    }

    public static class Helper {

        public static void setTableAppearance(final JTable table) {
            table.setShowGrid(true);
            table.setGridColor(new Color(239, 239, 255));
            table.setRowSelectionAllowed(false);
            table.setCellSelectionEnabled(true);
            table.setColumnSelectionAllowed(true);

        }

        public static void setColWidth(final JTable table, final int columnWidth) {
            for (int i = 0; i < table.getColumnCount(); i++) {
                TableColumn c = table.getColumnModel().getColumn(i);
                c.setMinWidth(columnWidth);
                c.setMaxWidth(columnWidth);
                c.setPreferredWidth(columnWidth);
                c.setWidth(columnWidth);
            }
        }

        public static void setColWidth(final JTable table, final int colindex, final int columnWidth) {
            TableColumn c = table.getColumnModel().getColumn(colindex);
            c.setMinWidth(columnWidth);
            c.setMaxWidth(columnWidth);
            c.setPreferredWidth(columnWidth);
            c.setWidth(columnWidth);
        }

        public static void synchVerticalScrolling(final JScrollPane[] scrolls) {
            if (scrolls.length < 2) {
                return;
            }

            JScrollPane root = scrolls[0];

            for (int i = 1; i < scrolls.length; i++) {
                scrolls[i].getVerticalScrollBar().setModel(root.getVerticalScrollBar().getModel());
            }
        }

        public static void synchSplitPanes(final JideSplitPane paneA, final JideSplitPane paneB) {
            attachSplitPane(paneA, paneB);
            attachSplitPane(paneB, paneA);
        }

        public static void attachSplitPane(final JideSplitPane whenIMoveThis, final JideSplitPane makeThisAlsoMove) {
            whenIMoveThis.addPropertyChangeListener(new PropertyChangeListener() {
                public void propertyChange(PropertyChangeEvent evt) {
                    makeThisAlsoMove.setDividerLocation(0, whenIMoveThis.getDividerLocation(0));
                }
            });
        }

        public static void synchHorizScrolling(final JScrollPane[] scrolls) {
            if (scrolls.length < 2) {
                return;
            }

            JScrollPane root = scrolls[0];

            for (int i = 1; i < scrolls.length; i++) {
                scrolls[i].getHorizontalScrollBar().setModel(root.getHorizontalScrollBar().getModel());
            }
        }

    }

} // End interface Stackable
