package xapps.api.vtools.stackables;

import com.jidesoft.status.LabelStatusBarItem;
import com.jidesoft.swing.RangeSlider;
import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.math.ObjectMatrix;
import edu.mit.broad.genome.swing.VerticalFlowLayout;
import xapps.api.vtools.Hooks;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;

/**
 * @author Aravind Subramanian
 *         Date: Jan 2, 2006
 *         Time: 12:18:39 PM
 */
public class StackablesManager extends ManagerHelper {

    private java.util.List fStackables;

    private Hooks.Matrix hook;

    private ObjectMatrix fStackableMatrix;

    private int fGridSize;

    private LabelStatusBarItem fStatusBarLabelItem;

    public static ToolBarMode FULL = new ToolBarMode("full");

    public static ToolBarMode SIMPLE = new ToolBarMode("simple");

    private ToolBarMode fToolBarMode;

    public static class ToolBarMode {
        String name;

        ToolBarMode(String name) {
            this.name = name;
        }

        public int hashCode() {
            return name.hashCode();
        }

        public boolean equals(Object obj) {
            if (obj instanceof ToolBarMode) {
                return ((ToolBarMode) obj).name.equals(name);
            }
            return false;
        }

    } // End class ToolBarMode


    public StackablesManager(final Hooks.Matrix hook,
                             final int numRowsOfImages,
                             final int numColsOfImages) {
        this(hook, numRowsOfImages, numColsOfImages, null);
    }

    /**
     * Class constructor
     */
    public StackablesManager(final Hooks.Matrix hook,
                             final int numRowsOfImages,
                             final int numColsOfImages,
                             final ToolBarMode toolBarMode) {

        this.hook = hook;
        this.fStackables = new ArrayList();
        this.fToolBarMode = toolBarMode;
        this.fStackableMatrix = new ObjectMatrix(numRowsOfImages, numColsOfImages);
        registerTablesForSavingData(hook.fRowTable, hook.fColTable, hook.stl_opt);
    }


    public JComponent createToolBar() {
        final JPanel panel = new JPanel(new VerticalFlowLayout());
        panel.add(new CreateGridSizeAction().but); // @note adding the button not the action


        final LabelStatusBarItem sblItem = new LabelStatusBarItem();
        sblItem.setForeground(Color.LIGHT_GRAY);
        sblItem.setText("Dim: " + hook.fDataset.getNumRow() + "R x " + hook.fDataset.getNumCol() + "C");
        panel.add(sblItem);

        fStatusBarLabelItem = new LabelStatusBarItem();
        fStatusBarLabelItem.setForeground(Color.BLUE);
        panel.add(fStatusBarLabelItem);

        return panel;
    }


    public void add(final int row, final int col, final Stackable stackable) {
        fStackables.add(stackable);

        Object curr = fStackableMatrix.getElement(row, col);
        if (curr != null) {
            throw new IllegalArgumentException("Overwriting extant Stackable at row: " + row + " col: " + col);
            //klog.warn("Overwriting extant Stackable at row: " + row + " col: " + col);
        }

        fStackableMatrix.setElement(row, col, stackable);
    }

    // public for setting size during init
    public void setGridSize(final int size) {
        if (this.fGridSize != size) {
            setGridSize(size, fStackables);
            this.fGridSize = size;
        }
    }

    public void selectRows(int startIndex, int endIndex) {
        //klog.debug("Selecting startIndex: " + startIndex + " endIndex: " + endIndex);
        if (hook.fRowTable != null) {
            hook.fRowTable.getSelectionModel().setSelectionInterval(startIndex, endIndex);
            updateStatusBar();
        }

    }

    public void selectColumns(int startIndex, int endIndex) {
        //klog.debug("Selecting startIndex: " + startIndex + " endIndex: " + endIndex);

        if (hook.fColTable != null) {
            hook.fColTable.getSelectionModel().setSelectionInterval(startIndex, endIndex);
            updateStatusBar();
            updateSampleTemplateHeader();
        }

    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    // for saving
    //public void registerTablesForSavingData(final HStackables.HeatMapComponent comp, final JTable featureTable, final JTable sampleTable) {

    private void registerTablesForSavingData(final JTable featureTable,
                                             final JTable sampleTable,
                                             final VStackables.SampleAndTemplateLabels stl_opt) {

        if (stl_opt != null) {
            sampleTable.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent e) {
                    updateSampleTemplateHeader();
                    updateStatusBar();
                }
            });

            sampleTable.addMouseMotionListener(new MouseMotionAdapter() {
                public void mouseDragged(MouseEvent e) {
                    updateSampleTemplateHeader();
                    updateStatusBar();
                }
            });
        }

        featureTable.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                updateStatusBar();
            }
        });

        featureTable.addMouseMotionListener(new MouseMotionAdapter() {
            public void mouseDragged(MouseEvent e) {
                updateStatusBar();
            }
        });

    }

    private void updateSampleTemplateHeader() {
        if (hook.stl_opt != null) {
            final int[] cols = hook.fColTable.getSelectedColumns();
            //klog.debug(">>> " + cols);
            hook.stl_opt.setSelectedIndices(cols);
        }
    }

    private void updateStatusBar() {
        fStatusBarLabelItem.setText("Sel: " + hook.fRowTable.getSelectedRowCount() + "R" + " x " + hook.fColTable.getSelectedColumnCount() + "C");
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    /**
     * Inner class for the color map
     */
    class CreateColorMapAction extends AbstractAction {

        private JButton but;

        CreateColorMapAction() {
            super("Color Map", JarResources.getIcon("ColorMap.gif"));
            but = createButton(this);
        }

        public void actionPerformed(ActionEvent evt) {
            showInPopup(createColorMapChooser(fStackables), but, false);
        }

    }

    class CreateScaleModeAction extends AbstractAction {

        private JButton but;

        CreateScaleModeAction() {
            super("Scale Mode", JarResources.getIcon("ScaleMode.gif"));
            but = createButton(this);
        }

        public void actionPerformed(ActionEvent evt) {
            showInPopup(createScaleModeChooser(fStackables), but, false);
        }

    }

    class ShowLegendAction extends AbstractAction {

        private JButton but;

        ShowLegendAction() {
            super("Legend", JarResources.getIcon("Legend16.gif"));
            but = createButton(this);
        }

        public void actionPerformed(ActionEvent evt) {
            HStackables.HeatMapComponent hmc = _getHmc();
            if (hmc != null) {
                showInPopup(hmc.getLegend(), but, false);
            }
        }
    } // End class ShowLegendAction

    private HStackables.HeatMapComponent _getHmc() {

        for (int i = 0; i < fStackables.size(); i++) {
            Stackable st = (Stackable) fStackables.get(i);
            if (st instanceof HStackables.HeatMapComponent) {
                return ((HStackables.HeatMapComponent) st);
            }
        }

        return null;
    }

    /**
     * Inner class for the grid size
     * Sizes the grid
     * Not a modal action
     */
    class CreateGridSizeAction extends AbstractAction {

        private JButton but;
        // private JTextField tfSize;

        CreateGridSizeAction() {
            super("Grid Size", JarResources.getIcon("grid_size_small.gif"));
            but = createButton(this);
        }

        public void actionPerformed(ActionEvent evt) {
            showInPopup(createRangePanel(), but, false);

            // cant do this because the call to show popup is not blocking
            /*
            System.out.println(">>>> " + tfSize.getText());

            try {
                if (tfSize != null) {
                    setGridSize(Integer.parseInt(tfSize.getText()));
                }
            } catch (Throwable t) {
                Application.getWindowManager().showError(t);
            }
            */
        }

        private JPanel createRangePanel() {
            final JTextField minField = new JTextField();
            final JTextField samField = new JTextField();

            final RangeSlider rangeSlider = new RangeSlider(2, 200, fGridSize, 100); // min max, low high
            rangeSlider.setPaintTicks(true);
            rangeSlider.setMajorTickSpacing(10);
            rangeSlider.addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    if (rangeSlider.getModel().getValueIsAdjusting() == false) {
                        minField.setText("" + rangeSlider.getLowValue());
                        samField.setText("" + rangeSlider.getHighValue());
                        setGridSize(rangeSlider.getLowValue());
                    }

                }
            });

            minField.setText("" + rangeSlider.getLowValue());
            samField.setText("" + rangeSlider.getHighValue());

            JPanel minPanel = new JPanel(new BorderLayout());
            minPanel.add(new JLabel("Grid size"), BorderLayout.BEFORE_FIRST_LINE);
            minField.setEditable(true);
            minField.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    int value = Integer.parseInt(minField.getText());
                    rangeSlider.setLowValue(value);
                    setGridSize(value);
                }
            });

            minPanel.add(minField);

            JPanel samPanel = new JPanel(new BorderLayout());
            samPanel.add(new JLabel("Sample height", SwingConstants.TRAILING), BorderLayout.BEFORE_FIRST_LINE);
            samField.setEditable(true);
            samPanel.add(samField);

            JPanel textFieldPanel = new JPanel(new GridLayout(1, 3));
            textFieldPanel.add(minPanel);
            textFieldPanel.add(new JPanel());
            textFieldPanel.add(samPanel);

            JPanel panel = new JPanel(new BorderLayout());
            panel.add(rangeSlider, BorderLayout.CENTER);
            panel.add(textFieldPanel, BorderLayout.AFTER_LAST_LINE);

            return panel;
        }

    }

} // End class StackablesManager