package xapps.api.vtools.stackables;

import edu.mit.broad.genome.objects.ColorMap;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.swing.GVerticalLabel;
import edu.mit.broad.genome.swing.GuiHelper;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import gnu.trove.TIntArrayList;
import org.genepattern.clustering.hierarchical.ArrayTreePanel;
import org.genepattern.clustering.hierarchical.AtrGtrReader;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import java.awt.*;
import java.awt.image.BufferedImage;

/**
 * @author Aravind Subramanian
 *         Date: Jan 13, 2006
 *         Time: 6:06:42 PM
 */
public class VStackables {

    /**
     * Class for the sample and template legends
     */
    public static class SampleAndTemplateLabels implements Stackable.Vertical {

        private Template fTemplate_opt;

        private JTable fTable;

        private final GVerticalLabel vlabel = new GVerticalLabel();

        private int fWidth = 10;

        private int fHeaderHeight = 100;

        /**
         * Class constructor
         *
         * @param ds
         * @param t_opt
         */
        public SampleAndTemplateLabels(final Dataset ds,
                                       final Template t_opt) {

            this.fTemplate_opt = t_opt;

            // build the sample header table
            DefaultTableModel model = new DefaultTableModel();
            for (int c = 0; c < ds.getNumCol(); c++) {
                model.addColumn(ds.getColumnName(c));
            }

            this.fTable = new JTable(model);

            /*no effect
            vlabel.addMouseListener(new MouseAdapter() {
                        public void mouseClicked(MouseEvent e) {
                            System.out.println(">>>>> ");
                            vlabel.setForeground(Color.YELLOW);
                        }
                    });
            */


            fTable.getTableHeader().setDefaultRenderer(createHeaderRenderer());
            fTable.getTableHeader().setReorderingAllowed(false);
            fTable.getTableHeader().setResizingAllowed(false);

            // -------------------------------------------------------------------------------------------- //

            fTable.setGridColor(Color.BLACK);
            fTable.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
            fTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

        }

        public JComponent getComponent() {
            return fTable.getTableHeader();
        }

        public BufferedImage createImage() {
            return ManagerHelper.createBufferedImage(getComponent());
        }

        public void setCellWidth(final int cellWidth) {
            this.fWidth = cellWidth;
            fTable.setFont(new Font(fTable.getFont().getName(), fTable.getFont().getStyle(), cellWidth));
            Stackable.Helper.setColWidth(fTable, cellWidth);
        }

        // renders col labels for a bpog table.
        // labels are renderred vertically using java2d so that they fit better
        // labels can optionally be colored bg by a template
        private DefaultTableCellRenderer createHeaderRenderer() {
            return new DefaultTableCellRenderer() {
                public Component getTableCellRendererComponent(final JTable table,
                                                               final Object value,
                                                               final boolean isSelected,
                                                               final boolean hasFocus,
                                                               final int row,
                                                               final int col) {

                    // our own font for header as its vertical and need to synch its size with row size
                    int fsize = fWidth - 3;
                    if (fsize > Stackable.MAX_FONT_SIZE) { // not nice if its too big
                        fsize = 12;
                    }
                    Font font = new Font(Stackable.DEFAULT_FONT.getName(), Stackable.DEFAULT_FONT.getStyle(), fsize);


                    vlabel.setFont(font);

                    // table selection doesnt seem to work

                    if (isSelected == false) {
                        if (fTemplate_opt == null) {
                            vlabel.setForeground(Color.WHITE);
                        } else {
                            //System.out.println(">> " + fCds.getColumnName(col) + "\t" + fTemplate.getItemColor(col));
                            Color c = fTemplate_opt.getItemColor(col);
                            if (c != null) {
                                //log.debug("setting color: " + c);
                                //vlabel.setBackground(c); // no effect??
                                vlabel.setForeground(c);
                            }
                        }
                    } else {
                        vlabel.setForeground(Color.YELLOW);
                    }

                    if (selIndices != null && selIndices.contains(col)) {
                        vlabel.setForeground(Color.YELLOW);
                    }

                    vlabel.setText(value.toString());
                    vlabel.setPreferredSize(new Dimension(fWidth, fHeaderHeight));
                    vlabel.setToolTipText(value.toString());
                    //vlabel.setBorder(BorderFactory.createLineBorder(Color.WHITE, 3)); no effect
                    return vlabel;
                }

                /* has no handle on curr selected etc
                public void paintComponent(Graphics g) {

                    int col = 1;
                    boolean isSelected = false;
                    Object value = "foo";

                    // our own font for header as its vertical and need to synch its size with row size
                    int fsize = fWidth - 3;
                    if (fsize > Stackable.MAX_FONT_SIZE) { // not nice if its too big
                        fsize = 12;
                    }
                    Font font = new Font(Stackable.DEFAULT_FONT.getName(), Stackable.DEFAULT_FONT.getStyle(), fsize);

                    Graphics2D g2 = (Graphics2D) g;

                    if (isSelected == false) {
                        if (fTemplate_opt == null) {
                            g2.setColor(Color.WHITE);
                        } else {
                            //System.out.println(">> " + fCds.getColumnName(col) + "\t" + fTemplate.getItemColor(col));
                            Color c = fTemplate_opt.getItemColor(col);
                            if (c != null) {
                                g2.setColor(c);
                            }
                        }
                    } else {
                        g2.setColor(Color.YELLOW);
                    }

                    if (selIndices != null && selIndices.contains(col)) {
                        g2.setColor(Color.YELLOW);
                    }


                    g2.fillRect(0, 0, this.getWidth(), this.getHeight());    // this needs to be there for the fg color to show up
                    //Oriented from top to bottom
                    g2.setColor(Color.black);
                    g2.setFont(font);
                    AffineTransform at = AffineTransform.getRotateInstance(Math.toRadians(90));
                    g2.transform(at);
                    g2.drawString(value.toString(), 0, 0);

                    this.setToolTipText(value.toString());
                    g2.dispose();
                }
                */

            }; // End class SampleAndTemplateLabels


        }


        private TIntArrayList selIndices;

        public void setSelectedIndices(int[] sel) {
            if (selIndices == null) {
                selIndices = new TIntArrayList();
            }

            selIndices.clear();
            selIndices.add(sel);

            //System.out.println("selecting: " + selIndices);
            revalidateAndRepaint();
        }

        public void setSelectedIndices(int startIndex, int endIndex) {
            if (selIndices == null) {
                selIndices = new TIntArrayList();
            }


            selIndices.clear();
            for (int i = startIndex; i < endIndex + 1; i++) {
                selIndices.add(i);
            }

            //System.out.println("selecting: " + selIndices);
            revalidateAndRepaint();
        }

        void revalidateAndRepaint() {

            fTable.repaint();
            fTable.revalidate();
            fTable.getTableHeader().resizeAndRepaint();    // notice - else doesnt get done with table
        }


    } // End class SampleAndTemplateLabels


    public static class AtrTree implements Stackable.Vertical {

        private int fWidth = 10;

        private int fHeaderHeight = 150;

        private ArrayTreePanel arrayTree;

        /**
         * Class constructor
         *
         * @param atrReader
         * @param t_opt
         */
        public AtrTree(final AtrGtrReader atrReader) {
            this(new ArrayTreePanel(atrReader));
        }

        public AtrTree(final ArrayTreePanel atrTree) {
            this.arrayTree = atrTree;
            this.arrayTree.setBottomGutter(0);
            this.arrayTree.setTopGutter(0);

            arrayTree.setElementWidth(fWidth);

            System.out.println("############3 arrayTree: " + arrayTree);

            /*
            Dimension size = new Dimension(arrayTree.getPreferredSize().width,
                    fHeaderHeight);
            arrayTree.setPreferredSize(size);
            arrayTree.setSize(size);
            */
        }

        public BufferedImage createImage() {
            return arrayTree.snapshot();
        }

        public JComponent getComponent() {
            return arrayTree;
        }

        public void setCellWidth(final int cellWidth) {
            this.fWidth = cellWidth;
            arrayTree.setElementWidth(fWidth);
            Dimension size = new Dimension(arrayTree.getPreferredSize().width,
                    fHeaderHeight);
            arrayTree.setPreferredSize(size);
            arrayTree.setSize(size);
        }

    } // End class SampleAndTemplateLabels


    public static class Filler implements Stackable.Vertical {

        private JComponent fComp;

        /**
         * Class constructor
         *
         * @param ds
         * @param t_opt
         */
        public Filler(int width, int height) {
            this.fComp = GuiHelper.createNaPlaceholder();
            this.fComp.setPreferredSize(new Dimension(width, height));
        }

        public JComponent getComponent() {
            return fComp;
        }

        public void setCellWidth(int cellWidth) {

        }

        public BufferedImage createImage() {
            return ManagerHelper.createBufferedImage(getComponent());
        }

    } // End class SampleAndTemplateLabels

    public static class SampleAnnotationBlock implements Stackable.Vertical {

        private Vertical fReal;

        public SampleAnnotationBlock(final Dataset ds, final SampleAnnot sa) {

            this.fReal = new GenericSampleAnnotationBlock(ds, sa);

        }

        public void setCellWidth(int cellWidth) {
            fReal.setCellWidth(cellWidth);
        }

        public BufferedImage createImage() {
            return fReal.createImage();
        }

        public JComponent getComponent() {
            return fReal.getComponent();
        }

    } // End class SampleAnnotationBlock

    public static class SampleAnnotationBlockLegend implements Stackable.Horizontal {

        private Horizontal fReal;

        /**
         * Class constructor
         *
         * @param block
         */
        public SampleAnnotationBlockLegend(final SampleAnnot sa) {
            this.fReal = new GenericSampleAnnotationBlockLegend2(sa);
        }

        public JComponent getComponent() {
            return fReal.getComponent();
        }

        public void setCellHeight(int cellHeight) {
            fReal.setCellHeight(cellHeight);
        }

        public BufferedImage createImage() {
            return fReal.createImage();
        }
    }

    // @todo make this better
    public static class GenericSampleAnnotationBlock extends AbstractTableModel implements Stackable.Vertical {

        private SampleAnnot fSampleAnnot;

        private JTable sampleTable;

        private Dataset fDataset;

        /**
         * Class constructor
         * <p/>
         * ds needed because we want to restrict the ann displayed just to the ds columns (and order)
         *
         * @param sampleAnnot
         */
        public GenericSampleAnnotationBlock(final Dataset ds, final SampleAnnot sampleAnnot) {

            if (ds == null) {
                throw new IllegalArgumentException("Param ds cannot be null");
            }

            if (sampleAnnot == null) {
                throw new IllegalArgumentException("Param sampleAnnot cannot be null");
            }

            this.fDataset = ds;
            this.fSampleAnnot = sampleAnnot;

            sampleTable = new JTable();
            sampleTable.setModel(this);

            Helper.setTableAppearance(sampleTable);
            sampleTable.setGridColor(Color.LIGHT_GRAY); // make it more visible
            sampleTable.setDefaultRenderer(Object.class, createRenderer());

            sampleTable.setBorder(BorderFactory.createLineBorder(Color.BLACK, 1));

        }

        public void setCellWidth(int cellWidth) {
            sampleTable.setFont(new Font(sampleTable.getFont().getName(), sampleTable.getFont().getStyle(), cellWidth));
            Helper.setColWidth(sampleTable, cellWidth);
        }

        public BufferedImage createImage() {
            return ManagerHelper.createBufferedImage(getComponent());
        }

        public JComponent getComponent() {
            return sampleTable;
        }

        public int getColumnCount() {
            return fDataset.getNumCol();
        }

        public int getRowCount() {
            return 1;
        }

        public String getRowName(int r) {
            return "FOO";
        }

        public String getColumnName(int col) {
            return fDataset.getColumnName(col);
        }

        public Object getValueAt(int row, int col) {
            return getColumnName(col);
        }

        public Class getColumnClass(int col) {
            return String.class;
        }

        public boolean isEditable() {
            return false;
        }

        public boolean isCellEditable(int row, int col) {
            return false;
        }

        // Built in renderer because it is very closely tied to the model
        public DefaultTableCellRenderer createRenderer() {

            return new DefaultTableCellRenderer() {

                public Component getTableCellRendererComponent(JTable table,
                                                               Object value,
                                                               boolean isSelected,
                                                               boolean hasFocus,
                                                               int row,
                                                               int col) {

                    super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, col);

                    String txt = "";
                    String toolTipTxt = "";

                    if (value != null) {
                        toolTipTxt = value.toString();
                    }

                    Color color = Color.WHITE;

                    // @note the color map is a a VERTICAL map but we display it horiz and hence need to convert
                    // each row is a sample
                    // also not every value has a color
                    if (value != null && !isSelected) { // else cell selections cant be seen
                        String rowName = getRowName(row);
                        String colName = getColumnName(col);
                        ColorMap.Columns cm = fSampleAnnot.getColorMap();
                        if (cm != null) {
                            color = cm.getColor(rowName, colName);
                        }
                        txt = "";
                    }

                    if (isSelected) {
                        color = Color.YELLOW;
                    }

                    this.setBackground(color);
                    this.setText(txt);
                    this.setToolTipText(toolTipTxt);

                    return this;
                }
            };

        } // End method createRenderer


    } // End class GenericSampleAnnotationBlock

    public static class GenericSampleAnnotationBlockLegend2 implements Stackable.Horizontal {

        private JList legendList;

        /**
         * Class constructor
         *
         * @param block
         */
        public GenericSampleAnnotationBlockLegend2(final SampleAnnot annot) {
            legendList = new JList(new String[]{"FOO"});
            legendList.setBorder(BorderFactory.createLineBorder(Color.BLACK, 1));
        }

        public JComponent getComponent() {
            return legendList;
        }

        public void setCellHeight(int cellHeight) {
            legendList.setFont(new Font(legendList.getFont().getName(), legendList.getFont().getStyle(), cellHeight));
        }

        public BufferedImage createImage() {
            return ManagerHelper.createBufferedImage(getComponent());
        }
    }
}
