package xapps.api.vtools.viewers;

/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

import com.jidesoft.swing.JideBorderLayout;
import com.jidesoft.swing.JideBoxLayout;
import com.jidesoft.swing.JideScrollPane;
import com.jidesoft.swing.JideSplitPane;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.FeatureAnnot;
import edu.mit.broad.genome.objects.HClustDataset;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.swing.VerticalFlowLayout;
import edu.mit.broad.genome.viewers.AbstractViewer;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import org.genepattern.clustering.hierarchical.ArrayTreePanel;
import org.genepattern.clustering.hierarchical.GeneTreePanel;
import org.genepattern.clustering.hierarchical.Node;
import xapps.api.vtools.Hooks;
import xapps.api.vtools.stackables.HStackables;
import xapps.api.vtools.stackables.Stackable;
import xapps.api.vtools.stackables.StackablesManager;
import xapps.api.vtools.stackables.VStackables;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;

/**
 * HeatMapViewer.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public abstract class AbstractHeatMapViewer extends AbstractViewer {

    public static final String NAME = "HeatMapViewer";

    protected Dataset fDataset;

    protected Template fTemplate_opt;

    protected HClustDataset fHClust_optDataset;

    protected SampleAnnot fSampleAnnot;

    protected FeatureAnnot fFeatureAnnot;

    protected StackablesManager fManager;

    // Users must call initHMV if they use this constructor
    protected AbstractHeatMapViewer() {
        super();
    }

    /**
     * Class constructor
     *
     * @param ds
     * @param t_opt
     */
    public AbstractHeatMapViewer(final String title,
                                 final Icon icon,
                                 final Dataset ds,
                                 final Template t_opt,
                                 final HClustDataset hclust_opt) {
        initHMV(title, icon, ds, t_opt, hclust_opt);
    }

    protected void initHMV(final String title,
                           final Icon icon,
                           final Dataset ds,
                           final Template t_opt,
                           final HClustDataset hclust_opt) {

        if (ds == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }
        if (ds.getAnnot() == null) {
            throw new IllegalArgumentException("ds with annot required for this viewer: " + ds.getName());
        }

        this.fDataset = ds;
        this.fTemplate_opt = t_opt;
        this.fHClust_optDataset = hclust_opt;
        this.fFeatureAnnot = ds.getAnnot().getFeatureAnnot();

        this.fSampleAnnot = fDataset.getAnnot().getSampleAnnot_synched(fDataset.getColumnNamesArray());

        log.debug("VIZ annots feature: " + fFeatureAnnot + " sample: " + fSampleAnnot + " " + fDataset.getNumCol() + " " + fDataset.getNumRow());

        super.init(NAME, icon, title);

        // dont call this here -- > let the subclass call when its ready
        //jbInit();
    }

    // -------------------------------------------------------------------------------------------- //
    // @note implementing classes must override these methods
    protected abstract Hooks.Matrix getMyHook(JTable rowTable, JTable colTable, VStackables.SampleAndTemplateLabels stl);

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    private HStackables.GtrTree _makeGtr() {

        if (fHClust_optDataset == null || fHClust_optDataset.hasGtr() == false) {
            log.debug(">> hclust _makeGtr was null: " + fHClust_optDataset);
            return null;
        }

        HStackables.GtrTree gtr = new HStackables.GtrTree(fHClust_optDataset.getGeneTreePanel());
        final GeneTreePanel tree = (GeneTreePanel) gtr.getComponent();
        tree.setSelectedNodeColor(Color.BLUE);

        tree.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                final Node node = tree.getSelectedNode();
                if (node != null) {
                    System.out.println(">>>> " + node.getMinIndex() + " " + node.getMaxIndex());
                    fManager.selectRows(node.getMinIndex(), node.getMaxIndex());
                }
            }
        });

        log.debug("Making _makeGtr hclust: " + fHClust_optDataset);

        return gtr;
    }

    private VStackables.AtrTree _makeAtr() {

        if (fHClust_optDataset == null || fHClust_optDataset.hasAtr() == false) {
            log.debug(">> hclust _makeAtr was null: " + fHClust_optDataset);
            return null;
        }

        VStackables.AtrTree atr = new VStackables.AtrTree(fHClust_optDataset.getArrayTreePanel());

        final ArrayTreePanel tree = (ArrayTreePanel) atr.getComponent();

        tree.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                final Node node = tree.getSelectedNode();
                if (node != null) {
                    //System.out.println(">>>> " + node.getMinIndex() + " " + node.getMaxIndex());
                    // @note hack to get the atr off by 1 thing
                    fManager.selectColumns(node.getMinIndex() - 1, node.getMaxIndex() - 1);
                }
            }
        });

        log.debug("Making _makeAtr hclust: " + fHClust_optDataset);
        return atr;
    }


    protected HStackables.FeatureAnnotComponentSimple createFeatureAnnotComponent(Dataset ds) {
        return new HStackables.FeatureAnnotComponentWithMetrics(ds);
    }

    protected void jbInit() {

        // Create those stackables that we want
        final HStackables.HeatMapComponent hmc = new HStackables.HeatMapComponent(fDataset);
        final HStackables.FeatureAnnotComponentSimple fmc = createFeatureAnnotComponent(fDataset);

        final VStackables.SampleAnnotationBlock dmc = new VStackables.SampleAnnotationBlock(fDataset, fSampleAnnot);
        final VStackables.SampleAnnotationBlockLegend dmc_legend = new VStackables.SampleAnnotationBlockLegend(fSampleAnnot);

        final VStackables.AtrTree atr = _makeAtr();
        final HStackables.GtrTree gtr = _makeGtr();

        final VStackables.SampleAndTemplateLabels stl = new VStackables.SampleAndTemplateLabels(fDataset, fTemplate_opt);
        final Stackable.Horizontal fmc_header = fmc.getTableHeaderAsStackable();

        int numRows = 3;
        int numCols = 2;

        if (fHClust_optDataset != null) {
            numRows++;
            numCols++;
        }

        // Register the stackables with the manager
        this.fManager = new StackablesManager(getMyHook((JTable) fmc.getComponent(), (JTable) dmc.getComponent(), stl),
                numRows, numCols);

        if (fHClust_optDataset != null) {

            if (atr != null) {
                fManager.add(0, 1, atr);
            }

            if (gtr != null) {
                fManager.add(3, 0, gtr);
            }

            fManager.add(1, 1, dmc);
            fManager.add(1, 2, dmc_legend);

            fManager.add(2, 1, stl);
            fManager.add(3, 1, hmc);

            fManager.add(2, 2, fmc_header);
            fManager.add(3, 2, fmc);
        } else {

            fManager.add(0, 0, dmc);
            fManager.add(0, 1, dmc_legend);

            fManager.add(1, 0, stl);
            fManager.add(2, 0, hmc);

            fManager.add(1, 1, fmc_header);
            fManager.add(2, 1, fmc);

        }

        // -------------------------------------------------------------------------------------------- //

        // Then lay them out in some custom way
        // Rather than place everything in a jscrollpane, place only those that you think might
        JideSplitPane split_hmc_and_fmc = new JideSplitPane(JideSplitPane.HORIZONTAL_SPLIT);
        split_hmc_and_fmc.add(hmc.getComponent(), JideBoxLayout.FLEXIBLE);
        split_hmc_and_fmc.add(fmc.getComponent(), JideBoxLayout.FLEXIBLE);

        JideScrollPane center_scroll = new JideScrollPane(split_hmc_and_fmc);

        // -------------------------------------------------------------------------------------------- //
        // Next one

        final JideScrollPane top_scroll = _createTopScroll(dmc, dmc_legend, stl, fmc_header, atr);

        // because we synch this, the fa table will get sized to the left pane of the top split
        Stackable.Helper.synchHorizScrolling(new JScrollPane[]{center_scroll, top_scroll});

        final JideSplitPane center_split = new JideSplitPane(JideSplitPane.VERTICAL_SPLIT);
        center_split.setProportionalLayout(true);
        center_split.setShowGripper(true);
        center_split.add(top_scroll, JideBoxLayout.FLEXIBLE);
        center_split.add(center_scroll, JideBoxLayout.FLEXIBLE);

        this.setLayout(new BorderLayout());

        // Finally make the gene cluster pane if available
        if (gtr != null) {
            JPanel mainPanel = new JPanel(new JideBorderLayout());
            mainPanel.add(center_split, JideBorderLayout.CENTER);
            mainPanel.add(createLhsSplit(gtr, center_split, center_scroll), JideBorderLayout.WEST);
            this.add(mainPanel, BorderLayout.CENTER);
        } else {
            this.add(center_split, BorderLayout.CENTER);
        }

        this.add(fManager.createToolBar(), BorderLayout.WEST);
        //this.add(fManager.getStatusBar(), BorderLayout.SOUTH); // akes up too much space

        // starting size
        fManager.setGridSize(10);

        this.revalidate();
    }

    private JideSplitPane createLhsSplit(final HStackables.GtrTree gtr,
                                         final JideSplitPane centerSplit,
                                         final JScrollPane scroll_center) {

        final JideSplitPane lhsSplit = new JideSplitPane(JideSplitPane.VERTICAL_SPLIT);

        final Component lhs_filler = createEmptyFiller();
        final JideScrollPane gtrScroll = new JideScrollPane(wrap(gtr.getComponent())); // needs to be wrapped other wise scorlling doesnt work well
        Stackable.Helper.synchVerticalScrolling(new JScrollPane[]{gtrScroll, scroll_center});

        lhsSplit.add(lhs_filler, JideBoxLayout.FLEXIBLE);
        lhsSplit.add(gtrScroll, JideBoxLayout.FLEXIBLE);

        lhsSplit.setDividerLocation(0, centerSplit.getDividerLocation(0));
        Stackable.Helper.synchSplitPanes(centerSplit, lhsSplit);

        return lhsSplit;
    }

    private JideScrollPane _createTopScroll(
            final VStackables.SampleAnnotationBlock dmc,
            final VStackables.SampleAnnotationBlockLegend dmc_legend,
            final VStackables.SampleAndTemplateLabels stl,
            final Stackable.Horizontal fmc_header,
            final VStackables.AtrTree atr) {

        // @note IMP Filling vertical (true below) is imp for when there is no atr
        // otherwise there is emoty space in the header that doesnt look as good
        JPanel pan_on_top_of_hmc = new JPanel(new BorderLayout());
        JPanel pan_on_top_of_fa;

        if (atr != null) {
            pan_on_top_of_hmc.add(atr.getComponent(), BorderLayout.NORTH);
            pan_on_top_of_hmc.add(dmc.getComponent(), BorderLayout.CENTER);
            pan_on_top_of_hmc.add(stl.getComponent(), BorderLayout.SOUTH);

            pan_on_top_of_fa = new JPanel(new VerticalFlowLayout(VerticalFlowLayout.TOP, 0, 0, true, false));
            pan_on_top_of_fa.add(createEmptyFiller(atr.getComponent().getPreferredSize().height));
            pan_on_top_of_fa.add(dmc_legend.getComponent());
            pan_on_top_of_fa.add(fmc_header.getComponent());

        } else {
            dmc.getComponent().setBorder(BorderFactory.createLineBorder(Color.BLACK, 1));
            pan_on_top_of_hmc = new JPanel(new VerticalFlowLayout(VerticalFlowLayout.TOP, 0, 0, true, true));
            pan_on_top_of_hmc.add(dmc.getComponent());
            pan_on_top_of_hmc.add(stl.getComponent());

            // using border layout cause the alignment b/w the 2 panels to be lost
            //pan_on_top_of_fa = new JPanel(new VerticalFlowLayout(VerticalFlowLayout.TOP, 0, 0, true, true));
            //pan_on_top_of_fa = new JPanel(new BorderLayout());
            //pan_on_top_of_fa.add(dmc_legend.getComponent(), BorderLayout.NORTH);
            //pan_on_top_of_fa.add(fmc_header.getComponent(), BorderLayout.CENTER);
            //pan_on_top_of_fa.add(createEmptyFiller(stl.getComponent().getPreferredSize().height), BorderLayout.SOUTH);

            pan_on_top_of_fa = new JPanel(new VerticalFlowLayout(VerticalFlowLayout.TOP, 0, 0, true, false));
            pan_on_top_of_fa.add(dmc_legend.getComponent());
            pan_on_top_of_fa.add(fmc_header.getComponent());
            pan_on_top_of_fa.add(createEmptyFiller(stl.getComponent().getPreferredSize().height), BorderLayout.SOUTH);
        }

        JideSplitPane split_top = new JideSplitPane(JideSplitPane.HORIZONTAL_SPLIT);
        split_top.add(pan_on_top_of_hmc, JideBoxLayout.FLEXIBLE);
        split_top.add(pan_on_top_of_fa, JideBoxLayout.FLEXIBLE);

        return new JideScrollPane(split_top);
    }

    public JMenuBar getJMenuBar() {
        return EMPTY_MENU_BAR;
    }

}        // End AbstractHeatMapViewer

/*--- Formatted in Sun Java Convention Style on Thu, Oct 17, '02 ---*/
