package xapps.api.vtools.viewers;

/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.HClustDataset;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.xbench.actions.ObjectAction;
import edu.mit.broad.xbench.core.Widget;
import xapps.api.vtools.Hooks;
import xapps.api.vtools.stackables.VStackables;

import javax.swing.*;

/**
 * HeatMapViewer.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class HeatMapViewer extends AbstractHeatMapViewer {

    public static final Icon ICON = JarResources.getIcon("HeatMap.gif");

    public static final Icon ICON_HC = JarResources.getIcon("Cdt.png");

    /**
     * Builds a Viewer on specified dataset object
     *
     * @param gset
     */
    public HeatMapViewer(final Dataset ds) {
        this(ds, null, null);
    }

    /**
     * Class constructor
     *
     * @param ds
     * @param t_opt
     */
    public HeatMapViewer(final Dataset ds,
                         final Template t_opt,
                         final HClustDataset hclust_opt) {
        super.initHMV(formatTitle(ds), getMyIcon(hclust_opt), ds, t_opt, hclust_opt);
        jbInit();
    }

    protected Hooks.Matrix getMyHook(final JTable rowTable, final JTable colTable, final VStackables.SampleAndTemplateLabels stl) {
        return new Hooks.Gex(fDataset, fSampleAnnot, fTemplate_opt, rowTable, colTable, stl);
    }

    static Icon getMyIcon(final HClustDataset hclust_opt) {
        if (hclust_opt != null) {
            return ICON_HC;
        } else {
            return ICON;
        }
    }

    /**
     * Inner class representing an action for the viewer
     */
    public static class Action extends ObjectAction {

        private Dataset fDataset;
        private Template fTemplate_opt;
        private HClustDataset fHClust_optDataset;

        /**
         * Class constructor
         */
        public Action() {

            this.putValue(Action.NAME, getName());
            this.putValue(Action.SMALL_ICON, getIcon());
            this.putValue(Action.SHORT_DESCRIPTION, getDescription());
        }

        /**
         * Class constructor
         *
         * @param dsa
         */
        public Action(final Dataset ds, final Template t_opt, final HClustDataset hclust_opt) {
            this();
            this.fDataset = ds;
            this.fTemplate_opt = t_opt;
            this.fHClust_optDataset = hclust_opt;
        }

        public void setObject(Object obj) {
            this.fDataset = (Dataset) obj;
        }

        public Widget getWidget() {
            return new HeatMapViewer(fDataset, fTemplate_opt, fHClust_optDataset);
        }

        public String getId() {
            return "HeatMapViewer";
        }

        public String getName() {
            return getId();
        }

        public Icon getIcon() {
            return ICON;
        }

        public String getDescription() {
            return "View the heat map";
        }

    } // End Inner class Action

} // End HeatMapViewer
