package xapps.browser;

import com.jidesoft.swing.JideTabbedPane;
import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.swing.GuiHelper;
import edu.mit.broad.genome.viewers.AbstractViewer;
import edu.mit.broad.vdb.msigdb.MSigDB;
import edu.mit.broad.xbench.core.api.Application;
import foxtrot.Task;
import foxtrot.Worker;
import org.apache.log4j.Logger;
import xapps.gsea.GseaWebResources;
import xtools.api.param.FileParam;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.util.StringTokenizer;

/**
 * @author Aravind Subramanian
 *         Date: Feb 26, 2006
 *         Time: 2:21:27 PM
 */
public class MSigDBViewerContainer extends AbstractViewer {

    private Logger log = XLogger.getLogger(MSigDBViewerContainer.class);

    public static final String NAME = "MSigDBBrowser";

    public static final Icon ICON = JarResources.getIcon("msigdb16.png");

    private MSigDBViewerContainer fInstance = this;

    private JideTabbedPane fTabbedPane;

    private FileParam fFileOrURLParam;

    private MyProgressMonitorHook fHook;

    private MSigDB curr_msigdb;

    private JComponent fFiller;

    /**
     * Class constructor
     */
    public MSigDBViewerContainer() {
        super(NAME, ICON, "MSigDB gene sets browser");

        jbInit();
    }

    /**
     * Class constructor
     *
     * @return
     */
    public MSigDB getMSigDB() {
        return curr_msigdb;
    }

    private void jbInit() {

        this.setLayout(new BorderLayout(15, 15));


        this.fHook = new MyProgressMonitorHook();

        this.fFileOrURLParam = new FileParam("path", "File path or URL to the MSigDB database", true);

        this.fFileOrURLParam.setValue_url(GseaWebResources.getMSigDB_current_xml_fileName());


        JButton bLoad = new JButton("Load database", JarResources.getIcon("Run16.png"));
        bLoad.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {

                if (!fFileOrURLParam.isSpecified()) {
                    Application.getWindowManager().showMessage("No file path or URL specified. First enter one and then try loading");
                } else {

                    try {
                        Worker.post(new Task() {
                            public Object run() throws Exception {
                                boolean firstTime = (curr_msigdb == null);
                                String fileName = fFileOrURLParam.getValue().toString();
                                if (fileName.startsWith("msigdb_v")) {
                                    fileName = GseaWebResources.getFTPBase() + "/" + fileName;
                                }

                                curr_msigdb = ParserFactory.readMSigDB(fileName, true, true);

                                MSigDBViewer widget = new MSigDBViewer(curr_msigdb, true, fTabbedPane);
                                try {
                                    if (firstTime) {
                                        fTabbedPane.remove(fFiller);
                                        //fTabbedPane.removeTabAt(0); // get rid if the filler
                                    }
                                } catch (Throwable t) {
                                }

                                fTabbedPane.addTab(pathorurl2name(fileName) + " [version=" + curr_msigdb.getVersion() + " build=" + curr_msigdb.getBuildDate() + "]", widget);
                                fTabbedPane.setSelectedComponent(widget);
                                return null;
                            }
                        });
                    } catch (Throwable t) {
                        Application.getWindowManager().showError("Trouble launching make reports", t);
                    } finally {
                        ParserFactory.uninstallProgressMonitorHook(); // @note
                    }
                }
            }
        });

        //JPanel topPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 15, 5));
        JPanel topPanel = new JPanel(new BorderLayout(10, 10));
        topPanel.setBackground(Color.DARK_GRAY);
        topPanel.setBorder(BorderFactory.createLineBorder(Color.DARK_GRAY));

        JLabel label = new JLabel(fFileOrURLParam.getHtmlLabel_v2());
        label.setForeground(Color.WHITE);
        topPanel.add(label, BorderLayout.WEST);

        //GFileFieldPlusChooser gfc = (GFileFieldPlusChooser) fFileOrURLParam.getSelectionComponent();
        //gfc.getTextField().setPreferredSize(new Dimension(gfc.getTextField().getPreferredSize().width + 100, gfc.getTextField().getPreferredSize().height));

        topPanel.add(fFileOrURLParam.getSelectionComponent().getComponent(), BorderLayout.CENTER);
        topPanel.add(bLoad, BorderLayout.EAST);

        // -------------------------------------------------------------------------------------------- //

        this.setLayout(new BorderLayout(10, 10));
        this.add(topPanel, BorderLayout.NORTH);

        this.fTabbedPane = new JideTabbedPane(JideTabbedPane.TOP);
        this.fTabbedPane.setTabEditingAllowed(true);
        //this.fTabbedPane.setBoxStyleTab(true);
        this.fTabbedPane.setShowCloseButton(true);
        this.fTabbedPane.setShowCloseButtonOnTab(true);
        this.fTabbedPane.setShowGripper(true); // just a decorator no action
        this.fTabbedPane.setShowIconsOnTab(true);
        this.fTabbedPane.setShowTabButtons(true);

        this.fFiller = GuiHelper.createWaitingPlaceholder();
        fTabbedPane.addTab("", fFiller);

        this.add(fTabbedPane, BorderLayout.CENTER);

        this.revalidate();

    }


    private static String pathorurl2name(final String path) {
        String name;
        if (NamingConventions.isURL(path)) {
            name = path;
            StringTokenizer tok = new StringTokenizer(name, "/");
            while (tok.hasMoreTokens()) {
                name = tok.nextToken();
            }
        } else {
            File file = new File(path);
            name = file.getName();
        }

        return NamingConventions.removeExtension(name);
    }

    /**
     * For the ParserFactory
     */
    class MyProgressMonitorHook implements ParserFactory.ProgressMonitorHook {

        public ProgressMonitorInputStream getProgressMonitorInputStream(File inputFile, String msg) throws IOException {
            ProgressMonitorInputStream pis = new ProgressMonitorInputStream(fInstance, msg, new FileInputStream(inputFile));
            pis.getProgressMonitor().setNote("Loading gene sets database & annotation");
            pis.getProgressMonitor().setMillisToDecideToPopup(0);
            return pis;
        }

        public ProgressMonitorInputStream getProgressMonitorInputStream(URL url, String msg) throws IOException {
            URLConnection uc = url.openConnection();
            InputStream in = uc.getInputStream();

            //ProgressMonitorInputStream pis = new ProgressMonitorInputStream(fInstance, msg, url.openStream());
            ProgressMonitorInputStream pis = new ProgressMonitorInputStream(fInstance, msg, in);
            pis.getProgressMonitor().setMaximum(uc.getContentLength());
            pis.getProgressMonitor().setNote("Loading gene sets database & annotation");
            pis.getProgressMonitor().setMillisToDecideToPopup(0);
            return pis;
        }

        public OutputStream getProgressMonitorOutputStream(File file, String msg) throws IOException {
            return new FileOutputStream(file);
        }

        // @todo
        public OutputStream getProgressMonitorOutputStream(URL url, String msg) throws IOException {
            throw new NotImplementedException();
        }

    } // End class AppDataImportProgressMonitor

}