package xapps.browser;

import com.jidesoft.swing.JideTabbedPane;
import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.alg.cluster.ClusteringAlgs;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.viewers.Viewer;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.vdb.map.Chip2ChipMapper;
import edu.mit.broad.vdb.map.MGeneSetMatrix;
import edu.mit.broad.vdb.msigdb.DeconAccessions;
import edu.mit.broad.vdb.msigdb.GeneSetAnnotation;
import edu.mit.broad.vdb.msigdb.MSigDB;
import edu.mit.broad.vdb.msigdb.MSigDBImpl;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import edu.mit.broad.xbench.core.api.Application;
import org.genepattern.gsea.GeneSetOverlapWidget;
import xapps.api.vtools.AbstractVToolWithParams;
import xapps.api.vtools.Hooks;
import xapps.api.vtools.viewers.HeatMapViewerSimple;
import xapps.gsea.GeneSetAnnotationWithOverlaps;
import xtools.api.param.*;

import javax.swing.*;
import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Feb 26, 2006
 *         Time: 3:26:11 PM
 */
public class VToolsForMSigDBBrowser {

    public static class ExportGeneSets extends AbstractVToolWithParams {

        private GeneSetMatrixFormatParam fGmFormatParam;

        private ChipChooserMultiParam fChipsTargetParam;

        private ViewerHook hook;

        // private BooleanParam fOnlySelectedFieldsParam;
        private SelectedOrAllParam fOnlySelectedFieldsParam;


        /**
         * Class constructor
         *
         * @param msigdb
         * @param selectedGeneSets
         */
        public ExportGeneSets(final ViewerHook hook) {
            super("Export gene sets", new MyBaseHook("export_", hook));
            this.hook = hook;
        }

        public Viewer execute() throws Exception {

            GeneSetMatrix gm;

            /*
            if (hook.isProcessOnlySelected()) {
                final String[] selNames = hook.getSelectedGeneSetNames();
                if (selNames == null || selNames.length == 0) {
                    Application.getWindowManager().showMessage("Nothing selected. Select one or more gene sets and try again");
                    return null;
                }

                gm = hook.getMSigDB().createGeneSetMatrix(selNames, false); // always get as symbols
            } else {
                gm = hook.getMSigDB().createGeneSetMatrix(false); // always get as symbols
            }

            */

            final String[] selNames = hook.getSelectedGeneSetNames();
            if (fOnlySelectedFieldsParam.isOnlySelected() && selNames != null && selNames.length > 0) {
                gm = hook.getMSigDB().createGeneSetMatrix(selNames, false); // always get as symbols
            } else {
                gm = hook.getMSigDB().createGeneSetMatrix(hook.getAllDisplayedGeneSetNames(), false); // always get as symbols
            }

            final Chip2ChipMapper mapper = VdbRuntimeResources.getGeneSymbolMapper(fChipsTargetParam.getChipCombo(), false);

            final MGeneSetMatrix mgm = mapper.map(gm, false);

            String newName = getSaveName() + "." + mapper.getTargetChip().getName();
            final GeneSetMatrix mapped_gm = mgm.getMappedGeneSetMatrix("mapped_").cloneShallow(newName);

            final File gmFile = kReport.savePageInvisibly2Cache(mapped_gm, fGmFormatParam);

            // @note also register as gene sets
            //ParserFactory.registerWithCache(gmFile, mapped_gm.getGeneSets());
            ParserFactory.read(gmFile, false);

            Application.getWindowManager().showMessage("Saved GeneSetMatrix at: " + gmFile.getPath());

            return null;
        }

        public void declareParams() {
            // i dont think we want verbose options here
            // lets just go form gene symbol to whatever (via gene symbols)

            //ChipChooserMultiParam fChipsSourceParam = ParamFactory.createChipsSourceParam(true);
            //ChipComboModeReqdParam fCombineChipsModeReqd = new ChipComboModeReqdParam();
            //this.fChip2ChipMapperParam = new Chip2ChipMapperOptParam(true, true, fChipsSourceParam, fChipsTargetParam);

            this.fGmFormatParam = new GeneSetMatrixFormatParam(false);
            this.fChipsTargetParam = ParamFactory.createChipsTargetParam(true);

            //fParamSet.addParam(fChipsSourceParam);
            //fParamSet.addParam(fChip2ChipMapperParam);
            //fParamSet.addParam(fCombineChipsModeReqd);

            fParamSet.addParam(fChipsTargetParam);
            fParamSet.addParam(fGmFormatParam);

            //fOnlySelectedFieldsParam = Hooks.createOnlySelectedFieldsParam();
            fOnlySelectedFieldsParam = new SelectedOrAllParam();
            fParamSet.addParam(fOnlySelectedFieldsParam);
        }

        public String getTitle() {
            return "Export selected gene sets";
        }

        public String getHelpURL() {
            //return JarResources.getHelpURL(this.getClass().getName());
            return JarResources.getHelpURL("xapps_mbrowser_ExportGeneSets"); // override as default too long for heidi
        }

    } // End class ExportGeneSets


    public static class DisplayOverlaps extends AbstractVToolWithParams {

        private ViewerHook hook;

        private ModeReqdParam fOrientationParam;


        public DisplayOverlaps(final ViewerHook hook) {
            super("Display overlaps", new MyBaseHook("overlaps_", hook));
            this.hook = hook;
        }

        public Viewer execute() throws Exception {

            final String[] selNames = hook.getSelectedGeneSetNames();
            if (selNames == null || selNames.length == 0) {
                Application.getWindowManager().showMessage("Nothing selected. Select one or more gene sets and try again");
                return null;
            }

            final GeneSetMatrix gm = hook.getMSigDB().createGeneSetMatrix(selNames, false);
            Dataset ds_clust = ClusteringAlgs.cluster(gm).getDatasetOrdered();

            final String[] all_gene_names = gm.getAllMemberNamesOnlyOnce();
            SampleAnnot sann;
            FeatureAnnot fann;


            if (fOrientationParam.getStringIndexChoosen() == 0) { // cols are gene sets
                sann = hook.getMSigDB().getAsSampleAnnot();
                ds_clust = new DatasetGenerators().transpose(ds_clust);
                fann = new FeatureAnnotImpl(ds_clust, null);
            } else { // rows are gene sets
                sann = new SampleAnnotImpl("GENES", all_gene_names, new ColumnsImpl(all_gene_names));
                fann = hook.getMSigDB().getAsFeatureAnnot();
                // ds already in the right way
            }

            // set the annotation
            ds_clust = ((DefaultDataset) ds_clust).cloneShallow(new AnnotImpl(fann, sann));

            //BufferedImage bi = VizFactory.createGramImager().createBpogImage(ds_clust);
            // _openIt(new ImageViewer(ds_clust.getName(), bi);
            HeatMapViewerSimple hmv = new HeatMapViewerSimple(ds_clust);
            return _openIt("# of sets: " + selNames.length, "Overlaps: " + selNames.length,
                    hmv, hook.getViewerWindow_opt());
        }

        public void declareParams() {
            fOrientationParam = new ModeReqdParam("rows", "Orientation of overlap matrix", "Orientation of overlap matrix", new String[]{"Gene_sets_as_columns", "Gene_sets_as_rows"});
            fParamSet.addParam(fOrientationParam);
        }

        public String getTitle() {
            return "Display overlaps between selected gene sets";
        }

    } // End class DisplayOverlaps


    public static class DisplayOverlapsJosh extends AbstractVToolWithParams {

        private ViewerHook hook;

        private SelectedOrAllParam fOnlySelectedFieldsParam;

        public DisplayOverlapsJosh(final ViewerHook hook) {
            super("Display overlaps", new MyBaseHook("overlaps_", hook));
            this.hook = hook;
        }

        public Viewer execute() throws Exception {

            GeneSetMatrix gm;

            final String[] selNames = hook.getSelectedGeneSetNames();
            if (fOnlySelectedFieldsParam.isOnlySelected() && selNames != null && selNames.length > 0) {
                gm = hook.getMSigDB().createGeneSetMatrix(selNames, false); // always get as symbols
            } else {
                gm = hook.getMSigDB().createGeneSetMatrix(false); // always get as symbols
            }

            GeneSetOverlapWidget gso = new GeneSetOverlapWidget(hook.getViewerWindow_opt());
            gso.setData(gm);

            return null;
        }

        public void declareParams() {

            fOnlySelectedFieldsParam = new SelectedOrAllParam();
            fParamSet.addParam(fOnlySelectedFieldsParam);
        }

        public String getTitle() {
            return "Display overlaps between selected gene sets";
        }

    } // End class ExportGeneSets

    /**
     * @author Aravind Subramanian
     *         Date: Jan 25, 2006
     *         Time: 3:10:32 PM
     */
    public static class SearchByGene extends AbstractVToolWithParams {

        private JTabbedPane viewer_window_opt;

        private MSigDB msigdb;

        private StringInputParam fGeneNameParam;

        public SearchByGene(final MSigDB msigdb, final JTabbedPane viewer_window_opt) {
            super("Search by gene", new MyBaseHook("gene_", null));
            this.viewer_window_opt = viewer_window_opt;
            this.msigdb = msigdb;
        }

        public Viewer execute() throws Exception {

            String geneName = fGeneNameParam.getString().toUpperCase(); // @note

            if (!VdbRuntimeResources.getChip_Seq_Accession().isProbe(geneName)) {
                Application.getWindowManager().showMessage("This is not a valid gene symbol (or known alias): " + geneName);
                return null;
            } else {
                DeconAccessions.Lookup lp = DeconAccessions.creatLookup(geneName);
                geneName = lp.getSymbol();
            }

            List hits = new ArrayList();
            for (int i = 0; i < msigdb.getNumGeneSets(); i++) {
                GeneSetAnnotation ga = msigdb.getGeneSetAnnotation(i);
                if (ga.getGeneSet(false).isMember(geneName)) { // @todo make this case IN sensitive
                    hits.add(ga);
                }
            }

            klog.debug("# of hits: " + hits.size());

            final MSigDB hits_msigdb = new MSigDBImpl(geneName, msigdb.getVersion(), msigdb.getBuildDate(),
                    (GeneSetAnnotation[]) hits.toArray(new GeneSetAnnotation[hits.size()]));

            final MSigDBViewer widget = new MSigDBViewer(hits_msigdb, false);

            return _openIt("# of hits: " + hits.size(), "Gene search: " + geneName, widget, viewer_window_opt);
        }

        public void declareParams() {
            this.fGeneNameParam = new StringInputParam("gene", "Gene (only symbols, case INsensitive)", "Gene name", null, true);
            fParamSet.addParam(fGeneNameParam);
        }

        public String getTitle() {
            return "Search for gene sets that contain the specified gene";
        }

        public String getHelpURL() {
            //return JarResources.getHelpURL(this.getClass().getName());
            return JarResources.getHelpURL("xapps_mbrowser_SearchByGene"); // override as default too long for heidi
        }

    } // End class SearchByGene


    /**
     * @author Aravind Subramanian
     *         Date: Jan 25, 2006
     *         Time: 3:10:32 PM
     */
    public static class SearchByGeneSet extends AbstractVToolWithParams {

        private JTabbedPane viewer_window_opt;

        private MSigDB msigdb;

        private GeneSetMultiChooserParam fGeneSetParam;


        public SearchByGeneSet(final MSigDB msigdb,
                               final JTabbedPane viewer_window_opt) {
            super("Search by gene set", new MyBaseHook("geneset_", null));
            this.viewer_window_opt = viewer_window_opt;
            this.msigdb = msigdb;

        }

        public int findInt(Collection set, GeneSet gset) {
            Iterator e = set.iterator();
            int cnt = 0;
            while (e.hasNext()) {
                if (gset.isMember(e.next().toString())) {
                    cnt++;
                }
            }
            return cnt;
        }

        public Viewer execute() throws Exception {

            final GeneSet gset = fGeneSetParam.getGeneSetCombo();
            final List hits = new ArrayList();

            for (int i = 0; i < msigdb.getNumGeneSets(); i++) {
                final GeneSetAnnotation gsa = msigdb.getGeneSetAnnotation(i);
                final FSet query = (FSet) gsa.getGeneSet(false);
                int inter = findInt(query.getMembers_quick(), gset);
                if (inter > 0) {
                    hits.add(new GeneSetAnnotationWithOverlaps.One(gsa, inter, query.getNumMembers()));
                }

                if (i % 250 == 0) {
                    System.out.println("Done searching " + (i + 1) + " / " + msigdb.getNumGeneSets());
                }
            }

            klog.debug("# of hits: " + hits.size());

            GeneSetAnnotationWithOverlaps.One[] inters = (GeneSetAnnotationWithOverlaps.One[]) hits.toArray(new GeneSetAnnotationWithOverlaps.One[hits.size()]);

            GeneSetAnnotationWithOverlaps gso = new GeneSetAnnotationWithOverlaps(gset, inters);

            /*
            GeneSetAnnotation[] anns = new GeneSetAnnotation[inters.length];
            for (int i = 0; i < inters.length; i++) {
                anns[i] = inters[i].gsa;
            }

            final MSigDB hits_msigdb = new MSigDBImpl(gset.getName(true), msigdb.getVersion(), msigdb.getBuildDate(), anns);

            final MSigDBBrowserWidget widget = new MSigDBBrowserWidget(hits_msigdb, false);

            return _openIt("# of hits: " + hits.size(), "Gene set search: " + gset.getName(true), widget, viewer_window_opt);
            */

            return _openIt("# of hits: " + hits.size(),
                    "Gene set search: " + NamingConventions.removeExtension(gset.getName(true)) + "( " + gset.getNumMembers() + ")",
                    gso.createViewer(msigdb), viewer_window_opt);
        }

        public void declareParams() {
            this.fGeneSetParam = new GeneSetMultiChooserParam("gset", "Gene set (ids must be symbols)", "Gene set");
            fParamSet.addParam(fGeneSetParam);
        }

        public String getTitle() {
            return "Search for overlapping gene sets";
        }

        public String getHelpURL() {
            //return JarResources.getHelpURL(this.getClass().getName());
            return JarResources.getHelpURL("xapps_mbrowser_SearchByGeneSet"); // override as default too long for heidi
        }
    } // End class SearchByGeneSet

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public static interface ViewerHook {

        public String[] getSelectedGeneSetNames();

        public String[] getAllDisplayedGeneSetNames();

        public int getNumSelectedRows();

        public MSigDB getMSigDB();

        public boolean isProcessOnlySelected();

        public JideTabbedPane getViewerWindow_opt();

    } // End inner class ViewerHook


    static class MyBaseHook implements Hooks.BaseHook {

        private JTextField tfName;

        private String fSaveNameHint;

        private ViewerHook hook;

        MyBaseHook(final String save_name_hint_prefix, ViewerHook hook) {
            this.fSaveNameHint = save_name_hint_prefix;
            this.hook = hook;
        }

        // @note need to change this to save name
        public String getSaveNameHint() {
            return fSaveNameHint;
        }

        public String getComponentMessage() {
            if (hook != null) {
                return "# of gene sets selected: " + hook.getNumSelectedRows();
            } else {
                return null;
            }
        }

        public JTextField getTextField() {

            if (tfName == null) {
                tfName = new JTextField(60);

                tfName.setBorder(BorderFactory.createTitledBorder("Enter a short name for the result"));
            }

            String sn = getSaveNameHint();
            if (sn != null && sn.length() > 0) {
                tfName.setText(sn);
            }

            return tfName;
        }

        public String getSaveName() { // dont make safe - give back whatever user asks for
            if (tfName != null) {
                return tfName.getText();
            } else {
                return "na";
            }
        }

    } // End inner class MyBaseHook

    private static Viewer _openIt(final String msg,
                                  final String tabName,
                                  final Viewer v,
                                  final JTabbedPane viewer_window_opt) {

        Application.getWindowManager().showMessage(msg + ". Results are displayed in a new tab");

        if (viewer_window_opt != null) {
            viewer_window_opt.addTab(tabName, (JComponent) v);
            viewer_window_opt.setSelectedComponent((JComponent) v);
            return null;
        } else {
            return v;
        }

    }

} // End class VToolsForMSigDBBrowser
