/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xapps.gsea;

import xtools.api.Tool;
import xtools.chip2chip.Chip2Chip_orig_restore_later;
import xtools.genesettools.FindManyOverlaps;
import xtools.genesettools.FindOverlaps;
import xtools.genesettools.GeneSetHeatMap;
import xtools.genesettools.GeneSetStats;
import xtools.munge.CollapseDataset;
import xtools.munge.XtractDataset;

/**
 * Configuration and utils common to all xapplications.
 */
public class GseaAppConf {

    // @maint known extensions
    private static String[] KNOWN_FILE_EXTS = new String[]{"res", "gct", "pcl", "txt", "grp", "gmx", "gmt", "cls", "rnk", "chip"}; // @maint

    public static void setGseaApp() {
        System.setProperty("GSEA", Boolean.TRUE.toString());
    }

    public static Tool[] createUtilityTools() {

        return new Tool[]{
                new FindOverlaps(),
                new FindManyOverlaps(),
                new GeneSetStats(),
                new GeneSetHeatMap(),
                new XtractDataset(),
                new CollapseDataset(),
                new Chip2Chip_orig_restore_later()
        };

    }

    public static javax.swing.filechooser.FileFilter[] createAllFileFilters() {

        StringBuffer buf = new StringBuffer("GSEA supported file types [");
        for (int i = 0; i < KNOWN_FILE_EXTS.length; i++) {
            buf.append(KNOWN_FILE_EXTS[i]);
            if (i != KNOWN_FILE_EXTS.length - 1) {
                buf.append(",");
            }
        }

        buf.append(']');

        return new GseaFileFilter[]{
                new GseaFileFilter(KNOWN_FILE_EXTS, buf.toString()),
                new GseaFileFilter(new String[]{"res", "gct", "pcl", "txt"}, "Dataset files [res, gct, pcl, txt]"),
                new GseaFileFilter(new String[]{"grp", "gmt", "gmx"}, "Gene set database files [grp, gmx, gmt]"),
                new GseaFileFilter(new String[]{"cls"}, "Phenotype files [cls]")
        };
    }

    public static javax.swing.filechooser.FileFilter createGseaFileFilter() {

        StringBuffer buf = new StringBuffer("GSEA supported file types [");
        for (int i = 0; i < KNOWN_FILE_EXTS.length; i++) {
            buf.append(KNOWN_FILE_EXTS[i]);
            if (i != KNOWN_FILE_EXTS.length - 1) {
                buf.append(",");
            }
        }

        buf.append(']');

        return new GseaFileFilter(KNOWN_FILE_EXTS, buf.toString());
    }

} // End class Conf
