package xapps.gsea;


import com.jgoodies.looks.HeaderStyle;
import com.jgoodies.looks.Options;
import com.jidesoft.docking.DefaultDockableHolder;
import com.jidesoft.docking.DefaultDockingManager;
import com.jidesoft.docking.DockingManager;
import com.jidesoft.swing.JideBorderLayout;
import com.jidesoft.swing.JideButton;
import com.jidesoft.swing.JideSplitPane;
import com.jidesoft.swing.JideSwingUtilities;
import edu.mit.broad.genome.Conf;
import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.swing.ImageComponent;
import edu.mit.broad.genome.viewers.AboutViewer;
import edu.mit.broad.genome.viewers.SystemConsoleViewerAction;
import edu.mit.broad.xbench.actions.ShowAppRuntimeHomeDirAction;
import edu.mit.broad.xbench.actions.ShowDefaultOutputDirAction;
import edu.mit.broad.xbench.actions.XAction;
import edu.mit.broad.xbench.actions.ext.BrowserAction;
import edu.mit.broad.xbench.core.ApplicationDialog;
import edu.mit.broad.xbench.core.StatusBar;
import edu.mit.broad.xbench.core.WindowComponent;
import edu.mit.broad.xbench.core.api.*;
import edu.mit.broad.xbench.prefs.PreferencesWidget;
import edu.mit.broad.xbench.prefs.XPreferencesFactory;
import edu.mit.broad.xbench.tui.TaskManager;
import xapps.api.AppDataLoaderAction;
import xapps.api.AppToolLauncherAction;
import xapps.api.PastAnalysisAction;
import xapps.api.frameworks.WorkspaceToolBar;
import xapps.api.frameworks.fiji.StatusBarJideImpl;
import xapps.api.frameworks.fiji.WindowManagerImplJideTabbedPane;
import xapps.browser.MSigDBViewerAction;
import xtools.api.Tool;
import xtools.chip2chip.Chip2Chip;
import xtools.gsea.Gsea;
import xtools.gsea.GseaPreranked;
import xtools.munge.CollapseDataset;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.File;

/**
 *
 */
public class GseaFijiTabsApplicationFrame extends DefaultDockableHolder implements Application.Handler {

    static {

        System.setProperty("GSEA", Boolean.TRUE.toString()); // needed for vdb manager to work properly

    }

    private static final int INITIAL_LHS_WIDTH = 200;

    //public static final String BUILD_DATE = "Feb10_2006_build";
    public static final String RPT_CACHE_BUILD_DATE = "April4_2006_build";

    private static String USER_VISIBLE_FRAME_TITLE = "GSEA v2.0.10 (Gene set enrichment analysis -- Broad Institute)";

    /**
     * Application's Icon that people see in their operating system task bar
     */
    private static final Image ICON = JarResources.getImage("XBench.gif");

    // @note IMP IMP: this is the name under which docking prefs etc are stored
    public static final String PROFILE_NAME = "gsea";

    // -------------------------------------------------------------------------------------------- //

    private StatusBar fStatusBar;

    private GseaFijiTabsApplicationFrame fFrame = this;

    private WindowAdapter fWindowListener;

    private SystemConsoleViewerAction fScva;

    private MyWindowManagerImplJideTabbedPane fWindowManager;

    /**
     * Class constructor
     *
     * @throws HeadlessException
     */
    public GseaFijiTabsApplicationFrame() {
        super(USER_VISIBLE_FRAME_TITLE);

        fFrame.setVisible(false);

        //fFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        fFrame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE); // we catch and ask
        fFrame.setIconImage(ICON);

        // add a widnow listener to do clear up when windows closing.
        fWindowListener = new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                exitApplication(e);
            }
        };

        fFrame.addWindowListener(fWindowListener);

        // Set the profile key
        fFrame.getDockingManager().setProfileKey(PROFILE_NAME);

        // Uses light-weight outline. There are several options here.
        fFrame.getDockingManager().setOutlineMode(DockingManager.PARTIAL_OUTLINE_MODE);

        // Now let's start to addFrame()
        fFrame.getDockingManager().beginLoadLayoutData();

        fFrame.getDockingManager().setInitSplitPriority(DefaultDockingManager.SPLIT_SOUTH_NORTH_EAST_WEST);

        // -------------------------------------------------------------------------------------------- //

        this.fWindowManager = new MyWindowManagerImplJideTabbedPane();

        this.fStatusBar = new StatusBarJideImpl();

        Application.registerHandler(this);

        // add menu bar
        fFrame.setJMenuBar(createMenuBar());
        jbInit();
    }

    public void makeVisible(final boolean bring2front) {
        // load layout information from previous session. This indicates the end of beginLoadLayoutData() method above.
        // This makes the frame visible
        fFrame.getDockingManager().loadLayoutData();

        // disallow drop dockable frame to workspace area
        fFrame.getDockingManager().getWorkspace().setAcceptDockableFrame(false);

        if (bring2front) {
            fFrame.toFront();
        }
    }

    // private Runnable fRunnable;

    private AppToolLauncherAction fGseaTool_launcher;
    private AppDataLoaderAction fAppDataLoaderAction;
    private Gsea fGseaTool;

    // contains routines that make future displays faster
    public void init_bg_while_splashing() {

        try {
            Application.getFileManager().getFileChooser();
            Application.getFileManager().getDirChooser("test");
        } catch (Throwable t) {
            System.out.println("Error background initing: " + t);
        }

        fAppDataLoaderAction.getWidget(); // init it

        this.fGseaTool_launcher.getWidget();

        System.out.println("Done initing things while splashing");
    }

    private void jbInit() {

        fFrame.getContentPane().setLayout(new BorderLayout());

        JideSplitPane split = new JideSplitPane();

        split.add(createToolBar());

        split.setShowGripper(true);
        split.setContinuousLayout(false);

        split.add(fWindowManager.getTabbedPane());

        fWindowManager.openWindow(createStartupPanel());

        try {
            fWindowManager.getTabbedPane().setTabClosableAt(0, false);
        } catch (Throwable t) {

        }

        fFrame.getContentPane().add(split);

        // create one project tab for current project
        fFrame.getContentPane().add(fStatusBar.getAsComponent(), BorderLayout.AFTER_LAST_LINE);
    }

    private JComponent createToolBar() {

        WorkspaceToolBar tb1 = new WorkspaceToolBar(JToolBar.VERTICAL);
        //WorkspaceToolBar2 tb1 = new WorkspaceToolBar2(4, 1);

        tb1.setFloatable(false);
        tb1.setBorder(BorderFactory.createTitledBorder("Steps in GSEA analysis"));

        // rhs buttons
        int width = 75;
        int height = 60;
        Font font = new Font("Arial", Font.BOLD, 12);

        int struth = 15;

        // put the main thing first

        fAppDataLoaderAction = new AppDataLoaderAction();

        fGseaTool = new Gsea();
        this.fGseaTool_launcher = new AppToolLauncherAction(fGseaTool, fGseaTool.getParamSet(),
                //JarResources.getIcon("GseaApp16.gif")), width, height, font, true);
                "Run Gsea", JarResources.getIcon("Gsea_app16_v2.png"));

        tb1.add(JarResources.toURL("LocalFileExplorerWidget32.gif"), "Load data          ",
                fAppDataLoaderAction, width, height, font, true);
        tb1.add(Box.createVerticalStrut(struth));

        tb1.add(JarResources.toURL("GseaApp24.gif"), "Run GSEA           ",
                fGseaTool_launcher, width, height, font, true);
        tb1.add(Box.createVerticalStrut(struth));

        tb1.add(JarResources.toURL("Lev32.gif"), "Leading edge analysis", new LeadingEdgeReportAction(), width, height, font, true);
        tb1.add(Box.createVerticalStrut(struth));


        WorkspaceToolBar tb2 = new WorkspaceToolBar(JToolBar.VERTICAL);
        //WorkspaceToolBar2 tb2 = new WorkspaceToolBar2(3, 1);
        tb2.setBorder(BorderFactory.createTitledBorder("Gene set tools"));
        tb2.setFloatable(false);

        //Tool fChip2ChipTool = new Chip2Chip();
        Tool fChip2ChipTool = new Chip2Chip();
        tb2.add(JarResources.toURL("Chip2Chip24_b.gif"), "Chip2Chip mapping",
                new AppToolLauncherAction(fChip2ChipTool, fChip2ChipTool.getParamSet(),
                        "Chip2Chip", JarResources.getIcon("Chip2Chip16.gif")), width, height, font, true);

        tb2.add(Box.createVerticalStrut(struth));

        tb2.add(JarResources.toURL("msigdb.png"), "Browse MSigDB", new MSigDBViewerAction(), width, height, font, true);
        tb2.add(Box.createVerticalStrut(struth));

        WorkspaceToolBar tb3 = new WorkspaceToolBar(JToolBar.VERTICAL);
        //WorkspaceToolBar2 tb3 = new WorkspaceToolBar2(2, 1);
        tb3.setBorder(BorderFactory.createEtchedBorder());
        tb3.setFloatable(false);

        tb3.add(JarResources.toURL("past_analysis32.gif"), "Analysis history", new PastAnalysisAction(), width, height, font, true);
        tb3.add(Box.createVerticalStrut(struth));

        tb3.add(Box.createVerticalStrut(50));

        WorkspaceToolBar tb = new WorkspaceToolBar(JToolBar.VERTICAL);
        tb.setFloatable(false);
        tb.add(tb1);
        tb.add(tb2);
        tb.add(tb3);

        /*
        JPanel tb = new JPanel(new GridLayout(3, 1));
        tb.add(tb1);
        tb.add(tb2);
        tb.add(tb3);
        tb.setPreferredSize(new Dimension(200, 500));
        */

        JideSplitPane pane = new JideSplitPane(JideSplitPane.VERTICAL_SPLIT);
        pane.setShowGripper(true);
        pane.add(tb);
        pane.add(createProcessForToolBar());

        return pane;
    }

    private WindowComponent createStartupPanel() {

        final JPanel mainPanel = new JPanel(new JideBorderLayout());
        final JScrollPane sp = new JScrollPane(new ImageComponent(JarResources.getImage("intro_screen.jpg"), false));

        return new WindowComponent() {

            public JComponent getWindowUI() {
                return sp;
            }

            public String getTitle() {
                return "Home";
            }

            public boolean allowClose() {
                return false;
            }

            public Icon getIcon() {
                return null;
            }

        };
    }

    private JComponent createProcessForToolBar() {

        TaskManager tm = TaskManager.getInstance();
        final JTable taskTable = tm.createTable();
        tm.setOnClickShowResultsInBrowserOnly(true); // @note

        JScrollPane sp = new JScrollPane(taskTable);
        sp.setPreferredSize(new Dimension(INITIAL_LHS_WIDTH, 350));

        //final JPanel leftPanel = new SimpleInternalPanel("<html><body>Processes&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp<a href=\"dd\">Click for details...</a></body></html>");
        //final JPanel leftPanel = new SimpleInternalPanel("<html><body>Processes&nbsp&nbsp&nbsp&nbsp Click 'status' field for results</body></html>");

        final JPanel leftPanel = new JPanel(new BorderLayout());
        leftPanel.add(new JLabel("<html><body>Processes: click 'status' field for results</body></html>"), BorderLayout.NORTH);
        leftPanel.setBackground(Color.LIGHT_GRAY);
        leftPanel.addMouseListener(new MouseAdapter() {
            public void mouseEntered(MouseEvent e) {
                leftPanel.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
            }

            public void mouseExited(MouseEvent e) {

                leftPanel.setCursor(Cursor.getDefaultCursor());
            }

        });

        taskTable.addMouseListener(new MouseAdapter() {
            public void mouseEntered(MouseEvent e) {
                taskTable.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
            }

            public void mouseExited(MouseEvent e) {
                taskTable.setCursor(Cursor.getDefaultCursor());
            }
        });


        leftPanel.setBorder(BorderFactory.createTitledBorder("GSEA reports"));
        /*
        leftPanel.addMouseListener(new MouseAdapter() {

            public void mouseClicked(MouseEvent e) {

                if (fScva == null) {
                    fScva = new SystemConsoleViewerAction();
                }

                System.out.println("Showing details");
                fScva.actionPerformed(null);

            }
        });
        */

        leftPanel.add(sp, BorderLayout.CENTER);

        JideButton but = new JideButton("Show results folder");
        but.setAction(new ShowDefaultOutputDirAction("Show results folder"));
        but.setRolloverEnabled(true);
        but.setBorderPainted(false);
        but.setButtonStyle(JideButton.HYPERLINK_STYLE);
        //but.setAlwaysShowHyperlink(true);
        leftPanel.add(but, BorderLayout.SOUTH);

        return leftPanel;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    /**
     * @return
     * @note Placing this here rather in ActionFactory as a developing aid
     * - > quicker launch of XReg isnt initialized at startup
     * The applications menu bar is defined here.
     * @maint if new actions are added -> need to review to see if they should also
     * be added to the menu bar.
     */

    private JMenuBar createMenuBar() {

        JMenuBar menuBar = new JMenuBar();

        // @note JGOODIES SUGGESTIONS
        menuBar.putClientProperty(Options.HEADER_STYLE_KEY, HeaderStyle.SINGLE);

        menuBar.add(createJMenu("File", new Object[]{
                new MyExitAction()
        }));

        GseaPreferencesAction gpa = new GseaPreferencesAction();

        menuBar.add(createJMenu("Options", new Object[]{
                gpa,
                null,
                gpa.fAsk,
                gpa.fOnline,
                null,
                gpa.fMedian,
                gpa.fFix,
                gpa.fBiased,
                null,
                new ClearFileHistoryAction()
        }));

        menuBar.add(createJMenu("Downloads", new Object[]{
                new DownloadArrayAnnotationsAction(),
                new DownloadGseaExamplesAction()}));

        final GseaPreranked gsea_tool = new GseaPreranked();
        final CollapseDataset cd = new CollapseDataset();


        menuBar.add(createJMenu("Tools",
                new Object[]{
                        new AppToolLauncherAction(gsea_tool, gsea_tool.getParamSet(),
                                "Run Gsea on a Pre-Ranked gene list", JarResources.getIcon("Gsea_app16_v2.png")
                        ),
                        new AppToolLauncherAction(cd, cd.getParamSet(),
                                "Collapse Dataset from Probes to Symbols", JarResources.getIcon("ProjectSpecific16.png"))
                }));

        menuBar.add(createJMenu("Help", new Object[]{
                new GseaHelpAction("GSEA web site",
                        "Open the GSEA website in a web browser",
                        GseaWebResources.getGseaBaseURL()),
                new GseaHelpAction(),
                null,
                new ShowAppRuntimeHomeDirAction("Show GSEA home folder"),
                new ShowDefaultOutputDirAction("Show GSEA output folder (default location)"),
                //null,
                // new SoftwareUpdatesAction(),
                null,
                new FeedbackAction(),
                null,
                new CreditsAction(),
                new AboutAction(),
                JarResources.getBuildInfo()
        }));

        return menuBar;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    class ClearFileHistoryAction extends XAction {

        public ClearFileHistoryAction() {
            this.putValue(NAME, getName());
            this.putValue(Action.SHORT_DESCRIPTION, getDescription());
        }

        public void actionPerformed(final ActionEvent evt) {
            boolean res = Application.getWindowManager().showConfirm("Clear file history", "Are you sure you want to erase all file history");
            if (res) {
                Application.getFileManager().getRecentFilesStore().clearAll();
                if (fAppDataLoaderAction != null) {
                    fAppDataLoaderAction.getWidget();
                }
            }
        }

        public String getId() {
            return "ClearAction";
        }

        public String getName() {
            return "Clear recent file history";
        }

        public Icon getIcon() {
            return null;
        }

        public String getDescription() {
            return "Clear recent file history (from the 'Load Data' panel)";
        }
    }    // End inner class ExitAction


    static class DownloadGeneSetsAction extends BrowserAction {
        public DownloadGeneSetsAction() {
            super("Download gene sets", "Query and download gene sets from the Broad Institutes database", null, GseaWebResources.getMSigDBURL());
        }
    }

    static class DownloadArrayAnnotationsAction extends BrowserAction {
        public DownloadArrayAnnotationsAction() {
            super("Download chip annotations", "Download array annotation files - useful to annotate GSEA reports", null, GseaWebResources.getArrayAnnotationsURL());
        }
    }

    static class DownloadGseaExamplesAction extends BrowserAction {
        public DownloadGseaExamplesAction() {
            super("Download example datasets", "Download example datasets for GSEA - expression data, phenotype labels and gene sets files", null, GseaWebResources.getGseaExamplesURL());
        }
    }

    static class SoftwareUpdatesAction extends BrowserAction {
        public SoftwareUpdatesAction() {
            super("Check for software updates", "Check the GSEA website for software updates", null, GseaWebResources.getGseaSoftwareURL());
        }
    }

    /**
     * @param name
     * @param objs -> array of Action objects interspersed with nulls wherever
     *             a seperator is needed.
     *             For example new Object[]{Foo, Bar, null, Zoo};
     * @return
     */
    private JMenu createJMenu(final String name, final Object[] objs) {

        JMenu menu = new JMenu(name, true);    // true -> can tear off

        for (int i = 0; i < objs.length; i++) {
            if (objs[i] == null) {
                menu.addSeparator();
            } else {
                if (objs[i] instanceof JMenuItem) {
                    menu.add((JMenuItem) objs[i]);
                } else if (objs[i] instanceof String) {
                    menu.add(objs[i].toString());
                } else {
                    menu.add(new JMenuItem((Action) objs[i]));
                }
            }
        }

        return menu;
    }

    private void exitApplication(WindowEvent e_opt) {
        boolean ask = XPreferencesFactory.kAskBeforeAppShutdown.getBoolean();
        if (ask) {
            final boolean res = getWindowManager().showConfirm("Exit the application?");
            if (!res) {
                return;
            }
        }
        /*
        if (e_opt != null) {
            fWindowListener.windowClosing(e_opt);
        }
        */

        fFrame.removeWindowListener(fWindowListener);
        fWindowListener = null;

        if (fFrame.getDockingManager() != null) {
            fFrame.getDockingManager().saveLayoutData();
        }

        fFrame.dispose();
        fFrame = null;
        if (Conf.isDebugMode() == false) {
            Conf.exitSystem(false);
        }
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    /**
     * For gsea preferences
     */
    class GseaPreferencesAction extends AbstractAction {

        private JCheckBoxMenuItem fAsk;
        private JCheckBoxMenuItem fOnline;
        private JCheckBoxMenuItem fMedian;
        private JCheckBoxMenuItem fFix;
        private JCheckBoxMenuItem fBiased;

        /**
         * Class constructor
         */
        public GseaPreferencesAction() {
            this.putValue(Action.NAME, getName());
            this.putValue(Action.SMALL_ICON, getIcon());
            this.putValue(Action.SHORT_DESCRIPTION, getDescription());

            fAsk = XPreferencesFactory.createCheckBoxMenuItem(XPreferencesFactory.kAskBeforeAppShutdown);
            fOnline = XPreferencesFactory.createCheckBoxMenuItem(XPreferencesFactory.kOnlineMode);
            fMedian = XPreferencesFactory.createCheckBoxMenuItem(XPreferencesFactory.kMedian);
            fFix = XPreferencesFactory.createCheckBoxMenuItem(XPreferencesFactory.kFixLowVar);
            fBiased = XPreferencesFactory.createCheckBoxMenuItem(XPreferencesFactory.kBiasedVar);
        }

        public void actionPerformed(final ActionEvent evt) {
            // make this new every time so that the GUI doestn cache settings
            // (i.e read from rpefs)

            GseaPreferencesDialog opt = new GseaPreferencesDialog(fFrame, "Preferences");
            opt.pack();
            JideSwingUtilities.globalCenterWindow(opt);
            opt.setVisible(true);

            // @note hack to get the menu in synch with the prefs. Lazy to make a listener scheme
            if (fAsk != null) {
                fAsk.setSelected(XPreferencesFactory.kAskBeforeAppShutdown.getBoolean());
                fOnline.setSelected(XPreferencesFactory.kOnlineMode.getBoolean());
                fMedian.setSelected(XPreferencesFactory.kMedian.getBoolean());
                fFix.setSelected(XPreferencesFactory.kFixLowVar.getBoolean());
                fBiased.setSelected(XPreferencesFactory.kBiasedVar.getBoolean());
            }
        }

        public String getId() {
            return "PreferencesAction";
        }

        public String getName() {
            return "Preferences ...";
        }

        public Icon getIcon() {
            return PreferencesWidget.ICON;
        }

        public String getDescription() {
            return "View and modify application wide preferences";
        }

    }

    private static JComponent createGseaAppInfoComponent() {

        JPanel panel = new JPanel(new BorderLayout());

        JComponent ic = new ImageComponent(JarResources.getImage("gsea_about2.png"), false);
        ic.setPreferredSize(new Dimension(ic.getPreferredSize().width, 400));

        JScrollPane sp = new JScrollPane(ic);
        sp.setPreferredSize(new Dimension(ic.getPreferredSize().width, 400));

        panel.add(sp, BorderLayout.CENTER);
        //panel.add(ic, BorderLayout.CENTER);

        StringBuffer buf = new StringBuffer("Gene Set Enrichment Analysis (GSEA)\n\n");

        buf.append("Debug mode (for development)  : ").append(Conf.isDebugMode()).append('\n');
        buf.append("\nMemory Available on Machine : ").append(Runtime.getRuntime().maxMemory() / 1000000).append("Mb\n");
        buf.append("(At least 256Mb of memory is recommended. More the better!)\n");
        buf.append("Broad Institute of MIT and Harvard\n");
        buf.append("Support web site: ").append(GseaWebResources.getGseaBaseURL()).append('\n');
        buf.append("Support email: ").append(GseaWebResources.getGseaEmail()).append('\n');
        buf.append('\n');

        final JTextArea ta = new JTextArea(buf.toString());
        ta.setEditable(false);
        ta.setPreferredSize(new Dimension(ic.getPreferredSize().width, 100));
        sp = new JScrollPane(ta);
        sp.setPreferredSize(new Dimension(ic.getPreferredSize().width, 100));
        panel.add(sp, BorderLayout.SOUTH);

        return panel;
    }

    private static JComponent createMethodologyInfoComponent() {

        StringBuffer buf = new StringBuffer("Gene Set Enrichment Analysis (GSEA)\n\n");

        buf.append("This software implements the methodology described in:\n\n");
        buf.append("Gene Set Enrichment Analysis: A Knowledge-Based Approach for Interpreting\nGenome-wide Expression Profiles\n");
        buf.append("PNAS 2005\n\n");
        buf.append("Aravind Subramanian, Pablo Tamayo et al.\n");

        buf.append("\n");
        buf.append("The Kolmogorov-Smirnov statistic for mining gene expression data is described in Lamb, J., Ramaswamy, S., Ford, H. L., Contreras, B., Martinez, R. V., Kittrell, F. S., Zahnow, C. A., Patterson, N., Golub, T. R., and Ewen, M. E. (2003). A mechanism of cyclin D1 action encoded in the patterns of gene expression in human cancer. Cell 114, 323-334.\n");
        buf.append("GSEA was first reported in:" +
                " Mootha, V. K., Lindgren, C. M., Eriksson, K. F., Subramanian, A., Sihag, S.," +
                " Lehar, J., Puigserver, P., Carlsson, E., Ridderstrale, M., Laurila, E., et al. (2003)." +
                "  PGC-1alpha-responsive genes involved in oxidative phosphorylation are coordinately " +
                "downregulated in human diabetes. Nat Genet 34, 267-273.\n");
        buf.append("GSEA v2 software is described in TBD\n");
        buf.append("The MSigDB database is described in TBD\n");
        buf.append("\nDepending on your work, there are various ways of citing the GSEA methodology and software.\n");
        buf.append("Please check our website for a full list of contributors to GSEA concepts and publications\n");

        buf.append("Broad Institute of MIT and Harvard\n\n");

        final JTextArea ta = new JTextArea(buf.toString());
        ta.setEditable(false);
        return new JScrollPane(ta);
    }

    private JComponent createLicenseComponent() {
        String text;
        try {
            text = ParseUtils.slurp(JarResources.toURL("GseaLicense.txt"), false);
        } catch (Throwable t) {
            text = "Could not read GSEA license .. please consult the GSEA website";
        }

        JTextArea ta = new JTextArea(100, 85);
        ta.setText(text);
        ta.setEditable(false);
        return new JScrollPane(ta);
    }

    private JComponent createSoftwareCreditsComponent() {

        StringBuffer buf = new StringBuffer("Software credits\n\n");
        buf.append("GSEA desktop software design & implementation:\n\n");
        buf.append("Aravind Subramanian (as part of PhD thesis)\n");
        buf.append("Project Advisors: Jill Mesirov, Pablo Tamayo, Todd Golub & Eric Lander\n");
        buf.append("\n");

        buf.append("Important software contributions were made by:\n");
        buf.append("Joshua Gould (heat map, leading edge & several other utilities)\n");
        buf.append("\n");

        buf.append("Heidi Kuehn wrote the documentation and wiki\n");
        buf.append("\n");

        buf.append("Additionally, we were helped by:\n");
        buf.append("Lukas Karlsson, Jesse Ross, and Sebastian Diaz (wiki, systems)\n");
        buf.append("Charlotte Henson, Jim Lerner, Helga Thorvaldsdottir, and Michael Reich (Cancer Informatics)\n");

        buf.append("\n");
        buf.append("Please check our website for a full list of contributors to GSEA concepts and publications\n");

        buf.append("\nBroad Institute of MIT and Harvard\n");
        buf.append("\n--------------------------------------------------------------------\n");
        buf.append("3rd party software acknowledgements:\n");
        buf.append("Sun Microsystems (http://java.sun.com): for Java!\n");
        buf.append("JIDE Software (http://www.jidesoft.com): a terrific docking & GUI component library\n");
        buf.append("Karsten Lentzsch (http://www.jgoodies.com): JGoodies look and feel and Forms library\n");
        buf.append("Intellij IDEA (http://www.jetbrains.com): Java IDE\n");
        buf.append("David Gilbert (http://www.jfree.org/jfreechart): GSEA plots use the JFreeChart library\n");
        buf.append("Log4J (http://logging.apache.org/log4jt): Logging toolkit\n");
        buf.append("Trove (http://trove4j.sourceforge.net/): High performance collections library\n");
        buf.append("Erich Gamma & Kent Beck (http://www.junit.org): JUnit testing framework\n");
        buf.append("Apache Jakarta ECS (http://jakarta.apache.org/ecs/): HTML element construction set API\n");
        buf.append("\n\n");
        buf.append("Built on xtools using the FijiTabs framework");

        final JTextArea ta = new JTextArea(buf.toString());
        ta.setEditable(false);
        return new JScrollPane(ta);
    }

    class CreditsAction extends AbstractAction {

        /**
         * Class constructor
         */
        CreditsAction() {
            this.putValue(Action.NAME, getName());
            this.putValue(Action.SMALL_ICON, getIcon());
            this.putValue(Action.SHORT_DESCRIPTION, getDescription());
        }

        public void actionPerformed(final ActionEvent evt) {
            JTabbedPane tp = new JTabbedPane();
            tp.addTab("Software", createSoftwareCreditsComponent());
            tp.addTab("Algorithm", createMethodologyInfoComponent());
            fWindowManager.showMessage("Credits", tp);
        }

        public String getId() {
            return "CreditsAction";
        }

        public String getName() {
            return "Credits";
        }

        public Icon getIcon() {
            return JarResources.getIcon("Credits.gif");
        }

        public String getDescription() {
            return "Credits and acknowledgements for GSEA java software";
        }

    }

    class AboutAction extends AbstractAction {

        AboutAction() {
            this.putValue(Action.NAME, "About");
            this.putValue(Action.SMALL_ICON, getIcon());
            this.putValue(Action.SHORT_DESCRIPTION, getDescription());
        }

        /**
         * IMP intentionally made modal and app wide
         *
         * @param evt
         */
        public void actionPerformed(ActionEvent evt) {
            JComponent gseaAppInfo = createGseaAppInfoComponent();
            ApplicationDialog.showWidget(new AboutViewer(new String[]{"Application info", "License"},
                    new JComponent[]{gseaAppInfo, createLicenseComponent()}), true);
        }

        public String getId() {
            return "AboutAction";
        }

        public String getName() {
            return "About";
        }

        public Icon getIcon() {
            return JarResources.getIcon("XBench.gif");
        }

        public String getDescription() {
            return "Information about the GSEA Application";
        }
    }

    class MyWindowManagerImplJideTabbedPane extends WindowManagerImplJideTabbedPane {

        MyWindowManagerImplJideTabbedPane() {
            super(fFrame);
        }

        public JPopupMenu createPopupMenu(final Object obj) {
            return GseaActionRegistry.getInstance().createPopup(obj);
        }

        public XAction createAction(final Class clazz, final File files[]) {
            return GseaActionRegistry.getInstance().createAction(clazz, files);
        }

        // simply import
        public boolean runDefaultAction(final Object obj) {
            return GseaActionRegistry.getInstance().runDefaultAction(obj);
        }

    } // End class MyWindowManagerImplJideTabbedPane

    public static final String TITLE = "Exit";

    class MyExitAction extends XAction {
        public MyExitAction() {
            this.putValue(Action.NAME, TITLE);
            this.putValue(Action.SHORT_DESCRIPTION, getDescription());
        }

        public void actionPerformed(final ActionEvent evt) {
            exitApplication(null);
        }

        public String getId() {
            return "ExitAction";
        }

        public String getName() {
            return TITLE;
        }

        public Icon getIcon() {
            return null;
        }

        public String getDescription() {
            return "Quit the GSEA application";
        }
    }    // End inner class ExitAction

    // -------------------------------------------------------------------------------------------- //
    // --------------------------- APPLICATION HANDLER IMPLEMENTATION ------------------------------ //
    // -------------------------------------------------------------------------------------------- //

    private static final VdbManager fVdbmanager = new VdbManagerForGsea(RPT_CACHE_BUILD_DATE);

    private ToolManagerImpl fToolManager;

    private FileManager fFileManager;

    public String getName() {
        return "GSEA";
    }

    public boolean isFirstExecutionOfNewVersion() {
        return false; // @todo
    }

    public ToolManager getToolManager() {

        if (fToolManager == null) {
            this.fToolManager = new ToolManagerImpl();
        }

        return fToolManager;
    }

    public FileManager getFileManager() {
        if (fFileManager == null) {
            this.fFileManager = new FileManagerImpl();
        }

        return fFileManager;
    }

    public VdbManager getVdbManager() {
        return fVdbmanager;
    }

    public WindowManager getWindowManager() throws HeadlessException {
        return fWindowManager;
    }

    public StatusBar getStatusBar() {
        return fStatusBar;
    }

    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //
    // -------------------------------------------------------------------------------------------- //


} // End class GseaFiji2Application

