/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xapps.gsea;

import edu.mit.broad.genome.parsers.AuxUtils;
import xtools.api.Tool;

/**
 * @author Aravind Subramanian
 *         Date: Feb 5, 2005
 *         Time: 7:02:54 PM
 */
public class GseaWebResources {

    private final static String GSEA_FTP_SERVER = "gseaftp.broadinstitute.org";
    //private final static String GSEA_FTP_SERVER_USERNAME = "username";
    //private final static String GSEA_FTP_SERVER_PASSWORD = "password";
    private final static String GSEA_FTP_SERVER_USERNAME = "gs3as3cr3tz";
    private final static String GSEA_FTP_SERVER_PASSWORD = "asdf7hEGw";    
    private final static String GSEA_FTP_SERVER_BASE_DIR = "/pub/gsea";
    private final static String GSEA_FTP_SERVER_CHIPFILES_SUB_DIR = "annotations";
    private final static String GSEA_FTP_SERVER_GENESETS_SUB_DIR = "gene_sets";
    private final static String GSEA_FTP_SERVER_MSIGDB_FILE_CURRENT = "msigdb_v5.0.xml";

    public static String getGseaFTPServer() {
        return GSEA_FTP_SERVER;
    }

    public static String getGseaFTPServerUserName() {
        return GSEA_FTP_SERVER_USERNAME;
    }

    public static String getGseaFTPServerPassword() {
        return GSEA_FTP_SERVER_PASSWORD;
    }

    public static String getFTPBase() {
        return "ftp://" + GSEA_FTP_SERVER + GSEA_FTP_SERVER_BASE_DIR;
    }

    public static String getGseaFTPServerChipDir() {
        return GSEA_FTP_SERVER_BASE_DIR + "/" + GSEA_FTP_SERVER_CHIPFILES_SUB_DIR;
    }

    public static String getGseaFTPServerGeneSetsDir() {
        return GSEA_FTP_SERVER_BASE_DIR + "/" + GSEA_FTP_SERVER_GENESETS_SUB_DIR;
    }

    public static String getMSigDB_current_xml_fileName() {
        return GSEA_FTP_SERVER_MSIGDB_FILE_CURRENT;
    }

    public static String getGseaBaseURL() {
        if (edu.mit.broad.genome.Conf.isDebugMode()) {
            return "http://wwwdev.broadinstitute.org/gsea";
        } else {
            return "http://www.broadinstitute.org/gsea";
        }
    }
    
    public static String getGsaaBaseURL() {
        if (edu.mit.broad.genome.Conf.isDebugMode()) {
            return "http://gsaa.unc.edu/login";
        } else {
            return "http://gsaa.unc.edu/login";
        }
    }    

    public static String getCancerHomePageURL() {
        return "http://www.broadinstitute.org/cancer";
    }

    public static String getGseaEmail() {
        return "gsea@broadinstitute.org";
    }
    
    public static String getGsaaEmail() {
        return "qxiong@email.unc.edu";
    }    

    public static String getGseaURLDisplayName() {
        return "www.broadinstitute.org/GSEA";
    }
    
    public static String getGsaaURLDisplayName() {
        return "gsaa.unc.edu/GSAA";
    }    

    public static String getGseaImagesURL() {
        return getGseaBaseURL() + "/images";
    }

    public static String getGseaHelpURL() {
        return getGseaBaseURL() + "/wiki";
    }
    
    public static String getGsaaHelpURL() {
        return getGsaaBaseURL() + "/documentation.html";
    }    

    public static String getGseaFeedbackURL() {
        return getGseaBaseURL() + "/software/feedback.html";
    }
    
    public static String getGsaaFeedbackURL() {
        return getGsaaBaseURL() + "/contact.html";
    }    

    public static String getGseaHelp_with_gui_URL() {
        return getGseaBaseURL() + "/doc/gsea_desktop_application_tutorial.pps";
    }

    public static String getGseaDataFormatsHelpURL() {
        return getGseaBaseURL() + "/wiki/index.php/Data_formats";
    }

    public static String getMSigDBURL() {
        return getGseaBaseURL() + "/msigdb/msigdb_index.html";
    }


    public static String getArrayAnnotationsURL() {
        //return getGseaBaseURL() + "/resources/arrays_index.html";
        return getFTPBase() + "/annotations";
    }

    public static String getGseaExamplesURL() {
        return getGseaBaseURL() + "/datasets.jsp";
    }

    public static String getGseaSoftwareURL() {
        return getGseaBaseURL() + "/software/software_index.html";
    }

    public static String getGseaToolHelpURL(Tool tool) {
        return getGseaBaseURL() + "/doc/" + tool.getName() + ".html";
    }

    public static String getGseaToolHelpURL(String toolName) {
        return getGseaBaseURL() + "/doc/" + toolName + ".html";
    }

    public static String getGeneSetURL(String gsetName) {
        gsetName = AuxUtils.getAuxNameOnlyNoHash(gsetName);
        return getGseaBaseURL() + "/msigdb/cards/" + gsetName + ".html";
    }

    public static String getPubMedURL(final String pmid) {
        return "http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?cmd=Retrieve&db=PubMed&dopt=Citation&list_uids=" + pmid;
    }

} // End classes AppWebResources
