package xapps.gsea;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jidesoft.grid.*;
import com.jidesoft.swing.*;
import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.models.NumberedProxyModel;
import edu.mit.broad.genome.models.StringDataframeModel;
import edu.mit.broad.genome.objects.StringDataframe;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentDb;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.reports.EnrichmentReports;
import edu.mit.broad.genome.swing.GuiHelper;
import edu.mit.broad.genome.swing.fields.GDirFieldPlusChooser;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;
import edu.mit.broad.genome.viewers.AbstractViewer;
import edu.mit.broad.xbench.core.api.Application;
import foxtrot.Task;
import foxtrot.Worker;
import gnu.trove.THashSet;
import org.genepattern.gsea.LeadingEdgeWidget;
import xapps.api.vtools.ParamSetFormForAFew;
import xtools.api.param.DirParam;
import xtools.api.param.Param;
import xtools.api.param.ReportCacheChooserParam;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableModel;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Feb 23, 2006
 *         Time: 11:06:17 PM
 */
public class LeadingEdgeReportViewer extends AbstractViewer {

    public static final String NAME = "LeadingEdgeReportWidget";

    public static final Icon ICON = JarResources.getIcon("Lev16_b.gif");

    private ReportCacheChooserParam fReportParam;

    private DirParam fDirParam;

    private SortableTable sortableTable;

    private File curr_gseaResultDir;

    private EnrichmentDb curr_edb;

    private JComponent fFiller;

    private Component tableComponent;

    private LeadingEdgeReportViewer fInstance = this;

    /**
     * Builds a Viewer on specified FSet object
     *
     * @param gset
     */
    public LeadingEdgeReportViewer() {
        super(NAME, ICON, "Leading edge analysis");
        jbInit();
    }


    private JideTabbedPane sharedTabbedPane;

    private void jbInit() {

        JButton bBuild = new JButton("Load GSEA Results");

        bBuild.addActionListener(new ActionListener() {
            public void actionPerformed(final ActionEvent e) {
                if (fReportParam.isSpecified() && fDirParam.isSpecified()) {
                    Application.getWindowManager().showMessage("Both cache and a brows'ed directory were specified. Only 1 can be specified. Delete one and try again");
                } else if (!fReportParam.isSpecified() && !fDirParam.isSpecified()) {
                    Application.getWindowManager().showMessage("No GSEA result folder was specified. Specify one and try again");
                } else {
                    try {
                        Worker.post(new Task() {
                            public Object run() throws Exception {

                                Application.getWindowManager().getRootFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

                                if (fReportParam.isSpecified()) {
                                    curr_gseaResultDir = fReportParam.getReportDir();
                                } else {
                                    curr_gseaResultDir = fDirParam.getDir();
                                }

                                EnrichmentDb edb = ParserFactory.readEdb(curr_gseaResultDir, true);
                                log.debug("edb: " + edb.getQuickInfo());

                                curr_edb = edb;

                                boolean first = false;
                                if (sharedTabbedPane == null) {
                                    first = true;
                                    sharedTabbedPane = new JideTabbedPane();
                                    sharedTabbedPane.setHideOneTab(true);
                                    sharedTabbedPane.setTabEditingAllowed(true);
                                    sharedTabbedPane.setShowCloseButtonOnTab(true);

                                }

                                LeadingEdgeWidget josh_widget = new LeadingEdgeWidget(sharedTabbedPane, edb);
                                try {
                                    sharedTabbedPane.setSelectedComponent(josh_widget.getViewAndSearchComponent());
                                } catch (Throwable t) {
                                }


                                if (first) {
                                    fInstance.remove(fFiller);
                                    fInstance.add(sharedTabbedPane, BorderLayout.CENTER);
                                    //sharedTabbedPane.setTabClosableAt(0, false);
                                }


                                fInstance.revalidate();

                                return null;

                            }
                        });
                    } catch (Throwable t) {
                        Application.getWindowManager().showError("Trouble loading enrichment database", t);
                    } finally {
                        Application.getWindowManager().getRootFrame().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                        ParserFactory.uninstallProgressMonitorHook(); // @note
                    }
                }
            }
        });

        this.fReportParam = new ReportCacheChooserParam("Select a GSEA result from the application cache", false);
        this.fDirParam = new DirParam("dir", "[ OR ] Locate a GSEA result folder from the file system", "[ OR ] Locate a GSEA report folder from the file system", false);
        final Param[] params = new Param[]{fReportParam, fDirParam};

        // -------------------------------------------------------------------------------------------- //
        StringBuffer colStr = _createColStr();
        StringBuffer rowStr = _createRowStr(params);

        PanelBuilder builder = ParamSetFormForAFew.createPanelBuilder(colStr, rowStr);
        CellConstraints cc = new CellConstraints();

        int rowcnt = 3;
        for (int i = 0; i < params.length; i++) {
            GFieldPlusChooser chooser = params[i].getSelectionComponent();

            if (params[i].isFileBased()) {
                if (chooser instanceof GDirFieldPlusChooser) {
                    (((GDirFieldPlusChooser) chooser)).getTextField().setBackground(ParamSetFormForAFew.LIGHT_GREEN);
                }
            }

            JLabel label = new JLabel(params[i].getHtmlLabel_v3());

            ParamSetFormForAFew.enableToolTips(label, params[i]);
            builder.add(label, cc.xy(1, rowcnt));
            builder.add(chooser.getComponent(), cc.xy(3, rowcnt));
            rowcnt += 2; // because the spaces also count as a row
        }

        builder.add(new JLabel(""), cc.xy(1, rowcnt));
        builder.add(bBuild, cc.xy(3, rowcnt));

        final JPanel paramPanel = builder.getPanel();
        // -------------------------------------------------------------------------------------------- //

        this.setLayout(new BorderLayout(10, 10));
        this.add(paramPanel, BorderLayout.NORTH);

        this.fFiller = GuiHelper.createWaitingPlaceholder();
        this.add(fFiller, BorderLayout.CENTER);

        //this.add(createControlPanel(), BorderLayout.SOUTH);

        this.revalidate();
    }


    public String[] getSelectedResultNames() {
        final int[] rows = sortableTable.getSelectedRows();
        final String[] names = new String[rows.length];
        for (int i = 0; i < rows.length; i++) {
            names[i] = sortableTable.getModel().getValueAt(rows[i], 1).toString();
        }

        return names;
    }

    public int getNumSelectedRows() {
        return sortableTable.getSelectedRows().length;
    }

    private Component createViewAndSearchComponent(final EnrichmentDb edb, final String normModeName) {

        StringDataframe sdf = EnrichmentReports.createGseaLikeReport_sdf(edb, normModeName);

        THashSet floatColumnNames = new THashSet();
        floatColumnNames.add("SIZE");
        floatColumnNames.add("ES");
        floatColumnNames.add("NES");
        floatColumnNames.add("NOM p-val");
        floatColumnNames.add("FDR q-val");
        floatColumnNames.add("FWER p-val");

        StringDataframeModel dmodel = new StringDataframeModel(sdf, floatColumnNames);
        dmodel.setFeatureName("GENE SET");

        final TableModel tableModel = new NumberedProxyModel(dmodel);

        final QuickFilterPane quickFilterPane = new QuickFilterPane(new SortableTableModel(tableModel), new int[]{1});

        JPanel quickSearchPanel = new JPanel(new FlowLayout(FlowLayout.LEADING));

        QuickTableFilterField filterField = new QuickTableFilterField(quickFilterPane.getDisplayTableModel(),
                new int[]{1});
        quickSearchPanel.add(filterField);
        quickSearchPanel.setBorder(new JideTitledBorder(new PartialEtchedBorder(PartialEtchedBorder.LOWERED, PartialSide.NORTH),
                "", JideTitledBorder.LEADING, JideTitledBorder.ABOVE_TOP));

        JideSplitPane pane = new JideSplitPane(JideSplitPane.VERTICAL_SPLIT);
        /*
        quickFilterPane.setBorder(BorderFactory.createCompoundBorder(new JideTitledBorder(new PartialEtchedBorder(PartialEtchedBorder.LOWERED, PartialSide.NORTH), "QuickFilterPane", JideTitledBorder.LEADING, JideTitledBorder.ABOVE_TOP),
                BorderFactory.createEmptyBorder(6, 0, 0, 0)));
        pane.addPane(quickFilterPane);
        */

        JPanel tablePanel = new JPanel(new BorderLayout(2, 2));
        tablePanel.setBorder(BorderFactory.createCompoundBorder(new JideTitledBorder(new PartialEtchedBorder(PartialEtchedBorder.LOWERED, PartialSide.NORTH), "Filtered result List", JideTitledBorder.LEADING, JideTitledBorder.ABOVE_TOP),
                BorderFactory.createEmptyBorder(0, 0, 0, 0)));
        final JLabel label = new JLabel(filterField.getDisplayTableModel().getRowCount() + " out of " + tableModel.getRowCount() + " gene sets");
        label.setHorizontalAlignment(SwingConstants.CENTER);
        label.setForeground(GuiHelper.COLOR_DARK_GREEN);
        label.setBorder(BorderFactory.createEmptyBorder(2, 0, 2, 0));

        this.sortableTable = new SortableTable(filterField.getDisplayTableModel());

        sortableTable.setDefaultRenderer(String.class, new ESRenderer(dmodel));

        ListSelectionModel lsm = sortableTable.getSelectionModel();
        lsm.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION); // @note allow many ranked lists to be selected

        lsm.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                //Ignore extra messages.
                if (e.getValueIsAdjusting()) {
                }

                // updateSelectedLists();
            }
        });

        sortableTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        filterField.setTable(sortableTable);
        JScrollPane scrollPane = new JScrollPane(sortableTable, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

        filterField.getDisplayTableModel().addTableModelListener(new TableModelListener() {
            public void tableChanged(TableModelEvent e) {
                if (e.getSource() instanceof FilterableTableModel) {
                    int count = ((TableModel) e.getSource()).getRowCount();
                    label.setText(count + " out of " + tableModel.getRowCount() + " gene sets");
                }
            }
        });

        tablePanel.add(label, BorderLayout.BEFORE_FIRST_LINE);
        tablePanel.add(scrollPane);
        pane.addPane(tablePanel);

        JPanel panel = new JPanel(new BorderLayout(3, 3));
        panel.add(quickSearchPanel, BorderLayout.BEFORE_FIRST_LINE);
        panel.add(pane);

        return panel;
    }


    /**
     * GramTableCellRenderer for table headers.
     * Headers in bold font -- looks like excel headers.
     */
    class ESRenderer extends DefaultTableCellRenderer {

        int nameIndex;
        int esIndex;
        StringDataframeModel sdf;

        ESRenderer(final StringDataframeModel gsea_report_sdf) {
            this.sdf = gsea_report_sdf;
            nameIndex = gsea_report_sdf.getColumnIndex("GENE SET") + 1;
            esIndex = gsea_report_sdf.getColumnIndex(EnrichmentReports.ColNames.ES) + 1;
        }

        public Component getTableCellRendererComponent(JTable table, Object value,
                                                       boolean isSelected, boolean hasFocus, int row, int col) {

            row = sortableTable.getActualRowAt(row);


            super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, col);


            if (col == nameIndex) {
                float es = sdf.getStringDataframe().getElementAsFloat(row, esIndex);
                if (XMath.isPositive(es)) {
                    this.setForeground(Color.RED);
                } else {
                    this.setForeground(Color.BLUE);
                }
                this.setText(value.toString());
            } else {
                this.setForeground(Color.BLACK);
            }

            return this;
        }
    }    // End BoldHeaderRenderer

    private static StringBuffer _createColStr() {
        return new StringBuffer("220dlu,      4dlu,        250dlu,   4dlu,  40dlu"); // columns
    }

    private static StringBuffer _createRowStr(final Param[] params) {
        StringBuffer rowStr = new StringBuffer();
        rowStr.append("pref, 10dlu,"); // for the spacer
        for (int i = 0; i < params.length + 1; i++) { // +1 for the button
            rowStr.append("pref, 5dlu");
            if (params.length != i - 1) {
                rowStr.append(",");
            }
        }
        return rowStr;
    }

    public JMenuBar getJMenuBar() {
        return EMPTY_MENU_BAR;
    }

} // End class LeadingEdgeReportWidget
