/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xapps.gsea;

import com.jidesoft.comparator.ObjectComparatorManager;
import com.jidesoft.swing.SplashScreen;
import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.xbench.ComparatorFactory2;
import org.apache.log4j.Logger;
import xapps.api.MostMain;

import javax.swing.*;

/**
 * Main class of GSEA application
 * Use this in jar files, command line etc to launch the application.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class Main extends MostMain {

    private static final Logger klog = XLogger.getLogger(Main.class);

    /**
     * Class Constructor.
     */
    public Main() {

        super();

        super.setLnF();

        ImageIcon splash = JarResources.getImageIcon("gsea_splash.png");
        SplashScreen.create(splash);
        SplashScreen.show();

        // start up the application

        final GseaFijiTabsApplicationFrame frame = new GseaFijiTabsApplicationFrame();

	try {
	    frame.init_bg_while_splashing();
	} catch (Throwable t) {
	    System.out.println("Error while initializing .., things may not work");
	    t.printStackTrace();
	}

	SplashScreen.hide();

        frame.makeVisible(true);

        // A global object that can register comparator with a type and a ComparatorContext.
        ObjectComparatorManager.registerComparator(Integer.class, new ComparatorFactory2.IntegerComparator());
        ObjectComparatorManager.registerComparator(Float.class, new ComparatorFactory2.FloatComparator());

    }

    /**
     * Main method to launch the Tools Desktop application
     *
     * @param args Ignored
     */
    public static void main(final String[] args) {

        try {
            new Main();
        } catch (Throwable e) {
            e.printStackTrace();
            klog.fatal("Could not create application", e);
        }
    }
}    // End Main
