package xapps.gsea;

import edu.mit.broad.genome.Conf;
import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.TraceUtils;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.objects.GenesOfInterest;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import edu.mit.broad.xbench.core.api.DescriptedFile;
import edu.mit.broad.xbench.core.api.VdbManager;
import edu.mit.broad.xbench.prefs.XPreferencesFactory;
import org.apache.log4j.Logger;

import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Jan 5, 2006
 *         Time: 1:10:01 PM
 */
public class VdbManagerForGsea implements VdbManager {

    private String fCacheDate;

    private static final Logger klog = XLogger.getLogger(VdbManagerForGsea.class);

    /**
     * Class constructor
     */
    public VdbManagerForGsea(final String buildDate) {
        this.fCacheDate = buildDate;
    }

    // @note this is the key preference - everything else is relative to this
    // Except for the vdb tweak
    public File getRuntimeHomeDir() {
        return XPreferencesFactory.kAppRuntimeHomeDir;
    }

    public DescriptedFile[] getDescriptedFiles() {
        return new DescriptedFile[]{
                new DescriptedFile(getDefaultOutputDir(), "OUTPUT"),
                new DescriptedFile(getRuntimeHomeDir(), "HOME"),
                new DescriptedFile(getExamplesDir(), "EXAMPLES"),
                new DescriptedFile(getTmpDir(), "TMP")
        };
    }

    public File getTmpDir() {
        return _mkdir(new File(getRuntimeHomeDir(), "tmp"));
    }

    public File getExamplesDir() {
        return _mkdir(new File(getRuntimeHomeDir(), "xexamples"));
    }

    public File getTestDataDir() {
        return _mkdir(new File(getRuntimeHomeDir(), "test"));
    }

    public File getReportsCacheDir() { // @note that the reports cache dir is specific to the version.
        return _mkdir(new File(getRuntimeHomeDir(), "reports_cache_" + fCacheDate));
    }

    public File getDatabasesDir() {
        return _mkdir(new File(getRuntimeHomeDir(), "databases")); // @note dont notimpl exception out -- vdbruntime doesnt like that
    }

    public File getDefaultOutputDir() {
        //File out = _mkdir(new File(getRuntimeHomeDir(), "output"));
        File out = _mkdir(XPreferencesFactory.kDefaultReportsOutputDir.getDir(false));
        String dn = NamingConventions.createNiceEnglishDate_for_dirs();
        return _mkdir(new File(out, dn));
    }

    private static File _mkdir(final File dir) {
        //TraceUtils.showTrace();
        if (dir.exists() == false) {
            boolean made = dir.mkdir();
            if (!made) {
                if (Conf.isDebugMode()) {
                    TraceUtils.showTrace();
                }
                klog.fatal("Could not make dir: " + dir);
            } else {
                klog.info("Made Vdb dir: " + dir);
            }
        }

        return dir;
    }

    public SampleAnnot getDefaultSampleAnnot() {
        return null;
        //throw new NotImplementedException();
    }

    public GenesOfInterest getDefaultGenesOfInterest() {
        return null;
        //throw new NotImplementedException();
    }

} // End class VdbManagerForGsea

