package xexamples;

import edu.mit.broad.genome.objects.GeneSetMatrix;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.map.Chip2ChipMapper;
import edu.mit.broad.vdb.map.Chip2ChipMapperSymbol;
import edu.mit.broad.vdb.map.MGeneSet;
import edu.mit.broad.vdb.map.MGeneSetMatrix;

import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Feb 10, 2007
 *         Time: 9:17:22 PM
 */
public class Chip2ChipExample1 {


    public static void main(String[] args) {

        try {

            // (1) Read in a GeneSet to-be-mapped (this MUST be in gene symbols)
            GeneSetMatrix gm = ParserFactory.readGeneSetMatrix(new File("c2.v1.gmt"), false);
            System.out.println("genesets: " + gm.getQuickInfo());

            // (2) Read in the TARGET chip
            Chip target_chip = ParserFactory.readChip(new File("HG_U133A.chip"), false);

            // Construct a mapper
            Chip2ChipMapper mapper = new Chip2ChipMapperSymbol(VdbRuntimeResources.getChip_Gene_Symbol(), target_chip);

            // Map it
            MGeneSetMatrix mgms = mapper.map(gm, true);

            for (int i = 0; i < mgms.getNumMappedSets(); i++) {
                MGeneSet mapped_gene_set = mgms.getMappedGeneSet(i);
                System.out.println("Before mapping: " + gm.getGeneSet(i).getNumMembers() + " after mapping: " + mapped_gene_set.getMappedGeneSet(true).getNumMembers());
            }

            // Save the mapped genesetmatrix

            ParserFactory.saveGmt(mgms.getMappedGeneSetMatrix("someprefix"), new File("my_mapped_gene_sets.gmt"));

        } catch (Throwable t) {
            t.printStackTrace();
        }

    }

}
