package xexamples;

import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.alg.cluster.ClusteringAlgs;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.GenesOfInterest;
import edu.mit.broad.genome.objects.HClustDataset;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.utils.ImageUtils;
import edu.mit.broad.genome.utils.SystemUtils;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.vdb.map.Chip2ChipMapper;
import edu.mit.broad.vdb.map.Chip2ChipMapperSymbol;
import edu.mit.broad.vdb.map.MGeneSet;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import org.genepattern.gsea.HCLAlgorithm;

import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Feb 10, 2007
 *         Time: 10:15:38 PM
 */
public class ClusterGeneSetExample {

    static {
        if (!SystemUtils.isPropertyDefined("java.awt.headless")) {
            System.setProperty("java.awt.headless", "true");
        }
    }

    public static void main(String[] args) {


        try {

            VdbRuntimeResources.setChipPathBase("/web/ftp/pub/gsea/annotations/");

            // (1) READ IN THE DATA OBJECTS
            String dataset_path = "ftp://gseaftp.broadinstitute.org/pub/gsea/compendia/gnf2.gct";
            String geneset_path = "http://www.broadinstitute.org/gsea/msigdb/pages/V$GATA1_02.grp";

            GeneSet gset = ParserFactory.readGeneSet(geneset_path, ParserFactory.createInputStream(geneset_path), false);
            Dataset ds = ParserFactory.readDataset(dataset_path, ParserFactory.createInputStream(dataset_path), true); // cache as its the compendium

            // (2) READ IN THE SAMPLE AND GENE ANNOTATIONS
            SampleAnnot sin = ParserFactory.readSampleAnnot("gnf2.sin.xls", true);
            GenesOfInterest gin = ParserFactory.readGenesOfInterest("gsea.gin.xls", true);
            ds.getAnnot().setChip(sin.getChip(), gin.getAsColorMap());
            ds.getAnnot().setSampleAnnot(sin);

            // (3) MAP THE GENE SET TO THE COMPENDIUMS FORMAT
            Chip2ChipMapper mapper = new Chip2ChipMapperSymbol(VdbRuntimeResources.getChip_Gene_Symbol(), sin.getChip());
            MGeneSet gset_mapped = mapper.map(gset, false);
            System.out.println("Done mapping. Before: " + gset.getNumMembers() + " after mapping: " + gset_mapped.getMappedGeneSet(true).getNumMembers());

            // (4) EXTRACT JUST THE GENE SET FROM THE DATASET BEFORE CLUSTERING
            ds = new DatasetGenerators().extractRows(ds, gset_mapped.getMappedGeneSet(true));
            ds = new DatasetGenerators().extractRows("foo", ds, new int[]{0, 1, 2});
            System.out.println("# of rows in extracted dataset: " + ds.getQuickInfo());

            // (5) CLUSTER THE DATASET
            final HClustDataset hclust = ClusteringAlgs.cluster(ds, HCLAlgorithm.SINGLE_LINKAGE, HCLAlgorithm.PEARSON, HCLAlgorithm.BOTH);

            // (6) CREATE AND SAVE IMAGES
            HClustDataset.Images images = hclust.createImages(10); // size of the grid
            ImageUtils.savePng(images.fullImage, new File("full_image.png"));

        } catch (Throwable t) {
            t.printStackTrace();
        }

    }


}
