package xexamples;

import edu.mit.broad.genome.alg.GeneSetGenerators;
import edu.mit.broad.genome.alg.gsea.*;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.GeneSetMatrix;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentDb;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentResult;
import edu.mit.broad.genome.parsers.ClsParser;
import edu.mit.broad.genome.parsers.GctParser;
import edu.mit.broad.genome.parsers.GmtParser;

import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Feb 10, 2007
 *         Time: 9:18:10 PM
 *         <p/>
 *         This is a simple example of running gsea on a matrix of gene sets and obtaining result objects
 */
public class GseaExample {

    public static void main(String[] args) {

        try {

            Dataset ds = (Dataset) new GctParser().parse("my_ds", new File("Leukemia_collapsed_symbols.gct")).get(0);
            GeneSetMatrix gm = (GeneSetMatrix) new GmtParser().parse("foo", new File("c2.v1.gmt")).get(0);
            Template t = (Template) new ClsParser().parse("my_phenotype", new File("Leukemia.cls")).get(0);

            int min_set_size = 25;
            int max_set_size = 500;
            // @note that the gene sets are first restricted to the dataset feature space and then the size thresholds are pplied
            GeneSet[] gsets = GeneSetGenerators.removeGeneSetsBySize(gm, min_set_size, max_set_size, ds);

            KSTests tests = new KSTests();

            int num_permutations = 100;
            int num_markers = 100; // used only for marker analysis - not a gsea algorithm parameter

            boolean permute_phenotype = true; // if false, then gene sets are permuted

            GeneSetScoringTable scoring_scheme = new GeneSetScoringTables.Weighted();
            // another option is the classic kolmogorov-smirnov statistic
            //GeneSetScoringTable scoring_scheme = new GeneSetScoringTables.Classic();

            // OK, parameters all done, make the call
            EnrichmentDb edb = tests.executeGsea(ds, t, gsets, num_permutations, scoring_scheme, permute_phenotype, num_markers);
            System.out.println("Done gsea executing");

            // EDB now has the stats but not the FDRs yet
            // FDRs can be calc in many ways, heres the gsea way:

            PValueCalculator pvc = new PValueCalculatorImpls.GseaImpl("meandiv");
            final EnrichmentResult[] results = pvc.calcNPValuesAndFDR(edb.getResults());
            final EnrichmentDb edb_with_fdr = edb.cloneDeep(results);

            // access statistics from within the edb object ..

            for (int i = 0; i < edb_with_fdr.getNumResults(); i++) {
                EnrichmentResult result = edb_with_fdr.getResult(i);
                System.out.println("name: " + result.getGeneSetName() +
                        " ES: " + result.getScore().getES() +
                        " NES: " + result.getScore().getNES() +
                        " NP: " + result.getScore().getNP() +
                        " FDR: " + result.getScore().getFDR() +
                        " FWER: " + result.getScore().getFWER()
                );
            }


        } catch (Throwable t) {
            t.printStackTrace();
        }

    }

}
