package xexamples;

import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.GeneSetMatrix;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.utils.ImageUtils;
import edu.mit.broad.xbench.heatmap.GramImagerImpl;

import java.awt.image.BufferedImage;
import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Feb 10, 2007
 *         Time: 9:09:40 PM
 */
public class HeatMapExample {

    public static void main(String[] args) {

        try {

            // (1) Read in a Dataset
            Dataset ds = ParserFactory.readDataset(new File("Leukemia_collapsed_symbols.gct"), false);

            // (2) Read in a GeneSet (IMP: the gene set members need to be in the same "format" as the dataset (i.e gene symbols, or affy probe ids)
            GeneSetMatrix gm = ParserFactory.readGeneSetMatrix(new File("c2.v1.gmt"), false);
            System.out.println("genesets: " + gm.getQuickInfo());
            GeneSet plotThisGeneSet = gm.getGeneSet(0); // The first gene set in the collection

            // (3) Read in a phenotype
            Template t = ParserFactory.readTemplate(new File("Leukemia.cls"), false);

            // (4) Create a heat map of the objects and save to file
            GramImagerImpl gim = new GramImagerImpl();
            BufferedImage image = gim.createBpogImage(ds, t, plotThisGeneSet);
            ImageUtils.savePng(image, new File("my_heat_map.png"));

        } catch (Throwable t) {
            t.printStackTrace();
        }

    }

}
