package xexamples;

import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.GeneSetMatrix;
import edu.mit.broad.genome.parsers.GctParser;
import edu.mit.broad.genome.parsers.GmtParser;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.vdb.msigdb.MSigDB;

import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Feb 10, 2007
 *         Time: 8:56:38 PM
 *         <p/>
 *         An example of using the library to parse in objects
 */
public class ParsingExamples {


    public static void main(String[] args) {

        try {
            // READ IN A DATASET

            // Use the ParserFactory and optionally the cache
            Dataset ds = ParserFactory.readDataset(new File("Leukemia.gct"), true);
            System.out.println("dataset: " + ds.getQuickInfo());

            // Read the object in directly
            Dataset ds2 = (Dataset) new GctParser().parse("foo", new File("Leukemia.gct")).get(0);
            System.out.println("dataset: " + ds2.getQuickInfo());

            // READ IN A COLLECTION OF GENE SETS

            // Via ParserFactory
            GeneSetMatrix gm = ParserFactory.readGeneSetMatrix(new File("c2.v1.gmt"), true);
            System.out.println("genesets: " + gm.getQuickInfo());

            // Directly
            GeneSetMatrix gm2 = (GeneSetMatrix) new GmtParser().parse("foo", new File("c2.v1.gmt")).get(0);
            System.out.println("genesets: " + gm2.getQuickInfo());

            // READ IN THE ENTIRE MSIGDB DATABASE
            MSigDB msigdb = ParserFactory.readMSigDB("ftp://gseaftp.broadinstitute.org/pub/gsea/msigdb_v2.xml", true, true);
            System.out.println("MSigDB: " + msigdb.getQuickInfo());

        } catch (Throwable t) {
            t.printStackTrace();
        }


    }

}
