/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtest.api;


import edu.mit.broad.genome.XLogger;
import junit.runner.SimpleTestCollector;
import junit.runner.TestCollector;
import org.apache.log4j.Logger;

import javax.swing.*;
import java.awt.*;


/**
 * Extends junit's swing TestRunner to also print stack traces of
 * errors and failures to stderr. This helps in debugging environments which
 * support clicking on a line of a stack trace and automatically opening
 * up the source file.
 * <p/>
 * In addition, subclass so that we can access the junit swing runner
 * frame (its protected).
 * <p/>
 * Also allows for tool setting of the TestCollector
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class GenomeTestRunner extends junit.swingui.TestRunner {

    protected final Logger log = XLogger.getLogger(GenomeTestRunner.class);

    /**
     * Default test collector
     */
    private TestCollector fCollector = new SimpleTestCollector();

    /**
     * Class Constructor.
     */
    public GenomeTestRunner() {

        super();

        System.out.println("[JUNIT] prop file is at: " + System.getProperty("user.home"));
    }

    public static void run(final java.lang.Class aClass) {
        System.out.println("-->>>>##### RUNNING ACLASS: " + aClass);
        junit.swingui.TestRunner.run(aClass);
    }

    /**
     * Class Constructor.
     * Sets the TestCollector
     */
    public GenomeTestRunner(final TestCollector collector) {

        super();

        this.fCollector = collector;
    }

    public void addError(junit.framework.Test test, Throwable t) {

        super.addError(test, t);
        log.error(t.getMessage(), t);

        //t.printStackTrace();
    }

    public void addFailure(junit.framework.Test test, junit.framework.AssertionFailedError afe) {

        super.addFailure(test, afe);

        //afe.printStackTrace();
        log.error(afe.getMessage(), afe);
    }

    /**
     * This is the magic - we return a custom TestCollector
     * (The junit properties file is ignored - easier to switch programmtically)
     */
    public void browseTestClasses() {

        GenomeTestSelector selector = new GenomeTestSelector(fFrame, fCollector);

        if (selector.isEmpty()) {
            JOptionPane.showMessageDialog(
                    fFrame,
                    "No Test Cases found.\nCheck that the configured \'TestCollector\' is supported on this platform.");

            return;
        }

        //selector.show();
        selector.setVisible(true);

        String className = selector.getSelectedItem();

        if (className != null) {
            setSuite(className);
        }
    }

    /**
     * set / change the TestCollector
     *
     * @param collector
     */
    public void setTestCollector(TestCollector collector) {
        this.fCollector = collector;
    }

    /**
     * Subclass so that we can access the junit swing runner frame (its protected)
     * Usage:
     * GenomeTestRunner runner = new GenomeTestRunner()
     * runner.getUserInterfaceComponents()
     * (dont call main)
     */
    public Component[] getUserInterfaceComponents() {

        fFrame = createUI(null);

        fFrame.pack();

        Container cont = fFrame.getContentPane();

        //log.debug("JUnit container: " + cont + " ncomps: " + cont.getComponentCount());
        return cont.getComponents();
    }
}    // End GenomeTestRunner

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
