/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtest.api;


import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;


/**
 * Configuration object for Test configuration properties.
 * <p/>
 * Implementations of TestConf must define the constatnst here in the properties
 * file.
 * The properties file must have the LHS enty in the file.
 * For example:
 * <p/>
 * RES_FILE=some/location/of/a/test/res/file.res
 * <p/>
 * allfoo_URL must be benetah the base url locatiuon and hence specified relative to it.
 * so RES_URL=some_res_name and not RES_URL=some/location/of/a/test/res/file.res
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 * @veto Improve how const from properties file are looked up - its silly to have to
 * define a const like RES_URL = "RES_URL"; and then lookup a key of
 * RES_URL in the properties file.
 * Vetoed - dont see a better way of doing this.
 * @done Consider incorporating the bloch enum ideom to the constants.
 * In a sense done by moving out the configuration from Test to this interface.
 * But still working with Strings (rather than definoing a Conf accessor object)
 * as they (strings) are a natural fit with properties.
 */
public interface TestConf {

    public static final String RES_URL = "RES_URL";
    public static final String SMALL_RES_URL = "SMALL_RES_URL";
    public static final String DATAFRAME_URL = "DATAFRAME_URL";

    public static final String STRINGDATAFRAME_URL = "STRINGDATAFRAME_URL";


    public static final String GCT_URL = "GCT_URL";
    public static final String ANN_URL = "ANN_URL";
    public static final String CLS_URL = "CLS_URL";
    public static final String CLS2_URL = "CLS2_URL";
    public static final String CLS_NUMERIC_URL = "CLS_NUMERIC_URL";
    public static final String GRP_URL = "GRP_URL";
    public static final String GMX_URL = "GMX_URL";
    public static final String GMT_URL = "GMT_URL";
    public static final String FFN_URL = "FFN_URL";
    public static final String NNR_FOLDER_URL = "NNR_FOLDER_URL";


    public static final String JDBC_USER = "test.jdbc.user";
    public static final String JDBC_PASSWD = "test.jdbc.passwd";
    public static final String JDBC_URL = "test.jdbc.url";

    // Stanford data files
    public static final String STANFORD_DATA_URL = "STANFORD_DATA_URL";
    public static final String CDT_URL = "CDT_URL";
    public static final String GTR_URL = "GTR_URL";
    public static final String ATR_URL = "ATR_URL";
    public static final String SMALL_CDT_URL = "SMALL_CDT_URL";
    public static final String SMALL_GTR_URL = "SMALL_GTR_URL";
    public static final String SMALL_ATR_URL = "SMALL_ATR_URL";


    public static final String MAP_URL = "MAP_URL";

    // Feature Map files
    public static final String FEATURE_MAP_URL = "FEATURE_MAP_URL";

    // Test images
    public static final String JPG_IMAGE_URL = "JPG_IMAGE_URL";

    public URL getConfUrl();

    public String getProperty(String key);

    /**
     * Returns URL for the root of the test data dir
     */
    public URL getDataRoot() throws MalformedURLException;

    /**
     * Returns a URL object for the specified test resource name
     * Example: getResource(TestConf.RES_URL);
     */
    public URL getUrl(String key) throws MalformedURLException;

    public File getFile(String key) throws IOException;
}    // End TestConf

/*--- Formatted in Sun Java Convention Style on Sat, Oct 26, '02 ---*/
