/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtest.api;


/**
 * Object that defines various test types.
 * Different test types might, for instance, have different testing harnesses.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 * @see Enumerated type Bloch ideom #21
 */
public class TestType {

    // @maint If a new TestType is added the getTestType factory method below
    // needs to be updated
    public static final TestType UNIT = new TestType("unit");
    public static final TestType GUI = new TestType("vdbgui");
    public static final TestType RANDOM = new TestType("random");
    public static final TestType PERF = new TestType("perf");
    private final String fType;

    // private to disable external object construction
    private TestType(String type) {
        fType = type;
    }

    public String toString() {
        return fType;
    }

    public boolean equals(Object type) {

        if (type.hashCode() == hashCode()) {
            return true;
        } else {
            return false;
        }
    }

    public int hashCode() {
        return this.toString().hashCode();
    }

    /**
     * Factory method for TesTypes given a String.
     *
     * @maint Must manually be kept in synch with the TestType constructors
     * defined above.
     */

    // need this annoying if else construct as sometimes the test type needs to be constructed
    // given a string from something such as a system property. See TestAll for more.
    public static TestType getTestType(String type) {

        if (type == null) {
            throw new NullPointerException("method param type cannot be null: " + type);
        }

        if (type.equals(TestType.UNIT.toString())) {
            return TestType.UNIT;
        } else if (type.equals(TestType.GUI.toString())) {
            return TestType.GUI;
        } else if (type.equals(TestType.PERF.toString())) {
            return TestType.PERF;
        } else if (type.equals(TestType.RANDOM.toString())) {
            return TestType.RANDOM;
        } else {
            throw new IllegalArgumentException("Unknown test type string: " + type);
        }
    }
}    // End TestType

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
