/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;

import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.genome.parsers.DataFormat;
import edu.mit.broad.genome.parsers.ParserFactory;

import java.awt.event.ActionListener;
import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Jul 28, 2003
 *         Time: 9:56:13 AM
 */
public abstract class AbstractPobChooserParam extends AbstractObjectChooserParam {

    private boolean fMultipleAllowed = true;
    private ChooserHelper.MyPobActionListener fAl;
    private Class[] fPobClasses;

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     * @param pobClassArr
     * @param pobClasses
     * @param hints
     * @param defs
     * @param reqd
     * @param multipleAlllowed
     */
    AbstractPobChooserParam(String name, String desc, Class pobClassArr, Class[] pobClasses,
                            Object[] hints, Object[] defs,
                            boolean reqd, boolean multipleAlllowed) {

        super(name, pobClassArr, desc, hints, defs, reqd);
        this.fMultipleAllowed = multipleAlllowed;
        this.fPobClasses = pobClasses;
    }

    AbstractPobChooserParam(String name, String nameEnglish, String desc, Class pobClassArr, Class[] pobClasses,
                            Object[] hints, Object[] defs,
                            boolean reqd, boolean multipleAlllowed) {

        super(name, nameEnglish, pobClassArr, desc, hints, defs, reqd);
        this.fMultipleAllowed = multipleAlllowed;
        this.fPobClasses = pobClasses;
    }

    AbstractPobChooserParam(String name, String desc, Class pobClassArr, Class pobClass,
                            Object[] hints, Object[] defs,
                            boolean reqd, boolean multipleAlllowed) {

        this(name, desc, pobClassArr, new Class[]{pobClass}, hints, defs, reqd, multipleAlllowed);
    }

    AbstractPobChooserParam(String name, String nameEnglish, String desc, Class pobClassArr, Class pobClass,
                            Object[] hints, Object[] defs,
                            boolean reqd, boolean multipleAlllowed) {

        this(name, nameEnglish, desc, pobClassArr, new Class[]{pobClass}, hints, defs, reqd, multipleAlllowed);
    }

    /**
     * @param value
     */
    public void setValue(Object value) {

        /*
        if (value == null) {
            super.setValue(value);
       // } else if (value instanceof PersistentObject && DataFormat.isCompatibleRepresentationClass(value, fPobClasses)) {  // @note NOT checking getType as thats an array
        } else if (value instanceof PersistentObject) {  // @note NOT checking getType as thats an array
            super.setValue(new PersistentObject[]{(PersistentObject) value});
        //} else if (value instanceof PersistentObject[] && DataFormat.isCompatibleRepresentationClass((PersistentObject[]) value, fPobClasses)) { // @note NOT checking getType as thats an array
          //  super.setValue(value);
        } else if (value instanceof String) {
            super.setValue((String) value);
        } else if (value instanceof String[]) {
            this.setValue((String[]) value);
        } else {
            throw new IllegalArgumentException(
                    "Invalid type, only " + getTypes() + ", its array[], String[] and comma-delim parsable String accepted. Specified: "
                    + value + " class: " + value.getClass());
        }
        */

        if (value instanceof String[]) {
            this.setValue((String[]) value);
        } else if (value instanceof String) {
            super.setValue((String) value);
        } else {
            super.setValue(value);
        }
    }

    /**
     * @param paths
     */
    public void setValue(String[] paths) {
        try {

            /*
            System.out.println("$$$$$$$$$ " + fPobClasses + " " + fPobClasses.length);
            for (int i=0; i < fPobClasses.length; i++) {
                System.out.println(">> " + fPobClasses[i]);
            }
            */


            PersistentObject[] pobs = new PersistentObject[paths.length];
            for (int i = 0; i < paths.length; i++) {
                File file = new File(paths[i]);
                PersistentObject pob = ParserFactory.read(file, true);
                if (DataFormat.isCompatibleRepresentationClass(pob, fPobClasses)) { // @note not checking getType as that returns an array
                    pobs[i] = pob;
                } else {
                    throw new RuntimeException("Unexpected parsed value: " + pob + " class: " + pob.getClass() + " from file: " + file + " expecting: " + getTypes());
                }
            }

            super.setValue(pobs);
        } catch (Throwable t) {
            throw new RuntimeException(t);
        }
    }

    // overr base cl method to do for both pobs and strings
    protected static String format(Object[] vals) {
        if (vals == null) {
            return "";
        }

        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < vals.length; i++) {
            if (vals[i] == null) {
                continue;
            }

            if (vals[i] instanceof PersistentObject) {
                String p = ParserFactory.getCache().getSourcePath(vals[i]);
                buf.append(p);
            } else {
                buf.append(vals[i].toString().trim());
            }

            if (i != vals.length - 1) {
                buf.append(',');
            }
        }

        return buf.toString();
    }

    // have to make the strs into paths
    public String getValueStringRepresentation(boolean full) {

        Object val = getValue();

        if (val == null) {
            return null;
        }

        Object[] objs;

        if (val instanceof Object[]) {
            objs = (Object[]) val;
        } else {
            objs = new Object[]{val};
        }

        return format(objs);
    }

    // override base impl
    protected ActionListener getActionListener() {
        if (fAl == null) {
            this.fAl = new ChooserHelper.MyPobActionListener(fPobClasses, fMultipleAllowed);
            fAl.setChooser(fChooser);
        }

        return fAl;
    }

    public boolean isFileBased() {
        return true;
    }

    /*
    // @note overrriding here as we dont want to use a SET (want order to be preserved)
    protected String[] _parse(String s) {
        log.debug("BEING USUSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS");
        return ParseUtils.string2strings(s, ",", true);
    }
    */

} // End AbstractPobChooserParam
