/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.map.Chip2ChipMapper;
import edu.mit.broad.vdb.map.MappingDbType;
import edu.mit.broad.xbench.RendererFactory2;
import edu.mit.broad.xbench.core.api.Application;
import xtools.api.ui.Chip2ChipMapperChooserUI;
import xtools.api.ui.NamedModel;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.CompoundBorder;
import javax.swing.border.TitledBorder;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.*;


/**
 * choose 1 or more string
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 * @note diff from StringsInputParam -> the chpices are presented in a JListWindow and not
 * in a text area
 */
class Chip2ChipMapperAbstractChooserParam extends AbstractObjectChooserParam {

    private boolean fMultipleAllowed = true;

    private MyPobActionListener fAl;

    protected Chip2ChipMapperChooserUI fChooserChip2Chip;

    private IChipParam fSourceChipParam;

    private IChipParam fTargetChipParam;

    private Chip[] fCurrSourceChips;
    private Chip[] fCurrTargetChips;


    /**
     * Class constructor
     *
     * @param name
     * @param nameEnglish
     * @param desc
     * @param reqd
     * @param multipleAlllowed
     */
    Chip2ChipMapperAbstractChooserParam(final String name,
                                        final String nameEnglish,
                                        final String desc,
                                        final boolean reqd,
                                        final boolean multipleAlllowed,
                                        final IChipParam sourceChipParam,
                                        final IChipParam targetChipParam) {
        super(name, nameEnglish,
                MappingDbType.class,
                desc, new MappingDbType[]{}, new MappingDbType[]{}, reqd);


        this.fSourceChipParam = sourceChipParam;
        this.fTargetChipParam = targetChipParam;
        this.fMultipleAllowed = multipleAlllowed;
    }

    public void setValue(MappingDbType mdb) {
        super.setValue(new MappingDbType[]{mdb});
    }

    public void setValue(MappingDbType[] mdbs) {
        super.setValue(mdbs);
    }

    //-----------------------------------------------------------------------//
    //------------------------- CORE METHODS --------------------------------//
    //-----------------------------------------------------------------------//

    protected Object[] _getObjects() throws Exception {

        Object val = getValue();

        if (val != null) {
            log.debug("value = " + val + " class: " + val.getClass());
            return ParseUtils.string2strings((String) val, ",", true);
        } else {
            return new Object[]{};
        }

    }

    protected Chip2ChipMapper[] _getMappers() throws Exception {
        Object[] objs = _getObjects();

        log.debug("num of selections: " + objs.length);

        if (isReqd() && objs.length == 0) {
            throw new IllegalArgumentException("Must specify parameter: " + getName());
        }

        List list = new ArrayList();

        // @note this needs a hack for gene - gene symbol mapping from the cmd line
        // (as there is no real file - its generated internally)
        // @see VdbRuntimeResources for more
        for (int i = 0; i < objs.length; i++) {
            log.debug("obj: " + objs[i].getClass());
            File f = new File(objs[i].toString());

            if (1 == 1) {
                throw new NotImplementedException(); // @todo restore for original chip2chip
            }
            // /list.add(VdbRuntimeResources.getChip2ChipMapper(f)); // @bnote how we access the cache
        }

        return (Chip2ChipMapper[]) list.toArray(new Chip2ChipMapper[list.size()]);
    }

    private static String[] _parse(String s) {

        if (s == null) {
            throw new IllegalArgumentException("Parameter s cannot be null");
        }

        Set vals = ParseUtils.string2stringsSet(s, ",", false); // only commas!!

        System.out.println("to parse>" + s + "< got: " + vals);

        Set use = new HashSet();
        for (Iterator it = vals.iterator(); it.hasNext();) {
            String key = it.next().toString();
            if (key.length() > 0) {
                use.add(key);
            }
        }

        return (String[]) use.toArray(new String[use.size()]);
    }

    public void setValue(String commadelimvalues) {
        setValue(_parse(commadelimvalues));
    }

    // override base class method to do for both pobs and strings
    protected static String format(Object[] vals) {
        if (vals == null) {
            return "";
        }

        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < vals.length; i++) {

            if (vals[i] == null) {
                continue;
            }

            klog.debug(vals[i].getClass());

            buf.append(vals[i].toString().trim());

            if (i != vals.length - 1) {
                buf.append(',');
            }
        }

        return buf.toString();
    }

    // override base impl
    protected ActionListener getActionListener() {
        if (fAl == null) {
            this.fAl = new MyPobActionListener(fMultipleAllowed);
            fAl.setChooser(fChooserChip2Chip);
        }

        return fAl;
    }

    public boolean isFileBased() {
        return true;
    }

    // redo from the abstract super class here as we dont want to swap out the model
    // (model is the datasets etc)
    private class MyPobActionListener implements ActionListener {

        private boolean fMultipleAllowed;
        private Chip2ChipMapperChooserUI fChooserChip2Chip;

        public MyPobActionListener(boolean multipleAllowed) {
            this.fMultipleAllowed = multipleAllowed;
        }

        // cant have this in the class constructor as the action list needs to
        // be instantiated before the chooser object is made
        public void setChooser(Chip2ChipMapperChooserUI chooserChip2Chip) {
            this.fChooserChip2Chip = chooserChip2Chip;
        }

        DefaultListCellRenderer rend = new RendererFactory2.CommonLookListRenderer();

        public void actionPerformed(ActionEvent e) {

            if (fChooserChip2Chip == null) {
                //klog.debug("Chooser not yet inited: " + fChooser);
                return;
            }

            int selmode;
            if (!fMultipleAllowed) {
                selmode = ListSelectionModel.SINGLE_SELECTION;
            } else {
                selmode = ListSelectionModel.MULTIPLE_INTERVAL_SELECTION;
            }

            // chip params must be specified
            if (!fSourceChipParam.isSpecified()) {
                Application.getWindowManager().showMessage("Source chip must be specified before this parameter can be accessed");
                return;
            }

            if (!fTargetChipParam.isSpecified()) {
                Application.getWindowManager().showMessage("Target chip must be specified before this parameter can be accessed");
                return;
            }

            Chip[] sourceChips = null;
            Chip[] targetChips = null;

            VdbRuntimeResources.MappedChipWrapper[] types = new VdbRuntimeResources.MappedChipWrapper[]{};
            try {
                sourceChips = fSourceChipParam.getChips();
                targetChips = fTargetChipParam.getChips();
                //types = VdbRuntimeResources.getMappingDbTypesWrapper(sourceChips, targetChips);
                if (1 == 1) {
                    throw new NotImplementedException(); // @todo restore for original chip2chip
                }
            } catch (Throwable t) {
                t.printStackTrace();
            }

            DefaultComboBoxModel dcm = new DefaultComboBoxModel(types);
            JList jl = fChooserChip2Chip.getJListWindow().getJList();

            if (jl != null) {
                if (types.length == 0) {
                    Border border = new CompoundBorder(new TitledBorder("No mappings available for these chips"),
                            new TitledBorder("Check website for downloads"));
                    jl.setBorder(border);
                } else {
                    jl.setBorder(BorderFactory.createEmptyBorder());
                }
            }

            NamedModel model = new NamedModel("Mapping Databse", dcm);
            Object[] sels = fChooserChip2Chip.getJListWindow().showDirectlyWithModels(model, selmode, rend);

            if ((sels == null) || (sels.length == 0)) { // <-- @note

                if (fCurrSourceChips != null) {
                    if (sourceChips == fCurrSourceChips && targetChips == fCurrTargetChips) {

                    } else {
                        fChooserChip2Chip.setText(""); // erase it
                    }
                }


            } else {

                String[] paths = new String[sels.length];
                for (int i = 0; i < sels.length; i++) {
                    paths[i] = ((VdbRuntimeResources.MappedChipWrapper) sels[i]).getPath();
                }

                String str = ChooserHelper.formatFiles_or_Paths(paths);
                fChooserChip2Chip.setText(str);
            }

            fCurrSourceChips = sourceChips;
            fCurrTargetChips = targetChips;
        }
    }

    // have to make the strs into paths
    public String getValueStringRepresentation(boolean full) {

        Object val = getValue();

        if (val == null) {
            return null;
        }

        // log.debug("value: " + val.getClass() + " " + val);

        if (val instanceof String) {
            return (String) val;
        } else if (val instanceof Object[]) {
            Object[] objs = (Object[]) val;
            return format(objs);
        } else {
            return format(new Object[]{val});
        }

    }

    public GFieldPlusChooser getSelectionComponent() {

        if (fChooserChip2Chip == null) {
            fChooserChip2Chip = new Chip2ChipMapperChooserUI(false);
            fChooserChip2Chip.setCustomActionListener(getActionListener());
            String text = this.getValueStringRepresentation(false);
            if (text == null) {
                text = format((Object[]) getDefault());
            }

            if (isFileBased()) { // as otherwise lots of exceptions thrown if user edits a bad file
                // @todo but problem is that no way to cancel and "null out" a choice once made
                //fChooser.getTextField().setEditable(false);
            }

            fChooserChip2Chip.setText(text);
            ParamHelper.addDocumentListener(fChooserChip2Chip.getTextField(), this);
        }

        return fChooserChip2Chip;
    }


}    // End class
