/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.swing.fields.GOptionsFieldPlusChooser;
import edu.mit.broad.xbench.RendererFactory2;
import edu.mit.broad.xbench.core.ObjectBindery;
import org.apache.log4j.Logger;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Jul 26, 2003
 *         Time: 9:48:57 PM
 */
public class ChooserHelper {

    private static Logger klog = XLogger.getLogger(ChooserHelper.class);


    // overr base cl method to do for both pobs and strings
    public static String formatPob(Object[] vals) {
        if (vals == null) {
            return "";
        }

        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < vals.length; i++) {
            if (vals[i] == null) {
                continue;
            }

            if (vals[i] instanceof PersistentObject) {
                String p = ParserFactory.getCache().getSourcePath(vals[i]);
                buf.append(p);
            } else {
                buf.append(vals[i].toString().trim());
            }

            if (i != vals.length - 1) {
                buf.append(',');
            }
        }

        return buf.toString();
    }

    public static String formatFiles_or_Paths(Object[] vals) {
        if (vals == null) {
            return "";
        }

        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < vals.length; i++) {
            if (vals[i] == null) {
                continue;
            }

            if (vals[i] instanceof File) {
                String p = ((File) vals[i]).getPath();
                buf.append(p);
            } else {
                buf.append(vals[i].toString().trim());
            }

            if (i != vals.length - 1) {
                buf.append(',');
            }
        }

        return buf.toString();
    }


    // redo from the abstract super class here as we dont want to swap out the model
    // (model is the datasets etc)
    protected static class MyPobActionListener implements ActionListener {

        private boolean fMultipleAllowed;
        private boolean doneRend;
        private GOptionsFieldPlusChooser fChooser;
        private Class[] fPobClasses;

        public MyPobActionListener(Class[] pobClasses, boolean multipleAllowed) {
            this.fPobClasses = pobClasses;
            this.fMultipleAllowed = multipleAllowed;
        }

        // cant have this in the class constructor as the action list needs to
        // be instantiated before the chooser object is made
        public void setChooser(GOptionsFieldPlusChooser chooser) {
            this.fChooser = chooser;
        }

        public void actionPerformed(ActionEvent e) {

            if (fChooser == null) {
                klog.debug("Chooser not yet inited: " + fChooser);
                return;
            }

            if (!doneRend) {
                JList jl = fChooser.getJListWindow().getJList();
                jl.setCellRenderer(new RendererFactory2.CommonLookListRenderer());
                doneRend = true;
            }

            if (fMultipleAllowed == false) {
                fChooser.getJListWindow().setListSelectionMode(ListSelectionModel.SINGLE_SELECTION);
            }

            // cant just set the jlists model, as it changes upon call to show
            ComboBoxModel model = ObjectBindery.getModel(fPobClasses);
            Object[] sels = fChooser.getJListWindow().showDirectlyWithModel(model);

            if ((sels == null) || (sels.length == 0)) { // <-- @note
            } else {
                String[] paths = new String[sels.length];
                for (int i = 0; i < sels.length; i++) {
                    paths[i] = ParserFactory.getCache().getSourcePath(sels[i]);
                }

                String str = formatPob(sels);
                fChooser.setText(str);
            }
        }
    }

} // End ChooserHelper
