/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;


import edu.mit.broad.genome.math.ColorScheme;
import edu.mit.broad.genome.math.ColorSchemes;
import edu.mit.broad.genome.swing.fields.GComboBoxField;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Object to capture commandline params</p>
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ColorSchemeParam extends AbstractParam implements ActionListener {

    private GComboBoxField cbOptions;

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     */
    public ColorSchemeParam(boolean reqd) {
        this(new ColorSchemes.BroadCancer(), reqd);
    }

    /**
     * Class constructor
     *
     * @param def
     * @param reqd
     */
    public ColorSchemeParam(ColorScheme def, boolean reqd) {
        super(COLOR_SCHEME, ColorScheme.class, COLOR_SCHEME_DESC,
                def, ColorSchemes.createAll(), reqd);
    }

    protected ColorSchemeParam(String name, String desc, ColorScheme def, boolean reqd) {
        super(name, ColorScheme.class, desc, def, ColorSchemes.createAll(), reqd);
    }

    public ColorSchemeParam(ColorScheme[] schemes, boolean reqd) {
        super(METRIC, ColorScheme.class, METRIC_DESC,
                schemes[0], schemes, reqd);
    }

    public boolean isFileBased() {
        return false;
    }

    public void setValue(Object value) {

        if (value == null) {
            super.setValue(null);
        } else {
            super.setValue(ColorSchemes.lookup(value));
        }
    }

    public void setValue(ColorScheme cs) {
        super.setValue(cs);
    }

    public ColorScheme getColorScheme() {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        return (ColorScheme) val;
    }


    public GFieldPlusChooser getSelectionComponent() {

        if (cbOptions == null) {
            cbOptions = ParamHelper.createActionListenerBoundHintsComboBox(false, this, this);
            ParamHelper.safeSelectValueDefaultByString(cbOptions.getComboBox(), this);
        }

        return cbOptions;

    }

    public void actionPerformed(ActionEvent evt) {
        this.setValue((ColorScheme) ((JComboBox) cbOptions.getComponent()).getSelectedItem());
    }

}    // End class ColorSchemeParam

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
