/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;


import edu.mit.broad.genome.alg.CorrMetric;
import edu.mit.broad.genome.alg.DatasetMetrics;
import edu.mit.broad.genome.alg.Metrics;
import edu.mit.broad.genome.swing.fields.GComboBoxField;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;
import java.util.Map;


/**
 * <p> Object to capture commandline params</p>
 * <p> </p>
 * <p> </p>
 * <p> </p>
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class CorrMetricParam extends AbstractParam implements ActionListener {

    private GComboBoxField cbOptions;

    public CorrMetricParam(boolean reqd) {
        this(CMETRIC, reqd);
    }

    public CorrMetricParam(String name, boolean reqd) {
        this(name, CMETRIC_DESC, reqd);
    }

    public CorrMetricParam(String name, String desc, boolean reqd) {
        super(name, CorrMetric.class, desc,
                new Metrics.Pearson(), Metrics.createAllCorrMetrics(), reqd);
    }

    public void setValue(Object value) {

        if (value == null) {
            super.setValue(null);
        } else {
            super.setValue(Metrics.lookupCorrMetric(value));
        }
    }

    public void setValue(CorrMetric cm) {
        super.setValue(cm);
    }

    public boolean isFileBased() {
        return false;
    }

    public CorrMetric getCorrMetric() {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        System.out.println("### " + val.getClass());
        return (CorrMetric) val;
    }

    public GFieldPlusChooser getSelectionComponent() {

        if (cbOptions == null) {
            cbOptions = ParamHelper.createActionListenerBoundHintsComboBox(false, this, this);
            ParamHelper.safeSelectValueDefaultByString(cbOptions.getComboBox(), this);
        }

        return cbOptions;

    }

    public void actionPerformed(ActionEvent evt) {
        this.setValue((CorrMetric) ((JComboBox) cbOptions.getComponent()).getSelectedItem());
    }

    public Map getDefaultMetricParams() {
        return Collections.unmodifiableMap(DatasetMetrics.getDefaultMetricParams());
    }

    // result hack to allow setting mean / median
    public Map getMetricParams(BooleanParam medianParam) {
        return ParamFactory.getMetricParams(medianParam);
    }

}    // End class CorrMetricParam

/*--- Formatted in Sun Java Convention Style on Thu, Nov 7, '02 ---*/
