/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package xtools.api.param;

import edu.mit.broad.genome.alg.fdr.CutoffByPValue;


/**
 * CutoffByPValue param
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class CutoffParam extends StringInputParam {

    /**
     * Class constructor
     *
     * @param name
     * @param type
     * @param desc
     */
    public CutoffParam(boolean reqd) {

        super("cutoff",
                "CutoffByPValue lower_pvalue,upper_pvalue,at_npermutations", new String[]{},
                reqd);
    }

    public String getLongDesc() {
        if (fLongDesc == null) {
            fLongDesc = new StringBuffer("<html><body>").append(getDesc()).append("<br>");
            fLongDesc.append("Stop permutations for objects with p-values that when<br>");
            fLongDesc.append("<pre>current_pvalue GREATER THAN lower_pvalue AND current_pvalue LESS THAN upper_pvalue</pre><br>");
            fLongDesc.append("The cutoff is applied at_npermutations<br>");
            fLongDesc.append("This is useful to specify when running against a large number of gene sets<br>");
            fLongDesc.append("Format Example:<pre>0.1,0.9,100</pre>");
            fLongDesc.append("</body></html");
        }
        return fLongDesc.toString();
    }

    public boolean isFileBased() {
        return false;
    }

    public CutoffByPValue getCutoff() {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        return CutoffByPValue.createCutoff(val.toString());
    }

    public void runSanityCheck(int nperms) {
        CutoffByPValue c = getCutoff();
        if (c.getNPerm() > nperms) {
            throw new IllegalArgumentException("CutoffByPValue nperms: " + c.getNPerm() + " cannot be > than nperms: " + nperms);
        }
    }

    /*
    public boolean isSpecified() {


        if(fValue == null) {
            return false;
        } else {
            return true;    // irrespective of the value
        }
    }
    */

}    // End class CutoffParam

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
