/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;


import edu.mit.broad.genome.objects.strucs.DatasetMode;
import edu.mit.broad.genome.swing.fields.GComboBoxField;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Object to capture commandline params</p>
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class DatasetModeParam extends AbstractParam implements ActionListener {

    private GComboBoxField cbOptions;

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     */
    public DatasetModeParam(boolean reqd) {
        this(DatasetMode.NAT, reqd);
    }

    public DatasetModeParam(DatasetMode def, boolean reqd) {
        super(DATASET_MODE, DATASET_MODE_ENGLISH, DatasetMode.class, DATASET_MODE_DESC,
                def, DatasetMode.createAllModes(), reqd);
    }

    protected DatasetModeParam(String name, String desc, DatasetMode def, boolean reqd) {
        super(name, DatasetMode.class, desc,
                def, DatasetMode.createAllModes(), reqd);
    }

    public boolean isFileBased() {
        return false;
    }

    public void setValue(Object value) {

        if (value == null) {
            super.setValue(null);
        } else {
            super.setValue(DatasetMode.lookupMode(value));
        }
    }

    public void setValue(DatasetMode mode) {
        super.setValue(mode);
    }

    public DatasetMode getMode() {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        return (DatasetMode) val;
    }


    public GFieldPlusChooser getSelectionComponent() {

        if (cbOptions == null) {
            cbOptions = ParamHelper.createActionListenerBoundHintsComboBox(false, this, this);
            ParamHelper.safeSelectValueDefaultByString(cbOptions.getComboBox(), this);
        }

        return cbOptions;

    }

    public void actionPerformed(ActionEvent evt) {
        this.setValue((DatasetMode) ((JComboBox) cbOptions.getComponent()).getSelectedItem());
    }


}    // End class DatasetModeParam

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
