/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;

import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.GeneSet;


/**
 * Chooser one or more Datasets
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class DatasetMultiChooserParam extends DatasetChooserAbstractParam {

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     */
    public DatasetMultiChooserParam(String name, String desc, boolean reqd) {
        super(name, desc, reqd, true);
    }

    public DatasetMultiChooserParam(String name, boolean reqd) {
        this(name, RES_DESC_MULTI, reqd);
    }

    public DatasetMultiChooserParam(boolean reqd) {
        this(RES, reqd);
    }

    public Dataset[] getDatasets() throws Exception {
        return super._getDatasets();
    }

    public Dataset[] getDatasets(final GeneSetOptParam spaceParam) throws Exception {
        Dataset[] dss = getDatasets();
        if (spaceParam != null && spaceParam.isSpecified()) {
            GeneSet space = spaceParam.getGeneSet(dss[0]);
            DatasetGenerators gen = new DatasetGenerators();
            for (int i = 0; i < dss.length; i++) {
                dss[i] = gen.extractRows(dss[i], space);
            }
        }

        return dss;
    }

    public Dataset[] getDatasets(final GeneSetReqdParam spaceParam) throws Exception {
        Dataset[] dss = getDatasets();

        GeneSet space = spaceParam.getGeneSet(dss[0]);
        DatasetGenerators gen = new DatasetGenerators();
        for (int i = 0; i < dss.length; i++) {
            dss[i] = gen.extractRows(dss[i], space);
        }


        return dss;
    }

}    // End class DatasetMultiChooserParam
