/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;

import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import xtools.api.AbstractTool;


/**
 * choose 1 or more string
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 * @note diff from StringsInputParam -> the chpices are presented in a JListWindow and not
 * in a text area
 */
public class DatasetOptParam extends DatasetChooserAbstractParam {

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     */
    public DatasetOptParam(String name, String desc) {
        super(name, desc, false, false);
    }

    public DatasetOptParam(String name) {
        this(name, RES_DESC);
    }

    public DatasetOptParam() {
        this(RES);
    }

    public void setValue(final Dataset ds) {
        super.setValue(ds);
    }

    public Dataset getDataset() throws Exception {
        Dataset[] dss = _getDatasets();
        if (dss.length != 1) {
            throw new IllegalStateException("Program error - too many dss expected 1, got: " + dss.length);
        }

        return dss[0];
    }

    public Dataset getDataset(final SampleAnnot sampleAnnot, final GeneSetOptParam gsetParam) throws Exception {
        Dataset ds = getDataset(sampleAnnot);
        if (gsetParam != null && gsetParam.isSpecified()) {
            ds = new DatasetGenerators().extractRows(ds, gsetParam.getGeneSet());
        }

        log.debug(ds.getQuickInfo());
        return ds;
    }

    public Dataset getDataset(final SampleAnnot annot) throws Exception {
        Dataset ds = getDataset();
        AbstractTool.setChip(ds, annot.getChip(ds));
        AbstractTool.setSampleAnnot(ds, annot);

        // problem is that if some samples are removed from the annot because of bad qual ds will never know
        // @too is this needed?
        log.debug("fullDs: " + ds.getQuickInfo() + " " + annot.getNumSamples());
        if (ds.getNumCol() != annot.getNumSamples()) {
            throw new IllegalStateException();
        }

        return ds;
    }

}    // End class DatasetSingleChooserParam


