/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;


import edu.mit.broad.genome.swing.fields.GComboBoxField;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;


/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class FloatParam extends AbstractParam implements ActionListener {

    private GComboBoxField cbOptions;

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     * @param hint
     * @param reqd
     */
    public FloatParam(String name, String desc, float def_andonly_hint, boolean reqd) {
        super(name, Float.class, desc, new Float(def_andonly_hint), reqd);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     * @param hint
     * @param reqd
     */
    public FloatParam(String name, String desc, float[] hintsanddef, boolean reqd) {
        super(name, Float.class, desc, toFloats(hintsanddef), reqd);
    }

    public FloatParam(String name, String desc, float def, float[] hintsanddef, boolean reqd) {
        super(name, Float.class, desc, new Float(def), toFloats(hintsanddef), reqd);
    }

    public boolean isFileBased() {
        return false;
    }

    public void setValue(Object value) {
        //log.debug("#####Setting FloatParam to: " + value);

        if (value == null) {
            super.setValue(value);
        } else if (value instanceof Float) {
            super.setValue(value);
        } else if (value instanceof String) {
            if (value.toString().length() == 0) {
                super.setValue(new Float(0));
            } else {
                super.setValue(new Float(((String) value)));
            }
        } else {
            throw new IllegalArgumentException("Invalid type, only Float accepted. Specified: "
                    + value + " class: " + value.getClass());
        }
    }

    public void setValue(float value) {
        this.setValue(new Float(value));
    }

    public float getFValue() {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        return ((Float) val).floatValue();
    }

    public GFieldPlusChooser getSelectionComponent() {

        if (cbOptions == null) {
            cbOptions = ParamHelper.createActionListenerBoundHintsComboBox(true, this, this);
        }

        return cbOptions;
    }

    public void actionPerformed(ActionEvent evt) {
        this.setValue(((JComboBox) cbOptions.getComponent()).getSelectedItem());
    }
}    // End class FloatParam

/*--- Formatted in Sun Java Convention Style on Thu, Nov 7, '02 ---*/
