/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;


import edu.mit.broad.genome.parsers.ParseUtils;


/**
 * choose 1 or more string
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class FloatsChooserParam extends AbstractObjectChooserParam {

    /**
     * Class constructor
     *
     * @param name
     * @param type
     * @param desc
     */
    public FloatsChooserParam(String name, String desc, Float def_andonly_hint, boolean reqd) {
        this(name, desc, new Float[]{def_andonly_hint}, new Float[]{def_andonly_hint}, reqd);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     * @param reqd
     */
    public FloatsChooserParam(String name, String desc, boolean reqd) {
        this(name, desc, new Float[]{}, new Float[]{}, reqd);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     * @param hint
     * @param reqd
     */
    public FloatsChooserParam(String name, String desc, Float[] defs, Float[] hints, boolean reqd) {
        super(name, Float.class, desc, defs, hints, reqd);
    }

    public FloatsChooserParam(String name, String desc, float[] hints, boolean reqd) {
        super(name, Float.class, desc, toFloats(hints), toFloats(hints), reqd);
    }

    public FloatsChooserParam(String name, String desc, float[] defs, float[] hints, boolean reqd) {
        super(name, Float.class, desc, toFloats(defs), toFloats(hints), reqd);
    }

    public FloatsChooserParam(String name, String desc, float def, float[] hints, boolean reqd) {
        super(name, Float.class, desc, toFloats(new float[]{def}), toFloats(hints), reqd);
    }

    /*
    private static Float[] toFloats(float[] fls) {
        Float[] f = new Float[fls.length];
        for (int i=0; i < fls.length; i++) {
            f[i] = new Float((fls[i]));
        }

        return f;
    }
    */

    public boolean isFileBased() {
        return false;
    }

    /**
     * @param value String[] or a String(comma delimited parsed, space insensitive)
     */
    public void setValue(Object value) {
        if (value == null) {
            super.setValue(value);
        } else if (value instanceof Float[]) {
            super.setValue(value);
        } else if (value instanceof String) {
            super.setValue(_parse((String) value));
        } else {
            throw new IllegalArgumentException("Invalid type, only Float[] and comma-delim parsable String accepted. Specified: "
                    + value + " class: " + value.getClass());
        }
    }

    protected Float[] _parse(String s) {
        return ParseUtils.string2Floats(s);
    }

    public void setValue(String commadelimvalues) {
        super.setValue(_parse(commadelimvalues));
    }

    public void setValue(String[] values) {
        if (values == null) {
            return;
        }
        Float[] fls = new Float[values.length];
        for (int i = 0; i < values.length; i++) {
            fls[i] = new Float(values[i].toString());
        }
        super.setValue(fls);
    }

    public Float[] getFloats() {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        return (Float[]) val;
    }

    public float[] getFloats2() {

        Float[] f = getFloats();
        float[] ret = new float[f.length];
        for (int i = 0; i < f.length; i++) {
            ret[i] = f[i].floatValue();
        }

        return ret;
    }

    public float getFloat2(int pos) {
        return getFloat(pos).floatValue();
    }

    public Float getFloat(int pos) {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        return ((Float[]) val)[pos];
    }

    // param full ignored
    public String getValueStringRepresentation(boolean full) {

        Object val = getValue();

        if (val == null) {
            return null;
        }

        //log.debug("Class: " + val.getClass() + " >" + val + "<");

        Float[] ss = (Float[]) val;

        return format(ss);
    }

}    // End class FloatsChooserParam

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
