package xtools.api.param;

/**
 * @author Aravind Subramanian
 * Date: Jun 2, 2006
 * Time: 9:18:05 PM
 */
/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

import edu.mit.broad.genome.alg.GeneSetGenerators;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.parsers.AuxUtils;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;
import edu.mit.broad.vdb.chip.Chip;
import xtools.api.ui.GeneSetFieldPlusChooser;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.*;


/**
 * choose 1 or more string
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 * @note diff from StringsInputParam -> the chpices are presented in a JListWindow and not
 * in a text area
 */
class GeneSetChooserAbstractParam extends AbstractPobChooserParam {

    private boolean fMultipleAllowed = true;

    private MyPobActionListener fAl;

    protected GeneSetFieldPlusChooser fPlusChooser;

    /**
     * Class constructor
     *
     * @param name
     * @param nameEnglish
     * @param desc
     * @param reqd
     * @param multipleAlllowed
     */
    GeneSetChooserAbstractParam(String name, String nameEnglish, String desc, boolean reqd, boolean multipleAlllowed) {
        super(name, nameEnglish, desc, GeneSet[].class, GeneSet.class, new GeneSet[]{}, new GeneSet[]{}, reqd, multipleAlllowed);

        this.fMultipleAllowed = multipleAlllowed;

    }

    public void setValue(GeneSet gset) {
        super.setValue(new GeneSet[]{gset});
    }

    public void setValue(GeneSet[] gsets) {
        super.setValue(gsets);
    }

    //----------------------------------------------------------------------------
    //------------------------------------------------------------------------
    //----------------------------------------------------------------------------
    //------------------------------------------------------------------------
    //----------------------------------------------------------------------------
    //------------------------------------------------------------------------

    public GeneSet[] getGeneSets(final IntegerParam minParam,
                                 final IntegerParam maxParam,
                                 final DatasetOptParam dsParam) throws Exception {

        GeneSet[] gsets = _getGeneSets();

        boolean do_cloning = true;

        if (dsParam.isSpecified()) {
            if (minParam.isSpecified()) {
                gsets = GeneSetGenerators.removeGeneSetsSmallerThan(gsets, minParam.getIValue(), dsParam.getDataset(), true);
                do_cloning = false;
            }

            if (maxParam.isSpecified()) {
                gsets = GeneSetGenerators.removeGeneSetsLargerThan(gsets, maxParam.getIValue(), dsParam.getDataset(), do_cloning);
            }
        } else {
            if (minParam.isSpecified()) {
                gsets = GeneSetGenerators.removeGeneSetsSmallerThan(gsets, minParam.getIValue());
            }

            if (maxParam.isSpecified()) {
                gsets = GeneSetGenerators.removeGeneSetsLargerThan(gsets, maxParam.getIValue());
            }
        }

        return gsets;
    }


    public GeneSet[] getGeneSets(IntegerParam minParam, IntegerParam maxParam, boolean removeNativeGmNames) throws Exception {

        GeneSet[] gsets = _getGeneSets();

        if (minParam.isSpecified()) {
            gsets = GeneSetGenerators.removeGeneSetsSmallerThan(gsets, minParam.getIValue());
        }

        if (maxParam.isSpecified()) {
            gsets = GeneSetGenerators.removeGeneSetsLargerThan(gsets, maxParam.getIValue());
        }

        return gsets;
    }

    //-----------------------------------------------------------------------//
    //------------------------- CORE METHODS --------------------------------//
    //-----------------------------------------------------------------------//

    protected Object[] _getObjects() throws Exception {

        Object val = getValue();

        Object[] objs;

        if (val instanceof String) {
            String[] paths = _parse(val.toString());
            objs = new Object[paths.length];
            for (int p = 0; p < paths.length; p++) {
                if (AuxUtils.isAux(paths[p])) {
                    objs[p] = ParserFactory.readGeneSet(new File(paths[p]), true);
                } else {
                    objs[p] = ParserFactory.read(new File(paths[p]), true);
                }
            }
        } else if (val instanceof Object[]) {
            objs = (Object[]) val;
        } else {
            objs = new Object[]{val};
        }

        return objs;
    }

    protected GeneSet[] _getGeneSets() throws Exception {

        Object[] objs = _getObjects();

        //log.debug("num of selections: " + objs.length);

        if (isReqd() && objs.length == 0) {
            throw new IllegalArgumentException("Must specify GeneSetMatrix parameter: " + getName());
        }

        List gsets = new ArrayList();

        for (int i = 0; i < objs.length; i++) {
            if (objs[i] instanceof GeneSetMatrix) {
                gsets.addAll(((GeneSetMatrix) objs[i]).getGeneSetsL());
            } else if (objs[i] instanceof GeneSet) {
                gsets.add(objs[i]);
            } else if (objs[i] instanceof RankedList) {
                gsets.add(new FSet(((RankedList) objs[i]).getName(), ((RankedList) objs[i]).getRankedNames(), false));
            } else if (objs[i] instanceof Dataset) {
                gsets.add(((Dataset) objs[i]).getRowNamesGeneSet());
            } else if (objs[i] instanceof Chip) {
                gsets.add(((Chip) objs[i]).getProbeNamesAsGeneSet());
            } else if (objs[i] instanceof GenesOfInterest) {
                gsets.add(((GenesOfInterest) objs[i]).getAsOneGeneSet());
            } else {
                throw new IllegalArgumentException("Unknown object: " + objs[i]);
            }
        }

        return (GeneSet[]) gsets.toArray(new GeneSet[gsets.size()]);
    }

    /**
     * @param paths
     * @note special overriding pf base class
     * can be a combo of genesets and genematrices
     */
    public void setValue(String[] paths) {
        try {

            GeneSet[] objs = new GeneSet[paths.length];
            for (int i = 0; i < paths.length; i++) {
                objs[i] = ParserFactory.readGeneSet(new File(paths[i]), true);
            }

            super.setValue(objs);
        } catch (Throwable t) {
            throw new RuntimeException(t);
        }

    }

    private static String[] _parse(String s) {

        if (s == null) {
            throw new IllegalArgumentException("Parameter s cannot be null");
        }

        Set vals = ParseUtils.string2stringsSet(s, ",", false); // only commas!!

        System.out.println("to parse>" + s + "< got: " + vals);

        Set use = new HashSet();
        for (Iterator it = vals.iterator(); it.hasNext();) {
            String key = it.next().toString();
            if (key.length() > 0) {
                use.add(key);
            }
        }

        return (String[]) use.toArray(new String[use.size()]);
    }

    public void setValue(String commadelimvalues) {
        setValue(_parse(commadelimvalues));
    }

    // override base class method to do for both pobs and strings
    protected static String format(Object[] vals) {
        if (vals == null) {
            return "";
        }

        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < vals.length; i++) {

            if (vals[i] == null) {
                // do nothing
            } else {
                //log.debug(vals[i].getClass());
                if (vals[i] instanceof PersistentObject) {
                    String p = ParserFactory.getCache().getSourcePath(vals[i]);
                    buf.append(p);
                } else {
                    buf.append(vals[i].toString().trim());
                }

                if (i != vals.length - 1) {
                    buf.append(',');
                }
            }
        }

        return buf.toString();
    }

    // override base impl

    protected ActionListener getActionListener() {

        if (fAl == null) {
            this.fAl = new MyPobActionListener(fMultipleAllowed);
            fAl.setChooser(fPlusChooser);
        }

        return fAl;

    }


    public boolean isFileBased() {
        return true;
    }


    // redo from the abstract super class here as we dont want to swap out the model
    // (model is the datasets etc)
    // REIMPLEMENT HERE as we want to (possibly) use both genesets and genesetmatrixes
    private static class MyPobActionListener implements ActionListener {

        private boolean fMultipleAllowed;

        private GeneSetFieldPlusChooser fFieldPlusChooser;

        public MyPobActionListener(boolean multipleAllowed) {
            this.fMultipleAllowed = multipleAllowed;
        }

        // cant have this in the class constructor as the action list needs to
        // be instantiated before the chooser object is made
        public void setChooser(GeneSetFieldPlusChooser plusChooser) {
            this.fFieldPlusChooser = plusChooser;
        }

        public void actionPerformed(ActionEvent e) {

            if (fFieldPlusChooser == null) {
                //klog.debug("Chooser not yet inited: " + fChooser);
                return;
            }

            int selmode;
            if (fMultipleAllowed == false) {
                selmode = ListSelectionModel.SINGLE_SELECTION;
            } else {
                selmode = ListSelectionModel.MULTIPLE_INTERVAL_SELECTION;
            }

            //Object[] sels = fFieldPlusChooser.showMe(model, selmode, rend);
            Object[] sels = fFieldPlusChooser.showMeDirectly();

            if ((sels == null) || (sels.length == 0)) { // <-- @note
            } else {
                String[] paths = new String[sels.length];
                for (int i = 0; i < sels.length; i++) {
                    paths[i] = ParserFactory.getCache().getSourcePath(sels[i]);
                }

                String str = ChooserHelper.formatPob(sels);
                fFieldPlusChooser.setText(str);
            }
        }
    }


    // have to make the strs into paths
    public String getValueStringRepresentation(boolean full) {

        Object val = getValue();

        if (val == null) {
            return null;
        }

        // log.debug("value: " + val.getClass() + " " + val);

        if (val instanceof String) {
            return (String) val;
        } else if (val instanceof Object[]) {
            Object[] objs = (Object[]) val;
            return format(objs);
        } else {
            return format(new Object[]{val});
        }

    }

    public GFieldPlusChooser getSelectionComponent() {

        if (fPlusChooser == null) {
            int mode;
            if (fMultipleAllowed) {
                mode = ListSelectionModel.MULTIPLE_INTERVAL_SELECTION;
            } else {
                mode = ListSelectionModel.SINGLE_SELECTION;
            }
            fPlusChooser = new GeneSetFieldPlusChooser(false, mode);
            fPlusChooser.setCustomActionListener(getActionListener());
            String text = this.getValueStringRepresentation(false);
            if (text == null) {
                text = format((Object[]) getDefault());
            }

            if (isFileBased()) { // as otherwise lots of exceptions thrown if user edits a bad file
                // @todo but problem is that no way to cancel and "null out" a choice once made
                //fChooser.getTextField().setEditable(false);
            }

            fPlusChooser.setText(text);
            ParamHelper.addDocumentListener(fPlusChooser.getTextField(), this);
        }

        return fPlusChooser;
    }

}    // End class GeneSetChooserParam
