/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;


import edu.mit.broad.genome.objects.DefaultGeneSetMatrix;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.GeneSetMatrix;
import edu.mit.broad.genome.objects.PersistentObject;

import java.awt.event.ActionListener;

/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class GeneSetMatrixReqdParam extends PobParam implements ActionListener {

    /**
     * Class constructor
     */
    public GeneSetMatrixReqdParam() {
        this(GMX, GMX_ENGLISH, GMX_DESC);
    }

    public GeneSetMatrixReqdParam(String name) {
        this(name, GMX_ENGLISH, GMX_DESC);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param nameEmglish
     * @param desc
     */
    public GeneSetMatrixReqdParam(String name, String nameEmglish, String desc) {
        super(name, nameEmglish, GeneSetMatrix.class, desc, new GeneSetMatrix[]{}, true);
    }

    public void setValue(GeneSetMatrix gmx) {
        super.setValue(gmx);
    }

    public GeneSetMatrix getGeneSetMatrix() throws Exception {
        PersistentObject pob = getPob();
        if (pob instanceof GeneSetMatrix) {
            return (GeneSetMatrix) pob;
        } else if (pob instanceof GeneSet) {
            return new DefaultGeneSetMatrix(pob.getName(), new GeneSet[]{(GeneSet) pob});
        } else {
            throw new IllegalArgumentException("Unknown pob type: " + pob);
        }
    }

    public GeneSet[] getGeneSets() throws Exception {
        return getGeneSetMatrix().getGeneSets();
    }

    public GeneSetMatrix getGeneSetMatrix(StringMultiInputParam gsetNames) throws Exception {
        GeneSetMatrix gm = getGeneSetMatrix();

        if (gsetNames.isSpecified()) {
            GeneSet[] gsets = gm.getGeneSets(gsetNames.getStringsS());
            return new DefaultGeneSetMatrix(gm.getName() + "_" + gsets.length, gsets);
        } else {
            return gm;
        }
    }

    // duplicates NOT checked for
    public GeneSetMatrix getGeneSetMatrixOrdered(StringMultiInputParam gsetNames) throws Exception {
        GeneSetMatrix gm = getGeneSetMatrix();

        if (gsetNames.isSpecified()) {
            String[] names = gsetNames.getStrings();
            GeneSet[] gsets = new GeneSet[names.length];
            for (int i = 0; i < names.length; i++) {
                gsets[i] = gm.getGeneSet(names[i]);
            }
            return new DefaultGeneSetMatrix(gm.getName() + "_" + gsets.length, gsets);
        } else {
            return gm;
        }
    }

}    // End class GeneSetMatrixParam

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
