/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;


import edu.mit.broad.genome.alg.gsea.*;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.RankedList;
import edu.mit.broad.genome.swing.fields.GComboBoxField;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Object to capture commandline params</p>
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class GeneSetScoringTableReqdParam extends AbstractParam implements ActionListener {

    private GComboBoxField cbOptions;

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     */
    public GeneSetScoringTableReqdParam() {
        this(new GeneSetScoringTables.Weighted());
    }

    public GeneSetScoringTableReqdParam(GeneSetScoringTable def) {
        super(SCORING_SCHEME, SCORING_SCHEME_ENGLISH, GeneSetScoringTable.class, SCORING_SCHEME_DESC,
                def, GeneSetScoringTables.createAllScoringTables(), true);
    }

    public GeneSetScoringTableReqdParam(GeneSetScoringTable def, GeneSetScoringTable[] options) {
        super(SCORING_SCHEME, SCORING_SCHEME_ENGLISH, GeneSetScoringTable.class, SCORING_SCHEME_DESC,
                def, options, true);
    }

    public GeneSetCohortGenerator createGeneSetCohortGenerator(boolean silent) {
        GeneSetScoringTable table = getGeneSetScoringTable();
        return new DefaultGeneSetCohort.Generator(table, silent);
    }

    public GeneSetCohort createGeneSetCohort(final RankedList rl,
                                             final GeneSet[] gsets,
                                             final boolean silent) {
        GeneSetScoringTable table = getGeneSetScoringTable();
        return new DefaultGeneSetCohort(table, rl, gsets, silent);
    }

    public GeneSetScoringTable getGeneSetScoringTable() {
        return (GeneSetScoringTable) getValue();
    }

    public boolean isFileBased() {
        return false;
    }

    public void setValue(Object value) {

        if (value == null) {
            super.setValue(null);
        } else {
            super.setValue(GeneSetScoringTables.lookupGeneSetScoringTable(value));
        }
    }

    public void setValue(GeneSetScoringTable table) {
        super.setValue(table);
    }


    public GFieldPlusChooser getSelectionComponent() {

        if (cbOptions == null) {
            cbOptions = ParamHelper.createActionListenerBoundHintsComboBox(false, this, this);
            ParamHelper.safeSelectValueDefaultByString(cbOptions.getComboBox(), this);
        }

        return cbOptions;

    }

    public void actionPerformed(ActionEvent evt) {
        this.setValue((GeneSetScoringTable) ((JComboBox) cbOptions.getComponent()).getSelectedItem());
    }


}    // End class GeneSetCohortReqdParam

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
