/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;


import edu.mit.broad.genome.alg.Imputer;
import edu.mit.broad.genome.alg.Imputers;
import edu.mit.broad.genome.swing.fields.GComboBoxField;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Object to capture commandline params</p>
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ImputerParam extends AbstractParam implements ActionListener {

    private GComboBoxField cbOptions;

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     */
    public ImputerParam(boolean reqd) {
        this(new Imputers.Average(), reqd);
    }

    public ImputerParam(Imputer def, boolean reqd) {
        super(IMPUTER, Imputer.class, IMPUTER_DESC,
                def, Imputers.createAll(), reqd);
    }

    protected ImputerParam(String name, String desc, Imputer def, boolean reqd) {
        super(name, Imputer.class, desc,
                def, Imputers.createAll(), reqd);
    }

    public ImputerParam(Imputer[] imputers, boolean reqd) {
        super(IMPUTER, Imputer.class, IMPUTER_DESC,
                imputers[0], imputers, reqd);
    }

    public boolean isFileBased() {
        return false;
    }

    public void setValue(Object value) {

        if (value == null) {
            super.setValue(null);
        } else {
            super.setValue(Imputers.lookup(value));
        }
    }

    public void setValue(Imputer imputer) {
        super.setValue(imputer);
    }

    public Imputer getImputer() {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        return (Imputer) val;
    }


    public GFieldPlusChooser getSelectionComponent() {

        if (cbOptions == null) {
            cbOptions = ParamHelper.createActionListenerBoundHintsComboBox(false, this, this);
            ParamHelper.safeSelectValueDefaultByString(cbOptions.getComboBox(), this);
        }

        return cbOptions;

    }

    public void actionPerformed(ActionEvent evt) {
        this.setValue(((JComboBox) cbOptions.getComponent()).getSelectedItem());
    }


}    // End class ImputerParam

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
