/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;


import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;
import edu.mit.broad.genome.swing.fields.GStringsInputFieldPlusChooser;


/**
 * entry 1 or more string
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 * @imp enters in a text are not a list chooser
 */
public class IntegerMultiInputParam extends AbstractParam {

    private GStringsInputFieldPlusChooser fChooser;
    private static final String PARSE_DELIMS = new String(",\t\n");// dont parse on spaces

    /**
     * Class constructor
     *
     * @param name
     * @param type
     * @param desc
     */
    public IntegerMultiInputParam(String name, String nameEnglish, String desc, Integer def_andonly_hint, boolean reqd) {
        this(name, nameEnglish, desc, new Integer[]{def_andonly_hint}, reqd);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     * @param reqd
     */
    public IntegerMultiInputParam(String name, String nameEnglish, String desc, boolean reqd) {
        this(name, nameEnglish, desc, new Integer[]{}, reqd);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     * @param hint
     * @param reqd
     */
    public IntegerMultiInputParam(String name, String nameEnglish, String desc, Integer[] def_and_hints, boolean reqd) {
        super(name, nameEnglish, Integer.class, desc, def_and_hints, reqd);
    }

    /**
     * @param value String[] or a String(comma delimited parsed, space insensitive)
     */
    public void setValue(Object value) {

        // TraceUtils.showTrace();

        if (value == null) {
            super.setValue(value);
        } else if (value instanceof Integer[]) {
            super.setValue(value);
        } else if (value instanceof String[]) {
            super.setValue(_parse((String[]) value));
        } else if (value instanceof String) {
            super.setValue(_parse((String) value));
        } else {
            throw new IllegalArgumentException("Invalid type, only int[], Integer[], String[] and comma-delim parsable String accepted. Specified: "
                    + value + " class: " + value.getClass());
        }
    }

    private int[] _parse(String s) {
        //log.debug("setting: " + s);
        return ParseUtils.string2ints(s);
    }

    private int[] _parse(String[] ss) {
        int[] ints = new int[ss.length];
        for (int i = 0; i < ss.length; i++) {
            ints[i] = Integer.parseInt(ss[i]);
        }

        return ints;
    }

    public void setValue(String commadelimvalues) {
        super.setValue(_parse(commadelimvalues));
    }


    public int[] getInts() {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        if (val instanceof int[]) {
            return (int[]) val;
        }

        Object[] ival = new Object[]{val};

        int[] ints = new int[ival.length];
        for (int i = 0; i < ints.length; i++) {
            ints[i] = Integer.parseInt(ival[i].toString());
        }

        return ints;
    }

    // param full ignored
    public String getValueStringRepresentation(boolean full) {

        Object val = getValue();

        if (val == null) {
            return null;
        }

        if (val instanceof int[]) {
            return format((int[]) val);
        } else {
            Object[] ss = (Object[]) val;
            return format(ss);
        }
    }

    protected static String format(Object[] vals) {

        if (vals == null) {
            return "";
        }

        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < vals.length; i++) {
            if (vals[i] == null) {
                // do nothing
            } else {

                buf.append(vals[i].toString().trim());
                if (i != vals.length - 1) {
                    buf.append(',');
                }
            }
        }

        return buf.toString();
    }

    protected static String format(int[] vals) {

        if (vals == null) {
            return "";
        }

        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < vals.length; i++) {

            buf.append(vals[i]);

            if (i != vals.length - 1) {
                buf.append(',');
            }
        }

        return buf.toString();
    }

    public int size() {
        return ((String[]) getValue()).length;
    }

    public GFieldPlusChooser getSelectionComponent() {
        if (fChooser == null) {
            String text = this.getValueStringRepresentation(false);
            if (text == null) {
                text = format((String[]) getDefault());
            }

            if (text == null) {
                text = "";
            }

            fChooser = new GStringsInputFieldPlusChooser(text, PARSE_DELIMS);
            //fChooser.setListSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
            ParamHelper.addDocumentListener(fChooser.getTextField(), this);
        }

        return fChooser;

    }

    public boolean isFileBased() {
        return false;
    }

}    // End class IntegerMultiInputParam

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
