/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;


import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.parsers.ParseUtils;
import gnu.trove.TIntArrayList;


/**
 * choose 1 or more string
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class IntegersChooserParam extends AbstractObjectChooserParam {

    /**
     * Class constructor
     *
     * @param name
     * @param type
     * @param desc
     */
    public IntegersChooserParam(String name, String desc, Integer def_andonly_hint, boolean reqd) {
        this(name, desc, new Integer[]{def_andonly_hint}, new Integer[]{def_andonly_hint}, reqd);
    }

    public IntegersChooserParam(String name, String englishName, String desc, Integer def_andonly_hint, boolean reqd) {
        this(name, englishName, desc, new Integer[]{def_andonly_hint}, new Integer[]{def_andonly_hint}, reqd);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     * @param reqd
     */
    public IntegersChooserParam(String name, String desc, boolean reqd) {
        this(name, desc, new Integer[]{}, new Integer[]{}, reqd);
    }

    public IntegersChooserParam(String name, String englishName, String desc, boolean reqd) {
        this(name, englishName, desc, new Integer[]{}, new Integer[]{}, reqd);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     * @param hint
     * @param reqd
     */
    public IntegersChooserParam(String name, String desc, Integer[] defs, Integer[] hints, boolean reqd) {
        super(name, Integer.class, desc, defs, hints, reqd);
    }

    public IntegersChooserParam(String name, String englishName, String desc, Integer[] defs, Integer[] hints, boolean reqd) {
        super(name, englishName, Integer.class, desc, defs, hints, reqd);
    }

    public IntegersChooserParam(String name, String desc, Integer[] defs, boolean reqd) {
        this(name, desc, defs, defs, reqd);
    }

    public IntegersChooserParam(String name, String desc, int[] defs, boolean reqd) {
        this(name, desc, toIntegers(defs), reqd);
    }

    private static Integer[] toIntegers(int[] ints) {
        Integer[] it = new Integer[ints.length];
        for (int i = 0; i < ints.length; i++) {
            it[i] = new Integer((ints[i]));
        }

        return it;
    }

    /**
     * @param value String[] or a String(comma delimited parsed, space insensitive)
     */
    public void setValue(Object value) {
        //log.debug("Setting Integers value: " + value);

        if (value == null) {
            super.setValue(value);
        } else if (value instanceof Integer[]) {
            super.setValue(value);
        } else if (value instanceof String) {
            super.setValue(_parse((String) value));
        } else {
            throw new IllegalArgumentException("Invalid type, only Integer[] and comma-delimited parsable string accepted. Specified: "
                    + value + " class: " + value.getClass());
        }
    }

    public boolean isFileBased() {
        return false;
    }

    private Integer[] _parse(String s) {
        return ParseUtils.string2Ints(s);
    }

    public void setValue(String commadelimvalues) {
        super.setValue(_parse(commadelimvalues));
    }

    public void setValue(String[] values) {
        super.setValue(values);
    }

    public Integer[] getInts() {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        return (Integer[]) val;
    }

    public int[] getInts2() {
        Integer[] ints = getInts();
        int[] values = new int[ints.length];
        for (int i = 0; i < ints.length; i++) {
            values[i] = ints[i].intValue();
        }

        return values;
    }

    public boolean isSpecified(int i) {
        TIntArrayList all = new TIntArrayList(getInts2());
        return all.contains(i);
    }

    public int getMax() {
        return XMath.max(getInts2());
    }

    public int getMin() {
        return XMath.min(getInts2());
    }

    public int getInt2(int pos) {
        return getInt(pos).intValue();
    }

    public Integer getInt(int pos) {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        return ((Integer[]) val)[pos];
    }

    // param full ignored
    public String getValueStringRepresentation(boolean full) {

        Object val = getValue();

        if (val == null) {
            return null;
        }

        Integer[] ss = (Integer[]) val;

        return format(ss);
    }

}    // End class IntegersChooserParam

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
