/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;


import edu.mit.broad.genome.alg.Metric;
import edu.mit.broad.genome.alg.Metrics;
import edu.mit.broad.genome.swing.fields.GComboBoxField;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Map;

/**
 * Object to capture commandline params</p>
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class MetricParam extends AbstractParam implements ActionListener {

    private GComboBoxField cbOptions;

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     */
    public MetricParam(boolean reqd) {
        this(new Metrics.Signal2Noise(), reqd);
    }

    public MetricParam(Metric def, boolean reqd) {
        super(METRIC, METRIC_ENGLISH, Metric.class, METRIC_DESC,
                def, Metrics.createAllMetrics(), reqd);
    }

    protected MetricParam(String name, String desc, Metric def, boolean reqd) {
        super(name, Metric.class, desc,
                def, Metrics.createAllMetrics(), reqd);
    }

    public MetricParam(final Metric[] metrics, final boolean reqd) {
        this(metrics[0], metrics, reqd);
    }

    public MetricParam(final Metric def, final Metric[] metrics, final boolean reqd) {
        super(METRIC, METRIC_ENGLISH, Metric.class, METRIC_DESC, def, metrics, reqd);
    }

    public boolean isFileBased() {
        return false;
    }

    public void setValue(Object value) {

        if (value == null) {
            super.setValue(null);
        } else {
            super.setValue(Metrics.lookupMetric(value));
        }
    }

    public void setValue(Metric metric) {
        super.setValue(metric);
    }

    public Metric getMetric() {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        return (Metric) val;
    }


    public GFieldPlusChooser getSelectionComponent() {

        if (cbOptions == null) {
            cbOptions = ParamHelper.createActionListenerBoundHintsComboBox(false, this, this);
            ParamHelper.safeSelectValueDefaultByString(cbOptions.getComboBox(), this);
        }

        return cbOptions;

    }

    public void actionPerformed(ActionEvent evt) {
        this.setValue((Metric) ((JComboBox) cbOptions.getComponent()).getSelectedItem());
    }

    // result hack to allow setting mean / median
    public Map getMetricParams(BooleanParam medianParam) {
        return ParamFactory.getMetricParams(medianParam);
    }

    public Map getMetricParams(boolean useMedian) {
        return ParamFactory.getMetricParams(useMedian);
    }

}    // End class MetricParam

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
