/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;

import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import xtools.api.AbstractTool;

import java.awt.event.ActionListener;

public class MirnaDatasetChooserParam extends NumParam implements ActionListener {
    /**
     * Class constructor
     */
    public MirnaDatasetChooserParam() {
        this(miRNA, miRNA_ENGLISH, miRNA_DESC);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     */
    public MirnaDatasetChooserParam(final String name, final String desc) {
        super(name, Dataset.class, desc, new Dataset[]{}, true);
    }

    public MirnaDatasetChooserParam(final String name) {
        super(name, Dataset.class, miRNA_DESC, new Dataset[]{}, true);
    }

    public MirnaDatasetChooserParam(final String name, final String nameEnglish, final String desc) {
        super(name, nameEnglish, Dataset.class, desc, new Dataset[]{}, true);
    }

    public void setValue(final Dataset ds) {
        super.setValue(ds);
    }

    // Override this parameter for custom dataset impls
    public Dataset getDataset() throws Exception {
        return (Dataset) getNumDataset();
    }

    public Dataset getDataset(final Chip chip) throws Exception {
        Dataset ds = getDataset();
        AbstractTool.setChip(ds, chip);
        return ds;
    }

    public Dataset getDataset(final SampleAnnot sampleAnnot) throws Exception {
        Dataset ds = getDataset();
        AbstractTool.setChip(ds, sampleAnnot.getChip(ds));
        AbstractTool.setSampleAnnot(ds, sampleAnnot);

        // problem is that if some samples are removed from the annot because of bad qual ds will never know
        // @too is this needed?
        log.debug("fullDs: " + ds.getQuickInfo() + " " + sampleAnnot.getNumSamples());
        if (ds.getNumCol() != sampleAnnot.getNumSamples()) {
            throw new IllegalStateException();
        }

        return ds;
    }

    public Dataset getDataset(final SampleAnnot sampleAnnot, final GeneSetOptParam gsetParam) throws Exception {
        Dataset ds = getDataset(sampleAnnot);
        if (gsetParam != null && gsetParam.isSpecified()) {
            ds = new DatasetGenerators().extractRows(ds, gsetParam.getGeneSet());
        }

        log.debug(ds.getQuickInfo());
        return ds;
    }


    public Dataset getDataset(final ChipOptParam chipParam, final SampleAnnotParam sampleAnnotParam) throws Exception {
        Dataset ds = getDataset();
        AbstractTool.setChip(ds, chipParam);
        AbstractTool.setSampleAnnot(ds, sampleAnnotParam);
        return ds;
    }

    public Dataset getDataset(final ChipChooserMultiParam chipParam, final SampleAnnotParam sampleAnnotParam) throws Exception {
        Dataset ds = getDataset();
        AbstractTool.setChip(ds, chipParam);
        AbstractTool.setSampleAnnot(ds, sampleAnnotParam);
        return ds;
    }

    public Dataset getDataset(final TemplateSingleChooserParam tsp) throws Exception {
        if (tsp != null && tsp.isSpecified()) {
            return new DatasetGenerators().extract(getDataset(), tsp.getTemplate()).getDataset();
        } else {
            return getDataset();
        }
    }

    public Dataset[] getDatasets(final TemplateMultiChooserParam clsParam) throws Exception {

        if (!clsParam.isSpecified()) {
            return new Dataset[]{getDataset()};
        }

        final DatasetGenerators gen = new DatasetGenerators();
        final Dataset fullDs = getDataset();

        final Template[] tss = clsParam.getTemplates(fullDs);
        final Dataset[] dss = new Dataset[tss.length];

        for (int t = 0; t < tss.length; t++) {
            dss[t] = gen.extract(fullDs, tss[t]).getDataset();
        }

        return dss;
    }

    public Dataset getDataset(final GeneSet gset) throws Exception {
        return new DatasetGenerators().extractRows(getDataset(), gset);
    }

    public Dataset getDataset(final GeneSet gset, final SampleAnnotParam annotParam) throws Exception {
        Dataset ds = getDataset(gset);
        AbstractTool.setSampleAnnot(ds, annotParam);
        return ds;
    }

    public Dataset getDataset(final GeneSetOptParam param) throws Exception {
        if (param != null && param.isSpecified()) {
            return new DatasetGenerators().extractRows(getDataset(), param.getGeneSet());
        } else {
            return getDataset();
        }
    }

    public Dataset getDataset(final GeneSetReqdParam param) throws Exception {
        if (param != null && param.isSpecified()) {
            return new DatasetGenerators().extractRows(getDataset(), param.getGeneSet());
        } else {
            return getDataset();
        }
    }

    public Dataset getDataset(final GeneSetMatrixMultiChooserParam gmp) throws Exception {
        if (gmp != null && gmp.isSpecified()) {
            return getDataset(gmp.getGeneSetCombo());
        } else {
            return getDataset();
        }
    }

    public Dataset getDataset(final TemplateSingleChooserParam tsp,
                              final GeneSetMatrixMultiChooserParam gsetParam) throws Exception {
        Dataset ds = getDataset(tsp);
        if (gsetParam != null && gsetParam.isSpecified()) {
            return new DatasetGenerators().extractRows(ds.getName(), ds, gsetParam.getGeneSetCombo());
        } else {
            return ds;
        }
    }

    public Dataset getDataset(final TemplateSingleChooserParam tsp,
                              final GeneSetOptParam gsetParam) throws Exception {
        Dataset ds = getDataset(tsp);
        if (gsetParam != null && gsetParam.isSpecified()) {
            return new DatasetGenerators().extractRows(ds.getName(), ds, gsetParam.getGeneSet());
        } else {
            return ds;
        }
    }

}    // End class MirnaDatasetChooserParam




