/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;

import xtools.api.Tool;

import java.io.File;
import java.io.Serializable;
import java.util.Properties;

/**
 * @author Aravind Subramanian
 *         Date: Feb 22, 2005
 *         Time: 10:58:58 PM
 */
public interface ParamSet extends Serializable {

    public static final String PARAM_FILE = "param_file";

    public GuiParam getGuiParam();

    public ReportDirParam getAnalysisDirParam();

    public ReportLabelParam getReportLabelParam();

    public FoundMissingFile fileCheckingFill(Properties prop);

    public String getAsCygwinCommand(final Tool tool);

    public String getAsUnixCommand(final Tool tool);

    public String getAsLsfJavaCommand(String lsfBase);

    public boolean isRequiredAllSet();

    public int getNumParams();

    public Param[] getParams();

    public Param getParam(int pos);

    public Param[] getParams(Param.Type thisType, boolean excludeCommonOnes);

    public Properties toProperties();

    public String getAsCommand(boolean includecp, boolean fullfilepaths, boolean ignoreguiparam);

    /**
     * fill params but dont fill those that have missing files
     * instead put them into a FoundMissingFile[] missing object and return that
     *
     * @param prop
     */

    public static class FoundMissingFile {

        public File[] foundFiles;
        public File[] missingFiles;
        public String[] foundFilesParamNames;

    }

} // End interface ParamSet
