/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;


import edu.mit.broad.genome.alg.Metric;
import edu.mit.broad.genome.swing.fields.GComboBoxField;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Object to capture choices gene set, template
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class PermuteTypeChooserParam extends AbstractParam implements ActionListener {

    private GComboBoxField cbOptions;

    public static final String BOTH = "both";
    public static final String GENE_SET = "gene_set";
    public static final String PHENOTYPE = "phenotype";

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     */
    private PermuteTypeChooserParam(String[] options, boolean reqd) {
        super(PERMUTE, PERMUTE_ENGLISH, Metric.class, PERMUTE_DESC, options, reqd);
    }


    public static PermuteTypeChooserParam createAll(boolean reqd) {
        return new PermuteTypeChooserParam(new String[]{PHENOTYPE, GENE_SET, BOTH}, reqd);
    }

    public static PermuteTypeChooserParam createTemplateOrGeneSet(boolean reqd) {
        return new PermuteTypeChooserParam(new String[]{PHENOTYPE, GENE_SET}, reqd);
    }

    public static PermuteTypeChooserParam createTemplate(boolean reqd) {
        return new PermuteTypeChooserParam(new String[]{PHENOTYPE}, reqd);
    }

    public static PermuteTypeChooserParam createGeneSet(boolean reqd) {
        return new PermuteTypeChooserParam(new String[]{GENE_SET}, reqd);
    }
    
    /*public static PermuteTypeChooserParam createTemplateOrGeneSet(boolean reqd) {
        return new PermuteTypeChooserParam(new String[]{PHENOTYPE, GENE_SET}, reqd);
    }*/

    public void setValue(Object value) {

        if (value == null) {
            super.setValue(null);
        } else {
            super.setValue(value);
        }
    }

    public void setValue(String val) {
        super.setValue(val);
    }

    public boolean permuteGeneSet() {
        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        if ((val.toString().equals(GENE_SET)) || (val.toString().equals(BOTH))) {
            return true;
        } else {
            return false;
        }
    }

    public boolean permuteTemplate() {
        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        if ((val.toString().equals(PHENOTYPE)) || (val.toString().equals(BOTH))) {
            return true;
        } else {
            return false;
        }
    }

    public boolean permuteBoth() {
        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        if (val.toString().equals(BOTH)) {
            return true;
        } else {
            return false;
        }
    }

    public GFieldPlusChooser getSelectionComponent() {

        if (cbOptions == null) {
            cbOptions = ParamHelper.createActionListenerBoundHintsComboBox(false, this, this);
            ParamHelper.safeSelectValueDefaultByString(cbOptions.getComboBox(), this);
        }

        return cbOptions;

    }

    public void actionPerformed(ActionEvent evt) {
        this.setValue(((JComboBox) cbOptions.getComponent()).getSelectedItem());
    }

    public boolean isFileBased() {
        return false;
    }

}    // End class PermuteTypeChooserParam

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
