/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;

import edu.mit.broad.genome.alg.TemplateStats;
import edu.mit.broad.genome.swing.fields.GComboBoxField;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Object to capture RandomizerType in commandline params</p>
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class RandomRankedListModeParam extends AbstractParam implements ActionListener {

    private GComboBoxField cbOptions;

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     */
    public RandomRankedListModeParam(boolean reqd) {
        this(TemplateStats.RandomRankedListMode.RANDOM, reqd);
    }

    /**
     * @param def
     * @param reqd
     */
    public RandomRankedListModeParam(TemplateStats.RandomRankedListMode def, boolean reqd) {
        super(RND_LIST_TYPE, TemplateStats.RandomRankedListMode.class, RND_LIST_TYPE_DESC,
                def, TemplateStats.RandomRankedListMode.createAllModes(), reqd);
    }

    public void setValue(Object value) {

        if (value == null) {
            super.setValue(null);
        } else {
            super.setValue(TemplateStats.RandomRankedListMode.lookupRandomizerType(value));
        }
    }

    public void setValue(TemplateStats.RandomRankedListMode rt) {
        super.setValue(rt);
    }

    public TemplateStats.RandomRankedListMode getMode() {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        return (TemplateStats.RandomRankedListMode) val;
    }


    public GFieldPlusChooser getSelectionComponent() {

        if (cbOptions == null) {
            cbOptions = ParamHelper.createActionListenerBoundHintsComboBox(false, this, this);
            ParamHelper.safeSelectValueDefaultByString(cbOptions.getComboBox(), this);
        }

        return cbOptions;

    }

    public void actionPerformed(ActionEvent evt) {
        this.setValue((TemplateStats.RandomRankedListMode) ((JComboBox) cbOptions.getComponent()).getSelectedItem());

    }

    public boolean isFileBased() {
        return false;
    }

}    // End class RandomizerTypeParam

