/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;

import edu.mit.broad.genome.Printf;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.genome.objects.RankedList;
import edu.mit.broad.genome.objects.RankedListDb;
import edu.mit.broad.genome.parsers.ParserFactory;

import java.io.File;

/**
 * choose 1 or more string
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 * @note diff from StringsInputParam -> the chpices are presented in a JListWindow and not
 * in a text area
 */
class RankedListChooserAbstractParam extends AbstractPobChooserParam {

    private RankedListDb fRdb_opt;

    /**
     * @param name
     * @param desc
     * @param reqd
     * @param multipleAlllowed
     */
    RankedListChooserAbstractParam(String name, String nameEnglish, String desc, boolean reqd, boolean multipleAlllowed) {
        super(name, nameEnglish, desc, RankedList[].class, RankedList.class, new RankedList[]{}, new RankedList[]{}, reqd, multipleAlllowed);
    }

    RankedListChooserAbstractParam(String name, String nameEnglish, String desc, String[] rlsNames, boolean reqd, boolean multipleAlllowed, RankedListDb rdb) {
        super(name, nameEnglish, desc, String[].class, RankedList.class,
                rlsNames, rlsNames, reqd, multipleAlllowed);

        if (rdb == null) {
            throw new IllegalArgumentException("Param rdb cannot be null in this mode");
        }

        this.fRdb_opt = rdb;
    }

    RankedListChooserAbstractParam(String name, String nameEnglish, String desc, boolean reqd, boolean multipleAlllowed, RankedListDb rdb) {
        this(name, nameEnglish, desc, rdb.getRowNamesArray(), reqd, multipleAlllowed, rdb);
    }

    /**
     * @param paths
     * @note special overriding pf base class
     * can be a combo of genesets and genematrices
     */
    public void setValue(String[] paths) {
        try {
            RankedList[] objs = new RankedList[paths.length];
            for (int i = 0; i < paths.length; i++) {
                objs[i] = ParserFactory.readRankedList(new File(paths[i]), true);
            }

            super.setValue(objs);
        } catch (Throwable t) {
            throw new RuntimeException(t);
        }
    }

    public void setValue(RankedList rl) {
        super.setValue(new RankedList[]{rl});
    }

    public void setValue(RankedList[] rls) {
        super.setValue(rls);
    }

    public RankedList[] _getRankedLists() throws Exception {

        Object obj = getValue();
        if (obj instanceof String[]) { // i.e rdb mode
            String[] names = (String[]) obj;
            if (fRdb_opt == null) {
                throw new IllegalStateException("Rdb cannot be null in this mode: " + fRdb_opt + " " + Printf.outs(names));
            }
            return fRdb_opt.getRankedLists(names);
        }

        final PersistentObject[] pobs = (PersistentObject[]) getValue();
        final RankedList[] gsets = new RankedList[pobs.length];

        for (int i = 0; i < pobs.length; i++) {
            log.debug(pobs[i].getClass());
            gsets[i] = (RankedList) pobs[i];
        }

        return gsets;
    }

}    // End class RankedListChooserParam
