/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;


import edu.mit.broad.genome.math.LabelledVectorProcessor;
import edu.mit.broad.genome.math.LabelledVectorProcessors;
import edu.mit.broad.genome.swing.fields.GComboBoxField;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Object to capture commandline params</p>
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class RankedListProcessorParam extends AbstractParam implements ActionListener {

    private GComboBoxField cbOptions;

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     */
    public RankedListProcessorParam(boolean reqd) {
        this(new LabelledVectorProcessors.None(), reqd);
    }

    public RankedListProcessorParam(LabelledVectorProcessor def, boolean reqd) {
        super(RANKED_LIST_PROCESSORS, RANKED_LIST_PROCESSORS_ENGLISH, LabelledVectorProcessor.class, RANKED_LIST_PROCESSORS_DESC,
                def, LabelledVectorProcessors.createAllProcessors(), reqd);
    }

    protected RankedListProcessorParam(String name, String desc, LabelledVectorProcessor def, boolean reqd) {
        super(name, LabelledVectorProcessor.class, desc,
                def, LabelledVectorProcessors.createAllProcessors(), reqd);
    }

    public boolean isFileBased() {
        return false;
    }

    public void setValue(Object value) {

        if (value == null) {
            super.setValue(null);
        } else {
            super.setValue(LabelledVectorProcessors.lookupProcessor(value));
        }
    }

    public void setValue(LabelledVectorProcessor rp) {
        super.setValue(rp);
    }

    public LabelledVectorProcessor getProcessor() {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        return (LabelledVectorProcessor) val;
    }


    public GFieldPlusChooser getSelectionComponent() {

        if (cbOptions == null) {
            cbOptions = ParamHelper.createActionListenerBoundHintsComboBox(false, this, this);
            ParamHelper.safeSelectValueDefaultByString(cbOptions.getComboBox(), this);
        }

        return cbOptions;

    }

    public void actionPerformed(ActionEvent evt) {
        this.setValue((LabelledVectorProcessor) ((JComboBox) cbOptions.getComponent()).getSelectedItem());
    }


}    // End class RankedListProcessorParam

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
