/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;


import edu.mit.broad.genome.swing.fields.GDirFieldPlusChooser;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;
import edu.mit.broad.xbench.core.api.Application;

import java.io.File;

/**
 * To capture the dir in which to place the file(s) produced by an analysis
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ReportDirParam extends DirParam {

    /**
     * Class constructor
     *
     * @param def
     * @param reqd
     */
    public ReportDirParam(final boolean reqd) {
        super(OUT, OUT_ENGLISH, OUT_DESC, Application.getVdbManager().getDefaultOutputDir(), reqd);
    }

    public String getLongDesc() {
        if (fLongDesc == null) {
            fLongDesc = new StringBuffer("<html><body>").append(getDesc()).append("<br>");
            fLongDesc.append("All files produced by the tool will be placed in a folder by this name (no spaces allowed)<br>");
            fLongDesc.append("Any existing data in the folder will NOT be overwritten<br>");
            fLongDesc.append("For example, a single folder can be shared between multiple tool invocations when analyzing data in a project");
            fLongDesc.append("</body></html>");
        }

        return fLongDesc.toString();
    }

    public boolean isFileBased() {
        return true;
    }

    // DONT rename this getName. Duh!!
    public File getAnalysisDir() {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        return (File) getValue();
    }

    /**
     * Override so that we can add a make this my default button
     *
     * @return
     */
    public GFieldPlusChooser getSelectionComponent() {

        if (fChooser == null) {
            fChooser = new GDirFieldPlusChooser();
            if (getValue() != null) {
                // set the value to specified
                fChooser.setValue(getValue());

            } else { // cancelled
                fChooser.setValue(getDefault());
            }

            ParamHelper.addDocumentListener(fChooser.getTextField(), this);

        }

        // @note imp to ask vdb each time for the value as it might have changed since last instantiated
        // @HMMMM
        //fChooser.setValue(Application.getVdbManager().getDefaultOutputDir());

        return fChooser;
    }

}    // End class AnalysisDirParam

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
