/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;


import edu.mit.broad.genome.utils.Strings;

/**
 * To capture a label to affix to a reports
 * Different from analysis name -> typically not shared very much
 * analysis name -> several diff iterations, all kinds of stuff
 * reports name -> specific to one or a few diff types of analysis done in a larger analysis project
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ReportLabelParam extends StringInputParam {

    public static final String DEFAULT_LABEL = "my_analysis";

    /**
     * Class constructor
     *
     * @param reqd
     */
    public ReportLabelParam(boolean reqd) {
        this(DEFAULT_LABEL, reqd);
    }

    /**
     * Class constructor
     *
     * @param def
     * @param reqd
     */
    public ReportLabelParam(String def, boolean reqd) {
        super(RPT, RPT_ENGLISH, RPT_DESC, def, reqd);
    }

    public String getLongDesc() {
        if (fLongDesc == null) {
            fLongDesc = new StringBuffer("<html><body>").append(getDesc()).append("<br>");
            fLongDesc.append("Typically a descriptive prefix to label the results that this Tool produces<br>");
            fLongDesc.append("No spaces allowed<br>");
            fLongDesc.append("</body></html>");
        }

        return fLongDesc.toString();
    }

    //overridden for the space check
    public void setValue(String value) {

        if (Strings.contains(value, " ")) {
            throw new IllegalArgumentException("Analysis name cannot contain spaces. Specified: "
                    + value);
        }

        //log.debug("Setting analysis name to: " + value);
        super.setValue(value);
    }

    // DONT rename this getName!!
    public String getReportLabel() {

        Object val = super.getValue();

        if (val == null) {
            return "";
        } else {
            return val.toString();
        }

        /*
        if(val == null) {
            throw new NullPointerException(
                "Null param value. Always check isSpecified() before calling");
        }
        */


    }

    public boolean isFileBased() {
        return false;
    }

}    // End class ReportLabelParam

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
