/**
 * Institute for Genome Science & Policy
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2009 by the
 * Institute for Genome Science & Policy/Duke University.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Institute for Genome Science & Policy nor Duke University can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;

import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.genome.parsers.DataFormat;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.genome.swing.fields.GComboBoxField;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

abstract class SnpParam extends AbstractParam implements ActionListener {

    private GComboBoxField cbOptions;

    protected SnpParam(String name, Class type, String desc, Object[] hintsanddef,
                       boolean reqd) {

        super(name, type, desc, hintsanddef, reqd);

    }

    /**
     * Class constructor
     */
    protected SnpParam(String name, Class type, String desc, Object def_andonly_hint,
                       boolean reqd) {

        super(name, type, desc, def_andonly_hint, reqd);

    }

    protected SnpParam(String name, String nameEnglish, Class type, String desc, Object def_andonly_hint,
                       boolean reqd) {

        super(name, nameEnglish, type, desc, def_andonly_hint, reqd);

    }

    /**
     * explicitly specified default
     *
     * @param name
     * @param type
     * @param desc
     * @param def
     * @param hints
     * @param reqd
     */
    protected SnpParam(String name, Class type, String desc, Object def, Object[] hints,
                       boolean reqd) {

        super(name, type, desc, def, hints, reqd);
    }


    protected SnpParam(String name, String nameEnglish, Class type, String desc, Object def, Object[] hints,
                       boolean reqd) {

        super(name, nameEnglish, type, desc, def, hints, reqd);
    }


    public void setValue(Object value) {

        if (value == null) {
            super.setValue(value);
        } else if (value instanceof String) {
            this.setValue((String) value);
        } else if (value instanceof File) {
            this.setValue((File) value);
        } else if (DataFormat.isCompatibleRepresentationClass(value, getTypes())) {
            super.setValue(value);
        } else {
            throw new IllegalArgumentException("Invalid type! Only " + getTypes() + " File and String (path) accepted. Specified: "
                    + value + " class: " + value.getClass());
        }

    }

    public void setValue(String filepath) {
        super.setValue(new File(filepath));
    }

    public void setValue(File file) {
        super.setValue(file);
    }

    /**
     * Subclasses can have a custom-more-specifically (i.e type-cast) wrapper if they choose
     *
     * @return
     * @throws Exception
     */
    protected PersistentObject getSnp() throws Exception {
        /*
        if (getValue() != null) {
            log.debug("Value = " + getValue() + " class: " + getValue().getClass());
        }
        */

        Object val = getValue();

        // @note added march 2006
        if (val instanceof Object[] && ((Object[]) val).length == 1) {
            val = ((Object[]) val)[0];
        }


        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        } else if (val instanceof PersistentObject) {
            return (PersistentObject) val;
        } else if (val instanceof File) {
            return ParserFactory.read((File) val, true);
        } else {
            throw new RuntimeException("Unexpected value >" + val + "< of class: " + val.getClass() + " for param: " + getName());
        }

    }

    public boolean isFileBased() {
        return true;
    }

    public GFieldPlusChooser getSelectionComponent() {

        if (cbOptions == null) {
            cbOptions = ParamHelper.createActionListenerBoundPobComboBox(this, getTypes());
            ParamHelper.safeSelectPobValueDefaultOrFirst(cbOptions.getComboBox(), this);
        }

        return cbOptions;
    }

    // One problem with this is that we will respond even when a new pob
    // is parsed into the application
    public void actionPerformed(ActionEvent evt) {
        this.setValue(((JComboBox) cbOptions.getComponent()).getSelectedItem());
    }

    public String getValueStringRepresentation(boolean full) {

        try {
            if (getValue() == null) {
                return null;
            } else {
                File file = ParserFactory.getCache().getSourceFile(getSnp());
                if (full) {
                    return file.getAbsolutePath();
                } else {
                    return file.getName();
                }
            }
        } catch (Exception e) {
            log.warn("Unexpected error", e);
            return null;
        }
    }

} // End SnpParam

