/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;


import edu.mit.broad.genome.parsers.ParseUtils;

import java.util.HashSet;
import java.util.Set;


/**
 * choose 1 or more string
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 * @note diff from StringsInputParam -> the chpices are presented in a JListWindow and not
 * in a text area
 */
abstract class StringChooserAbstractParam extends AbstractObjectChooserParam {

    private boolean fMultipleAlowed = true;

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     * @param hint
     * @param reqd
     */
    StringChooserAbstractParam(String name, String desc, String[] defs, String[] hints, boolean reqd, boolean multipleAlllowed) {
        super(name, String.class, desc, defs, hints, reqd);
        this.fMultipleAlowed = multipleAlllowed;
    }

    StringChooserAbstractParam(String name, String nameEnglish, String desc, String[] defs, String[] hints, boolean reqd, boolean multipleAlllowed) {
        super(name, nameEnglish, String.class, desc, defs, hints, reqd);
        this.fMultipleAlowed = multipleAlllowed;
    }

    /**
     * @param value String[] or a String(comma delimited parsed, space insensitive)
     */
    public void setValue(Object value) {

        if (value == null) {
            super.setValue(value);
        } else if (value instanceof String[]) {
            super.setValue(value);
        } else if (value instanceof String) {
            super.setValue(_parse((String) value));
        } else {
            throw new IllegalArgumentException("Invalid type, only String[] and comma-delim parsable String accepted. Specified: "
                    + value + " class: " + value.getClass());
        }
    }

    protected String[] _parse(String s) {
        return ParseUtils.string2strings(s, ",", false); // only commas!!
    }

    public String[] getStrings() {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        return (String[]) val;
    }

    public Set getStringsSet() {
        String[] ss = getStrings();
        Set set = new HashSet();

        for (int i = 0; i < ss.length; i++) {
            set.add(ss[i]);
        }

        return set;
    }

    // @todo improve
    public int[] getSpecifiedPositions() {

        Object[] hints = getHints();

        Set set = getStringsSet();

        int[] pos = new int[set.size()];
        int cnt = 0;
        for (int i = 0; i < hints.length; i++) {
            if (set.contains(hints[i])) {
                pos[cnt++] = i;
            }
        }

        return pos;
    }

    public boolean isSpecified(String s) {
        return getStringsSet().contains(s);
    }

    public String getString(int pos) {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        return ((String[]) val)[pos];
    }

    // param full ignored
    public String getValueStringRepresentation(boolean full) {

        Object val = getValue();

        if (val == null) {
            return null;
        }

        String[] ss = (String[]) val;

        return format(ss);
    }

    public boolean isFileBased() {
        return false;
    }

}    // End class AbstractStringChooserParam

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
