/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.param;


import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;
import edu.mit.broad.genome.swing.fields.GSafeCharsField;


/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class StringInputParam extends AbstractParam {

    private GSafeCharsField tfName;

    private char[] addSafe;

    // @todo fix this = also the safe chars filed is very sloow
    public void addSafeChars(char[] cs) {
        this.addSafe = cs;
    }

    /**
     * Class constructor
     *
     * @param name
     * @param type
     * @param desc
     */
    public StringInputParam(String name, String desc, String def_andonly_hint, boolean reqd) {
        this(name, desc, new String[]{def_andonly_hint}, reqd);
    }

    public StringInputParam(String name, String nameEnglish, String desc, String def_andonly_hint, boolean reqd) {
        super(name, nameEnglish, String.class, desc, new String[]{def_andonly_hint}, reqd);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     * @param hint
     * @param reqd
     */
    public StringInputParam(String name, String desc, String[] def_and_hints, boolean reqd) {
        super(name, String.class, desc, def_and_hints, reqd);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param desc
     * @param def_and_hints
     * @param reqd
     */
    public StringInputParam(String name, String desc, boolean reqd) {
        super(name, String.class, desc, new String[]{}, reqd);
    }

    public void setValue(Object value) {

        if (value == null) {
            super.setValue(value);
        } else if (value instanceof String) {
            String ss = ((String) value).trim();
            if (ss.length() == 0) {
                super.setValue(null);
            } else {
                this.setValue(ss);
            }
        } else {
            throw new IllegalArgumentException("Invalid type, only String accepted. Specified: "
                    + value + " class: " + value.getClass());
        }
    }

    public void setValue(String value) {
        super.setValue(value);
    }

    public String getString() {

        Object val = super.getValue();

        if (val == null) {
            throw new NullPointerException("Null param value. Always check isSpecified() before calling");
        }

        return (String) val;
    }

    public Vector getVector() {
        return new Vector(ParseUtils.string2floats(getString()));
    }

    public GFieldPlusChooser getSelectionComponent() {

        if (tfName == null) {
            Object t = this.getValue();

            if (t == null) {
                t = this.getDefault();
            }

            if (t != null) {
                tfName = new GSafeCharsField(t.toString());
            } else {
                tfName = new GSafeCharsField();
            }

            if (addSafe != null) {
                tfName.addSafeChars(addSafe);
            }

            ParamHelper.addDocumentListener(tfName, this);
        }

        return tfName;
    }

    public boolean isFileBased() {
        return false;
    }

}    // End class StringParam

/*--- Formatted in Sun Java Convention Style on Thu, Nov 7, '02 ---*/
