/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.ui;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.xbench.core.api.Application;
import edu.mit.broad.xbench.core.api.DialogDescriptor;
import org.apache.log4j.Logger;

import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;


/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class Chip2ChipMapperChooserWindow {

    private static final Logger klog = XLogger.getLogger(Chip2ChipMapperChooserWindow.class);
    private JList jlMappingDb;

    private NamedModel fModel;
    private int fSelectionMode = ListSelectionModel.SINGLE_SELECTION;

    /**
     * Class Constructor.
     */
    public Chip2ChipMapperChooserWindow() {
    }

    public JList getJList() {
        return jlMappingDb;
    }

    /**
     * Model must already filled
     *
     * @return
     */
    public Object[] show() {

        if (jlMappingDb == null) {
            return null;
        }

        // carefull with rebuild / reset the model here -> that ruins the selection policy
        jlMappingDb.setModel(fModel.model);
        jlMappingDb.setSelectionMode(fSelectionMode);

        return _just_show();
    }


    public Object[] showDirectlyWithModels(NamedModel model, int selMode, DefaultListCellRenderer rend) {
        this.fModel = model;

        // carefull with rebuild / reset the model here -> that ruins the selection policy
        if (jlMappingDb == null) {
            jlMappingDb = new JList();
            jlMappingDb.setCellRenderer(rend);
        }

        this.fSelectionMode = selMode;
        jlMappingDb.setModel(model.model);
        jlMappingDb.setSelectionMode(selMode);


        return _just_show();
    }

    private JTabbedPane tab;

    // just the showing part, abs no setting data
    private Object[] _just_show() {

        String text = "Select a chip";

        if (fSelectionMode == ListSelectionModel.MULTIPLE_INTERVAL_SELECTION) {
            text = "Select one or more chips";
        }

        if (tab == null) {
            tab = new JTabbedPane();
            tab.addTab(fModel.name, new JScrollPane(jlMappingDb));
        }

        JPanel dummy = new JPanel(new BorderLayout()); // @note needed else the input widget comes up real small in the dd
        dummy.add(tab, BorderLayout.CENTER);

        DialogDescriptor desc = Application.getWindowManager().createDialogDescriptor(text, dummy);

        desc.enableDoubleClickableJList(jlMappingDb);

        int res = desc.show();

        if (res == DialogDescriptor.CANCEL_OPTION) {
            return null;
        } else {

            java.util.List allValues = new ArrayList();

            Object[] sels = jlMappingDb.getSelectedValues();
            if (sels != null) {
                for (int i = 0; i < sels.length; i++) {
                    if (sels[i] != null) {
                        allValues.add(sels[i]);
                    }
                }
            }

            return allValues.toArray(new Object[allValues.size()]);
        }
    }

    /**
     * @param mode one of the ListSelectionModel constants
     */
    public void setListSelectionMode(int mode) {

        this.fSelectionMode = mode;

        if (jlMappingDb == null) {
            return;
        }

        jlMappingDb.setSelectionMode(fSelectionMode);

    }

}        // End MappingDbTypeChooserWindow

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
