/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.ui;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.swing.GuiHelper;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;
import edu.mit.broad.genome.swing.fields.GOptionsFieldPlusChooser;
import org.apache.log4j.Logger;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class GeneSetFieldPlusChooser extends JPanel implements GFieldPlusChooser {

    private final Logger log = XLogger.getLogger(GOptionsFieldPlusChooser.class);

    private JTextField tfEntry = new JTextField(40);

    private JButton bEntry = new JButton(GuiHelper.ICON_ELLIPSIS);

    private ChooserWindow fWindow;

    // needed as otherwise a default one is added and then again one another one is added
    // if the setCustomActionListener is called
    /**
     * Class constructor
     */
    public GeneSetFieldPlusChooser(boolean addDefaultActionListener, int selMode) {

        GeneSetTabbedChooserPanel chooser = new GeneSetTabbedChooserPanel(selMode);
        this.fWindow = new ChooserWindow(chooser);

        if (addDefaultActionListener) {
            init();
        } else {
            jbInit();
        }
    }

    public void setCustomActionListener(ActionListener customActionListener) {
        bEntry.addActionListener(customActionListener);
    }

    private void init() {

        jbInit();
        bEntry.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Object[] sels = fWindow.show();
                format(sels);
            }
        });
    }

    public Object[] showMeDirectly() {
        Object[] sels = fWindow.show();
        //format(sels);
        return sels;
    }

    private void jbInit() {

        this.setLayout(new BorderLayout());
        tfEntry.setEditable(true);
        this.add(tfEntry, BorderLayout.CENTER);
        this.add(bEntry, BorderLayout.EAST);
    }

    private void format(final Object[] sels) {

        if (sels == null) {
            tfEntry.setText("");
            return;
        }

        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < sels.length; i++) {
            if (sels[i] == null) {
                continue;
            }

            buf.append(sels[i].toString().trim());

            if (i != sels.length - 1) {
                buf.append(',');
            }
        }

        tfEntry.setText(buf.toString());
    }

    public String getText() {
        return tfEntry.getText();
    }

    public void setText(String text) {
        tfEntry.setText(text);
    }

    /**
     * so that the tf can have its events listened to
     *
     * @return
     */
    public JTextField getTextField() {
        return tfEntry;
    }

    public Object getValue() {
        return getText();
    }

    public JComponent getComponent() {
        return this;
    }

    public void setValue(Object obj) {
        if (obj == null) {
            this.setText(null);
        } else {
            this.setText(obj.toString());
        }
    }


}    // End GOptionsFieldPlusChooser

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/

