/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.api.ui;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.objects.FSet;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.parsers.ParserFactory;
import edu.mit.broad.xbench.RendererFactory2;
import edu.mit.broad.xbench.core.ObjectBindery;
import org.apache.log4j.Logger;

import javax.swing.*;
import java.io.File;
import java.util.ArrayList;


/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class GeneSetTabbedChooserPanel extends JTabbedPane implements ChooserPanel {

    private static final Logger klog = XLogger.getLogger(GeneSetTabbedChooserPanel.class);

    private JList jlGeneSet;

    private JTextArea taGenes;

    private NamedModel fModel;

    private int fSelectionMode = ListSelectionModel.SINGLE_SELECTION;

    /**
     * Class Constructor.
     */
    public GeneSetTabbedChooserPanel(int selMode) {

        this.fModel = new NamedModel("GeneSets(grp)", ObjectBindery.getModel(GeneSet.class));

        // carefull with rebuild / reset the model here -> that ruins the selection policy
        this.jlGeneSet = new JList();

        DefaultListCellRenderer rend = new RendererFactory2.CommonLookListRenderer();
        jlGeneSet.setCellRenderer(rend);

        this.fSelectionMode = selMode;
        jlGeneSet.setModel(fModel.model);
        jlGeneSet.setSelectionMode(selMode);

        taGenes = new JTextArea();
        taGenes.setText(""); // @note

        this.addTab(fModel.name, new JScrollPane(jlGeneSet));
        this.addTab("Text Entry", new JScrollPane(taGenes));

    }

    public JComponent getChooser() {
        return this;
    }

    public String getTitle() {

        String text = "Select a gene set";

        if (fSelectionMode == ListSelectionModel.MULTIPLE_INTERVAL_SELECTION) {
            text = "Select one or more gene set(s)";
        }

        return text;
    }

    public Object[] getChoosenObjects() {

        java.util.List allValues = new ArrayList();

        Object[] sels = jlGeneSet.getSelectedValues();
        if (sels != null) {
            for (int i = 0; i < sels.length; i++) {
                if (sels[i] != null) {
                    allValues.add(sels[i]);
                }
            }
        }

        // add text are stuff as a gene set
        String s = taGenes.getText();

        if (s != null) {
            String[] strs = ParseUtils.string2strings(s, "\t\n", false); // we want things synched
            if (strs.length != 0) {
                GeneSet gset = new FSet("from_text_area_", strs);
                try {
                    ParserFactory.save(gset, File.createTempFile(gset.getName(), ".grp"));
                } catch (Throwable t) {
                    klog.error(t);
                }
                allValues.add(gset);
            }
        }
        return allValues.toArray(new Object[allValues.size()]);

    }

    public JList[] getJListsForDoubleClick() {
        return new JList[]{jlGeneSet};
    }

    /**
     * @param mode one of the ListSelectionModel constants
     */
    public void setListSelectionMode(final int mode) {

        this.fSelectionMode = mode;

        if (jlGeneSet == null) {
            return;
        }

        jlGeneSet.setSelectionMode(fSelectionMode);

    }

}        // End GeneSetChooserWindow

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/

/*

      public Object[] showChooser() {
        klog.debug("Doing show  ...");

        if (jlGeneSet == null) {
            return null;
        }

        // carefull with rebuild / reset the model here -> that ruins the selection policy
        jlGeneSet.setModel(fModel.model);
        jlGeneSet.setSelectionMode(fSelectionMode);

        return _just_show();
    }
    public Object[] showDirectlyWithModels(final NamedModel model, final int selMode, final DefaultListCellRenderer rend) {
        this.fModel = model;

        // carefull with rebuild / reset the model here -> that ruins the selection policy
        if (jlGeneSet == null) {
            jlGeneSet = new JList();
            jlGeneSet.setCellRenderer(rend);
        }

        this.fSelectionMode = selMode;
        jlGeneSet.setModel(model.model);
        jlGeneSet.setSelectionMode(selMode);


        return _just_show();
    }


    // just the showing part, abs no setting data
    private Object[] _just_show() {


        if (fSelectionMode == ListSelectionModel.MULTIPLE_INTERVAL_SELECTION) {
            text = "Select One or More Option(s)";
        }

        taGenes.setText(""); // @note

        this.addTab(fModel.name, new JScrollPane(jlGeneSet));
        this.addTab("Text Entry", new JScrollPane(taGenes));

        //JPanel dummy = new JPanel(new BorderLayout()); // @note needed else the input widget comes up real small in the dd
        //dummy.add(tab, BorderLayout.CENTER);

        DialogDescriptor desc = new DialogDescriptor(text, this);
        desc.enableDoubleClickableJList(jlGeneSet);

        desc.setPreferredSize(DD_SIZE);
        int res = desc.show();
        DD_SIZE = desc.getSize();
        desc.dispose();

        if (res == DialogDescriptor.CANCEL_OPTION) {
            return null;
        } else {

            java.util.List allValues = new ArrayList();

            Object[] sels = jlGeneSet.getSelectedValues();
            if (sels != null) {
                for (int i = 0; i < sels.length; i++) {
                    if (sels[i] != null) {
                        allValues.add(sels[i]);
                    }
                }
            }

            // add text are stuff as a gene set
            String s = taGenes.getText();

            if (s != null) {
                String[] strs = ParseUtils.string2strings(s, "\t\n", false); // we want things synched
                if (strs.length != 0) {
                    GeneSet gset = new FSet("from_text_area_", strs);
                    try {
                        ParserFactory.save(gset, File.createTempFile(gset.getName(), ".grp"));
                    } catch (Throwable t) {
                        klog.error(t);
                    }
                    allValues.add(gset);
                }
            }
            return allValues.toArray(new Object[allValues.size()]);
        }
    }
    */