/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.chip2chip;

import xtools.api.AbstractTool;
import xtools.api.ToolCategory;
import xtools.api.param.Chip2ChipMapperOptParam;
import xtools.api.param.ChipChooserMultiParam;
import xtools.api.param.Param;
import xtools.api.param.ParamFactory;

/**
 * Commandline Tool to run enrichment tests over a matrix of gene sets
 * <p/>
 * Runs on 1 res file, 1 template that can be ova/allpairs and 1 gmt file
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public abstract class AbstractMappingTool extends AbstractTool {

    protected final ChipChooserMultiParam fChipsSourceParam = ParamFactory.createChipsSourceParam(true);

    protected final ChipChooserMultiParam fChipsTargetParam = ParamFactory.createChipsTargetParam(true);

    protected final Chip2ChipMapperOptParam fChip2ChipMapperParam = new Chip2ChipMapperOptParam(true, true, fChipsSourceParam, fChipsTargetParam);

    /**
     * Class constructor
     *
     * @param properties
     */
    protected AbstractMappingTool() {
    }

    protected abstract Param[] getAdditionalParams();

    public void declareParams() {

        fParamSet.addParam(fChipsSourceParam);
        fParamSet.addParam(fChipsTargetParam);
        fParamSet.addParam(fChip2ChipMapperParam);
        Param[] add = getAdditionalParams();
        for (int i = 0; i < add.length; i++) {
            fParamSet.addParam(add[i]);
        }

    }

    public ToolCategory getCategory() {
        return ToolCategory.MAPPING_TOOLS;
    }

}    // End AbstractMappingTool

/*--- Formatted in Sun Java Convention Style on Sun, Oct 27, '02 ---*/
