package xtools.chip2chip;

/**
 * @author Aravind Subramanian
 * Date: Jun 27, 2006
 * Time: 5:00:54 PM
 */

/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

import edu.mit.broad.genome.objects.GeneSetMatrix;
import edu.mit.broad.genome.parsers.DataFormat;
import edu.mit.broad.genome.reports.api.ReportIndexState;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.vdb.map.Chip2ChipMapper;
import edu.mit.broad.vdb.map.Chip2ChipMapperSymbol;
import xtools.api.AbstractTool;
import xtools.api.ToolCategory;
import xtools.api.param.*;

import java.util.Properties;

/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class Chip2Chip extends AbstractTool {

    private final GeneSetMatrixMultiChooserParam fGeneSetMatrixParam = new GeneSetMatrixMultiChooserParam(Param.GMX,
            "Gene sets database (symbols only)", "Gene sets database (symbols only)", true);

    private final BooleanParam fShowEtiology = ParamFactory.createShowEtiologyParam(true, false);

    private final GeneSetMatrixFormatParam fGmFormatParam = new GeneSetMatrixFormatParam(DataFormat.GMT_FORMAT, false);

    protected final ChipChooserMultiParam fChipsTargetParam = ParamFactory.createChipsTargetParam(true);

    /**
     * Class constructor
     *
     * @param properties
     */
    public Chip2Chip(Properties properties) {
        super.init(properties);
    }

    public Chip2Chip(String[] args) {
        super.init(args);
    }

    /**
     * For ParamSet interrogation use only -- not executable
     *
     * @param name
     */
    public Chip2Chip() {
        declareParams();
    }

    public void execute() throws Exception {
        // to preserve memory & for custom indexing
        final ReportIndexState state = new ReportIndexState(true, false, true, Chip2Chip_orig_restore_later.createHeader());
        startExec(state);

        final GeneSetMatrix gm = fGeneSetMatrixParam.getGeneSetMatrixCombo(true);
        final Chip2ChipMapper[] mappers = new Chip2ChipMapper[]{new Chip2ChipMapperSymbol(VdbRuntimeResources.getChip_Gene_Symbol(), fChipsTargetParam.getChipCombo())};

        Chip2Chip_orig_restore_later.doMapping(mappers, gm, fShowEtiology.isTrue(), fGmFormatParam, fReport);

        doneExec();
    }

    public String getDesc() {
        return "Map probe sets between chip platforms";
    }

    public void declareParams() {
        fParamSet.addParam(fGeneSetMatrixParam);
        fParamSet.addParam(fGmFormatParam);
        fParamSet.addParam(fShowEtiology);
        fParamSet.addParam(fChipsTargetParam);

    }

    public ToolCategory getCategory() {
        return ToolCategory.MAPPING_TOOLS;
    }

    public static void main(String[] args) {
        Chip2Chip tool = new Chip2Chip(args);
        tool_main(tool);
    }

}    // End Symbol2Probe