/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package xtools.genesettools;


import edu.mit.broad.genome.alg.InfoCrystalAlg;
import edu.mit.broad.genome.objects.BitSetDataset;
import edu.mit.broad.genome.objects.GeneSetMatrix;
import edu.mit.broad.genome.reports.api.ToolReport;
import xtools.api.AbstractTool;
import xtools.api.ReportBlocks;
import xtools.api.ToolCategory;
import xtools.api.param.GeneSetMatrixMultiChooserParam;

import java.io.File;
import java.util.Properties;

/**
 * Commandline Tool to run the InfoCrystal Tests
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class FindOverlaps extends AbstractTool {

    private final GeneSetMatrixMultiChooserParam fGeneSetMatrixParam = new GeneSetMatrixMultiChooserParam(true);

    /**
     * Class constructor
     *
     * @param properties
     */
    public FindOverlaps(Properties properties) {
        super.init(properties);
    }

    /**
     * @param args
     */
    public FindOverlaps(String[] args) {
        super.init(args);
    }

    /**
     * For ParamSet interrogation use only -- not executable
     *
     * @param name
     */
    public FindOverlaps() {
        declareParams();
    }

    /**
     * @throws java.lang.Exception
     */
    public void execute() throws Exception {

        startExec(false); // we make our own index below

        final GeneSetMatrix gm = fGeneSetMatrixParam.getGeneSetMatrixCombo(true);

        doOverlaps(gm, fReport);

        doneExec();

    }

    public static void doOverlaps(final GeneSetMatrix gm, final ToolReport report) {

        final ReportBlocks.SimpleBlockListing list = new ReportBlocks.SimpleBlockListing("Overlaps are listed below", report);

        try {
            final InfoCrystalAlg.XWay xway = InfoCrystalAlg.createXWay(gm);
            final File file_overlap_counts = report.savePage("overlap_counts", "Overlap counts between input gene sets", xway.getAsString());
            final File file_overlap_gmx = report.savePage(xway.getGeneSetMatrix());
            list.add("Overlaps", file_overlap_counts);
            list.add("Gene sets made from overlaps", file_overlap_gmx);
        } catch (Throwable t) {
            t.printStackTrace();
        }

        klog.info("Doing the rest for bsd anyway ...");

        // rows are member names, cols are gset names
        final BitSetDataset bsd = gm.getGeneSetMembershipBitSetDataset();
        final File file_bsd = report.savePageXls(bsd.toDataframe(true, true));

        list.add("Association matrix of features", file_bsd);
        list.close();

    }

    public ToolCategory getCategory() {
        return ToolCategory.GENESET_TOOLS;
    }

    public void declareParams() {
        fParamSet.addParam(fGeneSetMatrixParam);
    }

    public String getDesc() {
        return "Find overlaps between gene sets (max allowed is 5)";
    }

    public static void main(String[] args) {

        FindOverlaps tool = new FindOverlaps(args);
        tool_main(tool);

    }

}    // End InfoCrystal


