/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome;

import edu.mit.broad.genome.utils.SystemUtils;

import java.util.HashMap;
import java.util.Map;

/**
 * IMP IMP IMP: Keep this class thin and light as it is loaded up at startup
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class Conf {

    public static int EXIT_CODE_ERROR = 1;
    public static int EXIT_CODE_OK = 0;

    public static void exitSystem(final boolean thereWasAnError) {

        if (thereWasAnError) {
            System.exit(EXIT_CODE_ERROR);
        } else {
            System.exit(EXIT_CODE_OK);
        }

    }

    // default is to do it
    public static boolean doSplash() {
        String p = SystemUtils.getProperty("splash", false);
        if (p == null || p.length() == 0) {
            return true;
        }
        return Boolean.valueOf(p).booleanValue();
    }

    public static boolean isGseaApp() {
        return SystemUtils.isPropertyTrue("GSEA");
    }

    public static boolean isXServlet() {
        return SystemUtils.isPropertyDefined("XSERVLET");
    }

    /**
     * Did the application launch is debug mode
     * (or is the application currently in debug mode)
     *
     * @return true if in debug mode
     */
    public static boolean isDebugMode() {

        final String debug = SystemUtils.getProperty(Constants.DEBUG_MODE_KEY, false);

        if (debug == null) {
            return false;
        } else {
            return Boolean.valueOf(debug).booleanValue();
        }
    }

    public static boolean isUserHomeDirSpecified() {

        final String home = getUserHomeDirIfSpecified();

        return !(home == null || home.length() == 0);
    }

    public static String getUserHomeDirIfSpecified() {

        String str = SystemUtils.getProperty("home", false);

        if (str == null || str.length() == 0) {
            /*
            // Try harder
            str = SystemUtils.getProperty("foo_home", false); // dont that is different

            if (str != null && str.length() != 0) {
                return str;
            }
            */

            // give up
            return null;
        } else {
            return str;
        }
    }

    public static boolean isUserAravind() {
        return (SystemUtils.getUserName().equalsIgnoreCase("aravind"));
    }

    public static boolean isMakeReportDirOffMode() {
        final String ts = SystemUtils.getProperty(Constants.MAKE_REPORT_DIR_KEY, false);

        //System.out.println("$$$$$$ chekcing isDeterministicReportNameMode: " + ts);

        if (ts == null || ts.length() == 0) {
            return false;
        } else {
            // is timestamp is specified as false then it IS det
            boolean det = !Boolean.valueOf(ts).booleanValue();
            if (det) {
                System.out.println("Working in DETERMINISTIC report mode -- a subdir ts will not be applied");
            }
            return det;
        }
    }

    public static class Counter {
        private int num;

        public int getCurr() {
            return num;
        }

        public void increment() {
            num++;
        }
    }


    private static Map kClassNameCounterMap;

    public static Counter getCounter(String className) {

        if (kClassNameCounterMap == null) {
            kClassNameCounterMap = new HashMap();
        }

        Object curr = kClassNameCounterMap.get(className);
        if (curr == null) {
            curr = new Counter();
            kClassNameCounterMap.put(className, curr);
        }

        return (Counter) curr;
    }

} // End inner class Conf
