/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome;

import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.LabelledVector;
import edu.mit.broad.genome.objects.Template;


/**
 * For use by classes that wich to make an object immutable.
 * (in cases where the 2 class object iimmuatbelobject pattern is hard
 * (or unintuitive) to implement
 * <p/>
 * Immuted -> made immutable
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class MismatchedSizeException extends RuntimeException {

    private static final String ERROR = "Mismatched col/row//array/vector lengths";

    public MismatchedSizeException() {
        super(ERROR);
    }

    public MismatchedSizeException(Matrix a, Matrix b) {
        super("Mismatched matrix row or col sizes: " + " a: " + a.getNumRow() + " & b: " + b.getNumRow() + " a: " + a.getNumCol() + " & b: " + b.getNumCol());
    }

    public MismatchedSizeException(Vector a, Vector b, int bIndex) {
        super(ERROR + " a: " + a.getSize() + " & b: " + b.getSize() + " at b_index " + bIndex);
    }

    public MismatchedSizeException(Vector a, Vector b) {
        super(ERROR + " a: " + a.getSize() + " & b: " + b.getSize());
    }

    public MismatchedSizeException(LabelledVector a, LabelledVector b) {
        super(ERROR + " a: " + a.getSize() + " & b: " + b.getSize());
    }

    public MismatchedSizeException(Object[] a, Object[] b, int bIndex) {
        super(ERROR + " a: " + a.length + " & b: " + b.length + " at b_index " + bIndex);
    }

    public MismatchedSizeException(Object[] a, Object[] b) {
        super(ERROR + " a: " + a.length + " & b: " + b.length);
    }

    public MismatchedSizeException(String arrayEnglishNameA, Object[] a, String arrayEnglishnameB, Object[] b) {
        super(ERROR + " for " + arrayEnglishNameA + ": " + a.length + " & " + arrayEnglishnameB + " :" + b.length);
    }

    public MismatchedSizeException(String arrayEnglishNameA, int lenA, String arrayEnglishnameB, int lenB) {
        super(ERROR + " for " + arrayEnglishNameA + ": " + lenA + " & " + arrayEnglishnameB + " :" + lenB);
    }

    public MismatchedSizeException(Dataset ds, Template t) {
        super(ERROR + " for Dataset " + ds.getName() + ": " + ds.getNumCol() + " & Template " + t.getName() + " :" + t.getNumItems());
    }

}    // End MismatchedSizeException

