package edu.mit.broad.genome.alg;

import edu.mit.broad.genome.Headers;

import java.io.File;
import java.util.Map;

/**
 * @author Aravind Subramanian
 *         Date: Jun 14, 2005
 *         Time: 12:56:09 PM
 */
public class AlgMap {

    public static String getExcludeName(Map params) {
        return getParam(Headers.EXCLUDE_NAME, params);
    }

    public static File getDir(Map params) {
        return new File(getParam(Headers.DIR, params));
    }

    public static int getTopX(Map params) {
        return getParam(Headers.TOP_X, params, 10);
    }

    public static int getBotX(Map params) {
        return getParam(Headers.BOT_X, params, 10);
    }

    public static String[] getColNames(Map params) {
        return AlgMap.getParamArr(Headers.COL_NAMES, params);
    }

    public static String[] getRowNames(Map params) {
        return AlgMap.getParamArr(Headers.ROW_NAMES, params);
    }

    public static boolean isUp(Map params) {
        return AlgMap.getParamReqd(Headers.UP, params);
    }

    public static boolean isDown(Map params) {
        return AlgMap.getParamReqd(Headers.DOWN, params);
    }

    public static boolean isMedian(Map params) {
        return AlgMap.getParam(Headers.USE_MEDIAN, params, true);     // default
    }

    public static boolean isMean(Map params) {
        return !AlgMap.getParam(Headers.USE_MEDIAN, params, true);
    }

    public static boolean isBiased(Map params) {
        return getParam(Headers.USE_BIASED, params, false);
    }

    public static boolean isFixLowVar(Map params) {
        return getParam(Headers.FIX_LOW, params, true); // default
    }

    public static String getMedianOrMeanLabel(Map params) {
        boolean usemedian = getParam(Headers.USE_MEDIAN, params, true);
        if (usemedian) {
            return "median";
        } else {
            return "mean";
        }
    }

    private static boolean getParamReqd(String paramName, Map params) {

        if (params == null) {
            throw new IllegalArgumentException("No defaults params null");
        }


        if (params.get(paramName) != null) {
            return ((Boolean) params.get(paramName)).booleanValue();
        }

        throw new IllegalArgumentException("No defaults unspecified value for: " + paramName);
    }

    private static String getParam(String paramName, Map params) {

        if (params == null) {
            return null;
        }

        if (params.get(paramName) != null) {
            return (String) params.get(paramName);
        }

        return null;
    }

    private static boolean getParam(String paramName, Map params, boolean def) {

        if (params == null) {
            return def;
        }

        boolean ret = def;    // default

        if (params.get(paramName) != null) {
            ret = ((Boolean) params.get(paramName)).booleanValue();
        }

        return ret;
    }

    private static int getParam(String paramname, Map params, int def) {

        if (params == null) {
            return def;
        }

        int ret = def;    // default

        if (params.get(paramname) != null) {
            ret = ((Integer) params.get(paramname)).intValue();
        }

        return ret;
    }

    private static String[] getParamArr(String paramname, Map params) {

        if (params == null) {
            return null;
        }

        if (params.get(paramname) != null) {
            return ((String[]) params.get(paramname));
        }

        return null;
    }

} // End class AlgMap
