/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.Vector;
import org.apache.log4j.Logger;

/**
 * @author Aravind Subramanian
 *         Date: Dec 15, 2003
 *         Time: 7:08:57 PM
 */
public class Imputers {

    /**
     * For logging support
     */
    protected static final transient Logger klog = XLogger.getLogger(Imputers.class);

    // @maint add a metric and this array might need updating
    public static Imputer[] createAll() {
        return new Imputer[]{
                new Average(), new ExpectationMaximization()
        };
    }

    public static Imputer lookup(Object obj) {

        if (obj == null) {
            throw new NullPointerException("Cannot lookup for null object");
        }

        if (obj instanceof Imputer) {
            return (Imputer) obj;
        }

        Imputer[] all = createAll();

        for (int i = 0; i < all.length; i++) {
            if (all[i].getName().equalsIgnoreCase(obj.toString())) {
                return all[i];
            }
        }

        throw new RuntimeException("Cannot lookup Imputer by: " + obj);
    }


    static abstract class AbstractImputer implements Imputer {

        public int hashCode() {
            return getName().hashCode();
        }

        public boolean equals(Object obj) {

            if (obj instanceof Imputer) {
                return getName().equalsIgnoreCase(((Imputer) obj).getName());
            }

            return false;
        }

        public String toString() {
            return getName();
        }
    }

    /**
     * Average.
     */
    public static class Average extends AbstractImputer {

        public static final String NAME = "Average";

        public Vector impute(Vector iv) {
            Vector v = new Vector(iv, false); // imp to not share data
            float tot = 0;
            float num = 0;
            for (int i = 0; i < v.getSize(); i++) {
                if (Float.isNaN(v.getElement(i)) == false) {
                    tot += v.getElement(i);
                    num++;
                }
            }

            float avg = tot / num;

            for (int i = 0; i < v.getSize(); i++) {
                if (Float.isNaN(v.getElement(i))) {
                    v.setElement(i, avg);
                }
            }

            return v;
        }

        public String getName() {
            return NAME;
        }
    }

    /**
     * ExpectationMaximization
     */
    public static class ExpectationMaximization extends AbstractImputer {

        public static final String NAME = "ExpectationMaximization";

        public Vector impute(Vector v) {
            throw new NotImplementedException();
        }

        public String getName() {
            return NAME;
        }

    }


} // End class Imputers
