/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg;


import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.Template;

import java.util.Map;


/**
 * Specification of a Metric
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public interface Metric {

    public static final Type CATEGORICAL = new Type("categorical");

    public static final Type CONTINUOUS = new Type("continuous");

    public static final Type CAT_AND_CONT = new Type("cat_and-cont");

    public String getName();

    public Metric cloneMetric();

    public String getMissingInfo();

    /**
     * Contract: small numbers should correspond to small distance between vectors
     * and hence close neighbors.
     * <p/>
     * keys of the params map correspond to one of the constants defined above.
     * Typical example of params are USE_MEDIAN and USE_BIASED
     */
    public double getScore(Vector profile, Template template, Map params);

    public double getScore(Vector x, Vector y, Map params);

    public Type getType();

    public boolean isCategorical();

    public boolean isContinuous();

    public int getMinNumSamplesNeededPerClassForCalculation();

    /**
     * Inner class struc
     */
    static class Type {

        private String name;

        Type(String name) {
            this.name = name;
        }

        public boolean equals(Object obj) {
            if (obj instanceof Type) {
                return (((Type) obj).name).equals(name);
            }

            return false;
        }

        public int hashCode() {
            return name.hashCode();
        }

    }

}    // End Metric

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
