/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg;

import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.Template;

/**
 * Collection of simple sanity checking realted utility methods
 */
public class SanityKit {

    public static void enforceEqualNum(Dataset[] dss, Template[] tss) {
        if (dss.length != tss.length) {
            throw new IllegalArgumentException("# of datasets: " + dss.length + " not equal to num of templates: " + tss.length);
        }
    }

    public static void enforceEqualNumRows(Dataset[] dss) {
        getAndEnforceEqualNumRows(dss);
    }

    public static int getAndEnforceEqualNumRows(Dataset[] dss) {
        if (dss.length == 0) {
            throw new IllegalArgumentException("No datasets provides in param -- length 0 array");
        }


        int nrows = dss[0].getNumRow();

        for (int i = 0; i < dss.length; i++) {
            if (dss[i].getNumRow() != nrows) {
                throw new IllegalArgumentException("Dataset at: " + i + " has unequal num of rows: " + dss[i].getNumRow() + " expecting: " + nrows);
            }
        }

        return nrows;
    }

    public static int getAndEnforceEqualNumRows(Dataset dsOne, Dataset dsTwo) {
        return getAndEnforceEqualNumRows(new Dataset[]{dsOne, dsTwo});
    }

    public static void enforceEqualNumRows(Dataset dsOne, Dataset dsTwo) {
        getAndEnforceEqualNumRows(new Dataset[]{dsOne, dsTwo});
    }

    public static String getName(Dataset ds, Template t) {
        return ds.getName() + "_" + t.getName();
    }

    public static Dataset[] combine(Dataset ds, Dataset[] dss) throws Exception {
        Dataset[] ret = new Dataset[1 + dss.length];
        ret[0] = ds;
        for (int i = 1; i < ret.length; i++) {
            ret[i] = dss[i - 1];
        }

        return ret;
    }

    public static Template[] combine(Template t, Template[] tss) throws Exception {
        Template[] ret = new Template[1 + tss.length];
        ret[0] = t;
        for (int i = 1; i < ret.length; i++) {
            ret[i] = tss[i - 1];
        }

        return ret;
    }

} // End SanityKit
