/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg;


/**
 * @author Aravind Subramanian
 *         Date: May 31, 2004
 *         Time: 7:21:56 PM
 */
public class TwoMer {

    private String A;
    private String B;

    private float value; // some value

    /**
     * ORDER is MEANINGLESS @see hashCode & equals below)
     * Class constructor
     *
     * @param a
     * @param b
     */
    public TwoMer(String a, String b) {

        if (a == null) {
            throw new IllegalArgumentException("Parameter A cannot be null");
        }

        if (b == null) {
            throw new IllegalArgumentException("Parameter B cannot be null");
        }

        this.A = a;
        this.B = b;
    }

    /**
     * Class constructor
     *
     * @param a
     * @param b
     * @param value
     */
    public TwoMer(String a, String b, float value) {
        this(a, b);
        this.value = value;
    }

    public float getValue() {
        return value;
    }

    public String getA() {
        return A;
    }

    public String getB() {
        return B;
    }

    public boolean equals(Object obj) {
        if (obj instanceof TwoMer) {
            TwoMer tm = (TwoMer) obj;
            if (tm.A.equals(this.A) && tm.B.equals(this.B)) {
                return true;
            }
            if (tm.A.equals(this.B) && tm.B.equals(this.A)) {
                return true;
            }
        }

        return false;
    }

    public String toString() {
        return A + ":" + B;
    }

    public String toString(char delim) {
        return A + delim + B;
    }

    public int hashCode() {
        return A.hashCode() + B.hashCode();
    }

} // End class TwoMer
