/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.alg.cluster;


import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.Vector;
import org.apache.log4j.Logger;

import java.util.Map;


/**
 * Contains several implementation of LinkageMetrics
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class LinkageMetrics {

    /**
     * For logging support
     */
    protected static final transient Logger klog = XLogger.getLogger(LinkageMetrics.class);

    // @maint add a metric and this array might need updating

    public static LinkageMetric[] createAll() {
        return new LinkageMetric[]{
                //new Cosine(), new Euclidean(), new Manhatten(), new Pearson(), new FeatureVar(),

        };
    }

    /**
     * Private class constructor to prevent construction outside.
     */
    private LinkageMetrics() {
    }


    public static LinkageMetric lookup(Object obj) {

        if (obj == null) {
            throw new NullPointerException("Cannot lookup for null object");
        }

        if (obj instanceof LinkageMetric) {
            return (LinkageMetric) obj;
        }

        LinkageMetric[] all = createAll();

        for (int i = 0; i < all.length; i++) {
            if (all[i].getName().equalsIgnoreCase(obj.toString())) {
                return all[i];
            }
        }

        throw new RuntimeException("Cannot lookup LinkageMetric by: " + obj);
    }


    /**
     * Euclidean.
     */
    public static class AverageLinkage implements LinkageMetric {

        public static final String NAME = "Average Linkage";

        /**
         * Class Constructor.
         */
        public AverageLinkage() {
        }

        public Vector merge(Vector x, Vector y, Map params) {
            Vector res = new Vector(x.getSize());
            for (int i = 0; i < res.getSize(); i++) {
                res.setElement(i, (x.getElement(i) + y.getElement(i) / 2));
            }

            return res;
        }

        public float[] merge(float[] x, float[] y, Map params) {
            float[] res = new float[x.length];
            for (int i = 0; i < res.length; i++) {
                res[i] = (x[i] + y[i]) / 2;
            }

            return res;
        }

        public String getName() {
            return NAME;
        }

        public LinkageMetric cloneLinkageMetric() {
            return new AverageLinkage();
        }

        public String toString() {
            return NAME;
        }
    }

}    // End LinkageMetrics


