package edu.mit.broad.genome.alg.filters;

import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.alg.GeneSetGenerators;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.FSet;
import edu.mit.broad.genome.objects.GeneSet;

import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Oct 1, 2005
 *         Time: 4:24:18 PM
 */
public class FilteredDatasetImpl implements FilteredDataset {

    private String fFilterName;

    private Dataset fOrigDs;

    private String fInfo;

    private GeneSet fPassing;

    /**
     * Class constructor
     *
     * @param filterName
     * @param origDataset
     * @param passing
     * @param info
     */
    public FilteredDatasetImpl(final String filterName,
                               final Dataset origDataset,
                               final List passing,
                               final String info) {
        // @note naming method
        this(filterName, origDataset, new FSet(origDataset.getName() + "_" + filterName + "_" + passing, passing, false), info);
    }

    public FilteredDatasetImpl(final String filterName,
                               final Dataset origDataset,
                               final String[] passing,
                               final String info) {
        // @note naming method
        this(filterName, origDataset, new FSet(origDataset.getName() + "_" + filterName + "_" + passing.length, passing), info);
    }

    /**
     * Class constructor
     *
     * @param filterName
     * @param origDataset
     * @param passing
     * @param info
     */
    public FilteredDatasetImpl(final String filterName,
                               final Dataset origDataset,
                               final GeneSet passing,
                               final String info) {

        if (origDataset == null) {
            throw new IllegalArgumentException("Param origDataset cannot be null");
        }

        if (passing == null) {
            throw new IllegalArgumentException("Param passing cannot be null");
        }

        this.fFilterName = filterName;
        this.fOrigDs = origDataset;
        this.fPassing = passing;
        this.fInfo = info;
    }

    public String getName() {
        return fFilterName;
    }

    public String getInfo() {
        return fInfo;
    }

    private Dataset fPassingDs; // filled lazilly

    private Dataset fFailingDs; // filled lazilly

    public Dataset getDataset(boolean passing) {

        if (passing) {
            if (fPassingDs == null) {
                this.fPassingDs = new DatasetGenerators().extractRows(getName(), fOrigDs, fPassing);
            }

            return fPassingDs;
        } else {
            if (fFailingDs == null) {
                this.fFailingDs = new DatasetGenerators().extractRows(getName(), fOrigDs, fPassing, false);
            }

            return fFailingDs;
        }
    }

    private GeneSet fFailing; // filled lazilly

    public GeneSet getGeneSet(boolean passing) {
        if (passing) {
            return fPassing;
        } else {
            if (fFailing == null) {
                this.fFailing = GeneSetGenerators.subtractGeneSet(fOrigDs.getRowNamesGeneSet(), fPassing);
            }

            return fFailing;
        }
    }

    public Dataset getDatasetOriginal() {
        return fOrigDs;
    }

    public int getNumOriginalFeatures() {
        return fOrigDs.getNumRow();
    }

} // End class FilteredDatasetImpl
