/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg.gsea;

import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.GeneSetMatrix;
import edu.mit.broad.genome.objects.RankedList;

import java.util.List;

/**
 * Defines a collection of gene sets and their associated scoring scheme (weights)
 */
public interface GeneSetCohort {

    public RankedList getRankedList();

    public GeneSetCohort clone(final GeneSet[] gsets, final boolean qualifySetsAgainstRankedList);

    public GeneSet getGeneSet(int gsetNum);

    public GeneSet getGeneSet(String gsetName);

    public GeneSet[] getGeneSets();

    // @note justin guinneys addition
    public int[] genesetIndicesForGene(final String geneName); // Indices of those gene SETS that contain the specified gene

    // can be expensive
    public GeneSetMatrix getGeneSetMatrix(String[] gsetNames);

    public GeneSetMatrix getGeneSetMatrix();

    public List getGeneSetNames();

    public String[] getGeneSetNamesArray();

    public int getNumGeneSets();

    public int getNumGenes(boolean uniqueOnly);

    public int getNumLabels();

    /* DONT provide this!!! ITS WRONG (NOT SORTED BY METRIC!!!)
    public String getLabel(int r) {
        return fLabels.get(r).toString();
    }
    */

    /**
     * SCORING SCHEME RELATED
     */
    public double getHitPoints(int gsetNum, String geneName);

    public double getHitPoints_ssea(int gsetNum, String geneName);

    public double getHitPoints_gsaasnp(int gsetNum, String geneName);

    public double getMissPoints(int gsetNum, String geneName);

    public double getMissPoints_native(int gsetNum, String geneName);

    public boolean isMember(int gsetNum, String geneName);

    public int getNumTrue(int gsetNum);

    public double getPotential(int gsetNum, boolean pos);

} // End class GeneSetCohort
