/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.alg.gsea;

import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.ScoreMode;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentResult;

/**
 * @author Aravind Subramanian
 *         Date: Aug 7, 2003
 *         Time: 3:40:25 PM
 */
public interface MhtMode {

    public float calcVar(Vector v, ScoreMode smode);

    public float calcMean(Vector v, ScoreMode smode);

    public float calcNes(float es, Vector rndEss, ScoreMode smode);

    public Vector calcNes(Vector rndEss, ScoreMode smode);

    public void setData(EnrichmentResult[] dtgs);

    /**
     * all the usages below need setData
     */
    public String getName();

    public Matrix getRndNESMatrix();

    public int getDiscoveryMatrixSize();

    public int getValidationMatrixSize();

    public MhtMode cloneShallow();

    /**
     * ACCESS BY TAGNAME
     */

    public Vector getRndNES(String tgName);

    public float getRealNES(String tgName);

    public float getDiscoveryPValue(String tgName);

    public float getDiscoveryPValue(String tgName, float topXPercent);

    public float getValidationPValue(String tgName);

    public float getValidationPValue(String tgName, float topXPercent);

    public float getValidationType2PValue(String tgName);

    /**
     * ACCESS BY GENESETNAME
     */

    public Vector getRndNES_bygsetname(String gsetName);

    public float getRealNES_bygsetname(String gsetName);

    public float getDiscoveryPValue_bygsetname(String gsetName);

    public float getDiscoveryPValue_bygsetname(String gsetName, float topXPercent);

    public float getValidationPValue_bygsetname(String gsetName);

    public float getValidationPValue_bygsetname(String gsetName, float topXPercent);

    public float getValidationType2PValue_bygsetname(String gsetName);


    /**
     * ACCESS BY INDEX
     */
    public Vector getRndNES(int r);

    public float getRealNES(int r);

    public float getDiscoveryPValue(int r);

    public float getDiscoveryPValue(int r, float topXPercent);

    public float getValidationPValue(int r);

    public float getValidationPValue(int r, float topXPercent);

    public float getValidationType2PValue(int r);


    /**
     * access by inded for MW - just a few selected methods
     */

    public float getRealNMW(int r);

    public Vector getRndNMW(int r);

    public float getDiscoveryPValue_mw(int r);

    public float getValidationPValue_mw(int r);

    public Matrix getRndNMWMatrix();


} // End MhtMode


