/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.charts;


import edu.mit.broad.genome.charts.jfreechart.editor.GenomeChartPanel;
import org.jfree.chart.JFreeChart;

import javax.swing.*;


/**
 * Why a new set of Frames, events etc?
 * The popups in JFrreChart dont work properyly for the right-click->Properties ...
 * menu option (the rest i.e save, print etc seem to work)
 * Plus there are way too many options in the "Properties ..."
 * JPopupMenu option -- very confusing.
 * <p/>
 * So, implemented the generic user interaction capability again here with selected
 * options. Its useful having the user interaction so that user can change the
 * title, colors etc (apart from the zooming stuff which works well in
 * the JFreeChart implementation).
 * <p/>
 * So, modified:
 * <p/>
 * ChartFrame
 * ChartPanel (reused several components such as the zoming, but reimplemented
 * the properties ... option)
 * <p/>
 * A frame for displaying a chart.
 * <p/>
 * Code Originally from com.jrefinery.chart.ChartFrame
 *
 * @author JFreeChart guys
 * @author Aravind Subramanian -- modifications for xomics
 */
public class XChartFrame extends JFrame {

    /**
     * The chart panel.
     */
    protected final GenomeChartPanel chartPanel;

    /**
     * Constructs a frame for a chart.
     *
     * @param caption The frame title.
     * @param chart   The chart.
     */
    public XChartFrame(final JFreeChart chart) {
        this(chart, false);
    }

    public XChartFrame(final XChart chart) {
        this(chart.getFreeChart(), false);
    }

    /**
     * Constructs a frame for a chart.
     *
     * @param caption The frame title.
     * @param chart   The chart.
     */
    public XChartFrame(final JFreeChart chart, final boolean scrollPane) {

        super(chart.getTitle().getText());

        chartPanel = new GenomeChartPanel(chart);

        if (scrollPane) {
            this.setContentPane(new JScrollPane(chartPanel));
        } else {
            this.setContentPane(chartPanel);
        }

    }

    /**
     * Returns the chart panel for the frame.
     *
     * @return The chart panel.
     */
    public GenomeChartPanel getChartPanel() {
        return this.chartPanel;
    }
}    // End GenomeChartFrame


