/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.charts;

import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.NotImplementedException;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.Plot;

import javax.swing.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

/**
 * simple wrapper class
 * <p/>
 * requirements
 * <p/>
 * easy setting of "name" of chart (not as complex as the title thing in jfreechart_
 * hides some aspects of jfreechart_test isage from client code
 * <p/>
 * possible to extend later to use some other chartind library in addition to jfreechart_test
 * might later add things like custome sizes and display properties that can have state / user prefs
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */


public class XChartImpl implements XChart {

    private JFreeChart fFreeChart;

    private String fName;

    private String fCaption;

    private static final Icon ICON = JarResources.getIcon("Chart.gif");

    public XChart cloneShallow(final String newName) {
        if (newName == null) {
            throw new IllegalArgumentException("Param newName cannot be null");
        }
        this.fName = newName;
        return this;
    }

    /**
     * Class constructor
     *
     * @param name
     * @param caption
     * @param chart
     */
    public XChartImpl(final String name, final String caption, final JFreeChart chart) {
        init(name, caption, chart);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param plotTitle
     * @param caption
     * @param plot
     * @param createLegend
     */
    public XChartImpl(final String name,
                      final String plotTitle,
                      final String caption,
                      final Plot plot, final boolean createLegend) {

        if (plot == null) {
            throw new IllegalArgumentException("Param chart cannot be null");
        }

        init(name, caption, new JFreeChart(plotTitle, JFreeChart.DEFAULT_TITLE_FONT, plot, createLegend));
    }

    private void init(final String name, final String caption, final JFreeChart jfc) {

        if (jfc == null) {
            throw new IllegalArgumentException("Param jfc cannot be null");
        }

        if (name == null) {
            throw new IllegalArgumentException("Param name cannot be null");
        }

        if (caption == null) {
            this.fCaption = Constants.NA;
            //throw new IllegalArgumentException("Parameter caption cannot be null");
        } else {
            this.fCaption = caption;
        }

        this.fName = name; // must be file name safe
        this.fFreeChart = jfc;
    }

    // imp that it be protected
    // we dont want to expose jfreechart_test usage
    // but sometimes need to
    //protected JFreeChart getFreeChart() {
    public JFreeChart getFreeChart() {
        return fFreeChart;
    }

    // file name safe.
    public String getName() {
        return fName;
    }

    public String getTitle() {
        return fFreeChart.getTitle().getText();
    }

    public String getCaption() {
        return fCaption;
    }

    public Icon getIcon() {
        return ICON;
    }

    public BufferedImage createBufferedImage(int width, int height) {
        return fFreeChart.createBufferedImage(width, height);
    }

    public void saveAsPNG(File inFile, int width, int height) throws IOException {
        ChartUtilities.saveChartAsPNG(inFile, this.fFreeChart, width, height);
    }

    public void saveData(final File inFile) throws Exception {
        // @todo
        throw new NotImplementedException();
    }

} // End Chart


