/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.charts;

import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.CombinedDomainXYPlot;
import org.jfree.chart.plot.Plot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;

import javax.swing.*;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

/**
 * simple wrapper class
 * <p/>
 * similar to Chart except combo chart
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */


public class XComboDomainChart implements XComboChart {

    private CombinedDomainXYPlot fCombinedXYPlot;
    private XChart fCombinedChart;
    private JFreeChart fCombinedFreeChart;

    private String fComboCaption;

    private String[] fSubplotTitles;

    /**
     * contains Charts as elements
     */
    private java.util.List fSubCharts;

    private static final Icon ICON = JarResources.getIcon("Chart.gif");

    private final Logger log = XLogger.getLogger(XComboDomainChart.class);

    /**
     * Class constructor
     *
     * @param comboName
     * @param comboTitle
     * @param xyplots
     * @param subplotTitles
     */
    public XComboDomainChart(final String name,
                             final String comboTitle,
                             final String comboCaption,
                             final String domainAxisLabel,
                             final XYPlot[] xyplots,
                             final int[] weights,
                             final String[] subplotTitles) {

        init(name, comboTitle, comboCaption, domainAxisLabel, xyplots, weights, subplotTitles);

        for (int i = 0; i < getNumSubPlots(); i++) {
            fSubCharts.add(null); // lazily make
        }
    }

    /**
     * Only XYPlots
     *
     * @param comboName
     * @param charts
     */
    public XComboDomainChart(final String name,
                             final String comboTitle,
                             final String comboCaption,
                             final String domainAxisLabel,
                             final XChart[] charts) {
        XYPlot[] xyplots = new XYPlot[charts.length];
        String[] subplotTitles = new String[charts.length];
        for (int i = 0; i < charts.length; i++) {
            Plot plot = charts[i].getFreeChart().getPlot();
            if (plot instanceof XYPlot) {
                xyplots[i] = (XYPlot) plot; // combinedxy plots work off xyplots only
            } else {
                throw new IllegalArgumentException("Only XYPlots allowed. Found: " + plot.getClass() + " at: " + i);
            }

            subplotTitles[i] = charts[i].getName();
        }

        int[] weights = new int[xyplots.length];
        for (int i = 0; i < xyplots.length; i++) {
            weights[i] = 1;
        }

        init(name, comboTitle, comboCaption, domainAxisLabel, xyplots, weights, subplotTitles);

        for (int i = 0; i < getNumSubPlots(); i++) {
            //log.debug("reusing chart: " + charts[i].getDesc());
            fSubCharts.add(charts[i]); // reuse
        }
    }

    public XComboDomainChart(final String name,
                             final String comboTitle,
                             final String comboCaption,
                             final String domainAxisLabel,
                             final XChart[] charts,
                             final int[] weights) {

        XYPlot[] xyplots = new XYPlot[charts.length];
        String[] subplotTitles = new String[charts.length];
        for (int i = 0; i < charts.length; i++) {
            Plot plot = charts[i].getFreeChart().getPlot();
            if (plot instanceof XYPlot) {
                xyplots[i] = (XYPlot) plot; // combinedxy plots work off xyplots only
            } else {
                throw new IllegalArgumentException("Only XYPlots allowed. Found: " + plot.getClass() + " at: " + i);
            }

            subplotTitles[i] = charts[i].getName();
        }

        init(name, comboTitle, comboCaption, domainAxisLabel, xyplots, weights, subplotTitles);

        for (int i = 0; i < getNumSubPlots(); i++) {
            //log.debug("reusing chart: " + charts[i].getDesc());
            fSubCharts.add(charts[i]); // reuse
        }
    }

    // common initialization routine
    /**
     * shared domain axis
     * doc says cannot use the ChartFactory to make charts - need to use constructors.
     * Each of the subplots has to have a null domain axis (X), since they share the
     * parent plots axis.
     */
    private void init(final String name,
                      final String comboTitle,
                      final String comboCaption,
                      final String domainAxisLabel,
                      final XYPlot[] xyplots,
                      final int[] weights,
                      final String[] subplotTitles) {

        if (xyplots == null) {
            throw new IllegalArgumentException("Param xyplots cannot be null");
        }

        this.fCombinedXYPlot = new CombinedDomainXYPlot(new NumberAxis(domainAxisLabel));
        fCombinedXYPlot.setOrientation(PlotOrientation.VERTICAL); // @note

        for (int i = 0; i < xyplots.length; i++) {
            xyplots[i].setDomainAxis(null);// needed i think (yes)
            fCombinedXYPlot.add(xyplots[i], weights[i]);    // weight - amt of space to give it
        }

        this.fComboCaption = comboCaption;
        this.fSubplotTitles = subplotTitles;
        this.fCombinedFreeChart = new JFreeChart(comboTitle, JFreeChart.DEFAULT_TITLE_FONT, fCombinedXYPlot, true);


        this.fCombinedChart = new XChartImpl(name, fComboCaption, fCombinedFreeChart);
        this.fSubCharts = new ArrayList();
    }

    public CombinedDomainXYPlot getCombinedXYPlot() {
        return fCombinedXYPlot;
    }

    public XChart getCombinedChart() {
        return fCombinedChart;
    }

    public XChart getCombinedChart(String newName) {
        return fCombinedChart.cloneShallow(newName);
    }

    public void saveAsPNG(File inFile, int width, int height) throws IOException {
        getCombinedChart().saveAsPNG(inFile, width, height);
    }

    public JFreeChart getCombinedFreeChart() {
        return fCombinedFreeChart;
    }

    public int getNumSubPlots() {
        return fCombinedXYPlot.getSubplots().size();
    }

    public XChart getSubChart(int index) {

        Object obj = fSubCharts.get(index);
        if (obj != null) {
            return (XChart) obj;
        }

        Plot plot = (Plot) fCombinedXYPlot.getSubplots().get(index);
        XChart chart = new XChartImpl("index_" + index, fComboCaption + "_subplot" + index, fSubplotTitles[index], plot, true);
        fSubCharts.set(index, chart);
        return chart;
    }

    public XYPlot getSubXYPlot(int index) {
        return (XYPlot) fCombinedXYPlot.getSubplots().get(index);
    }

    public String getCaption() {
        return fComboCaption;
    }

    public Icon getIcon() {
        return ICON;
    }

} // End Chart


