/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.charts.jfreechart.editor;

import edu.mit.broad.genome.swing.fields.*;
import org.jfree.chart.axis.Axis;
import org.jfree.chart.axis.NumberAxis;

import javax.swing.*;
import java.util.ArrayList;

/**
 * <p> </p>
 * <p> </p>
 * <p> </p>
 * <p> </p>
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */

class AxisEditor {

    /**
     * domain axis stuff
     */
    protected GStringField fAxisLabelChooser;
    protected GColorFieldPlusChooser fAxisLabelColorChooser;
    protected GColorFieldPlusChooser fAxisTickColorChooser;
    protected JCheckBox cxShowAxisTickLabels;
    protected JCheckBox cxShowAxisTickMarks;
    protected JCheckBox cxShowAxisGridLines;    // not always available, only valid for NumbersAxis

    private Axis fAxis;

    protected GFieldPlusChooserTable fTable;

    /**
     * Class Constructor.
     */
    protected AxisEditor(Axis axis) {

        if (axis == null) throw new IllegalArgumentException("Param axis cannot be null");

        java.util.List fields = new ArrayList();
        java.util.List names = new ArrayList();

        this.fAxis = axis;

        this.fAxisLabelChooser = new GStringField(axis.getLabel());
        fields.add(fAxisLabelChooser);
        names.add("Axis Label");

        this.fAxisLabelColorChooser = new GColorFieldPlusChooser();
        fields.add(fAxisLabelColorChooser);
        names.add("Label Color");

        this.fAxisTickColorChooser = new GColorFieldPlusChooser();
        fields.add(fAxisTickColorChooser);
        names.add("Tick Color");

        this.cxShowAxisTickLabels = new JCheckBox("", axis.isTickLabelsVisible());
        fields.add(new GCheckBoxField(cxShowAxisTickLabels));
        names.add("Show Tick Labels");

        this.cxShowAxisTickMarks = new JCheckBox("", axis.isTickMarksVisible());
        fields.add(new GCheckBoxField(cxShowAxisTickMarks));
        names.add("Show Tick Marks");

        // figure out what type of axis we have and instantiate the appropriate panel
        if (axis instanceof NumberAxis) {
            /*
            cxShowAxisGridLines = new JCheckBox("",
                    ((NumberAxis) axis).isGridLinesVisible()); // method seems to have dissepeared in new version 0.9.6
            fields.add(new GCheckBoxField(cxShowAxisGridLines));
            names.add("Grid Lines");
            */
        }

        fTable = new GFieldPlusChooserTable((String[]) names.toArray(new String[names.size()]),
                (GFieldPlusChooser[]) fields.toArray(new GFieldPlusChooser[fields.size()]));


    }


    public void updateChoices() {

        fAxis.setLabel(fAxisLabelChooser.getValue().toString());

        if (fAxisTickColorChooser.getColor() != null) {
            fAxis.setTickLabelPaint(fAxisTickColorChooser.getColor());
        }

        if (fAxisLabelColorChooser.getColor() != null) {
            fAxis.setLabelPaint(fAxisLabelColorChooser.getColor());
        }


        fAxis.setTickMarksVisible(cxShowAxisTickMarks.isSelected());
        fAxis.setTickLabelsVisible(cxShowAxisTickLabels.isSelected());

        /* ,ethod disseaperaed in new version?
        if(fAxis instanceof NumberAxis) {
            ((NumberAxis) fAxis).setGridLinesVisible(
                    cxShowAxisGridLines.isSelected());
        }
        */
    }


} // End AxisEditor

