/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.charts.jfreechart.editor;


import edu.mit.broad.genome.swing.GuiHelper;
import edu.mit.broad.genome.swing.fields.*;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.Axis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.Plot;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.title.TextTitle;
import org.jfree.chart.title.Title;

import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;

/**
 * Original code from com.jrefinery.chart.ui.ChartPropertyEditPanel
 * <p/>
 * Modified by Aravind Subramanian for xomics.
 *
 * @todo Implement further customization options
 * <p/>
 * A panel for editing chart properties (includes subpanels for the title, legend and plot).
 * @see edu.mit.broad.genome.charts.XChartFrame and XChartpanel for details.
 */
public class GenomeChartPropertyEditPanel extends JPanel {

    /**
     * IMP IMP IMP
     * We dont make these refer directly to the prop -> user might cancel and
     * we dont want the real ones (in chart) changing unless user hits Ok
     * So declare new vars to hold the properties that we are (posiibly) editing.
     */
    private JTabbedPane fTabbedPane;
    private JFreeChart fChart;

    //private LegendTitle fOriginalLegend;

    /**
     * chart stuff
     */
    private GStringField fChartTitleChooser;
    private GColorFieldPlusChooser fChartBgColorChooser;
    private GColorFieldPlusChooser fPlotBgColorChooser;
    private GCheckBoxField fChartAntiAliasChooser;

    /**
     * legend panel stuff
     */
    private GCheckBoxField fLegendOnOffChooser;
    private GComboBoxField fLegendOutlineStrokeChooser;
    private GColorFieldPlusChooser fLegendOutlineColorChooser;
    private GColorFieldPlusChooser fLegendBgColorChooser;
    private GColorFieldPlusChooser fLegendSeriesColorChooser;
    /*
    public final StrokeSample[] kAvailableStrokeSamples =
        new StrokeSample[]{ new MyStrokeSample(new BasicStroke(1.0f)),
                            new MyStrokeSample(new BasicStroke(2.0f)),
                            new MyStrokeSample(new BasicStroke(3.0f)),
                            new MyStrokeSample(new BasicStroke(4.0f)) };
    */
    /*
    public final StrokeSample[] kAvailableStrokeSamples =
            new StrokeSample[]{new StrokeSample(new BasicStroke(1.0f)),
                    new StrokeSample(new BasicStroke(2.0f)),
                    new StrokeSample(new BasicStroke(3.0f)),
                    new StrokeSample(new BasicStroke(4.0f))};
                    */

    private AxisEditor fDomain;
    private AxisEditor fRange;

    /**
     * Class Constructor.
     * The property panel is made up of a number of sub-panels that are
     * displayed in the tabbed pane.
     */
    public GenomeChartPropertyEditPanel(final JFreeChart chart) {

        if (chart == null) {
            throw new IllegalArgumentException("Param chart cannot be null");
        }

        this.fChart = chart;
        //this.fOriginalLegend = fChart.getLegend();

        init();
    }

    private void init() {
        jbInit();
    }

    private void jbInit() {

        this.fTabbedPane = new JTabbedPane(JTabbedPane.TOP);

        fTabbedPane.addTab("Chart", new JScrollPane(createChartControls()));

        fTabbedPane.addTab("Legend", new JScrollPane(createLegendControls()));

        // domain
        Axis axis = getDomainAxis();
        Component axisComp;

        if (axis == null) {
            axisComp = GuiHelper.createNaPlaceholder();
        } else {
            this.fDomain = new AxisEditor(axis);
            axisComp = new JScrollPane(fDomain.fTable);
        }

        fTabbedPane.add("Domain Axis (X)", axisComp);

        // range
        axis = getRangeAxis();

        if (axis == null) {
            axisComp = GuiHelper.createNaPlaceholder();
        } else {
            fRange = new AxisEditor(axis);
            axisComp = new JScrollPane(fRange.fTable);
        }

        fTabbedPane.add("Range Axis (Y)", axisComp);
        this.setLayout(new BorderLayout());
        this.add(fTabbedPane, BorderLayout.CENTER);
    }

    /**
     * null if not a text title
     *
     * @return
     */
    private TextTitle getTextTitle() {

        Title title = fChart.getTitle();

        if (title instanceof TextTitle) {
            return (TextTitle) title;
        } else if (title != null) {
            return new TextTitle(title.getID());
        } else {
            return new TextTitle("na_title");
        }
    }

    private GFieldPlusChooserTable createChartControls() {

        java.util.List fields = new ArrayList();
        java.util.List names = new ArrayList();
        TextTitle title = getTextTitle();

        if (title != null) {
            fChartTitleChooser = new GStringField(title.getText());

            fields.add(fChartTitleChooser);
            names.add("Chart Title");
        }

        fChartBgColorChooser = new GColorFieldPlusChooser();
        fields.add(fChartBgColorChooser);
        names.add("Chart Background (External)");

        fPlotBgColorChooser = new GColorFieldPlusChooser();
        fields.add(fPlotBgColorChooser);
        names.add("Chart Background (Internal)");

        fChartAntiAliasChooser = new GCheckBoxField(new JCheckBox("", fChart.getAntiAlias()));
        fields.add(fChartAntiAliasChooser);
        names.add("Anti Alias");

        return new GFieldPlusChooserTable((String[]) names.toArray(new String[names.size()]),
                (GFieldPlusChooser[]) fields.toArray(new GFieldPlusChooser[fields.size()]));
    }

    /**
     * Creates the Legend panel
     *
     * @return
     * @maint assuming StandardLegend
     */
    private GFieldPlusChooserTable createLegendControls() {

        java.util.List fields = new ArrayList();
        java.util.List names = new ArrayList();

        // legend stuff

        fLegendOutlineColorChooser = new GColorFieldPlusChooser();
        fields.add(fLegendOutlineColorChooser);
        names.add("Outline Color");

        fLegendBgColorChooser = new GColorFieldPlusChooser();
        fields.add(fLegendBgColorChooser);
        names.add("Legend Background");

        fLegendSeriesColorChooser = new GColorFieldPlusChooser();
        fields.add(fLegendSeriesColorChooser);
        names.add("Series Labels Color");

        // IMP placing the combobox first makes the components be NOT editable
        // really wierd
        // ditto with the jcheckbox

        fLegendOnOffChooser = new GCheckBoxField(new JCheckBox("", true));
        fields.add(fLegendOnOffChooser);
        names.add("Legend On");

        //StandardLegend slegend = (StandardLegend) fOriginalLegend;
        //JComboBox selector = new JComboBox(kAvailableStrokeSamples);
        JComboBox selector = new JComboBox();

        //selector.setSelectedItem(slegend.getOutlineStroke());
        //selector.setRenderer(new StrokeSample(new BasicStroke(1)));

        fLegendOutlineStrokeChooser = new GComboBoxField(selector);

        fields.add(fLegendOutlineStrokeChooser);
        names.add("Stroke Outline");

        return new GFieldPlusChooserTable((String[]) names.toArray(new String[names.size()]),
                (GFieldPlusChooser[]) fields.toArray(new GFieldPlusChooser[fields.size()]));
    }

    /**
     * Updates the properties of a chart to match the properties defined
     * on this panel.     *
     *
     * @maint need to manually keep in synch with panel contents
     */
    public void updateChartProperties() {

        // update the chart
        if (fChartBgColorChooser.getColor() != null) {
            fChart.setBackgroundPaint(fChartBgColorChooser.getColor());
            fChart.getPlot().setBackgroundPaint(fPlotBgColorChooser.getColor());
        }

        TextTitle title = getTextTitle();

        if (title != null) {
            title.setText(fChartTitleChooser.getValue().toString());
        }

        fChart.setAntiAlias(fChartAntiAliasChooser.isSelected());

        // update the legend
        /*
        boolean dolegend = fLegendOnOffChooser.isSelected();
        if (dolegend) { // remeber use fOriginalLegend
            fChart.setLegend(fOriginalLegend);
            StandardLegend slegend = (StandardLegend) fOriginalLegend;
            StrokeSample ss = (StrokeSample) fLegendOutlineStrokeChooser.getValue();

            if (ss != null) {
                slegend.setOutlineStroke(ss.getStroke());
            }

            if (fLegendOutlineColorChooser.getColor() != null) {
                slegend.setOutlinePaint(fLegendOutlineColorChooser.getColor());
            }

            if (fLegendBgColorChooser.getColor() != null) {
                slegend.setBackgroundPaint(fLegendBgColorChooser.getColor());
            }

            if (fLegendSeriesColorChooser.getColor() != null) {
                slegend.setItemPaint(fLegendSeriesColorChooser.getColor());
            }

        } else {
            fChart.setLegend(null);
        }
        */

        // update the domain axis
        Axis domainAxis = getDomainAxis();

        if (domainAxis != null) {
            fDomain.updateChoices();
        }

        // update the Rangle axis
        Axis rangeAxis = getRangeAxis();

        if (rangeAxis != null) {
            fRange.updateChoices();
        }
    }

    /**
     * might return null if not gettable
     *
     * @return
     */
    private Axis getDomainAxis() {

        Plot plot = fChart.getPlot();
        Axis axis = null;

        if (plot instanceof CategoryPlot) {
            axis = ((CategoryPlot) plot).getDomainAxis();
        } else if (plot instanceof XYPlot) {
            axis = ((XYPlot) plot).getDomainAxis();
        }

        return axis;
    }

    /**
     * might return null if not gettable
     *
     * @return
     */
    private Axis getRangeAxis() {

        Plot plot = fChart.getPlot();
        Axis axis = null;

        if (plot instanceof CategoryPlot) {
            axis = ((CategoryPlot) plot).getRangeAxis();
        } else if (plot instanceof XYPlot) {
            axis = ((XYPlot) plot).getRangeAxis();
        }

        return axis;
    }

    // just to get the combo box display the names

    /**
     * Class MyStrokeSample
     *
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    /*
    class MyStrokeSample extends StrokeSample {

        MyStrokeSample(BasicStroke s) {
            super((Stroke) s);
        }

        public String toString() {
            return((BasicStroke) super.getStroke()). .getName();
        }
    }
    */
}

/*--- Formatted in Sun Java Convention Style on Sun, Sep 29, '02 ---*/
