package edu.mit.broad.genome.io;

import com.enterprisedt.net.ftp.FTPConnectMode;
import com.enterprisedt.net.ftp.FTPTransferType;
import com.enterprisedt.net.ftp.FileTransferClient;
import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;
import xapps.gsea.GseaWebResources;

import java.io.File;
import java.io.FileInputStream;
import java.net.URL;
import java.util.StringTokenizer;

/**
 * This class is used to ftp download gmt, chip and msigdb files from
 * the Broad GSEA ftp server gseaftp.broadinstitute.org.  
 */
public class FTPUtils {

    private static final Logger klog = XLogger.getLogger(FTPUtils.class);

    public static FileInputStream ftpDownloadAndCreateInputStream(final URL ftpURL ) throws Exception {

        if (!ftpURL.getProtocol().equalsIgnoreCase("ftp")) {
            throw new IllegalArgumentException("expected ftp url");
        }

        if (!ftpURL.getHost().equalsIgnoreCase(GseaWebResources.getGseaFTPServer())) {
            throw new IllegalArgumentException("unexpected hostname: " + ftpURL.getHost());
        }

        String fullPath = ftpURL.getPath();
        String fileName = null;
        StringTokenizer tok = new StringTokenizer(ftpURL.getPath(), "/");
        while (tok.hasMoreTokens()) {
            fileName = tok.nextToken();
        }
        int index = fullPath.indexOf(fileName);
        String remoteDirectory = fullPath.substring(0, index);

        FTPUtils.FTPClientAndFile ftpc = _createClient(fileName, remoteDirectory);

        System.out.println(">>>" + ftpc.fileName + "<");
        File dest = File.createTempFile("ftp_gsea", ftpc.fileName);

        ftpc.ftp.downloadFile(dest.getPath(), ftpc.fileName);

        // shutdown ftp connection
        ftpc.ftp.disconnect();

        return new FileInputStream(dest);
    }

    private static FTPUtils.FTPClientAndFile _createClient(final String fileName, final String chDir) throws Exception {

        String host = GseaWebResources.getGseaFTPServer();
        String userName = GseaWebResources.getGseaFTPServerUserName();
        String password = GseaWebResources.getGseaFTPServerPassword();

        klog.debug("FTP connected to: host: " + host + " chDir: " + chDir + " filename: " + fileName);

        FileTransferClient ftp = new FileTransferClient();

        // set remote host
        ftp.setRemoteHost(host);
        ftp.setUserName(userName);
        ftp.setPassword(password);

        // connect to the server
        ftp.connect();

        // set transfer type to binary
        ftp.setContentType(FTPTransferType.BINARY);

        // set connect mode to Passive
        ftp.getAdvancedFTPSettings().setConnectMode(FTPConnectMode.PASV);

        // change to directory containing msigdb
        ftp.changeDirectory(chDir);

        FTPUtils.FTPClientAndFile ftpClientAndFile = new FTPUtils.FTPClientAndFile();
        ftpClientAndFile.ftp = ftp;
        ftpClientAndFile.fileName = fileName;
        return ftpClientAndFile;
    }


    static class FTPClientAndFile {
        FileTransferClient ftp;
        String fileName;
    }


} // End class FTPUtils
