/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.io;


import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.io.File;
import java.util.ArrayList;


/**
 * Transferable/Clipboard object tp represent one or more Files.
 * Usually for use via Clipboard or dnd.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class FileTransferable implements Transferable, ClipboardOwner {

    private final java.util.List fFiles;
    private final String fPaths;
    private final String fNames;

    /**
     * Class Constructor.
     *
     * @param file
     */
    public FileTransferable(File file) {
        this(new File[]{file});
    }

    /**
     * Class Constructor.
     *
     * @param files
     */
    public FileTransferable(File[] files) {

        if (files != null) {
            fFiles = new ArrayList(files.length);

            for (int i = 0; i < files.length; i++) {
                fFiles.add(files[i]);
            }

            StringBuffer buf = new StringBuffer();
            StringBuffer buf2 = new StringBuffer();

            for (int i = 0; i < fFiles.size(); i++) {
                buf.append(((File) fFiles.get(i)).getAbsolutePath()).append('\n');
                buf2.append(((File) fFiles.get(i)).getName()).append('\n');
            }

            fPaths = buf.toString();
            fNames = buf2.toString();
        } else {
            fFiles = new ArrayList();
            fPaths = "";
            fNames = "";
        }
    }

    public File[] getFiles() {
        return (File[]) fFiles.toArray(new File[fFiles.size()]);
    }

    public String getFilePaths() {
        return fPaths;
    }

    public String getFileNames() {
        return fNames;
    }

    public Object getTransferData(DataFlavor flavor) {

        if (flavor.equals(DataFlavor.javaFileListFlavor)) {
            return fFiles;
        } else if (flavor.equals(DataFlavor.stringFlavor)) {
            return fPaths;
        } else {
            return null;
        }
    }

    public DataFlavor[] getTransferDataFlavors() {
        return new DataFlavor[]{DataFlavor.javaFileListFlavor, DataFlavor.stringFlavor};
    }

    public boolean isDataFlavorSupported(DataFlavor flavor) {

        if ((flavor.equals(DataFlavor.javaFileListFlavor))
                || (flavor.equals(DataFlavor.stringFlavor))) {
            return true;
        }

        return false;
    }

    public void lostOwnership(Clipboard clipboard, Transferable contents) {

        //log.debug("Lost ownership");
    }

}    // End class FileTransferable

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
