/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.io;


import edu.mit.broad.genome.objects.DataType;
import edu.mit.broad.genome.objects.PersistentObject;

import java.awt.datatransfer.DataFlavor;


/**
 * Construct a DataFlavor that represents one PersistentObject
 * or a list of PersistentObject's.
 * Typically used for datatransfer - via clipboard or dnd.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class PobFlavor extends DataType {

    private final boolean fInited;

    /**
     * all PersistentObject classes
     */
    public static final PobFlavor ALL_POB = new PobFlavor(PersistentObject.class);
    public static final DataFlavor pobListFlavor = new DataFlavor(javaSerializedObjectMimeType
            + ";class=java.util.List", "PobList");

    /**
     * Class Constructor.
     */
    public PobFlavor(Class pobClass) {

        super(pobClass, "PersistentObject");

        if (!PersistentObject.class.isAssignableFrom(pobClass)) {
            fInited = false;

            throw new IllegalArgumentException("Invalid pobClass: " + pobClass);
        }

        fInited = true;
    }

    /**
     * Class Constructor.
     */
    public PobFlavor(Class pobClass, String name) {

        super(pobClass, name);

        if (!PersistentObject.class.isAssignableFrom(pobClass)) {
            fInited = false;

            throw new IllegalArgumentException("Invalid pobClass: " + pobClass);
        }

        fInited = true;
    }

    /**
     * All public API's must call
     */
    protected void checkInit() {

        if (!fInited) {
            throw new IllegalStateException("Object was not initialized properly");
        }
    }

    public boolean equals(DataFlavor df) {
        return this.equals((Object) df);
    }

    /**
     * This is the trick.
     * This method overrides the parent impl to check for not just for
     * equality but also if the class represented by df is a sub-class
     * (isAssignableFrom() the class represented by this ClassFlavor)
     */
    public boolean equals(PobFlavor pf) {

        checkInit();

        Class cl = pf.getRepresentationClass();

        return this.getRepresentationClass().isAssignableFrom(cl);
    }

    public boolean equals(Object obj) {

        if (obj instanceof PobFlavor) {
            return this.equals((PobFlavor) obj);
        } else {
            return false;
        }
    }

    /**
     * IMP to NOT use. As this makes a
     * String("foo) equal to an NodeFlavor("foo")
     */

    //public int hashCode() {
    //return fExt.hashCode();
    //}
    public static Class[] toClasses(DataFlavor[] flavors) {

        Class[] classes = new Class[flavors.length];

        for (int i = 0; i < flavors.length; i++) {
            classes[i] = flavors[i].getRepresentationClass();
        }

        return classes;
    }
}    // End PobFlavor

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
