/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.math;

import edu.mit.broad.genome.utils.ImmutedException;


/**
 * very similar to Matrix except boolean + many inapprpriate methods gotten trid off.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class BooleanMatrix implements IMatrix {

    /*
     * SEE Matrix for many more imp notes
     */

    /**
     * @maint IMP IMP: if any fields are added to this class, make sure
     * to update the constructors and the set(Matrix method
     * <p/>
     * The data of the Matrix.(1-D array. The (i,j) element is stored in elementData[i*fColCnt + j])
     * <p/>
     * The data of the Matrix.(1-D array. The (i,j) element is stored in elementData[i*fColCnt + j])
     */

    /**
     * The data of the Matrix.(1-D array. The (i,j) element is stored in elementData[i*fColCnt + j])
     */
    protected boolean elementData[];

    /**
     * The number of rows in this matrix.
     */
    private int fRowCnt;

    /**
     * The number of columns in this matrix.
     */
    private int fColCnt;
    private boolean fImmuted;

    /**
     * Class Constructor.
     * Must call set() after this form of the constructor.
     */
    public BooleanMatrix() {
    }

    /**
     * Constructs an fRowCnt by fColCnt all-false matrix.
     * Note that even though row and column numbering begins with
     * zero, fRowCnt and fColCnt will be one larger than the maximum
     * possible matrix index values.
     *
     * @param nrows number of rows in this matrix.
     * @param ncols number of columns in this matrix.
     */
    public BooleanMatrix(int nrows, int ncols) {

        if (nrows < 0) {
            throw new NegativeArraySizeException(nrows + " < 0");
        }

        if (ncols < 0) {
            throw new NegativeArraySizeException(ncols + " < 0");
        }

        this.fRowCnt = nrows;
        this.fColCnt = ncols;
        elementData = new boolean[fRowCnt * fColCnt];
    }

    /**
     * Constructs a new Matrix and copies the initial values
     * from the parameter matrix.
     *
     * @param matrix the source of the initial values of the new Matrix
     *               <p/>
     *               as added: if shared values, then this matrix shares the elements with the specified amtric
     *               else the array is systemarrycopied
     */
    public BooleanMatrix(BooleanMatrix matrix, boolean shareValues) {

        this.fRowCnt = matrix.fRowCnt;
        this.fColCnt = matrix.fColCnt;

        if (shareValues) {
            this.fImmuted = matrix.fImmuted;
        } else {
            this.fImmuted = false;
        }

        if (shareValues) {
            this.elementData = matrix.elementData;
        } else {
            int newSize = fRowCnt * fColCnt;

            this.elementData = new boolean[newSize];

            System.arraycopy(matrix.elementData, 0, elementData, 0, newSize);
        }
    }

    /**
     * Changes the size of this matrix dynamically.  If the size is increased
     * no data values will be lost.  If the size is decreased, only those data
     * values whose matrix positions were eliminated will be lost.
     *
     * @param nrows number of desired rows in this matrix
     * @param ncols number of desired columns in this matrix
     */
    public void setSize(int nrows, int ncols) {

        checkImmutable();

        if ((fRowCnt < 0) || (fColCnt < 0)) {
            throw new NegativeArraySizeException("nrows or ncols < 0");
        }

        int oldfRowCnt = this.fRowCnt;
        int oldfColCnt = this.fColCnt;
        int oldSize = this.fRowCnt * this.fColCnt;

        this.fRowCnt = nrows;
        this.fColCnt = ncols;

        int newSize = fRowCnt * fColCnt;
        boolean[] oldData = elementData;

        if (oldfColCnt == fColCnt) {

            // no need to reload elements.
            if (fRowCnt <= oldfRowCnt) {
                return;
            }

            // have to allocate memory.
            elementData = new boolean[newSize];

            // copy the old data.
            System.arraycopy(oldData, 0, elementData, 0, oldSize);

            // no need to pad. 0.0 is automaticly padded.
            // for (int i = oldSize; i < newSize; i++)
            //     elementData[i] = 0.0;
        } else {
            elementData = new boolean[newSize];

            for (int i = 0; i < oldfRowCnt; i++) {
                System.arraycopy(oldData, i * oldfColCnt, elementData, i * fColCnt, oldfColCnt);
            }
        }
    }

    /**
     * Returns the number of rows in this matrix.
     *
     * @return number of rows in this matrix
     */
    public int getNumRow() {
        return fRowCnt;
    }

    /**
     * Returns the number of colmuns in this matrix.
     *
     * @return number of columns in this matrix
     */
    public int getNumCol() {
        return fColCnt;
    }

    public int getDim() {
        return getNumRow() * getNumCol();
    }

    /**
     * Retrieves the value at the specified row and column of this matrix.
     *
     * @param row    the row number to be retrieved (zero indexed)
     * @param column the column number to be retrieved (zero indexed)
     * @return the value at the indexed element
     */
    public boolean getElement(int row, int column) {

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        if (fColCnt <= column) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " > matrix's fColCnt:"
                    + fColCnt);
        }

        if (column < 0) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " < 0");
        }

        return elementData[row * fColCnt + column];
    }

    public boolean isTrue(int row, int column) {
        return getElement(row, column);
    }

    /**
     * Modifies the value at the specified row and column of this matrix.
     *
     * @param row    the row number to be modified (zero indexed)
     * @param column the column number to be modified (zero indexed)
     * @param value  the new matrix element value
     */
    public void setElement(int row, int column, boolean value) {

        checkImmutable();

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        if (fColCnt <= column) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " > matrix's fColCnt:"
                    + fColCnt);
        }

        if (column < 0) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " < 0");
        }

        elementData[row * fColCnt + column] = value;
    }

    /**
     * a safe copy is returned.
     */
    public boolean[] getRow(int row) {

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        boolean[] ret = new boolean[fColCnt];

        System.arraycopy(elementData, row * fColCnt, ret, 0, fColCnt);

        return ret;
    }

    /**
     * Copy the values from the array into the specified row of this
     * matrix.
     *
     * @param row   the row of this matrix into which the array values
     *              will be copied.
     * @param array the source array
     */
    public void setRow(int row, float array[]) {

        checkImmutable();

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        if (array.length < fColCnt) {
            throw new ArrayIndexOutOfBoundsException("array length:" + array.length
                    + " < matrix's fColCnt=" + fColCnt);
        }

        System.arraycopy(array, 0, elementData, row * fColCnt, fColCnt);
    }

    /**
     * Copy the values from the array into the specified column of this
     * matrix.
     *
     * @param col   the column of this matrix into which the array values
     *              will be copied.
     * @param array the source array
     */
    public void setColumn(int col, boolean array[]) {

        checkImmutable();

        if (fColCnt <= col) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > matrix's fColCnt="
                    + fColCnt);
        }

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        if (array.length < fRowCnt) {
            throw new ArrayIndexOutOfBoundsException("array length:" + array.length
                    + " < matrix's fRowCnt:" + fRowCnt);
        }

        for (int i = 0; i < fRowCnt; i++) {
            elementData[i * fColCnt + col] = array[i];
        }
    }

    /**
     * Returns a string that contains the values of this Matrix.
     *
     * @return the String representation
     */
    public String toString() {

        String nl = System.getProperty("line.separator");
        StringBuffer out = new StringBuffer("[");

        out.append(nl);

        for (int i = 0; i < fRowCnt; i++) {
            out.append("  [");

            for (int j = 0; j < fColCnt; j++) {
                if (0 < j) {
                    out.append("\t");
                }

                out.append(elementData[i * fColCnt + j]);
            }

            if (i + 1 < fRowCnt) {
                out.append("]");
                out.append(nl);
            } else {
                out.append("] ]");
            }
        }

        return out.toString();
    }

    /*@todo dont know how to handle ?
   public int hashCode() {
       int hash = 0;
       for (int i = 0; i < fRowCnt*fColCnt; i++) {
           long bits = Double.doubleToLongBits(elementData[i]);
           hash ^= (int)(bits ^ (bits >> 32));
       }
       return hash;
   }
   */

    /**
     * Returns true if all of the data members of Matrix4d m1 are
     * equal to the corresponding data members in this Matrix4d.
     *
     * @param m1 The matrix with which the comparison is made.
     * @return true or false
     */
    public boolean equals(BooleanMatrix m1) {

        if (m1 == null) {
            return false;
        }

        if (m1.fRowCnt != fRowCnt) {
            return false;
        }

        if (m1.fColCnt != fColCnt) {
            return false;
        }

        for (int i = 0; i < fRowCnt; i++) {
            for (int j = 0; j < fColCnt; j++) {
                if (elementData[i * fColCnt + j] != m1.elementData[i * fColCnt + j]) {
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * Returns true if the Object o1 is of type Matrix and all of the data
     * members of t1 are equal to the corresponding data members in this
     * Matrix.
     *
     * @param o1 the object with which the comparison is made.
     */
    public boolean equals(Object o1) {
        return (o1 != null) && (o1 instanceof Matrix);
    }

    public void setImmutable() {
        fImmuted = true;
    }

    public boolean isImmutable() {
        return fImmuted;
    }

    private void checkImmutable() {

        if (fImmuted) {
            throw new ImmutedException();
        }
    }

}    // End BooleanMatrix

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
