/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.math;


import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;

import java.awt.*;


/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ColorVector extends Vector {

    private static final transient Logger klog = XLogger.getLogger(ColorVector.class);

    /**
     * 2 color scheme - for P values for instance - white to green
     * The other two colors (blue and red) vary from 255 (min green) to 0 (max green)
     * to effect this caling.
     * (red == blue always and b/w 255 and 0, green == 0 always)
     */
    public static final int RED = 0;
    public static final int GREEN = 1;
    public static final int BLUE = 2;
    private final float fMax;
    private final float fMin;
    private int fType = 0;

    /**
     * type one of red , green or blue
     * Values ARE not shared
     */
    public ColorVector(Vector vector, int type) {

        super(vector);

        this.fMax = max();
        this.fMin = min();
        this.fType = type;
    }

    public Color getColor(int index) {

        float val = getElement(index);
        float c = ((val - fMin) / (fMax - fMin)) * 1;

        //log.debug("color c = " + c);
        if (fType == RED) {
            return new Color(1, c, c);
        } else if (fType == GREEN) {
            return new Color(c, 1, c);
        } else if (fType == BLUE) {
            return new Color(c, c, 1);
        } else {
            throw new RuntimeException("Unknown type: " + fType);
        }
    }
}    // End ColorVector

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
