/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.math;


import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;


/**
 * An object encapsulating a double element of an array.
 * Captures it (double) value and position in the array.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class DoubleElement {

    public int fIndex;
    public double fValue;
    private static final Logger klog = XLogger.getLogger(DoubleElement.class);
    private static final double COMP_PREC = 0.0000000001;

    /**
     * Class Constructor.
     */
    public DoubleElement(int index, double value) {
        this.fIndex = index;
        this.fValue = value;
    }

    /**
     * Class Constructor.
     * Index and value are parsed from the spec strings
     */
    public DoubleElement(String index, String value) {
        this.fIndex = Integer.parseInt(index);
        this.fValue = Double.parseDouble(value);
    }

    /**
     * a list with DoubleElement as elements
     */
    public static void sort(final SortMode sort, final Order order, final List felist) {

        Collections.sort(felist, new DoubleElement.DoubleElementComparator(sort, order.isAscending()));

    }

    public static DoubleElement[] sort(final SortMode sort, final Order order, final DoubleElement[] dels) {
        List list = DoubleElement.toList(dels);
        sort(sort, order, list);

        return (DoubleElement[]) list.toArray(new DoubleElement[list.size()]);
    }

    /**
     * @param delist A list with DoubleElement as elements
     *               Typically used after a sort.
     */
    public static int[] indexArray(List delist) {

        //log.debug("## " + delist);
        int[] indexes = new int[delist.size()];

        for (int i = 0; i < delist.size(); i++) {
            indexes[i] = ((DoubleElement) delist.get(i)).fIndex;
        }

        return indexes;
    }

    /**
     * @param delist A list with DoubleElement as elements
     * @return
     */
    public static double[] valueArray(List delist) {

        double[] values = new double[delist.size()];

        for (int i = 0; i < delist.size(); i++) {
            values[i] = ((DoubleElement) delist.get(i)).fValue;
        }

        return values;
    }

    /**
     * array of indices of each del
     */
    public static int[] indexArray(DoubleElement[] dels) {

        //log.debug("## " + delist);
        int[] indexes = new int[dels.length];

        for (int i = 0; i < dels.length; i++) {
            indexes[i] = dels[i].fIndex;
        }

        return indexes;
    }

    /**
     * array of values of each del
     */
    public static double[] valueArray(DoubleElement[] dels) {

        double[] values = new double[dels.length];

        for (int i = 0; i < dels.length; i++) {
            values[i] = dels[i].fValue;
        }

        return values;
    }


    public static DoubleElement[] createDoubleElementsFromValues(double[] values) {
        DoubleElement[] des = new DoubleElement[values.length];

        for (int i = 0; i < values.length; i++) {
            des[i] = new DoubleElement(i, values[i]);
        }

        return des;

    }

    public static DoubleElement[] createAndOrderDoubleElementsFromValues(SortMode sort, Order order, double[] values) {

        DoubleElement[] dels = createDoubleElementsFromValues(values);
        return sort(sort, order, dels);

    }

    public static List toList(DoubleElement[] dels) {
        List list = new ArrayList(dels.length);
        for (int i = 0; i < dels.length; i++) {
            list.add(dels[i]);
        }

        return list;
    }

    public static float[] toFloats(DoubleElement[] dels) {
        float[] floats = new float[dels.length];
        for (int i = 0; i < dels.length; i++) {
            floats[i] = (float) dels[i].fValue;
        }

        return floats;
    }

    public static float[] toFloats(List dels) {
        float[] floats = new float[dels.size()];
        for (int i = 0; i < dels.size(); i++) {
            floats[i] = (float) ((DoubleElement) dels.get(i)).fValue;
        }

        return floats;
    }

    static class DoubleElementComparator implements java.util.Comparator {

        private final SortMode fSort;
        private final Boolean fAscending;

        public DoubleElementComparator(SortMode sort, boolean ascending) {
            this.fSort = sort;
            this.fAscending = ascending;
        }

        public int compare(Object obj1, Object obj2) {

            if ((obj1 == null) && (obj2 == null)) {
                return 0;     // cant compare
            } else if (obj1 == null) {
                return fAscending ? -1 : +1;    // null is always least
            } else if (obj2 == null) {
                return fAscending ? +1 : -1;    // null is always least
            }

            double d1 = ((DoubleElement) obj1).fValue;
            double d2 = ((DoubleElement) obj2).fValue;

            if (Double.isNaN(d1) && Double.isNaN(d2)) {
                return 0;
            } else if (Double.isNaN(d1)) {
                return fAscending ? -1 : +1;
            } else if (Double.isNaN(d2)) {
                return fAscending ? +1 : -1;
            }

            if (d1 < d2) {
                return fAscending ? -1 : +1;
            } else if (d1 > d2) {
                return fAscending ? +1 : -1;
            } else
            return 0;
        }



    }
}

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
