/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.math;


import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.utils.ImmutedException;
import org.apache.log4j.Logger;

/**
 * Object representing a continuous FuzzyRange of floats..
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class FuzzyRange implements Range {

    private static final Logger klog = XLogger.getLogger(FuzzyRange.class);
    private double fMin;
    private double fMax;
    private double fFuzz;

    private int fCount;

    private boolean fImmutable;

    public Range cloneShallow() {
        return new FuzzyRange(getMin(), getMax(), getFuzz());
    }

    /**
     * Class Constructor.
     */
    public FuzzyRange(double min, double max) {
        init(min, max);
    }

    /**
     * Class Constructor.
     */
    public FuzzyRange(double min, double max, double fuzz) {
        init(min - fuzz, max + fuzz);
        this.fFuzz = fuzz;
    }

    private void init(double min, double max) {
        if (max < min) {
            throw new IllegalArgumentException("max: " + max + " is less than min: " + min);
        }

        this.fMin = min;
        this.fMax = max;
    }

    public void setImmutable() {
        fImmutable = true;
    }

    private void checkImmutable() {
        if (fImmutable) {
            throw new ImmutedException();
        }
    }

    public void increment() {
        checkImmutable();
        fCount++;
    }

    public int getCount() {
        return fCount;
    }

    /**
     * always inclusive
     */
    public boolean isInRange(double valueInclusive) {

        if ((valueInclusive >= fMin) && (valueInclusive <= fMax)) {
            return true;
        } else {
            return false;
        }
    }

    public double getMax() {
        return fMax;
    }

    public double getMin() {
        return fMin;
    }

    public double getFuzz() {
        return fFuzz;
    }

    public double getSpread() {
        return fMax - fMin + 1;    // because both are inclusive
    }

    public double getMean() {
        return ((fMin + fMax) / 2.0d);
    }

    public String getName() {
        return new StringBuffer().append(fMin).append(':').append(fMax).toString();
    }

    /**
     * truncates the floats into ints
     */
    public String getIntFuzzyRangeName() {
        return new StringBuffer().append((int) fMin).append('-').append((int) fMax).toString();
    }

    public String getSummary() {

        StringBuffer buf = new StringBuffer("");
        buf.append("Data Max: ").append(getMax());
        buf.append("Data Min: ").append(getMin());
        buf.append("Spread  : ").append(getSpread());
        buf.append("Fuzz  : ").append(getFuzz());
        buf.append("Count   :  ").append(getCount());

        return buf.toString();
    }

}    // End FuzzyRange

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
