/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.math;


import edu.mit.broad.genome.utils.containers.ClassNamePair;


/**
 * enum construct
 */
public class GraphMode {

    public static final GraphMode LINEAR = new GraphMode("Linear");
    public static final GraphMode LOG = new GraphMode("Log");

    /**
     * @maint manually keep array in sync with declarations above and below
     */
    public static final GraphMode[] ALL = new GraphMode[]{LINEAR, LOG};
    public static final ClassNamePair[] ALL_CNP = new ClassNamePair[]{
            new ClassNamePair(LINEAR.getClass(),
                    LINEAR.fType),
            new ClassNamePair(LOG.getClass(), LOG.fType),};
    private final String fType;

    /**
     * @maint manually keep array in sync with declarations above and below
     * <p/>
     * Privatized class constructor.
     * <p/>
     * <p/>
     * Privatized class constructor.
     */

    /**
     * Privatized class constructor.
     */
    private GraphMode(String type) {
        this.fType = type;
    }

    public String toString() {
        return fType;
    }

    public boolean isLinear() {

        if (this.fType.equalsIgnoreCase(LINEAR.fType)) {
            return true;
        }

        return false;
    }

    public boolean IsLog() {

        if (this.fType.equalsIgnoreCase(LOG.fType)) {
            return true;
        }

        return false;
    }

    public boolean equals(Object obj) {

        if (obj instanceof GraphMode) {
            if (((GraphMode) obj).fType.equals(this.fType)) {
                return true;
            }
        }

        return false;
    }

    public int hashCode() {
        return fType.hashCode();
    }

    /**
     * a lookup metod for dir
     */
    public static GraphMode lookup(Object obj) {

        if (obj == null) {
            throw new NullPointerException("Null dir not allowed");
        }

        if (obj instanceof GraphMode) {
            return (GraphMode) obj;
        }

        if (obj instanceof String) {
            if (obj.toString().equalsIgnoreCase(LINEAR.fType)) {
                return LINEAR;
            } else if (obj.toString().equalsIgnoreCase(LOG.fType)) {
                return LOG;
            }
        }

        throw new IllegalArgumentException("Unable to lookup GraphMode String: " + obj);
    }
}    // End GraphMode

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
