/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.math;

import edu.mit.broad.genome.MismatchedSizeException;
import edu.mit.broad.genome.objects.GeneSet;

/**
 * @author Aravind Subramanian
 *         Date: Jun 22, 2004
 *         Time: 7:26:09 PM
 */
public class HyperGeometricStruc {

    private double fPValue;
    private int fOverlapSize;
    private int fQuerySetSize;
    private int fDbSize;
    private GeneSet fOverlapSet;

    /**
     * Class constructor
     *
     * @param pvalue
     * @param overlapNum
     * @param querySetSize
     * @param dbSize
     */
    public HyperGeometricStruc(double pvalue, int overlapNum, int querySetSize, int dbSize, GeneSet overlapSet) {

        if (overlapSet == null) {
            throw new IllegalArgumentException("Parameter overlapSet cannot be null");
        }

        if (overlapSet.getNumMembers() != overlapNum) {
            throw new MismatchedSizeException("OverlapSet size", overlapSet.getNumMembers(), "OverlapNum", overlapNum);
        }

        if (overlapSet.getNumMembers() == 0) {
            throw new IllegalArgumentException();
        }

        this.fPValue = pvalue;
        this.fOverlapSize = overlapNum;
        this.fQuerySetSize = querySetSize;
        this.fDbSize = dbSize;
        this.fOverlapSet = overlapSet;

    }

    public double getPValue() {
        return fPValue;
    }

    public int getOverlap() {
        return fOverlapSize;
    }

    public int getQuerySize() {
        return fQuerySetSize;
    }

    public int getDbSize() {
        return fDbSize;
    }

    public GeneSet getOverlapSet() {
        return fOverlapSet;
    }

} // End class HyperGeometricStruc
