/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.math;


import edu.mit.broad.genome.TraceUtils;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.distrib.RangeFactory;
import edu.mit.broad.genome.objects.AbstractObject;
import edu.mit.broad.genome.objects.LabelledVector;
import edu.mit.broad.genome.utils.ArrayUtils;
import edu.mit.broad.genome.utils.ImmutedException;
import gnu.trove.TFloatArrayList;
import gnu.trove.TFloatHashSet;
import org.apache.log4j.Logger;

import java.util.Arrays;
import java.util.List;
import java.util.ArrayList;


/**
 * all methods that chnage the values of teh matrix sgouldfirst check the imm/mutablity state
 * of the matrix.
 * <p/>
 * * Lost of code/ideas copied form Kenji Hiranabe, Eiwa System Management, Inc.
 * GMatrix part of the javax.vecmath library (unoffical (his) impl)
 * http://java.sun.com/products/java-media/3D/forDevelopers/j3dapi/index.html
 * <p/>
 * as note: made sevral of the row related methods unfinal as need to override
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class Matrix extends AbstractObject implements IMatrix {

    /**
     * (javax.vecmath note)
     * Implementation note:
     * The size of the matrix does NOT automaticly grow.
     * It does only when setSize is called. I believe this is the spec
     * and makes less confusion, less user bugs.
     */

    /**
     * @maint IMP IMP IMP IMP IMP
     * If any fields are added to this class, make sure to update
     * the constructors and the set(Matrix m) method
     * Also, make sure to do an immutability check if needed.
     * <p/>
     * The data of the Matrix.(1-D array. The (i,j) element is stored in elementData[i*fColCnt + j])
     * Intentionally made protected so that alg methods and such in this package have direct (aka quiok) access
     * <p/>
     * <p/>
     * The data of the Matrix.(1-D array. The (i,j) element is stored in elementData[i*fColCnt + j])
     * Intentionally made protected so that alg methods and such in this package have direct (aka quiok) access
     */

    /**
     * The data of the Matrix.(1-D array. The (i,j) element is stored in elementData[i*fColCnt + j])
     * Intentionally made protected so that alg methods and such in this package have direct (aka quiok) access
     */
    protected float elementData[];

    /**
     * The number of rows in this matrix.
     */
    private int fRowCnt;

    /**
     * The number of columns in this matrix.
     */
    private int fColCnt;

    /**
     * Flag for whether this Matrix can have its values altered or not
     */
    private boolean fImmuted;

    /**
     * For logging support
     */
    private static final transient Logger klog = XLogger.getLogger(Matrix.class);

    /**
     * Class Constructor.
     * Must call set() after this form of the constructor.
     */
    public Matrix() {
    }

    /**
     * Class Constructor.
     * Constructs an fRowCnt by fColCnt all zero matrix. (as change)
     * Note that even though row and column numbering begins with
     * zero, fRowCnt and fColCnt will be one larger than the maximum
     * possible matrix index values.
     *
     * @param nrows number of rows in this matrix.
     * @param ncols number of columns in this matrix.
     */
    public Matrix(int nrows, int ncols) {
        init(nrows, ncols);
    }

    static int name_cnt;

    private void init(int nrows, int ncols) {
        super.initialize("matrix_" + name_cnt++);
        if (nrows < 0) {
            throw new NegativeArraySizeException(nrows + " < 0");
        }

        if (ncols < 0) {
            throw new NegativeArraySizeException(ncols + " < 0");
        }

        this.fRowCnt = nrows;
        this.fColCnt = ncols;
        this.elementData = new float[fRowCnt * fColCnt];

        setZero();
    }

    public String getQuickInfo() {
        return getNumRow() + " x " + getNumCol();
    }

    /**
     * Class Constructor.
     * Data is copied over from specified float array.
     * Constructs an fRowCnt by fColCnt matrix initialized to the values
     * in the matrix array.  The array values are copied in one row at
     * a time in row major fashion.  The array should be at least
     * fRowCnt*fColCnt in length.
     * Note that even though row and column numbering begins with
     * zero, fRowCnt and fColCnt will be one larger than the maximum
     * possible matrix index values.
     *
     * @param nrows       number of rows in this matrix.
     * @param ncols       number of columns in this matrix.
     * @param elementData a 1D array that specifies a matrix in row major fashion
     */
    public Matrix(int nrows, int ncols, float[] elementData) {

        if (nrows < 0) {
            throw new NegativeArraySizeException(nrows + " < 0");
        }

        if (ncols < 0) {
            throw new NegativeArraySizeException(ncols + " < 0");
        }

        this.fRowCnt = nrows;
        this.fColCnt = ncols;
        this.elementData = new float[fRowCnt * fColCnt];

        set(elementData);
    }

    public Matrix(int nrows, int ncols, TFloatArrayList floats) {

        if (nrows < 0) {
            throw new NegativeArraySizeException(nrows + " < 0");
        }

        if (ncols < 0) {
            throw new NegativeArraySizeException(ncols + " < 0");
        }

        this.fRowCnt = nrows;
        this.fColCnt = ncols;
        this.elementData = floats.toNativeArray();
    }

    /**
     * Class constructor
     *
     * @param bm
     * @param setTrueAsOne
     */
    public Matrix(BooleanMatrix bm, boolean setTrueAsOne) {

        if (bm == null) {
            throw new IllegalArgumentException("Parameter bm cannot be null");
        }

        this.fRowCnt = bm.getNumRow();
        this.fColCnt = bm.getNumCol();
        this.elementData = new float[bm.elementData.length];

        int trueval;
        int falseval;

        if (setTrueAsOne) {
            trueval = 1;
            falseval = 0;
        } else {
            trueval = 0;
            falseval = 1;
        }

        for (int i = 0; i < bm.elementData.length; i++) {
            if (bm.elementData[i]) {
                this.elementData[i] = trueval;
            } else {
                this.elementData[i] = falseval;
            }
        }

    }

    /**
     * Class Constructor.
     * Data is NOT shared
     *
     * @param matrixA
     * @param matrixB
     */
    public Matrix(final Matrix matrixA, final Matrix matrixB, final boolean addAsRows) {
        this(new Matrix[]{matrixA, matrixB}, addAsRows);
    }

    /**
     * Class Constructor.
     * Data is NOT shared
     * matrices must be of same number of columns
     * (nrows can be arb - thats the point!)
     */
    public Matrix(final Matrix[] matrices, final boolean addAsRows) {

        if (matrices.length == 0) {
            throw new IllegalArgumentException("Param matrices cannot be zero length");
        }

        if (addAsRows) {
            int c = enforceSameNumOfCols(matrices);
            this.fRowCnt = getTotalNumRows(matrices);
            this.fColCnt = c;
            this.elementData = new float[fRowCnt * fColCnt];

            int pos = 0;

            for (int i = 0; i < matrices.length; i++) {
                System.arraycopy(matrices[i].elementData, 0, elementData, pos,
                        matrices[i].elementData.length);

                pos += matrices[i].elementData.length;
            }
        } else {
            int nrows = enforceSameNumOfRows(matrices);
            int ncols = getTotalNumCols(matrices);
            Matrix combo = new Matrix(nrows, ncols);
            int coln = 0;
            for (int m = 0; m < matrices.length; m++) {
                for (int c = 0; c < matrices[m].getNumCol(); c++) {
                    combo.setColumn(coln++, matrices[m].getColumn(c));
                }
            }
            init(combo, true);
        }
    }

    /**
     * Class Constructor.
     * Constructs a new Matrix and copies the initial values
     * from the parameter matrix.
     *
     * @param matrix the source of the initial values of the new Matrix
     *               <p/>
     *               as added: if shared values, then this matrix shares the elements with the specified amtric
     *               else the array is systemarrycopied
     */
    public Matrix(Matrix matrix, boolean shareValues) {
        init(matrix, shareValues);
    }

    private void init(Matrix matrix, boolean shareValues) {

        if (matrix == null) {
            throw new IllegalArgumentException("Param matrix cannot be null");
        }

        this.fRowCnt = matrix.fRowCnt;
        this.fColCnt = matrix.fColCnt;

        // DONT thats the whole point! (often)
        //this.fImmutable = matrix.fImmutable;

        if (shareValues) {
            this.fImmuted = matrix.fImmuted;
        } else {
            this.fImmuted = false;
        }

        if (shareValues) {
            this.elementData = matrix.elementData;
        } else {
            int newSize = fRowCnt * fColCnt;

            this.elementData = new float[newSize];

            System.arraycopy(matrix.elementData, 0, elementData, 0, newSize);
        }
    }

    public Matrix(TFloatHashSet[] floats, boolean addAsRows, boolean allowUnequalLengths) {

        Vector[] vss = new Vector[floats.length];
        for (int i = 0; i < floats.length; i++) {
            vss[i] = new Vector(floats[i].toArray());
        }

        initV(vss, addAsRows, allowUnequalLengths);

    }

    /**
     * Class Constructor.
     * Vectors are added 0 to n-1 top to bottom
     * Vectors must be of equal length
     * Data is not shared
     */
    public Matrix(Vector[] vectors, boolean addAsRows, boolean allowUnequalLengths) {
        initV(vectors, addAsRows, allowUnequalLengths);
    }

    // core init method for vector based construction
    private void initV(Vector[] vectors, boolean addAsRows, boolean allowUnequalLengths) {
        if (addAsRows) {
            int ncols = XMath.max(vectors);
            init(vectors.length, ncols);
            for (int i = 0; i < vectors.length; i++) {
                if (!allowUnequalLengths && vectors[i].getSize() != ncols) {
                    throw new RuntimeException("Unequal col len: " + ncols + " and "
                            + vectors[i].getSize());
                }

                this.setRow(i, vectors[i]);
            }
        } else {
            int nrows = XMath.max(vectors);
            init(nrows, vectors.length);
            for (int i = 0; i < vectors.length; i++) {
                if (!allowUnequalLengths && vectors[i].getSize() != nrows) {
                    throw new RuntimeException("Unequal row len: " + nrows + " and "
                            + vectors[i].getSize());
                }

                this.setColumn(i, vectors[i], !allowUnequalLengths); // @note NOT enrocing bounds as inout
            }
        }
    }

    public Matrix(List vectors, boolean addAsRows, boolean allowUnequalLengths) {
        this((Vector[]) vectors.toArray(new Vector[vectors.size()]), addAsRows, allowUnequalLengths);
    }

    public Matrix(LabelledVector[] lvs, boolean addAsRows, boolean allowUnequalLengths) {
        this(LabelledVector.toVectors(lvs, false), addAsRows, allowUnequalLengths);
    }

    /**
     * The cloned matrix is NOT immutable even if the matrix it was cloned from was.
     */
    public Matrix cloneDeep() {
        klog.debug("Making cloneDeep of Matrix");
        TraceUtils.showTrace();
        return new Matrix(this, false);
    }

    /**
     * Fill the matrix with specified value
     *
     * @param val
     */
    public void fill(int val) {
        checkImmutable();
        for (int i = 0; i < elementData.length; i++) {
            elementData[i] = val;
        }
    }

    public void fill(float val) {
        checkImmutable();
        for (int i = 0; i < elementData.length; i++) {
            elementData[i] = val;
        }
    }

    public ArrayList toList() {
        ArrayList dist=new ArrayList();
        for(int i=0; i<fRowCnt; i++){
            ArrayList row=new ArrayList();
            for(int j=0; j<fColCnt; j++){
                row.add(new Float(elementData[i * fColCnt + j]));
            }
            dist.add(row);
        }
        return dist;
    }

    /**
     * Copies a sub-matrix derived from this matrix into the target matrix.
     * The upper left of the sub-matrix is located at (rowSource, colSource);
     * the lower right of the sub-matrix is located at
     * (lastRowSource,lastColSource).  The sub-matrix is copied into the
     * the target matrix starting at (rowDest, colDest).
     *
     * @param rowSource the top-most row of the sub-matrix
     * @param colSource the left-most column of the sub-matrix
     * @param numRow    the number of rows in the sub-matrix
     * @param numCol    the number of columns in the sub-matrix
     * @param rowDest   the top-most row of the position of the copied sub-matrix
     *                  within the target matrix
     * @param colDest   the left-most column of the position of the copied sub-matrix
     *                  within the target matrix
     * @param target    the matrix into which the sub-matrix will be copied
     */
    public void copySubMatrix(int rowSource, int colSource, int numRow, int numCol,
                              int rowDest, int colDest, Matrix target) {

        if ((rowSource < 0) || (colSource < 0) || (rowDest < 0) || (colDest < 0)) {
            throw new ArrayIndexOutOfBoundsException("rowSource,colSource,rowDest,colDest < 0.");
        } else if ((fRowCnt < numRow + rowSource) || (fColCnt < numCol + colSource)) {
            throw new ArrayIndexOutOfBoundsException("Source Matrix too small.");
        } else if ((target.fRowCnt < numRow + rowDest) || (target.fColCnt < numCol + colDest)) {
            throw new ArrayIndexOutOfBoundsException("Target Matrix too small.");
        }

        for (int i = 0; i < numRow; i++) {
            for (int j = 0; j < numCol; j++) {
                target.elementData[(i + rowDest) * fColCnt + (j + colDest)] =
                        elementData[(i + rowSource) * fColCnt + (j + colSource)];
            }
        }
    }

    /**
     * Changes the size of this matrix dynamically.  If the size is increased
     * no data values will be lost.  If the size is decreased, only those data
     * values whose matrix positions were eliminated will be lost.
     *
     * @param nrows number of desired rows in this matrix
     * @param ncols number of desired columns in this matrix
     */
    public void setSize(int nrows, int ncols) {

        checkImmutable();

        if ((fRowCnt < 0) || (fColCnt < 0)) {
            throw new NegativeArraySizeException("nrows or ncols < 0");
        }

        int oldfRowCnt = this.fRowCnt;
        int oldfColCnt = this.fColCnt;
        int oldSize = this.fRowCnt * this.fColCnt;

        this.fRowCnt = nrows;
        this.fColCnt = ncols;

        int newSize = fRowCnt * fColCnt;
        float[] oldData = elementData;

        if (oldfColCnt == fColCnt) {

            // no need to reload elements.
            if (fRowCnt <= oldfRowCnt) {
                return;
            }

            // have to allocate memory.
            elementData = new float[newSize];

            // copy the old data.
            System.arraycopy(oldData, 0, elementData, 0, oldSize);

            // no need to pad. 0.0 is automaticly padded.
            // for (int i = oldSize; i < newSize; i++)
            //     elementData[i] = 0.0;
        } else {
            elementData = new float[newSize];

            setZero();

            for (int i = 0; i < oldfRowCnt; i++) {
                System.arraycopy(oldData, i * oldfColCnt, elementData, i * fColCnt, oldfColCnt);
            }
        }
    }

    /**
     * Sets the value of this matrix to the values found in the array parameter.
     * The values are copied in one row at a time, in row major
     * fashion.  The array should be at least equal in length to
     * the number of matrix rows times the number of matrix columns
     * in this matrix.
     *
     * @param matrix the row major source array
     */
    public void set(float matrix[]) {

        checkImmutable();

        int size = fRowCnt * fColCnt;

        System.arraycopy(matrix, 0, elementData, 0, size);
    }

    /**
     * Sets the value of this matrix to the values found in matrix m1.
     *
     * @param m1 the source matrix
     */
    public void set(Matrix m1, boolean sharedvalues) {

        checkImmutable();

        // This implementation is in 'no automatic size grow' policy.
        // When size mismatch, exception will be thrown from the below.
        if ((m1.fRowCnt < fRowCnt) || (m1.fColCnt < fColCnt)) {
            throw new ArrayIndexOutOfBoundsException("m1 smaller than this matrix");
        }

        if (sharedvalues) {
            this.elementData = m1.elementData;
            this.fRowCnt = m1.fRowCnt;
            this.fColCnt = m1.fColCnt;
            this.fImmuted = m1.fImmuted;
        } else {
            System.arraycopy(m1.elementData, 0, elementData, 0, fRowCnt * fColCnt);
        }
    }

    /**
     * Returns the number of rows in this matrix.
     *
     * @return number of rows in this matrix
     */
    public int getNumRow() {
        return fRowCnt;
    }

    /**
     * Returns the number of colmuns in this matrix.
     *
     * @return number of columns in this matrix
     */
    public int getNumCol() {
        return fColCnt;
    }

    public int getDim() {
        return getNumRow() * getNumCol();
    }

    public String getDimS() {
        return getNumRow() + "*" + getNumCol();
    }

    /**
     * Retrieves the value at the specified row and column of this matrix.
     *
     * @param row    the row number to be retrieved (zero indexed)
     * @param column the column number to be retrieved (zero indexed)
     * @return the value at the indexed element
     */
    public float getElement(int row, int column) {

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        if (fColCnt <= column) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " > matrix's fColCnt:"
                    + fColCnt);
        }

        if (column < 0) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " < 0");
        }

        return elementData[row * fColCnt + column];
    }

    /**
     * When you dont care about rows and columns
     * A bit faster than doing the other way
     *
     * @param absoluteIndex
     * @return
     */
    public float getElement(int absoluteIndex) {
        return elementData[absoluteIndex];
    }

    public void setElement(final int absoluteIndex, final float value) {
        checkImmutable();
        elementData[absoluteIndex] = value;
    }

    public int getAbsoluteIndex(final int row, final int column) {
        return row * fColCnt + column;
    }

    public float[] toArray() {
        float[] dest = new float[elementData.length];
        System.arraycopy(elementData, 0, dest, 0, elementData.length);
        return dest;
    }

    public double[] toArrayD() {
        double[] dest = new double[elementData.length];
        for (int i = 0; i < elementData.length; i++) {
            dest[i] = elementData[i];
        }

        return dest;
    }

    public Vector toVector() {
        float[] dest = new float[elementData.length];
        System.arraycopy(elementData, 0, dest, 0, elementData.length);
        return new Vector(dest);
    }

    public int getAbsoluteSize() {
        return elementData.length;
    }

    /**
     * Modifies the value at the specified row and column of this matrix.
     *
     * @param row    the row number to be modified (zero indexed)
     * @param column the column number to be modified (zero indexed)
     * @param value  the new matrix element value
     */
    public void setElement(int row, int column, float value) {

        checkImmutable();

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        if (fColCnt <= column) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " > matrix's fColCnt:"
                    + fColCnt);
        }

        if (column < 0) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " < 0");
        }

        elementData[row * fColCnt + column] = value;
    }

    public void setElementByIndexPos(int indexPos, float value) {

        checkImmutable();

        if (elementData.length <= indexPos) {
            throw new ArrayIndexOutOfBoundsException("row:" + indexPos + " > matrix's length:"
                    + elementData.length);
        }

        if (indexPos < 0) {
            throw new ArrayIndexOutOfBoundsException("indexPos:" + indexPos + " < 0");
        }

        elementData[indexPos] = value;
    }

    /**
     * A safe copy is returned.
     * but note efficient as directly copied into Vectors array
     */
    public Vector getRowV(int row) {

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        Vector vector = new Vector(fColCnt);

        System.arraycopy(elementData, row * fColCnt, vector.elementData, 0, fColCnt);

        return vector;
    }

    /**
     * A safe copy is returned.
     */
    public float[] getRow(int row) {

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        float[] ret = new float[fColCnt];

        System.arraycopy(elementData, row * fColCnt, ret, 0, fColCnt);

        return ret;
    }

    /**
     * Places the values of the specified column into the array parameter.
     *
     * @param col   the target column number
     * @param array the array into which the column values will be placed
     */
    private void _getColumnInPlace(int col, float array[]) {

        if (fColCnt <= col) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > matrix's fColCnt:"
                    + fColCnt);
        }

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        if (array.length < fRowCnt) {
            throw new ArrayIndexOutOfBoundsException("array.length:" + array.length
                    + " < matrix's fRowCnt=" + fRowCnt);
        }

        for (int i = 0; i < fRowCnt; i++) {
            array[i] = elementData[i * fColCnt + col];
        }
    }

    /**
     * Data is NOt shared
     *
     * @param col
     * @return
     */
    public Vector getColumnV(int col) {

        if (fColCnt <= col) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > matrix's fColCnt:"
                    + fColCnt);
        }

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        Vector vector = new Vector(fRowCnt);

        for (int i = 0; i < fRowCnt; i++) {
            vector.setElement(i, elementData[i * fColCnt + col]);
        }

        return vector;
    }

    /**
     * A safe copy is returned.
     */
    public float[] getColumn(int col) {

        if (fColCnt <= col) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > matrix's fColCnt:"
                    + fColCnt);
        }

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        float[] ret = new float[fRowCnt];

        for (int i = 0; i < fRowCnt; i++) {
            ret[i] = elementData[i * fColCnt + col];
        }

        return ret;
    }

    private Vector fColMaxes;

    public Vector getColumnMaxes() {

        if (fColMaxes == null) {
            fColMaxes = new Vector(getNumCol());
            for (int c = 0; c < getNumCol(); c++) {
                fColMaxes.setElement(c, getColumnV(c).max());
            }
        }

        return fColMaxes;
    }

    private Vector fColMins;

    public Vector getColumnMins() {

        if (fColMins == null) {
            fColMins = new Vector(getNumCol());
            for (int c = 0; c < getNumCol(); c++) {
                fColMins.setElement(c, getColumnV(c).min());
            }
        }

        return fColMins;
    }

    /**
     * Copy the values from the array into the specified row of this
     * matrix.
     *
     * @param row   the row of this matrix into which the array values
     *              will be copied.
     * @param array the source array
     */
    public void setRow(int row, float array[]) {

        checkImmutable();

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        if (array.length < fColCnt) {
            throw new ArrayIndexOutOfBoundsException("array length:" + array.length
                    + " < matrix's fColCnt=" + fColCnt);
        }

        System.arraycopy(array, 0, elementData, row * fColCnt, fColCnt);
    }

    public void setRow(int row, float value_for_all) {
        for (int c = 0; c < getNumCol(); c++) {
            setElement(row, c, value_for_all);
        }
    }


    /**
     * Data is NOT shared
     * Copy the values from the array into the specified row of this
     * matrix.
     *
     * @param row    the row of this matrix into which the vector values
     *               will be copied.
     * @param vector the source vector
     */
    public void setRow(int row, Vector vector) {

        checkImmutable();

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        int vecSize = vector.getSize();

        if (vecSize < fColCnt) {
            throw new ArrayIndexOutOfBoundsException("vector's size:" + vecSize
                    + " < matrix's fColCnt=" + fColCnt);
        }

        for (int i = 0; i < fColCnt; i++) {
            elementData[row * fColCnt + i] = vector.getElement(i);

            // if may use package friendly accessibility, would do;
            // System.arraycopy(vector.elementData, 0, elementData, row*fColCnt, fColCnt);
        }
    }

    public void setRow_permit_smaller(int row, Vector vector) {

        checkImmutable();

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        int vecSize = vector.getSize();

        /*
        if (vecSize < fColCnt) {

            throw new ArrayIndexOutOfBoundsException("vector's size:" + vecSize
                    + " < matrix's fColCnt=" + fColCnt);
        }
        */

        for (int i = 0; i < vecSize; i++) {
            elementData[row * fColCnt + i] = vector.getElement(i);

            // if may use package friendly accessibility, would do;
            // System.arraycopy(vector.elementData, 0, elementData, row*fColCnt, fColCnt);
        }
    }

    /**
     * Copy the values from the array into the specified column of this
     * matrix.
     *
     * @param col   the column of this matrix into which the array values
     *              will be copied.
     * @param array the source array
     */
    public void setColumn(int col, float array[]) {

        checkImmutable();

        if (fColCnt <= col) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > matrix's fColCnt="
                    + fColCnt);
        }

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        if (array.length < fRowCnt) {
            throw new ArrayIndexOutOfBoundsException("array length:" + array.length
                    + " < matrix's fRowCnt:" + fRowCnt);
        }

        for (int i = 0; i < fRowCnt; i++) {
            elementData[i * fColCnt + col] = array[i];
        }
    }

    public void setColumn(int col, int array[]) {

        checkImmutable();

        if (fColCnt <= col) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > matrix's fColCnt="
                    + fColCnt);
        }

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        if (array.length < fRowCnt) {
            throw new ArrayIndexOutOfBoundsException("array length:" + array.length
                    + " < matrix's fRowCnt:" + fRowCnt);
        }

        for (int i = 0; i < fRowCnt; i++) {
            elementData[i * fColCnt + col] = array[i];
        }
    }

    /**
     * Copy the values from the array into the specified column of this
     * matrix.
     *
     * @param col    the column of this matrix into which the vector values
     *               will be copied.
     * @param vector the source vector
     */

    public void setColumn(int col, Vector vector) {
        setColumn(col, vector, true);
    }

    private void setColumn(int col, Vector vector, boolean enforce_max_bound) {

        checkImmutable();

        if (enforce_max_bound && fColCnt <= col) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > matrix's fColCnt="
                    + fColCnt);
        }

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        int vecSize = vector.getSize();

        if (enforce_max_bound && vecSize < fRowCnt) {
            throw new ArrayIndexOutOfBoundsException("vector size:" + vecSize
                    + " < matrix's fRowCnt=" + fRowCnt);
        }

        for (int i = 0; i < vector.getSize(); i++) {
            elementData[i * fColCnt + col] = vector.getElement(i);
        }
    }

    /**
     * Multiplies the transpose of matrix m1 times the transpose of matrix m2, and places the
     * result into this.
     *
     * @param m1 The matrix on the left hand side of the multiplication
     * @param m2 The matrix on the right hand side of the multiplication
     */
    public void mulTransposeBoth(Matrix m1, Matrix m2) {

        checkImmutable();
        multiply(m2, m1);
        transpose();
    }

    /**
     * Multiplies matrix m1 times the transpose of matrix m2, and places the
     * result into this.
     */
    public void mulTransposeRight(Matrix m1, Matrix m2) {

        checkImmutable();

        if ((m1.fColCnt != m2.fColCnt) || (fRowCnt != m1.fRowCnt) || (fColCnt != m2.fRowCnt)) {
            throw new ArrayIndexOutOfBoundsException("matrices mismatch");
        }

        for (int i = 0; i < fRowCnt; i++) {
            for (int j = 0; j < fColCnt; j++) {
                float sum = (float) 0.0;

                for (int k = 0; k < m1.fColCnt; k++) {
                    sum += m1.elementData[i * m1.fColCnt + k] * m2.elementData[j * m2.fColCnt + k];
                }

                this.elementData[i * fColCnt + j] = sum;
            }
        }
    }

    /**
     * Multiplies the transpose of matrix m1 times the matrix m2, and places the
     * result into this.
     *
     * @param m1 The matrix on the left hand side of the multiplication
     * @param m2 The matrix on the right hand side of the multiplication
     */
    public void mulTransposeLeft(Matrix m1, Matrix m2) {
        transpose(m1);
        multiply(m2);
    }

    /**
     * Transposes this matrix in place.
     * <p/>
     * Note: Not a mem eff way of doing this (see note below)
     */
    public void transpose() {

        checkImmutable();

        /* orig GMatrix impl - doesnt seem to work?
        for(int i = 0; i < fRowCnt; i++) {
            for(int j = i + 1; j < fColCnt; j++) {  // note j starts from i+1
                float result = elementData[i*fColCnt + j];

                System.out.println(">> " + (i*fColCnt + j) + " >> " + (j*fColCnt + i));
                elementData[i*fColCnt + j] = elementData[j*fColCnt + i];
                elementData[j*fColCnt + i] = result;
            }
        }
        */

        //Print.out(elementData);
        float[] array = new float[elementData.length];
        int pos = 0;

        for (int c = 0; c < this.getNumCol(); c++) {
            float[] col = new float[this.getNumRow()];
            _getColumnInPlace(c, col);
            System.arraycopy(col, 0, array, pos, col.length);

            pos += col.length;
        }

        int oldrowcnt = this.fRowCnt;
        int oldcolcnt = this.fColCnt;

        this.elementData = array;
        this.fColCnt = oldrowcnt;
        this.fRowCnt = oldcolcnt;
    }

    /**
     * Places the matrix values of the transpose of matrix m1 into this matrix. <p>
     *
     * @param m1 the matrix to be transposed (but not modified)
     */
    public void transpose(Matrix m1) {

        checkImmutable();
        set(m1, false);
        transpose();
    }

    /**
     * Returns a string that contains the values of this Matrix.
     *
     * @return the String representation
     */
    public String toString() {

        String nl = System.getProperty("line.separator");
        StringBuffer out = new StringBuffer("[");

        out.append(nl);

        for (int i = 0; i < fRowCnt; i++) {
            out.append("  [");

            for (int j = 0; j < fColCnt; j++) {
                if (0 < j) {
                    out.append("\t");
                }

                out.append(elementData[i * fColCnt + j]);
            }

            if (i + 1 < fRowCnt) {
                out.append("]");
                out.append(nl);
            } else {
                out.append("] ]");
            }
        }

        return out.toString();
    }

    /**
     * Returns a hash number based on the data values in this
     * object.  Two different Matrix objects with identical data values
     * (ie, returns true for equals(Matrix) ) will return the same hash
     * number.  Two objects with different data members may return the
     * same hash value, although this is not likely.
     *
     * @return the integer hash value
     */
    public int hashCode() {

        int hash = 0;

        for (int i = 0; i < fRowCnt * fColCnt; i++) {
            long bits = Double.doubleToLongBits(elementData[i]);

            hash ^= (int) (bits ^ (bits >> 32));
        }

        return hash;
    }

    /**
     * Returns true if all of the data members of Matrix4d m1 are
     * equal to the corresponding data members in this Matrix4d.
     *
     * @param m1 The matrix with which the comparison is made.
     * @return true or false
     */
    public boolean equals(Matrix m1) {

        if (m1 == null) {
            return false;
        }

        if (m1.fRowCnt != fRowCnt) {
            return false;
        }

        if (m1.fColCnt != fColCnt) {
            return false;
        }

        for (int i = 0; i < fRowCnt; i++) {
            for (int j = 0; j < fColCnt; j++) {
                if (elementData[i * fColCnt + j] != m1.elementData[i * fColCnt + j]) {
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * Returns true if the Object o1 is of type Matrix and all of the data
     * members of t1 are equal to the corresponding data members in this
     * Matrix.
     *
     * @param o1 the object with which the comparison is made.
     */
    public boolean equals(Object o1) {
        return (o1 != null) && (o1 instanceof Matrix) && equals((Matrix) o1);
    }

    /**
     * Returns true if the L-infinite distance between this matrix and
     * matrix m1 is less than or equal to the epsilon parameter,
     * otherwise returns false. The L-infinite distance is equal to
     * MAX[i=0,1,2, . . .n ; j=0,1,2, . . .n ; abs(this.m(i,j) - m1.m(i,j)] .
     *
     * @param m1      The matrix to be compared to this matrix
     * @param epsilon the threshold value
     * @deprecated The double version of this method should be used.
     */
    public boolean epsilonEquals(Matrix m1, float epsilon) {

        if (m1.fRowCnt != fRowCnt) {
            return false;
        }

        if (m1.fColCnt != fColCnt) {
            return false;
        }

        for (int i = 0; i < fRowCnt; i++) {
            for (int j = 0; j < fColCnt; j++) {
                if (epsilon < Math.abs(elementData[i * fColCnt + j]
                        - m1.elementData[i * fColCnt + j])) {
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * Returns true if the L-infinite distance between this matrix and
     * matrix m1 is less than or equal to the epsilon parameter,
     * otherwise returns false. The L-infinite distance is equal to
     * MAX[i=0,1,2, . . .n ; j=0,1,2, . . .n ; abs(this.m(i,j) - m1.m(i,j)] .
     *
     * @param m1      The matrix to be compared to this matrix
     * @param epsilon the threshold value
     */
    public boolean epsilonEquals(Matrix m1, double epsilon) {

        if (m1.fRowCnt != fRowCnt) {
            return false;
        }

        if (m1.fColCnt != fColCnt) {
            return false;
        }

        for (int i = 0; i < fRowCnt; i++) {
            for (int j = 0; j < fColCnt; j++) {
                if (epsilon < Math.abs(elementData[i * fColCnt + j]
                        - m1.elementData[i * fColCnt + j])) {
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * Returns the trace of this matrix.
     *
     * @return the trace of this matrix.
     */
    public double trace() {

        int min = (fRowCnt < fColCnt)
                ? fRowCnt
                : fColCnt;
        double trace = 0.0;

        for (int i = 0; i < min; i++) {
            trace += elementData[i * fColCnt + i];
        }

        return trace;
    }

    /**
     * Sets this matrix to a uniform scale matrix; all of the values are reset.
     *
     * @param scale The new scale value
     */
    public void setScale(float scale) {

        checkImmutable();
        setZero();

        int min = (fRowCnt < fColCnt)
                ? fRowCnt
                : fColCnt;

        for (int i = 0; i < min; i++) {
            elementData[i * fColCnt + i] = scale;
        }
    }

    public void sortColumns(SortMode sort, Order order) {
        checkImmutable();

        for (int c = 0; c < getNumCol(); c++) {
            Vector v = getColumnV(c); // gets us a safe copy
            v.sort(sort, order);
            this.setColumn(c, v);
        }

    }

    /**
     * Sets the value of this matrix to the result of multiplying itself
     * with matrix m1 (this = this * m1).
     *
     * @param m1 the other matrix
     */
    public void multiply(Matrix m1) {

        checkImmutable();

        // alias-safe.
        multiply(this, m1);
    }

    /**
     * Sets the value of this matrix to the result of multiplying
     * the two argument matrices together (this = m1 * m2).
     *
     * @param m1 the first matrix
     * @param m2 the second matrix
     */
    public void multiply(Matrix m1, Matrix m2) {

        checkImmutable();

        // for alias-safety, decided to new double [fColCnt*fRowCnt].
        // Is there any good way to avoid this big new ?
        if (fRowCnt != m1.fRowCnt) {
            throw new ArrayIndexOutOfBoundsException("fRowCnt:" + fRowCnt + " != m1.fRowCnt:"
                    + m1.fRowCnt);
        }

        if (fColCnt != m2.fColCnt) {
            throw new ArrayIndexOutOfBoundsException("fColCnt:" + fColCnt + " != m2.fColCnt:"
                    + m2.fColCnt);
        }

        if (m1.fColCnt != m2.fRowCnt) {
            throw new ArrayIndexOutOfBoundsException("m1.fColCnt:" + m1.fColCnt + " != m2.fRowCnt:"
                    + m2.fRowCnt);
        }

        float[] newData = new float[fColCnt * fRowCnt];

        for (int i = 0; i < fRowCnt; i++) {
            for (int j = 0; j < fColCnt; j++) {
                float sum = (float) 0.0;

                for (int k = 0; k < m1.fColCnt; k++) {
                    sum += m1.elementData[i * m1.fColCnt + k] * m2.elementData[k * m2.fColCnt + j];
                }

                newData[i * fColCnt + j] = sum;
            }
        }

        this.elementData = newData;
    }

    /**
     * Computes the outer product of the two vectors; multiplies the
     * the first vector by the transpose of the second vector
     * and places the matrix result into this matrix. This matrix must
     * be as big or bigger than getSize(v1)xgetSize(v2).
     *
     * @param v1 the first vector, treated as a row vector
     * @param v2 the second vector, treated as a column vector
     */
    public void multiply(Vector v1, Vector v2) {

        checkImmutable();

        if (fRowCnt < v1.getSize()) {
            throw new IllegalArgumentException("fRowCnt:" + fRowCnt + " < v1.getSize():"
                    + v1.getSize());
        }

        if (fColCnt < v2.getSize()) {
            throw new IllegalArgumentException("fColCnt:" + fColCnt + " < v2.getSize():"
                    + v2.getSize());
        }

        for (int i = 0; i < fRowCnt; i++) {
            for (int j = 0; j < fColCnt; j++) {
                elementData[i * fColCnt + j] = v1.getElement(i) * v2.getElement(j);
            }
        }
    }

    /**
     * Sets the value of this matrix to sum of itself and matrix m1.
     *
     * @param m1 the other matrix
     */
    public void add(Matrix m1) {

        checkImmutable();

        if ((fRowCnt != m1.fRowCnt) || (fColCnt != m1.fColCnt)) {
            throw new IllegalArgumentException("this:(" + fRowCnt + "x" + fColCnt + ") != m1:("
                    + m1.fRowCnt + "x" + m1.fColCnt + ").");
        }

        for (int i = 0; i < fRowCnt * fColCnt; i++) {
            elementData[i] += m1.elementData[i];
        }
    }

    /**
     * Sets the value of this matrix to the matrix sum of matrices m1 and m2.
     *
     * @param m1 the first matrix
     * @param m2 the second matrix
     */
    public void add(Matrix m1, Matrix m2) {

        checkImmutable();

        if ((fRowCnt != m1.fRowCnt) || (fColCnt != m1.fColCnt)) {
            throw new IllegalArgumentException("this:(" + fRowCnt + "x" + fColCnt + ") != m1:("
                    + m1.fRowCnt + "x" + m1.fColCnt + ").");
        }

        if ((fRowCnt != m2.fRowCnt) || (fColCnt != m2.fColCnt)) {
            throw new IllegalArgumentException("this:(" + fRowCnt + "x" + fColCnt + ") != m2:("
                    + m2.fRowCnt + "x" + m2.fColCnt + ").");
        }

        for (int i = 0; i < fRowCnt * fColCnt; i++) {
            elementData[i] = m1.elementData[i] + m2.elementData[i];
        }
    }

    /**
     * Sets the value of this matrix to the matrix difference of itself
     * and matrix m1 (this = this - m1).
     *
     * @param m1 the other matrix
     */
    public void subtract(Matrix m1) {

        checkImmutable();

        if ((fRowCnt != m1.fRowCnt) || (fColCnt != m1.fColCnt)) {
            throw new IllegalArgumentException("this:(" + fRowCnt + "x" + fColCnt + ") != m1:("
                    + m1.fRowCnt + "x" + m1.fColCnt + ").");
        }

        for (int i = 0; i < fRowCnt * fColCnt; i++) {
            elementData[i] -= m1.elementData[i];
        }
    }

    /**
     * Sets the value of this matrix to the matrix difference
     * of matrices m1 and m2 (this = m1 - m2).
     *
     * @param m1 the first matrix
     * @param m2 the second matrix
     */
    public void subtract(Matrix m1, Matrix m2) {

        checkImmutable();

        if ((fRowCnt != m1.fRowCnt) || (fColCnt != m1.fColCnt)) {
            throw new IllegalArgumentException("this:(" + fRowCnt + "x" + fColCnt + ") != m1:("
                    + m1.fRowCnt + "x" + m1.fColCnt + ").");
        }

        if ((fRowCnt != m2.fRowCnt) || (fColCnt != m2.fColCnt)) {
            throw new IllegalArgumentException("this:(" + fRowCnt + "x" + fColCnt + ") != m2:("
                    + m2.fRowCnt + "x" + m2.fColCnt + ").");
        }

        for (int i = 0; i < fRowCnt * fColCnt; i++) {
            elementData[i] = m1.elementData[i] - m2.elementData[i];
        }
    }

    /**
     * Negates the value of this matrix: this = -this.
     */
    public void negate() {

        checkImmutable();

        for (int i = 0; i < elementData.length; i++) {
            elementData[i] = -1 * elementData[i];
        }
    }

    /**
     * Sets the value of this matrix to the negation of the Matrix parameter.
     *
     * @param m1 The source matrix
     */
    public void negate(Matrix m1) {

        checkImmutable();
        set(m1, false);
        negate();
    }

    public void threshold(float minVal, float maxVal) {

        checkImmutable();

        for (int i = 0; i < elementData.length; i++) {
            if (elementData[i] < minVal) {
                elementData[i] = minVal;
            }

            if (elementData[i] > maxVal) {
                elementData[i] = maxVal;
            }
        }
    }

    public void thresholdToZeroInThisRange(float minVal, float maxVal) {

        checkImmutable();

        for (int i = 0; i < elementData.length; i++) {
            if (elementData[i] > minVal && elementData[i] < maxVal) {
                elementData[i] = 0;
            }
        }
    }

    // norm each row by the max of that row
    public Matrix normalize(final boolean by_row) {

        final Matrix m = new Matrix(getNumRow(), getNumCol());
        if (by_row) {
            for (int r = 0; r < getNumRow(); r++) {
                Vector v = getRowV(r);
                v.divide(v.max());
                m.setRow(r, v);
            }
        } else {

            for (int c = 0; c < getNumCol(); c++) {
                Vector v = getColumnV(c);
                v.divide(v.max());
                m.setColumn(c, v);
            }
        }

        return m;
    }

    public Matrix normalizeToMinNegOneMaxOne(final boolean byRow) {

        final Matrix m = new Matrix(getNumRow(), getNumCol());
        if (byRow) {
            for (int r = 0; r < getNumRow(); r++) {
                Vector v = getRowV(r);
                v.normalizeToMinNegOneMaxOne();
                m.setRow(r, v);
            }
        } else {

            for (int c = 0; c < getNumCol(); c++) {
                Vector v = getColumnV(c);
                v.normalizeToMinNegOneMaxOne();
                m.setColumn(c, v);
            }
        }

        return m;
    }

    public Matrix normalizeByZScore(final boolean byRow, final boolean useMean) {

        final Matrix m = new Matrix(getNumRow(), getNumCol());
        if (byRow) {
            for (int r = 0; r < getNumRow(); r++) {
                Vector v = getRowV(r);
                v.normalizeByZScore(useMean);
                m.setRow(r, v);
            }
        } else {

            for (int c = 0; c < getNumCol(); c++) {
                Vector v = getColumnV(c);
                v.normalizeByZScore(useMean);
                m.setColumn(c, v);
            }
        }

        return m;
    }

    public Matrix normalizeByZScore_mad(final boolean byRow, final boolean useMean) {

        final Matrix m = new Matrix(getNumRow(), getNumCol());
        if (byRow) {
            for (int r = 0; r < getNumRow(); r++) {
                Vector v = getRowV(r);
                v.normalizeByZScore_mad(useMean);
                m.setRow(r, v);
            }
        } else {

            for (int c = 0; c < getNumCol(); c++) {
                Vector v = getColumnV(c);
                v.normalizeByZScore_mad(useMean);
                m.setColumn(c, v);
            }
        }

        return m;
    }

    public Matrix normalizeByGsea(final boolean byRow) {

        final Matrix m = new Matrix(getNumRow(), getNumCol());
        if (byRow) {
            for (int r = 0; r < getNumRow(); r++) {
                Vector v = getRowV(r);
                v.normalizeByGsea();
                m.setRow(r, v);
            }
        } else {

            for (int c = 0; c < getNumCol(); c++) {
                Vector v = getColumnV(c);
                v.normalizeByGsea();
                m.setColumn(c, v);
            }
        }

        return m;
    }

    /**
     * every element is this / m1
     *
     * @param m1
     */
    public void divide(Matrix m1, boolean setDivByZeroAsZero) {

        checkImmutable();

        _enforceEqualDimensions(this, m1);

        for (int i = 0; i < this.elementData.length; i++) {
            if (setDivByZeroAsZero && m1.elementData[i] == 0) {
                this.elementData[i] = 0;
            } else {
                this.elementData[i] = this.elementData[i] / m1.elementData[i];
            }
        }
    }

    public void log() {

        checkImmutable();

        for (int i = 0; i < this.elementData.length; i++) {

            float f = this.elementData[i];
            if (f == 0) {
                f = (float) 0.00000001;
            }

            this.elementData[i] = (float) Math.log(f); // base E
        }
    }

    public void log2() {

        checkImmutable();

        for (int i = 0; i < this.elementData.length; i++) {

            float f = this.elementData[i];
            if (f == 0) {
                f = (float) 0.00000001;
            }

            this.elementData[i] = (float) (XMath.log2(f));
        }
    }

    public void log10() {

        checkImmutable();

        for (int i = 0; i < this.elementData.length; i++) {

            float f = this.elementData[i];
            if (f == 0) {
                f = (float) 0.00000001;
            }

            this.elementData[i] = (float) (XMath.log10(f));
        }
    }

    public void exp(double num) {

        checkImmutable();

        for (int i = 0; i < this.elementData.length; i++) {
            float f = this.elementData[i];
            // first arg raised to the power of the second arg
            this.elementData[i] = (float) Math.pow(num, f);
        }
    }

    /**
     * Sets all the values in this matrix to zero.
     */
    public void setZero() {

        checkImmutable();

        for (int i = 0; i < fRowCnt * fColCnt; i++) {
            elementData[i] = (float) 0.0;
        }
    }

    /**
     * Subtracts this matrix from the identity matrix and puts the values
     * back into this (this = I - this).
     */
    public void identityMinus() {

        checkImmutable();
        negate();

        int min = (fRowCnt < fColCnt)
                ? fRowCnt
                : fColCnt;

        for (int i = 0; i < min; i++) {
            elementData[i * fColCnt + i] += (float) 1.0;
        }
    }

    public void setImmutable() {
        this.fImmuted = true;
    }

    public boolean isImmutable() {
        return fImmuted;
    }

    private void checkImmutable() {

        if (fImmuted) {
            throw new ImmutedException();
        }
    }

    /**
     * Siginifcance by row
     *
     * @todo signfic needs review by pt
     */
    public Vector sigByRow(double level) {

        Vector v = new Vector(fRowCnt);

        for (int r = 0; r < fRowCnt; r++) {
            v.setElement(r, this.getRowV(r).sig(level));
        }

        return v;
    }

    public Vector sigByCol(double level) {

        Vector v = new Vector(fColCnt);

        for (int c = 0; c < fColCnt; c++) {
            v.setElement(c, this.getColumnV(c).sig(level));
        }

        return v;
    }

    /**
     * (the first max value)
     * max element in the entire matrix
     * absolute -> entri matrix
     * else by row.
     */
    public Matrix.Cell max() {

        Matrix.Cell max = new Matrix.Cell();

        max.value = Float.NEGATIVE_INFINITY;

        for (int i = 0; i < fRowCnt; i++) {
            for (int j = 0; j < fColCnt; j++) {
                if (elementData[i * fColCnt + j] > max.value) {
                    max.row = i;
                    max.col = j;
                    max.value = elementData[i * fColCnt + j];
                }
            }
        }

        return max;
    }

    // the first min value
    public Matrix.Cell min() {

        Matrix.Cell min = new Matrix.Cell();

        min.value = Float.POSITIVE_INFINITY;

        for (int i = 0; i < fRowCnt; i++) {
            for (int j = 0; j < fColCnt; j++) {
                if (elementData[i * fColCnt + j] < min.value) {
                    min.row = i;
                    min.col = j;
                    min.value = elementData[i * fColCnt + j];
                }
            }
        }

        return min;
    }

    // must more efficento than getting each row and doind max
    public Matrix.Cell[] maxByRow() {

        Matrix.Cell[] maxes = new Matrix.Cell[fRowCnt];

        for (int i = 0; i < fRowCnt; i++) {
            maxes[i] = new Cell();
            maxes[i].value = Float.NEGATIVE_INFINITY;

            for (int j = 0; j < fColCnt; j++) {
                if (elementData[i * fColCnt + j] > maxes[i].value) {
                    maxes[i].row = i;
                    maxes[i].col = j;
                    maxes[i].value = elementData[i * fColCnt + j];
                }
            }
        }

        return maxes;
    }

    public Vector removeValues(final float removeElementsThatEqualThisValue) {
        TFloatArrayList remaining = new TFloatArrayList();
        for (int r = 0; r < this.getNumRow(); r++) {
            for (int c = 0; c < this.getNumCol(); c++) {
                float score = this.getElement(r, c);
                if (score != 0) {
                    remaining.add(score);
                }
            }
        }

        return new Vector(remaining);
    }

    public double mean() {
        return new Vector(elementData, true).mean();
    }

    public double median() {
        return new Vector(elementData, true).median();
    }

    public double var(boolean biased, boolean fixlow) {
        return new Vector(elementData, true).var(biased, fixlow);
    }

    public double stddev(boolean biased, boolean fixlow) {
        return new Vector(elementData, true).stddev(biased, fixlow);
    }

    /**
     * absolute binning --- entire matrix
     * adds to the specified ranges.
     */
    public void bin(final Range[] ranges, boolean enforceAllElementsMustBeBinned, boolean avoiddoubleBinning) {

        if (ranges == null) {
            throw new IllegalArgumentException("Param ranges cannot be null");
        }

        // note that a point on the bourndary will arbitarilly be put into the first Range encountered
        int numBinned = 0;
        for (int i = 0; i < elementData.length; i++) {
            boolean binned = false;
            for (int j = 0; j < ranges.length; j++) {
                if (ranges[j].isInRange(elementData[i])) {
                    ranges[j].increment();
                    binned = true;
                    numBinned++;
                    if (avoiddoubleBinning) {
                        break; // @imp to avoid double binning
                    }
                }
            }

            if (enforceAllElementsMustBeBinned && !binned) {
                throw new RuntimeException("Value was not binned as it didnt fall in any of the Ranges: " + elementData[i]);
            }
        }

        // cant do this as the ranges might already have stuff inthem
        //int tot = RangeUtils.getTotalCount(ranges);
        int tot = numBinned;
        if (enforceAllElementsMustBeBinned && tot != elementData.length) {
            throw new IllegalStateException("Bad binning! Binned: " + tot + " but elements in matrix: " + elementData.length);
        }

    }

    /**
     * Discretize the matrix into the specified ranges.
     *
     * @return A whole new discretized matrix. The floats are not shared.
     *         Matrix will contain value from 0 to ranges-1
     */

    // More efficient to create a new matrix here (note diff with the other discz api)
    // Iterate through matrix assigning each element to one and only one range.
    // Unlike binning, the number of discretized(binned) elements must exactly
    // equal the numb of elements in the matrix.
    public Matrix discretize(Range[] ranges) {

        Matrix newmatrix = new Matrix(getNumRow(), getNumCol());

        for (int i = 0; i < fRowCnt * fColCnt; i++) {
            for (int j = 0; j < ranges.length; j++) {
                if (ranges[j].isInRange(elementData[i])) {    // checking this matrix
                    newmatrix.elementData[i] = j;           // but assigning to new matrix
                    break;
                }
            }
        }

        return newmatrix;
    }

    /**
     * equalk numbf of lements actually equal numb of DIFF(float) elements.
     * changes the matrix "in place"
     * matrix must be not be immutable for this to work
     * <p/>
     * each disc has classdensity elements
     * <p/>
     * Sort the array
     * Make it unique
     * Now, the number of ranges = uniq.length / classdensity
     * Create the ranges
     * Iterate through untouched (unsorted, ununiqueized) array and file ranges.
     */
    public Matrix discretize(int classdensity) {

        Matrix newmatrix = new Matrix(this, false);

        klog.debug("Starting uniq for discretize");

        float[] uniq = ArrayUtils.unique(newmatrix.elementData);

        klog.debug("Done uniq for discretize");
        Arrays.sort(uniq);
        klog.debug("Done sort for discretize");

        Range[] ranges = RangeFactory.createRanges(classdensity, uniq);

        klog.debug("Number of ranges = " + ranges.length);

        for (int i = 0; i < fRowCnt * fColCnt; i++) {
            for (int j = 0; j < ranges.length; j++) {
                if (ranges[j].isInRange(this.elementData[i])) {
                    newmatrix.elementData[i] = j;

                    break;
                }
            }
        }

        return newmatrix;
    }

    /**
     * Rond off all matrix elements to said decimal places
     * numb_decimal_places=1 then 45.89899 -> 45.9
     *
     * @throws ImmutedException if matrix is immutable
     */
    public void roundOff(int numb_decimal_places) {

        checkImmutable();

        for (int i = 0; i < elementData.length; i++) {
            elementData[i] = (float) XMath.roundOff(elementData[i], numb_decimal_places);
        }
    }

    private static void _enforceEqualDimensions(Matrix a, Matrix b) {
        if (a.getNumRow() != b.getNumRow()) {
            throw new IllegalArgumentException("Mismatched matrices: must be of equal row lengths a: " + a.getNumRow() + " b: " + b.getNumRow());
        }

        if (a.getNumCol() != b.getNumCol()) {
            throw new IllegalArgumentException("Mismatched matrices: must be of equal col lengths a: " + a.getNumCol() + " b: " + b.getNumCol());
        }
    }

    private static int enforceSameNumOfCols(Matrix[] matrices) {
        int ncols = matrices[0].getNumCol();

        for (int i = 0; i < matrices.length; i++) {
            if (matrices[i].getNumCol() != ncols) {
                throw new RuntimeException("Unequal ncols: " + ncols + " and "
                        + matrices[i].getNumCol() + "at index: " + i);
            }
        }

        return ncols;
    }

    private static int enforceSameNumOfRows(Matrix[] matrices) {
        int nrows = matrices[0].getNumRow();

        for (int i = 0; i < matrices.length; i++) {
            if (matrices[i].getNumRow() != nrows) {
                throw new RuntimeException("Unequal nrows: " + nrows + " and "
                        + matrices[i].getNumRow() + "at index: " + i);
            }
        }

        return nrows;
    }

    public static int getTotalNumRows(Matrix[] matrices) {
        int nrows = 0;
        for (int i = 0; i < matrices.length; i++) {
            nrows += matrices[i].getNumRow();
        }

        return nrows;
    }

    public static int getTotalNumCols(Matrix[] matrices) {
        int ncols = 0;
        for (int i = 0; i < matrices.length; i++) {
            ncols += matrices[i].getNumCol();
        }

        return ncols;
    }

    /**
     * Represents a single matrix cell.
     */
    public static class Cell {
        public int row;
        public int col;
        public float value;
    }    // End Cell

}        // End Matrix

/**
 * Sets this Matrix to the identity matrix.
 */
/*
public void setIdentity() {
    checkImmutable();
    setZero();
    int min = fRowCnt < fColCnt ? fRowCnt : fColCnt;
    for (int i = 0; i < min; i++)
        elementData[i*fColCnt + i] = (float)1.0;
}
*/

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
