/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.math;

/**
 * @author Aravind Subramanian
 *         Date: Jul 4, 2003
 *         Time: 7:39:55 PM
 */
public class NormMode {

    private String fType;

    public static final NormMode NO_NORM = new NormMode("NoNorm");

    public static final NormMode AREA_NORM = new NormMode("AreaNorm");

    public static final NormMode VAR_NORM = new NormMode("VarianceNorm");

    public static final NormMode VAR_NORM_TMP = new NormMode("VarianceNormTmp");

    public static final NormMode[] ALL = new NormMode[]{NO_NORM, AREA_NORM, VAR_NORM, VAR_NORM_TMP};

    /**
     * Privatized class constructor
     *
     * @param name
     */
    NormMode(String name) {
        this.fType = name;
    }

    public String toString() {
        return fType;
    }

    public boolean isRaw() {

        return this.fType.equalsIgnoreCase(NO_NORM.fType);

    }

    public boolean isArea() {

        return this.fType.equalsIgnoreCase(AREA_NORM.fType);

    }

    public boolean isVar() {

        return this.fType.equalsIgnoreCase(VAR_NORM.fType);

    }

    public boolean isVarTmp() {

        return this.fType.equalsIgnoreCase(VAR_NORM_TMP.fType);

    }

    public boolean equals(Object obj) {

        if (obj instanceof NormMode) {
            if (((NormMode) obj).fType.equals(this.fType)) {
                return true;
            }
        }

        return false;
    }

    public int hashCode() {
        return fType.hashCode();
    }

    /**
     * a lookup method for normmode object or string
     */
    public static NormMode lookup(Object obj) {

        if (obj == null) {
            throw new NullPointerException("Null normmode not allowed");
        }

        if (obj instanceof NormMode) {
            return (NormMode) obj;
        }

        String str = obj.toString();

        for (int i = 0; i < ALL.length; i++) {
            if (str.equalsIgnoreCase(ALL[i].fType)) {
                return ALL[i];
            }
        }

        throw new IllegalArgumentException("Unable to lookup NormMode: " + obj);
    }

} // End class NormMode
