/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.math;

import java.util.Collections;
import java.util.List;

/**
 * Equivalent of DoubleElement
 */
public class ObjectIndex {

    private Object fObject;
    private int fIndex;

    /**
     * Class constructor
     *
     * @param obj
     * @param index
     */
    public ObjectIndex(Object obj, int index) {
        this.fObject = obj;
        this.fIndex = index;
    }

    public Object getObject() {
        return fObject;
    }

    public int getIndex() {
        return fIndex;
    }


    public static void sort(SortMode sort, Order order, List oilist) {

        if (order.isAscending()) {
            Collections.sort(oilist, new AscendingValueComparator(sort));
        } else if (order.isDescending()) {
            Collections.sort(oilist, new DescendingValueComparator(sort));
        } else {
            throw new IllegalArgumentException("Unknown Order: " + order);
        }
    }

    public static ObjectIndex[] sort2array(SortMode sort, Order order, List oilist) {

        if (order.isAscending()) {
            Collections.sort(oilist, new AscendingValueComparator(sort));
        } else if (order.isDescending()) {
            Collections.sort(oilist, new DescendingValueComparator(sort));
        } else {
            throw new IllegalArgumentException("Unknown Order: " + order);
        }

        return (ObjectIndex[]) oilist.toArray(new ObjectIndex[oilist.size()]);
    }

    /**
     * Class AscendingValueComparator
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    static class AscendingValueComparator implements java.util.Comparator {

        private final SortMode fSort;

        public AscendingValueComparator(SortMode sort) {
            this.fSort = sort;
        }

        /**
         * Return -1 if o1 is less than o2, 0 if they're equal, +1 if o1 is greater than o2.
         */
        public int compare(Object pn1, Object pn2) {

            if ((pn1 == null) && (pn2 == null)) {
                return 0;     // cant compare
            } else if (pn1 == null) {
                return -1;    // null is always least
            } else if (pn2 == null) {
                return +1;    // null is always least
            }

            int d1 = ((ObjectIndex) pn1).fIndex;
            int d2 = ((ObjectIndex) pn2).fIndex;

            if (fSort.isAbsolute()) {
                d1 = Math.abs(d1);
                d2 = Math.abs(d2);
            }

            //System.out.println("Comparing " + fe1.value + " " + fe1.index + " and " + fe2.value + " " + fe2.index);
            if (d1 < d2) {
                return -1;
            }

            if (d1 > d2) {
                return +1;
            }

            return 0;
        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End AscendingValueComparator

    /**
     * Class DescendingValueComparator
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    static class DescendingValueComparator implements java.util.Comparator {

        private final SortMode fSort;

        public DescendingValueComparator(SortMode sort) {
            this.fSort = sort;
        }

        /**
         * Return -1 if o1 is less than o2, 0 if they're equal, +1 if o1 is greater than o2.
         */
        public int compare(Object pn1, Object pn2) {

            if ((pn1 == null) && (pn2 == null)) {
                return 0;     // cant compare
            } else if (pn1 == null) {
                return -1;    // null is always least
            } else if (pn2 == null) {
                return +1;    // null is always least
            }

            int d1 = ((ObjectIndex) pn1).fIndex;
            int d2 = ((ObjectIndex) pn2).fIndex;

            if (fSort.isAbsolute()) {
                d1 = Math.abs(d1);
                d2 = Math.abs(d2);
            }

            //System.out.println("Comparing " + fe1.value + " " + fe1.index + " and " + fe2.value + " " + fe2.index);
            if (d1 < d2) {
                return +1;
            }

            if (d1 > d2) {
                return -1;
            }

            return 0;
        }

        /**
         * Return true if this equals o2.
         */
        public boolean equals(Object o2) {
            return false;
        }
    }    // End DescendingValueComparator

} // End class ObjectIndex
