/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.math;


import edu.mit.broad.genome.utils.containers.ClassNamePair;


/**
 * enum construct
 */
public class Orientation {

    public static final Orientation ROW = new Orientation("By Row");
    public static final Orientation COL = new Orientation("By Col");
    public static final Orientation[] ALL = new Orientation[]{ROW, COL};
    public static final ClassNamePair[] ALL_CNP = new ClassNamePair[]{
            new ClassNamePair(ROW.getClass(), ROW.fType),
            new ClassNamePair(COL.getClass(), COL.fType)};
    private final String fType;

    /**
     * Privatized class constructor.
     */
    private Orientation(String type) {
        this.fType = type;
    }

    public String toString() {
        return fType;
    }

    public boolean isByRow() {

        return this.equals(ROW);

    }

    public boolean isByCol() {

        return this.equals(COL);

    }

    public boolean equals(Object obj) {

        if (obj instanceof Orientation) {
            if (((Orientation) obj).fType.equals(this.fType)) {
                return true;
            }
        }

        return false;
    }

    public int hashCode() {
        return fType.hashCode();
    }

    /**
     * a lookup metod for sort mode
     */
    public static Orientation lookup(Object obj) {

        if (obj == null) {
            throw new NullPointerException("Null mode cabbot be looked up");
        }

        if (obj instanceof Orientation) {
            return (Orientation) obj;
        }

        String s = obj.toString();

        if (s.equalsIgnoreCase(ROW.fType)) {
            return ROW;
        } else if (s.equalsIgnoreCase(COL.toString())) {
            return COL;
        }

        throw new IllegalArgumentException("Unable to lookup Orientation: " + obj);
    }
}    // End Orientation

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
