/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.math;


import edu.mit.broad.genome.utils.containers.ClassNamePair;


/**
 * enum construct
 */
public class ScaleMode {

    public static final ScaleMode REL_MEAN = new ScaleMode("Relative & Mean");
    public static final ScaleMode REL_MEDIAN = new ScaleMode("Relative & Median");
    public static final ScaleMode ABSOLUTE = new ScaleMode("Absolute");
    public static final ScaleMode REL_MEAN_ZERO_OMITTED = new ScaleMode("Relative & Mean (zero omitted)");
    public static final ScaleMode[] ALL = new ScaleMode[]{REL_MEAN, REL_MEDIAN, ABSOLUTE, REL_MEAN_ZERO_OMITTED};

    public static final ClassNamePair[] ALL_CNP = new ClassNamePair[]{
            new ClassNamePair(REL_MEAN.getClass(),
                    REL_MEAN.fType),
            new ClassNamePair(REL_MEDIAN.getClass(),
                    REL_MEDIAN.fType),
            new ClassNamePair(ABSOLUTE.getClass(),
                    ABSOLUTE.fType)};
    private final String fType;

    /**
     * Privatized class constructor.
     */
    private ScaleMode(final String type) {
        this.fType = type;
    }

    public String toString() {
        return fType;
    }

    public boolean isMean() {

        return this.equals(REL_MEAN) || this.equals(REL_MEAN_ZERO_OMITTED);
    }

    public boolean isMedian() {
        return this.equals(REL_MEDIAN);
    }

    public boolean isRelative() {

        return (this.equals(REL_MEAN)) || (this.equals(REL_MEDIAN) || this.equals(REL_MEAN_ZERO_OMITTED));
    }

    public boolean isAbsolute() {

        return this.equals(ABSOLUTE);
    }

    public boolean equals(final Object obj) {

        if (obj instanceof ScaleMode) {
            if (((ScaleMode) obj).fType.equals(this.fType)) {
                return true;
            }
        }

        return false;
    }

    public int hashCode() {
        return fType.hashCode();
    }

    /**
     * a lookup metod for sort mode
     */
    public static ScaleMode lookup(Object obj) {

        if (obj == null) {
            throw new NullPointerException("Null mode cabbot be looked up");
        }

        if (obj instanceof ScaleMode) {
            return (ScaleMode) obj;
        }

        String s = obj.toString();

        if (s.equalsIgnoreCase(REL_MEAN.fType)) {
            return REL_MEAN;
        } else if (s.equalsIgnoreCase(REL_MEDIAN.toString())) {
            return REL_MEDIAN;
        } else if (s.equalsIgnoreCase(ABSOLUTE.toString())) {
            return ABSOLUTE;
        }

        throw new IllegalArgumentException("Unable to lookup ScaleMode: " + obj);
    }
}    // End ScaleMode

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
