/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.math;


/**
 * @author Aravind Subramanian
 *         Date: Jul 10, 2004
 *         Time: 11:20:41 AM
 */
public class ScoreMode {

    public static final ScoreMode POS_ONLY = new ScoreMode("pos_only");
    public static final ScoreMode NEG_ONLY = new ScoreMode("neg_only");
    public static final ScoreMode POS_AND_NEG_TOGETHER = new ScoreMode("pos_and_neg_together");
    public static final ScoreMode POS_AND_NEG_SEPERATELY = new ScoreMode("pos_and_neg_seperately");

    public static final ScoreMode[] ALL = new ScoreMode[]{POS_ONLY, NEG_ONLY,
            POS_AND_NEG_TOGETHER,
            POS_AND_NEG_SEPERATELY};

    public static final ScoreMode[] POS_NEG = new ScoreMode[]{POS_ONLY, NEG_ONLY};


    private String fMode;

    ScoreMode(String mode) {
        this.fMode = mode;
    }

    public String toString() {
        return fMode;
    }

    public boolean equals(Object obj) {

        if (obj instanceof ScoreMode) {
            if (((ScoreMode) obj).fMode.equals(this.fMode)) {
                return true;
            }
        }

        return false;
    }

    public int hashCode() {
        return fMode.hashCode();
    }

    public String getName() {
        return fMode;
    }

    public boolean isPostiveOnly() {

        return fMode.equals(POS_ONLY.fMode);

    }

    public boolean isPostiveAndNegTogether() {

        return fMode.equals(POS_AND_NEG_TOGETHER.fMode);

    }

    public boolean isPostiveAndNegSeperately() {

        return fMode.equals(POS_AND_NEG_SEPERATELY.fMode);

    }

    public boolean isNegativeOnly() {

        return fMode.equals(NEG_ONLY.fMode);

    }

    /**
     * a lookup metod for score mode
     */
    public static ScoreMode lookup(Object obj) {

        if (obj == null) {
            throw new NullPointerException("Null mode not allowed");
        }

        if (obj instanceof ScoreMode) {
            return (ScoreMode) obj;
        }

        String s = obj.toString();

        if (s.equalsIgnoreCase(POS_ONLY.fMode)) {
            return POS_ONLY;
        } else if (s.equalsIgnoreCase(NEG_ONLY.toString())) {
            return NEG_ONLY;
        } else if (s.equalsIgnoreCase(POS_AND_NEG_TOGETHER.toString())) {
            return POS_AND_NEG_TOGETHER;
        } else if (s.equalsIgnoreCase(POS_AND_NEG_SEPERATELY.toString())) {
            return POS_AND_NEG_SEPERATELY;
        }

        throw new IllegalArgumentException("Unable to lookup ScoreMode String: " + obj);
    }

} // End class ScoreMode

