/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.math;


/**
 * @author Aravind Subramanian
 *         Date: May 30, 2003
 *         Time: 6:04:40 PM
 */
public class SimpleRange implements Range {

    private double fMin;

    private double fMax;

    private int fCount;

    private boolean fMutable = true;

    public SimpleRange(int min, int max) {
        this.fMin = min;
        this.fMax = max;
    }

    public SimpleRange(double min, double max) {
        this.fMin = min;
        this.fMax = max;
    }

    public static String toString_counts_only(final Range[] ranges, char delim) {
        StringBuffer buf = new StringBuffer();
        for (int r = 0; r < ranges.length; r++) {
            buf.append(ranges[r].getCount());
            if (r != ranges.length - 1) {
                buf.append(delim);
            }
        }

        return buf.toString();
    }

    public static Range[] cloneShallow(final Range[] ranges) {
        Range[] clones = new Range[ranges.length];
        for (int r = 0; r < ranges.length; r++) {
            clones[r] = ranges[r].cloneShallow();
        }

        return clones;
    }

    public Range cloneShallow() {
        return new SimpleRange(getMin(), getMax());
    }

    public void increment() {
        checkMutable();
        fCount++;
    }

    public int getCount() {
        return fCount;
    }

    public void setImmutable() {
        this.fMutable = false;
    }

    private void checkMutable() {
        if (!fMutable) {
            throw new IllegalStateException("SimpleBin is not mutable");
        }
    }

    public double getSpread() {
        return fMax - fMin + 1.0d;    // because both are inclusive
    }

    public boolean isInRange(double valueInclusive) {

        return (valueInclusive >= fMin) && (valueInclusive <= fMax);
    }

    public double getMin() {
        return fMin;
    }

    public double getMax() {
        return fMax;
    }

    public double getMean() {
        return ((fMin + fMax) / 2.0d);
    }

    public double getFuzz() {
        return 0;
    }

    public String getName() {
        return fMin + ":" + fMax;
    }

    public String toString() {
        return getName();
    }

    public int hashCode() {
        return toString().hashCode();
    }

} // End SimpleBin

