/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.math;


import edu.mit.broad.genome.utils.containers.ClassNamePair;


/**
 * enum construct
 */
public class SortMode {

    public static final SortMode REAL = new SortMode("real");
    public static final SortMode ABSOLUTE = new SortMode("abs");
    public static final SortMode[] ALL = new SortMode[]{REAL, ABSOLUTE};
    public static final ClassNamePair[] ALL_CNP = new ClassNamePair[]{
            new ClassNamePair(REAL.getClass(), REAL.fType),
            new ClassNamePair(ABSOLUTE.getClass(),
                    ABSOLUTE.fType)};
    private final String fType;

    /**
     * Privatized class constructor.
     */
    private SortMode(String type) {
        this.fType = type;
    }

    public String toString() {
        return fType;
    }

    public boolean equals(Object obj) {

        if (obj instanceof SortMode) {
            if (((SortMode) obj).fType.equals(this.fType)) {
                return true;
            }
        }

        return false;
    }

    public int hashCode() {
        return fType.hashCode();
    }

    public boolean isAbsolute() {

        return fType.equals(ABSOLUTE.fType);

    }

    public boolean isReal() {

        return fType.equals(REAL.fType);

    }

    /**
     * a lookup metod for sort mode
     */
    public static SortMode lookup(Object obj) {

        if (obj == null) {
            throw new NullPointerException("Null mode not allowed");
        }

        if (obj instanceof SortMode) {
            return (SortMode) obj;
        }

        String s = obj.toString();

        if (s.equalsIgnoreCase(REAL.fType)) {
            return REAL;
        } else if (s.equalsIgnoreCase(ABSOLUTE.toString())) {
            return ABSOLUTE;
        }

        throw new IllegalArgumentException("Unable to lookup direction String: " + obj);
    }
}    // End SortMode

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
