/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.math;


import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.Printf;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.objects.LabelledVector;
import edu.mit.broad.genome.objects.RankedList;
import edu.mit.broad.genome.reports.MultiElement;
import edu.mit.broad.genome.utils.ImmutedException;
import gnu.trove.TFloatArrayList;
import org.apache.log4j.Logger;

import java.awt.*;
import java.util.*;
import java.util.List;


/**
 * all methods that chnage the values of teh matrix sgouldfirst check the imm/mutablity state
 * of the matrix.
 * <p/>
 * Lost of code/ideas copied form Kenji Hiranabe, Eiwa System Management, Inc.
 * GStringMatrix part of the javax.vecmath library (unoffical (his) impl)
 * http://java.sun.com/products/java-media/3D/forDevelopers/j3dapi/index.html
 * <p/>
 * as note: made sevral of the row related methods unfinal as need to override
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class StringMatrix {

    /**
     * @maint IMP IMP: if any fields are added to this class,
     * make sure to update the constructors and the
     * set(StringMatrix method
     * <p/>
     * The data of the StringMatrix.(1-D array. The (i,j) element is stored in elementData[i*fColCnt + j])
     * <p/>
     * <p/>
     * The data of the StringMatrix.(1-D array. The (i,j) element is stored in elementData[i*fColCnt + j])
     */

    /**
     * The data of the StringMatrix.(1-D array. The (i,j) element is stored in elementData[i*fColCnt + j])
     */
    private String elementData[];

    // NOT always available - lazilly fileld
    private ObjectMatrix fColorMatrix;

    /**
     * The number of rows in this matrix.
     */
    private int fRowCnt;

    /**
     * The number of columns in this matrix.
     */
    private int fColCnt;
    private boolean fImmuted;
    private static final transient Logger klog = XLogger.getLogger(StringMatrix.class);

    /**
     * Class Constructor.
     * Must call set() after this form of teh constructor.
     */
    public StringMatrix() {
    }

    public void replace(final String thisStr, final String withThisStr) {
        // @NOTE HACK
        if (thisStr.equals("NaN") == false) {
            checkImmuted();
        }

        for (int i = 0; i < elementData.length; i++) {
            if (elementData[i] != null && elementData[i].equals(thisStr)) {
                elementData[i] = withThisStr;
            }
        }
    }

    // doea NOT affect the current matrix
    public StringMatrix toUpperCase() {
        final StringMatrix sm = new StringMatrix(getNumRow(), getNumCol());
        for (int r = 0; r < getNumRow(); r++) {
            for (int c = 0; c < getNumCol(); c++) {
                final String curr = getElement(r, c);
                if (curr != null) {
                    sm.setElement(r, c, curr.toUpperCase());
                }
            }
        }

        return sm;
    }

    /**
     * Constructs an fRowCnt by fColCnt all zero matrix. (as change)
     * Note that even though row and column numbering begins with
     * zero, fRowCnt and fColCnt will be one larger than the maximum
     * possible matrix index values.
     *
     * @param nrows number of rows in this matrix.
     * @param ncols number of columns in this matrix.
     */
    public StringMatrix(final int nrows, final int ncols) {

        if (nrows < 0) {
            throw new NegativeArraySizeException(nrows + " < 0");
        }

        if (ncols < 0) {
            throw new NegativeArraySizeException(ncols + " < 0");
        }

        this.fRowCnt = nrows;
        this.fColCnt = ncols;
        elementData = new String[fRowCnt * fColCnt];

        setZero();
    }

    /**
     * Class constructor
     *
     * @param matrixA
     * @param matrixB
     */
    public StringMatrix(final StringMatrix matrixA, final StringMatrix matrixB) {
        this(new StringMatrix[]{matrixA, matrixB});
    }

    /**
     * Class constructor
     *
     * @param m
     */
    public StringMatrix(final Matrix m) {

        this(m.getNumRow(), m.getNumCol());

        for (int i = 0; i < m.elementData.length; i++) {
            this.elementData[i] = m.elementData[i] + "";
        }
    }

    /**
     * data is not shared
     * matrices must be of same number of columns
     * (nrows can be arb - thats the point!)
     */
    public StringMatrix(final StringMatrix[] matrixs) {

        if (matrixs.length == 0) {
            throw new IllegalArgumentException("Cannot call this constructor with zero length StringMatrix[] array");
        }

        int c = matrixs[0].getNumCol();

        for (int i = 0; i < matrixs.length; i++) {
            if (matrixs[i].getNumCol() != c) {
                throw new RuntimeException("Unequal ncols: " + c + " and "
                        + matrixs[i].getNumCol());
            }
        }

        int nrows = 0;

        for (int i = 0; i < matrixs.length; i++) {
            nrows += matrixs[i].getNumRow();
        }

        this.fRowCnt = nrows;
        this.fColCnt = c;
        this.elementData = new String[fRowCnt * fColCnt];

        int pos = 0;

        for (int i = 0; i < matrixs.length; i++) {
            System.arraycopy(matrixs[i].elementData, 0, elementData, pos,
                    matrixs[i].elementData.length);

            pos += matrixs[i].elementData.length;
        }
    }

    /**
     * Constructs a new StringMatrix and copies the initial values
     * from the parameter matrix.
     *
     * @param matrix the source of the initial values of the new StringMatrix
     *               <p/>
     *               as added: if shared values, then this matrix shares the elements with the specified amtric
     *               else the array is systemarrycopied
     */
    public StringMatrix(final StringMatrix matrix, final boolean shareValues) {

        this.fRowCnt = matrix.fRowCnt;
        this.fColCnt = matrix.fColCnt;

        if (shareValues) {
            this.fImmuted = matrix.fImmuted;
        } else {
            this.fImmuted = false;
        }

        if (shareValues) {
            this.elementData = matrix.elementData;
        } else {
            int newSize = fRowCnt * fColCnt;

            this.elementData = new String[newSize];

            System.arraycopy(matrix.elementData, 0, elementData, 0, newSize);
        }
    }

    /**
     * Sets all the values in this matrix to zero.
     */
    public void setZero() {

        checkImmuted();

        for (int i = 0; i < fRowCnt * fColCnt; i++) {
            elementData[i] = "";
        }
    }

    /**
     * the cloned matrix is NOT immutable even if the matrix it was cloned from was.
     */
    public StringMatrix cloneDeep() {

        klog.debug("Making cloneDeep of StringMatrix");

        return new StringMatrix(this, false);
    }

    public void fill(String s) {
        checkImmuted();
        for (int i = 0; i < elementData.length; i++) {
            elementData[i] = s;
        }
    }

    /**
     * Copies a sub-matrix derived from this matrix into the target matrix.
     * The upper left of the sub-matrix is located at (rowSource, colSource);
     * the lower right of the sub-matrix is located at
     * (lastRowSource,lastColSource).  The sub-matrix is copied into the
     * the target matrix starting at (rowDest, colDest).
     *
     * @param rowSource the top-most row of the sub-matrix
     * @param colSource the left-most column of the sub-matrix
     * @param numRow    the number of rows in the sub-matrix
     * @param numCol    the number of columns in the sub-matrix
     * @param rowDest   the top-most row of the position of the copied sub-matrix
     *                  within the target matrix
     * @param colDest   the left-most column of the position of the copied sub-matrix
     *                  within the target matrix
     * @param target    the matrix into which the sub-matrix will be copied
     */
    public void copySubStringMatrix(int rowSource, int colSource, int numRow, int numCol,
                                    int rowDest, int colDest, StringMatrix target) {

        if ((rowSource < 0) || (colSource < 0) || (rowDest < 0) || (colDest < 0)) {
            throw new ArrayIndexOutOfBoundsException("rowSource,colSource,rowDest,colDest < 0.");
        } else if ((fRowCnt < numRow + rowSource) || (fColCnt < numCol + colSource)) {
            throw new ArrayIndexOutOfBoundsException("Source StringMatrix too small.");
        } else if ((target.fRowCnt < numRow + rowDest) || (target.fColCnt < numCol + colDest)) {
            throw new ArrayIndexOutOfBoundsException("Target StringMatrix too small.");
        }

        for (int i = 0; i < numRow; i++) {
            for (int j = 0; j < numCol; j++) {
                target.elementData[(i + rowDest) * fColCnt + (j + colDest)] =
                        elementData[(i + rowSource) * fColCnt + (j + colSource)];
            }
        }
    }

    /**
     * Changes the size of this matrix dynamically.  If the size is increased
     * no data values will be lost.  If the size is decreased, only those data
     * values whose matrix positions were eliminated will be lost.
     *
     * @param nrows number of desired rows in this matrix
     * @param ncols number of desired columns in this matrix
     */
    public void setSize(int nrows, int ncols) {

        checkImmuted();

        if ((fRowCnt < 0) || (fColCnt < 0)) {
            throw new NegativeArraySizeException("nrows or ncols < 0");
        }

        int oldfRowCnt = this.fRowCnt;
        int oldfColCnt = this.fColCnt;
        int oldSize = this.fRowCnt * this.fColCnt;

        this.fRowCnt = nrows;
        this.fColCnt = ncols;

        int newSize = fRowCnt * fColCnt;
        String[] oldData = elementData;

        if (oldfColCnt == fColCnt) {

            // no need to reload elements.
            if (fRowCnt <= oldfRowCnt) {
                return;
            }

            // have to allocate memory.
            elementData = new String[newSize];

            // copy the old data.
            System.arraycopy(oldData, 0, elementData, 0, oldSize);

            // no need to pad. 0.0 is automaticly padded.
            // for (int i = oldSize; i < newSize; i++)
            //     elementData[i] = 0.0;
        } else {
            elementData = new String[newSize];

            setZero();

            for (int i = 0; i < oldfRowCnt; i++) {
                System.arraycopy(oldData, i * oldfColCnt, elementData, i * fColCnt, oldfColCnt);
            }
        }
    }

    /**
     * Returns the number of rows in this matrix.
     *
     * @return number of rows in this matrix
     */
    public int getNumRow() {
        return fRowCnt;
    }

    /**
     * Returns the number of colmuns in this matrix.
     *
     * @return number of columns in this matrix
     */
    public int getNumCol() {
        return fColCnt;
    }

    /**
     * Retrieves the value at the specified row and column of this matrix.
     *
     * @param row    the row number to be retrieved (zero indexed)
     * @param column the column number to be retrieved (zero indexed)
     * @return the value at the indexed element
     */
    public String getElement(int row, int column) {

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        if (fColCnt <= column) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " > matrix's fColCnt:"
                    + fColCnt);
        }

        if (column < 0) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " < 0");
        }

        return elementData[row * fColCnt + column];
    }

    public int getElementPos(int row, int col) {

        if (row >= getNumRow()) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:" + getNumRow());
        }

        if (col >= getNumCol()) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > matrix's fColCnt:" + getNumCol());
        }

        return (row * fColCnt) + col;
    }

    public void setElement(int row, int column, int value) {
        setElement(row, column, Integer.toString(value));
    }

    public void setElement(int row, int column, float value) {
        setElement(row, column, Float.toString(value));
    }

    public void setElement(int row, int column, float value, int precision) {
        if (!Float.isNaN(value)) {
            this.setElement(row, column, Printf.format(value, precision));
        } else {
            setElement(row, column, value);
        }
    }

    public void setElement(int row, int column, float[] values) {
        setElement(row, column, toString(values));
    }

    public void setElement(int row, int column, double[] values) {
        setElement(row, column, toString(values));
    }

    public void setElement(int row, int column, int[] values) {
        setElement(row, column, toString(values));
    }

    private static String toString(float[] values) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < values.length; i++) {
            buf.append(values[i]);
            if (i != values.length - 1) {
                buf.append(Constants.INTRA_FIELD_DELIM);
            }
        }

        return buf.toString();
    }

    private static String toString(int[] values) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < values.length; i++) {
            buf.append(values[i]);
            if (i != values.length - 1) {
                buf.append(Constants.INTRA_FIELD_DELIM);
            }
        }

        return buf.toString();
    }

    private static String toString(double[] values) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < values.length; i++) {
            buf.append(values[i]);
            if (i != values.length - 1) {
                buf.append(Constants.INTRA_FIELD_DELIM);
            }
        }

        return buf.toString();
    }


    public void setElement(int row, int column, double value) {
        setElement(row, column, Double.toString(value));
    }

    public void setElement(int row, int column, double value, int precision) {
        if (!Double.isNaN(value)) {
            this.setElement(row, column, Printf.format(value, precision));
        } else {
            setElement(row, column, value);
        }
    }

    public void setElement(int row, int column, final Collection strings, char delim) {

        final StringBuffer buf = new StringBuffer();
        for (Iterator it = strings.iterator(); it.hasNext();) {
            Object key = it.next();
            if (key != null) {
                buf.append(key);
            }

            if (it.hasNext()) {
                buf.append(delim);
            }

        }

        setElement(row, column, buf.toString());
    }

    public void setElement(int row, int column, boolean value) {
        this.setElement(row, column, Boolean.toString(value));
    }

    public void setElement(int row, int column, StringBuffer value) {
        if (value != null) {
            setElement(row, column, value.toString());
        }
    }

    public void setElement(int row, int column, MultiElement value) {
        if (value != null) {
            setElement(row, column, value.toString());
        }
    }

    public void setElement(int row, int column, Object value) {
        if (value != null) {
            setElement(row, column, value.toString());
        }
    }

    /**
     * Modifies the value at the specified row and column of this matrix.
     *
     * @param row    the row number to be modified (zero indexed)
     * @param column the column number to be modified (zero indexed)
     * @param value  the new matrix element value
     */
    public void setElement(int row, int column, String value) {

        checkImmuted();

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        if (fColCnt <= column) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " > matrix's fColCnt:"
                    + fColCnt);
        }

        if (column < 0) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " < 0");
        }

        elementData[row * fColCnt + column] = value;
    }

    public void setElement(int row, int column, final Number value) {
        if (value != null) {
            this.setElement(row, column, value.toString());
        }
    }

    public void setElement(int row, int column, final Vector v) {
        if (v != null) {
            this.setElement(row, column, v.toString(','));
        }
    }

    public void setElement(int row, int column, final Vector v, int precision) {
        if (v != null) {
            this.setElement(row, column, v.toString(',', precision));
        }
    }

    public void setElement(int row, int column, final LabelledVector lv) {
        if (lv != null) {
            this.setElement(row, column, lv.toString(','));
        }
    }

    public void setElement(int row, int column, final RankedList rl) {
        if (rl != null) {
            this.setElement(row, column, rl.toString(','));
        }
    }

    public void setElement(int row, int column, Set values) {

        if (values == null || values.size() == 0) {
            // dont set anything - let it be null
            //this.setElement(row, column, Constants.NULL);
        } else {

            StringBuffer buf = new StringBuffer();
            for (Iterator it = values.iterator(); it.hasNext();) {
                Object key = it.next();
                buf.append(key.toString());
                if (it.hasNext()) {
                    buf.append(Constants.INTRA_FIELD_DELIM);
                }
            }

            this.setElement(row, column, buf.toString());
        }
    }

    public void setElement(int row, int column, String[] values) {
        if (values == null || values.length == 0) {
            return;
        }

        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < values.length; i++) {
            buf.append(values[i]);
            if (i != values.length - 1) {
                buf.append(Constants.INTRA_FIELD_DELIM);
            }
        }

        this.setElement(row, column, buf.toString());
    }

    /**
     * a safe copy is returned.
     */
    public String[] getRow(int row) {

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        final String[] ret = new String[fColCnt];

        System.arraycopy(elementData, row * fColCnt, ret, 0, fColCnt);

        return ret;
    }

    /**
     * Copy the values from the array into the specified row of this
     * matrix.
     *
     * @param row   the row of this matrix into which the array values
     *              will be copied.
     * @param array the source array
     */
    public void setRow(int row, String array[]) {

        checkImmuted();

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row: " + row + " > matrix's fRowCnt: "
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row: " + row + " < 0");
        }

        if (array.length < fColCnt) {
            throw new ArrayIndexOutOfBoundsException("specified array length: " + array.length
                    + " < matrix's fColCnt: " + fColCnt);
        }

        System.arraycopy(array, 0, elementData, row * fColCnt, fColCnt);
    }

    /**
     * Copy the values from the array into the specified column of this
     * matrix.
     *
     * @param col   the column of this matrix into which the array values
     *              will be copied.
     * @param array the source array
     */
    public void setColumn(int col, String array[]) {

        checkImmuted();

        if (fColCnt <= col) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > matrix's fColCnt="
                    + fColCnt);
        }

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        // @todo assess if i should allow this
        /// imp imp IMP note that any missing elements are left untouched
        // if you change this make sure to check the other setCol methods
        /*
      int num;
      if (array.length < fRowCnt) {
          num = array.length;
      } else {
          num = fRowCnt;
      }
      */

        // see if this check should be disabled in favor of auto-bufferring
        if (array.length < fRowCnt) {
            throw new ArrayIndexOutOfBoundsException("array length:" + array.length
                    + " < matrix's fRowCnt:" + fRowCnt);
        }

        int num = fRowCnt;
        for (int i = 0; i < num; i++) {
            elementData[i * fColCnt + col] = array[i];
        }
    }

    public void setColumn(int col, Collection coll, boolean autoBuffer) {

        if (autoBuffer == false && coll.size() != getNumRow()) {
            throw new IllegalArgumentException("Number of objects in coll: " + coll.size() + " not equal to numRows: " + getNumRow() + " AND autobuffer was: " + autoBuffer);
        }

        String[] values = new String[getNumRow()];
        int cnt = 0;
        for (Iterator it = coll.iterator(); it.hasNext();) {
            values[cnt++] = it.next().toString();
        }

        this.setColumn(col, values);
    }

    public void setColumn(int col, Vector floats) {
        setColumn(col, floats.elementData);
    }

    public void setColumn(int col, Vector floats, boolean autoBuffer) {
        setColumn(col, floats.elementData, autoBuffer);
    }

    public void setColumn(int col, float floats[]) {
        this.setColumn(col, floats, false);
    }

    public void setColumn(int col, float floats[], boolean autoBuffer) {

        checkImmuted();

        if (fColCnt <= col) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > matrix's fColCnt="
                    + fColCnt);
        }

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        if (autoBuffer) { // @note dont exception out
            final String nan = Float.toString(Float.NaN);
            for (int i = 0; i < fRowCnt; i++) {
                if (i >= floats.length) {
                    elementData[i * fColCnt + col] = nan;
                } else {
                    elementData[i * fColCnt + col] = Float.toString(floats[i]);
                }
            }
        } else {

            if (floats.length < fRowCnt) {
                throw new ArrayIndexOutOfBoundsException("array length:" + floats.length
                        + " < matrix's fRowCnt:" + fRowCnt);
            }

            for (int i = 0; i < fRowCnt; i++) {
                elementData[i * fColCnt + col] = Float.toString(floats[i]);
            }
        }
    }

    public void setColumn(int col, int ints[]) {

        checkImmuted();

        if (fColCnt <= col) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > matrix's fColCnt="
                    + fColCnt);
        }

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        if (ints.length < fRowCnt) {
            throw new ArrayIndexOutOfBoundsException("array length:" + ints.length
                    + " < matrix's fRowCnt:" + fRowCnt);
        }

        for (int i = 0; i < fRowCnt; i++) {
            elementData[i * fColCnt + col] = Integer.toString(ints[i]);
        }
    }

    /**
     * a safe copy is returned.
     */
    public String[] getColumn(int col) {

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        String[] ret = new String[fRowCnt];

        for (int i = 0; i < fRowCnt; i++) {
            ret[i] = elementData[i * fColCnt + col];
        }

        return ret;
    }

    public Vector getColumnAsVector(int coln, boolean omitNulls) {
        TFloatArrayList floats = new TFloatArrayList();
        for (int r = 0; r < getNumRow(); r++) {
            String s = getElement(r, coln);
            if (s == null && omitNulls) {
                // do nothing
            } else if (s == null && !omitNulls) {
                floats.add(Float.NaN); // @note
            } else {
                floats.add(Float.parseFloat(s));
            }
        }

        return new Vector(floats);
    }


    public List getColumnL(int col) {

        if (fColCnt <= col) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > matrix's fColCnt:"
                    + fColCnt);
        }

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        List values = new ArrayList(fRowCnt);

        for (int i = 0; i < fRowCnt; i++) {
            values.add(elementData[i * fColCnt + col]);
        }

        return values;
    }

    /**
     * Returns a string that contains the values of this StringMatrix.
     *
     * @return the String representation
     */
    public String toString() {

        String nl = System.getProperty("line.separator");
        StringBuffer out = new StringBuffer("[");

        out.append(nl);

        for (int i = 0; i < fRowCnt; i++) {
            out.append("  [");

            for (int j = 0; j < fColCnt; j++) {
                if (0 < j) {
                    out.append("\t");
                }

                out.append(elementData[i * fColCnt + j]);
            }

            if (i + 1 < fRowCnt) {
                out.append("]");
                out.append(nl);
            } else {
                out.append("] ]");
            }
        }

        return out.toString();
    }

    /**
     * Returns a hash number based on the data values in this
     * object.  Two different StringMatrix objects with identical data values
     * (ie, returns true for equals(StringMatrix) ) will return the same hash
     * number.  Two objects with different data members may return the
     * same hash value, although this is not likely.
     *
     * @return the integer hash value
     */
    public int hashCode() {

        int hash = 0;

        for (int i = 0; i < fRowCnt * fColCnt; i++) {
            long bits = (elementData[i].hashCode());

            hash ^= (int) (bits ^ (bits >> 32));
        }

        return hash;
    }

    /**
     * Returns true if all of the data members of StringMatrix4d m1 are
     * equal to the corresponding data members in this StringMatrix4d.
     *
     * @param m1 The matrix with which the comparison is made.
     * @return true or false
     */
    public boolean equals(final StringMatrix m1) {

        if (m1 == null) {
            return false;
        }

        if (m1.fRowCnt != fRowCnt) {
            return false;
        }

        if (m1.fColCnt != fColCnt) {
            return false;
        }

        for (int i = 0; i < fRowCnt; i++) {
            for (int j = 0; j < fColCnt; j++) {
                if (elementData[i * fColCnt + j] != m1.elementData[i * fColCnt + j]) {
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * Returns true if the Object o1 is of type StringMatrix and all of the data
     * members of t1 are equal to the corresponding data members in this
     * StringMatrix.
     *
     * @param o1 the object with which the comparison is made.
     */
    public boolean equals(Object o1) {
        return (o1 != null) && (o1 instanceof StringMatrix) && equals((StringMatrix) o1);
    }

    /**
     * Sets the value of this matrix to sum of itself and matrix m1.
     *
     * @param m1 the other matrix
     */
    public void add(StringMatrix m1) {

        checkImmuted();

        if ((fRowCnt != m1.fRowCnt) || (fColCnt != m1.fColCnt)) {
            throw new IllegalArgumentException("this:(" + fRowCnt + "x" + fColCnt + ") != m1:("
                    + m1.fRowCnt + "x" + m1.fColCnt + ").");
        }

        for (int i = 0; i < fRowCnt * fColCnt; i++) {
            elementData[i] += m1.elementData[i];
        }
    }

    /**
     * Sets the value of this matrix to the matrix sum of matrices m1 and m2.
     *
     * @param m1 the first matrix
     * @param m2 the second matrix
     */
    public void add(StringMatrix m1, StringMatrix m2) {

        checkImmuted();

        if ((fRowCnt != m1.fRowCnt) || (fColCnt != m1.fColCnt)) {
            throw new IllegalArgumentException("this:(" + fRowCnt + "x" + fColCnt + ") != m1:("
                    + m1.fRowCnt + "x" + m1.fColCnt + ").");
        }

        if ((fRowCnt != m2.fRowCnt) || (fColCnt != m2.fColCnt)) {
            throw new IllegalArgumentException("this:(" + fRowCnt + "x" + fColCnt + ") != m2:("
                    + m2.fRowCnt + "x" + m2.fColCnt + ").");
        }

        for (int i = 0; i < fRowCnt * fColCnt; i++) {
            elementData[i] = m1.elementData[i] + m2.elementData[i];
        }
    }

    public void setColor(int row, int col, Color color) {
        checkImmuted();
        if (fColorMatrix == null) {
            this.fColorMatrix = new ObjectMatrix(getNumRow(), getNumCol());
        }

        this.fColorMatrix.setElement(row, col, color);
    }

    public Color getColor(int row, int col) {
        if (fColorMatrix == null) {
            return null;
        } else {
            Object obj = fColorMatrix.getElement(row, col);
            if (obj != null) {
                return (Color) obj;
            } else {
                return null;
            }
        }
    }

    /**
     * Transposes this matrix in place.
     * <p/>
     * Note: Not a mem eff way of doing this (see note below)
     */
    public void transpose() {
        checkImmuted();

        String[] array = new String[elementData.length];
        int pos = 0;

        for (int c = 0; c < this.getNumCol(); c++) {
            String[] col = new String[this.getNumRow()];
            _getColumnInPlace(c, col);
            System.arraycopy(col, 0, array, pos, col.length);
            pos += col.length;
        }

        int oldrowcnt = this.fRowCnt;
        int oldcolcnt = this.fColCnt;

        this.elementData = array;
        this.fColCnt = oldrowcnt;
        this.fRowCnt = oldcolcnt;
    }

    /**
     * Places the values of the specified column into the array parameter.
     *
     * @param col   the target column number
     * @param array the array into which the column values will be placed
     */
    private void _getColumnInPlace(int col, String array[]) {

        if (fColCnt <= col) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > matrix's fColCnt:"
                    + fColCnt);
        }

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        if (array.length < fRowCnt) {
            throw new ArrayIndexOutOfBoundsException("array.length:" + array.length
                    + " < matrix's fRowCnt=" + fRowCnt);
        }

        for (int i = 0; i < fRowCnt; i++) {
            array[i] = elementData[i * fColCnt + col];
        }
    }

    public void setImmutable() {
        this.fImmuted = true;
    }

    public boolean isImmutable() {
        return fImmuted;
    }

    private void checkImmuted() {

        if (fImmuted) {
            throw new ImmutedException();
        }
    }
}    // End StringMatrix

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
