/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.math;

import edu.mit.broad.genome.Printf;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.utils.ImmutedException;
import gnu.trove.TDoubleArrayList;
import gnu.trove.TFloatArrayList;
import gnu.trove.TFloatHashSet;
import gnu.trove.TIntArrayList;
import java.math.BigDecimal;

import java.util.*;

/**
 * Lots of code and ideas copied form GVector.
 * <p/>
 * vector -> encapsulates an array of flaots
 * <p/>
 * Vector is very very mutable!
 * <p/>
 * why not use GVectpor dfirecly -> uses double while we can do with float.
 * has some inapp methods.
 * <p/>
 * * A float precision, general, and dynamically resizeable one
 * dimensional vector class. Index numbering begins with zero.
 * <p/>
 * Notes:
 * - before adding methoids you might want to check if already mimpl in GVector
 * - note that the constrictors do a sysarray copy for safety
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class Vector {

    // note: only elementCount data in elementData are valid.
    // elementData.length is the allocated size.
    // invariant: elementData.length >= elementCount.
    // @note (as) making these protected for package friendly use (by Matrix etc)

    /**
     * @maint IMP IMP IMP IMP
     * When adding methods that alter datain this Vector, make sure
     * to do a immutability check
     * <p/>
     * The number of elements in this vector
     * <p/>
     * <p/>
     * The number of elements in this vector
     */

    /**
     * The number of elements in this vector
     */
    protected int elementCount;

    /**
     * The raw floats
     */
    protected float elementData[];
    protected BigDecimal elementDataB[];

    /**
     * Holds computed set of data
     */
    public transient ComputeSet computeset = new ComputeSet();

    /**
     * Flag for whether this Vector can have its values altered or not
     */
    private boolean fImmuted;

    // -------------------------------------------------------------------------------------------- //
    // ---------------------------------primary constructors --------------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    /**
     * Class Constructor.
     * Constructs a new generalized mathematic Vector with zero value (0.0)
     * elements; length represents the number of elements in the
     * vector.
     *
     * @param length number of elements in this vector.
     */
    public Vector(final int length) {
        elementCount = length;
        elementData = new float[length];    // will be initialized to 0.0
        elementDataB = new BigDecimal[length];
        if (computeset == null) {
            computeset = new ComputeSet();
        }
    }

    public Vector(final int length, final float value) {
        elementCount = length;
        elementData = new float[length];    // will be initialized to value
        for (int i = 0; i < elementCount; i++) {
            elementData[i] = value;
        }
        if (computeset == null) {
            computeset = new ComputeSet();
        }
    }

    public Vector(final float values[], final boolean share) {
        this.elementCount = values.length;
        if (share) {
            this.elementData = values;
        } else {
            this.elementData = new float[elementCount];
            System.arraycopy(values, 0, elementData, 0, elementCount);
        }
        if (computeset == null) {
            computeset = new ComputeSet();
        }
    }

    public Vector(final Vector v, final ScoreMode smode) {

        TFloatArrayList floats = new TFloatArrayList();
        for (int i = 0; i < v.getSize(); i++) {
            float elem = v.getElement(i);
            if (smode.isPostiveOnly() && XMath.isPositive(elem)) {
                floats.add(elem);
            } else if (smode.isNegativeOnly() && XMath.isNegative(elem)) {
                floats.add(elem);
            } else if (smode.isPostiveAndNegTogether()) {
                floats.add(v.elementData);
                break;
            } else {
                throw new IllegalArgumentException("Unkniwn ScoreMode: " + smode);
            }
        }

        elementCount = floats.size();
        elementData = floats.toNativeArray();
        if (computeset == null) {
            computeset = new ComputeSet();
        }
    }

    /**
     * Class Constructor.
     * Vector is constructed by appending elements of vectors in order.
     * So, length of this Vector is the sum of lengths of individual vectors.
     * Resulting vector  is NOT immutable even if the specified Vector was immutable
     * (permits manipulation)
     *
     * @param vectors
     */
    public Vector(final Vector[] vectors) {

        if (vectors == null) {
            throw new IllegalArgumentException("Param vectors cannot be null");
        }

        int length = 0;

        for (int i = 0; i < vectors.length; i++) {
            length += vectors[i].getSize();
        }

        this.elementCount = length;
        this.elementData = new float[length];    // will be initialized to 0.0

        int pos = 0;

        for (int i = 0; i < vectors.length; i++) {
            System.arraycopy(vectors[i].elementData, 0, elementData, pos, vectors[i].elementCount);
            pos += vectors[i].elementCount;
        }
        if (computeset == null) {
            computeset = new ComputeSet();
        }
    }

    // -------------------------------------------------------------------------------------------- //
    // ------------------------------------SECONDARY CONSTRUCTORS ---------------------------------- //
    // -------------------------------------------------------------------------------------------- //

    public Vector(final int values[]) {
        this(values.length);
        for (int i = 0; i < values.length; i++) {
            elementData[i] = values[i];
        }
    }

    /**
     * Class Constructor.
     * Data IS copied - not shared.
     * Constructs a new generalized mathematic Vector with zero
     * elements; length represents the number of elements in the
     * vector. !! this comment is a bug in Sun's API !!
     *
     * @param values the values for the new vector.
     */
    public Vector(final float values[]) {
        this(values, false);
    }

    public Vector(final Matrix m) {
        this(m.elementData, false);
    }

    public Vector(final List dels) {
        this(DoubleElement.toFloats(dels));
    }

    public Vector(final DoubleElement[] dels) {
        this(DoubleElement.toFloats(dels));
    }

    public Vector(final Vector v, final boolean share) {
        this(v.elementData, share);
    }

    /**
     * Class Constructor.
     * Data IS copied - not shared.
     *
     * @param values
     */
    public Vector(final double[] values) {

        this(values.length);

        // http://groups.google.com/groups?hl=en&lr=&ie=UTF-8&threadm=f11f7eb5.0207310307.16d8597a%40posting.google.com&rnum=3&prev=/groups%3Fq%3Dken%2Bstickler%26hl%3Den%26lr%3D%26ie%3DUTF-8%26selm%3Df11f7eb5.0207310307.16d8597a%2540posting.google.com%26rnum%3D3
        // arraycopy is fast because it can just blat bits around; but bit blatting
        // only works when the types of the source and destination array slices are
        // the *same*. I bet you think that's a blot.]
        // so lop based copy seems the only way + is fast enough
        for (int i = 0; i < values.length; i++) {
            elementData[i] = (float) values[i];
        }

        // doesnt work
        //System.arraycopy(values, 0, elementData, 0, elementCount);
    }

    public Vector(final String space_delim_floats) {
        this(ParseUtils.string2floats(space_delim_floats));
    }

    /**
     * Class Constructor.
     * Data IS copied - no sharing
     *
     * @param d
     */
    public Vector(final TDoubleArrayList d) {
        this(d.toNativeArray());
    }

    /**
     * Class Constructor.
     * Data IS copied - no sharing
     *
     * @param f
     */
    public Vector(final TFloatArrayList f) {
        this(f.toNativeArray());
    }

    public Vector(final TIntArrayList f) {
        this(f.toNativeArray());
    }

    /**
     * Class Constructor.
     * Data IS copied - no sharing
     * Constructs a new Vector and copies the initial values from
     * the parameter vector.
     * Resulting vector  is NOT immutable even if the specified Vector was immutable
     * (permits manipulation)
     *
     * @param vector the source for the new Vector's initial values
     */
    public Vector(Vector vector) {
        this(vector.elementCount, vector);    // al elements
    }

    /**
     * Class Constructor.
     * Data IS copied - no sharing
     * Constructs a new Vector and copies the initial values from
     * the parameter vector.
     * Resulting vector  is NOT immutable even if the specified Vector was immutable
     * (permits manipulation)
     *
     * @param vector the source for the new Vector's initial values
     */
    public Vector(final int numelementstouse, final Vector vector) {

        this(numelementstouse);

        System.arraycopy(vector.elementData, 0, elementData, 0, numelementstouse);

        //this.fImmutable = vector.fImmutable; // NO DONT -- see note above
    }

    /**
     * Class Constructor.
     * Data IS copied - no sharing.
     * Constructs a new Vector by copying length elements from the
     * array parameter.  The parameter length must be less than or
     * equal to vector.length.
     *
     * @param vector The array from which the values will be copied.
     * @param length The number of values copied from the array.
     */
    public Vector(final float vector[], final int length) {

        // ArrayIndexOutOfBounds occur if length > vector.legnth
        this(length);

        System.arraycopy(vector, 0, elementData, 0, elementCount);
    }

    /**
     * Class Constructor.
     * Creates a vector with elements from start to stop with increments of 1
     * (inclusive of start and stop)
     * i.e
     * # of elements = stop-start +1
     * element 0 -> gets value start
     * element 1 -> gets value start+1
     * ...
     * ...
     * element(stop) -> gets value stop
     */
    public Vector(final int start, final int stopInclusive) {

        this(stopInclusive - start + 1);

        int cnt = 0;

        for (int i = start; i < stopInclusive + 1; i++) {
            this.setElement(cnt++, i);
        }
    }

    /**
     * Class Constructor.
     * Vector is constructed by appending the elements of A and B
     * in order.
     * So, resulting vector is of lenth lenA + lenB.
     * Resulting vector  is NOT immutable even if the specified Vector was immutable
     * (permits manipulation)
     *
     * @param vectorA
     * @param vectorB
     */
    public Vector(final Vector vectorA, final Vector vectorB) {
        this(new Vector[]{vectorA, vectorB});
    }

    /**
     * Each element of tye resulting vector is a merge (on some stat such as mean or median)
     * Each input vector must be of same size
     *
     * @param vs
     * @return
     */
    public static Vector createByMerging(final Vector[] vs, final VectorFunctor.Statistic stat) {

        enforceEqualSize(vs, true);
        int size = vs[0].getSize();

        Vector merged = new Vector(size);

        for (int pos = 0; pos < size; pos++) {
            Vector v = new Vector(vs.length);
            for (int i = 0; i < vs.length; i++) {
                v.setElement(i, vs[i].getElement(pos));
            }

            double val = stat.getValue(v);
            merged.setElement(pos, (float) val);
        }

        return merged;

    }

    /**
     * sort of like a running mean
     * <p/>
     * Iinput vectors need NO be of equal size
     * Typically the inout vectors wil defer by 1 ele,knet increments (but no reason for this to be always true)
     *
     * @param vs
     * @return
     */
    public static Vector createRunningProfile(final Vector[] vs, final VectorFunctor.Statistic stat) {

        Vector merged = new Vector(vs.length);

        for (int i = 0; i < vs.length; i++) {
            merged.setElement(i, (float) stat.getValue(vs[i]));
        }

        return merged;
    }

    public static void enforceEqualSize(final Vector[] vs, final boolean barfIfZeroArraySize) {
        if (vs == null) {
            throw new IllegalArgumentException("Param vs cannot be null");
        }

        if (vs.length == 0) {
            if (barfIfZeroArraySize) {
                throw new IllegalArgumentException("Zero length vector array not allowed: " + vs.length);
            } else {
                return;
            }
        }

        boolean atleast1err = false;

        int size = vs[0].getSize();
        for (int i = 0; i < vs.length; i++) {
            if (vs[i].getSize() != size) {
                atleast1err = true;
                break;
            }

        }

        if (atleast1err) {
            StringBuffer buf = new StringBuffer("Not all vectors have the same size. Offenders are\n");

            for (int i = 0; i < vs.length; i++) {
                buf.append(i).append(" size: ").append(vs[i].getSize());
            }

            throw new IllegalArgumentException(buf.toString());
        }

    }

    public static int findMaxSize(final Vector[] vs) {

        if (vs == null) {
            throw new IllegalArgumentException("Param vs cannot be null");
        }

        int max = Integer.MIN_VALUE;

        for (int i = 0; i < vs.length; i++) {
            if (max < vs[i].getSize()) {
                max = vs[i].getSize();
            }
        }

        return max;

    }

    public static int findMinSize(final Vector[] vs) {

        if (vs == null) {
            throw new IllegalArgumentException("Param vs cannot be null");
        }

        int min = Integer.MAX_VALUE;

        for (int i = 0; i < vs.length; i++) {
            if (min > vs[i].getSize()) {
                min = vs[i].getSize();
            }
        }

        return min;

    }


    public void exp(double num) {

        checkImmutable();

        for (int i = 0; i < this.elementData.length; i++) {
            float f = this.elementData[i];
            // first arg raised to the power of the second arg
            this.elementData[i] = (float) Math.pow(num, f);
        }
    }

    /**
     * @return sum of all elements in the Vector
     */
    public double sum() {

        double sum = 0;

        for (int i = 0; i < elementCount; i++) {
            sum += elementData[i];
        }

        return sum;
    }


    public double sum_abs() {

        double sum = 0;

        for (int i = 0; i < elementCount; i++) {
            sum += Math.abs(elementData[i]);
        }

        return sum;
    }

    public double sum(int startIndexInclusive, int stopIndexEXclusive) {

        double sum = 0;

        for (int i = startIndexInclusive; i < stopIndexEXclusive; i++) {
            sum += elementData[i];
        }

        return sum;
    }

    public double sum(ScoreMode smode) {
        return _sum(smode)[0];
    }

    private double[] _sum(ScoreMode smode) {

        double sum = 0;
        int numUsed = 0;

        for (int i = 0; i < elementCount; i++) {
            if (smode.isPostiveAndNegTogether()) {
                return new double[]{sum(), getSize()};
            } else if (smode.isPostiveOnly() && XMath.isPositive(elementData[i])) {
                sum += elementData[i];
                numUsed++;
            } else if (smode.isNegativeOnly() && XMath.isNegative(elementData[i])) {
                sum += elementData[i];
                numUsed++;
            }
        }

        return new double[]{sum, numUsed};
    }

    public double sumNanSafe() {

        double sum = 0;

        for (int i = 0; i < elementCount; i++) {
            if (Float.isNaN(elementData[i])) { // dont add input Nans
                // keep going
            } else {
                sum += elementData[i];
            }
        }

        return sum;
    }

    /**
     * sum of the square root of every element
     * Cannot use if any of the elements are negative.
     *
     * @return
     */
    public double sqrtsum() {

        double sqrtsum = 0;

        for (int i = 0; i < elementCount; i++) {
            sqrtsum += Math.sqrt(elementData[i]);
        }

        return sqrtsum;
    }

    /**
     * Sum of the squares of every element
     *
     * @return
     */
    // IMP that return doubles!!
    public double squaresum() {

        double squaresum = 0;

        for (int i = 0; i < elementCount; i++) {
            squaresum += elementData[i] * elementData[i];
        }

        return squaresum;
    }

    public double squaresumNanSafe() {

        double squaresum = 0;

        for (int i = 0; i < elementCount; i++) {
            if (Float.isNaN(elementData[i])) { // dont add input Nans
                // keep going
            } else {
                squaresum += elementData[i] * elementData[i];
            }
        }

        return squaresum;
    }

    /**
     * @return arithmetic mean
     */
    public double mean() {
        computeset.mean = sum() / elementCount;
        return computeset.mean;
    }

    public double meanOrMedian(final boolean useMean) {
        if (useMean) {
            return mean();
        } else {
            return median();
        }
    }

    public double mad(final boolean useMean) {

        final double center = meanOrMedian(useMean);

        final Vector v = new Vector(elementData.length);
        for (int i = 0; i < elementData.length; i++) {
            v.setElement(i, Math.abs(elementData[i] - center));
        }

        return v.meanOrMedian(useMean);
    }

    public double mean(final int startIndexInclusive, final int stopIndexEXclusive) {
        return sum(startIndexInclusive, stopIndexEXclusive) / (stopIndexEXclusive - startIndexInclusive);
    }

    public double mean(final ScoreMode smode) {
        return _mean(smode)[0];
    }

    private double[] _mean(final ScoreMode smode) {
        double[] ss = _sum(smode);
        return new double[]{ss[0] / ss[1], ss[1]};
    }

    /**
     * Calculates the median
     */
    public double median() {
        if (elementCount == 0) {
            return Float.NaN;
        }

        // mangelos
        int aLen = elementCount;
        float[] v1 = new float[aLen];

        System.arraycopy(elementData, 0, v1, 0, aLen);
        Arrays.sort(v1);

        int ind = (aLen - 1) / 2;

        if (XMath.isEven(aLen)) {
            return (v1[ind] + v1[aLen / 2]) / 2;
        } else {
            return v1[ind];
        }
    }

    public int medianIndex() {

        if (elementCount == 0) {
            return -1;
        }

        // mangelos
        int aLen = elementCount;
        float[] v1 = new float[aLen];

        System.arraycopy(elementData, 0, v1, 0, aLen);
        Arrays.sort(v1);

        int ind = (aLen - 1) / 2;

        return ind;
    }

    private Vector fNaNless;

    public Vector toVectorNaNless() {

        if (fNaNless == null) {
            TFloatArrayList v1 = new TFloatArrayList();
            for (int i = 0; i < getSize(); i++) {
                float val = getElement(i);
                if (!Float.isNaN(val)) {
                    v1.add(val);
                }
            }

            if (v1.size() == 0) {
                fNaNless = this; // i.e no nans in the data
            } else {
                fNaNless = new Vector(v1);
                fNaNless.setImmutable();
            }
        }

        return fNaNless;
    }

    /**
     *
     */
    public float sig(double level) {

        float[] v1 = new float[elementCount];

        System.arraycopy(elementData, 0, v1, 0, elementCount);
        Arrays.sort(v1);

        int sigindx = (int) (level * elementCount);

        return elementData[sigindx];
    }

    /**
     * Trouble is the fixing low thing. Know how to do that
     * for stddevs but nto vars. So, get var, getsttdev,m fix and then get var!
     * <p/>
     * var = sum(x_i-mean)**2 / N
     *
     * @return the variance of the vector unbiased (div by n-1) or biased(divide by n)
     * @see http://davidmlane.com/hyperstat/A16252.html
     *      unbiased variance -> n-1
     *      biased variance -> using n
     *      Most commonly used is unbiased.
     *      MIT generally does NOT use biased -> FALSE
     */
    public double var(boolean biased, boolean fixlow) {

        if (fixlow) {
            stddev(biased, fixlow);
            double var = computeset.stddev * computeset.stddev;
            computeset.var = var;
            return var;
        } else {
            return _var(biased);
        }
    }


    /**
     * the real var calcs are here
     *
     * @param biased
     * @return
     */
    private double _var(boolean biased) {

        double oldvar = 0.0;
        int len = elementCount;

        if (!biased) {
            len--;
        }

        // Variance of 1 point is 0 (we are returning the biased variance in this case)
        if (len <= 0) {
            return oldvar;
        }

        double mean = mean();

        computeset.mean = mean;

        for (int i = 0; i < elementCount; i++) {
            double tmp = elementData[i] - mean;
            oldvar += tmp * tmp;
        }

        // DONT set computeset!
        return oldvar / len;
    }


    /**
     * @return The std aa_indev of vector
     *         // Some heuristics for adjusting variance based on data from affy chips
     *         // NOTE: problem occurs when we threshold to a value, then that artificially
     *         //   reduces the variance in the data
     *         /
     *         // First, we make the variance at least a fixed percent of the mean
     *         // If the mean is too small, then we use an absolute variance
     *         /
     *         // However, we don't want to bias our algs for affy data, e.g. we may
     *         // get data in 0..1 and in that case it is not appropriate to use
     *         // an absolute standard deviation of 10 - will kill the signal.
     *         /
     */
    public double stddev(boolean biased, boolean fixlow) {

        double stddev = Math.sqrt(_var(biased));    // @note call to _var and not var
        double mean = computeset.mean;                    // avoid recalc

        if (fixlow) {
            double minallowed = (0.20 * Math.abs(mean));

            // In the case of a zero mean, assume the mean is 1
            if (minallowed == 0) {
                minallowed = 0.20;
            }

            if (minallowed < stddev) {
                // keep orig
            } else {
                stddev = minallowed;
            }
        }

        computeset.stddev = stddev;

        return computeset.stddev;
    }

    /**
     * defined as stddev / mean
     *
     * @param biased
     * @param fixlow
     */
    public double vard(boolean biased, boolean fixlow) {
        return this.stddev(biased, fixlow) / this.mean();
    }


    public float max_TTTT(String name) {

        float max = Float.NEGATIVE_INFINITY;
        int maxat = -1;

        for (int i = 0; i < elementCount; i++) {
            if (elementData[i] > max) {
                max = elementData[i];
                maxat = i;
            }
        }

        if (computeset == null) {
            computeset = new ComputeSet();
        }
        computeset.max = max;
        computeset.maxindex = maxat;

        if (maxat == -1) {
            //System.out.println("WARNING: could not find max for: " + name + " " + toString(','));
            max = Float.NaN;
        }
        //System.out.println(max + "\t" + maxat);

        return max;
    }


    public float max_cached() {
        if (computeset == null || Float.isNaN((float) computeset.max)) {
            max();
        }

        return (float) computeset.max;
    }

    public float min_cached() {
        if (computeset == null || Float.isNaN((float) computeset.min)) {
            min();
        }

        return (float) computeset.min;
    }

    /**
     * @return the highest value element of this Vector
     */
    public float max() {

        float max = Float.NEGATIVE_INFINITY;
        int maxat = -1;

        for (int i = 0; i < elementCount; i++) {
            if (elementData[i] > max) {
                max = elementData[i];
                maxat = i;
            }
        }

        if (computeset == null) {
            computeset = new ComputeSet();
        }
        computeset.max = max;
        computeset.maxindex = maxat;

        if (maxat == -1) {
            //System.out.println("WARNING: could not find max for: " + toString(','));
            max = Float.NaN;
        }
        //System.out.println(max + "\t" + maxat);

        return max;
    }

    public int maxAtIndex() {

        float max = Float.NEGATIVE_INFINITY;
        int maxat = -1;

        for (int i = 0; i < elementCount; i++) {
            if (elementData[i] > max) {
                max = elementData[i];
                maxat = i;
            }
        }

        if (maxat == -1) {
            throw new IllegalStateException("No max found. Values are:\n" + Printf.outs(elementData));
        }

        computeset.max = max;
        computeset.maxindex = maxat;

        return maxat;
    }

    // not that max can be max +ve or max -ves
    public float maxDevFrom0() {

        float maxPositive = max();
        float minNegative = min();

        if (Math.abs(minNegative) > maxPositive) {
            return minNegative;
        } else {
            return maxPositive;
        }

    }

    public int maxDevFrom0Index() {

        float maxPositive = max();
        float minNegative = min();

        if (Math.abs(minNegative) > maxPositive) {
            return minAtIndex();
        } else {
            return maxAtIndex();
        }
    }

    /**
     * @return the highest value element of this Vector
     */
    public float min() {

        float min = Float.POSITIVE_INFINITY;

        for (int i = 0; i < elementCount; i++) {
            if (elementData[i] < min) {
                min = elementData[i];
            }
        }

        computeset.min = min;

        return min;
    }

    public int minAtIndex() {

        float min = Float.POSITIVE_INFINITY;
        int minat = -1;


        for (int i = 0; i < elementCount; i++) {
            if (elementData[i] < min) {
                min = elementData[i];
                minat = i;
            }
        }

        if (minat == -1) {
            throw new IllegalStateException("No max found. Values are:\n" + Printf.outs(elementData));
        }

        computeset.min = min;
        computeset.minindex = minat;
        return minat;
    }

    // Ifd there are more than 1 index that have the same min
    public TIntArrayList minAtIndex_all_of_them() {
        int minIndex = minAtIndex();
        float minValue = getElement(minIndex);
        return getElementIndices(minValue);
    }

    public TIntArrayList getElementIndices(float thatHaveThisValue) {
        TIntArrayList els = new TIntArrayList();
        for (int i = 0; i < elementData.length; i++) {
            if (elementData[i] == thatHaveThisValue) {
                els.add(i);
            }
        }

        return els;
    }

    public TFloatHashSet getUniqueValues() {
        return new TFloatHashSet(elementData);
    }

    /**
     * Returns the square root of the sum of the squares of this
     * vector (its length in n-dimensional space).
     *
     * @return length of this vector
     */
    public double norm() {
        checkImmutable();
        return Math.sqrt(normSquared());
    }

    /**
     * Returns the sum of the squares of this vector (its length
     * sqaured in n-dimensional space). <p>
     *
     * @return length squared of this vector
     */
    public double normSquared() {
        checkImmutable();
        double s = 0.0;

        for (int i = 0; i < elementCount; i++) {
            s += elementData[i] * elementData[i];
        }

        return s;
    }

    /**
     * Sets the value of this vector to the normalization of
     * vector v1.
     *
     * @param v1 the un-normalized vector
     */
    public void normalize(Vector v1) {
        checkImmutable();
        set(v1);
        normalize();
    }

    /**
     * Normalizes this vector in place.
     */
    public void normalize() {
        checkImmutable();
        double len = norm();

        // zero-div may happen.
        for (int i = 0; i < elementCount; i++) {
            elementData[i] /= len;
        }
    }

    /**
     * Normalizes to mean of 0 var of 1
     */
    public void normalizeToMeanStddev(boolean biased, boolean fixlow) {
        checkImmutable();

        float mean = (float) mean();
        float stddev = (float) stddev(biased, fixlow);

        subtract(mean);
        divide(stddev);
    }

    public void normalizeToMeanVar(boolean biased, boolean fixlow) {
        checkImmutable();

        float mean = (float) mean();
        float var = (float) var(biased, fixlow);

        subtract(mean);
        divide(var);
    }

    public void normalizeToMinZeroMaxOne() {
        checkImmutable();

        double max = max();
        double min = min();

        double denom = max - min;

        for (int i = 0; i < elementCount; i++) {
            elementData[i] = (float) ((elementData[i] - min) / denom);
        }

    }

    public void normToMaxOneMinNegOne_IgnoringZeroes_seperately(boolean include_zeroes_justin_style) {

        checkImmutable();

        // first split into pos and neg (IMP: ignore zeroes as they are null)
        TFloatArrayList pos = new TFloatArrayList();
        TFloatArrayList neg = new TFloatArrayList();

        for (int i = 0; i < getSize(); i++) {
            float val = getElement(i);

            if (include_zeroes_justin_style && val == 0) {
                pos.add(val);
                neg.add(val);
            }

            if (val > 0) {
                pos.add(val);
            } else if (val < 0) {
                neg.add(val);
            }
        }

        if (neg.size() == 0) {
            float pos_max = XMath.max(pos);
            float pos_min = XMath.min(pos);
            float pos_denom = pos_max - pos_min;

            // now norm
            for (int i = 0; i < getSize(); i++) {
                float val = getElement(i);
                if (val > 0) {
                    this.setElement(i, (val - pos_min) / pos_denom);
                } else if (val < 0) {
                    throw new IllegalStateException();
                } else {
                    this.setElement(i, 0);
                }
            }

        } else if (pos.size() == 0) {
            float neg_max = XMath.max(neg);
            float neg_min = XMath.min(neg);
            float neg_denom = neg_max - neg_min;

            for (int i = 0; i < getSize(); i++) {
                float val = getElement(i);
                if (val > 0) {
                    throw new IllegalStateException();
                } else if (val < 0) {
                    float d = (val - neg_max) / neg_denom;
                    this.setElement(i, -1 * Math.abs(d));
                } else {
                    this.setElement(i, 0);
                }
            }

        } else {
            float pos_max = XMath.max(pos);
            float pos_min = XMath.min(pos);
            float neg_max = XMath.max(neg);
            float neg_min = XMath.min(neg);

            float pos_denom = pos_max - pos_min;
            float neg_denom = neg_max - neg_min;

            // now norm
            for (int i = 0; i < getSize(); i++) {
                float val = getElement(i);
                if (val > 0) {
                    this.setElement(i, (val - pos_min) / pos_denom);
                } else if (val < 0) {
                    float d = (val - neg_max) / neg_denom;
                    this.setElement(i, -1 * Math.abs(d));
                } else {
                    if (Float.isNaN(val)) {
                        this.setElement(i, Float.NaN);
                    } else {
                        this.setElement(i, 0);
                    }
                }
            }
        }

    }

    public void normToMaxOneMinNegOne_IgnoringZeroes_together() {

        checkImmutable();

        // first split into pos and neg (IMP: ignore zeroes as they are null)
        TFloatArrayList pos = new TFloatArrayList();
        TFloatArrayList neg = new TFloatArrayList();

        for (int i = 0; i < getSize(); i++) {
            float val = getElement(i);
            if (val > 0) {
                pos.add(val);
            } else if (val < 0) {
                neg.add(val);
            }
        }

        if (neg.size() == 0) {
            float pos_max = XMath.max(pos);
            float pos_min = XMath.min(pos);
            float pos_denom = pos_max - pos_min;

            // now norm
            for (int i = 0; i < getSize(); i++) {
                float val = getElement(i);
                if (val > 0) {
                    this.setElement(i, (val - pos_min) / pos_denom);
                } else if (val < 0) {
                    throw new IllegalStateException();
                } else {
                    this.setElement(i, 0);
                }
            }

        } else if (pos.size() == 0) {
            float neg_max = XMath.max(neg);
            float neg_min = XMath.min(neg);
            float neg_denom = neg_max - neg_min;

            for (int i = 0; i < getSize(); i++) {
                float val = getElement(i);
                if (val > 0) {
                    throw new IllegalStateException();
                } else if (val < 0) {
                    float d = (val - neg_max) / neg_denom;
                    this.setElement(i, -1 * Math.abs(d));
                } else {
                    this.setElement(i, 0);
                }
            }

        } else {
            float pos_max = XMath.max(pos);
            float pos_min = XMath.min(pos);
            float pos_denom = pos_max - pos_min;
            float neg_max = XMath.max(neg);
            float neg_min = XMath.min(neg);
            float neg_denom = neg_max - neg_min;

            // now norm
            for (int i = 0; i < getSize(); i++) {
                float val = getElement(i);
                if (val > 0) {
                    this.setElement(i, (val - pos_min) / pos_denom);
                } else if (val < 0) {
                    float d = (val - neg_max) / neg_denom;
                    this.setElement(i, -1 * Math.abs(d));
                } else {
                    this.setElement(i, 0);
                }
            }
        }

    }

    public void normalizeToMinNegOneMaxOne() {
        checkImmutable();

        final double max = max();
        final double min = min();

        final double denom = max - min;

        for (int i = 0; i < elementCount; i++) {
            elementData[i] = (float) ((((elementData[i] - min) / denom) * 2) - 1);
        }

    }

    //  x - mean(x) / std(x)
    public void normalizeByZScore(final boolean useMean) {
        checkImmutable();

        final double center = meanOrMedian(useMean);
        final double std = stddev(true, false);

        for (int i = 0; i < elementCount; i++) {
            elementData[i] = (float) ((elementData[i] - center) / std);
        }

    }

    public void normalizeByZScore_mad(boolean useMean) {
        checkImmutable();
        final double center = meanOrMedian(useMean);
        final double mad = mad(useMean);

        for (int i = 0; i < elementCount; i++) {
            elementData[i] = (float) ((elementData[i] - center) / mad);
            if (Float.isInfinite(elementData[i])) {
                elementData[i] = 0;
            }
        }

    }

    // replace each point by the average of its ratio to all other points
    public Vector normalizeByMFC(final boolean useMean, float min_val, float max_val) {

        float[] mfs = new float[getSize()];

        for (int i = 0; i < getSize(); i++) {
            float it = getElement(i);
            if (it < min_val) {
                it = min_val;
            }

            if (it > max_val) {
                it = max_val;
            }

            final TFloatArrayList ratios = new TFloatArrayList();
            for (int j = 0; j < getSize(); j++) {
                if (i != j) {
                    float elem = getElement(j);

                    if (elem < min_val) {
                        elem = min_val;
                    }

                    if (elem > max_val) {
                        elem = max_val;
                    }

                    float ratio = it / elem;
                    ratios.add(ratio);
                }
            }
            mfs[i] = XMath.meanmed(ratios, useMean);
        }

        return new Vector(mfs);
    }

    // @note doesnt necc make sense because if only 1 on one side then norm is wierd
    public void normalizeByGsea() {
        checkImmutable();

        final Vector onlyPos = extract(ScoreMode.POS_ONLY);
        final Vector onlyNeg = extract(ScoreMode.NEG_ONLY);

        float meanPos = (float) onlyPos.mean();

        float meanNeg = (float) onlyNeg.mean();

        for (int i = 0; i < elementCount; i++) {

            if (XMath.isPositive(elementData[i])) {
                elementData[i] = elementData[i] / meanPos;
            } else {
                elementData[i] = elementData[i] / meanNeg;
            }
        }
    }

    private Vector fSortedVectorForPercentiles;

    public float getPercentile(float per) {

        int elementIndex = (int) ((float) getSize() * per);

        // Then get elements, sort them and return the per

        if (fSortedVectorForPercentiles == null) {
            this.fSortedVectorForPercentiles = new Vector(this);
            fSortedVectorForPercentiles.sort(SortMode.REAL, Order.ASCENDING); // high percentile is a good score
        }

        return fSortedVectorForPercentiles.getElement(elementIndex);
    }

    /**
     * Sets the value of this vector to the scalar multiplication of
     * the scale factor with the vector v1.
     *
     * @param s  the scalar value
     * @param v1 the source vector
     */
    public void scale(float s, Vector v1) {
        checkImmutable();
        set(v1);
        scale(s);
    }

    /**
     * Scales this vector by the scale factor s.
     *
     * @param s the scalar value
     */
    public void scale(float s) {
        checkImmutable();

        for (int i = 0; i < elementCount; i++) {
            elementData[i] *= s;
        }
    }

    /**
     * Sets the value of this vector to the scalar multiplication by
     * s of vector v1 plus vector v2 (this = s*v1 + v2).
     *
     * @param s  the scalar value
     * @param v1 the vector to be multiplied
     * @param v2 the vector to be added
     */
    public void scaleSum(float s, Vector v1, Vector v2) {
        checkImmutable();
        float[] v1data = v1.elementData;
        float[] v2data = v2.elementData;

        if (elementCount != v1.elementCount) {
            throw new ArrayIndexOutOfBoundsException("this.size:" + elementCount + " != v1's size:"
                    + v1.elementCount);
        }

        if (elementCount != v2.elementCount) {
            throw new ArrayIndexOutOfBoundsException("this.size:" + elementCount + " != v2's size:"
                    + v2.elementCount);
        }

        for (int i = 0; i < elementCount; i++) {
            elementData[i] = s * v1data[i] + v2data[i];
        }
    }

    /**
     * Sets the value of every element in this vector to sum of itself and the
     * corresponding element in the  specified vector
     *
     * @param vector the second vector
     */
    public void sum(Vector vector) {
        checkImmutable();
        float[] v1data = vector.elementData;

        if (elementCount != vector.elementCount) {
            throw new ArrayIndexOutOfBoundsException("this.size:" + elementCount + " != v2's size:"
                    + vector.elementCount);
        }

        for (int i = 0; i < elementCount; i++) {
            elementData[i] += v1data[i];
        }
    }

    public void sumNanSafe(Vector vector) {
        checkImmutable();
        float[] v1data = vector.elementData;

        if (elementCount != vector.elementCount) {
            throw new ArrayIndexOutOfBoundsException("this.size:" + elementCount + " != v2's size:"
                    + vector.elementCount);
        }

        for (int i = 0; i < elementCount; i++) {

            if (Float.isNaN(v1data[i])) { // dont add input Nans
                continue;
            }

            if (Float.isNaN(elementData[i])) { // if i'm a Nan, set me to the input whatever it is
                elementData[i] = v1data[i];
            } else {
                elementData[i] += v1data[i]; // regular
            }
        }
    }

    public void divide(float factor) {
        checkImmutable();
        for (int i = 0; i < elementCount; i++) {
            elementData[i] = elementData[i] / factor;
        }
    }

    public void multiply(float factor) {
        checkImmutable();
        for (int i = 0; i < elementCount; i++) {
            elementData[i] = elementData[i] * factor;
        }
    }

    public void divide(final Vector v) {
        checkImmutable();
        enforceEqualSize(new Vector[]{this, v}, true);
        for (int i = 0; i < elementCount; i++) {
            elementData[i] = elementData[i] / v.elementData[i];
        }
    }

    // (p-v) / ( (p+v) / 2)
    public void amplitude(final Vector v) {
        checkImmutable();
        enforceEqualSize(new Vector[]{this, v}, true);
        for (int i = 0; i < elementCount; i++) {
            float numr = elementData[i] - v.elementData[i];
            float denr = (elementData[i] + v.elementData[i]) / 2.0f;
            elementData[i] = numr / denr;
        }
    }

    public void subtract(float factor) {
        checkImmutable();
        for (int i = 0; i < elementCount; i++) {
            elementData[i] = elementData[i] - factor;
        }
    }

    public void add(float factor) {
        checkImmutable();
        for (int i = 0; i < elementCount; i++) {
            elementData[i] = elementData[i] + factor;
        }
    }

    /**
     * Sets the value of this vector to the vector sum of vectors
     * vector1 and vector2.
     *
     * @param vector1 the first vector
     * @param vector2 the second vector
     */
    public void sum(Vector vector1, Vector vector2) {
        checkImmutable();
        set(vector1);
        sum(vector2);
    }

    /**
     * sum of products of elements of 2 vectors
     * Sum xi * yi
     */
    public double sumprod(Vector y) {

        if (this.getSize() != y.getSize()) {
            throw new RuntimeException("Unequal vector sizes x: " + getSize() + " y: "
                    + y.getSize());
        }

        double prodsum = 0;

        for (int i = 0; i < this.getSize(); i++) {
            prodsum += this.getElement(i) * y.getElement(i);
        }

        return prodsum;
    }

    /**
     * Negates the value of this vector: this = -this.
     */
    public void negate() {

        checkImmutable();

        for (int i = 0; i < elementCount; i++) {
            elementData[i] = -elementData[i];
        }
    }

    /**
     * Sets all the values in this vector to zero.
     */
    public void zero() {

        checkImmutable();

        for (int i = 0; i < elementCount; i++) {
            elementData[i] = 0;
        }
    }

    /**
     * shortens the vector
     */
    public void truncate(int newsize) {

        checkImmutable();

        if (newsize == elementCount) {
            return;    // no change needed
        }

        if (newsize > this.getSize()) {
            throw new IllegalArgumentException("Truncated size: " + newsize
                    + " cannot be more than current size: " + getSize());
        }

        float[] oldData = elementData;

        elementData = new float[newsize];

        System.arraycopy(oldData, 0, elementData, 0, newsize);

        elementCount = newsize;
    }

    /**
     * Vector gets sorted internally
     * from min to max.
     * (the real value of every element is used)
     * Vector data order DOES change
     */
    public void sort() {
        checkImmutable();
        Arrays.sort(elementData);
    }

    /**
     * Vector gets sorted internally from max to min
     * (the real value of every element is used)
     * Vector elements DO change
     */
    public void revsort() {
        checkImmutable();
        Arrays.sort(elementData);
        reverse();
    }

    /**
     * The vectors data is NOT changed
     * changes the vector so that it now has only positive values -- NO!
     */
    public void abssort() {
        abs();
        sort();
    }

    public void abs() {
        //checkImmutable();

        for (int i = 0; i < elementData.length; i++) {
            elementData[i] = Math.abs(elementData[i]);
        }
    }

    /**
     * @param mode
     * @param order
     */
    public void sort(SortMode mode, Order order) {

        checkImmutable();

        if (mode.isAbsolute()) {
            abs();
        }

        if (order.isAscending()) {
            sort();
        } else {
            revsort();
        }

    }

    public Vector removeValues(final float removeElementsThatEqualThisValue) {
        TFloatArrayList remaining = new TFloatArrayList();
        for (int i = 0; i < this.getSize(); i++) {
            float score = this.getElement(i);
            if (score != removeElementsThatEqualThisValue) {
                remaining.add(score);
            }
        }

        return new Vector(remaining);
    }

    public void replaceValues(final float replaceElementsThatEqualThisValue, final float withThisValue) {
        checkImmutable();
        for (int i = 0; i < this.getSize(); i++) {
            float val = this.getElement(i);
            if (Float.isNaN(val) && Float.isNaN(replaceElementsThatEqualThisValue)) {
                this.setElement(i, withThisValue);
            }

            if (val == replaceElementsThatEqualThisValue) {
                this.setElement(i, withThisValue);
            }
        }
    }

    /**
     * The vector's elements are reversed.
     * Vector elements DO change.
     */
    public void reverse() {

        checkImmutable();

        float tmp[] = new float[elementData.length];
        int start = elementCount - 1;

        for (int i = 0; i < elementCount; i++) {
            tmp[start - i] = elementData[i];
        }

        elementData = tmp;
    }

    /**
     * Data IS replicated
     * Specified Vector param is NOT altered.
     *
     * @return
     */
    public static DoubleElement[] toDoubleElements(Vector v) {

        DoubleElement[] dels = new DoubleElement[v.getSize()];

        for (int i = 0; i < v.getSize(); i++) {
            dels[i] = new DoubleElement(i, v.getElement(i));
        }

        return dels;
    }

    /**
     * Data IS replicated
     * Specified Vector param is NOT altered.
     *
     * @return
     */
    public static AddressedVector toIndexedVector(final Vector v) {
        return new AddressedVector(v, false);
    }

    public int[] toRanks(final SortMode sort, final Order order, boolean startFromOne) {

        int prefix = 0;
        if (startFromOne) {
            prefix = 1;
        }

        DoubleElement[] dels = _sort(new AddressedVector(this, true), sort, order);
        int[] ranks = new int[this.getSize()];
        for (int i = 0; i < dels.length; i++) {
            ranks[i] = dels[i].fIndex + prefix;
        }

        return ranks;
    }

    public Vector toRanksV(final SortMode sort, final Order order, boolean startFromOne) {
        return new Vector(toRanks(sort, order, startFromOne));
    }

    private static DoubleElement[] _sort(final AddressedVector av, final SortMode sort, final Order order) {
        DoubleElement[] dels = new DoubleElement[av.getSize()];
        for (int i = 0; i < av.getSize(); i++) {
            dels[i] = new DoubleElement(av.getAddress(i), av.getScore(i));
        }

        return DoubleElement.sort(sort, order, dels);
    }


    /**
     * Sets the value of this vector to the values found in the
     * array parameter. The array should be at least equal in
     * length to the number of elements in the vector.
     *
     * @param vector the source array
     */
    public void set(double vector[]) {

        checkImmutable();

        // note: only this.elementCount data is copied.(no auto-grow)
        System.arraycopy(vector, 0, elementData, 0, elementCount);
    }

    /**
     * same as set but doesnt do a systemcopy. so memory space shared
     *
     * @see set(double vector[])
     */
    public void setDirectly(float[] data) {

        checkImmutable();

        this.elementData = data;
        this.elementCount = data.length;
    }

    /**
     * Sets the value of this vector to the values found in
     * vector vector.
     *
     * @param vector the source vector
     */
    public void set(Vector vector) {

        checkImmutable();

        // note: only this.elementCount data is copied.(no auto-grow)
        System.arraycopy(vector.elementData, 0, elementData, 0, elementCount);
    }

    /**
     * Returns the number of elements in this vector.
     *
     * @return number of elements in this vector
     */
    public int getSize() {
        return elementCount;
    }

    public int getSize(ScoreMode smode) {

        if (smode.isPostiveAndNegTogether()) {
            return getSize();
        } else if (smode.isPostiveAndNegSeperately()) {
            throw new IllegalArgumentException("Not a valid scoee mode");
        } else if (smode.isPostiveOnly()) {
            int cnt = 0;
            for (int i = 0; i < getSize(); i++) {
                if (XMath.isPositive(getElement(i))) {
                    cnt++;
                }
            }

            return cnt;

        } else if (smode.isNegativeOnly()) {
            int cnt = 0;
            for (int i = 0; i < getSize(); i++) {
                if (XMath.isNegative(getElement(i))) {
                    cnt++;
                }
            }

            return cnt;
        } else {
            throw new IllegalArgumentException("Unknown score mode: " + smode);
        }

    }

    public int getSizeNanSafe() {
        int size = 0;
        for (int i = 0; i < elementCount; i++) {
            if (Float.isNaN(elementData[i])) {
                // keep going
            } else {
                size++;
            }
        }

        return size;
    }

    /**
     * Retrieves the value at the specified index value of this
     * vector.
     *
     * @param index the index of the element to retrieve (zero indexed)
     * @return the value at the indexed element
     */
    public float getElement(int index) {

        try {
            return elementData[index];
        } catch (ArrayIndexOutOfBoundsException e) {
            throw new ArrayIndexOutOfBoundsException("index:" + index + " must be in [0, "
                    + (elementCount - 1) + "]");
        }
    }
    
    public BigDecimal getElementB(int index) {

        try {
            return elementDataB[index];
        } catch (ArrayIndexOutOfBoundsException e) {
            throw new ArrayIndexOutOfBoundsException("index:" + index + " must be in [0, "
                    + (elementCount - 1) + "]");
        }
    }    

    /**
     * Modifies the value at the specified index of this vector.
     *
     * @param index the index if the element to modify (zero indexed)
     * @param value the new vector element value
     */
    public void setElement(int index, float value) {

        checkImmutable();

        try {
            elementData[index] = value;
        } catch (ArrayIndexOutOfBoundsException e) {
            throw new ArrayIndexOutOfBoundsException("index:" + index + " must be in [0, "
                    + (elementCount - 1) + "]");
        }
    }
    
    public void setElementB(int index, BigDecimal value) {

        checkImmutable();

        try {
            elementDataB[index] = value;
        } catch (ArrayIndexOutOfBoundsException e) {
            throw new ArrayIndexOutOfBoundsException("index:" + index + " must be in [0, "
                    + (elementCount - 1) + "]");
        }
    }    

    public void setElement(int index, double value) {
        setElement(index, (float) value);
    }


    public String[] toStrings() {
        final String[] ss = new String[getSize()];
        for (int i = 0; i < getSize(); i++) {
            ss[i] = getElement(i) + "";
        }

        return ss;
    }

    public String toString() {

        if (getSize() == 0) {
            return "";
        }

        StringBuffer buf = new StringBuffer();

        buf.append("(");

        for (int i = 0; i < elementCount - 1; i++) {
            buf.append(elementData[i]);
            buf.append(",");
        }

        buf.append(elementData[elementCount - 1]);
        buf.append(")");

        return buf.toString();
    }

    public String toString(char delim) {

        if (getSize() == 0) {
            return "";
        }

        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < elementCount - 1; i++) {
            if (!Float.isNaN(elementData[i])) {
                buf.append(elementData[i]);
            }
            buf.append(delim);
        }

        if (!Float.isNaN(elementData[elementCount - 1])) {
            buf.append(elementData[elementCount - 1]);
        } else {
            buf.append(delim);
        }

        return buf.toString();
    }

    public String toString(char delim, int precision) {

        if (getSize() == 0) {
            return "";
        }

        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < elementCount - 1; i++) {
            if (!Float.isNaN(elementData[i])) {
                buf.append(Printf.format(elementData[i], precision));
            }
            buf.append(delim);
        }

        if (!Float.isNaN(elementData[elementCount - 1])) {
            buf.append(Printf.format(elementData[elementCount - 1], precision));
        } else {
            buf.append(delim);
        }

        return buf.toString();
    }

    public java.util.Vector toUtilVector() {

        java.util.Vector v = new java.util.Vector(getSize());

        for (int i = 0; i < elementCount; i++) {
            v.add(new Float(elementData[i]));
        }

        return v;
    }

    /**
     * returns a safe copy
     */
    public float[] toArray() {

        float[] fl = new float[elementData.length];

        System.arraycopy(elementData, 0, fl, 0, elementData.length);

        return fl;
    }

    public double[] toArrayDouble() {

        double[] dest = new double[elementData.length];
        for (int i = 0; i < elementData.length; i++) {
            dest[i] = elementData[i];
        }

        return dest;
    }

    /**
     * @return each elelentsint value. copy safe.
     */
    public int[] toArrayInt() {

        int[] in = new int[elementData.length];

        for (int i = 0; i < elementData.length; i++) {
            in[i] = (int) elementData[i];
        }

        return in;
    }

    public Vector toPositiveOnlyElementsV() {
        return new Vector(toPositiveOnlyElements());
    }

    public TFloatArrayList toPositiveOnlyElements() {

        TFloatArrayList tfl = new TFloatArrayList();

        for (int i = 0; i < elementData.length; i++) {
            if (XMath.isPositive(elementData[i])) {
                tfl.add(elementData[i]);
            }
        }

        return tfl;
    }

    public Vector toNegativeOnlyElementsV() {
        return new Vector(toNegativeOnlyElements());
    }

    public TFloatArrayList toNegativeOnlyElements() {

        TFloatArrayList tfl = new TFloatArrayList();

        for (int i = 0; i < elementData.length; i++) {
            if (XMath.isPositive(elementData[i])) {
                tfl.add(elementData[i]);
            }
        }

        return tfl;
    }

    /**
     * Returns a hash number based on the data values in this
     * object.  Two different Vector objects with identical data values
     * (ie, returns true for equals(Vector) ) will return the same hash
     * number.  Two objects with different data members may return the
     * same hash value, although this is not likely.
     *
     * @return the integer hash value
     */
    public int hashCode() {

        int hash = 0;

        for (int i = 0; i < elementCount; i++) {
            long bits = Double.doubleToLongBits(elementData[i]);

            hash ^= (int) (bits ^ (bits >> 32));
        }

        return hash;
    }

    /**
     * Returns true if all of the data members of Vector vector1
     * are equal to the corresponding data members in this Vector.
     *
     * @param vector1 The vector with which the comparison is made.
     * @return true or false
     */
    public boolean equals(Vector vector1) {

        if (vector1 == null) {
            return false;
        }

        if (elementCount != vector1.elementCount) {
            return false;
        }

        float[] v1data = vector1.elementData;

        for (int i = 0; i < elementCount; i++) {
            if (elementData[i] != v1data[i]) {
                return false;
            }
        }

        return true;
    }

    /**
     * Returns true if the Object o1 is of type Vector and all of the data
     * members of t1 are equal to the corresponding data members in this
     * Vector.
     *
     * @param o1 the object with which the comparison is made.
     */
    public boolean equals(Object o1) {
        return (o1 != null) && (o1 instanceof Vector) && equals((Vector) o1);
    }

    /**
     * Returns true if the L-infinite distance between this vector
     * and vector v1 is less than or equal to the epsilon parameter,
     * otherwise returns false. The L-infinite distance is equal
     * to MAX[abs(x1-x2), abs(y1-y2), . . . ]. <p>
     *
     * @param v1      The vector to be compared to this vector
     * @param epsilon the threshold value
     */
    public boolean epsilonEquals(Vector v1, double epsilon) {

        if (elementCount != v1.elementCount) {
            return false;
        }

        float[] v1data = v1.elementData;

        for (int i = 0; i < elementCount; i++) {
            if (Math.abs(elementData[i] - v1data[i]) > epsilon) {
                return false;
            }
        }

        return true;
    }

    /**
     * Returns the dot product of this vector and vector v1.
     *
     * @param v1 the other vector
     * @return the dot product of this and v1
     */
    public double dot(Vector v1) {

        float[] v1data = v1.elementData;

        if (elementCount != v1.elementCount) {
            throw new IllegalArgumentException("this.size:" + elementCount + " != v1.size:"
                    + v1.elementCount);
        }

        double sum = 0.0;

        for (int i = 0; i < elementCount; ++i) {
            sum += elementData[i] * v1data[i];
        }

        return sum;
    }

    /**
     * Returns the (n-space) angle in radians between this vector
     * and the vector parameter; the return value is constrained to
     * the range [0,PI].
     *
     * @param v1 The other vector
     * @return The angle in radians in the range [0,PI]
     */
    public double angle(Vector v1) {
        return Math.acos(dot(v1) / norm() / v1.norm());
    }

    /**
     * Linearly interpolates between vectors v1 and v2 and places
     * the result into this tuple: this = (1-alpha)*v1 + alpha*v2.
     *
     * @param v1    the first vector
     * @param v2    the second vector
     * @param alpha the alpha interpolation parameter
     *              <p/>
     *              Linearly interpolates between this vector and vector v1 and
     *              places the result into this tuple:
     *              this = (1-alpha)*this + alpha*v1.
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     * @deprecated the double version of this method should be used.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     *             <p/>
     *             Linearly interpolates between this vector and vector v1 and
     *             places the result into this tuple:
     *             this = (1-alpha)*this + alpha*v1.
     */

    /*
    public void interpolate(Vector v1, Vector v2, float alpha) {
        interpolate(v1, v2, alpha);
    }
    */

    /**
     * Linearly interpolates between this vector and vector v1 and
     * places the result into this tuple:
     * this = (1-alpha)*this + alpha*v1.
     *
     * @param v1    the first vector
     * @param alpha the alpha interpolation parameter
     */
    public void interpolate(Vector v1, float alpha) {

        float[] v1data = v1.elementData;

        if (elementCount != v1.elementCount) {
            throw new IllegalArgumentException("this.size:" + elementCount + " != v1.size:"
                    + v1.elementCount);
        }

        float beta = (float) (1.0 - alpha);

        for (int i = 0; i < elementCount; ++i) {
            elementData[i] = beta * elementData[i] + alpha * v1data[i];
        }
    }

    /**
     * binning
     */
    public void bin(Range[] ranges, boolean enforceAllElementsMustBeBinned) {

        // note that a point on the boundary will arbitarilly be put into the first Range encountered
        int numBinned = 0;
        for (int i = 0; i < getSize(); i++) {
            boolean binned = false;
            for (int j = 0; j < ranges.length; j++) {
                if (ranges[j].isInRange(elementData[i])) {
                    ranges[j].increment();
                    binned = true;
                    numBinned++;
                    break;
                }
            }

            if (enforceAllElementsMustBeBinned && !binned) {
                throw new RuntimeException("Value was not binned as it didnt fall in any of the Ranges: " + elementData[i]);
            }
        }

        // cant do this as the ranges might already have stuff inthem
        //int tot = RangeUtils.getTotalCount(ranges);
        int tot = numBinned;
        if (enforceAllElementsMustBeBinned && tot != elementData.length) {
            throw new IllegalStateException("Bad binning! Binned: " + tot + " but elements in Vector: " + elementData.length);
        }

    }

    public static Map floats2VectorMap(Map probeNameTroveFloatsMap) {

        Map map = new HashMap(probeNameTroveFloatsMap.size());
        Iterator it = probeNameTroveFloatsMap.keySet().iterator();

        while (it.hasNext()) {
            String probeName = it.next().toString();
            TFloatArrayList corrs = (TFloatArrayList) probeNameTroveFloatsMap.get(probeName);
            map.put(probeName, new Vector(corrs));
        }

        // paranoid check
        if (map.size() != probeNameTroveFloatsMap.size()) {
            throw new IllegalStateException("Wierdly corr output map has diff size: " + map.size() + " than input map: " + probeNameTroveFloatsMap.size());
        }

        return map;
    }


    /**
     * PNormalizes this vector in place.
     * min set to -1 and max set to +1
     * <p/>
     * size of vect doeesnt matter:
     *
     * @todo what is the correct terminology for this and how does it relate to normalize?
     */
    public void pnormalize() {

        checkImmutable();

        float pmin = -1;
        float min = this.min();
        float max = this.max();

        for (int i = 0; i < elementCount; i++) {
            elementData[i] = pmin + ((elementData[i] - min) / (max - min)) * 2;
        }
    }

    // -------------------------------------------------------------------------------------------- //
    // ----------------- PUBLIC STATIC UTLITY METHODS THAT WORK ON ARRAY OF VECTORS --------------- //
    // -------------------------------------------------------------------------------------------- //

    public static Vector mean(final Vector[] vectors) {

        enforceEqualSize(vectors, true);
        int size = vectors[0].getSize();
        Vector mean = new Vector(size);
        for (int i = 0; i < size; i++) {
            float[] scores = new float[vectors.length];
            for (int j = 0; j < vectors.length; j++) {
                scores[j] = vectors[j].getElement(i);
            }

            mean.setElement(i, XMath.mean(scores));
        }

        return mean;
    }

    public static Vector max(final Vector[] vectors) {

        enforceEqualSize(vectors, true);
        int size = vectors[0].getSize();
        Vector mean = new Vector(size);
        for (int i = 0; i < size; i++) {
            float[] scores = new float[vectors.length];
            for (int j = 0; j < vectors.length; j++) {
                scores[j] = vectors[j].getElement(i);
            }

            mean.setElement(i, XMath.max(scores));
        }

        return mean;
    }

    public static Vector median(final Vector[] vectors) {

        enforceEqualSize(vectors, true);
        int size = vectors[0].getSize();
        Vector median = new Vector(size);
        for (int i = 0; i < size; i++) {
            float[] scores = new float[vectors.length];
            for (int j = 0; j < vectors.length; j++) {
                scores[j] = vectors[j].getElement(i);
            }

            median.setElement(i, XMath.median(scores));
        }

        return median;
    }

    public static String getSummaryStatistics(final Vector[] vectors) {

        StringBuffer buf = new StringBuffer("Number of Vectors : ").append(vectors.length).append('\n');
        double min = Float.POSITIVE_INFINITY;
        double max = Float.NEGATIVE_INFINITY;

        for (int i = 0; i < vectors.length; i++) {
            if (min > vectors[i].min()) {
                min = vectors[i].computeset.min;
            }

            if (max < vectors[i].max()) {
                max = vectors[i].computeset.max;
            }
        }

        buf.append("Absolute min: ").append(min).append('\n');
        buf.append("Absolute max: ").append(max).append('\n');

        return buf.toString();
    }

    /**
     * @param vectors
     * @return Max size of all specified vectors
     */
    public static int maxSize(Vector[] vectors) {

        int max = Integer.MIN_VALUE;

        for (int i = 0; i < vectors.length; i++) {
            if (max < vectors[i].getSize()) {
                max = vectors[i].getSize();
            }
        }

        return max;
    }

    public static int minSize(Vector[] vectors) {

        int min = Integer.MAX_VALUE;

        for (int i = 0; i < vectors.length; i++) {
            if (min > vectors[i].getSize()) {
                min = vectors[i].getSize();
            }
        }

        return min;
    }

    public static Vector multiply(Vector x, Vector y) {
        enforceEqualSize(new Vector[]{x, y}, true);

        Vector prod = new Vector(x.getSize());

        for (int i = 0; i < x.getSize(); i++) {
            prod.setElement(i, x.getElement(i) * y.getElement(i));
        }

        return prod;
    }

    public static Vector multiply(Vector x, Vector y, Vector z) {
        enforceEqualSize(new Vector[]{x, y, z}, true);

        Vector prod = new Vector(x.getSize());

        for (int i = 0; i < x.getSize(); i++) {
            prod.setElement(i, x.getElement(i) * y.getElement(i) * z.getElement(i));
        }

        return prod;
    }

    /**
     * Make this Vector immutable.
     *
     * @param immutable
     */
    public void setImmutable() {
        this.fImmuted = true;
    }

    /**
     * @return wther ir not this vector is immutable
     */
    public boolean isImmutable() {
        return fImmuted;
    }

    private void checkImmutable() {
        if (fImmuted) {
            throw new ImmutedException();
        }
    }

    public boolean isNanOrZero() {
        for (int i = 0; i < getSize(); i++) {
            if (getElement(i) != 0 || !Float.isNaN(getElement(i))) {
                return false;
            }
        }

        return true;
    }

    public Vector extract(final int[] indices, final boolean include) {

        if (include) {
            float[] values = new float[indices.length];
            for (int i = 0; i < indices.length; i++) {
                values[i] = getElement(indices[i]);
            }

            return new Vector(values, true);
        } else {

            TIntArrayList dontUseMe_indices = new TIntArrayList(indices);
            TFloatArrayList useMeValues = new TFloatArrayList();
            for (int i = 0; i < getSize(); i++) {
                if (dontUseMe_indices.contains(i) == false) {
                    useMeValues.add(getElement(i));
                }
            }

            return new Vector(useMeValues);
        }
    }

    public Vector extract(float someScoreThatDecidesMode, ScoreMode smode) {
        if (smode.isPostiveAndNegSeperately()) {
            if (XMath.isPositive(someScoreThatDecidesMode)) {
                return extract(ScoreMode.POS_ONLY);
            } else {
                return extract(ScoreMode.NEG_ONLY);
            }

        } else {
            return extract(smode);
        }
    }

    public Vector extract(ScoreMode smode) {
        TFloatArrayList floats;

        if (smode.isPostiveAndNegTogether()) {
            return this; // no change
        } else if (smode.isPostiveOnly()) {
            floats = new TFloatArrayList();
            for (int i = 0; i < getSize(); i++) {
                float elem = getElement(i);
                if (XMath.isPositive(elem)) {
                    floats.add(elem);
                }
            }
        } else if (smode.isNegativeOnly()) {
            floats = new TFloatArrayList();
            for (int i = 0; i < getSize(); i++) {
                float elem = getElement(i);
                if (XMath.isNegative(elem)) {
                    floats.add(elem);
                }
            }
        } else {
            throw new IllegalArgumentException("Unknown smode: " + smode.getName());
        }

        return new Vector(floats.toNativeArray(), true);
    }

    /**
     * These are set each time a mean, media etc call is made.
     * If the vector changes, they do NOT auto change! Only change after the compute call is made again.
     * see metrics and the nnalg for whys this is a useful concept.
     */
    public class ComputeSet {

        public double mean = Float.NaN;
        public final double median = Float.NaN;
        public double stddev = Float.NaN;
        public double max = Float.NaN;
        public double min = Float.NaN;
        public double var = Float.NaN;
        public int maxindex = -1;
        public int minindex = -1;

        public void recompute() {

            mean();
            median();
            stddev(false, false);    // default
            var(false, false);
            max();
            min();
            vard(false, false);      // default
        }
    }    // End ComputeSet
}        // End Vector

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/

/*
private double _var(boolean biased, ScoreMode smode) {

    double oldvar = 0.0;
    double[] mm = _mean(smode);
    double mean = mm[0];
    int len = (int) mm[1];
    if (!biased) {
        len--;
    }

    // Variance of 1 point is 0 (we are returning the biased variance in this case)
    if (len <= 0) {
        return oldvar;
    }

    for (int i = 0; i < elementCount; i++) {

        if (smode.isPostiveOnly() && XMath.isPositive(elementData[i])) {
            double result = elementData[i] - mean;
            oldvar += result * result;
        } else if (smode)

        if (elementData[i])

    }

    // DONT set computeset!
    return oldvar / len;
}

   public double var(boolean biased, boolean fixlow, ScoreMode smode) {
    if (fixlow) {
        stddev(biased, fixlow);
        double var = computeset.stddev * computeset.stddev;
        computeset.var = var;
        return var;
    } else {
        return _var(biased);
    }
}

 public double stddev(boolean biased, boolean fixlow, ScoreMode smode) {

    double stddev = Math.sqrt(_var(biased));    // @note call to _var and not var
    double mean = computeset.mean;                    // avoid recalc

    if (fixlow) {
        double minallowed = (0.20 * Math.abs(mean));

        // In the case of a zero mean, assume the mean is 1
        if (minallowed == 0) {
            minallowed = 0.20;
        }

        if (minallowed < stddev) {
            ;    // keep orig
        } else {
            stddev = minallowed;
        }
    }

    computeset.stddev = stddev;

    return computeset.stddev;
}


 public double vard(boolean biased, boolean fixlow, ScoreMode smode) {
    return this.stddev(biased, fixlow, smode) / this.mean(smode);
}
*/