package edu.mit.broad.genome.math;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.utils.ImmutedException;
import org.apache.log4j.Logger;


/**
 * Each element is a Vector object
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class VectorMatrix implements IMatrix {

    /**
     * The data of the VectorMatrix.(1-D array. The (i,j) element is stored in elementData[i*fColCnt + j])
     */
    private Vector elementData[];

    /**
     * The number of rows in this matrix.
     */
    private int fRowCnt;

    /**
     * The number of columns in this matrix.
     */
    private int fColCnt;
    private boolean fImmuted;
    private static final transient Logger klog = XLogger.getLogger(VectorMatrix.class);

    /**
     * Class Constructor.
     * Must call set() after this form of teh constructor.
     * <p/>
     * Constructs an fRowCnt by fColCnt all zero matrix. (as change)
     * Note that even though row and column numbering begins with
     * zero, fRowCnt and fColCnt will be one larger than the maximum
     * possible matrix index values.
     *
     * @param nrows number of rows in this matrix.
     * @param ncols number of columns in this matrix.
     *              <p/>
     *              Constructs an fRowCnt by fColCnt all zero matrix. (as change)
     *              Note that even though row and column numbering begins with
     *              zero, fRowCnt and fColCnt will be one larger than the maximum
     *              possible matrix index values.
     * @param nrows number of rows in this matrix.
     * @param ncols number of columns in this matrix.
     *              <p/>
     *              Constructs an fRowCnt by fColCnt all zero matrix. (as change)
     *              Note that even though row and column numbering begins with
     *              zero, fRowCnt and fColCnt will be one larger than the maximum
     *              possible matrix index values.
     * @param nrows number of rows in this matrix.
     * @param ncols number of columns in this matrix.
     *              <p/>
     *              Constructs an fRowCnt by fColCnt all zero matrix. (as change)
     *              Note that even though row and column numbering begins with
     *              zero, fRowCnt and fColCnt will be one larger than the maximum
     *              possible matrix index values.
     * @param nrows number of rows in this matrix.
     * @param ncols number of columns in this matrix.
     */
    // public VectorMatrix() {
    //}

    /**
     * Constructs an fRowCnt by fColCnt all zero matrix. (as change)
     * Note that even though row and column numbering begins with
     * zero, fRowCnt and fColCnt will be one larger than the maximum
     * possible matrix index values.
     *
     * @param nrows number of rows in this matrix.
     * @param ncols number of columns in this matrix.
     */
    public VectorMatrix(int nrows, int ncols) {

        if (nrows < 0) {
            throw new NegativeArraySizeException(nrows + " < 0");
        }

        if (ncols < 0) {
            throw new NegativeArraySizeException(ncols + " < 0");
        }

        this.fRowCnt = nrows;
        this.fColCnt = ncols;
        this.elementData = new Vector[fRowCnt * fColCnt];
    }

    /**
     * Returns the number of rows in this matrix.
     *
     * @return number of rows in this matrix
     */
    public int getNumRow() {
        return fRowCnt;
    }

    public int getDim() {
        return getNumRow() * getNumCol();
    }

    /**
     * Returns the number of colmuns in this matrix.
     *
     * @return number of columns in this matrix
     */
    public int getNumCol() {
        return fColCnt;
    }

    /**
     * Retrieves the value at the specified row and column of this matrix.
     *
     * @param row    the row number to be retrieved (zero indexed)
     * @param column the column number to be retrieved (zero indexed)
     * @return the value at the indexed element
     */
    public Vector getElement(int row, int column) {

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        if (fColCnt <= column) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " > matrix's fColCnt:"
                    + fColCnt);
        }

        if (column < 0) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " < 0");
        }

        return elementData[row * fColCnt + column];
    }

    /**
     * Modifies the value at the specified row and column of this matrix.
     *
     * @param row    the row number to be modified (zero indexed)
     * @param column the column number to be modified (zero indexed)
     * @param value  the new matrix element value
     */
    public void setElement(final int row, final int column, final Vector value) {

        checkImmutable();

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        if (fColCnt <= column) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " <= matrix's fColCnt:"
                    + fColCnt);
        }

        if (column < 0) {
            throw new ArrayIndexOutOfBoundsException("column:" + column + " < 0");
        }

        elementData[row * fColCnt + column] = value;
    }

    /**
     * Copy the values from the array into the specified row of this
     * matrix.
     *
     * @param row   the row of this matrix into which the array values
     *              will be copied.
     * @param array the source array
     */
    public void setRow(final int row, final Vector array[]) {

        checkImmutable();

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row: " + row + " > matrix's fRowCnt: "
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row: " + row + " < 0");
        }

        if (array.length < fColCnt) {
            throw new ArrayIndexOutOfBoundsException("specified array length: " + array.length
                    + " < matrix's fColCnt: " + fColCnt);
        }

        System.arraycopy(array, 0, elementData, row * fColCnt, fColCnt);
    }

    /**
     * Copy the values from the array into the specified column of this
     * matrix.
     *
     * @param col   the column of this matrix into which the array values
     *              will be copied.
     * @param array the source array
     */
    public void setColumn(final int col, final Vector[] objs) {

        checkImmutable();

        if (fColCnt <= col) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " > matrix's fColCnt="
                    + fColCnt);
        }

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        if (objs.length < fRowCnt) {
            throw new ArrayIndexOutOfBoundsException("array length:" + objs.length
                    + " < matrix's fRowCnt:" + fRowCnt);
        }

        for (int i = 0; i < fRowCnt; i++) {
            elementData[i * fColCnt + col] = objs[i];
        }
    }

    /**
     * REAL objects returned a safe copy is NOT returned.
     */
    public Vector[] getColumn(final int col) {

        if (col < 0) {
            throw new ArrayIndexOutOfBoundsException("col:" + col + " < 0");
        }

        Vector[] ret = new Vector[fRowCnt];

        for (int i = 0; i < fRowCnt; i++) {
            ret[i] = elementData[i * fColCnt + col];
        }

        return ret;
    }

    /**
     * IMP REAL OBJECTS - NOT A SAFE COPY returned
     * NOT TESTED
     */
    public Vector[] getRow(final int row) {

        if (fRowCnt <= row) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " > matrix's fRowCnt:"
                    + fRowCnt);
        }

        if (row < 0) {
            throw new ArrayIndexOutOfBoundsException("row:" + row + " < 0");
        }

        Vector[] ret = new Vector[fColCnt];

        System.arraycopy(elementData, row * fColCnt, ret, 0, fColCnt);

        return ret;
    }


    public int hashCode() {

        int hash = 0;

        for (int i = 0; i < fRowCnt * fColCnt; i++) {
            long bits = (elementData[i].hashCode());

            hash ^= (int) (bits ^ (bits >> 32));
        }

        return hash;
    }

    public void setImmutable() {
        this.fImmuted = true;
    }

    public boolean isImmutable() {
        return fImmuted;
    }

    private void checkImmutable() {
        if (fImmuted) {
            throw new ImmutedException();
        }
    }
}    // End VectorMatrix
