/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.models;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.GeneSetMatrix;
import edu.mit.broad.genome.parsers.AuxUtils;
import org.apache.log4j.Logger;

import javax.swing.table.AbstractTableModel;


/**
 * GeneSetMatrix viewer thats like a dataset viewer - just dumps all sets column wise
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class GeneSetMatrixModel2 extends AbstractTableModel {

    private final Logger log = XLogger.getLogger(GeneSetMatrixModel2.class);
    private GeneSetMatrix fGmx;

    private int fMaxSetSize;

    /**
     * Class Constructor.
     * Cretaes a default row name based annotation for use.
     */
    public GeneSetMatrixModel2(final GeneSetMatrix gmx) {

        if (gmx == null) {
            throw new IllegalArgumentException("Param ds cannot be null");
        }

        this.fGmx = gmx;
        this.fMaxSetSize = fGmx.getMaxGeneSetSize();
    }

    public int getColumnCount() {
        return fGmx.getNumGeneSets();
    }

    public int getRowCount() {
        return fMaxSetSize;
    }

    public String getColumnName(int col) {
        return AuxUtils.getAuxNameOnlyNoHash(fGmx.getGeneSet(col).getName());
    }

    public Object getValueAt(int row, int col) {
        GeneSet gset = fGmx.getGeneSet(col);
        Object val = null;
        if (gset.getNumMembers() > row) {
            val = gset.getMember(row);
        }

        return val;
    }

    public Class getColumnClass(int col) {
        return String.class;
    }

    public boolean isEditable() {
        return false;
    }

    public boolean isCellEditable(int row, int col) {
        return false;
    }
}    // End GeneSetMatrixModel2

/*--- Formatted in Sun Java Convention Style on Thu, Oct 17, '02 ---*/
