/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.models;


import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.objects.StringDataframe;
import gnu.trove.THashSet;
import org.apache.log4j.Logger;

import javax.swing.table.AbstractTableModel;


/**
 * An implementation of AbstractTableModel for StringDataframes. <br>
 * <p/>
 * Does not replicate any of StringDataframe's data strcutures.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class StringDataframeModel extends AbstractTableModel {

    private final Logger log = XLogger.getLogger(StringDataframeModel.class);
    private StringDataframe fStringDataframe;

    private String fFeatureName = "Feature"; // default

    private THashSet floatColNames;

    /**
     * Class Constructor.
     * Cretaes a default row name based annotation for use.
     */
    public StringDataframeModel(final StringDataframe sdf, final THashSet floatColNames) {

        if (sdf == null) {
            throw new IllegalArgumentException("Param sdf cannot be null");
        }

        this.fStringDataframe = sdf;
        this.floatColNames = floatColNames;
    }

    public StringDataframeModel(final StringDataframe sdf) {
        this(sdf, null);
    }

    public StringDataframe getStringDataframe() {
        return fStringDataframe;
    }

    public int getColumnCount() {
        return fStringDataframe.getNumCol() + 1;
    }

    public int getRowCount() {
        return fStringDataframe.getNumRow();
    }

    public void setFeatureName(final String newName) {
        this.fFeatureName = newName;
    }

    public String getColumnName(int col) {
        if (col == 0) {
            return fFeatureName;
        } else {
            return fStringDataframe.getColumnName(col - 1);
        }
    }


    public int getColumnIndex(String colName) {
        if (colName.equals(fFeatureName)) {
            return 0;
        } else {
            return fStringDataframe.getColumnIndex(colName) - 1;
        }
    }

    public Object getValueAt(int row, int col) {

        if (col == 0) {
            return fStringDataframe.getRowName(row);
        }

        String cn = getColumnName(col);

        if (floatColNames != null && floatColNames.contains(cn)) {
            return new Float(fStringDataframe.getElement(row, col - 1));
        } else {
            return fStringDataframe.getElement(row, col - 1);
        }

    }

    public Class getColumnClass(int col) {
        return getValueAt(0, col).getClass();
    }

    public boolean isEditable() {
        return false;
    }

    public boolean isCellEditable(int row, int col) {
        return false;
    }

}    // End StringDataframeModel

/*--- Formatted in Sun Java Convention Style on Thu, Oct 17, '02 ---*/
