/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.models;

import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.Dataset;
import org.jfree.data.statistics.HistogramDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/**
 * @author Aravind Subramanian
 *         Date: Aug 10, 2004
 *         Time: 12:23:39 PM
 */
public class XChartModelFactory {

    public static class XYSeriesCollImpl extends XYSeriesCollection {

        public XYSeriesCollImpl(String[] labels, Vector x, Vector y) {
            super();
            for (int i = 0; i < x.getSize(); i++) {
                XYSeries xys = new XYSeries(labels[i]);
                xys.add(x.getElement(i), 5);
                xys.add(y.getElement(i), 10);
                super.addSeries(xys);
            }
        }
    }

    public static HistogramDataset createHistogramDataset(Dataset ds, int nbins) {
        return createHistogramDataset(ds.getMatrix(), nbins);
    }

    public static HistogramDataset createHistogramDataset(Matrix m, int nbins) {
        final HistogramDataset hds = new HistogramDataset();
        // waste to duplicate the array
        double[] ds = m.toArrayD();
        hds.addSeries("H1", ds, nbins);
        return hds;
    }

    public static HistogramDataset createHistogramDataset(final Vector v, final int nbins) {
        final HistogramDataset hds = new HistogramDataset();
        // waste to duplicate the array
        double[] ds = v.toArrayDouble();
        hds.addSeries("H1", ds, nbins);
        return hds;
    }

    public static HistogramDataset createHistogramDataset(Dataset ds, int nbins, double min, double max) {
        return createHistogramDataset(ds.getMatrix(), nbins, min, max);
    }

    public static HistogramDataset createHistogramDataset(Matrix m, int nbins, double min, double max) {
        final HistogramDataset hds = new HistogramDataset();
        // waste to duplicate the array
        double[] ds = m.toArrayD();
        hds.addSeries("H1", ds, nbins, min, max);
        return hds;
    }

    public static HistogramDataset createHistogramDataset(Vector v, int nbins, double min, double max) {
        final HistogramDataset hds = new HistogramDataset();
        // waste to duplicate the array
        double[] ds = v.toArrayDouble();
        hds.addSeries("H1", ds, nbins, min, max);
        return hds;
    }

} // End class XChartModelFactory
