/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.models;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.math.XYVector;
import org.jfree.data.DomainOrder;
import org.jfree.data.general.DatasetChangeListener;
import org.jfree.data.general.DatasetGroup;
import org.jfree.data.xy.XYDataset;

public class XYDatasetProxy implements XYDataset {

    private DatasetGroup fGroup;

    private final String[] fSeriesNames;

    private final XYVector[] fXYVectors;

    /**
     * Class constructor
     *
     * @param seriesname
     * @param x
     * @param y
     */
    public XYDatasetProxy(final String seriesname, final Vector x, final Vector y) {
        fGroup = new DatasetGroup();
        fSeriesNames = new String[]{seriesname};

        XYVector xy = new XYVector(x, y);
        fXYVectors = new XYVector[]{xy};
    }

    /**
     * Class constructor
     *
     * @param seriesname
     * @param y
     */
    public XYDatasetProxy(final String seriesname, final Vector y) {
        fGroup = new DatasetGroup();
        fSeriesNames = new String[]{seriesname};
        XYVector xy = new XYVector(new Vector(0, y.getSize() - 1), y);
        fXYVectors = new XYVector[]{xy};
    }

    // added dummy for jfreechart RC1 Nov 2005
    public int indexOf(java.lang.Comparable comparable) {
        return 0;
    }

    ASComparable foo;

    public java.lang.Comparable getSeriesKey(int series) {
        if (foo == null) {
            foo = new ASComparable(getSeriesName(0));
        }

        return foo;
    }

    public DatasetGroup getGroup() {
        return fGroup;
    }

    public void setGroup(DatasetGroup g) {
        this.fGroup = g;
    }

    public double getYValue(int series, int item) {
        XYVector xy = fXYVectors[series];
        return new Float(xy.y.getElement(item)).doubleValue();
    }

    public double getXValue(int series, int item) {
        XYVector xy = fXYVectors[series];
        return new Float(xy.x.getElement(item)).doubleValue();
    }


    public int getSeriesCount() {
        return fSeriesNames.length;
    }

    public int getItemCount(int series) {
        XYVector xy = fXYVectors[series];
        return xy.x.getSize();
    }

    public String getSeriesName(int series) {
        return fSeriesNames[series];
    }

    public Number getX(int i, int i1) {
        throw new NotImplementedException();
    }

    public Number getY(int i, int i1) {
        throw new NotImplementedException();
    }

    public void addChangeListener(DatasetChangeListener listener) {
    }

    public void removeChangeListener(DatasetChangeListener listener) {
    }

    public DomainOrder getDomainOrder() {
        throw new NotImplementedException();
    }

} // End class XYDatasetProxy

