package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.vdb.chip.Chip;

import java.util.*;

/**
 * @author Aravind Subramanian
 *         Date: Jan 16, 2006
 *         Time: 1:55:30 PM
 */
public abstract class AbstractRankedListDb_base extends AbstractObject implements RankedListDb {

    private List fRowNames;

    private List fColNames; // rl names (NOT sample names)

    private int fNumRows;

    private GeneSet fColNamesGeneSet; // lazilly filled

    private GeneSet fRowNamesGeneSet; // lazilly filled

    private Chip fChip;

    private Annot fAnnot_for_rl;

    /**
     * Class constructor
     * <p/>
     * Things should already be duplicated as needed
     *
     * @param rowNames
     * @param colNames
     */

    protected AbstractRankedListDb_base() {
    }

    // subclasses must override
    protected void init(final String name,
                        final List rowNames,
                        final List colNames,
                        final Chip chip,
                        final Annot annot_for_rl_opt) {
        super.initialize(name);

        /*
        if (chip == null) {
            throw new IllegalArgumentException("Param chip cannot be null");
        }

        if (annot_for_rl == null) {
            throw new IllegalArgumentException("Param annot_for_rl cannot be null");
        }

        if (annot_for_rl.getSampleAnnot_global() == null) {
            throw new IllegalArgumentException("Param annot_for_rl cannot have null sample annot");
        }
        */

        // Ok, alls well
        this.fRowNames = rowNames;
        this.fColNames = colNames;
        this.fNumRows = fRowNames.size();
        this.fChip = chip; // @note dont want to get this from the dataset as the ds might be jit
        this.fAnnot_for_rl = annot_for_rl_opt;
    }

    public Chip getChip() {
        return fChip;
    }

    public Annot getAnnot() {
        return fAnnot_for_rl;
    }

    public RankedListDb cloneShallow(final String[] forTheseNames, final boolean include) {
        if (include) {
            RankedList[] rls = new RankedList[forTheseNames.length];
            for (int i = 0; i < forTheseNames.length; i++) {
                rls[i] = getRankedList(forTheseNames[i]);
            }
            return new RankedListDbImpl(getName() + "_" + forTheseNames.length, rls, getChip(), getAnnot(), getDataset(), getTemplateCohort());
        } else {
            List rlsList = new ArrayList();
            List dontUse = Arrays.asList(forTheseNames);
            for (int i = 0; i < getNumRankedLists(); i++) {
                RankedList rl = getRankedList(i);
                if (dontUse.contains(rl.getName()) == false) {
                    rlsList.add(rl);
                }
            }
            return new RankedListDbImpl(getName() + "_" + forTheseNames.length,
                    (RankedList[]) rlsList.toArray(new RankedList[rlsList.size()])
                    , getChip(), getAnnot(), getDataset(), getTemplateCohort());
        }

    }

    public RankedList[] getRankedLists(final String[] rlNames) {
        final RankedList[] rls = new RankedList[rlNames.length];
        for (int r = 0; r < rlNames.length; r++) {
            rls[r] = getRankedList(rlNames[r]);
        }

        return rls;
    }

    public RankedList[] getRankedLists(final int[] colIndices) {
        final RankedList[] rls = new RankedList[colIndices.length];
        for (int r = 0; r < colIndices.length; r++) {
            rls[r] = getRankedList(colIndices[r]);
        }

        return rls;
    }


    public GeneSetMatrixPair createGeneSetMatrix(final int numMarkers) {
        final GeneSet[] gsets_up = new GeneSet[getNumRankedLists()];
        final GeneSet[] gsets_dn = new GeneSet[getNumRankedLists()];

        for (int r = 0; r < getNumRankedLists(); r++) {
            final GeneSetPair gsp = getRankedList(r).getNamesOfUpAndDnXRanks(numMarkers);
            gsets_up[r] = gsp.getUp();
            gsets_dn[r] = gsp.getDown();
        }

        return new GeneSetMatrixPairImpl(getName() + "_" + numMarkers, gsets_up, gsets_dn);
    }

    // -------------------------------------------------------------------------------------------- //

    public Dataset createDataset(final String name, final int[] rlColIndices) {
        final RankedList[] rls = new RankedList[rlColIndices.length];
        for (int r = 0; r < rlColIndices.length; r++) {
            rls[r] = getRankedList(rlColIndices[r]);
        }

        return _createDataset(name, rls);
    }

    public Dataset createDataset(final String name, final String[] rlColNames) {

        final RankedList[] rls = new RankedList[rlColNames.length];
        for (int r = 0; r < rlColNames.length; r++) {
            rls[r] = getRankedList(rlColNames[r]);
        }

        return _createDataset(name, rls);
    }

    private Dataset _createDataset(final String name, final RankedList[] rls) {
        if (rls == null || rls.length == 0) {
            throw new IllegalArgumentException("Param rls cannot be null nor zero size: " + rls);
        }

        final String[] rowNames = getRowNamesArray(); // @note this is the order that the thing was saved in the jit
        final Matrix m = new Matrix(rowNames.length, rls.length);
        final String[] colNames = new String[rls.length];

        for (int r = 0; r < rowNames.length; r++) {
            for (int c = 0; c < rls.length; c++) {
                if (r == 0) {
                    colNames[c] = rls[c].getName();
                }
                m.setElement(r, c, rls[c].getScore(rowNames[r])); // @note this is painfully slow
            }

            if (r % 500 == 0) {
                System.out.println("Done row: " + r + " / " + rowNames.length);
            }

        }

        return new DefaultDataset(name, m, rowNames, colNames, true, getAnnot());
    }

    // -------------------------------------------------------------------------------------------- //

    public List getRowNames() {
        return Collections.unmodifiableList(fRowNames);
    }

    public GeneSet getRowNamesGeneSet() {
        if (fRowNamesGeneSet == null) {
            fRowNamesGeneSet = new FSet(getName(), new HashSet(fRowNames));
        }
        return fRowNamesGeneSet;
    }

    public String[] getRowNamesArray() {
        return (String[]) fRowNames.toArray(new String[fRowNames.size()]);
    }

    public int getNumRow() {
        return fNumRows;
    }

    public int getNumRankedLists() {
        return fColNames.size();
    }

    // -------------------------------------------------------------------------------------------- //

    public List getRankedListNames() {
        return Collections.unmodifiableList(fColNames);
    }

    public GeneSet getRankedListNamesGeneSet() {
        if (fColNamesGeneSet == null) {
            fColNamesGeneSet = new FSet(getName(), new ArrayList(fColNames), true);
        }
        return fColNamesGeneSet;
    }

    public GeneSet getRankedListNamesGeneSet(final int[] indices) {
        List list = getRankedListNames(indices);
        return new FSet(getName(), list, true);
    }

    public String[] getRankedListNamesArray(final int[] indices) {
        List list = getRankedListNames(indices);
        return (String[]) list.toArray(new String[list.size()]);
    }

    public List getRankedListNames(final int[] indices) {
        List list = new ArrayList();
        for (int i = 0; i < indices.length; i++) {
            list.add(getRankedListName(indices[i]));
        }

        return list;
    }

    public String[] getRankedListNamesArray() {
        return (String[]) fColNames.toArray(new String[fColNames.size()]);
    }

    public String getRankedListName(final int coln) {
        return (String) fColNames.get(coln);
    }

    public int getRankedListIndex(final String colName) {
        return fColNames.indexOf(colName);
    }

    // -------------------------------------------------------------------------------------------- //

    public edu.mit.broad.genome.math.Vector getValues(final String featureName,
                                                      final XMath.Const scoreorRank) {

        final edu.mit.broad.genome.math.Vector v = new edu.mit.broad.genome.math.Vector(getNumRankedLists());
        for (int r = 0; r < getNumRankedLists(); r++) {
            //System.out.println(">>> " + getRankedList(r).getClass());
            v.setElement(r, getRankedList(r).getValue(featureName, scoreorRank));
        }

        return v;
    }

} // End class AbstractRankedListDb_base
