package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.Order;
import edu.mit.broad.genome.math.SortMode;
import edu.mit.broad.genome.math.Vector;

import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Dec 16, 2005
 *         Time: 8:27:50 PM
 */
public abstract class AbstractRankedListsDataset extends AbstractObject implements Dataset {


    public AbstractRankedListsDataset() {
    }


    public float getElement(final int row, final String colName) {
        return getElement(row, getColumnIndex(colName));
    }

    public Dataset cloneShallow(final String newName) {
        super.setName(newName);
        return this;
    }

    public String[] getColumnNamesArray() {
        List colNames = getColumnNames();
        return (String[]) colNames.toArray(new String[colNames.size()]);
    }

    public GeneSet getRowNamesGeneSet() {
        return new FSet(getName(), getRowNames(), false);
    }

    public float getElement(final String rowName, final int col) {
        return getElement(getRowIndex(rowName), col);
    }

    public float getElement(final String rowName, final String colName) {
        return getElement(getRowIndex(rowName), getColumnIndex(colName));
    }

    public String getQuickInfo() {
        return getNumRow() + "x" + getNumCol();
    }

    public Vector getRow(int rown) {
        Vector v = new Vector(getNumCol());
        for (int c = 0; c < getNumCol(); c++) {
            v.setElement(c, getElement(rown, c));
        }

        return v;
    }

    public Vector getRow(String rowName) {
        return getRow(getRowIndex(rowName));
    }

    public Vector[] getRows(GeneSet gset) {
        throw new NotImplementedException();// @todo
    }

    public Vector getColumn(int coln) {
        Vector v = new Vector(getNumRow());
        for (int r = 0; r < getNumRow(); r++) {
            v.setElement(r, getElement(r, coln));
        }

        return v;
    }

    public int[] getColumnIndices(final String[] colNames) {
        int[] indices = new int[colNames.length];
        for (int i = 0; i < colNames.length; i++) {
            indices[i] = getColumnIndex(colNames[i]);
        }

        return indices;
    }

    public Vector getColumn(final int coln, final GeneSet forTheseGenes) {
        throw new NotImplementedException();
    }

    public Vector getColumn(final String colName, final GeneSet forTheseGenes) {
        throw new NotImplementedException();
    }

    public LabelledVector getColumnLV(int coln) {
        throw new NotImplementedException();// @todo
    }

    public LabelledVector getRowLV(final String rowName) {
        return getRowLV(getRowIndex(rowName));
    }

    public LabelledVector getRowLV(final int rown) {
        return new LabelledVector(getRowName(rown), getColumnNamesArray(), getMatrix().getRowV(rown));
    }

    public RankedList getRowRL(final String rowName, final SortMode sort, final Order order) {
        return getRowLV(rowName).sort(sort, order);
    }

    public RankedList getRowRL(final int rown, final SortMode sort, final Order order) {
        return getRowLV(rown).sort(sort, order);
    }

    /**
     * @param colName
     * @return
     */
    public Vector getColumn(String colName) {
        return getColumn(getColumnIndex(colName));
    }

    public Vector[] getColumns(final String[] colNames) {
        Vector[] vss = new Vector[colNames.length];
        for (int i = 0; i < colNames.length; i++) {
            vss[i] = getColumn(colNames[i]);
        }
        return vss;
    }

    public LabelledVector getColumnLV(String colName) {
        throw new NotImplementedException();// @todo
    }

    public RankedList getColumnRL(String colName, SortMode sort, Order order) {
        throw new NotImplementedException();// @todo
    }

    public RankedList getColumnRL(int coln, SortMode sort, Order order) {
        throw new NotImplementedException();// @todo
    }


    public List getColumnNames(Template prefixTemplateClasses) {
        throw new NotImplementedException();// @todo
    }

    public GeneSet getColumnNamesGeneSet() {
        return new FSet(getName(), getColumnNames(), false);
    }


    public int getRowIndex(String rowName) {
        return getRowNames().indexOf(rowName);
    }

    public Matrix getMatrix() {
        throw new NotImplementedException();// @todo
    }

    public int getDim() {
        throw new NotImplementedException();// @todo
    }

    public Annot getAnnot() {
        return null;
    }

}
