package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.vdb.sampledb.SampleAnnot;
import org.apache.log4j.Logger;

/**
 * @author Aravind Subramanian
 *         Date: Nov 21, 2005
 *         Time: 2:42:47 PM
 */
public class AnnotImpl implements Annot {

    private FeatureAnnot fFeatureAnnot;

    private SampleAnnot fSampleAnnot;

    private static final Logger klog = XLogger.getLogger(AnnotImpl.class);

    /**
     * Class constructor
     * <p/>
     * Both parameters are nullable
     *
     * @param featureAnnot
     * @param sampleAnnot
     */
    public AnnotImpl(final FeatureAnnot featureAnnot, final SampleAnnot sampleAnnot) {

        if (featureAnnot == null) {
            throw new IllegalArgumentException("Param featureAnnot cannot be null");
        }

        if (sampleAnnot == null) {
            throw new IllegalArgumentException("Param sampleAnnot cannot be null");
            //klog.warn("Param sampleAnnot_optAnnot was null for fa: " + featureAnnot.getName());
            //TraceUtils.showTrace();
        }

        this.fFeatureAnnot = featureAnnot;
        this.fSampleAnnot = sampleAnnot;
    }

    public Annot cloneDeeep(final String[] sampleNames) {
        return new AnnotImpl(fFeatureAnnot, fSampleAnnot.cloneDeep(sampleNames));
    }

    public FeatureAnnot getFeatureAnnot() {
        return fFeatureAnnot;
    }

    // @note heres some magix
    // if sample annot is null
    public SampleAnnot getSampleAnnot_global() {
        return fSampleAnnot;
    }

    public SampleAnnot getSampleAnnot_synched(final String[] colNames) {
        if (colNames == null) {
            throw new IllegalArgumentException("Param colNames cannot be null");
        }

        if (fSampleAnnot == null) {
            throw new IllegalStateException("No available sample annot: " + fSampleAnnot);
        }

        return fSampleAnnot.cloneDeep(colNames);
    }

    public Chip getChip() {
        return fFeatureAnnot.getChip();
    }

    public void setChip(final Chip chip, final ColorMap.Rows cmr) {
        fFeatureAnnot.setChip(chip, cmr);
    }

    public void setSampleAnnot(final SampleAnnot sampleAnnot) {
        if (sampleAnnot != null) {
            if (this.fSampleAnnot != null) {
                klog.debug("replacing curr sample annot: " + fSampleAnnot.getQuickInfo() + " with: " + sampleAnnot.getQuickInfo());
            }
            this.fSampleAnnot = sampleAnnot;
        } else {
            klog.debug("Ignoring setAnnot as ann is: " + sampleAnnot);
        }
    }

} // End class AnnotImpl
