/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.objects;


import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.Order;
import edu.mit.broad.genome.math.SortMode;
import edu.mit.broad.genome.math.Vector;

import java.util.List;


/**
 * A Math object conceptually similar to R's Dataframe but with annotations
 * Basically a labelled Matrix.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public interface Dataset extends PersistentObject {

    public Dataset cloneShallow(final String newName);

    /**
     * @param rown
     * @return Vector of values of rown.
     */
    public Vector getRow(final int rown);

    public RankedList getRowRL(final String rowName, final SortMode sort, final Order order);

    public RankedList getRowRL(final int rown, final SortMode sort, final Order order);

    public LabelledVector getRowLV(final int rown);

    public LabelledVector getRowLV(final String roweName);

    /**
     * @param rowName
     * @return
     */
    public Vector getRow(final String rowName);

    public Vector[] getRows(final GeneSet gset);

    /**
     * @param coln
     * @return Vector of values of coln.
     */
    public Vector getColumn(final int coln);

    public Vector getColumn(final int coln, final GeneSet forTheseGenes);

    public Vector getColumn(final String colName, final GeneSet forTheseGenes);

    public LabelledVector getColumnLV(final int coln);

    public LabelledVector getColumnLV(final String colName);

    /**
     * @param colName
     * @return
     */
    public Vector getColumn(final String colName);

    public Vector[] getColumns(final String[] colNames);


    public RankedList getColumnRL(final String colName, final SortMode sort, final Order order);

    public RankedList getColumnRL(final int coln, final SortMode sort, final Order order);

    /**
     * @param row
     * @param column
     * @return Value of element at specified cell.
     */
    public float getElement(final int row, final int column);

    public float getElement(final String rowName, final int column);

    public float getElement(final String rowName, final String colName);

    public float getElement(final int row, final String colName);

    /**
     * @param rown
     * @return Name of row at rown
     */
    public String getRowName(final int rown);

    /**
     * generally should return unmodifiable list
     */
    public List getRowNames();

    /**
     * @return
     */
    public GeneSet getRowNamesGeneSet();

    /**
     * Convenience method for accessing row names
     *
     * @return String array of row names
     */
    public String[] getRowNamesArray();

    /**
     * generally should return unmodifiable list
     */
    public List getColumnNames();

    public List getColumnNames(final Template prefixTemplateClasses);

    /**
     * @return
     */
    public GeneSet getColumnNamesGeneSet();

    /**
     * Convenience method for accessing column names
     *
     * @return String array of column names
     */
    public String[] getColumnNamesArray();

    /**
     * @param coln
     * @return Name of column at position coln
     */
    public String getColumnName(final int coln);

    /**
     * @param rowname Name of row
     * @return Position of row in the Dataset. -1 if not found.
     */
    public int getRowIndex(final String rowName);

    /**
     * @param colname Name of column
     * @return Position of specified column in the Dataset. -1 if not found.
     */
    public int getColumnIndex(final String colName);

    public int[] getColumnIndices(final String[] colNames);

    /**
     * @return Number of rows in Dataset
     */
    public int getNumRow();

    /**
     * @return Number of columns in Dataset
     */
    public int getNumCol();

    /**
     * Should return an safe i.e unmodifiable Matrix.
     * Generally implementors might prefer to return an immutable view rather
     * than a copy.
     * Similar to returning an unmodifiable view of a Collection.
     */
    public Matrix getMatrix();

    public int getDim();

    public Annot getAnnot();


}    // End Dataset

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
