/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.Order;
import edu.mit.broad.genome.math.SortMode;
import edu.mit.broad.genome.math.Vector;

import java.util.List;


/**
 * Wrapper to cover a Dataset when it has to be returned by a method.
 * Though a Dataset is immutabke, objects with methods that return a Dataset can run into
 * trouble if they return a ref to their real Dataset
 * So, instead return a wrapper.
 *
 * @see HClustDatasetStanford for a usage example
 */
public class DatasetProxy extends AbstractObject implements Dataset {

    private Dataset fRealDataset;

    /**
     * @param ds
     */
    public DatasetProxy(final Dataset ds) {

        if (ds == null) {
            throw new IllegalArgumentException("Parameter ds cannot be null");
        }

        super.initialize(ds.getName());
        this.fRealDataset = ds;
    }

    public Dataset cloneShallow(final String newName) {
        throw new NotImplementedException();
    }

    /**
     * Class constructor
     *
     * @param newName
     * @param ds
     */
    public DatasetProxy(final String newName, final Dataset ds) {
        super(newName);

        if (ds == null) {
            throw new IllegalArgumentException("Parameter ds cannot be null");
        }

        this.fRealDataset = ds;
    }

    public Annot getAnnot() {
        return fRealDataset.getAnnot();
    }

    public Vector getRow(final int rown) {
        return fRealDataset.getRow(rown);
    }

    public Vector getColumn(final int coln) {
        return fRealDataset.getColumn(coln);
    }

    public Vector getColumn(final int coln, final GeneSet forTheseGenes) {
        return fRealDataset.getColumn(coln, forTheseGenes);
    }

    public Vector getColumn(final String colName, final GeneSet forTheseGenes) {
        return fRealDataset.getColumn(colName, forTheseGenes);
    }

    public Vector getColumn(final String colName) {
        return fRealDataset.getColumn(colName);
    }

    public Vector[] getColumns(final String[] colNames) {
        return fRealDataset.getColumns(colNames);
    }

    public LabelledVector getColumnLV(final int coln) {
        return fRealDataset.getColumnLV(coln);
    }

    public LabelledVector getColumnLV(final String colName) {
        return fRealDataset.getColumnLV(colName);
    }

    public RankedList getColumnRL(final int coln, SortMode sort, Order order) {
        return fRealDataset.getColumnRL(coln, sort, order);
    }

    public RankedList getColumnRL(final String colName, SortMode sort, Order order) {
        return fRealDataset.getColumnRL(colName, sort, order);
    }

    public float getElement(final int rown, final int coln) {
        return fRealDataset.getElement(rown, coln);
    }

    public float getElement(final int row, final String colName) {
        return fRealDataset.getElement(row, colName);
    }

    public float getElement(final String rowName, final int col) {
        return fRealDataset.getElement(rowName, col);
    }

    public float getElement(final String rowName, final String colName) {
        return fRealDataset.getElement(rowName, colName);
    }

    public String getRowName(final int rown) {
        return fRealDataset.getRowName(rown);
    }

    public Vector getRow(final String rowName) {
        return fRealDataset.getRow(rowName);
    }

    public Vector[] getRows(final GeneSet gset) {
        return fRealDataset.getRows(gset);
    }

    public List getRowNames() {
        return fRealDataset.getRowNames();
    }

    public int getRowIndex(final String rowname) {
        return fRealDataset.getRowIndex(rowname);
    }

    public String[] getRowNamesArray() {
        return fRealDataset.getRowNamesArray();
    }

    public List getColumnNames() {
        return fRealDataset.getColumnNames();
    }

    public List getColumnNames(final Template prefixTemplateClasses) {
        return fRealDataset.getColumnNames(prefixTemplateClasses);
    }

    public int getColumnIndex(final String colname) {
        return fRealDataset.getColumnIndex(colname);
    }

    public int[] getColumnIndices(final String[] colNames) {
        return fRealDataset.getColumnIndices(colNames);
    }

    public String[] getColumnNamesArray() {
        return fRealDataset.getColumnNamesArray();
    }

    public String getColumnName(final int coln) {
        return fRealDataset.getColumnName(coln);
    }

    public int getNumRow() {
        return fRealDataset.getNumRow();
    }

    public int getNumCol() {
        return fRealDataset.getNumCol();
    }

    public int getDim() {
        return fRealDataset.getDim();
    }

    public Matrix getMatrix() {
        return fRealDataset.getMatrix();
    }

    public String getQuickInfo() {
        return fRealDataset.getQuickInfo();
    }

    public GeneSet getColumnNamesGeneSet() {
        return fRealDataset.getColumnNamesGeneSet();
    }

    public GeneSet getRowNamesGeneSet() {
        return fRealDataset.getRowNamesGeneSet();
    }

    public LabelledVector getRowLV(final String rowName) {
        return fRealDataset.getRowLV(rowName);
    }

    public LabelledVector getRowLV(final int rown) {
        return fRealDataset.getRowLV(rown);

    }

    public RankedList getRowRL(final String rowName, final SortMode sort, final Order order) {
        return fRealDataset.getRowRL(rowName, sort, order);

    }

    public RankedList getRowRL(final int rown, final SortMode sort, final Order order) {
        return fRealDataset.getRowRL(rown, sort, order);

    }

} // End class DatasetWrapper
