/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.parsers.AuxUtils;
import edu.mit.broad.genome.parsers.ParserFactory;

import java.io.File;
import java.util.HashSet;
import java.util.Set;

/**
 * @author Aravind Subramanian
 *         Date: Jun 15, 2004
 *         Time: 9:21:15 AM
 */
public class DefaultGeneSetOfGeneSetNames implements GeneSetOfGeneSetNames {

    private String[] fGeneSetNames;
    private GeneSetMatrix fContentGeneSetMatrix;

    /**
     * Class constructor
     *
     * @param geneSetNames
     * @param contentGm
     */
    public DefaultGeneSetOfGeneSetNames(String[] geneSetNames, GeneSetMatrix contentGm) {
        this.init(geneSetNames, contentGm);
    }

    public DefaultGeneSetOfGeneSetNames(File gsetNamesFile, GeneSetMatrix contentGm) throws Exception {
        this(ParserFactory.readGeneSet(gsetNamesFile, false), contentGm); // dont load from cache as a favor, quick anyway
    }

    /**
     * Class constructor
     *
     * @param geneSetofGeneSetNames
     * @param contentGm
     */
    public DefaultGeneSetOfGeneSetNames(GeneSet geneSetofGeneSetNames, GeneSetMatrix contentGm) {

        if (geneSetofGeneSetNames == null) {
            throw new IllegalArgumentException("Parameter geneSetofGeneSetNames cannot be null");
        }

        if (contentGm == null) {
            throw new IllegalArgumentException("Parameter contentGm cannot be null");
        }

        this.init(geneSetofGeneSetNames.getMembersArray(), contentGm);

    }

    private void init(String[] gsetNames, GeneSetMatrix contentGm) {

        // upfront check
        // also check for duplicates
        Set names = new HashSet();
        for (int i = 0; i < gsetNames.length; i++) {
            String lu = AuxUtils.getAuxNameOnlyNoHash(gsetNames[i]); // impt to do aux deconv as the gsets used for enrichment may not e the same file as was used for metasetification
            contentGm.getGeneSet(lu);
            if (names.contains(lu)) {
                throw new IllegalArgumentException("Duplicate gene set name: " + gsetNames[i]);
            }

            names.add(lu);
        }

        this.fGeneSetNames = gsetNames;
        this.fContentGeneSetMatrix = contentGm;
    }

    public String[] getGeneSetNames() {
        return fGeneSetNames; // @todo consider cloning
    }

    public GeneSet[] getContentAsGeneSets() {
        return fContentGeneSetMatrix.getGeneSets(fGeneSetNames);
    }

    public GeneSetMatrix getContentAsGeneSetMatrix() {
        return new DefaultGeneSetMatrix(fContentGeneSetMatrix.getName() + "_extr_" + fGeneSetNames.length,
                fContentGeneSetMatrix.getGeneSets(fGeneSetNames));
    }

    public int getNumGeneSets() {
        return fGeneSetNames.length;
    }

} // End class DefaultGogn
