/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import java.util.Collection;
import java.util.Comparator;

/**
 * @author Aravind Subramanian
 *         Date: Jun 24, 2003
 *         Time: 9:01:48 PM
 *         <p/>
 *         Simply defines a set of things(by name) that belong to the same FSet
 *         A list of names - genes/probes etc. Only 1 class in a given FSet (unlike Templates)
 *         A vertical template in some ways
 *         <p/>
 *         (synonyms: Tags, Tag group, GeneSet, BioSet)
 *         For visualization purposes, a FSet can also be associated with an icon and color.
 *         <p/>
 *         There should be no duplicate members
 */
public interface GeneSet extends ISet {

    public String getName(final boolean stripAux);

    // only name and num of members are checked
    public boolean equalsShallow(final GeneSet gset);

    public GeneSet cloneDeep();

    public GeneSet cloneShallow();

    public GeneSet cloneShallow(final String newName);

    public GeneSet cloneShallow(final String newName, final String newNameEnglish);

    public GeneSet cloneDeep(final String newName, final String newNameEnglish);

    public GeneSet cloneDeep(final String newName, final Comparator comp);

    public GeneSet cloneDeep(final Dataset qualify);

    public GeneSet cloneDeep(final RankedList qualify);

    public GeneSet cloneDeep(final Collection qualify);

    /**
     * @return number of row names  found in the dataset that are members of this gene set
     */
    public int getNumMembers(final Dataset ds);

    public int getNumMembers(final RankedList rl);

    public int getNumMembers(final LabelledList ll);

}    // End Interface GeneSet
  