/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import java.util.BitSet;
import java.util.List;
import java.util.Set;

/**
 * Essentially a collection of (probably related) GeneSets
 * In additin has colors and icons and names
 * <p/>
 * Lightweigth container for a bunch of genesets -- geneset data is not duplicated
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public interface GeneSetMatrix extends PersistentObject {

    public GeneSetMatrix cloneShallow(String newName);

    public boolean containsSet(final String gsetName);

    public int getNumGeneSets();

    /**
     * return a ref to the real GeneSet -> @todo consider cloning?
     *
     * @param i
     * @return
     */
    public GeneSet getGeneSet(final int i);


    public GeneSet getGeneSet(final String gsetName);

    /**
     * All genesets
     * directly -- no cloning
     *
     * @return
     */
    public GeneSet[] getGeneSets();

    public List getGeneSetsL();

    public GeneSet[] getGeneSets(final int minSizeInclusive, final int maxSizeInclusive);

    public List getGeneSetsL(final int minSizeInclusive, final int maxSizeInclusive);

    public GeneSet[] getGeneSets(final Set gsetNames);

    public GeneSet[] getGeneSets(final String[] gsetNames);

    public String[] getGeneSetNames(final boolean stripAux);

    public List getGeneSetNamesL(final boolean stripAux);

    public Set getGeneSetNamesS(final boolean stripAux);

    public Dataset createTagMatrix(final boolean genesAsRows, final boolean bufferzeros);

    public GeneSetMatrix getGeneSetMatrix(final GeneSet gsetNames);

    /**
     * The number of members in the biggest GeneSet.
     *
     * @return
     */
    public int getMaxGeneSetSize();

    public String[] getAllMemberNames();

    public List getAllMemberNamesL();

    /**
     * non-redundant list of names of features across
     * genesets in this GeneSetMatrix
     *
     * @return
     */
    public String[] getAllMemberNamesOnlyOnce();

    public Set getAllMemberNamesOnlyOnceS();

    public GeneSet getAllMemberNames_gset();

    public String[] getGeneSetMembershipNames(final String probeName);

    public GeneSet[] getGeneSetMembershipSets(final String probeName);

    public BitSet getGeneSetMembershipBitSet(final String probeName);

    public BitSetDataset getGeneSetMembershipBitSetDataset(final String name, final String[] memberNames);

    public BitSetDataset getGeneSetMembershipBitSetDataset();

    public String getGeneSetName(final int i);

} // End interface GeneSetMatrix
