package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.MismatchedSizeException;

/**
 * @author Aravind Subramanian
 *         Date: Jul 15, 2005
 *         Time: 9:26:34 PM
 */
public class GeneSetMatrixPairImpl implements GeneSetMatrixPair {

    private GeneSetMatrix gm_up;
    private GeneSetMatrix gm_dn;

    private String fName;

    /**
     * Class constructor
     *
     * @param name
     * @param gps
     */
    public GeneSetMatrixPairImpl(final String name, final GeneSetPair[] gps) {
        this(name, _gm(gps, true), _gm(gps, false));
    }

    public GeneSetMatrixPairImpl(final String name, final GeneSet[] ups, GeneSet[] dns) {
        this(name, new DefaultGeneSetMatrix("up", ups), new DefaultGeneSetMatrix("dn", dns));
    }

    /**
     * Class constructor
     *
     * @param up
     * @param dn
     */
    public GeneSetMatrixPairImpl(final String name, final GeneSetMatrix up, final GeneSetMatrix dn) {

        // sanity checks
        if (up == null) {
            throw new IllegalArgumentException("Param gm_up cannot be null");
        }
        if (dn == null) {
            throw new IllegalArgumentException("Param gm_dn cannot be null");
        }

        if (up.getNumGeneSets() != dn.getNumGeneSets()) {
            throw new MismatchedSizeException("gene matrix up", up.getNumGeneSets(), "gene matrix down", dn.getNumGeneSets());
        }

        this.fName = name;
        this.gm_up = up;
        this.gm_dn = dn;

    }

    /**
     * Class constructor
     *
     * @param up
     * @param dn
     */
    public GeneSetMatrixPairImpl(final GeneSetMatrix up, final GeneSetMatrix dn) {
        this(up.getName() + "_" + dn.getName(), up, dn);
    }

    public String getName() {
        return fName;
    }

    public int getSize() {
        return gm_up.getNumGeneSets();
    }

    public GeneSetPair get(int i) {
        return new GeneSetPairImpl(getUp().getGeneSet(i), getDown().getGeneSet(i));
    }

    public GeneSetMatrix getUp() {
        return gm_up;
    }

    public GeneSetMatrix getDown() {
        return gm_dn;
    }

    public GeneSetMatrix getAll(String name) {
        return new DefaultGeneSetMatrix(name, new GeneSetMatrix[]{gm_up, gm_dn}, true);
    }

    private static GeneSetMatrix _gm(final GeneSetPair[] gps, final boolean up) {
        final GeneSet[] gsets = new GeneSet[gps.length];
        for (int i = 0; i < gps.length; i++) {
            if (up) {
                gsets[i] = gps[i].getUp();
            } else {
                gsets[i] = gps[i].getDown();
            }
        }

        String name;
        if (up) {
            name = "up";
        } else {
            name = "dn";
        }

        return new DefaultGeneSetMatrix(name, gsets);
    }

} // End class GeneSetMatrixPairImpl
