/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import edu.mit.broad.vdb.chip.Chip;

import java.awt.*;

/**
 * Database of genes of interest
 * <p/>
 * Projects can maintain their own little db of why genes are relevant
 */
public interface GenesOfInterest extends PersistentObject {

    public String[] getGeneListNames();

    public GeneList[] getGeneLists();

    public GeneSet[] getGeneSets();

    public GeneSet[] getGeneSets_as_symbols_restricted(final Chip chip) throws Exception;

    public GeneSet[] getGeneSets(final Chip chip) throws Exception;

    public GeneSetMatrix getAsGeneSetMatrix();

    public GeneSet getAsOneGeneSet();

    public GeneSet getAsOneGeneSet(final Chip chip) throws Exception;

    public ColorMap.Rows getAsColorMap();

    public GeneList getGeneList(final String glName);

    public boolean isGeneList(final String glName);

    public GeneList getGeneList(final int g);

    public GeneSet getGeneSet(final int i);

    public GeneSet getGeneSet(final int i, final Chip chip) throws Exception;

    public GeneSet getGeneSet(final String glName);

    public GeneSet getGeneSet(final String glName, final Chip chip) throws Exception;

    public int getGeneListIndex(final String glName);

    public int getNumGeneLists();

    public int getMaxNumOfGenes();

    public String[] getMembershipNames(final String symbol);

    /**
     * Interface for one GeneList
     */
    public interface GeneList {

        public String getName();

        public Color getColor();

        public int getNumGenes();

        public Gene getGene(final int i);

        public Gene getGene(final String geneName);

        public GeneSet toGeneSet();

        public GeneSet toGeneSet(final Chip chip) throws Exception;

        public boolean isMember(final String geneName);

    }

    public static interface Gene {

        public String getGeneListName();

        public String getName();

        public String getNotes();

    }

} // End interface GenesofInterest
