/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import java.util.Collection;
import java.util.List;
import java.util.Set;

/**
 * @author Aravind Subramanian
 *         Date: Jun 24, 2003
 *         Time: 9:01:48 PM
 *         <p/>
 *         Simply defines a set of things(by name) that belong to the same Set
 *         A list of names - genes/probes etc. Only 1 class in a given ISet (unlike Templates)
 *         A vertical template in some ways
 *         <p/>
 *         (synonyms: Tags, Tag group, GeneSet, BioSet)
 *         For visualization purposes, a ISet can also be associated with an icon and color.
 *         <p/>
 *         There should be no duplicate members
 */
public interface ISet extends PersistentObject {

    /**
     * @param pos
     * @return Name of member at position pos in the group
     */
    public String getMember(int pos);

    public int getPosition(String member);

    /**
     * Checks if specified name belongs to this ISet.
     *
     * @param name
     * @return
     */
    public boolean isMember(String name);

    /**
     * @return Number of members of this ISet
     */
    public int getNumMembers();

    /**
     * @return Unmodifiable list of members of this ISet
     */
    public List getMembers();

    public Set getMembersS();

    /**
     * @return
     */
    public String[] getMembersArray();

    public int getNumMembers(final Collection coll);

    public int getNumMembers(final GeneSet b);

    public String toString(final char delim);

}    // End Interface ISet
