package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;

import java.util.*;

/**
 * @author Aravind Subramanian
 *         Date: Oct 27, 2005
 *         Time: 11:24:22 AM
 */
public class Medline extends AbstractObject {

    private static final Logger klog = XLogger.getLogger(Medline.class);

    private Record[] fRecords;

    /**
     * Class constructor
     *
     * @param file
     */
    public Medline(final String name, final List records) {
        this(name, (Record[]) records.toArray(new Record[records.size()]));
    }

    /**
     * Class constructor
     *
     * @param name
     * @param records
     */
    public Medline(final String name, final Record[] records) {
        super(name);
        this.fRecords = records;
    }

    public String getQuickInfo() {
        return null;
    }

    public Record getRecord(final String pmid) {
        _initMap();

        final Object obj = fPMIDRecordMap.get(pmid);
        if (obj != null) {
            return (Record) obj;
        } else {
            return null;
        }

    }

    // lazilly inited
    private Map fPMIDRecordMap;

    public void _initMap() {
        if (this.fPMIDRecordMap == null) {
            this.fPMIDRecordMap = new HashMap();
            for (int i = 0; i < fRecords.length; i++) {
                String id = fRecords[i].getPMID();
                this.fPMIDRecordMap.put(id, fRecords[i]);
            }
        }
    }

    /**
     * Internal class for a single records
     */
    public static class Record {

        private String fPMID;

        private String fAbstract;

        private String fAuthor;
        private String fYear;
        private String fTitle;
        private String fJournal;

        private Set fKeywords;

        private String fURL;

        public Record(final String pmid,
                      final String author,
                      final String year,
                      final String title,
                      final String journal,
                      final String abs,
                      final String url,
                      final Set keywords) {

            if (pmid == null) {
                throw new IllegalArgumentException("Param pmid cannot be null: " + pmid + " " + author + " " + year + " " + journal);
            }
            /*
            if (author == null) {
                throw new IllegalArgumentException("Param author cannot be null: " + pmid + " " + author + " " + year + " " + journal);
            }
            if (year == null) {
                throw new IllegalArgumentException("Param year cannot be null: " + pmid + " " + author + " " + year + " " + journal);
            }
            if (title == null) {
                throw new IllegalArgumentException("Param title cannot be null: " + pmid + " " + author + " " + year + " " + journal);
            }
            if (journal == null) {
                throw new IllegalArgumentException("Param journal cannot be null: " + pmid + " " + author + " " + year + " " + journal);
            }

            if (url == null) {
                throw new IllegalArgumentException("Param url cannot be null: " + pmid + " " + author + " " + year + " " + journal);
            }

            */

            if (abs == null) {
                //klog.warn("Param abs is null: " + pmid + " " + author + " " + year + " " + journal);
                throw new IllegalArgumentException("Param abs cannot be null: " + pmid + " " + author + " " + year + " " + journal);
            }


            if (keywords == null || keywords.isEmpty()) {
                //klog.warn("Param keywords is null/empty: " + keywords + " " + pmid + " " + author + " " + year + " " + journal);
                //throw new IllegalArgumentException("Param keywords cannot be null or null: " + keywords + " " + pmid + " " + author + " " + year + " " + journal);
            }

            this.fPMID = pmid;
            this.fAuthor = author;
            this.fYear = year;
            this.fTitle = title;
            this.fJournal = journal;
            this.fAbstract = abs;
            this.fURL = url;
            this.fKeywords = keywords;
        }

        public String getPMID() {
            return fPMID;
        }

        public String getAbstract() {
            return fAbstract;
        }

        public String getCitation() {
            return fAuthor + " ," + fTitle + " ," + fJournal + " ," + fYear;
        }

        public String getURL() {
            return fURL;
        }

        public Set getKeywords() {
            if (fKeywords != null) {
                return Collections.unmodifiableSet(fKeywords);
            } else {
                return null;
            }
        }

    } // End class Record

} // End class Medline
