/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.MismatchedSizeException;
import edu.mit.broad.genome.objects.strucs.ProbeNeighborhoodEsStruc;

import java.util.ArrayList;
import java.util.List;

/**
 *
 */
public class Nedb extends AbstractObject {

    private ProbeNeighborhoodEsStruc[] fPnes;

    private GeneSet[] fGeneSets;

    private List fNeighNames;

    private Dataset fDataset;

    /**
     * Class constructor
     *
     * @param name
     * @param gcoh
     * @param pnes
     */
    public Nedb(String name, Dataset ds, GeneSet[] gsets, ProbeNeighborhoodEsStruc[] pnes) {
        super(name);

        if (ds == null) {
            throw new IllegalArgumentException("Parameter ds cannot be null");
        }

        if (gsets == null) {
            throw new IllegalArgumentException("Parameter gcoh cannot be null");
        }


        this.fDataset = ds;
        this.fNeighNames = new ArrayList(pnes.length);

        for (int i = 0; i < pnes.length; i++) {
            if (gsets.length != pnes[i].getSize()) {
                throw new MismatchedSizeException();
            }

            String probeName = pnes[i].getProbeName();
            if (fNeighNames.indexOf(probeName) != -1) {
                throw new IllegalArgumentException("Duplicated probe neighborhood: " + probeName);
            }

            fNeighNames.add(probeName);
        }

        this.fPnes = pnes;
        this.fGeneSets = gsets;
    }

    public Dataset getFullDataset() {
        return fDataset;
    }

    public String[] getGeneSetNames() {
        String[] names = new String[getNumGeneSets()];
        for (int g = 0; g < getNumGeneSets(); g++) {
            names[g] = getGeneSet(g).getName();
        }

        return names;
    }

    public String getNeighName(int n) {
        return fNeighNames.get(n).toString();
    }

    public String getQuickInfo() {
        return null;
    }

    public int getNumNeighs() {
        return fPnes.length;
    }

    public int getNumGeneSets() {
        return fGeneSets.length;
    }

    public String getGeneSetName(int g) {
        return fGeneSets[g].getName();
    }

    public GeneSet getGeneSet(int g) {
        return fGeneSets[g];
    }

    public ProbeNeighborhoodEsStruc getPnes(String probeName) {
        return fPnes[fNeighNames.indexOf(probeName)];
    }

    public ProbeNeighborhoodEsStruc getPnes(int np) {
        return fPnes[np];
    }

    public String[] getNeighNames() {
        return (String[]) fNeighNames.toArray(new String[fNeighNames.size()]);
    }

} // End class Nedb
