/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.math.Vector;

import java.util.ArrayList;
import java.util.List;

/**
 * Class representing 1 neighbor and its corr to the seed
 */

public class Neighbor {

    private String fName;
    private float fCorr;

    /**
     * @param name
     * @param corr
     */
    public Neighbor(final String name, final float corr) {
        this.fName = name;
        this.fCorr = corr;
    }

    public Neighbor(final String name, final String corr) {
        this(name, Float.parseFloat(corr));
    }

    public String getName() {
        return fName;
    }

    public float getCorr() {
        return fCorr;
    }

    public static List toNames(final Neighbor[] neighbors) {
        List names = new ArrayList(neighbors.length);
        for (int i = 0; i < neighbors.length; i++) {
            names.add(neighbors[i].getName());
        }

        return names;
    }

    public static Vector toCorrs(final Neighbor[] neighbors) {
        Vector corrs = new Vector(neighbors.length);
        for (int i = 0; i < neighbors.length; i++) {
            corrs.setElement(i, neighbors[i].getCorr());
        }

        return corrs;
    }

} // End class Neighbor
