/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.math.CachedMetricStats;
import edu.mit.broad.genome.math.MetricStats;
import edu.mit.broad.genome.math.Vector;

/**
 * Quickly calculates & caches Neighborhood stats for specified dataset
 */

public class QuickNeighborhoodDatasetImpls {

    public static class Pearson extends AbstractNeighDataset {

        /**
         * Class constructor
         * This auto inits the calculations for the QuickPearson
         * (might take awhile)
         *
         * @param ds
         * @param hood
         */
        public Pearson(final Dataset ds) {

            if (ds == null) {
                throw new IllegalArgumentException("Parameter ds cannot be null");
            }


            CachedMetricStats.Pearson[] pss = new CachedMetricStats.Pearson[ds.getNumRow()];

            for (int r = 0; r < ds.getNumRow(); r++) {
                pss[r] = new CachedMetricStats.Pearson(ds.getRow(r));
            }

            super.initCache(ds, pss);
        }

        public MetricStats createMetricStats(final Vector v) {
            return new CachedMetricStats.Pearson(v);
        }
    }

    public static class Euclidean extends AbstractNeighDataset {

        /**
         * Class constructor
         * This auto inits the calculations for the QuickPearson
         * (might take awhile)
         *
         * @param ds
         * @param hood
         */
        public Euclidean(final Dataset ds) {

            if (ds == null) {
                throw new IllegalArgumentException("Parameter ds cannot be null");
            }


            CachedMetricStats.Euclidean[] pss = new CachedMetricStats.Euclidean[ds.getNumRow()];

            for (int r = 0; r < ds.getNumRow(); r++) {
                pss[r] = new CachedMetricStats.Euclidean(ds.getRow(r));
            }

            super.initCache(ds, pss);
        }

        public MetricStats createMetricStats(Vector v) {
            return new CachedMetricStats.Euclidean(v);
        }
    }

    /**
     * Class constructor
     */
    public static class Spearman extends AbstractNeighDataset {

        /**
         * Class constructor
         * This auto inits the calculations for the QuickPearson
         * (might take awhile)
         *
         * @param ds
         * @param hood
         */
        private Dataset ds;

        public Spearman(final Dataset ds) {

            if (ds == null) {
                throw new IllegalArgumentException("Parameter ds cannot be null");
            }

            this.ds = ds;

            CachedMetricStats.Spearman[] sps = new CachedMetricStats.Spearman[ds.getNumRow()];

            for (int r = 0; r < ds.getNumRow(); r++) {
                sps[r] = new CachedMetricStats.Spearman(ds.getRow(r), ds);
            }

            super.initCache(ds, sps);
        }

        public MetricStats createMetricStats(Vector v) {
            return new CachedMetricStats.Spearman(v, ds);
        }
    }

} // End class QuickNeighborhoodDataset
