package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.objects.strucs.DatasetTemplate;
import edu.mit.broad.vdb.chip.Chip;

import java.util.List;

/**
 * The ranked lists occur as columns
 * <p/>
 *
 * @author Aravind Subramanian
 *         Date: Jan 9, 2006
 *         Time: 5:20:31 PM
 */
public interface RankedListDb extends PersistentObject {

    // -------------------------------------------------------------------------------------------- //
    // across all ranked lists
    public edu.mit.broad.genome.math.Vector getValues(final String featureName,
                                                      final XMath.Const scoreorRank);

    // -------------------------------------------------------------------------------------------- //

    public Dataset createDataset(final String name, final int[] rlColIndices);

    public Dataset createDataset(final String name, final String[] rlColNames);

    public GeneSetMatrixPair createGeneSetMatrix(final int numMarkers);

    // -------------------------------------------------------------------------------------------- //

    public RankedListDb cloneShallow(final String[] forTheseNames, final boolean include);

    // -------------------------------------------------------------------------------------------- //

    public Extracted extract(final String rlName);

    public Extracted extract(final int coln);

    public Extracted extract(final String rlName, final GeneSet gset);

    public Extracted extract(final int colIndex, final GeneSet gset);

    public Extracted extract(final String rlName, final int upAndDnX);

    public Extracted extract(final int colIndex, final int upAndDnX);

    public TemplateCohort.Comp getComp(final String rlName);

    public TemplateCohort.Comp getComp(final int rlColIndex);

    public Dataset getDataset();

    public Chip getChip();

    public Annot getAnnot();

    public TemplateCohort getTemplateCohort();

    // -------------------------------------------------------------------------------------------- //
    // @note disabled because these can take a long time to execute
    //public Vector getRow(final String rowName);

    // @note disabled because these can take a long time to execute
    //public Vector[] getRows(final GeneSet gset);

    // -------------------------------------------------------------------------------------------- //

    public RankedList getRankedList(final int coln);

    public RankedList getRankedList(final String rlName);

    public RankedList[] getRankedLists(final String[] rlNames);

    public RankedList[] getRankedLists(final int[] colIndices);

    // -------------------------------------------------------------------------------------------- //
    /**
     * Interface
     */
    public static interface Extracted {

        public RankedList getRankedList();

        // The dataset is UNsorted
        public DatasetTemplate getDatasetTemplate();

        public Dataset getDatasetSorted();

        public String getName();

        public TemplateCohort.Comp getComp();

    } // End interface Extracted

    public static class OrderedSampleNames {

        public String[] names;

        public GeneSet[] namesByRl;

    }


    public OrderedSampleNames getSampleNames(final int[] rlIndices, final boolean classA);

    // -------------------------------------------------------------------------------------------- //

    public List getRowNames();

    public GeneSet getRowNamesGeneSet();

    public String[] getRowNamesArray();

    // @note These have no meaning in this context
    // public Vector getRow(final int rown);
    //public int getRowIndex(final String rowName);
    //public String getRowName(final int rown);

    // -------------------------------------------------------------------------------------------- //

    public List getRankedListNames();

    public List getRankedListNames(int[] indices);

    public GeneSet getRankedListNamesGeneSet();

    public GeneSet getRankedListNamesGeneSet(int[] indices);

    public String[] getRankedListNamesArray();

    public String[] getRankedListNamesArray(int[] indices);

    public String getRankedListName(final int coln);

    public int getRankedListIndex(final String colName);

    // -------------------------------------------------------------------------------------------- //

    public int getNumRow();

    public int getNumRankedLists();

} // End class RankedListDatabase
