package edu.mit.broad.genome.objects;

import edu.mit.broad.vdb.chip.Chip;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Jan 9, 2006
 *         Time: 5:25:39 PM
 */
public class RankedListDbImpl extends AbstractRankedListDb {

    private RankedList[] fRankedLists;

    /**
     * Class constructor
     *
     * @param rls
     */
    public RankedListDbImpl(final String name,
                            final RankedList[] rls,
                            final Chip chip,
                            final Annot annot_for_rl,
                            final Dataset fullDs,
                            final TemplateCohort tch) {

        // all rls need to have the same number of elements
        List colNames = new ArrayList();
        int numRows;
        if (rls.length > 0) {
            numRows = rls[0].getSize();
            for (int c = 0; c < rls.length; c++) {
                if (rls[c].getSize() != numRows) {
                    throw new IllegalArgumentException("Unequal length of ranked lists: " + numRows + " and " + rls[c].getSize() + " at index: " + c + " name: " + rls[c].getName());
                }
                colNames.add(rls[c].getName());
            }
        }

        if (rls.length == 0) {
            throw new IllegalArgumentException("Zero length ranled list: " + rls);
        }

        this.fRankedLists = rls; // each list becomes a column

        super.init(name, rls[0].getRankedNames(), colNames, chip, annot_for_rl, fullDs, tch); // arbitrary ranked list @note NOT doing a deep check
    }

    // -------------------------------------------------------------------------------------------- //

    public RankedList getRankedList(final int coln) {
        return fRankedLists[coln];
    }

    public RankedList getRankedList(final String colName) {
        return fRankedLists[getRankedListIndex(colName)];
    }

} // End class RankedListDbImpl
