package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.vdb.chip.Chip;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Jan 16, 2006
 *         Time: 1:55:30 PM
 */
public class RankedListDbSimple extends AbstractRankedListDb_base implements RankedListDb {

    private Dataset fFullDataset;

    private RankedList[] fRankedLists;


    /**
     * Class constructor
     * <p/>
     * Things should already be duplicated as needed
     *
     * @param rowNames
     * @param colNames
     */

    public RankedListDbSimple(final String name,
                              final Annot annot_for_rl_opt,
                              final Dataset fullDs_opt,
                              final RankedList[] rls) {

        if (rls == null) {
            throw new IllegalArgumentException("Param rls cannot be null");
        }

        final List colNames = new ArrayList();
        int numRows;
        if (rls.length > 0) {
            numRows = rls[0].getSize();
            for (int c = 0; c < rls.length; c++) {
                if (rls[c].getSize() != numRows) {
                    throw new IllegalArgumentException("Unequal length of ranked lists: " + numRows + " and " + rls[c].getSize() + " at index: " + c + " name: " + rls[c].getName());
                }
                colNames.add(rls[c].getName());
            }
        }

        this.fRankedLists = rls; // each list becomes a column
        this.fFullDataset = fullDs_opt;

        Chip chip = null;
        if (annot_for_rl_opt != null) {
            chip = annot_for_rl_opt.getChip();
        } else {
            if (fullDs_opt != null && fullDs_opt.getAnnot() != null) {
                chip = fFullDataset.getAnnot().getChip();
            }
        }

        super.init(name, rls[0].getRankedNames(), colNames, chip, annot_for_rl_opt);
    }

    public String getQuickInfo() {
        return "RankedLists " + fRankedLists.length;
    }

    public RankedListDb cloneShallow(final String[] forTheseNames, final boolean include) {
        if (include) {
            RankedList[] rls = new RankedList[forTheseNames.length];
            for (int i = 0; i < forTheseNames.length; i++) {
                rls[i] = getRankedList(forTheseNames[i]);
            }
            return new RankedListDbImpl(getName() + "_" + forTheseNames.length, rls, getChip(), getAnnot(), getDataset(), null);
        } else {
            List rlsList = new ArrayList();
            List rlsList_names = new ArrayList();
            List dontUse = Arrays.asList(forTheseNames);
            log.debug("Dont use: " + dontUse);
            for (int i = 0; i < getNumRankedLists(); i++) {
                RankedList rl = getRankedList(i);
                //System.out.println(">> " + rl.getName());
                String name = ParseUtils.string2strings(rl.getName(), ".", false)[0]; // because rl names are: DENDA2_5.Lineage.HSC
                if (dontUse.contains(name) == false) {
                    rlsList.add(rl);
                    rlsList_names.add(name);
                }
            }

            log.debug("## # excluded: " + forTheseNames.length + " # included: " + rlsList.size());
            Dataset ds = new DatasetGenerators().extractColumns(fFullDataset.getName(), fFullDataset, rlsList_names);
            Annot annot = getAnnot().cloneDeeep((String[]) rlsList_names.toArray(new String[rlsList_names.size()]));
            return new RankedListDbImpl(getName() + "_" + rlsList.size(),
                    (RankedList[]) rlsList.toArray(new RankedList[rlsList.size()])
                    , getChip(), annot, ds, null);
        }

    }

    public RankedList getRankedList(final int coln) {
        return fRankedLists[coln];
    }

    public RankedList getRankedList(final String colName) {
        return fRankedLists[getRankedListIndex(colName)];
    }

    public OrderedSampleNames getSampleNames(final int[] rlIndices, final boolean classA) {
        throw new NotImplementedException();
    }

    // @note optional
    public Dataset getDataset() {
        return fFullDataset;
    }

    // -------------------------------------------------------------------------------------------- //

    public TemplateCohort getTemplateCohort() {
        throw new NotImplementedException();
    }

    public TemplateCohort.Comp getComp(final String rlName) {
        throw new NotImplementedException();
    }

    public TemplateCohort.Comp getComp(final int rlColIndex) {
        throw new NotImplementedException();
    }

    public Extracted extract(final String rlName) {
        throw new NotImplementedException();
    }

    public Extracted extract(final int coln) {
        throw new NotImplementedException();
    }

    public Extracted extract(final String rlName, final GeneSet gset) {
        throw new NotImplementedException();
    }

    public Extracted extract(final int colIndex, final GeneSet gset) {
        throw new NotImplementedException();
    }

    public Extracted extract(final String rlName, final int upAndDnX) {
        throw new NotImplementedException();
    }

    public Extracted extract(final int colIndex, final int upAndDnX) {
        throw new NotImplementedException();
    }

} // End class AbstractRankedListDb
