package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.math.ScoreMode;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.parsers.ParserFactory;

import java.io.File;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Dec 14, 2005
 *         Time: 9:59:06 PM
 */
public class RankedListJITImpl extends AbstractObject implements RankedList {

    private File fRlFile;

    private RankedList fRl;

    private boolean fUseCache;

    /**
     * Class constructor
     *
     * @param rlFile
     * @param useCache
     */
    public RankedListJITImpl(final File rlFile, final boolean useCache) {
        if (rlFile == null || rlFile.exists() == false) {
            throw new IllegalArgumentException("rlFile doesnt not exist: " + rlFile);
        }

        super.initialize(rlFile.getName());
        this.fUseCache = useCache;
        this.fRlFile = rlFile;
    }

    private RankedList _rl() {
        try {
            if (fRl == null) {
                this.fRl = ParserFactory.readRankedList(fRlFile, fUseCache);
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

        return fRl;
    }


    public RankedList cloneShallowRL(String newName) {
        return _rl().cloneShallowRL(newName);
    }

    public String getQuickInfo() {
        return _rl().getQuickInfo();
    }

    public boolean isMember(final String name) {
        return _rl().isMember(name);
    }

    public String getRankName(final int rank) {
        return _rl().getRankName(rank);
    }

    public List getRankedNames() {
        return _rl().getRankedNames();
    }

    public String[] getRankedNamesArray() {
        return _rl().getRankedNamesArray();
    }

    public float getValue(final String featureName,
                          final XMath.Const scoreorRank) {
        if (scoreorRank.isScore()) {
            return getScore(featureName);
        } else {
            return getRank(featureName);
        }
    }

    public int getRank(final String name) {
        return _rl().getRank(name);
    }

    public String toString(final char delim) {
        return _rl().toString(delim);
    }

    public Vector getScoresV(final boolean clonedCopy) {
        return _rl().getScoresV(clonedCopy);
    }

    public float getScore(final int rank) {
        return _rl().getScore(rank);
    }

    public float getScore(final String name) {
        return _rl().getScore(name);
    }

    public float[] getScores(final GeneSet gset) {
        return _rl().getScores(gset);

    }

    public int[] getRanks(final GeneSet gset) {
        return _rl().getRanks(gset);
    }

    public Vector getRanksV(final GeneSet gset) {
        return _rl().getRanksV(gset);
    }

    public RankedList extractRanked(final GeneSet gset) {
        return _rl().extractRanked(gset);
    }

    public RankedList extractRanked(final String[] names) {
        return _rl().extractRanked(names);
    }

    public RankedList extractRanked(final List names) {
        return _rl().extractRanked(names);
    }

    public GeneSet toGeneSet() {
        return _rl().toGeneSet();
    }

    public String toString(final char delim, boolean newLine) {
        return _rl().toString(delim, newLine);
    }

    public int getSize() {
        return _rl().getSize();
    }

    public int getSize(final ScoreMode smode) {
        return _rl().getSize(smode);
    }

    public List getNamesOfUpOrDnXRanks(int topOrBotX, boolean top) {
        return _rl().getNamesOfUpOrDnXRanks(topOrBotX, top);
    }

    public String[] getNamesOfUpOrDnXRanks_array(int topOrBotX, boolean top) {
        return _rl().getNamesOfUpOrDnXRanks_array(topOrBotX, top);
    }

    public GeneSet getNamesOfUpOrDnXRanks_gs(int topOrBotX, boolean top) {
        return _rl().getNamesOfUpOrDnXRanks_gs(topOrBotX, top);
    }

    public GeneSetPair getNamesOfUpAndDnXRanks(int topAndBotX) {
        return _rl().getNamesOfUpAndDnXRanks(topAndBotX);
    }

    public RankedList extractRanked(final int topAndBotX) {
        return _rl().extractRanked(topAndBotX);
    }

    public RankedList extractRanked(final int topOrBotX, final boolean top) {
        return _rl().extractRanked(topOrBotX, top);
    }

    public RankedList extractRanked(float minRankScore, float maxRankScore) {
        return _rl().extractRanked(minRankScore, maxRankScore);
    }

    public int getNumScores(float minRankScore, float maxRankScore) {
        return _rl().getNumScores(minRankScore, maxRankScore);

    }

    public GeneSet getScores_gs(float minRankScore, float maxRankScore) {
        return _rl().getScores_gs(minRankScore, maxRankScore);
    }


    public MetricWeightStruc getMetricWeightStruc() {
        return _rl().getMetricWeightStruc();
    }

    public RankedList extractRanked(final ScoreMode smode) {
        return _rl().extractRanked(smode);
    }

} // End class RankedListJITImpl
