package edu.mit.broad.genome.objects;

/**
 * @author Aravind Subramanian
 * Date: Sep 13, 2005
 * Time: 5:13:44 PM
 */

import java.util.ArrayList;
import java.util.List;


/**
 * A Math object conceptually similar to R's Dataframe but with annotations
 * Basically a labelled Matrix.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class RankedListsAsColumnsDataset extends AbstractRankedListsDataset implements Dataset {

    private int fNumRows;

    private List fColNames;

    public RankedList[] fRankedLists;

    /**
     * Class constructor
     *
     * @param name
     * @param rls
     */
    public RankedListsAsColumnsDataset(final String name, final RankedList[] rls) {
        super.initialize(name);

        this.fColNames = new ArrayList();
        if (rls.length > 0) {
            this.fNumRows = rls[0].getSize();
            for (int c = 0; c < rls.length; c++) {
                if (rls[c].getSize() != fNumRows) {
                    throw new IllegalArgumentException("Unequal length of ranked lists: " + fNumRows + " and " + rls[c].getSize() + " at index: " + c + " name: " + rls[c].getName());
                }
                this.fColNames.add(rls[c].getName());
            }
        } else {
            this.fNumRows = 0;
        }

        this.fRankedLists = rls; // each list becomes a column
    }


    public float getElement(int row, int column) {
        String rowName = getRowName(row);
        return fRankedLists[column].getScore(rowName);
    }

    // arbitarily make the first ranked lists row names this datasets row names
    public String getRowName(int rown) {
        return fRankedLists[0].getRankName(rown);
    }

    public List getRowNames() {
        return fRankedLists[0].getRankedNames();
    }

    public String[] getRowNamesArray() {
        return fRankedLists[0].getRankedNamesArray();
    }

    public List getColumnNames() {
        return fColNames;
    }

    public String getColumnName(int coln) {
        return fColNames.get(coln).toString();
    }

    public int getColumnIndex(String colName) {
        return fColNames.indexOf(colName);
    }

    public int getNumRow() {
        return fNumRows;
    }

    public int getNumCol() {
        return fColNames.size();
    }

}    // End RankedListsDataset

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/

