package edu.mit.broad.genome.objects;

/**
 * @author Aravind Subramanian
 * Date: Sep 13, 2005
 * Time: 5:13:44 PM
 */

import java.util.ArrayList;
import java.util.List;


/**
 * A Math object conceptually similar to R's Dataframe but with annotations
 * Basically a labelled Matrix.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class RankedListsAsRowsDataset extends AbstractRankedListsDataset implements Dataset {

    private int fNumRows;

    private List fRowNames;

    public RankedList[] fRankedLists;

    /**
     * IMP IMP IMP: Each ranked lost must have the exact same order of names!!!!!
     *
     * @param name
     * @param rls
     */
    public RankedListsAsRowsDataset(final String name, final RankedList[] rls) {
        super.initialize(name);

        this.fRowNames = new ArrayList();
        this.fNumRows = rls.length;
        for (int r = 0; r < rls.length; r++) {
            this.fRowNames.add(rls[r].getName());
        }

        this.fRankedLists = rls; // each list becomes a column
    }

    public float getElement(int row, int column) {
        return fRankedLists[row].getScore(column);
    }

    // arbitarily make the first ranked lists row names this datasets row names
    public String getRowName(int rown) {
        return fRowNames.get(rown).toString();
    }

    public List getRowNames() {
        return fRowNames;
    }

    public String[] getRowNamesArray() {
        List list = getRowNames();
        return (String[]) list.toArray(new String[list.size()]);
    }

    public List getColumnNames() {
        return fRankedLists[0].getRankedNames();
    }

    public String getColumnName(int coln) {
        return fRankedLists[0].getRankName(coln);
    }

    public int getColumnIndex(String colName) {
        return fRankedLists[0].getRank(colName);
    }

    public int[] getColumnIndices(final String[] colNames) {
        int[] indices = new int[colNames.length];
        for (int i = 0; i < colNames.length; i++) {
            indices[i] = getColumnIndex(colNames[i]);
        }

        return indices;
    }


    public int getNumRow() {
        return fNumRows;
    }

    public int getNumCol() {
        return fRankedLists[0].getSize();
    }

}    // End RankedListsDataset

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/

