/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import edu.mit.broad.vdb.chip.Chip;

import java.util.Collection;
import java.util.Iterator;

/**
 * --------------------------------------------
 * HELPER CLASS
 * --------------------------------------------
 */
final class SetHelper {

    /**
     * Privatized constructor to prevent extension
     */
    private SetHelper() {
    }

    static String toString(final char delim, final ISet iset) {

        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < iset.getNumMembers(); i++) {
            buf.append(iset.getMember(i));
            if (i != iset.getNumMembers() - 1) {
                buf.append(delim);
            }
        }

        return buf.toString();
    }

    static String toString(final char delim, final GeneSet gset, final Chip chip) {

        try {
            StringBuffer buf = new StringBuffer();

            for (int i = 0; i < gset.getNumMembers(); i++) {
                buf.append(chip.getProbe(gset.getMember(i)).getGene().getSymbol());
                if (i != gset.getNumMembers() - 1) {
                    buf.append(delim);
                }
            }

            return buf.toString();
        } catch (Throwable t) {
            t.printStackTrace();
            return t.getMessage();
        }
    }

    /**
     * @return number of row names  found in the dataset that are members of this fset
     */
    static int getNumMembers(final GeneSet gset, final Dataset ds) {

        int ntrue = 0;

        for (int r = 0; r < ds.getNumRow(); r++) {
            if (gset.isMember(ds.getRowName(r))) {
                ntrue++;
            }
        }

        return ntrue;
    }

    static int getNumMembers(final GeneSet a, final GeneSet b) {

        int ntrue = 0;

        for (int i = 0; i < b.getNumMembers(); i++) {
            if (a.isMember(b.getMember(i))) {
                ntrue++;
            }
        }

        return ntrue;
    }


    static int getNumMembers(final ISet iset, final Collection coll) {

        int ntrue = 0;

        for (Iterator it = coll.iterator(); it.hasNext();) {
            String key = it.next().toString();
            if (iset.isMember(key)) {
                ntrue++;
            }
        }

        return ntrue;
    }

    static int getNumMembers(final ISet iset, final GeneSet gset) {

        int ntrue = 0;

        for (int i = 0; i < gset.getNumMembers(); i++) {
            if (iset.isMember(gset.getMember(i))) {
                ntrue++;
            }
        }

        return ntrue;
    }

    static int getNumMembers(final ISet iset, final RankedList rl) {
        int ntrue = 0;

        for (int i = 0; i < rl.getSize(); i++) {
            if (iset.isMember(rl.getRankName(i))) {
                ntrue++;
            }
        }

        return ntrue;
    }

    static int getNumMembers(final ISet iset, final LabelledList ll) {
        int ntrue = 0;

        for (int i = 0; i < ll.getSize(); i++) {
            if (iset.isMember(ll.getLabel(i))) {
                ntrue++;
            }
        }

        return ntrue;
    }

} // End class SetHelper
