package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.MismatchedSizeException;
import edu.mit.broad.vdb.sampledb.Sample;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @author Aravind Subramanian
 *         Date: Sep 8, 2005
 *         Time: 11:23:25 PM
 */
public class TemplateFromAnnot extends TemplateImpl {

    private Sample[] fSamples;

    private String[] fClassASampleNames_opt;

    private String[] fClassBSampleNames_opt;

    public Template cloneShallow(final String newName) {
        super.setName(newName);
        return this;
    }

    /**
     * Class constructor
     *
     * @param t
     * @param samplesInOrder
     */
    public TemplateFromAnnot(final Template t, final Sample[] samplesInOrder, final String[] classASampleNames_opt, final String[] classBSampleNames_opt) {
        super((TemplateImpl) t);
        super.initialize(t.getName());
        this.init(samplesInOrder, classASampleNames_opt, classBSampleNames_opt);
    }

    private void init(final Sample[] samplesInOrder,
                      final String[] classASampleNames_opt,
                      final String[] classBSampleNames_opt) {

        if (getNumItems() != samplesInOrder.length) {
            throw new MismatchedSizeException("# of items: ", getNumItems(), "# of samples", samplesInOrder.length);
        }

        this.fSamples = samplesInOrder;
        this.fClassASampleNames_opt = classASampleNames_opt;
        this.fClassBSampleNames_opt = classBSampleNames_opt;
    }

    // ordered according to the template
    public Sample[] getSamples_ordered() {
        return fSamples;
    }


    public String[] getSampleNames(final int classIndex) {
        if (fClassASampleNames_opt == null) {
            throw new IllegalStateException("fClassASampleNames_opt: " + fClassASampleNames_opt);
        }

        if (classIndex == 0) {
            return fClassASampleNames_opt;
        } else {
            return fClassBSampleNames_opt;
        }

    }

    public List getSampleNames_list(final int classIndex) {
        return Arrays.asList(getSampleNames(classIndex));
    }

    public String[] getCelFileNames_ordered() {
        final String[] ss = new String[fSamples.length];
        for (int i = 0; i < fSamples.length; i++) {
            ss[i] = fSamples[i].getCelFileName();
        }

        return ss;
    }

    public String[] getSampleNames_ordered() {
        final String[] ss = new String[fSamples.length];
        for (int i = 0; i < fSamples.length; i++) {
            ss[i] = fSamples[i].getName();
        }

        return ss;
    }

    public List getSampleNames_ordered_list() {
        return Arrays.asList(getSampleNames_ordered());
    }

    public static String[] getSampleNames(final TemplateFromAnnot[] dts) {
        Set names = new HashSet();
        for (int i = 0; i < dts.length; i++) {
            String[] sn = dts[i].getSampleNames_ordered();
            for (int s = 0; s < sn.length; s++) {
                names.add(sn[s]);
            }
        }

        return (String[]) names.toArray(new String[names.size()]);
    }

} // End class TemplateFromAnnot
