package edu.mit.broad.genome.objects;

import java.util.*;

/**
 * @author Aravind Subramanian
 *         Date: Oct 25, 2005
 *         Time: 6:35:35 PM
 */
public class Transfac extends AbstractObject {

    private Record[] fRecords;

    /**
     * Class constructor
     *
     * @param file
     */
    public Transfac(final String name, final List records) {
        this(name, (Record[]) records.toArray(new Record[records.size()]));
    }

    /**
     * Class constructor
     *
     * @param name
     * @param records
     */
    public Transfac(final String name, final Record[] records) {
        super(name);
        this.fRecords = records;
    }

    public String getQuickInfo() {
        return null;
    }

    public Record getRecord(final String motif) {
        _initMap();

        final Object obj = fMotifRecordMap.get(motif);
        if (obj != null) {
            return (Record) obj;
        } else {
            return null;
        }

    }

    // lazilly inited
    private Map fMotifRecordMap;

    public void _initMap() {
        if (this.fMotifRecordMap == null) {
            this.fMotifRecordMap = new HashMap();
            for (int i = 0; i < fRecords.length; i++) {
                String[] motifs = fRecords[i].getMotifsArray();
                for (int s = 0; s < motifs.length; s++) { // Clober any existing one for this
                    if (fMotifRecordMap.containsKey(motifs[s])) {
                        log.warn("Duplicate entry for motif: " + motifs[s]);
                    }

                    this.fMotifRecordMap.put(motifs[s], fRecords[i]);
                }
            }
        }

    }

    /**
     * Internal class for a single records
     */
    public static class Record {

        private List fAnnotLines;

        private Set fMotifs;

        public Record(final Set motifs, final List annotLines) {
            this.fAnnotLines = annotLines;
            this.fMotifs = Collections.unmodifiableSet(motifs);
        }

        public Set getMotifs() {
            return fMotifs;
        }

        public String[] getMotifsArray() {
            return (String[]) fMotifs.toArray(new String[fMotifs.size()]);
        }

        public String getDesc() {
            StringBuffer buf = new StringBuffer();
            for (int i = 0; i < fAnnotLines.size(); i++) {
                String ss = fAnnotLines.get(i).toString();
                ss = ss.trim();
                buf.append(ss);
                if (i != fAnnotLines.size() - 1) {
                    buf.append("<br>\n");
                }
            }

            return buf.toString();
        }
    } // End class Record


} // End class Transfac
