/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects;

import edu.mit.broad.genome.math.XMath;
import edu.mit.broad.genome.parsers.AuxUtils;
import gnu.trove.TFloatArrayList;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Aravind Subramanian
 *         Date: Jul 3, 2004
 *         Time: 12:32:01 PM
 */
public class WSFSet extends FSet implements WSGeneSet {

    private float[] fWeights;

    /**
     * Class constructor
     *
     * @param gset
     * @param weights
     */
    public WSFSet(GeneSet gset, float[] weights) {
        super(gset);

        this.fWeights = weights;
    }

    public WSFSet(RankedList rl) {
        super(new FSet(rl.getName(), rl.getNameEnglish(), rl.getRankedNames(), true)); // @todo consider making more efficient
        this.fWeights = rl.getScores(this);
    }

    public String getName(boolean stripAux) {
        if (stripAux) {
            return AuxUtils.getAuxNameOnlyNoHash(getName());
        } else {
            return getName();
        }
    }

    public Member getMember(String name) {
        int index = super.fMembers.indexOf(name);
        if (index == -1) {
            throw new IllegalArgumentException("Not a member: " + name + "\n" + getMembersS());
        } else {
            return new MemberImpl(name, fWeights[index]);
        }
    }

    public WSGeneSet createPositiveScoresGeneSet() {
        TFloatArrayList floats = new TFloatArrayList();
        List labels = new ArrayList();
        for (int i = 0; i < fWeights.length; i++) {
            if (XMath.isPositive(fWeights[i])) {
                floats.add(fWeights[i]);
                labels.add(fMembers.get(i));
            }
        }

        return new WSFSet(new FSet(getName() + "_pos_only", getNameEnglish(), labels, false), floats.toNativeArray());
    }

    public WSGeneSet createNegativeScoresGeneSet() {
        TFloatArrayList floats = new TFloatArrayList();
        List labels = new ArrayList();
        for (int i = 0; i < fWeights.length; i++) {
            if (XMath.isNegative(fWeights[i])) {
                floats.add(fWeights[i]);
                labels.add(fMembers.get(i));
            }
        }

        return new WSFSet(new FSet(getName() + "_neg_only", getNameEnglish(), labels, false), floats.toNativeArray());
    }

    /**
     * Inner class
     */
    static class MemberImpl implements WSGeneSet.Member {
        String name;
        float weight;

        MemberImpl(String name, float weight) {
            this.name = name;
            this.weight = weight;
        }

        public String getName() {
            return name;
        }

        public float getWeight() {
            return weight;
        }

    } // End class MemberImpl

} // End class WSFSet

