package edu.mit.broad.genome.objects.esmatrix.db;

import edu.mit.broad.genome.objects.Dataset;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Fully IN-memory impl
 *
 * @author Aravind Subramanian
 *         Date: Jan 21, 2006
 *         Time: 9:16:06 PM
 */
public class EnrichmentDbMultiImpl extends AbstractEnrichmentDbMulti {

    private EnrichmentDb[] fEdbs;

    private Map fEdbNameEdbMap;

    /**
     * Class constructor
     *
     * @param edbs
     */
    public EnrichmentDbMultiImpl(final String name,
                                 final EnrichmentDb[] edbs) {

        if (edbs == null || edbs.length == 0) {
            throw new IllegalArgumentException("Param edbs cannot be null not zero: " + edbs);
        }

        final String[] compNames = new String[edbs.length];
        for (int c = 0; c < edbs.length; c++) {
            compNames[c] = edbs[c].getName();
        }

        final List list = edbs[0].getGeneSetNames(); // assume rest are like this

        initHere(name, edbs, (String[]) list.toArray(new String[list.size()]), compNames);

        //super.init(name, (String[]) list.toArray(new String[list.size()]), compNames);
    }

    public EnrichmentDbMultiImpl(final String name,
                                 final EnrichmentDb[] edbs,
                                 final String[] useOnlyTheseGeneSetNames,
                                 final String[] useOnlyTheseCompNames) {
        initHere(name, edbs, useOnlyTheseGeneSetNames, useOnlyTheseCompNames);
    }

    // @note that we are restricting to just the comps
    private void initHere(final String name,
                          final EnrichmentDb[] edbs_all,
                          final String[] useOnlyTheseGeneSetNames,
                          final String[] useOnlyTheseCompNames) {

        if (edbs_all == null) {
            throw new IllegalArgumentException("Param edbs_all cannot be null");
        }

        final Map nameEdbMap = new HashMap();
        for (int i = 0; i < edbs_all.length; i++) {
            if (nameEdbMap.containsKey(edbs_all[i].getName())) {
                throw new IllegalArgumentException("Duplicate edb with name: " + edbs_all[i].getName());
            }
            nameEdbMap.put(edbs_all[i].getName(), edbs_all[i]);
        }


        this.fEdbs = new EnrichmentDb[useOnlyTheseCompNames.length];
        this.fEdbNameEdbMap = new HashMap();
        for (int i = 0; i < useOnlyTheseCompNames.length; i++) {
            final String en = useOnlyTheseCompNames[i];
            Object obj = nameEdbMap.get(en);
            if (obj == null) {
                throw new IllegalArgumentException("No such edb: " + en);
            }
            this.fEdbs[i] = (EnrichmentDb) obj;
            fEdbNameEdbMap.put(fEdbs[i].getName(), fEdbs[i]);
        }

        super.init(name, useOnlyTheseGeneSetNames, useOnlyTheseCompNames);
    }

    public EnrichmentDbMulti cloneDeep(final String newName, final String[] useOnlyTheseComps) {
        final EnrichmentDb[] edbs_use = new EnrichmentDb[useOnlyTheseComps.length];
        for (int i = 0; i < useOnlyTheseComps.length; i++) {
            edbs_use[i] = getEdb(useOnlyTheseComps[i]);
        }

        return new EnrichmentDbMultiImpl(getName() + "_" + useOnlyTheseComps.length, edbs_use);
    }

    public EnrichmentDbMulti cloneDeep_gs(final String newName, final String[] useOnlyTheseGeneSetNames) {
        return new EnrichmentDbMultiImpl(newName, this.fEdbs, useOnlyTheseGeneSetNames, getEdbNamesArray());
    }

    public EnrichmentDbMulti cloneDeep(final String[] useOnlyTheseComps) {
        return cloneDeep(getName() + "_" + useOnlyTheseComps.length, useOnlyTheseComps);
    }

    public EnrichmentDbMulti cloneDeep(final String newName,
                                       final String[] useOnyTheseGeneSetNames,
                                       final String[] useOnlyTheseCompNames
    ) {
        return new EnrichmentDbMultiImpl(newName, this.fEdbs, useOnyTheseGeneSetNames, useOnlyTheseCompNames);
    }

    public EnrichmentDbMulti cloneDeep(final Dataset essDataset) {
        return new EnrichmentDbMultiImpl(essDataset.getName(), this.fEdbs,
                essDataset.getRowNamesArray(), essDataset.getColumnNamesArray());
    }

    public EnrichmentDb getEdb(final int i) {
        return fEdbs[i];
    }

    public EnrichmentDb getEdb(final String name) {
        Object obj = fEdbNameEdbMap.get(name);
        if (obj == null) {
            throw new IllegalArgumentException("No such edb: " + name + " \nAvailable: " + fEdbNameEdbMap.keySet());
        }

        return (EnrichmentDb) obj;
    }

} // End class EnrichmentMultiDbImpl
