/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects.esmatrix.db;

import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.GeneSetSignal;
import edu.mit.broad.genome.objects.RankedList;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.objects.strucs.FdrStruc;
import edu.mit.broad.vdb.chip.Chip;

/**
 * Object capturing one "cell" of info in the gsea matrix
 * Note that the dataset, template, gset everything can be different b/w
 * er's
 */
public interface EnrichmentResult {

    /// @note dont provide this because its not always clear what a generic  name is:
    // is it the name of the gset, the template or the rl??
    //public String getName();

    public EnrichmentScore getScore();

    public FdrStruc getFDR();

    public Vector getRndESS();

    public int getNumPerms();

    public GeneSetSignal getSignal();

    // DATA RELATED APIs

    // Results are never from datasets. Some are but not all are made from datasets
    //public Dataset getDataset();

    public RankedList getRankedList();

    public Template getTemplate();

    public GeneSet getGeneSet();

    public String getGeneSetName(); // @for convenience

    public Chip getChip();

} // End EnrichmentResult
