/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects.esmatrix.db;

import edu.mit.broad.genome.math.Vector;
import java.math.BigDecimal;

/**
 * @author Aravind Subramanian
 *         Date: Jul 3, 2003
 *         Time: 10:35:46 PM
 */
public interface EnrichmentScore {

    public static final Type ES = new Type("es");
    public static final Type NES = new Type("nes");
    public static final Type NP = new Type("np");
    public static final Type FDR = new Type("fwer");

    public float getES();
    public BigDecimal getESB();    

    public int getRankAtES();

    public float getRankScoreAtES();

    public Vector getESProfile();

    public Vector getESProfile_point_by_point_opt();

    public int getNumHits();

    public int[] getHitIndices();

    public float getNES();
    public BigDecimal getNESB();

    public float getNP();
    public BigDecimal getNPB();

    public float getFWER();
    public BigDecimal getFWERB();

    public float getFDR();
    public BigDecimal getFDRB();

    public boolean hasDeepInfo();

    public float getScore(Type type);
    public BigDecimal getScoreB(Type type);

    // -------------------------------------------------------------------------------------------- //

    /**
     * Inner class for a type
     */
    public static class Type {

        private String fName;

        Type(String name) {
            this.fName = name;
        }

        public String toString() {
            return fName;
        }

        public int hashCode() {
            return fName.hashCode();
        }

        public boolean equals(Object obj) {
            if (obj instanceof Type) {
                return ((Type) obj).fName.equals(fName);
            }

            return false;
        }

        public String getName() {
            return fName;
        }
    } // End class Type

} // End class EnrichmentScore
