package edu.mit.broad.genome.objects.esmatrix.db;

import edu.mit.broad.genome.math.Vector;
import java.math.BigDecimal;

/**
 * @author Aravind Subramanian
 *         Date: Jul 3, 2003
 *         Time: 10:35:46 PM
 */
public class EnrichmentScoreImplP implements EnrichmentScore {

    private int fNumHits;
    private BigDecimal fES;
    private int fRankAtES;
    private float fRankScoreAtES;

    private BigDecimal fNES;
    private BigDecimal fNP;
    private BigDecimal fFWER;
    private BigDecimal fFDR;

    // deep vars -- not always available
    private Vector fESProfile;
    private Vector fESProfile_point_by_point;
    private int[] fHitIndices;

    //GSAA_Seq Minp only  
    public EnrichmentScoreImplP(final BigDecimal es,
                               final BigDecimal nes,
                               final BigDecimal np,
                               final BigDecimal fdr,
                               final BigDecimal fwer) {
        init(es, nes, np, fdr, fwer);
    }       

    //GSAA-Seq only
    public EnrichmentScoreImplP(EnrichmentScore es,
                               final BigDecimal nes,
                               final BigDecimal np,
                               final BigDecimal fdr,
                               final BigDecimal fwer,
                               final String flag) {
        init(es.getESB(), nes, np, fdr, fwer);
    }

    //GSAA-Seq Minp only  
    protected void init(
            final BigDecimal es,
            final BigDecimal nes,
            final BigDecimal np,
            final BigDecimal fdr,
            final BigDecimal fwer) {

        this.fES = es;
        this.fNES = nes;
        this.fNP = np;
        this.fFDR = fdr;
        this.fFWER = fwer;
    }    

    public boolean hasDeepInfo() {
        return fESProfile != null;
    }
   
    public BigDecimal getESB() {
        return fES;
    }    

    public int getRankAtES() {
        return fRankAtES;
    }

    public float getRankScoreAtES() {
        return fRankScoreAtES;
    }
    
    public BigDecimal getNESB() {
        return fNES;
    }    
    
    public BigDecimal getNPB() {
        return fNP;
    }    
    
    public BigDecimal getFWERB() {
        return fFWER;
    }    
    
    public BigDecimal getFDRB() {
        return fFDR;
    }    

    public Vector getESProfile() {
        //ensureDeep();
        if (fESProfile != null) {
            fESProfile.setImmutable();
        }
        return fESProfile;
    }

    public Vector getESProfile_point_by_point_opt() {
        return fESProfile_point_by_point;
    }

    public int getNumHits() {
        return fNumHits;
    }

    public int[] getHitIndices() {
        //ensureDeep();
        return fHitIndices;
    }

    public BigDecimal getScoreB(Type type) {
        if (type == ES) {
            return getESB();
        } else if (type == NES) {
            return getNESB();
        } else if (type == FDR) {
            return getFDRB();
        } else if (type == NP) {
            return getNPB();
        } else {
            throw new IllegalArgumentException("Unknown Type: " + type);
        }
    }


    private void ensureDeep() {
        if (!hasDeepInfo()) {
            throw new IllegalStateException("KSScore was NOT consructed in deep mode -- info not available");
        }
    }

    @Override
    public float getES() {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public float getNES() {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public float getNP() {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public float getFWER() {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public float getFDR() {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public float getScore(Type type) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

} // End class KSScoreStruc
