package edu.mit.broad.genome.objects.strucs;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.vdb.chip.Chip;
import org.apache.log4j.Logger;

/**
 * @author Aravind Subramanian
 *         Date: Sep 6, 2005
 *         Time: 6:51:15 PM
 */
public class DatasetMode {


    public static final DatasetMode NAT = new DatasetMode("nat");
    public static final DatasetMode LOG2 = new DatasetMode("log");
    public static final DatasetMode NAT_SYMBOLS = new DatasetMode("nat_symbols");
    public static final DatasetMode LOG2_SYMBOLS = new DatasetMode("log_symbols");

    private Logger log = XLogger.getLogger(DatasetMode.class);

    private String fName;

    public static DatasetMode[] createAllModes() {
        return new DatasetMode[]{NAT, LOG2, NAT_SYMBOLS, LOG2_SYMBOLS};
    }

    /**
     * PRIVATIZED class constructor
     *
     * @param name
     */
    protected DatasetMode(final String name) {
        this.fName = name;
    }

    public boolean equals(final Object obj) {
        if (obj instanceof DatasetMode) {
            return ((DatasetMode) obj).fName.equals(this.fName);
        } else if (obj != null && obj.toString().equals(this.fName)) {
            return true;
        }

        return false;
    }

    public String toString() {
        return fName;
    }

    public String getName() {
        return fName;
    }

    // This is the magix
    // DO not cache here -- let client code handle that if needed
    // safer for memory reasons
    public Dataset extractDataset(final Dataset ds,
                                  final DatasetMode toMode,
                                  final Chip chip) {

        final DatasetMode currMode = this;

        if (currMode == toMode) {
            log.debug("currMode: " + currMode + " same as toMode: " + toMode + " nothing extracted");
            return ds;
        }

        log.info("Extracting dataset: " + ds.getName() + " from: " + this.getName() + " to: " + toMode.getName());

        final DatasetGenerators gen = new DatasetGenerators();

        if (currMode == NAT) {
            if (toMode == LOG2) {
                return gen.log2(ds);
            } else if (toMode == NAT_SYMBOLS) {
                return gen.collapse(ds, chip);
            } else if (toMode == LOG2_SYMBOLS) { // log and then collapse
                return gen.collapse(gen.log2(ds), chip);
            }
        } else if (currMode == LOG2) {
            if (toMode == NAT) {
                return gen.exp2(ds);
            } else if (toMode == NAT_SYMBOLS) {
                return gen.collapse(gen.exp2(ds), chip); // exp and then collapse
            } else if (toMode == LOG2_SYMBOLS) {
                return gen.collapse(ds, chip);
            }
        } else if (currMode == NAT_SYMBOLS) {
            if (toMode == NAT) {
                throw new NotImplementedException(); // symbol ds to probe ds not easy to do
            } else if (toMode == LOG2) {
                throw new NotImplementedException();
            } else if (toMode == LOG2_SYMBOLS) {
                return gen.log2(ds);
            }
        } else if (currMode == LOG2_SYMBOLS) {
            if (toMode == NAT) {
                throw new NotImplementedException();
            } else if (toMode == LOG2) {
                throw new NotImplementedException();
            } else if (toMode == NAT_SYMBOLS) {
                return gen.exp2(ds);
            }
        }

        throw new IllegalArgumentException("Unknown combination currMode: " + currMode + " and toMode " + toMode);
    }

    public static DatasetMode lookupMode(final Object obj) {

        if (obj == null) {
            throw new NullPointerException("Cannot lookup for null object");
        }

        if (obj instanceof DatasetMode) {
            return (DatasetMode) obj;
        }

        DatasetMode[] all = createAllModes();

        for (int i = 0; i < all.length; i++) {
            if (all[i].getName().equalsIgnoreCase(obj.toString())) {
                return all[i];
            }
        }

        throw new RuntimeException("Cannot lookupMode for: " + obj);
    }

} // End class DatasetModes
