/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects.strucs;

import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.alg.DatasetGenerators;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.GeneSet;
import org.apache.log4j.Logger;

import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Feb 12, 2005
 *         Time: 4:51:28 PM
 */
public class DatasetWrapperImpl2 implements DatasetWrapper {

    private Dataset fDataset;

    private DatasetWrapper fWrapped;

    private String[] fRestrictColNames_opt;

    private GeneSet fRestrict2Genes_opt;

    private boolean fDoneExtraction;

    private Logger log = XLogger.getLogger(DatasetWrapperImpl2.class);

    /**
     * Class constructor
     *
     * @param wrapped
     * @param restrict2Cols
     */
    public DatasetWrapperImpl2(final DatasetWrapper wrapped, final String[] restrict2Cols) {
        this.fWrapped = wrapped;
        this.fRestrictColNames_opt = restrict2Cols;
        this.fDoneExtraction = false;
    }

    /**
     * Class constructor
     *
     * @param wrapped
     * @param restrict2Genes
     */
    public DatasetWrapperImpl2(final DatasetWrapper wrapped, final GeneSet restrict2Genes) {
        this.fWrapped = wrapped;
        this.fRestrict2Genes_opt = restrict2Genes;
        this.fDoneExtraction = false;
    }

    public String getDatasetName() {
        String name = fWrapped.getDatasetName();
        if (fRestrictColNames_opt != null) {
            name = name + "_" + fRestrictColNames_opt.length;
        }

        if (fRestrict2Genes_opt != null) {
            name = name + "_" + fRestrict2Genes_opt.getName(true);
        }

        return name;
    }

    public File getFile() {
        return fWrapped.getFile();
    }

    public String getPath() {
        return getFile().getPath();
    }

    // be careful, very careful!!
    public Dataset getDataset(final boolean add2cache) {

        // if (fDataset == null) { @note IMP IMP cant do this check as dataset might be set in a constructor
        boolean wasExtr = false;
        boolean wasGotFromWrapped = false;
        if (!fDoneExtraction) {
            wasExtr = true;
            try {

                if (fDataset == null) { // @note dont re-read that might destroy the current state of dataset
                    wasGotFromWrapped = true;
                    this.fDataset = fWrapped.getDataset(add2cache); // delegate
                }

                if (fRestrictColNames_opt != null) {
                    String name = fDataset.getName() + "_" + fRestrictColNames_opt.length;
                    this.fDataset = new DatasetGenerators().extractColumns(name, fDataset, fRestrictColNames_opt);
                    log.debug("Restricting columns: " + fRestrictColNames_opt.length + " " + fDataset.getName());
                }

                if (fRestrict2Genes_opt != null) {
                    String name = fDataset.getName() + "_" + fRestrict2Genes_opt.getName(true);
                    this.fDataset = new DatasetGenerators().extractRows(name, fDataset, fRestrict2Genes_opt);
                    log.debug("Restricting genes: " + fRestrict2Genes_opt.getNumMembers() + " " + fDataset.getName());
                }

                this.fDoneExtraction = true;

            } catch (Throwable t) {
                throw new RuntimeException(t);
            }
        }

        log.debug("#### DATASET: " + fDataset.getQuickInfo() + " was extr: " + wasExtr + " wasGotFromWrapped: " + wasGotFromWrapped);
        //TraceUtils.showTrace();
        return fDataset;
    }

    public Dataset getDataset() {
        return getDataset(false);
    }

    public static Dataset[] toDatasets(final DatasetWrapper[] dts, final boolean add2cache) {
        Dataset[] dss = new Dataset[dts.length];
        for (int i = 0; i < dts.length; i++) {
            dss[i] = dts[i].getDataset(add2cache);
        }

        return dss;
    }

} // End class DatasetWrapperImpl2


