/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects.strucs;

/**
 * Enum construct
 */
public class GeneSetRandomizerType {

    private String name;

    public static final GeneSetRandomizerType RANDOM_TAGS_RANDOM_SIZE = new GeneSetRandomizerType("random_tags_and_random_size");
    public static final GeneSetRandomizerType RANDOM_TAGS_FIXED_SIZE = new GeneSetRandomizerType("random_tags_fixed_size");

    /**
     * Privatized class constructor
     *
     * @param name
     */
    private GeneSetRandomizerType(String name) {
        this.name = name;
    }

    public String toString() {
        return name;
    }

    /**
     * @return
     * @maint need to keep manually synched
     */
    public static GeneSetRandomizerType[] createAll() {
        return new GeneSetRandomizerType[]{RANDOM_TAGS_RANDOM_SIZE, RANDOM_TAGS_FIXED_SIZE};
    }

    public static GeneSetRandomizerType lookupRandomizerType(Object obj) {
        if (obj == null) {
            throw new IllegalArgumentException("Param rt cannot be null");
        }

        if (obj instanceof GeneSetRandomizerType) {
            return (GeneSetRandomizerType) obj;
        }

        String rt = obj.toString();

        if (rt.equalsIgnoreCase(RANDOM_TAGS_RANDOM_SIZE.name)) {
            return RANDOM_TAGS_RANDOM_SIZE;
        } else if (rt.equalsIgnoreCase(RANDOM_TAGS_FIXED_SIZE.name)) {
            return RANDOM_TAGS_FIXED_SIZE;
        } else {
            throw new IllegalArgumentException("Unknown RandomizationType: " + rt);
        }
    }

} // End class RandomizerType
