package edu.mit.broad.genome.objects.strucs;

import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.parsers.ParserFactory;

import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Feb 12, 2005
 *         Time: 4:51:28 PM
 */
public class GeneSetWrapperImpl implements GeneSetWrapper {

    private GeneSet fGeneSet;

    private File fFile;

    /**
     * Class constructor
     *
     * @param gset
     */
    public GeneSetWrapperImpl(final GeneSet gset) {

        if (gset == null) {
            throw new IllegalArgumentException("Param gset cannot be null");
        }

        this.fGeneSet = gset;
    }

    public GeneSetWrapperImpl(final String path) {
        this(new File(path));
    }

    /**
     * Class constructor
     *
     * @param gsetFile
     */
    public GeneSetWrapperImpl(final File gsetFile) {

        if (gsetFile == null) {
            throw new IllegalArgumentException("Param gsetFile cannot be null");
        }

        this.fFile = gsetFile;
    }

    public String getGeneSetName() {
        if (fGeneSet != null) {
            return fGeneSet.getName();
        } else {
            return NamingConventions.removeExtension(fFile.getName());
        }
    }

    public File getFile() {
        if (fFile == null) {
            this.fFile = ParserFactory.getCache().getSourceFile(fGeneSet);
        }

        return fFile;
    }


    public String getPath() {
        return getFile().getPath();
    }

    public GeneSet getGeneSet() {
        if (fGeneSet == null) {
            try {
                this.fGeneSet = ParserFactory.readGeneSet(fFile, false);
            } catch (Throwable t) {
                throw new RuntimeException(t);
            }
        }

        return fGeneSet;
    }

} // End class GeneSetWrapperImpl


