/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects.strucs;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.GeneSetSignal;
import edu.mit.broad.genome.objects.RankedList;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentResult;
import edu.mit.broad.genome.objects.esmatrix.db.EnrichmentScore;
import edu.mit.broad.vdb.chip.Chip;

/**
 *
 */
public class ProbeGeneSetScoreStruc implements EnrichmentResult {

    private String fProbeName;

    // NONE of the scores are normalized - all raw
    private float fRealEs;
    public GeneSet fGeneSet;

    /**
     * The following are not always available
     */
    private RankedList rlReal;

    private EnrichmentScore fScore;

    private Vector fRndESScores;


    public FdrStruc getFDR() {
        throw new NotImplementedException();
    }


    public Chip getChip() {
        throw new NotImplementedException();
    }

    /**
     * Class constructor
     *
     * @param realEs
     * @param rl
     * @param struc
     */
    public ProbeGeneSetScoreStruc(String probeName, GeneSet gset, RankedList rl,
                                  EnrichmentScore struc, Vector rndEsScores) {


        if (rl == null) {
            throw new IllegalArgumentException("Parameter rl cannot be null in this mode of isage");
        }

        if (struc == null) {
            throw new IllegalArgumentException("Parameter struc cannot be null in this mode of usage");
        }

        init(probeName, gset, struc.getES(), rl, struc, rndEsScores);
    }

    /**
     * Class constructor
     *
     * @param probeName
     * @param gset
     * @param realEs
     * @param rndEsScores
     */
    public ProbeGeneSetScoreStruc(String probeName, GeneSet gset, float realEs, Vector rndEsScores) {
        init(probeName, gset, realEs, null, null, rndEsScores);
    }

    private void init(String probeName, GeneSet gset, float realEs,
                      RankedList rlReal, EnrichmentScore struc, Vector rndEsScores) {

        if (probeName == null) {
            throw new IllegalArgumentException("Parameter probeName cannot be null");
        }

        if (gset == null) {
            throw new IllegalArgumentException("Parameter geneset cannot be null");
        }

        this.fProbeName = probeName;
        this.fGeneSet = gset;
        this.fRealEs = realEs;
        this.rlReal = rlReal;
        this.fScore = struc;
        this.fRndESScores = rndEsScores;
    }


    public GeneSetSignal getSignal() {
        throw new NotImplementedException();
    }


    public String getName() {
        return fProbeName;
    }

    public String getProbeName() {
        return fProbeName;
    }

    public RankedList getRankedList() {
        return rlReal;
    }

    public EnrichmentScore getScore() {
        if (fScore == null) {
            throw new IllegalStateException("Not made in deep mode - score na");
        }

        return fScore;
    }

    public float getES() {
        return fRealEs;
    }

    public Vector getRndESS() {
        //System.out.println("### " + fRealEs + " " + fRndEsScores.toString(' '));
        return new Vector(fRndESScores); // @note IMP IMp must return a safe copy!!!
    }

    public Vector getRndMW() {
        return new Vector(fRndESScores.getSize()); // @note IMP IMp must return a safe copy!!!
    }

    public float getRealMW() {
        return Float.NaN;
    }

    public Template getTemplate() {
        throw new NotImplementedException();
    }

    public GeneSet getGeneSet() {
        return fGeneSet;
    }

    public String getGeneSetName() {
        return fGeneSet.getName(true);
    }

    public int getNumHits() {
        return fScore.getNumHits();
    }

    public int getNumPerms() {
        return fRndESScores.getSize();
    }

    public int[] getHitIndices() {
        return fScore.getHitIndices();
    }

    public Vector getESProfile() {
        return fScore.getESProfile();
    }

    public int getRankAtEs() {
        throw new NotImplementedException();
    }

    public float getRankScoreAtEs() {
        throw new NotImplementedException();
    }

    public EnrichmentScore getESStruc() {
        throw new NotImplementedException();
    }

} // End class ProbeGeneSetScoreStruc
