package edu.mit.broad.genome.objects.strucs;

import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.objects.RankedList;
import edu.mit.broad.genome.parsers.ParserFactory;

import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Feb 12, 2005
 *         Time: 4:51:28 PM
 */
public class RankedListWrapperImpl implements RankedListWrapper {

    private RankedList fRankedList;

    private File fFile;

    /**
     * Class constructor
     *
     * @param rl
     */
    public RankedListWrapperImpl(final RankedList rl) {

        if (rl == null) {
            throw new IllegalArgumentException("Param rl cannot be null");
        }

        this.fRankedList = rl;
    }

    public RankedListWrapperImpl(final String path) {
        this(new File(path));
    }

    /**
     * Class constructor
     *
     * @param rlFile
     */
    public RankedListWrapperImpl(final File rlFile) {

        if (rlFile == null) {
            throw new IllegalArgumentException("Param tFile cannot be null");
        }

        this.fFile = rlFile;
    }

    public String getRankedListName() {
        if (fRankedList != null) {
            return fRankedList.getName();
        } else {
            return NamingConventions.removeExtension(fFile.getName());
        }
    }

    public File getFile() {
        if (fFile == null) {
            this.fFile = ParserFactory.getCache().getSourceFile(fRankedList);
        }

        return fFile;
    }


    public String getPath() {
        return getFile().getPath();
    }

    public RankedList getRankedList() {
        if (fRankedList == null) {
            try {
                this.fRankedList = ParserFactory.readRankedList(fFile, false);
            } catch (Throwable t) {
                throw new RuntimeException(t);
            }
        }

        return fRankedList;
    }

} // End class RankedListWrapperImpl



