/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects.strucs;

/**
 * Enum construct
 */
public class TemplateRandomizerType {

    private String name;

    public static final TemplateRandomizerType NO_BALANCE = new TemplateRandomizerType("no_balance");
    public static final TemplateRandomizerType BALANCED_CLASS0 = new TemplateRandomizerType("balance_class0");
    public static final TemplateRandomizerType BALANCED_CLASS1 = new TemplateRandomizerType("balance_class1");
    public static final TemplateRandomizerType EQUALIZE_AND_BALANCE = new TemplateRandomizerType("equalize_and_balance");

    /**
     * Privatized class constructor
     *
     * @param name
     */
    private TemplateRandomizerType(String name) {
        this.name = name;
    }

    public String toString() {
        return name;
    }

    /**
     * @return
     * @maint need to keep manually synched
     */
    public static TemplateRandomizerType[] createAll() {
        return new TemplateRandomizerType[]{NO_BALANCE, BALANCED_CLASS0, BALANCED_CLASS1, EQUALIZE_AND_BALANCE};
    }

    public static TemplateRandomizerType lookupRandomizerType(Object obj) {
        if (obj == null) {
            throw new IllegalArgumentException("Param rt cannot be null");
        }

        if (obj instanceof TemplateRandomizerType) {
            return (TemplateRandomizerType) obj;
        }

        String rt = obj.toString();

        if (rt.equalsIgnoreCase(NO_BALANCE.name)) {
            return NO_BALANCE;
        } else if (rt.equalsIgnoreCase(BALANCED_CLASS0.name)) {
            return BALANCED_CLASS0;
        } else if (rt.equalsIgnoreCase(BALANCED_CLASS1.name)) {
            return BALANCED_CLASS1;
        } else if (rt.equalsIgnoreCase(EQUALIZE_AND_BALANCE.name)) {
            return EQUALIZE_AND_BALANCE;           
        } else {
            throw new IllegalArgumentException("Unknown RandomizationType: " + rt);
        }
    }

} // End class RandomizerType
