/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.objects.strucs;

import edu.mit.broad.genome.NamingConventions;
import edu.mit.broad.genome.objects.Template;
import edu.mit.broad.genome.parsers.ParserFactory;

import java.io.File;

/**
 * @author Aravind Subramanian
 *         Date: Feb 12, 2005
 *         Time: 4:51:28 PM
 */
public class TemplateWrapperImpl implements TemplateWrapper {

    private Template fTemplate;

    private File fTemplateFile;

    /**
     * Class constructor
     *
     * @param t
     */
    public TemplateWrapperImpl(final Template t) {

        if (t == null) {
            throw new IllegalArgumentException("Param t cannot be null");
        }

        this.fTemplate = t;
    }

    public TemplateWrapperImpl(final String path) {
        this(new File(path));
    }

    /**
     * Class constructor
     *
     * @param tFile
     */
    public TemplateWrapperImpl(final File tFile) {

        if (tFile == null) {
            throw new IllegalArgumentException("Param tFile cannot be null");
        }

        this.fTemplateFile = tFile;
    }

    public String getTemplateName() {
        if (fTemplate != null) {
            return fTemplate.getName();
        } else {
            return NamingConventions.removeExtension(fTemplateFile.getName());
        }
    }

    public File getFile() {
        if (fTemplateFile == null) {
            this.fTemplateFile = ParserFactory.getCache().getSourceFile(fTemplate);
        }

        return fTemplateFile;
    }


    public String getPath() {
        return getFile().getPath();
    }

    public Template getTemplate() {
        if (fTemplate == null) {
            try {
                this.fTemplate = ParserFactory.readTemplate(fTemplateFile);
            } catch (Throwable t) {
                throw new RuntimeException(t);
            }
        }

        return fTemplate;
    }

} // End class TemplateWrapperImpl


