package edu.mit.broad.genome.objects.strucs;

import edu.mit.broad.genome.objects.AbstractObject;
import edu.mit.broad.genome.objects.FSet;
import edu.mit.broad.genome.objects.GeneSet;
import edu.mit.broad.genome.objects.LabelledVector;
import gnu.trove.THashMap;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author Aravind Subramanian
 *         Date: Dec 2, 2005
 *         Time: 4:05:38 PM
 */
public class TxtAffy extends AbstractObject {

    private Value[] fValues;

    public TxtAffy cloneDeep(final GeneSet gset, final boolean barfOnMissing) {

        Value[] values = new Value[gset.getNumMembers()];
        for (int i = 0; i < gset.getNumMembers(); i++) {
            values[i] = getValue(gset.getMember(i), barfOnMissing);
        }

        return new TxtAffy(getName() + "_" + gset.getName(true), values);
    }

    /**
     * Class constructor
     *
     * @param name
     * @param values
     */
    public TxtAffy(final String name, final Value[] values) {
        super(name);
        this.fValues = values;
    }

    public String getQuickInfo() {
        return null;
    }

    public int getNumValues() {
        return fValues.length;
    }

    public Value getValue(int i) {
        return fValues[i];
    }

    public GeneSet getPCallProbeSets() {
        List passes = new ArrayList();

        for (int i = 0; i < getNumValues(); i++) {
            if (getValue(i).isPass()) {
                passes.add(getValue(i).fFeatureName);
            }
        }

        return new FSet(getName() + "_pass", passes, false);
    }

    public LabelledVector toLV(final GeneSet gset, final boolean barfOnMissing) {

        final String[] labels = new String[gset.getNumMembers()];
        final float[] values = new float[gset.getNumMembers()];

        for (int i = 0; i < gset.getNumMembers(); i++) {
            labels[i] = gset.getMember(i);
            values[i] = getValue_f(gset.getMember(i), barfOnMissing);
        }

        return new LabelledVector(gset.getName(true), labels, values);
    }

    public LabelledVector toLV() {

        final String[] labels = new String[getNumValues()];
        final float[] values = new float[getNumValues()];

        for (int i = 0; i < getNumValues(); i++) {
            labels[i] = getValue(i).fFeatureName;
            values[i] = getValue(i).fSignal;
        }

        return new LabelledVector(getName(), labels, values);
    }

    public String toString_tab_delim() {
        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < getNumValues(); i++) {
            Value value = getValue(i);
            buf.append(value.fFeatureName).append('\t').append(value.fSignal);
            if (value.hasPCall()) {
                buf.append('\t').append(value.fCall);
            }

            buf.append('\n');
        }

        return buf.toString();
    }

    public String[] getFeatureNames() {
        String[] names = new String[getNumValues()];
        for (int i = 0; i < getNumValues(); i++) {
            names[i] = getValue(i).fFeatureName;
        }

        return names;
    }

    public LabelledVector toLabelledVector() {
        String[] names = new String[getNumValues()];
        float[] scores = new float[getNumValues()];
        for (int i = 0; i < getNumValues(); i++) {
            names[i] = getValue(i).fFeatureName;
            scores[i] = getValue(i).fSignal;
        }

        return new LabelledVector(getName(), names, scores);
    }

    private Map fNameValueMap;

    public Value getValue(final String probeName) {
        return getValue(probeName, true);
    }

    public float getValue_f(final String probeName, final boolean barfOnMissing) {
        Value value = getValue(probeName, barfOnMissing);
        if (value == null) {
            return 0.0f;
        } else {
            return value.fCall;
        }
    }

    public Value getValue(final String probeName, final boolean barfOnMissing) {
        if (this.fNameValueMap == null) {
            this.fNameValueMap = new THashMap();
            for (int i = 0; i < fValues.length; i++) {
                fNameValueMap.put(fValues[i].fFeatureName, fValues[i]);
            }
        }

        Object obj = fNameValueMap.get(probeName);
        if (obj == null) {
            if (barfOnMissing) {
                throw new IllegalArgumentException("No such probe: " + probeName);
            } else {
                return null;
            }
        } else {
            return (Value) obj;
        }
    }

    public static class Value {

        public String fFeatureName;

        public char fCall;

        public float fDetectionPValue;

        public float fSignal; // e.g gex

        public Value(final String featureName, char call, final float dpvalue, final float signal) {
            this.fFeatureName = featureName;
            this.fDetectionPValue = dpvalue;
            this.fSignal = signal;

            if (Character.isDefined(call) && call != Character.UNASSIGNED) {
                call = Character.toUpperCase(call);
                if (call == 'P' || call == 'A' || call == 'M') {
                    this.fCall = call;
                }
            } else {
                this.fCall = Character.UNASSIGNED;
            }
        }

        public boolean hasPCall() {
            if (Character.isDefined(fCall) && fCall != Character.UNASSIGNED) {
                if (fCall == 'P' || fCall == 'M' || fCall == 'A') {
                    return true;
                }
            }

            return false;
        }

        public boolean isPass() {
            return fCall == 'P';
        }

    }

} // End class TxtAffy

