/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.parsers;


import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.math.ObjectMatrix;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.genome.objects.esmatrix.db.DtgDataset;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

/**
 *
 */
public class DtgDatasetParser extends AbstractParser {

    /**
     * Class Constructor.
     */
    public DtgDatasetParser() {
        super(DtgDataset.class);
    }

    /**
     *
     */
    public void export(PersistentObject pob, File file) throws Exception {

        PrintWriter pw = startExport(pob, file);
        DtgDataset dtgds = (DtgDataset) pob;
        export(dtgds, pw);
        doneExport();

    }    // End export

    // does the real stuff
    public void export(DtgDataset dtgds, PrintWriter pw) throws Exception {

        pw.print(Constants.NAME + "\t");

        for (int i = 0; i < dtgds.getNumCol(); i++) {
            pw.print(dtgds.getColName(i));
            pw.print('\t');
        }

        pw.println();

        for (int r = 0; r < dtgds.getNumRow(); r++) {
            pw.print(dtgds.getRowName(r));
            pw.print('\t');
            for (int c = 0; c < dtgds.getNumCol(); c++) {
                StringBuffer buf = new StringBuffer();
                // @maint IMP the order is important (see below)
                // order also implicit in DtgDataset
                buf.append(dtgds.getRealEs(r, c)).append(',');
                buf.append(dtgds.getNes(r, c)).append(',');
                buf.append(dtgds.getNominalPValue(r, c)).append(',');
                buf.append(dtgds.getDiscoveryPValue(r, c)).append(',');
                buf.append(dtgds.getValidationPValue(r, c));
                buf.append('\t');
                pw.print(buf.toString());
            }
            pw.println();
        }

        pw.close();
    }


    /**
     * @returns 1 DtgDataset object
     * NO ann buiisness
     * @see above for format
     */
    public List parse(String sourcepath, InputStream is) throws Exception {
        startImport(sourcepath);
        BufferedReader bin = new BufferedReader(new InputStreamReader(is));
        DtgDataset dtgds = parse(sourcepath, bin);
        return unmodlist(new PersistentObject[]{dtgds});
    }

    /// does the real parsing
    // expects the bin to be untouched
    public DtgDataset parse(String objname, BufferedReader bin) throws Exception {
        String currLine = nextLine(bin);

        // 1st  non-empty, non-comment line is the col header:
        // First fields is to be ignored
        List colnames = ParseUtils.string2stringsList(currLine, "\t");
        log.debug("# cols found: " + colnames.size());

        colnames.remove(0);                                 // first elem is always nonesense

        // At this point, currLine should contain the first data line
        // data line: <row name> <tab> <ex1> <tab> <ex2> <tab>
        List lines = new ArrayList();

        currLine = nextLine(bin);

        // save all rows so that we can determine how many rows exist
        while (currLine != null) {
            lines.add(currLine);
            currLine = nextLine(bin);
        }

        DtgDataset dtgds = _parse(objname, lines, colnames);

        bin.close();

        doneImport();
        return dtgds;

    }

    private DtgDataset _parse(String objname, List lines, List colNames) throws Exception {

        ObjectMatrix matrix = new ObjectMatrix(lines.size(), colNames.size());
        List rowNames = new ArrayList();

        for (int r = 0; r < lines.size(); r++) {
            String currLine = (String) lines.get(r);
            List fields = ParseUtils.string2stringsList(currLine, " \t"); // spaces NOT allowed

            if (fields.size() != colNames.size() + 1) {
                throw new ParserException("Bad format - expect ncols: " + (colNames.size() + 1)
                        + " but found: " + fields.size() + " on line: "
                        + currLine);
            }

            String rowname = (String) fields.get(0);
            rowNames.add(rowname);

            // now each element is a csv for 1 Dtg
            for (int c = 1; c < fields.size(); c++) {
                String elem = fields.get(c).toString();
                float[] vals = ParseUtils.string2floats(elem, "\",");
                matrix.setElement(r, c - 1, vals);
            }
        }

        log.info("Completed parsing DtgDataset");

        DtgDataset dtgds = new DtgDataset(objname, matrix, rowNames, colNames);
        dtgds.addComment(fComment.toString());
        return dtgds;
    }


}    // End of class DtgDatasetParser

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
