package edu.mit.broad.genome.parsers;

/**
 * @author Aravind Subramanian
 * Date: Oct 21, 2006
 * Time: 6:21:26 PM
 */

/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */


import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.vdb.VdbRuntimeResources;
import edu.mit.broad.vdb.chip.Probe;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


/**
 * @see http://www.ncbi.nlm.nih.gov/projects/geo/info/soft2.html
 */

public class EntrezGeneWebPageParser extends AbstractParser {

    /**
     * Class Constructor.
     */
    public EntrezGeneWebPageParser() {
        super(Probe.class);
    }

    /**
     */
    public void export(PersistentObject pob, File file) throws Exception {
        throw new NotImplementedException();

    }

    public List parse(String sourcepath, InputStream is) throws Exception {
        throw new NotImplementedException();
    }

    /**
     * http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=gene&term=AB007953[accn]
     * <p/>
     * <?xml version="1.0"?>
     * <!DOCTYPE eSearchResult PUBLIC "-//NLM//DTD eSearchResult, 11 May 2002//EN" "http://www.ncbi.nlm.nih.gov/entrez/query/DTD/eSearch_020511.dtd">
     * <eSearchResult>
     * <Count>1</Count>
     * <RetMax>1</RetMax>
     * <RetStart>0</RetStart>
     * <IdList>
     * <Id>57240</Id>
     * <p/>
     * </IdList>
     * <TranslationSet>
     * </TranslationSet>
     * <TranslationStack>
     * <TermSet>
     * <Term>AB007953[accn]</Term>
     * <Field>accn</Field>
     * <Count>1</Count>
     * <p/>
     * <Explode>Y</Explode>
     * </TermSet>
     * <OP>GROUP</OP>
     * </TranslationStack>
     * <QueryTranslation>AB007953[accn]</QueryTranslation>
     * </eSearchResult>
     *
     * @param accName
     * @return
     * @throws Exception
     */
    public String parseSymbol(final String accName) throws Exception {
        super.setSilentMode(true);
        String sourcepath = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=gene&term=" + accName;
        if (accName.toUpperCase().startsWith("HS.") == false) { // i.e not a unigene accession
            sourcepath += "[accn]";
        }
        InputStream is = new URL(sourcepath).openConnection().getInputStream();
        startImport(sourcepath);

        final BufferedReader bin = new BufferedReader(new InputStreamReader(is));
        String currLine = nextLine(bin, false);

        Set ezids = new HashSet();

        // Sometimes there are >1 ez ids returned
        // save all and use the one that is current (i.e has a match from ez db symbol lookup)
        while (currLine != null) {

            //System.out.println(">>" + currLine);

            if (currLine.startsWith("<Id>")) {
                // parsing e.g <Id>57240</Id>
                String[] fields = ParseUtils.string2strings(currLine, "<>", false);
                ezids.add(fields[1]);
                //break;
            }

            if (currLine.startsWith("</IdList>")) {
                break;
            }

            currLine = nextLine(bin);
        }

        bin.close();
        doneImport();

        String symbol = null;
        if (ezids.isEmpty() == false) { // @note already anti-aliaed if needed
            return VdbRuntimeResources.entrezGeneId2GeneSymbol_one_or_null(ezids, false);
            /*
            if (symbol != null) {
                return new SimpleProbe2(accName, symbol, VdbRuntimeResources.getChip_Gene_Symbol());
            }
            */
        }

        System.out.println(">> " + ezids + " >" + symbol + "<" + " >" + sourcepath + "<");

        return null;
    }

    /*
    // I could have used the eutils esearch etc but i couldnt get them to return a gene symbol for a accession query
    // // http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?cmd=search&db=gene&term=D26157[accn]
    public Probe parseProbe(String accName) throws Exception {

        super.setSilentMode(true);
        String sourcepath = "http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?cmd=search&db=gene&term=" + accName + "[accn]";
        InputStream is = new URL(sourcepath).openConnection().getInputStream();
        startImport(sourcepath);

        final BufferedReader bin = new BufferedReader(new InputStreamReader(is));
        String currLine = nextLine(bin, false);

        String symbol = null;

        while (currLine != null) {

            //System.out.println(">>" + currLine);

            if (currLine.indexOf(PREF) != -1) {
                symbol = parseGeneSymbol(currLine);
                break;
            }
            currLine = nextLine(bin);
        }

        bin.close();
        doneImport();

        if (symbol != null) {
            return new SimpleProbe2(accName, symbol, VdbRuntimeResources.getChip_Gene_Symbol());
        }

        System.out.println(">> " + symbol + " >" + sourcepath + "<");

        return null;

    } // End parse()



    //   Official Symbol: Ptgir and Name: prostaglandin I receptor (IP) [Mus musculus]
    private static final String PREF = "<dd><b>Official Symbol:</b>";

    private static String parseGeneSymbol(String line) throws Exception {

        int index = line.indexOf(PREF) + PREF.length();
        String sub = line.substring(index);

        String[] ss = ParseUtils.string2strings(sub, " ", false);
        String symbol = ss[0].toUpperCase();

        if (VdbRuntimeResources.getAliasDb().isAlias(symbol)) {
            symbol = VdbRuntimeResources.getAliasDb().getSymbol(symbol);
        }

        System.out.println("line: " + line + ">"   + symbol);

        return symbol;
    }
    */

}    // End EntrezGeneWebPageParser
