/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.parsers;


import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.objects.GeneSetMatrix;
import edu.mit.broad.genome.objects.GenesOfInterest;
import edu.mit.broad.genome.objects.GenesOfInterestImpl;
import edu.mit.broad.genome.objects.PersistentObject;

import java.awt.*;
import java.io.File;
import java.io.InputStream;
import java.util.List;


/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 */

public class GinParser_xls extends AbstractParser {

    /**
     * Class Constructor.
     */
    public GinParser_xls() {
        super(GeneSetMatrix.class);
    }

    /**
     * Only accepts GenesOfInterest
     */
    public void export(PersistentObject gmpob, File file) throws Exception {
        throw new NotImplementedException();
    }


    // No header line
    // Stored as an excel file with every worksheet a gene list
    public List parse(String sourcepath, InputStream is) throws Exception {

        startImport(sourcepath);

        File file = new File(sourcepath);

        MiscParsers.ExcelLinesColorStruc[] strucs = MiscParsers.excel2lines_and_colors_all_worksheets(file);

        final GenesOfInterest.GeneList[] geneLists = new GenesOfInterest.GeneList[strucs.length];

        for (int c = 0; c < strucs.length; c++) {
            List lines = strucs[c].lines;
            Object colorObj = null;
            GenesOfInterest.Gene[] genes = new GenesOfInterest.Gene[lines.size() - 1]; // @note ignore the first line
            for (int i = 1; i < lines.size(); i++) {
                String line = lines.get(i).toString();
                String[] fields = ParseUtils.string2strings(line, "\t", false);
                String geneName = fields[0];

                if (colorObj == null) {
                    colorObj = strucs[c].valueColorMap.get(geneName); // All genes in this ws get the same color
                }

                StringBuffer buf = new StringBuffer();
                for (int f = 1; f < fields.length; f++) {
                    buf.append(fields[f]).append('\t');
                }
                String notes = null;
                if (buf.length() > 0) {
                    notes = buf.toString();
                }
                genes[i - 1] = new GenesOfInterestImpl.GeneImpl(geneName, strucs[c].sheetName, notes);
            }

            Color color = null;
            if (colorObj != null) {
                color = (Color) colorObj;
            }

            geneLists[c] = new GenesOfInterestImpl.GeneListImpl(file.getName(), strucs[c].sheetName, color, genes);
        }

        GenesOfInterestImpl gin = new GenesOfInterestImpl(file.getName(), geneLists);

        doneImport();

        return unmodlist(gin);

    } // End parse()


}    // End GenesOfInterestParser

/*& OLD WAY
// No header line
public List parse(String sourcepath, InputStream is) throws Exception {

    startImport(sourcepath);

    BufferedReader bin = new BufferedReader(new InputStreamReader(is));
    String currLine = nextLine(bin);

    int row = 0;
    Map geneNameSetMap = new HashMap();

    while (currLine != null) {
        String[] fields = ParseUtils.string2strings(currLine, "\t", false);

        if (fields.length != 2) {
            throw new ParserException("Invalid gene line: " + currLine + " at row: " + row + " expecting 2 columns");
        }

        Object prev = geneNameSetMap.get(fields[0]);
        if (prev == null) {
            prev = new HashSet();
        }

        ((Set) prev).add(fields[1]);
        geneNameSetMap.put(fields[0], prev);

        row++;
        currLine = nextLine(bin);
    }

    bin.close();

    GenesOfInterestImpl.Gene[] recds = new GenesOfInterestImpl.Gene[geneNameSetMap.size()];
    int cnt = 0;
    for (Iterator it = geneNameSetMap.keySet().iterator(); it.hasNext();) {
        String geneName = it.next().toString();
        Set set = (Set) geneNameSetMap.get(geneName);
        recds[cnt++] = new GenesOfInterestImpl.Gene(geneName, set);
    }

    doneImport();

    return unmodlist(new GenesOfInterestImpl(sourcepath, recds));

} // End parse()

*/