/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.parsers;


import edu.mit.broad.genome.Headers;
import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.Version;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.genome.objects.StringDataframe;
import edu.mit.broad.vdb.meg.DefaultGene;
import edu.mit.broad.vdb.meg.Gene;
import edu.mit.broad.vdb.meg.GeneDb;

import java.io.File;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 *
 */


public class HugoDbParser extends AbstractParser {


    /**
     * Class Constructor.
     */
    public HugoDbParser() {
        super(GeneDb.class);
    }

    /**
     * Export a Dataset to file in gct format
     * Only works with Datasets
     *
     * @see "Below for format"
     */
    public void export(final PersistentObject pob, final File file) throws Exception {
        throw new NotImplementedException();

    }    // End export

    public List parse(String sourcepath, InputStream is) throws Exception {
        return unmodlist(parseHugoDb(new File(sourcepath)));
    }

    public GeneDb parseHugoDb(final File file) throws Exception {

        final StringDataframe sdfHugo = ParserFactory.readStringDataframe(file, false);
        final List hugos = new ArrayList();

        for (int r = 0; r < sdfHugo.getNumRow(); r++) {
            String symbol = sdfHugo.getElement(r, Headers.Hugo.APPROVED_SYMBOL);
            String title = sdfHugo.getElement(r, Headers.Hugo.APPROVED_NAME);
            String status = sdfHugo.getElement(r, Headers.Hugo.STATUS);
            String cyto = sdfHugo.getElement(r, Headers.Hugo.CHROMOSOME);

            if (status.equalsIgnoreCase("Approved")) {
                Set aliases = ParseUtils.string2stringsSet(sdfHugo.getElement(r, Headers.Hugo.ALIASES), ",", false);
                Set accs = ParseUtils.string2stringsSet(sdfHugo.getElement(r, Headers.Hugo.ACCESSION_NUMBERS), ",", false);
                Set refSeqs = ParseUtils.string2stringsSet(sdfHugo.getElement(r, Headers.Hugo.REFSEQ_IDS), ",", false);
                Set all = new HashSet();
                all.addAll(accs);
                all.addAll(refSeqs);
                hugos.add(new DefaultGene(symbol, title, aliases, cyto, all));
            }
        }

        return new GeneDb(file, new Version(1), (Gene[]) hugos.toArray(new Gene[hugos.size()]));
    }


}    // End of class HugoDbParser

