/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.parsers;


import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.Printf;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.math.StringMatrix;
import edu.mit.broad.genome.math.Vector;
import edu.mit.broad.genome.objects.*;
import edu.mit.broad.genome.objects.strucs.TxtAffy;
import gnu.trove.TFloatArrayList;
import jxl.Cell;
import jxl.Sheet;
import jxl.Workbook;
import jxl.format.CellFormat;
import jxl.format.RGB;
import org.apache.log4j.Logger;

import java.awt.*;
import java.io.*;
import java.util.*;
import java.util.List;


public class MiscParsers {

    private static final Logger klog = XLogger.getLogger(MiscParsers.class);

    /**
     * Class Constructor.
     */
    private MiscParsers() {
    }

    public static void gct2arff(Dataset ds, File saveInFile) {


    }

    public static class ExcelSdfColorStruc {
        public String sheetName;
        public StringDataframe sdf;
        public Map stateColorMap;
    }

    public static class ExcelLinesColorStruc {
        public String sheetName;
        public List lines;
        public Map valueColorMap;
    }

    public static ExcelSdfColorStruc excel2sdf_and_colors(File xlsFile) throws Exception {
        ExcelSdfColorStruc es = new ExcelSdfColorStruc();
        es.sdf = excel2sdf(xlsFile); // only 1 ws expected and supported
        es.stateColorMap = parseColorMapFromExcel(xlsFile);
        return es;
    }

    /*
    public static ExcelSdfColorStruc excel2sdf_and_colors(String path, InputStream is) throws Exception {
        ExcelSdfColorStruc es = new ExcelSdfColorStruc();
        es.sdf = excel2sdf(path, is, 0); // only 1 ws expected and supported
        es.stateColorMap = parseColorMapFromExcel(path, is);
        return es;
    }
    */

    /* problem is that you cannot is twice the same file!!
    public static ExcelSdfColorStruc excel2sdf_and_colors(String sourcePath, InputStream is) throws Exception {
        ExcelSdfColorStruc es = new ExcelSdfColorStruc();
        es.sdf = excel2sdf(sourcePath, is, 0); // only 1 ws expected and supported
        es.stateColorMap = parseColorMapFromExcel(sourcePath, is);
        return es;
    }
    */

    public static ExcelSdfColorStruc[] excel2sdf_and_colors_all_worksheets(File xlsFile) throws Exception {
        InputStream is = new FileInputStream(xlsFile);
        final Workbook wk = Workbook.getWorkbook(is);
        ExcelSdfColorStruc[] strucs = new ExcelSdfColorStruc[wk.getNumberOfSheets()];
        for (int i = 0; i < wk.getNumberOfSheets(); i++) {
            Sheet sheet = wk.getSheet(i);
            strucs[i] = new ExcelSdfColorStruc();
            strucs[i].sdf = excel2sdf(xlsFile.getPath(), sheet);
            strucs[i].stateColorMap = parseColorMapFromExcel(xlsFile.getPath(), sheet);
            strucs[i].sheetName = sheet.getName();
        }

        return strucs;
    }

    public static ExcelLinesColorStruc[] excel2lines_and_colors_all_worksheets(File xlsFile) throws Exception {
        InputStream is = new FileInputStream(xlsFile);
        final Workbook wk = Workbook.getWorkbook(is);
        ExcelLinesColorStruc[] strucs = new ExcelLinesColorStruc[wk.getNumberOfSheets()];
        for (int i = 0; i < wk.getNumberOfSheets(); i++) {
            Sheet sheet = wk.getSheet(i);
            strucs[i] = new ExcelLinesColorStruc();
            strucs[i].lines = excel2LinesList(xlsFile.getPath(), sheet);
            strucs[i].valueColorMap = parseColorMapFromExcel(xlsFile.getPath(), sheet);
            strucs[i].sheetName = sheet.getName();
        }

        return strucs;
    }


    public static StringDataframe excel2sdf(File xlsFile) throws Exception {
        return excel2sdf(xlsFile.getPath(), new FileInputStream(xlsFile), 0); // only 1 ws supported
    }

    // Only 1 worksheet supported
    public static StringDataframe excel2sdf(final String sourcepath, InputStream is, int wsIndex) throws Exception {
        final Workbook wk = Workbook.getWorkbook(is);
        return excel2sdf(sourcepath, wk.getSheet(wsIndex));
    }

    public static StringDataframe excel2sdf(final String sourcepath, Sheet sheet) throws Exception {
        klog.info("Parsing excel from: " + sourcepath);

        int nrows = sheet.getRows();
        int ncols = sheet.getColumns();

        StringMatrix sm = new StringMatrix(nrows - 1, ncols - 1);

        // first row has colNames
        String[] colNames = new String[ncols - 1];
        String[] rowNames = new String[nrows - 1];

        Set all_rowNames = new HashSet();

        for (int r = 1; r < nrows; r++) { // Omit the first row as it has col headers
            String rn = _txt(r, 0, sheet, true);
            rowNames[r - 1] = rn;

            if (all_rowNames.contains(rn)) {
                throw new ParserException("Duplicate row_name in excel ss: >" + rn + "<" + "\n" + all_rowNames);
            }

            all_rowNames.add(rn);
        }

        for (int c = 1; c < sheet.getColumns(); c++) {
            for (int r = 0; r < sheet.getRows(); r++) {
                if (r == 0) { // first row are col headers
                    colNames[c - 1] = _txt(r, c, sheet, true);
                } else {
                    sm.setElement(r - 1, c - 1, _txt(r, c, sheet));
                }

            }
        }

        return new StringDataframe(new File(sourcepath).getName(), sm, rowNames, colNames, true);
    }

    public static List excel2LinesList(final String sourcepath, final Sheet sheet) throws Exception {
        //klog.info("Parsing excel from TTT: " + sourcepath);

        int nrows = sheet.getRows();

        List lines = new ArrayList();

        for (int r = 0; r < nrows; r++) {
            String txt = _txt_full_row(r, sheet);
            if (txt != null && txt.length() > 0) {
                lines.add(txt);
            }
        }

        return lines;
    }

    private static String _txt(int r, int c, Sheet sheet) throws ParserException {
        return _txt(r, c, sheet, false);
    }

    private static String _txt(int r, int c, Sheet sheet, boolean reqd) throws ParserException {
        final Cell cell = sheet.getCell(c, r); // @note
        String txt = cell.getContents();
        if (txt != null) {
            txt = txt.trim();
        }

        if (reqd && (txt == null || txt.length() == 0)) {
            throw new ParserException("Null or empty for r: " + r + " c: " + c);
        }

        return txt;
    }

    // a single line tab delim
    // no restriction on squareness etc -- each lkine can have a diff number of fields
    private static String _txt_full_row(int r, Sheet sheet) throws ParserException {

        StringBuffer buf = new StringBuffer();
        for (int c = 0; c < sheet.getColumns(); c++) {
            String txt = _txt(r, c, sheet, false);
            if (txt != null && txt.length() > 0) {
                buf.append(txt).append('\t');
            }
        }

        return buf.toString().trim();
    }

    public static Dataset getDatasetFromTxtFiles(final String name,
                                                 final File[] txtFiles,
                                                 final boolean signal) throws Exception {

        /* too memory intensive
        final TxtAffy[] tas = new TxtAffy[txtFiles.length];

        for (int i = 0; i < txtFiles.length; i++) {
            Parser parser = new TxtAffyParser();
            tas[i] = (TxtAffy) parser.parse(txtFiles[i].getName(), txtFiles[i]).get(0);
        }

        return TxtAffy.createDataset(name, tas);
        */

        Parser parserx = new TxtAffyParser();
        final TxtAffy ref = (TxtAffy) parserx.parse(txtFiles[0].getName(), txtFiles[0]).get(0);

        final int numRows = ref.getNumValues();
        final Matrix m = new Matrix(numRows, txtFiles.length);
        final APMMatrix apm = new APMMatrix(numRows, txtFiles.length);
        final List colNames = new ArrayList();
        final List rowNames = new ArrayList();
        for (int c = 0; c < txtFiles.length; c++) {
            Parser parser = new TxtAffyParser();
            TxtAffy ta = (TxtAffy) parser.parse(txtFiles[c].getName(), txtFiles[c]).get(0);
            colNames.add(ta.getName());
            for (int r = 0; r < numRows; r++) {
                final String probeName = ref.getValue(r).fFeatureName;
                final TxtAffy.Value value = ta.getValue(probeName);
                if (c == 0) {
                    rowNames.add(probeName);
                }

                float v;
                if (signal) {
                    v = value.fSignal;
                } else {
                    v = value.fDetectionPValue;
                }
                final String call = Character.toString(value.fCall);
                //System.out.println(v + " " + call);
                m.setElement(r, c, v);
                apm.setElement(r, c, call);
            }
        }

        return new DefaultDataset(name, m, rowNames, colNames, true, null, apm);
    }

    public static Dataset getDatasetFromTxtFiles(final String name,
                                                 final File[] txtFiles,
                                                 final boolean signal,
                                                 final GeneSet probeSetNames,
                                                 final boolean barfOnMissingValues) throws Exception {


        final Matrix m = new Matrix(probeSetNames.getNumMembers(), txtFiles.length);
        final APMMatrix apm = new APMMatrix(probeSetNames.getNumMembers(), txtFiles.length);
        final List colNames = new ArrayList();

        for (int c = 0; c < txtFiles.length; c++) {
            Parser parser = new TxtAffyParser();
            TxtAffy ta = (TxtAffy) parser.parse(txtFiles[c].getName(), txtFiles[c]).get(0);
            colNames.add(ta.getName());
            for (int r = 0; r < probeSetNames.getNumMembers(); r++) {
                final String probeName = probeSetNames.getMember(r);

                final TxtAffy.Value value = ta.getValue(probeName, barfOnMissingValues);

                float v;
                String call;

                if (value == null) {
                    v = 0;
                    call = "A";
                } else {
                    if (signal) {
                        v = value.fSignal;
                    } else {
                        v = value.fDetectionPValue;
                    }
                    call = Character.toString(value.fCall);
                }

                //System.out.println(v + " " + call);
                m.setElement(r, c, v);
                apm.setElement(r, c, call);
            }

            System.out.println("Done " + (c + 1) + " / " + txtFiles.length);
        }

        return new DefaultDataset(name, m, probeSetNames.getMembers(), colNames, true, null, apm);
    }

    /*
    public static Dataset getDatasetFromTxtFiles_signal(final String name, final File[] txtFiles) throws Exception {


        Parser parserx = new TxtAffyParser();
        final TxtAffy ref = (TxtAffy) parserx.parse(txtFiles[0].getName(), txtFiles[0]).get(0);

        final int numRows = ref.getNumStrucs();
        final Matrix m = new Matrix(numRows, txtFiles.length);
        final APMMatrix apm = new APMMatrix(numRows, txtFiles.length);
        final List colNames = new ArrayList();
        final List rowNames = new ArrayList();
        for (int c = 0; c < txtFiles.length; c++) {
            Parser parser = new TxtAffyParser();
            TxtAffy ta = (TxtAffy) parser.parse(txtFiles[c].getName(), txtFiles[c]).get(0);
            colNames.add(ta.getName());
            for (int r = 0; r < numRows; r++) {
                final String probeName = ref.getStruc(r).fFeatureName;
                final TxtAffy.Struc struc = ta.getStruc(probeName);
                if (c == 0) {
                    rowNames.add(probeName);
                }

                final float v = struc.fValue;
                final String signal = struc.fSignal;
                //System.out.println(v + " " + call);
                m.setElement(r, c, Float.parseFloat(signal));
                apm.setElement(r, c, signal);
            }
        }

        return new DefaultDataset(name + "_signal", m, rowNames, colNames, true, null, null);
    }
    */

    public static Map parseColorMapFromExcel(final File xlsFile) throws Exception {
        final Workbook wk = Workbook.getWorkbook(xlsFile);
        final Sheet sheet = wk.getSheet(0);
        return parseColorMapFromExcel(xlsFile.getPath(), sheet);
    }

    /**
     * key -> string, Value -> BACKGROUND color
     * Duplicates are overwritten
     */
    /* THE JXL API FOR FROM IS SEEMS TO BE BROKEN
    public static Map parseColorMapFromExcel(final String sourcepath, final InputStream is) throws Exception {
        final Workbook wk = Workbook.getWorkbook(is);
        final Sheet sheet = wk.getSheet(0);
        return parseColorMapFromExcel(sourcepath, sheet);
    }
    */
    public static Map parseColorMapFromExcel(final String sourcepath, final Sheet sheet) throws Exception {

        //klog.info("Parsing color map from excel: " + sourcepath);

        final Map valueColorMap = new HashMap();

        for (int c = 0; c < sheet.getColumns(); c++) {
            for (int r = 0; r < sheet.getRows(); r++) {
                String txt = _txt(r, c, sheet, false); // nulls are ok
                if (txt != null && txt.length() > 0) {
                    final Cell cell = sheet.getCell(c, r); // @note
                    final CellFormat format = cell.getCellFormat();
                    final jxl.format.Colour color = format.getBackgroundColour();
                    if (color != null) {
                        final RGB rgb = color.getDefaultRGB();
                        final Color cl = new Color(rgb.getRed(), rgb.getGreen(), rgb.getBlue());
                        if (cl != Color.WHITE) {
                            //System.out.println(txt + "\t" + cl + " " + r + ":" + c);
                            if (valueColorMap.containsKey(txt)) {
                                // fine easier to just paste them all in
                                //klog.warn("Duplicate colormap entry for: " + txt);
                            } else {
                                valueColorMap.put(txt, cl);
                            }
                        }
                    }
                }
            }
        }

        return valueColorMap;
    }

    //>YTAAYNGCT	138
    //PDE6D	PLAGL2	GATA3	BNC2	HSD11B1	FGF17	PPP2R2B	ASB7	EGR2	H2AFV	PRKCBP1	PLA2R1	ACCN3	FLRT3	INA	RIPX	ERO1LB	CCND2	ATBF1	C1ORF39	TNFSF10	ZFP36L1	KRTHB6	PCBP2	DIO2	POFUT1	FLJ30046	CITED1	CCKAR	NR6A1	SDCCAG33	DCTN3	EPSTI1	KIAA0261	CSMD3	PRDM13	DCX	RAP2B	MAPK8	SLC26A7	TRIM33	WINS1	KIBRA	ZCWPW1	PHYHIP	DKFZP761I1011	DKFZP564O123	DOCK11	HOXB7	H3F3B	ZFPM2	MEIS2	MGC33530	LOC56901	RHBDL4	ZIC4	HOXC6	HHEX	SUI1	ODD	OTX2	HOXB8	CDH13	LOXHD1	NUDT4	TYRO3	PPARGC1A	PI15	STIM2	CTNND2	ATF3	POU5F1	LMO4	C9ORF67	CLDN3	SYTL2	RPS6KB1	LNX	ONECUT2	LDB2	HNRNPG-T	VLDLR	HNRPC	PBX1	PPP1R16B	DBP	RAB27A	HS747E2A	SRP46	TPI1	FLJ20257	VAMP3	IL1RAPL1	ZNF238	ZNF423	FAP	CD72	RHOC	TAIP-2	NRXN1	NUP54	WNT16	CXCL13	AMPH	PHEX	AMPD3	PEA15	EFHD1	LLGL2	KLHL1	LUC7L2	CREM	PTGFR	CTCF	LOC55831	BCAR3	ZF	TNFSF5	TRIM8	ITGB3BP	KIAA1190	HRK	ASAH1	H2AFZ	KCNIP4	HTR2C	CHRDL1	FLJ30296	ACVR1C	CSPG4	PCBP1	SLC6A6	ZFHX1B	LIMK2	GAD1	FGF7	KLF14	PELI2
    public static GeneSetMatrix parseXhxMotifFile(final File file) throws Exception {

        BufferedReader bin = new BufferedReader(new FileReader(file));
        List rows = ParseUtils.slurpIntoList(bin, false);
        if (rows.size() == 0) {
            throw new ParserException("No data in file -- nothing to parse");
        }

        List gsets = new ArrayList();
        for (int i = 0; i < rows.size(); i++) {
            String line = rows.get(i).toString();
            if (line.startsWith(">")) {
                StringTokenizer tok = new StringTokenizer(line, "\t");
                String gsetName = tok.nextToken();
                String gsline = rows.get(i + 1).toString();
                String[] members = ParseUtils.string2strings(gsline, " \t", false);
                GeneSet gset = new FSet(gsetName, members);
                gsets.add(gset);
                i = i + 1;
            }
        }

        bin.close();

        return new DefaultGeneSetMatrix(file.getName(), gsets);
    }

    // result hack to get around the Vx's in the H matrix output by pablos code
    public static Matrix parseMatrix_hackH(File file) throws Exception {

        BufferedReader bin = new BufferedReader(new FileReader(file));

        List rows = ParseUtils.slurpIntoList(bin, false);
        if (rows.size() == 0) {
            throw new ParserException("No data in file -- nothing to parse");
        }

        // the first row contains the bad vchars
        // example: V3	V4	V5	V6	V7
        String[] fields = ParseUtils.string2strings(rows.get(0).toString(), "\t", false);
        TFloatArrayList floats = new TFloatArrayList();
        for (int i = 0; i < fields.length; i++) {
            if (fields[i].startsWith("V")) {
                ; // do nothing
            } else {
                floats.add(Float.parseFloat(fields[i]));
            }
        }

        float[] firstRow = floats.toNativeArray();

        if (firstRow.length == 0) {
            throw new ParserException("No data in file -- nothing to parse");
        }

        Matrix matrix = new Matrix(rows.size(), firstRow.length);

        matrix.setRow(0, firstRow);

        // save all rows so that we can determine how many rows exist
        // each row is a row of the matrix
        for (int r = 1; r < rows.size(); r++) {
            float[] row = ParseUtils.string2floats(rows.get(r).toString(), " \t");
            if (row.length != firstRow.length) {
                throw new ParserException("Unequal number of elements in row: " + (r + 1) + " found: " + row.length + " but expected: " + firstRow.length);
            }
            matrix.setRow(r, row);
        }

        bin.close();

        matrix.setImmutable(); // for safety
        return matrix;
    }

    // fully numbers - no label stuff
    public static Matrix parseMatrix(File file, boolean allowMissingValues) throws Exception {
        return parseMatrix(new BufferedReader(new FileReader(file)), allowMissingValues);
    }

    // @todo temp hack for ParserFactory.read
    public static Matrix parseMatrix(String path, InputStream is, boolean allowMissingValues) throws Exception {
        is.close();
        return parseMatrix(new File(path), allowMissingValues);
    }

    public static Matrix parseMatrix(BufferedReader bin, boolean allowMissingValues) throws Exception {

        List rows = ParseUtils.slurpIntoList(bin, false);
        if (rows.size() == 0) {
            throw new ParserException("No data in file -- nothing to parse");
        }

        // @todo
        if (allowMissingValues) {
            throw new NotImplementedException();
        }

        float[] firstRow = ParseUtils.string2floats(rows.get(0).toString(), " \t");

        if (firstRow.length == 0) {
            throw new ParserException("No data in file -- nothing to parse");
        }

        Matrix matrix = new Matrix(rows.size(), firstRow.length);

        // save all rows so that we can determine how many rows exist
        // each row is a row of the matrix
        for (int r = 0; r < rows.size(); r++) {
            float[] row = ParseUtils.string2floats(rows.get(r).toString(), " \t");
            if (row.length != firstRow.length) {
                throw new ParserException("Unequal number of elements in row: " + (r + 1) + " found: " + row.length + " but expected: " + firstRow.length);
            }
            matrix.setRow(r, row);
        }

        bin.close();

        matrix.setImmutable(); // for safety
        return matrix;
    }

    public static void save(Matrix m, File toFile) throws Exception {
        PrintWriter pw = new PrintWriter(new FileOutputStream(toFile));

        for (int r = 0; r < m.getNumRow(); r++) {
            pw.println(m.getRowV(r).toString('\t'));
        }

        pw.close();
    }

    public static void save(Vector v, File toFile) throws Exception {
        PrintWriter pw = new PrintWriter(new FileOutputStream(toFile));
        pw.println(v.toString('\t'));
        pw.close();
    }

    /**
     * Export a Dataset to file in svm fu format
     * <p/>
     * Exports to a svm fu dataset
     * <p/>
     * Format:
     * <p/>
     * 10 7129
     * 55 124 255 223 777 1
     * 363 999 333 ...    -1
     * ...                1
     * <p/>
     * 10 rows each row has 7129 + 1 elements.
     * The last element is the template class -1 or 1
     */
    public static void exportSvmFu(Dataset ds, Template t, File file) throws Exception {

        if (ds.getNumCol() != t.getNumItems()) {
            throw new IllegalArgumentException("Mismatched da: " + ds.getNumCol() + " and template: " + t.getNumItems());
        }

        if (t.getNumClasses() != 2) {
            throw new IllegalArgumentException("Only 2 class templates allowed");
        }

        PrintWriter pw = new PrintWriter(new FileOutputStream(file));

        pw.println(ds.getNumCol() + " " + ds.getNumRow());

        String class1Name = t.getClass(1).getName();

        String cid;

        for (int c = 0; c < ds.getNumCol(); c++) {
            Vector v = ds.getColumn(c);
            String cn = t.getClassByProfilePos(c).getName();
            if (cn.equals(class1Name)) {
                cid = "1";
            } else {
                cid = "-1";
            }

            String s = Printf.format(v, ' ');
            s = s.trim();
            s += ' ' + cid;
            pw.println(s);
        }

        pw.close();

    }    // End export


}    // End of class MiscParsers

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
