/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.parsers;


import edu.mit.broad.genome.TraceUtils;
import edu.mit.broad.genome.utils.ChainedException;
import org.apache.ecs.Element;
import org.apache.ecs.html.TD;
import org.apache.ecs.html.TR;
import org.apache.ecs.html.Table;


/**
 * An exception related to parsing data.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ParserException extends ChainedException {

    /**
     * Create an exception with a detail message.
     *
     * @param msg the message
     */
    public ParserException(final String msg) {
        super(msg);
    }

    /**
     * Create a chained exception.
     *
     * @param t the nested esception.
     */
    public ParserException(final Throwable t) {
        super(t);
    }

    /**
     * Create a chained exception along with a message
     *
     * @param msg the message
     * @param t   the nested esception.
     */
    public ParserException(final String msg, final Throwable t) {
        super(msg, t);
    }

    public Element createHtmlErrorMessage() {

        Table table = new Table();
        table.setBorder(2);

        TR tr = new TR();
        tr.addElement(new TD("Message"));
        tr.addElement(new TD(super.getMessage()));
        table.addElement(tr);

        tr = new TR();
        tr.addElement(new TD("Action"));
        tr.addElement(new TD("There was a problem parsing the input data file(s). " +
                "<br>Please check their data formats and try again. <br>" +
                "Details on data format are on the website"));
        table.addElement(tr);

        tr = new TR();
        tr.addElement(new TD("Details"));
        tr.addElement(new TD(TraceUtils.getAsString(this)));
        table.addElement(tr);

        return table;
    }


}    // End ParserException

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
