/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.parsers;

import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;

import java.util.List;

/**
 * Utilities to share amongs the stanford data format parsers.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */

public class ParserFormatsHelper {

    private static final Logger klog = XLogger.getLogger(ParserFormatsHelper.class);

    /**
     * Privatized class constructor
     */
    private ParserFormatsHelper() {
    }

    /**
     * separators can be tab or space(s)
     * <p/>
     * Extar or blank tabs are ignored.
     * extra whitespace is ignored
     * Format:
     * <p/>
     * GID	YORF	NAME	GWEIGHT	spo0	spo7	spo5	spo11	spo9	spo2	spo30
     *
     * @return a list with (for example) spo0, spo7 etc
     */
    public static List parseGidColumnLine(String line) {
        line = line.trim();

        // using space as a delim messes up when there is a header of the sort:
        // Pancreas_Normal__Pan_14N:CH2000031005AA/scale factor=1
        //List colnames = ParseUtils.split(line, " \t", true);
        List colnames = ParseUtils.string2stringsList(line, "\t");
        int size = colnames.size();
        for (int i = 0; i < size; i++) {
            if (colnames.get(i).toString().trim().length() == 0) colnames.remove(i);

        }

        colnames.remove(0); // get rid of GID
        colnames.remove(0); // get rid of YORF (or whatever)
        colnames.remove(0); // get rid of NAME
        colnames.remove(0); // get rid of GWEIGHT
        klog.debug("Number of columns = " + colnames.size());
        return colnames;
    }

    /**
     * separators can be tab or space(s)
     * <p/>
     * Extar or blank tabs are ignored.
     * extra whitespace is ignored
     * Format:
     * <p/>
     * YORF	NAME	GWEIGHT	spo0	spo7	spo5	spo11	spo9	spo2	spo30
     *
     * @return a list with (for example) spo0, spo7 etc
     */
    public static List parseHClustColumnLine(String line) {
        line = line.trim();
        List colnames = ParseUtils.string2stringsList(line, " \t");
        int size = colnames.size();
        for (int i = 0; i < size; i++) {
            if (colnames.get(i).toString().trim().length() == 0) colnames.remove(i);
        }

        colnames.remove(0); // get rid of YORF (or whatever)
        colnames.remove(0); // get rid of NAME
        colnames.remove(0); // get rid of GWEIGHT
        klog.debug("Number of columns = " + colnames.size());
        return colnames;
    }

    /**
     * @see parseColumnNames for more on seperators etc.
     *      <p/>
     *      Format:
     *      AID				ARRY0X	ARRY4X	ARRY3X	ARRY6X	ARRY5X	ARRY2X	ARRY1X    <---- use for arrayx names
     */
    public static List parseAidLine(String line) {
        line = line.trim();
        List arrnames = ParseUtils.string2stringsList(line, " \t");
        int size = arrnames.size();
        for (int i = 0; i < size; i++) {
            if (arrnames.get(i).toString().trim().length() == 0) arrnames.remove(i);
        }

        arrnames.remove(0); // get rid of AID
        klog.debug("Number of columns = " + arrnames.size());
        return arrnames;
    }

} // End ParserFormatsHelper

