package edu.mit.broad.genome.parsers;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.math.Matrix;
import edu.mit.broad.genome.objects.Dataset;
import edu.mit.broad.genome.objects.DefaultDataset;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.genome.utils.FileUtils;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

/**
 * Simple dataset format with no res, gct cdt etc frills. Description field is optional.
 * TXT Dataset format -> no obvious extension
 * (txt is used for a lot of different things)
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */

public class RMADatasetParser extends AbstractParser {

    /**
     * Class Constructor.
     */
    public RMADatasetParser() {
        super(Dataset.class);
    }

    /**
     * Only accepts Dataset objects.
     * see below for format produced
     */
    public void export(PersistentObject pob, File file) throws Exception {
        throw new NotImplementedException();

    }

    /**
     * cel_filea    cel_fileb ... (NOTE: the column headers state in the first place -- shift them by one!!)
     * ps1  0.23	-1.79	-1.29	-1.56	-2.12	-1.09	-1.12
     * ps2  0.41	-0.38	-0.89	-1.06	-1.6	-1.84	-1.6
     * ps3  0.61	-0.07	-1.29	-1.29	-2	    -1.84	-2.25
     */
    public List parse(final String hackINeedFullPath, final InputStream is) throws Exception {

        startImport(hackINeedFullPath);
        //int nlines = DataStoreUtils.countLines(is, true); // @note this form doesnt work  / not impl yet
        int nlines = FileUtils.countLines(hackINeedFullPath, true);
        int nfloatlines = nlines - 1;
        log.debug("Number of float lines = " + nfloatlines);

        BufferedReader bin = new BufferedReader(new InputStreamReader(is));
        String currLine = nextLine(bin);

        List colNames = ParseUtils.string2stringsList(currLine, " \t");
        int expectedNumFloatCols = colNames.size();

        log.debug("Number of columns = " + colNames.size());

        // Initialize the Dataset
        List rowNames = new ArrayList(nfloatlines);
        Matrix matrix = new Matrix(nfloatlines, colNames.size());

        int r = 0;
        currLine = nextLine(bin); // first line of float data
        while (currLine != null) {
            //System.out.println(">> " + currLine);

            List fields = ParseUtils.string2stringsList(currLine, "\t");
            if (fields.size() != expectedNumFloatCols + 1) {
                throw new ParserException("Invalid format on line: " + currLine + " expected # fields = " + expectedNumFloatCols + " but found: " + fields.size());
            }

            String featureName = (String) fields.get(0);
            rowNames.add(featureName.trim());

            // -- then, onto the floats
            List floats = fields.subList(1, fields.size());
            if (floats.size() != colNames.size()) {
                throw new ParserException("# of floats: " + floats.size() + " on line: " + r + " not equal to # of columns in header: " + colNames.size());
            }

            for (int c = 0; c < floats.size(); c++) {
                matrix.setElement(r, c, Float.parseFloat((String) floats.get(c)));
            }

            r++;
            currLine = nextLine(bin);
        }

        bin.close();

        // Initialize the Dataset and Annotation
        String name = new File(hackINeedFullPath).getName();
        //ann.addComment(fComment.toString());

        Dataset ds = new DefaultDataset(name, matrix, rowNames, colNames, true, null);
        ds.addComment(fComment.toString());

        doneImport();
        return unmodlist(new PersistentObject[]{ds});
    }

}    // End RMAOutputDatasetParser
