/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.parsers;


import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.genome.objects.StringDataframe;
import edu.mit.broad.genome.objects.TemplateCohort;

import java.awt.*;
import java.io.File;
import java.io.InputStream;
import java.util.List;


/**
 * Format Supported:
 * <p/>
 * <br><pre>
 * seed_name=foo
 * feature_name_1 corr
 * feature_name_1 corr
 * feature_name_1 corr
 * ...
 * <pre>
 * <br>
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */

public class TemplateCohortParser extends AbstractParser {

    /**
     * Class Constructor.
     */
    public TemplateCohortParser() {
        super(TemplateCohort.class);
    }

    /**
     * Only accepts tcho objects
     */
    public void export(PersistentObject tcoh, File file) throws Exception {
        throw new NotImplementedException();
    }

    /**
     * Parses in NeighSet files.
     */
    public List parse(String sourcepath, InputStream is) throws Exception {

        startImport(sourcepath);

        MiscParsers.ExcelSdfColorStruc exs = MiscParsers.excel2sdf_and_colors(new File(sourcepath));

        StringDataframe sdf = exs.sdf;


        if (sdf.getNumCol() < 5) {
            throw new IllegalArgumentException("Bad format for template cohort file .. expecting at least 4 columns");
        }

        TemplateCohort.Comp[] pairs = new TemplateCohort.Comp[sdf.getNumRow()];
        for (int r = 0; r < sdf.getNumRow(); r++) {

            String name = sdf.getRowName(r);
            Color color = null;
            Object obj = exs.stateColorMap.get(name);
            if (obj != null) {
                color = (Color) obj;
            }

            String categoryName = sdf.getElement(r, 0);

            String a_short_name = sdf.getElement(r, 1);
            String b_short_name = sdf.getElement(r, 3);

            Color a_color = null;
            obj = exs.stateColorMap.get(a_short_name);
            if (obj != null) {
                a_color = (Color) obj;
            }

            Color b_color = null;
            obj = exs.stateColorMap.get(b_short_name);
            if (obj != null) {
                b_color = (Color) obj;
            }

            pairs[r] = new TemplateCohort.Comp(name,
                    color,
                    categoryName,
                    a_short_name,
                    a_color,
                    sdf.getElement(r, 2),
                    b_short_name,
                    b_color,
                    sdf.getElement(r, 4));
        }


        doneImport();

        return unmodlist(new TemplateCohort(sdf.getName(), pairs));
    }    // End parse()

}    // End TemplateCohortParser

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
