/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.parsers;


import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.objects.PersistentObject;
import edu.mit.broad.genome.objects.Transfac;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 *
 */


public class TransfacParser extends AbstractParser {

    /**
     * Class Constructor.
     */
    public TransfacParser() {
        super(Transfac.class);
    }

    /**
     * Export a Dataset to file in gct format
     * Only works with Datasets
     *
     * @see "Above for format"
     */
    public void export(final PersistentObject pob, final File file) throws Exception {
        throw new NotImplementedException();

    }    // End export

    /**
     * @returns 1 Dataset object
     * Always DatasetAnnotation object produced , but if underlying df has none then na is used
     * @see above for format
     */
    public Transfac parseTransfac(String sourcepath, InputStream is) throws Exception {
        startImport(sourcepath);

        final BufferedReader bin = new BufferedReader(new InputStreamReader(is));

        // ignore the first 3 lines
        nextLine(bin);
        nextLine(bin);
        nextLine(bin);

        String currLine = nextLine(bin);
        log.debug("Staring with line: " + currLine);
        List lines = new ArrayList();
        List records = new ArrayList();
        while (currLine != null) {
            if (currLine.startsWith("//") && lines.size() != 0) {
                records.add(makeRecord(lines));
                lines.clear();
            } else {
                lines.add(currLine);
            }
            currLine = nextLine(bin);
        }


        bin.close();

        log.debug("# of Transfac reords: " + records.size() + " " + records.get(0).toString());

        return new Transfac(sourcepath, records);
    }

    public List parse(String sourcepath, InputStream is) throws Exception {
        return unmodlist(parseTransfac(sourcepath, is));
    }

    private static String[] OMIT_LINES = new String[]{"CO", "MX", "SQ", "DR", "BS", "GE", "AC", "ID", "DT"};

    private Transfac.Record makeRecord(List lines) {
        final Set motifs = new HashSet();
        for (int i = 0; i < lines.size(); i++) {
            if (lines.get(i).toString().startsWith("MX")) {
                String motif = _mxLine2Motif(lines.get(i).toString());
                if (motif != null) {
                    motifs.add(motif);
                }
            }
        }

        final List useLines = new ArrayList();
        for (int i = 0; i < lines.size(); i++) {
            String line = lines.get(i).toString();
            boolean omit = false;
            for (int s = 0; s < OMIT_LINES.length; s++) {
                if (line.startsWith(OMIT_LINES[s])) {
                    omit = true;
                    break;
                }
            }

            if (!omit) {
                useLines.add(line);
            }
        }

        return new Transfac.Record(motifs, useLines);
    }

    private static String _mxLine2Motif(String line) {
        int index = line.indexOf("V$");
        if (index != -1) {
            String mt = line.substring(index, line.length());
            mt = mt.trim();
            if (mt.endsWith(".")) {
                mt = mt.substring(0, mt.length() - 1);
            }
            return mt;
        }

        return null;
    }

}    // End of class TransfacParser

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
