/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.reports;

import edu.mit.broad.genome.Constants;

import java.util.Collection;
import java.util.Iterator;

/**
 * @author Aravind Subramanian
 *         Date: Jan 2, 2004
 *         Time: 3:07:16 AM
 */
public class NullSafeStringBuffer {

    private StringBuffer fBuf;

    private boolean fReplaceNullWithNULL;

    /**
     * Class constructor
     *
     * @param replaceNullWithNULL
     */
    public NullSafeStringBuffer(boolean replaceNullWithNULL) {
        fBuf = new StringBuffer();
        this.fReplaceNullWithNULL = replaceNullWithNULL;
    }

    public NullSafeStringBuffer(String s, boolean replaceNullWithNULL) {
        this(replaceNullWithNULL);
        this.append(s);
    }

    /**
     * Class constructor
     *
     * @param coll
     * @param replaceNullWithNULL
     * @param delim
     */
    public NullSafeStringBuffer(Collection coll, boolean replaceNullWithNULL, char delim) {
        fBuf = new StringBuffer();
        this.fReplaceNullWithNULL = replaceNullWithNULL;
        this.append(coll, delim);
    }

    public NullSafeStringBuffer append(String s) {
        if (s != null) {
            fBuf.append(s);
        } else {
            if (fReplaceNullWithNULL) {
                fBuf.append(Constants.NULL);
            }
        }

        return this;
    }

    public int length() {
        return fBuf.length();
    }

    public NullSafeStringBuffer append(char s) {
        fBuf.append(s);
        return this;
    }

    public NullSafeStringBuffer append(int i) {
        fBuf.append(i);
        return this;
    }

    public NullSafeStringBuffer append(Collection set, char delim) {
        if (set != null && set.isEmpty() == false) {
            for (Iterator it = set.iterator(); it.hasNext();) {
                Object key = it.next();
                fBuf.append(key.toString().trim());
                if (it.hasNext()) {
                    fBuf.append(delim);
                }
            }
        } else {
            this.append(null);
        }

        return this;
    }

    public String toString() {
        return fBuf.toString();
    }

    public int hashCode() {
        return fBuf.hashCode();
    }

    public boolean equals(Object obj) {
        return fBuf.equals(obj);
    }

    public NullSafeStringBuffer append(String[] ss, char delim) {

        for (int i = 0; i < ss.length; i++) {
            fBuf.append(ss[i]);
            if (i != ss.length - 1) {
                fBuf.append(delim);
            }
        }

        return this;
    }

} // End class NullSafeStringBuffer
