/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.reports.pages;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.objects.Dataframe;
import edu.mit.broad.genome.objects.IDataframe;
import edu.mit.broad.genome.objects.StringDataframe;
import edu.mit.broad.genome.parsers.DataframeParser;
import edu.mit.broad.genome.parsers.StringDataframeParser;
import edu.mit.broad.genome.reports.RichDataframe;
import edu.mit.broad.genome.reports.api.PicFile;

import java.io.File;
import java.io.OutputStream;

/**
 * A simple txt representation of an 'excel format' (i.e tab delimited)
 */
public class ExcelTxtPage implements Page {
    private String fName;
    private IDataframe fIdf;

    /**
     * Class constructor
     *
     * @param idf
     */
    public ExcelTxtPage(final String name, final IDataframe idf) {
        if (idf == null) {
            throw new IllegalArgumentException("Param idf cannot be null");
        }

        if (idf instanceof RichDataframe) {
            this.fIdf = ((RichDataframe) idf).getDataframe(); // @note hack
        } else {
            this.fIdf = idf;
        }

        this.fName = name;
    }

    public String getName() {
        return fName;
    }

    public String getExt() {
        return "xls";
    }

    public String getDesc() {
        return fIdf.getComment();
    }

    // @maint add more suppor tand cleanup interface later
    public PicFile[] write(OutputStream os, File parentDir) throws Exception {
        if (fIdf instanceof StringDataframe) {
            StringDataframeParser parser = new StringDataframeParser();
            parser.setSilentMode(true);
            parser.export(fIdf, os);
        } else if (fIdf instanceof Dataframe) {
            DataframeParser parser = new DataframeParser();
            parser.setSilentMode(true);
            parser.export(fIdf, os);
        } else {
            throw new NotImplementedException("idf object: " + fIdf);
        }

        return new PicFile[]{};
    }

} // End class ExcelTxtpage
