/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.reports.web;

import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;

/**
 * @author Aravind Subramanian
 *         Date: Sep 17, 2003
 *         Time: 6:15:47 PM
 */
public class WebResources {

    private static final Logger klog = XLogger.getLogger(WebResources.class);

    //final String prefix = "http://genome.ucsc.edu/cgi-bin/hgTracks?position=5q31";

    public static final WebResource GOLDENPATH_CYTO = new DefaultWebResource("GoldenPath",
            "http://genome.ucsc.edu/cgi-bin/hgTracks?position=");

    public static final WebResource NETAFFX = new DefaultWebResource("NetAffx",
            "https://www.affymetrix.com/LinkServlet?probeset=");

    public static final WebResource STANFORD_SOURCE_GENE = new DefaultWebResource("Source",
            "http://genome-www5.stanford.edu/cgi-bin/SMD/source/sourceResult?option=Name&choice=Gene&organism=Hs&criteria=");

    public static final WebResource ENTREZ_GENE_SYMBOL = new DefaultWebResource("Entrez",
            "http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?cmd=search&db=gene&term=");
    // http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?cmd=search&db=gene&term=BRCA1[sym]

    // http://source.stanford.edu/cgi-bin/source/sourceResult?option=CloneID&choice=Gene&criteria=IMAGE:1208978
    // http://source.stanford.edu/cgi-bin/source/sourceResult?option=CloneID&choice=cDNA&criteria=IMAGE:1208978
    public static final WebResource STANFORD_SOURCE_CLONE = new DefaultWebResource("Source",
            "http://source.stanford.edu/cgi-bin/source/sourceResult?option=CloneID&choice=cDNA&criteria=");

    //@todo
    public static final WebResource GOLDEN_PATH = new DefaultWebResource("GoldenPath",
            "https://www.affymetrix.com/LinkServlet?probeset=");

    public static final WebResource GEO_ID = new DefaultWebResource("GEO",
            "http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=");


    static class DefaultWebResource implements WebResource {
        private String fName;
        private String fPrefix;

        DefaultWebResource(String name, String prefix) {
            this.fName = name;
            this.fPrefix = prefix;
        }

        public String getName() {
            return fName;
        }

        // always of the form: prefix=
        public String getUrlPrefix() {
            return fPrefix;
        }

    }

} // End WebResources
