/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.swing;


import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;

/**
 * This class provides a panel for laying out labelled elements neatly with
 * all the labels and elements aligned down the screen.
 *
 * @author David Fraser
 * @author Michael Harris
 * @author Aravind Subramanian (xomics additions)
 * @version %I%, %G%
 * @see http://www.javaworld.com/javaworld/jw-10-2002/jw-1004-dialog.html?
 */

public class GLabelledItemPanel extends JPanel {

    /**
     * The row to add the next labelled item to
     */
    private int myNextItemRow = 0;

    /**
     * label alignemnt
     */
    private int fLabelAlignment = GridBagConstraints.NORTHWEST;

    private java.util.List labels;

    /**
     * This method is the default constructor.
     */
    public GLabelledItemPanel() {
        init();
    }

    /**
     * This method initialises the panel and layout manager.
     */
    private void init() {

        setLayout(new GridBagLayout());
        labels = new ArrayList();

        // Create a blank label to use as a vertical fill so that the
        // label/item pairs are aligned to the top of the panel and are not
        // grouped in the centre if the parent component is taller than
        // the preferred size of the panel.
        GridBagConstraints constraints = new GridBagConstraints();

        constraints.gridx = 0;
        constraints.gridy = 99;
        constraints.insets = new Insets(10, 0, 0, 0);
        constraints.weighty = 1.0;
        constraints.fill = GridBagConstraints.VERTICAL;

        JLabel verticalFillLabel = new JLabel();
        add(verticalFillLabel, constraints);
    }

    /**
     * One of GridBagConstraints.NORTHWEST or GridBagConstraints.NORTHEAST
     *
     * @param align
     */

    // as addition
    public void setLabelAlignment(int align) {
        this.fLabelAlignment = align;
    }

    /**
     * This method adds a labelled item to the panel. The item is added to
     * the row below the last item added.
     *
     * @param labelText The label text for the item.
     * @param item      The item to be added.
     */
    public void addItem(String labelText, JComponent item) {

        // Create the label and its constraints
        JLabel label = new JLabel(labelText);
        GridBagConstraints labelConstraints = new GridBagConstraints();

        labelConstraints.gridx = 0;
        labelConstraints.gridy = myNextItemRow;
        labelConstraints.insets = new Insets(10, 10, 0, 0);
        labelConstraints.anchor = fLabelAlignment;
        labelConstraints.fill = GridBagConstraints.NONE;

        add(label, labelConstraints);
        labels.add(label);

        // Add the component with its constraints
        GridBagConstraints itemConstraints = new GridBagConstraints();

        itemConstraints.gridx = 1;
        itemConstraints.gridy = myNextItemRow;
        itemConstraints.insets = new Insets(10, 10, 0, 10);
        itemConstraints.weightx = 1.0;
        itemConstraints.anchor = GridBagConstraints.WEST;
        itemConstraints.fill = GridBagConstraints.HORIZONTAL;

        add(item, itemConstraints);

        myNextItemRow++;
    }

    public Component add(String name, Component comp) {
        throw new IllegalArgumentException("Dont use -- use addItem instead");
    }

    public void setLabelText(int index, String newText) {
        ((JLabel) labels.get(index)).setText(newText);
    }

} // End GLabelledItemPanel

/*--- Formatted in Sun Java Convention Style on Wed, Oct 16, '02 ---*/
