/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.swing;

import javax.swing.*;
import java.util.ArrayList;
import java.util.List;

/**
 * combobx to acts a s aseletor for thread prioritities
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */

public class GThreadPriorityBox extends JComboBox {

    private static final String LOW_HTML = "<html><body>Low <font color=\"#C0C0C0\">(cpu usage)</font></body></html>";
    private static final String NORMAL_HTML = "<html><body>Normal <font color=\"#C0C0C0\">(cpu usage)</font></body></html>";
    private static final String HIGH_HTML = "<html><body>High <font color=\"#C0C0C0\">(cpu usage)</font></body></html>";

    private static final String LOW = "Low";
    private static final String NORMAL = "Normal";
    private static final String HIGH = "High";

    private boolean fShowGrayHelpText;

    /**
     * with default + the add ones
     *
     * @param additionalPriorities
     */
    public GThreadPriorityBox(final String[] additionalPriorities,
                              final boolean includeHigh,
                              final boolean defaultIsNormal,
                              final boolean showGrayHelpText) {
        super();

        this.fShowGrayHelpText = showGrayHelpText;

        List prs = new ArrayList();
        prs.add(_low());
        prs.add(_normal());

        if (includeHigh) {
            prs.add(_high());
        }

        for (int i = 0; i < additionalPriorities.length; i++) {
            prs.add(additionalPriorities[i]);
        }


        DefaultComboBoxModel model = new DefaultComboBoxModel(prs.toArray(new String[prs.size()]));

        this.setModel(model);
        if (defaultIsNormal) {
            this.setSelectedItem(_normal());
        } else {
            this.setSelectedItem(_low());
        }

        this.setToolTipText("Set the priority of this Task - low to high cpu usage");
    }

    public GThreadPriorityBox(final boolean includeHigh,
                              final boolean defaultIsNormal,
                              final boolean showGrayHelpText) {
        this(new String[]{}, includeHigh, defaultIsNormal, showGrayHelpText);
    }

    public void setSelectedPriority(int currPriority) {
        this.setSelectedItem(getPriorityStringValue(currPriority));
    }

    public String getPriorityStringValue(int priority) {
        if (priority == Thread.MIN_PRIORITY) {
            return _low();
        } else if (priority == Thread.NORM_PRIORITY) {
            return _normal();
        } else if (priority == Thread.MAX_PRIORITY) {
            return _high();
        } else {
            throw new IllegalArgumentException("Unknown Thread priority code: " + priority);
        }
    }

    public int getPriority() {
        String s = this.getSelectedItem().toString();
        if (s.equals(_low())) {
            return Thread.MIN_PRIORITY;
        } else if (s.equals(_high())) {
            return Thread.MAX_PRIORITY;
        } else if (s.equals(_normal())) {
            return Thread.NORM_PRIORITY;
        } else {
            return -1;
        }

    }


    private String _low() {
        if (fShowGrayHelpText) {
            return LOW_HTML;
        } else {
            return LOW;
        }
    }

    private String _normal() {
        if (fShowGrayHelpText) {
            return NORMAL_HTML;
        } else {
            return NORMAL;
        }
    }

    private String _high() {
        if (fShowGrayHelpText) {
            return HIGH_HTML;
        } else {
            return HIGH;
        }
    }

} // End class GThreadPriorityBox

