/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.swing;


import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.parsers.DataFormat;
import edu.mit.broad.genome.utils.containers.ClassNamePair;
import org.apache.log4j.Logger;

import javax.swing.*;
import javax.swing.plaf.basic.BasicComboBoxRenderer;
import javax.swing.table.DefaultTableCellRenderer;
import java.awt.*;
import java.util.Date;


/**
 * Title:  enter desc
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class RendererFactory {

    private static final Logger klog = XLogger.getLogger(RendererFactory.class);

    private RendererFactory() {
    }

    /**
     * Class TableCellRenderer
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    public static class TableCellRenderer extends DefaultTableCellRenderer {

        public Component getTableCellRendererComponent(JTable table, Object value,
                                                       boolean isSelected, boolean hasFocus, int row, int col) {

            if (value instanceof TextIconPair) {
                this.setText(((TextIconPair) value).text);
                this.setIcon(((TextIconPair) value).icon);
            } else {
                return super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row,
                        col);
            }

            return this;
        }
    }    // End TableCellRenderer


    /**
     * As row labels can be long and might not be fully visible, this class renders
     * the cell with a tool tip set to the entire label
     */
    public static class ToolTipLabelRenderer extends DefaultTableCellRenderer {

        public Component getTableCellRendererComponent(JTable table, Object value,
                                                       boolean isSelected, boolean hasFocus, int row, int column) {

            super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
            this.setFont(GuiHelper.FONT_DEFAULT_BOLD);

            //this.setForeground(Color.green);
            //this.setBackground(Color.yellow);
            this.setForeground(Color.black);

            if (value != null) {
                this.setToolTipText(value.toString());
            }

            return this;
        }
    }    // End ToolTipLabelRenderer

    /**
     * Class DataFormatListCellRenderer
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    public static class DataFormatListCellRenderer extends DefaultListCellRenderer {

        public Component getListCellRendererComponent(JList list, Object value, int index,
                                                      boolean isSelected, boolean cellHasFocus) {

            // doesnt work properly unless called
            super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);

            if (value instanceof DataFormat) {
                this.setText(((DataFormat) value).getName());
                this.setIcon(((DataFormat) value).getIcon());
            } else if (value == null) {
                return this;
            } else {
                this.setText(value.toString());
            }

            return this;
        }
    }    // End ListCellRenderer

    /**
     * rendering.
     * <p/>
     * <p/>
     * data structure to captyure a  class-mname pair relt
     * Alos provides a decorator method for comboboxes taht uses thes pairs ttha"
     * <p/>
     * 1) provides custom renderinhg to set the test as name
     * 2) override getselcted toreturn the class as object (an dnot classnamepair)
     */
    public static class ClassNamePairListCellRenderer extends DefaultListCellRenderer {

        public Component getListCellRendererComponent(JList list, Object value, int index,
                                                      boolean isSelected, boolean cellHasFocus) {

            // doesnt work properly unless called
            super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);

            if (value == null) {
                return this;
            }

            if (value instanceof ClassNamePair) {
                this.setText(((ClassNamePair) value).getPairName());
            } else {
                this.setText(value.toString());
            }

            return this;
        }
    }    // End ClassNamePairListCellRenderer


    /**
     * special icon renderer for objects of type:
     * Integer
     * Float / Double
     * String
     * Date
     * <p> </p>
     * <p> </p>
     * <p> </p>
     * <p> </p>
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    public static class ClassListCellRenderer extends DefaultListCellRenderer {

        public Component getListCellRendererComponent(JList list, Object value, int index,
                                                      boolean isSelected, boolean cellHasFocus) {

            // doesnt work properly unless called
            super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);

            if (value instanceof Integer) {
                this.setIcon(GuiHelper.ICON_INTEGER);
            } else if (value instanceof Float) {
                this.setIcon(GuiHelper.ICON_FLOAT);
            } else if (value instanceof Double) {
                this.setIcon(GuiHelper.ICON_FLOAT);
            } else if (value instanceof String) {
                this.setIcon(GuiHelper.ICON_STRING);
            } else if (value instanceof Date) {
                this.setIcon(GuiHelper.ICON_DATE);
            }

            this.setText(value.toString());

            return this;
        }
    }    // End ClassListCellRenderer


    /**
     * GramTableCellRenderer that displays a tooltip for each element of a JComboBox
     * This does NOT define a DataModel.
     * The combo box passed to this class needs to come with an
     * appropriate DataModel for the tool tips to be meaningful.
     * <p/>
     * The Tooltip is set to: model.getSelectedItem().toString()
     * The label is (as usual) whatever is passed to the
     * method: getListCellRendererComponent by the underlying DataModel.
     * <p/>
     * Known Bug Note :
     * When JComboBox is located near the border of a Frame, the tooltip
     * doesn't display outside the frame due to current Swing limitations.
     * <p/>
     * Credits: Code from javaworld.com tip 114?
     */
    public static class BasicComboBoxRendererWithTooltips extends BasicComboBoxRenderer {

        final JComboBox fCombo;

        public BasicComboBoxRendererWithTooltips(JComboBox aCombo) {

            super();

            fCombo = aCombo;
        }

        public Component getListCellRendererComponent(JList list, Object value, int index,
                                                      boolean isSelected, boolean cellHasFocus) {

            setFont(list.getFont());
            setText((value == null)
                    ? ""
                    : value.toString());

            if (isSelected) {
                setBackground(list.getSelectionBackground());
                setForeground(list.getSelectionForeground());

                if (index >= 0) {
                    if (this.getPreferredSize().width > fCombo.getSize().width) {
                        if (fCombo.getModel() != null) {
                            list.setToolTipText(fCombo.getModel().getSelectedItem().toString());
                        }
                    } else {
                        list.setToolTipText(null);
                    }
                }
            } else {
                setBackground(list.getBackground());
                setForeground(list.getForeground());
            }

            return this;
        }
    }    // End of class GComboBoxRendererWithTooltips

}        // End RendererFactory

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
