/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.swing;

/**
 * @author John Redmond (John.Redmond@mq.edu.au).
 * @version %I%, %G%
 * @(#)SCLayout.java	1.02 2 Jan 2002
 * <p/>
 * Copyright John Redmond (John.Redmond@mq.edu.au).
 * <p/>
 * This software is freely available for commercial and non-commercial purposes.
 * Acknowledgement of its source would be appreciated, but is not required.
 * @see http://www.javaworld.com/javaworld/javatips/jw-javatip129.html
 */

public class SCLayout extends SGLayout {

    /**
     * Creates a layout with the specified number of rows
     * and default alignments and gaps.
     * <p/>
     * vertical gaps are set to 0 and
     * X- and Y-alignments are set to FILL.
     *
     * @param rows the rows.
     */
    public SCLayout(int rows) {
        super(rows, 1);
    }

    /**
     * Creates a layout with the specified number of rows
     * and specified gap.
     * <p/>
     * vertical gaps are set to 0 and
     * X- and Y-alignments are set to FILL.
     *
     * @param rows the rows.
     * @param gap  the vertical gap, in pixels.
     */
    public SCLayout(int rows, int gap) {
        super(rows, 1, FILL, FILL, gap, 0);
    }

    /**
     * Creates a layout with the specified number of rows
     * and specified alignments and gaps.
     * <p/>
     * vertical gaps are set to 0 and
     * X- and Y-alignments are set to FILL.
     *
     * @param rows       the rows.
     * @param hAlignment the X-alignment.
     * @param vAlignment the Y-alignment.
     * @param gap        the vertical gap, in pixels.
     */
    public SCLayout(int rows, int hAlignment, int vAlignment, int gap) {
        super(rows, 1, hAlignment, vAlignment, 0, gap);
    }

    /**
     * Set up alignment for a specific cell.
     * <p/>
     *
     * @param index      the cell number.
     * @param hAlignment the X-alignment for the cell.
     * @param vAlignment the Y-alignment for the cell.
     */
    public void setAlignment(int index, int hAlignment, int vAlignment) {
        setRowAlignment(index, hAlignment, vAlignment);
    }

    /**
     * Set up scale value for a specific cell.
     * <p/>
     *
     * @param index the column number.
     * @param scale the scale value for the column.
     */
    public void setScale(int cell, double scale) {
        setRowScale(cell, scale);
    }
}
