package edu.mit.broad.genome.swing.choosers;

import edu.mit.broad.genome.NotImplementedException;
import edu.mit.broad.genome.swing.GuiHelper;
import edu.mit.broad.genome.swing.fields.GFieldPlusChooser;
import edu.mit.broad.genome.swing.windows.GTableWindow;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;


/**
 * <p>JTextField with a button that when clicked on popus ups a JList window where  i or more
 * selections can be made. The selections are then pasted as comma delimited text into
 * the JTextField</p>
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 * @note diff from a chooser in that the input field is a Text area and not a list (ie chooser)
 */
public class GTableInputFieldPlusChooser extends JPanel implements GFieldPlusChooser {

    private final JButton bEntry = new JButton(GuiHelper.ICON_ELLIPSIS);

    private JTable table;

    private LazyHandlers.Table customHandleForTableMaker;

    private GTableWindow fWindow;

    private JPanel panelForWindow;

    public GTableInputFieldPlusChooser() {
        init();
    }


    public GTableInputFieldPlusChooser(final LazyHandlers.Table customHandleForTableMaker) {
        this.customHandleForTableMaker = customHandleForTableMaker;
        init();
    }

    private void init() {

        jbInit();
        bEntry.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                lazillyMakeTable();
                fWindow.show();
            }
        });
    }

    private void lazillyMakeTable() {
        if (panelForWindow == null) {
            panelForWindow = new JPanel(new BorderLayout());

            if (customHandleForTableMaker == null) { // make our own vanilla table
                DefaultTableModel model = new DefaultTableModel(500, 10);
                table = new JTable(model);
                table.setRowSelectionAllowed(false);
                table.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);

                JButton button = new JButton("Clear");
                button.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        resetModel();
                    }
                });

                panelForWindow.add(button, BorderLayout.SOUTH);
            } else {
                // this seems to be important for the scroll bar to show up (has to be done in tanded with setting init cell width in the annot block)
                table = customHandleForTableMaker.createTable();
                table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
                //table.setAutoResizeMode(JTable.A);
            }

            panelForWindow.add(new JScrollPane(table), BorderLayout.CENTER);
        }

    }

    private void jbInit() {

        this.setLayout(new BorderLayout());
        this.add(Box.createHorizontalStrut(100), BorderLayout.CENTER);
        this.add(bEntry, BorderLayout.EAST);

        fWindow = new GTableWindow(this);
    }

    private void resetModel() {
        //System.out.println("resetting model");
        DefaultTableModel model = (DefaultTableModel) table.getModel();
        for (int r = 0; r < model.getRowCount(); r++) {
            for (int c = 0; c < model.getColumnCount(); c++) {
                model.setValueAt(null, r, c);
            }
        }
    }

    public Object getValue() {
        return table; // @todo
    }

    public JComponent getComponent() {
        return this;
    }

    // might return null if not inited yet
    public JTable getTable() {
        return table;
    }

    public void setValue(final Object obj) {
        throw new NotImplementedException();
    }

    public JPanel getComponentForWindow() {
        return panelForWindow;
    }


}    // End GTableInputFieldPlusChooser

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
