/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.swing.fields;


import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.swing.choosers.GFileFieldPlusChooser;
import edu.mit.broad.genome.utils.ColorUtils;
import edu.mit.broad.genome.utils.NamedInteger;
import edu.mit.broad.genome.utils.containers.Password;
import org.apache.log4j.Logger;

import javax.swing.*;
import java.awt.*;
import java.io.File;


/**
 * Class FieldFactory
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
final public class FieldFactory {

    private static final Logger klog = XLogger.getLogger(FieldFactory.class);

    /**
     * Privatized class constructor
     */
    private FieldFactory() {
    }


    public static GComboBoxField createTabPlacementField(int def) {


        NamedInteger[] options = new NamedInteger[]{
                new NamedInteger(JTabbedPane.TOP, "Top"),
                new NamedInteger(JTabbedPane.BOTTOM, "Bottom"),
                new NamedInteger(JTabbedPane.LEFT, "Left"),
                new NamedInteger(JTabbedPane.RIGHT, "Right")
        };
        JComboBox cb = new JComboBox(new DefaultComboBoxModel(options));

        for (int i = 0; i < options.length; i++) {
            if (options[i].getValue() == def) {
                cb.setSelectedIndex(i);
                break;
            }
        }

        return new GComboBoxField(cb);
    }

    /**
     * factory method that produces a GFieldPlusChooser based on type
     * of object specified
     */
    public static GFieldPlusChooser createFieldChooser(final Object value) {

        if (value instanceof File) {
            return new GFileFieldPlusChooser();
        } else if (value instanceof Color) {
            return new GColorFieldPlusChooser((Color) value);
        } else if (value instanceof Password) {
            return new GPasswordField2(((Password) value).getPassword());
        } else if (value instanceof String) {
            return new GStringField((String) value);
        } else if (value instanceof Integer) {
            return new GIntegerField((Integer) value);
        } else if (value instanceof Float) {
            return new GFloatField((Float) value);
        } else {
            klog.warn("Unknown fields: " + value);

            return new GStringField("Unknown fields: " + value);
        }
    }

    /**
     * @param field
     * @param value
     * @return
     */
    public static GFieldPlusChooser createFieldChooser(GFieldPlusChooser.Field field, String value) {

        if (field.equals(GFieldPlusChooser.FILE_FIELD)) {
            return FieldFactory.createFieldChooser(new File(value));
        } else if (field.equals(GFieldPlusChooser.DIR_FIELD)) {
            //log.debug("factory -- DIR fields " + fields.toString());
            GDirFieldPlusChooser comp = new GDirFieldPlusChooser();
            comp.setRootPath(value);
            return comp;
        } else if (field.equals(GFieldPlusChooser.COLOR_FIELD)) {
            Color color = ColorUtils.colorForName(value, false);

            if (color == null) {
                return new GColorFieldPlusChooser();
            } else {
                return FieldFactory.createFieldChooser(color);
            }
        } else if (field.equals(GFieldPlusChooser.STRING_FIELD)) {
            if (value == null) {
                return FieldFactory.createFieldChooser("");
            } else {
                return FieldFactory.createFieldChooser(value);
            }
        } else if (field.equals(GFieldPlusChooser.INTEGER_FIELD)) {
            try {
                Integer i = new Integer(value);

                return FieldFactory.createFieldChooser(i);
            } catch (NumberFormatException e) {
                klog.warn("Not an integer in fields!", e);

                return FieldFactory.createFieldChooser(new Integer(-999));
            }
        } else if (field.equals(GFieldPlusChooser.FLOAT_FIELD)) {
            try {
                Float i = new Float(value);

                return FieldFactory.createFieldChooser(i);
            } catch (NumberFormatException e) {
                klog.warn("Not an integer in fields!", e);

                return FieldFactory.createFieldChooser(new Float(-999));
            }
        } else {
            klog.warn("Unknown fields: " + field + " and value: " + value);
            return FieldFactory.createFieldChooser("Error! -- Unknown");
        }
    }
}    // End FieldFactory

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
