/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.swing.fields;


import edu.mit.broad.genome.Constants;
import edu.mit.broad.genome.parsers.ParseUtils;
import edu.mit.broad.genome.swing.GBase;
import edu.mit.broad.genome.swing.GuiHelper;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * A GFieldPlusChooser that contains a JTextField and a JButton. Clicking on the
 * button brings up a JColorChooser widget and allows the user to select a Color.
 * Once done, the Color choosen can be accessed through getValue().
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class GColorFieldPlusChooser extends JPanel implements GFieldPlusChooser {

    /**
     * The text fields that holds file path
     */
    private JTextField tfValue;

    /**
     * The choosen color
     */
    private Color fColor;

    private JColorChooser fChooser;

    private GBase fBase = new GBase();

    /**
     * Initialkizes to null colors and labels
     * NO label diaplayed in the chooser fields
     */
    public GColorFieldPlusChooser() {
        this(null, null);
    }

    /**
     * No label diaplayed in the chooser fields
     *
     * @param color
     */
    public GColorFieldPlusChooser(Color color) {
        this(null, color);
    }

    /**
     * displays specified label in chooser fields
     * Null color initilizaed
     *
     * @param label
     */
    public GColorFieldPlusChooser(final String label) {
        this(label, null);
    }

    /**
     * Class Constructor.
     *
     * @param label        can be null
     * @param defaultColor can be null
     */
    public GColorFieldPlusChooser(final String label, final Color defaultColor) {

        super();

        this.fColor = defaultColor;

        JButton but = new JButton(GuiHelper.ICON_ELLIPSIS);

        but.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (fChooser == null) { // make lazily
                    fChooser = new JColorChooser();
                }

                Color ret = JColorChooser.showDialog(fBase.getParent(), "Select Color", fColor);

                if (ret != null) {
                    fColor = ret;
                    tfValue.setText(ParseUtils.color2string(fColor));
                    tfValue.setForeground(fColor);
                    tfValue.setEnabled(false);
                }
            }
        });

        tfValue = new JTextField(Constants.NA);
        setColor(fColor);

        tfValue.setEditable(false);
        tfValue.setEnabled(false);    // always
        this.setLayout(new BorderLayout());

        if (label != null) { // only add if specified
            this.add(new JLabel(label), BorderLayout.WEST);
        }

        this.add(tfValue, BorderLayout.CENTER);
        this.add(but, BorderLayout.EAST);
    }

    /**
     * GFieldPlusChooser impl.
     * Might be null if no default Color was specified in constructor
     * OR
     * if user cancelled a selection (and there was no Color already present)
     *
     * @return A Color representing the users selection or if the selection was cancelled,
     *         the default Color specified in argument to class constructor.
     */
    public Object getValue() {
        return fColor;
    }

    /**
     * Same as getValue
     *
     * @return
     * @see getValue()
     */
    public Color getColor() {
        return fColor;
    }

    /**
     * GFieldPlusChooser impl.
     *
     * @return This instance of GColorFieldPlusChooser
     */
    public JComponent getComponent() {
        return this;
    }

    /**
     * Set the color
     *
     * @param color
     */
    public void setColor(Color color) {
        fColor = color;
        if (fColor != null) {
            tfValue.setText(ParseUtils.color2string(fColor));
            tfValue.setForeground(fColor);
        }
    }

    public void setValue(Object obj) {
        Color c;
        if (obj == null) {
            c = Color.black; // better than exceptioning out
        } else if (obj instanceof Color) {
            c = (Color) obj;
        } else {
            c = Color.getColor(obj.toString());
        }

        this.setColor(c);

    }

}    // End GColorFieldPlusChooser

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
