/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.swing.fields;


import javax.swing.*;
import java.awt.*;

/**
 * Interface for a GFieldPlusChooser.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public interface GFieldPlusChooser {

    public static Insets LEFT_INSETS = new Insets(1, 2, 1, 0);
    public static Dimension LEFT_DIM = new Dimension(250, 7);
    public static Insets RIGHT_INSETS = new Insets(1, 0, 1, 4);
    public static Dimension RIGHT_DIM = new Dimension(0, 0);

    public static final GFieldPlusChooser.Field FILE_FIELD = new Field("FILE");
    public static final GFieldPlusChooser.Field DIR_FIELD = new Field("DIR");
    public static final GFieldPlusChooser.Field COLOR_FIELD = new Field("COLOR");
    public static final GFieldPlusChooser.Field STRING_FIELD = new Field("STRING");
    public static final GFieldPlusChooser.Field INTEGER_FIELD = new Field("INT");
    public static final GFieldPlusChooser.Field FLOAT_FIELD = new Field("FLOAT");
    public static final GFieldPlusChooser.Field FONT_FIELD = new Field("FONT");
    public static final GFieldPlusChooser.Field DATE_FIELD = new Field("DATE");

    /**
     * @return An object containing the value choosen.
     */
    public Object getValue();

    /**
     * @return The Component that contains the GFieldPlusChooser.
     *         Often, the GFieldPlusChooser itself.
     *         <p/>
     *         Alternative impl would have this interface subclass Component, buts that is
     *         not possible in the java language specification.
     */
    public JComponent getComponent();

    /**
     * Change the choosen value programmatically
     *
     * @param obj
     */
    public void setValue(Object obj);

    /**
     * Enum construct and NamingConventions methods for JFieldChoosers
     * <
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    public class Field {

        private final String id;

        /**
         * Privatized Class Constructor.
         *
         * @param id
         */
        private Field(String id) {
            this.id = id;
        }

        public int hashCode() {
            return id.hashCode();
        }

        public boolean equals(Field field) {

            if (field == null) {
                return false;
            }

            if (field.id == this.id) {
                return true;
            }

            return false;
        }

        public boolean equals(Object obj) {

            if (obj instanceof Field) {
                return equals((Field) obj);
            }

            return false;
        }

        public static Field lookup(String fieldname) {

            if (fieldname == null) {
                throw new NullPointerException("Param fieldname cannot be null");
            }

            if (fieldname.equals(FILE_FIELD.id)) {
                return FILE_FIELD;
            } else if (fieldname.equalsIgnoreCase(DIR_FIELD.id)) {
                return DIR_FIELD;
            } else if (fieldname.equalsIgnoreCase(COLOR_FIELD.id)) {
                return COLOR_FIELD;
            } else if (fieldname.equalsIgnoreCase(STRING_FIELD.id)) {
                return STRING_FIELD;
            } else if (fieldname.equalsIgnoreCase(INTEGER_FIELD.id)) {
                return INTEGER_FIELD;
            } else if (fieldname.equalsIgnoreCase(FLOAT_FIELD.id)) {
                return FLOAT_FIELD;
            } else if (fieldname.equalsIgnoreCase(FONT_FIELD.id)) {
                return FONT_FIELD;
            } else if (fieldname.equalsIgnoreCase(DATE_FIELD.id)) {
                return DATE_FIELD;
            } else {
                throw new IllegalArgumentException("No such fields id: " + fieldname);
            }
        }
    } // End inner class Field


}

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/