/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.swing.fields;


import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.swing.GuiHelper;
import org.apache.log4j.Logger;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableColumn;
import java.awt.*;


/**
 * JTable all setup to accept JFields and display them properly ->
 * 2 columns:
 * col 0 -> label
 * col 1 -> the JField
 * <p/>
 * <p> </p>
 * <p> </p>
 * <p> </p>
 * <p> </p>
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class GFieldPlusChooserTable extends JTable {

    private final Logger log = XLogger.getLogger(this.getClass());

    private String[] fLabels;
    private GFieldPlusChooser[] fFields;
    private static final String[] COL_HEADERS = new String[]{"Property", "Value"};

    private static final int COL_LABEL = 0;

    private Model fModel;

    private boolean fModified;

    /**
     * empty constructor - must call setData() if using this form of the constructor
     */
    public GFieldPlusChooserTable() {
        // table visual properties
        this.setRowHeight(30);

        //this.setRowMargin(10);    // margins make the cbs etc nicer -- No!
        this.setCellSelectionEnabled(false);
        this.setRowSelectionAllowed(false);
        this.setCellSelectionEnabled(false);
        this.setShowVerticalLines(false);
        this.setGridColor(Color.black);
        this.setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS);
        this.getTableHeader().setReorderingAllowed(false);
    }

    /**
     * Number of labels must equal number of fields
     *
     * @param labels
     * @param fields
     */
    public GFieldPlusChooserTable(String[] labels, GFieldPlusChooser[] fields) {
        this();

        this.setData(labels, fields);

    }

    public void setData(String[] labels, GFieldPlusChooser[] fields) {
        if (labels == null) {
            throw new IllegalArgumentException("Param labels cannot be null");
        }

        if (fields == null) {
            throw new IllegalArgumentException("Param fields cannot be null");
        }

        if (labels.length != fields.length) {
            throw new IllegalArgumentException("# labels: " + labels.length
                    + " not equal to # fields: " + fields.length);
        }

        this.fLabels = labels;
        this.fFields = fields;

        this.fModel = new Model();
        this.setModel(fModel);

        //this.getTableHeader().setDefaultRenderer(new RendererFactory2.BoldHeaderRenderer());
        this.setDefaultRenderer(Object.class, new Renderer());

        // the components stay un-responsive unless this is set
        this.setDefaultEditor(Component.class, new Editor());

        // we know what the columns are so set their size explicitly
        TableColumn column = this.getColumnModel().getColumn(0);
        column.setMinWidth(5);
        int width = GuiHelper.Table.scanMaxColumnWidth(this, 0) + 15;    // some buffer
        //log.debug("Max width = " + width);
        column.setMaxWidth(width);
        column.setPreferredWidth(width);

    }

    public Object getLabelAt(int row) {
        return fLabels[row];
    }

    public GFieldPlusChooser getFieldChooserAt(int row) {
        return fFields[row];
    }

    public boolean isModified() {
        return fModified;
    }

    /**
     * Class Model
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    private class Model extends AbstractTableModel {

        public int getRowCount() {
            return fLabels.length;
        }

        public int getColumnCount() {
            return COL_HEADERS.length;
        }

        public Class getColumnClass(int col) {
            return getValueAt(0, col).getClass();
        }

        public String getColumnName(int col) {
            return COL_HEADERS[col];
        }

        /**
         * IMP do nothing - let the fields chooser take care of things
         *
         * @param obj
         * @param row
         * @param col
         */
        public void setValueAt(Object obj, int row, int col) {
            fModified = true;
        }

        public boolean isCellEditable(int row, int col) {

            if (col == COL_LABEL) {
                return false;
            } else {
                return true;
            }
        }

        public Object getValueAt(int row, int col) {

            if (col == COL_LABEL) {
                return fLabels[row];
            } else {
                return fFields[row];
            }
        }
    }    // End Model


    /**
     * <p> </p>
     * <p> </p>
     * <p> </p>
     * <p> </p>
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    private class Renderer extends DefaultTableCellRenderer {

        public Component getTableCellRendererComponent(JTable table, Object value,
                                                       boolean isSelected, boolean hasFocus, int row, int column) {

            super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);

            //log.debug("rendering value: " + value);
            if (column == COL_LABEL) {
                this.setFont(GuiHelper.FONT_DEFAULT_BOLD);
                this.setText(value.toString());
                this.setEnabled(true);

                return this;
            } else {
                GFieldPlusChooser field = (GFieldPlusChooser) value;
                Component comp = field.getComponent();

                comp.setForeground(table.getSelectionForeground());

                //comp.setBackground(LIGHT_BLUE);
                if (isSelected) {
                    comp.setForeground(table.getSelectionForeground());

                    //comp.setBackground(table.getSelectionBackground());
                } else {
                    comp.setForeground(table.getForeground());

                    //comp.setBackground(Color.lightGray);
                }

                return comp;
            }
        }
    }    // End inner class GramTableCellRenderer

    /**
     * <p> </p>
     * <p> </p>
     * <p> </p>
     * <p> </p>
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    private class Editor extends AbstractCellEditor implements TableCellEditor {

        private GFieldPlusChooser currField;

        /**
         * TableCellEditor impl.
         * The core method to implement to acheive desired effect.
         */
        public Component getTableCellEditorComponent(JTable table, Object value,
                                                     boolean isSelected, int row, int column) {

            //log.debug("editing for: " + value);

            currField = fFields[row];

            return currField.getComponent();
        }

        public Object getCellEditorValue() {
            if (currField == null) return null;
            return currField.getValue();
        }
    }    // End inner class Editor
}        // End GFieldPlusChooserTable

/*--- Formatted in Sun Java Convention Style on Sun, Sep 29, '02 ---*/
