/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.swing.fields;

import edu.mit.broad.genome.parsers.AuxUtils;

import java.awt.*;
import java.io.File;
import java.util.StringTokenizer;

/**
 * @author Aravind Subramanian
 *         Date: Jul 2, 2003
 *         Time: 9:53:56 PM
 */
public class GFieldUtils {

    public static final Color GOOD_COLOR = Color.blue;
    public static final Color ERROR_COLOR = Color.red;

    public static Color getFileFieldColor(File file) {
        if (file == null) {
            return ERROR_COLOR;
        }

        if (file.getName().startsWith("http") || file.getName().startsWith("ftp") || file.getName().startsWith("gseaftp")) {
            return GOOD_COLOR;
        }

        if (file.exists()) {
            return GOOD_COLOR;
        } else {
            return ERROR_COLOR;
        }
    }

    /**
     * Needs to accoint for that sometimes the field contains multiple paths
     * separated by commas.
     *
     * @param text
     * @return
     */
    public static Color getFileFieldColor(String pathOrPaths) {
        if (pathOrPaths == null) {
            return ERROR_COLOR;
        }

        if (pathOrPaths.startsWith("http") || pathOrPaths.startsWith("ftp") || pathOrPaths.startsWith("gseaftp")) {
            return GOOD_COLOR;
        }

        pathOrPaths = pathOrPaths.trim();

        if (pathOrPaths.indexOf(',') == -1) {
            return _getColor(pathOrPaths);
        }

        StringTokenizer tok = new StringTokenizer(pathOrPaths, ",");
        int cnt = tok.countTokens();
        for (int i = 0; i < cnt; i++) {
            if (isGoodPath(tok.nextToken()) == false) {
                return ERROR_COLOR;
            }
        }

        return GOOD_COLOR;
    }

    public static Color _getColor(String path) {
        if (isGoodPath(path)) {
            return GOOD_COLOR;
        } else {
            return ERROR_COLOR;
        }
    }

    public static boolean isGoodPath(String pathMaybeWithAux) {
        if (pathMaybeWithAux == null || pathMaybeWithAux.length() == 0) {
            return false;
        }

        if (pathMaybeWithAux.toLowerCase().startsWith("http") ||
                pathMaybeWithAux.toLowerCase().startsWith("www") ||
                pathMaybeWithAux.toLowerCase().startsWith("ftp") ||
                pathMaybeWithAux.toLowerCase().startsWith("gseaftp")) {
            return true;
        }

        File file = AuxUtils.getBaseFileFromFullPath(pathMaybeWithAux);
        if (file.exists()) {
            return true;
        } else {
            return false;
        }
    }

} // End GFieldUtils
