/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.swing.fields;


import javax.swing.*;
import java.awt.event.KeyEvent;


/**
 * @author Aravind Subramanian
 * @version %I%, %G%
 * @todo A TextField that accepts only nums and '.'
 */
public class GFloatField extends JTextField implements GFieldPlusChooser {

    // private static final Pattern BADCHARS = Pattern.compile("`~!@#$%^&*()_+=\\|\"':;?/><, ");

    /**
     * Class Constructor.
     * Constructs a new GFloatField.  A default model is created, the initial
     * float is null, and the number of columns is set to 0.
     */
    public GFloatField() {
        super();
    }

    /**
     * Class Constructor.
     * <p/>
     * Constructs a new GFloatField initialized with the specified Integer.
     * A default model is created and the number of columns is 0.
     *
     * @param i the Float to be displayed, or null
     */
    public GFloatField(Float f) {
        super(f.toString());
    }

    /**
     * Class Constructor.
     * <p/>
     * Constructs a new empty GFloatField with the specified number of columns.
     * A default model is created and the initial string is set to null.
     *
     * @param columns the number of columns to use to calculate
     *                the preferred width.  If columns is set to zero, the
     *                preferred width will be whatever naturally results from
     *                the component implementation.
     */
    public GFloatField(int cols) {
        super(cols);
    }

    /**
     * @return The user specified Float
     */
    public Object getValue() {

        if (this.getText() == null) {
            return null;
        } else {
            return new Float(this.getText());
        }
    }

    /**
     * If no entry, then Integer.MIN_VALUE is returned
     *
     * @return
     */
    public float getFloatValue() {
        String text = getText();

        if ((text == null) || (text.length() == 0)) {
            return Float.NEGATIVE_INFINITY;
        } else {
            return Float.parseFloat(text);

        }
    }

    /**
     * ditto to getIntValue, except that def is returned if no vakue entered
     *
     * @return
     */
    public float getFloatValue(float def) {
        String text = getText();

        if ((text == null) || (text.length() == 0)) {
            return def;
        } else {
            return Float.parseFloat(text);

        }
    }

    public JComponent getComponent() {
        return this;
    }

    public void setValue(Object obj) {
        Float f;

        if (obj == null) {
            f = new Float(Float.NEGATIVE_INFINITY); // better than exceptioning out
        } else if (obj instanceof Float) {
            f = (Float) obj;
        } else {
            f = new Float(obj.toString());

        }

        this.setText(f.toString());
    }

    /**
     * Does the float-enforcement.
     */
    public void processKeyEvent(KeyEvent ev) {

        char c = ev.getKeyChar();

        if (Character.isLetter(c) && !ev.isAltDown()) {
            ev.consume();

            return;
        }

        /*
        if((Character.isLetter(c) && !ev.isAltDown()) || Strings.badchars().pattern().indexOf(c) > -1) {
           ev.consume();
           return;
       }
       */
        if ((c == '-') && (getDocument().getLength() > 0)) {
            ev.consume();
        } else {
            super.processKeyEvent(ev);
        }
    }
}    // End GFloatField

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
