/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.swing.fields;


import javax.swing.*;
import java.awt.*;
import java.text.NumberFormat;
import java.text.ParseException;

/**
 * Allows the user to only enter a whole number.  This doesn't not need to be used
 * in Java 1.4 or latter.
 *
 * @author kohm
 * @version %I%, %G%
 * @see FormatTextTest
 * @see javax.swing.JFormattedTextField
 */
public class GIntegerField extends JTextField implements GFieldPlusChooser {

    /**
     * needed to make a "beep"
     */
    private final Toolkit toolkit = Toolkit.getDefaultToolkit();

    /**
     * number formatter
     */
    private final NumberFormat integerFormatter;

    /**
     * constructor
     */
    public GIntegerField(final Number value, final int columns) {
        this(value.intValue(), columns);
    }

    public GIntegerField(final Number value) {
        this(value, 5);
    }

    /**
     * constructor
     */
    public GIntegerField(final int value, final int columns) {
        this(value, columns, NumberFormat.getNumberInstance());
    }

    /**
     * constructor
     */
    public GIntegerField(final int value, final int columns, final NumberFormat format) {

        super(new ChangeValidatedDocument(format), null, columns);

        integerFormatter = format;

        format.setParseIntegerOnly(true);
        setInt(value);
    }


    public void setInt(int value) {
        super.setText(integerFormatter.format(value));
    }

    /**
     * overriden now will throw number format exeption if text s not a number
     */
    public void setText(String text) {

        if ((text != null) && (text.length() > 0)) {
            final int value = Integer.parseInt(text);

            super.setText(integerFormatter.format(value));
        } else {
            super.setText("0");
        }
    }

    /**
     * @return The user specified Integer
     */
    public Object getValue() {

        try {
            if (this.getText() == null) {
                return null;
            } else {
                return new Integer(this.getText());
            }
        } catch (Throwable t) {
            return null;
        }
    }

    /**
     * If no entry, then Integer.MIN_VALUE is returned
     *
     * @return
     */
    public int getIntValue() {

        int retVal = Integer.MIN_VALUE;

        try {
            retVal = integerFormatter.parse(getText()).intValue();
        } catch (ParseException e) {

            // This should never happen because insertString allows
            // only properly formatted data to get in the fields.
            toolkit.beep();

            throw new IllegalStateException("Somehow the value is not a number: '" + getText()
                    + "'");
        }

        return retVal;

    }

    /**
     * ditto to getIntValue, except that def is returned if no vakue entered
     *
     * @return
     */
    public int getIntValue(int def) {
        String text = getText();

        if ((text == null) || (text.length() == 0)) {
            return def;
        } else {
            return Integer.parseInt(text);

        }
    }

    public void setValue(Object obj) {
        Integer i;

        if (obj == null) {
            i = new Integer(Integer.MIN_VALUE); // better than exceptioning out
        } else if (obj instanceof Float) {
            i = (Integer) obj;
        } else {
            i = new Integer(obj.toString());

        }

        this.setText(i.toString());
    }

    public JComponent getComponent() {
        return this;
    }


}    // End IntegerTextField

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
