/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.swing.fields;


import javax.swing.*;
import javax.swing.text.Document;

/**
 * Subclass of swing's JPasswordField so that we can implement
 * GFieldPlusChooser and hence be generated by the FieldFactory.
 * In all other respects, identical behaviour to JPasswordField.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class GPasswordField2 extends javax.swing.JPasswordField implements GFieldPlusChooser {

    /**
     * Constructs a new GPasswordField2, with a default document, null starting
     * text string, and 0 column width.
     */
    public GPasswordField2() {
        super();
    }

    /**
     * Constructs a new JPasswordField that uses the given text storage
     * model and the given number of columns.  This is the constructor
     * through which the other constructors feed.  The echo character is
     * set to '*'.  If the document model is null, a default one will be
     * created.
     *
     * @param doc     the text storage to use
     * @param txt     the text to be displayed, null if none
     * @param columns the number of columns to use to calculate
     *                the preferred width >= 0.  If columns is set to zero, the
     *                preferred width will be whatever naturally results from
     *                the component implementation.
     */
    public GPasswordField2(Document doc, String txt, int columns) {
        super(doc, txt, columns);
    }

    /**
     * Constructs a new empty JPasswordField with the specified
     * number of columns.  A default model is created, and the initial string
     * is set to null.
     *
     * @param columns the number of columns >= 0
     */
    public GPasswordField2(int columns) {
        super(columns);
    }

    /**
     * Constructs a new JPasswordField initialized with the specified text.
     * The document model is set to the default, and the number of columns to 0.
     *
     * @param text the text to be displayed, null if none
     */
    public GPasswordField2(String text) {
        super(text);
    }

    /**
     * Constructs a new JPasswordField initialized with the specified text
     * and columns.  The document model is set to the default.
     *
     * @param text    the text to be displayed, null if none
     * @param columns the number of columns >= 0
     */
    public GPasswordField2(String text, int columns) {
        super(text, columns);
    }

    /**
     * Wrapper for the GFieldPlusChooser interface around JPasswordField.getPassword()
     *
     * @return character array with the password.
     */
    public Object getValue() {
        return getPassword();
    }

    public void setValue(Object obj) {
        String p;

        if (obj == null) {
            throw new IllegalArgumentException("Param obj cannot be null");
        } else {
            p = obj.toString();
        }

        super.setText(p);
    }

    /**
     * GFieldPlusChooser impl.
     *
     * @return This instance of GPasswordField2
     */
    public JComponent getComponent() {
        return this;
    }

}    // End GPasswordField2

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
