/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.swing.fields;


import javax.swing.*;
import java.awt.event.KeyEvent;


/**
 * A JTextField subclass that accepts only letters and "."
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class GStringField extends JTextField implements GFieldPlusChooser {

    /**
     * Class Constructor.
     * Constructs a new GStringField.  A default model is created, the initial
     * string is null, and the number of columns is set to 0.
     */
    public GStringField() {
        super();
    }

    /**
     * Class Constructor.
     * <p/>
     * Constructs a new GStringField initialized with the specified Integer.
     * A default model is created and the number of columns is 0.
     *
     * @param s the String to be displayed, or null
     */
    public GStringField(String s) {
        super(s);
    }

    /**
     * Class Constructor.
     * <p/>
     * Constructs a new empty GStringField with the specified number of columns.
     * A default model is created and the initial string is set to null.
     *
     * @param columns the number of columns to use to calculate
     *                the preferred width.  If columns is set to zero, the
     *                preferred width will be whatever naturally results from
     *                the component implementation.
     */
    public GStringField(int cols) {
        super(cols);
    }

    /**
     * @return The user specified String
     */
    public Object getValue() {
        return this.getText();
    }

    public void setValue(Object obj) {

        if (obj == null) {
            super.setText(null);
        } else {
            super.setText(obj.toString());
        }

    }

    /**
     * @return The user specified String
     */
    public String getString() {
        return this.getText();
    }

    public JComponent getComponent() {
        return this;
    }

    /**
     * Does the string-enforcement.
     */
    public void processKeyEvent(KeyEvent ev) {

        char c = ev.getKeyChar();

        if (Character.isDigit(c) && !ev.isAltDown()) {
            ev.consume();

            return;
        }

        /*
        if((Character.isDigit(c) && !ev.isAltDown()) || Strings.badchars().pattern().indexOf(c) > -1) {
            ev.consume();
            return;
        }
        */
        if ((c == '-') && (getDocument().getLength() > 0)) {
            ev.consume();
        } else {
            super.processKeyEvent(ev);
        }
    }
}    // End IntegerTextField

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
