/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.swing.windows;


import edu.mit.broad.genome.swing.GBase;
import edu.mit.broad.genome.swing.GuiHelper;
import edu.mit.broad.xbench.core.api.DialogDescriptor;

import javax.swing.*;
import java.awt.*;


/**
 * JList that pops up in a new window by itself with 2 buttons - cancel and accept
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class GListWindow extends GBase {

    private final JList jlOptions = new JList();
    private DefaultListModel fModel = new DefaultListModel();
    private int fSelectionMode = ListSelectionModel.SINGLE_SELECTION;

    private Action fHelp_action_opt;

    /**
     * comp on north of the dialog desc box optional
     */
    private JComponent fNorthComponent;

    /**
     * Class Constructor.
     */
    public GListWindow(final Action help_action_opt) {
        init(help_action_opt);
    }

    /**
     * Class Constructor.
     *
     * @param options
     */
    public GListWindow(final Object[] options, final Action help_action_opt) {
        init(help_action_opt);
        fillModel(options);
    }

    /**
     * @param model
     */
    public GListWindow(final DefaultListModel model, final Action help_action_opt) {
        init(help_action_opt);
        fModel = model;
    }

    private void init(final Action help_action_opt) {

        //klog.debug("^^^^^^^ " + help_action_opt);
        //TraceUtils.showTrace();

        this.fHelp_action_opt = help_action_opt;

        // as a convenince lets make a double click (or enter key) on the jlist == an attach button press
        //GuiHelper.Event.enableDoubleClick(jlOptions, fActionListener);

    }

    public void setHelpAction(final Action help_action_opt) {
        this.fHelp_action_opt = help_action_opt;
    }

    public void setListModel(final DefaultListModel model) {
        this.fModel = model;
    }

    public void setOptions(final Object[] options) {
        fillModel(options);
    }

    public JList getJList() {
        return jlOptions;
    }

    public void setRenderer(final DefaultListCellRenderer rend) {
        jlOptions.setCellRenderer(rend);
    }

    /**
     * selectedOnes can be null
     *
     * @param options
     * @param selectedOnes
     * @return
     */
    public Object[] show(Object[] options, Object[] selectedOnes) {
        fillModel(options);
        return _show(selectedOnes);
    }

    /**
     * Model already filled
     *
     * @param selectedOnes
     * @return
     */
    public Object[] selectAndShow(Object[] selectedOnes) {
        if (selectedOnes.length > 1) {
            this.setListSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        }
        return _show(selectedOnes);
    }

    /**
     * Model must already filled
     *
     * @return
     */
    public Object[] show() {
        return _show(new Object[]{});
    }

    // does NOT show window (ie window already showing -- used in conjunction with the nrothcomponent thing)
    public void updateData(Object[] options, Object[] selectedObjects) {
        fillModel(options);
        // carefull with rebuild / reset the model here -> that ruins the selection policy
        jlOptions.setModel(fModel);
        jlOptions.setSelectionMode(fSelectionMode);
        GuiHelper.List2.setSelected(selectedObjects, jlOptions, fModel);
    }

    public Object[] showDirectlyWithModel(ListModel model) {

        // carefull with rebuild / reset the model here -> that ruins the selection policy
        jlOptions.setModel(model);
        jlOptions.setSelectionMode(fSelectionMode);
        return _just_show();
    }

    /**
     * @return value selected or null is user cancelled
     */
    private Object[] _show(final Object[] selectedOnes) {

        // carefull with rebuild / reset the model here -> that ruins the selection policy
        jlOptions.setModel(fModel);
        jlOptions.setSelectionMode(fSelectionMode);

        GuiHelper.List2.setSelected(selectedOnes, jlOptions, fModel);

        return _just_show();
    }

    // just the showing part, abs no setting data
    private Object[] _just_show() {

        String text = "Select an option";

        if (fSelectionMode == ListSelectionModel.MULTIPLE_INTERVAL_SELECTION) {
            text = "Select one or more option(s)";
        }

        /*
        DialogDescriptor desc;

        if (fNorthComponent == null) {
            desc = new DialogDescriptor(text, new JScrollPane(jlOptions));
        } else {
            JPanel panel = new JPanel(new BorderLayout());
            panel.add(fNorthComponent, BorderLayout.NORTH);
            panel.add(new JScrollPane(jlOptions), BorderLayout.CENTER);
            desc = new DialogDescriptor(text, panel);
        }

        desc.enableDoubleClickableJList(jlOptions);
        int res = desc.show();

        desc.dispose();
        if (res == DialogDescriptor.CANCEL_OPTION) {
            return null;
        } else {
            return jlOptions.getSelectedValues();    // @note value*s*
        }

        */

        DialogDescriptor desc;

        if (fNorthComponent == null) {
            desc = createDialogDescriptor(text, new JScrollPane(jlOptions), fHelp_action_opt);
        } else {
            JPanel panel = new JPanel(new BorderLayout());
            panel.add(fNorthComponent, BorderLayout.NORTH);
            panel.add(new JScrollPane(jlOptions), BorderLayout.CENTER);
            desc = createDialogDescriptor(text, panel, fHelp_action_opt);
        }

        desc.enableDoubleClickableJList(jlOptions);
        int res = desc.show();

        if (res == DialogDescriptor.CANCEL_OPTION) {
            return null;
        } else {
            return jlOptions.getSelectedValues();    // @note value*s*
        }

    }

    public void setDialogWindowNorthComponent(final JComponent component) {
        this.fNorthComponent = component;
    }

    private void fillModel(Object[] options) {
        fModel = new DefaultListModel();

        for (int i = 0; i < options.length; i++) {
            fModel.addElement(options[i]);
        }
    }

    /**
     * @param mode one of the ListSelectionModel constants
     */
    public void setListSelectionMode(int mode) {

        this.fSelectionMode = mode;

        jlOptions.setSelectionMode(fSelectionMode);
    }

}        // End GListWindow

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
