/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.utils;

import edu.mit.broad.genome.TraceUtils;
import edu.mit.broad.genome.WebException;
import org.apache.ecs.Element;
import org.apache.ecs.html.TD;
import org.apache.ecs.html.TR;
import org.apache.ecs.html.Table;


/**
 * @author Brian Goetz
 * @author extended by Aravind Subramanian
 * @version %I%, %G%
 * @see http://www.javaworld.com/javaworld/jw-09-2001/jw-0914-exceptions-p2.html
 */
public class ChainedException extends WebException {

    private Throwable cause = null;

    /**
     * Class constructor
     *
     * @param message
     */
    public ChainedException(String message) {
        super(message);
    }

    /**
     * Class constructor
     *
     * @param message
     * @param cause
     */
    public ChainedException(String message, Throwable cause) {
        super(message);
        this.cause = cause;
    }

    /**
     * Class constructor
     *
     * @param cause
     */
    public ChainedException(Throwable cause) {
        this(cause.getMessage(), cause);
    }

    public Throwable getCause() {
        return cause;
    }

    public void printStackTrace() {

        super.printStackTrace();

        if (cause != null) {
            System.err.println("Caused by:");
            cause.printStackTrace();
        }
    }

    public void printStackTrace(java.io.PrintStream ps) {

        super.printStackTrace(ps);

        if (cause != null) {
            ps.println("Caused by:");
            cause.printStackTrace(ps);
        }
    }

    public void printStackTrace(java.io.PrintWriter pw) {

        super.printStackTrace(pw);

        if (cause != null) {
            pw.println("Caused by:");
            cause.printStackTrace(pw);
        }
    }

    public Element createHtmlErrorMessage() {

        Table table = new Table();
        table.setBorder(2);

        TR tr = new TR();
        tr.addElement(new TD("Message"));
        tr.addElement(new TD(super.getMessage()));
        table.addElement(tr);

        tr = new TR();
        tr.addElement(new TD("Details"));
        tr.addElement(new TD(TraceUtils.getAsString(this)));
        table.addElement(tr);

        return table;

    }
}

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
