/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.utils;


import java.lang.reflect.Modifier;
import java.util.StringTokenizer;


/**
 * Class instantiation, class type checking and java.lang.reflec related utiltities.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ClassUtils {

    /**
     * Runs checks to see if the specified class can be instantiated (without actually instantiating
     * servbes as a dbc check.,
     * faster than instaiting generally
     * I'm not sure if this is a thorough check but it should at least catch the obvious errors.
     * Needs the class to be public
     */
    public static boolean isInstantiable(Class clazz) {

        int mod = clazz.getModifiers();

        if (Modifier.isAbstract(mod)) {
            return false;
        }

        if (Modifier.isInterface(mod)) {
            return false;
        }

        if (!Modifier.isPublic(mod)) {
            return false;
        }

        return true;
    }

    /**
     * @return The short name of clazz by stripping off the package name.
     *         For example: the short name of javax.swing.JTable.class
     *         is "JTable"
     */
    public static String shorten(Class clazz) {
        return shorten(clazz.getName());
    }

    public static String shorten(String name) {

        StringTokenizer tok = new StringTokenizer(name, ".");
        int cnt = tok.countTokens();

        for (int i = 0; i < cnt - 1; i++) {
            tok.nextToken();
        }

        return tok.nextToken();
    }

    public static String shorten(Object obj) {

        if (obj == null) {
            return null;
        }

        return shorten(obj.getClass());
    }

    /**
     * Turns edu.mit.broad.genome.objects.Dataset into Dataset
     * MP if no period (.") found, returns pkgname specified untouched
     *
     * @param pkgname
     * @return
     */
    public static String packageName2ClassName(String pkgname) {

        if (pkgname == null) {
            throw new IllegalArgumentException("Param pkgname cannot be null");
        }


        if (pkgname.indexOf(".") == -1) {
            return pkgname; // untouched
        }

        StringTokenizer tok = new StringTokenizer(pkgname, ".");
        String n = null;
        while (tok.hasMoreTokens()) {
            n = tok.nextToken();
        }

        return n;

    }

    /**
     * A Class.toString() yields, Class edu.mit.....Foo
     * But to do a Class.forName, the method needs just the class name -
     * For example the method truncates a string from:
     *  "class edu/mit/broad/genome/ JarResources"
     *  to
     *  JarResources
     *
     * If class prefix not found, no truncation done, no errors thrown either.
     */
    /*
    public static String trim(String name) {

        StringTokenizer tok = new StringTokenizer(name, " ");

        if(tok.countTokens() == 1) {
            return name;
        } else {
            tok.nextToken();

            return tok.nextToken();
        }

        //int indx = name.lastIndexOf("class");
        //if (indx == -1) return name;
        //else return name.substring(indx, name.length());
    }
    */
}    // End ClassUtils

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
