/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.utils;


import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;

import java.io.ObjectStreamException;
import java.io.Serializable;
import java.util.*;


/**
 * A set of {@link Collection} related utility methods.
 * <p/>
 * Adapted from: package org.apache.commons.collections;
 * <p/>
 * <bold>xomics Note:</bold><br>
 * Before writing a new method to this class check the jakarta-commons
 * website to see if there is a new version.
 * <p/>
 * <p/>
 * Some code from:
 * Trove - Copyright (c) 1997-2000 Walt Disney Internet Group
 * For more information about Tea, please see http://opensource.go.com/.
 *
 * @author Rodney Waldhoff
 * @author Aravind Subramanian (adpated for xomics and a few wi methods)
 * @version %I%, %G%
 */
public class CollectionUtils {

    /**
     * For logging support
     */
    private static final Logger klog = XLogger.getLogger(CollectionUtils.class);
    public static final Enumeration EMPTY_ENUMERATION = new EmptyEnum();

    /**
     * This Map is already provided in JDK1.3.
     */
    public static final Map EMPTY_MAP = new EmptyMap();


    public static void removeAll_case_insensitive(List removeFromThis, List removeThese) {
        removeFromThis.removeAll(findMatches_case_insensitive(removeFromThis, removeThese));
    }

    public static void removeAll_case_insensitive(Set removeFromThis, Set removeThese) {
        removeFromThis.removeAll(findMatches_case_insensitive(removeFromThis, removeThese));
    }

    public static Set findMatches_case_insensitive(Set toThis, Set fromThese) {
        return findMatches_case_insensitive(new ArrayList(toThis), new ArrayList(fromThese));
    }

    // This is the core method
    // Trouble if 2 elements with diff cases
    public static Set findMatches_case_insensitive(List toThis, List fromThese) {
        List toThis_uc = toUpperCase(toThis);

        Set matches_native_case = new HashSet();
        for (int i = 0; i < fromThese.size(); i++) {
            String a = fromThese.get(i).toString().toUpperCase();
            if (toThis_uc.contains(a)) {
                int index = toThis_uc.indexOf(a);
                matches_native_case.add(toThis.get(index)); // native case
            }
        }

        return matches_native_case;
    }

    public static List toUpperCase(List list) {

        for (int i = 0; i < list.size(); i++) {
            Object obj = list.get(i);
            if (obj != null) {
                list.set(i, obj.toString().toUpperCase());
            }
        }

        return list;
    }

    public static Set toUpperCase(Set set) {

        Set uc = new HashSet();
        for (Iterator iterator = set.iterator(); iterator.hasNext();) {
            Object key = iterator.next();
            uc.add(key.toString().toUpperCase());
        }

        return uc;
    }

    public static Set toLowerCase(Set set) {

        Set uc = new HashSet();
        for (Iterator iterator = set.iterator(); iterator.hasNext();) {
            Object key = iterator.next();
            uc.add(key.toString().toLowerCase());
        }

        return uc;
    }

    public static List toLowerCase(List list) {

        for (int i = 0; i < list.size(); i++) {
            Object obj = list.get(i);
            if (obj != null) {
                list.set(i, obj.toString().toLowerCase());
            }
        }

        return list;
    }


    public static Set toSet(String[] names) {
        if (names == null) {
            return new HashSet();
        }

        Set set = new HashSet(names.length);
        for (int i = 0; i < names.length; i++) {
            set.add(names[i]);
        }

        return set;
    }

    public static Map mergeStringValues(final Map a, final Map b, String delim) {
        Map combo = new HashMap();
        Map bMirror = new HashMap(b);

        Iterator it = a.keySet().iterator();
        while (it.hasNext()) {
            Object key = it.next();
            Object valA = a.get(key);
            StringBuffer buf = new StringBuffer();
            if (valA != null) {
                buf.append(valA.toString()).append(delim);
            }

            if (bMirror.containsKey(key)) {
                Object valB = bMirror.get(key);
                if (valB != null) {
                    buf.append(valB.toString()).append(delim);
                }
                bMirror.remove(key);
            }

            combo.put(key, buf.toString());
        }

        it = bMirror.keySet().iterator();
        while (it.hasNext()) {
            Object key = it.next();
            Object val = bMirror.get(key);
            if (a.containsKey(key)) {
                throw new IllegalStateException("Error - not possible key: " + key);
            }
            StringBuffer buf = new StringBuffer();
            if (val != null) {
                buf = buf.append(val.toString()).append(delim);
            }
            combo.put(key, buf.toString());
        }

        trimStringValues(combo);
        return combo;
    }

    public static void trimStringValues(Map map) {
        Iterator it = map.keySet().iterator();
        while (it.hasNext()) {
            Object key = it.next();
            Object val = map.get(key);
            if (val != null) {
                String str = val.toString();
                str = str.trim();
                map.put(key, val);
            }
        }
    }

    public static List createSeriesStrings(int start, int stop) {
        List list = new ArrayList(stop - start);
        for (int i = start; i < stop; i++) {
            list.add(i + "");
        }

        return list;
    }

    public static List createSeriesStrings(int start, int stop, String commonPrefix) {
        List list = new ArrayList(stop - start);
        for (int i = start; i < stop; i++) {
            list.add(commonPrefix + i);
        }

        return list;
    }

    /**
     * Similar to EMPTY_MAP, except the put operation is supported, but the
     * values are immediately forgotten.
     */
    public static final Map VOID_MAP = new VoidMap();

    public static String toStringTabDelimited(Collection coll) {
        StringBuffer buf = new StringBuffer();
        Iterator it = coll.iterator();
        while (it.hasNext()) {
            buf.append(it.next()).append('\t');
        }

        buf.append('\n');
        return buf.toString();
    }


    /**
     * size of enum
     */
    public static int size(Enumeration en) {

        int cnt = 0;

        while (en.hasMoreElements()) {
            en.nextElement();

            cnt++;
        }

        return cnt;
    }

    /**
     * Move specified Object Up by 1 index in the specified List.
     * Uncertain if it will work if > instance of same object in the list.
     * <p/>
     * <pre>
     * Example, element at index = 2, gets moved to index = 1
     * </pre>
     * <p/>
     * This method is boundary-safe - when an object reaches the top of the List
     * it is not moved up anymore.
     *
     * @return the new index of Object a in the List
     */
    public static int moveUp(Object a, List list) {

        int curr = list.indexOf(a);

        if (curr == -1) {
            throw new IllegalArgumentException("Specified object is not in the list obj=" + a);
        }

        if (curr > 0) {
            Object b = list.get(curr - 1);

            list.set(curr - 1, a);
            list.set(curr, b);
        }

        return curr - 1;
    }

    /**
     * Moves the specified object down by 1 index in the specified List.
     * Uncertain if it will work if > instance of same object in the list.
     * <p/>
     * <pre>
     * Example, element at index = 2, gets moved to index = 3
     * </pre>
     * <p/>
     * This method is boundary-safe - when an object reaches the bottom of the List
     * it is not moved down anymore.
     *
     * @return the new index of Object a in the List
     */
    public static int moveDown(Object a, List list) {

        int curr = list.indexOf(a);

        if (curr == -1) {
            throw new IllegalArgumentException("Specified object is not in the list obj=" + a);
        }

        if (curr < list.size() - 1) {
            Object b = list.get(curr + 1);

            list.set(curr + 1, a);
            list.set(curr, b);
        }

        return curr + 1;
    }

    /**
     * Use with care on big lists
     */
    public static void printf(List list) {

        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < list.size(); i++) {
            buf.append(list.get(i)).append(", ");

            if ((i != 0) && (i % 10 == 0)) {
                buf.append("\n");
            }
        }

        System.out.println("----- Number of elements=" + list.size());
        System.out.println(buf.toString());
    }

    /**
     * Returns a {@link Collection} containing the union
     * of the given {@link Collection}s.
     * <p/>
     * The cardinality of each element in the returned {@link Collection}
     * will be equal to the maximum of the cardinality of that element
     * in the two given {@link Collection}s.
     *
     * @see Collection#addAll
     */
    public static Collection union(final Collection a, final Collection b) {

        ArrayList list = new ArrayList();
        Map mapa = getCardinalityMap(a);
        Map mapb = getCardinalityMap(b);
        Set elts = new HashSet(a);

        elts.addAll(b);

        Iterator it = elts.iterator();

        while (it.hasNext()) {
            Object obj = it.next();

            for (int i = 0, m = Math.max(getFreq(obj, mapa), getFreq(obj, mapb)); i < m; i++) {
                list.add(obj);
            }
        }

        return list;
    }

    // as
    public static Set unionSet(Collection[] colls) {

        Set union = new HashSet();
        for (int i = 0; i < colls.length; i++) {
            union.addAll(colls[i]);
        }

        return union;
    }

    /**
     * Returns a {@link Collection} containing the intersection
     * of the given {@link Collection}s.
     * <p/>
     * The cardinality of each element in the returned {@link Collection}
     * will be equal to the minimum of the cardinality of that element
     * in the two given {@link Collection}s.
     *
     * @see Collection#retainAll
     */
    public static Collection intersection(final Collection a, final Collection b) {

        ArrayList list = new ArrayList();
        Map mapa = getCardinalityMap(a);
        Map mapb = getCardinalityMap(b);
        Set elts = new HashSet(a);

        elts.addAll(b);

        Iterator it = elts.iterator();

        while (it.hasNext()) {
            Object obj = it.next();

            for (int i = 0, m = Math.min(getFreq(obj, mapa), getFreq(obj, mapb)); i < m; i++) {
                list.add(obj);
            }
        }

        return list;
    }

    /**
     * Returns a {@link Collection} containing the exclusive disjunction
     * (symmetric difference) of the given {@link Collection}s.
     * <p/>
     * The cardinality of each element <i>e</i> in the returned {@link Collection}
     * will be equal to
     * <tt>max(cardinality(<i>e</i>,<i>a</i>),cardinality(<i>e</i>,<i>b</i>)) - min(cardinality(<i>e</i>,<i>a</i>),cardinality(<i>e</i>,<i>b</i>))</tt>.
     * <p/>
     * This is equivalent to
     * <tt>{@link #subtract subtract}({@link #union union(a,b)},{@link #intersection intersection(a,b)})</tt>
     * or
     * <tt>{@link #union union}({@link #subtract subtract(a,b)},{@link #subtract subtract(b,a)})</tt>.
     */
    public static Collection disjunction(final Collection a, final Collection b) {

        ArrayList list = new ArrayList();
        Map mapa = getCardinalityMap(a);
        Map mapb = getCardinalityMap(b);
        Set elts = new HashSet(a);

        elts.addAll(b);

        Iterator it = elts.iterator();

        while (it.hasNext()) {
            Object obj = it.next();

            for (int i = 0,
                    m = ((Math.max(getFreq(obj, mapa), getFreq(obj, mapb)))
                            - (Math.min(getFreq(obj, mapa), getFreq(obj, mapb))));
                 i < m; i++) {
                list.add(obj);
            }
        }

        return list;
    }

    /**
     * Returns a {@link Collection} containing <tt><i>a</i> - <i>b</i></tt>.
     * The cardinality of each element <i>e</i> in the returned {@link Collection}
     * will be the cardinality of <i>e</i> in <i>a</i> minus the cardinality
     * of <i>e</i> in <i>b</i>, or zero, whichever is greater.
     *
     * @see Collection#removeAll
     */
    public static Collection subtract(final Collection a, final Collection b) {

        ArrayList list = new ArrayList(a);
        Iterator it = b.iterator();

        while (it.hasNext()) {
            list.remove(it.next());
        }

        return list;
    }

    /**
     * Returns a {@link Map} mapping each unique element in
     * the given {@link Collection} to an {@link Integer}
     * representing the number of occurances of that element
     * in the {@link Collection}.
     * An entry that maps to <tt>null</tt> indicates that the
     * element does not appear in the given {@link Collection}.
     */
    public static Map getCardinalityMap(final Collection col) {

        HashMap count = new HashMap();
        Iterator it = col.iterator();

        while (it.hasNext()) {
            Object obj = it.next();
            Integer c = (Integer) (count.get(obj));

            if (null == c) {
                count.put(obj, new Integer(1));
            } else {
                count.put(obj, new Integer(c.intValue() + 1));
            }
        }

        return count;
    }

    /**
     * Returns <tt>true</tt> iff <i>a</i> is a sub-collection of <i>b</i>,
     * that is, iff the cardinality of <i>e</i> in <i>a</i> is less
     * than or equal to the cardinality of <i>e</i> in <i>b</i>,
     * for each element <i>e</i> in <i>a</i>.
     *
     * @see #isProperSubCollection
     * @see Collection#containsAll
     */
    public static boolean isSubCollection(final Collection a, final Collection b) {

        ArrayList list = new ArrayList();
        Map mapa = getCardinalityMap(a);
        Map mapb = getCardinalityMap(b);
        Iterator it = a.iterator();

        while (it.hasNext()) {
            Object obj = it.next();

            if (getFreq(obj, mapa) > getFreq(obj, mapb)) {
                return false;
            }
        }

        return true;
    }

    /**
     * Returns <tt>true</tt> iff <i>a</i> is a <i>proper</i> sub-collection of <i>b</i>,
     * that is, iff the cardinality of <i>e</i> in <i>a</i> is less
     * than or equal to the cardinality of <i>e</i> in <i>b</i>,
     * for each element <i>e</i> in <i>a</i>, and there is at least one
     * element <i>f</i> such that the cardinality of <i>f</i> in <i>b</i>
     * is strictly greater than the cardinality of <i>f</i> in <i>a</i>.
     *
     * @see #isSubCollection
     * @see Collection#containsAll
     */
    public static boolean isProperSubCollection(final Collection a, final Collection b) {

        // XXX optimize me!
        return CollectionUtils.isSubCollection(a, b)
                && (!(CollectionUtils.isEqualCollection(a, b)));
    }

    /**
     * Returns <tt>true</tt> iff the given {@link Collection}s contain
     * exactly the same elements with exactly the same cardinality.
     * <p/>
     * That is, iff the cardinality of <i>e</i> in <i>a</i> is
     * equal to the cardinality of <i>e</i> in <i>b</i>,
     * for each element <i>e</i> in <i>a</i> or <i>b</i>.
     */
    public static boolean isEqualCollection(final Collection a, final Collection b) {

        if (a.size() != b.size()) {
            return false;
        } else {
            Map mapa = getCardinalityMap(a);
            Map mapb = getCardinalityMap(b);

            if (mapa.size() != mapb.size()) {
                return false;
            } else {
                Iterator it = mapa.keySet().iterator();

                while (it.hasNext()) {
                    Object obj = it.next();

                    if (getFreq(obj, mapa) != getFreq(obj, mapb)) {
                        return false;
                    }
                }

                return true;
            }
        }
    }

    /**
     * Returns the number of occurrences of <i>obj</i>
     * in <i>col</i>.
     */
    public static int cardinality(Object obj, final Collection col) {

        int count = 0;
        Iterator it = col.iterator();

        while (it.hasNext()) {
            Object elt = it.next();

            if (((null == obj) && (null == elt)) || obj.equals(elt)) {
                count++;
            }
        }

        return count;
    }

    /**
     * Finds the first element in the given collection which matches the given predicate
     *
     * @return the first element of the collection which matches the predicate or null if none could be found
     */
    public static Object find(Collection collection, Predicate predicate) {

        if ((collection != null) && (predicate != null)) {
            for (Iterator iter = collection.iterator(); iter.hasNext();) {
                Object item = iter.next();

                if (predicate.evaluate(item)) {
                    return item;
                }
            }
        }

        return null;
    }


    /**
     * Selects all elements from inputCollection which match the given predicate
     * into an output collection
     */
    public static Collection select(Collection inputCollection, Predicate predicate) {

        ArrayList answer = new ArrayList(inputCollection.size());

        select(inputCollection, predicate, answer);

        return answer;
    }

    /**
     * Selects all elements from inputCollection which match the given predicate
     * and adds them to outputCollection
     */
    public static void select(Collection inputCollection, Predicate predicate,
                              Collection outputCollection) {

        if ((inputCollection != null) && (predicate != null)) {
            for (Iterator iter = inputCollection.iterator(); iter.hasNext();) {
                Object item = iter.next();

                if (predicate.evaluate(item)) {
                    outputCollection.add(item);
                }
            }
        }
    }

    /**
     * Adds all elements in the iteration to the given collection
     */
    public static void addAll(Collection collection, Iterator iterator) {

        while (iterator.hasNext()) {
            collection.add(iterator.next());
        }
    }

    /**
     * Adds all elements in the enumeration to the given collection
     */
    public static void addAll(Collection collection, Enumeration enumeration) {

        while (enumeration.hasMoreElements()) {
            collection.add(enumeration.nextElement());
        }
    }

    /**
     * Adds all elements in the array to the given collection
     */
    public static void addAll(Collection collection, Object[] elements) {

        for (int i = 0, size = elements.length; i < size; i++) {
            collection.add(elements[i]);
        }
    }

    private static final int getFreq(final Object obj, final Map freqMap) {

        try {
            return ((Integer) (freqMap.get(obj))).intValue();
        } catch (NullPointerException e) {

            // ignored
        } catch (NoSuchElementException e) {

            // ignored
        }

        return 0;
    }

    public static Collection findCommon(Collection a, Collection b) {
        Set common = new HashSet();

        Iterator it = b.iterator();
        while (it.hasNext()) {

            Object bn = it.next();
            if (a.contains(bn)) {
                common.add(bn);
            }
        }

        return common;
    }

    public static Set findCommon(Set a, Set b) {
        return (Set) findCommon((Collection) a, (Collection) b);
    }

    public static int findNumCommon(Set a, Set b) {

        int num = 0;

        Iterator it = b.iterator();
        while (it.hasNext()) {

            Object bn = it.next();
            if (a.contains(bn)) {
                num++;
            }
        }

        return num;

    }


    /**
     * Class EmptyEnum
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    private static class EmptyEnum implements Enumeration, Serializable {

        public boolean hasMoreElements() {
            return false;
        }

        public Object nextElement() throws NoSuchElementException {
            throw new NoSuchElementException();
        }

        // Serializable singleton classes should always do this.
        private Object readResolve() throws ObjectStreamException {
            return EMPTY_ENUMERATION;
        }
    }

    ;

    /**
     * Class EmptyMap
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    private static class EmptyMap implements Map, Serializable {

        public int size() {
            return 0;
        }

        public boolean isEmpty() {
            return true;
        }

        public boolean containsKey(Object key) {
            return false;
        }

        public boolean containsValue(Object value) {
            return false;
        }

        public Object get(Object key) {
            return null;
        }

        public Object put(Object key, Object value) {
            throw new UnsupportedOperationException("Cannot put into immutable empty map");
        }

        public Object remove(Object key) {
            return null;
        }

        public void putAll(Map map) {
            throw new UnsupportedOperationException("Cannot put into immutable empty map");
        }

        public void clear() {
        }

        public Set keySet() {
            return Collections.EMPTY_SET;
        }

        public Collection values() {
            return Collections.EMPTY_LIST;
        }

        public Set entrySet() {
            return Collections.EMPTY_SET;
        }

        // Serializable singleton classes should always do this.
        private Object readResolve() throws ObjectStreamException {
            return EMPTY_MAP;
        }
    }

    /**
     * Class VoidMap
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    private static class VoidMap extends EmptyMap {

        public Object put(Object key, Object value) {
            return null;
        }

        public void putAll(Map map) {
        }

        // Serializable singleton classes should always do this.
        private Object readResolve() throws ObjectStreamException {
            return VOID_MAP;
        }
    }
}    // End CollectionUtils

/*--- Formatted in Sun Java Convention Style on Thu, Nov 7, '02 ---*/
