/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.utils;


import Acme.JPM.Encoders.GifEncoder;
// removed these in order to support compilation under jdk 7
// import com.sun.image.codec.jpeg.JPEGCodec;
// import com.sun.image.codec.jpeg.JPEGImageEncoder;
import com.sun.media.jai.codec.PNGEncodeParam;
import edu.mit.broad.genome.XLogger;
import edu.mit.broad.genome.swing.image.NamedImage;
import org.apache.log4j.Logger;

import javax.media.jai.JAI;
import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.image.*;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;


/**
 * <p> </p>
 * <p> </p>
 * <p> </p>
 * <p> </p>
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 * @see http://www.jalice.net for many usefule examples / hints
 */

public class ImageUtils {

    private static final Logger klog = XLogger.getLogger(ImageUtils.class);

    /**
     * Privatized Class Constructor.
     */
    private ImageUtils() {
    }

    public static BufferedImage createHeadlessNaImage() {
        BufferedImage bi = new BufferedImage(50, 50, 1);
        Graphics g = bi.getGraphics();
        g.setColor(Color.WHITE);
        g.fillRect(0, 0, bi.getWidth(), bi.getHeight());
        g.setColor(Color.BLACK);
        g.drawString("Images NA in Headless mode", 2, 10);
        g.dispose();
        return bi;
    }

    /**
     * Convert a BufferedImage into an Image
     * <p/>
     * with a filter
     */
    public static Image toImage(final BufferedImage bi) {

        AffineTransformOp aop = new AffineTransformOp(new AffineTransform(),
                AffineTransformOp.TYPE_BILINEAR);
        BufferedImageFilter filter = new BufferedImageFilter(aop);
        FilteredImageSource source = new FilteredImageSource(bi.getSource(), filter);
        return Toolkit.getDefaultToolkit().createImage(source);
    }

    /**
     * @param bi
     * @return
     */
    public static Image toImage2(final BufferedImage bi) {
        return Toolkit.getDefaultToolkit().createImage(bi.getSource());
    }

    /**
     * @param image
     * @param obs
     * @return
     */
    public static BufferedImage toBufferedImage(final Image image, final ImageObserver obs) {
        final int w = image.getWidth(obs);
        final int h = image.getHeight(obs);
        final BufferedImage bufferImage = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB_PRE);
        final Graphics2D context = bufferImage.createGraphics();
        context.drawImage(image, 0, 0, obs);
        return bufferImage;
    }

    /**
     * @param bi
     * @return
     */
    public static BufferedImage copy(final BufferedImage bi) {
        return bi.getSubimage(0, 0, bi.getWidth(), bi.getHeight());
    }

    /**
     * @param orig   The name of image file.
     * @param thumb  The name of thumbnail file.
     *               Will be created if necessary.
     * @param maxDim The width and height of
     *               the thumbnail must
     *               be maxDim pixels or less.
     * @see http://developer.java.sun.com/developer/TechTips/1999/tt1021.html
     *      <p/>
     *      Reads an image in a file and creates
     *      a thumbnail in another file.
     */
    public static BufferedImage createThumbnail(final float maxDim, final Image inImage) {

        // Determine the scale.
        double scale = (double) maxDim / (double) inImage.getHeight(null);

        if (inImage.getWidth(null) > inImage.getHeight(null)) {
            scale = (double) maxDim / (double) inImage.getWidth(null);
        }

        // Determine size of new image.
        //One of them
        // should equal maxDim.
        int scaledW = (int) (scale * inImage.getWidth(null));
        int scaledH = (int) (scale * inImage.getHeight(null));

        // Create an image buffer in
        //which to paint on.
        BufferedImage outImage = new BufferedImage(scaledW, scaledH, BufferedImage.TYPE_INT_RGB);

        // Set the scale.
        AffineTransform tx = new AffineTransform();

        // If the image is smaller than
        //the desired image size,
        // don't bother scaling.
        if (scale < 1.0d) {
            tx.scale(scale, scale);
        }

        // Paint image.
        Graphics2D g2d = outImage.createGraphics();

        g2d.drawImage(inImage, tx, null);
        g2d.dispose();

        return outImage;
    }

    /**
     * Doesnt work well for large images! -- use the GifEncoder instead
     * JPEG-encode the image and write to file.
     *
     * @param outImage
     * @param file
     * @throws IOException
     */
/* relies on classes not supported under jdk 7.  GSEA code never calls on method; hence OK to
 * remove.
    public static void saveJpeg(final BufferedImage outImage, final File file) throws IOException {
        OutputStream os = new FileOutputStream(file);
        JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(os);
        encoder.encode(outImage);
        os.close();
    }
*/

    public static void savePng(final BufferedImage outImage, final File file) {
        JAI.create("filestore", outImage, file.getPath(), "png", new PNGEncodeParam.RGB());
    }

    /**
     * @param outImage
     * @param file
     * @throws IOException
     */
    public static void saveGif(final Image outImage, final File file) throws IOException {

        FileOutputStream outfile = new FileOutputStream(file.getPath());
        GifEncoder encoder = new GifEncoder(outImage, outfile);

        encoder.encode();
        outfile.flush();
        outfile.close();
    }

    /**
     * @param top
     * @param bot
     * @return
     */
    public static BufferedImage appendVertically(final BufferedImage top, final BufferedImage bot) {

        int width = Math.max(top.getWidth(), bot.getWidth());
        int height = top.getHeight() + bot.getHeight();
        BufferedImage cumm = new BufferedImage(width, height, 1);
        Graphics2D g2d = cumm.createGraphics();

        g2d.drawImage(top, null, 0, 0);
        g2d.drawImage(bot, null, 0, top.getHeight());
        g2d.dispose();

        return cumm;
    }

    /**
     * @param left
     * @param right
     * @return
     */
    public static BufferedImage appendHorizontally(final BufferedImage left, final BufferedImage right) {

        int width = left.getWidth() + right.getWidth();
        int height = Math.max(left.getHeight(), right.getHeight());
        BufferedImage cumm = new BufferedImage(width, height, 1);
        Graphics2D g2d = cumm.createGraphics();

        g2d.drawImage(left, null, 0, 0);
        g2d.drawImage(right, null, left.getWidth(), 0);
        g2d.dispose();

        return cumm;
    }

    public static BufferedImage appendHorizontally(final NamedImage[] nis) {
        BufferedImage[] bufs = new BufferedImage[nis.length];
        for (int i = 0; i < bufs.length; i++) {
            bufs[i] = (BufferedImage) nis[i].getImage(); // @todo fix me
        }

        return appendHorizontally(bufs);
    }

    public static BufferedImage appendHorizontally(final BufferedImage[] bifs) {

        if (bifs == null) {
            throw new IllegalArgumentException("Param bifs cannot be null");
        }

        if (bifs.length == 0) {
            throw new IllegalArgumentException("Cannot make appended image as there are no input images: " + bifs.length);
        }

        int maxheight = findMaxHeight(bifs);
        int totalwidth = getTotalWidth(bifs);

        klog.info("Total width: " + totalwidth + " maxheight: " + maxheight);

        BufferedImage cumm = new BufferedImage(totalwidth, maxheight, 1);
        Graphics2D g2d = cumm.createGraphics();
        int x = 0;

        for (int i = 0; i < bifs.length; i++) {
            int w = bifs[i].getWidth();
            klog.debug(" Width: " + w + " height: " + bifs[i].getHeight());
            g2d.drawImage(bifs[i], null, x, 0);

            x += w;
        }

        g2d.dispose();

        return cumm;
    }

    /**
     * @param bifs
     * @return
     */
    public static int findMaxHeight(final BufferedImage[] bifs) {

        if (bifs == null) {
            throw new IllegalArgumentException("Param bifs cannot be null");
        }

        int max = 0;

        for (int i = 0; i < bifs.length; i++) {

            if (bifs[i] == null) {
                throw new IllegalArgumentException("bif cannot be null at: " + i);
            }

            if (bifs[i].getHeight() > max) {
                max = bifs[i].getHeight();
            }
        }

        return max;
    }

    /**
     * @param bifs
     * @return
     */
    public static int findMaxWidth(final BufferedImage[] bifs) {

        if (bifs == null) {
            throw new IllegalArgumentException("Param bifs cannot be null");
        }

        int max = 0;

        for (int i = 0; i < bifs.length; i++) {
            if (bifs[i].getWidth() > max) {
                max = bifs[i].getWidth();
            }
        }

        return max;
    }

    /**
     * @param bifs
     * @return
     */
    public static int getTotalWidth(final BufferedImage[] bifs) {

        if (bifs == null) {
            throw new IllegalArgumentException("Param bifs cannot be null");
        }

        int total = 0;

        for (int i = 0; i < bifs.length; i++) {
            total += bifs[i].getWidth();
        }

        return total;
    }

    /**
     * @param bifs
     * @return
     */
    public static int getTotalHeight(final BufferedImage[] bifs) {

        if (bifs == null) {
            throw new IllegalArgumentException("Param bifs cannot be null");
        }

        int total = 0;

        for (int i = 0; i < bifs.length; i++) {
            total += bifs[i].getHeight();
        }

        return total;
    }
}    // End ImageUtils

/*--- Formatted in Sun Java Convention Style on Wed, Oct 16, '02 ---*/
