/*
 * From: http://members.aol.com/MSchmelng/
 *
 * original package ViolinStrings;
 *
* @(#)Strings.java 1.00 2000/05/31
 *
 *
 *  Copyright notice:
 *
 * (C) Michael Schmeling 1998, 2000 - All Rights Reserved
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the author be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. Altered version of the source code or the class files must be stored
 *    with a different Java package name, so that users can clearly distinguish
 *    those altered versions from the original software and can use them safely
 *    together with the original software.
 * 4. This notice may not be removed or altered from any source distribution.
 *
 * Michael Schmeling
 * MSchmelng@aol.com
 *
 * If you use the library in a commercial product I would appreciate that
 * you send me a few lines with a short description of your product.
 * However, this is not a requirement.
 * The source is provided for free but without warranty of any kind.
 * The library has been entirely written by Michael Schmeling, it does not
 * contain third-party code.
 *
 * THE AUTHOR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
 * SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 *
 * Contributions to this package where made by:
 *    Eric Jablow
 *
 */

/**
 * @note For xomicsCombined several methods from pther packages (see below) to the
 * violin strings class to make even more of a monstor Strings class.
 * A doughnut one me for anyone who needs a String related method that isnt already
 * in this class.
 */

/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.utils;


import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileReader;
import java.io.PrintWriter;
import java.util.*;


/**
 * The <code>Strings</code> class provides a number of useful string manipulation
 * methods. It cannot be instantiated.
 * <p/>
 * You would typically use it in the following manner:
 * <p><blockquote><pre>
 *    String s = "abcdefg";
 *    String x = Strings.leftJustify(s, 20);
 * </pre></blockquote><p>
 * <p/>
 * <p/>
 * Some of the methods just call the equivalent methods from java.lang.String.
 * They are provided to collect all string methods in one place.
 *
 * @author Michael Schmeling
 * @version 0.90 02/12/98
 * @see String
 * @see StringBuffer
 * @see Character
 */
public class Strings {

    /**
     * Characters which are "generally" safe for naming files, etc. For example
     * chars such as spaces and , that some systems might barf at are avoided.
     */
    public final static String SAFE_NAME_CHARACTERS = "^[a-zA-Z\\_.\\-0-9]+$";

    protected Strings() {

        // No instantiation of this class permitted
    }

    /**
     * Checks if two characters are equal.
     * You can specify that case should be ignored.
     *
     * @param c1         first character
     * @param c2         second character
     * @param ignoreCase if true case is ignored when comparing characters
     * @return true if the characters are equal, false otherwise
     * @see java.lang.string#equalsIgnoreCase(String)
     */
    public static boolean equals(char c1, char c2, boolean ignoreCase) {

        if (ignoreCase) {
            return (Character.toUpperCase(c1) == Character.toUpperCase(c2))
                    || (Character.toLowerCase(c1) == Character.toLowerCase(c2));
        } else {
            return c1 == c2;
        }
    }

    /**
     * Checks if two strings are equal.
     * You can specify that case should be ignored.
     *
     * @param s1         first string
     * @param s2         second string
     * @param ignoreCase if true case is ignored when comparing characters
     * @return true if the strings are equal, false otherwise
     * @see java.lang.string#equalsIgnoreCase(String)
     */
    public static boolean equals(String s1, String s2, boolean ignoreCase) {

        if (ignoreCase) {
            return s1.equalsIgnoreCase(s2);
        } else {
            return s1.equals(s2);
        }
    }

    /**
     * Compares two strings.
     *
     * @param s1 the first string
     * @param s2 second string
     * @return 0 if the two strings are equal, a value less than zero if
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       <code>s1</code> is lexicographically less than <code>s2</code>,
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       and a value greater than
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       zero if <code>s1</code> is lexicographically greater than <code>s2</code>.
     * @see java.lang.string#compareTo(String)
     */
    public static int compare(String s1, String s2) {
        return s1.compareTo(s2);
    }

    /**
     * Returns true if the string starts with the specified prefix.
     *
     * @param s          the input string
     * @param prefix     the prefix
     * @param ignoreCase if true case is ignored when comparing characters
     * @return true if the string starts with the specified prefix.
     * @see java.lang.string#startsWith(String)
     * @see java.lang.string#regionMatches(boolean,int,String,int,int)
     */
    // working????
    public static boolean startsWith(String s, String prefix, boolean ignoreCase) {
        return s.regionMatches(ignoreCase, 0, prefix, 0, prefix.length());
    }

    /**
     * Returns true if the string starts with the specified prefix.
     *
     * @param s      the input string
     * @param prefix the prefix
     * @return true if the string starts with the specified prefix.
     * @see java.lang.string#startsWith(String)
     */
    public static boolean startsWith(String s, String prefix) {
        return s.startsWith(prefix);
    }

    /**
     * Returns true if the string starts with the specified prefix.
     *
     * @param s      the input string
     * @param prefix the prefix
     * @param start  the starting position for the test
     * @return true if the string starts with the specified prefix.
     * @see java.lang.string#startsWith(String,int)
     */
    public static boolean startsWith(String s, String prefix, int start) {
        return s.startsWith(prefix, start);
    }

    /**
     * Returns true if the string starts with the specified prefix.
     *
     * @param s          the input string
     * @param prefix     the prefix
     * @param start      the starting position for the test
     * @param ignoreCase if true case is ignored when comparing characters
     * @return true if the string starts with the specified prefix.
     * @see java.lang.string#startsWith(String,int)
     * @see java.lang.string#regionMatches(boolean,int,String,int,int)
     */
    public static boolean startsWith(String s, String prefix, int start, boolean ignoreCase) {
        return s.regionMatches(ignoreCase, start, prefix, 0, prefix.length());
    }

    /**
     * Returns true if the string ends with the specified suffix.
     *
     * @param s          the input string
     * @param suffix     the suffix
     * @param ignoreCase if true case is ignored when comparing characters
     * @return true if the string ends with the specified suffix.
     * @see java.lang.string#endsWith(String)
     * @see java.lang.string#regionMatches(boolean,int,String,int,int)
     */
    public static boolean endsWith(String s, String suffix, boolean ignoreCase) {

        int pos = s.length() - suffix.length();

        return s.regionMatches(ignoreCase, pos, suffix, 0, suffix.length());
    }

    /**
     * Returns true if the string ends with the specified suffix.
     *
     * @param s      the input string
     * @param suffix the suffix
     * @return true if the string ends with the specified suffix.
     * @see java.lang.string#endsWith(String)
     */
    public static boolean endsWith(String s, String suffix) {
        return s.endsWith(suffix);
    }

    /**
     * Returns the index of the first ocurrence of the specified string.
     *
     * @param s     the string to be inspected
     * @param c     the string to be searched for
     * @param start the starting position for the search
     * @return the index of the first ocurrence of the specified string.
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       If the specified string is not found -1 is returned.
     * @see java.lang.string#indexOf(String,int)
     */
    public static int indexOf(String s, String c, int start) {
        return s.indexOf(c, start);
    }

    /**
     * Returns the index of the first ocurrence of the specified string.
     * You can specify that case should be ignored.
     *
     * @param s          the string to be inspected
     * @param c          the string to be searched for
     * @param start      the starting position for the search
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the index of the first ocurrence of the specified string.
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       If the specified string is not found -1 is returned.
     * @see java.lang.string#indexOf(String,int)
     */
    public static int indexOf(String s, String c, int start, boolean ignoreCase) {

        if (ignoreCase) {
            int ls = s.length();
            int lc = c.length();
            int lc1 = lc - 1;
            int le = ls - lc;

            if ((lc == 0) || (start > le)) {
                return -1;
            }

            int ns = (start < 0)
                    ? 0
                    : start;
            char c0 = c.charAt(0);

            while (ns < le) {
                if (equals(s.charAt(ns), c0, ignoreCase)) {
                    if (s.regionMatches(ignoreCase, ns + 1, c, 1, lc1)) {
                        return ns;
                    }
                }

                ns++;
            }

            return -1;
        } else {
            return s.indexOf(c, start);
        }
    }

    /**
     * Returns the index of the first ocurrence of the specified string, starting
     * at the beginning of the string.
     * You can specify that case should be ignored.
     *
     * @param s          the string to be inspected
     * @param c          the string to be searched for
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the index of the first ocurrence of the specified string.
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       If the specified string is not found -1 is returned.
     */
    public static int indexOf(String s, String c, boolean ignoreCase) {
        return indexOf(s, c, 0, ignoreCase);
    }

    /**
     * Returns the index of the first ocurrence of the specified string.
     *
     * @param s the string to be inspected
     * @param c the string to be searched for
     * @return the index of the first ocurrence of the specified string.
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       If the specified string is not found -1 is returned.
     * @see java.lang.string#indexOf(String)
     */
    public static int indexOf(String s, String c) {
        return s.indexOf(c);
    }

    /**
     * Returns the index of the first ocurrence of the specified character.
     *
     * @param s the string to be inspected
     * @param c the character to be searched for
     * @return the index of the first ocurrence of the specified character.
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       If the specified character is not found -1 is returned.
     * @see java.lang.string#indexOf(char)
     */
    public static int indexOf(String s, char c) {
        return s.indexOf(c);
    }

    /**
     * Returns the index of the first ocurrence of the specified character, starting
     * at the specified position.
     * You can specify that case should be ignored.
     *
     * @param s          the string to be inspected
     * @param c          the character to be searched for
     * @param start      the starting position for the search
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the index of the first ocurrence of the specified character.
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       If the specified character is not found -1 is returned.
     * @see java.lang.string#indexOf(char,int)
     */
    public static int indexOf(String s, char c, int start, boolean ignoreCase) {

        if (ignoreCase) {
            int ls = s.length();

            if (start < 0) {
                start = 0;
            }

            if (start >= ls) {
                return -1;
            }

            for (int n = start; n < ls; n++) {
                if (equals(s.charAt(n), c, ignoreCase)) {
                    return n;
                }
            }

            return -1;
        } else {
            return s.indexOf(c, start);
        }
    }

    /**
     * Returns the index of the first ocurrence of the specified character.
     * You can specify that case should be ignored.
     *
     * @param s          the string to be inspected
     * @param c          the character to be searched for
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the index of the first ocurrence of the specified character.
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       If the specified character is not found -1 is returned.
     */
    public static int indexOf(String s, char c, boolean ignoreCase) {
        return indexOf(s, c, 0, ignoreCase);
    }

    /**
     * Returns the index of the first ocurrence of the specified character.
     *
     * @param s     the string to be inspected
     * @param c     the character to be searched for
     * @param start the starting position for the search
     * @return the index of the first ocurrence of the specified character.
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       If the specified character is not found -1 is returned.
     * @see java.lang.string#indexOf(char,int)
     */
    public static int indexOf(String s, char c, int start) {
        return s.indexOf(c, start);
    }

    /**
     * Returns the index of the first character that is contained in the
     * specified set of characters.
     *
     * @param s     the string to be inspected
     * @param c     the set of characters
     * @param start the starting position for the search
     * @return the index of the first character that is contained in the
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       specified set of characters.
     *         If no character from the string is in the specified set of
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       characters -1 is returned.
     */
    public static int indexOfAnyOf(String s, String c, int start) {

        int lc = c.length();
        int ls = s.length();
        int pos = ls;

        for (int n = 0; n < lc; n++) {
            char ch = c.charAt(n);
            int i = s.indexOf(ch, start);

            if ((i >= 0) && (i < pos)) {
                pos = i;
            }
        }

        return ((pos < ls)
                ? pos
                : -1);
    }

    /**
     * Returns the index of the first character that is contained in the
     * specified set of characters.
     * You can specify that case should be ignored.
     *
     * @param s          the string to be inspected
     * @param c          the set of characters
     * @param start      the starting position for the search
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the index of the first character that is contained in the
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       specified set of characters.
     *         If no character from the string is in the specified set of
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       characters -1 is returned.
     */
    public static int indexOfAnyOf(String s, String c, int start, boolean ignoreCase) {

        int lc = c.length();
        int ls = s.length();
        int pos = ls;

        for (int n = 0; n < lc; n++) {
            char ch = c.charAt(n);
            int i = indexOf(s, ch, start, ignoreCase);

            if ((i >= 0) && (i < pos)) {
                pos = i;
            }
        }

        return ((pos < ls)
                ? pos
                : -1);
    }

    /**
     * Returns the index of the first character that is contained in the
     * specified set of characters.
     * You can specify that case should be ignored.
     *
     * @param s          the string to be inspected
     * @param c          the set of characters
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the index of the first character that is contained in the
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       specified set of characters.
     *         If no character from the string is in the specified set of
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       characters -1 is returned.
     */
    public static int indexOfAnyOf(String s, String c, boolean ignoreCase) {
        return indexOfAnyOf(s, c, 0, ignoreCase);
    }

    /**
     * Returns the index of the first character that is contained in the
     * specified set of characters.
     *
     * @param s the string to be inspected
     * @param c the set of characters
     * @return the index of the first character that is contained in the
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       specified set of characters.
     *         If no character from the string is in the specified set of
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       characters -1 is returned.
     */
    public static int indexOfAnyOf(String s, String c) {
        return indexOfAnyOf(s, c, 0);
    }

    /**
     * Returns the index of the first character that is not contained in the
     * specified set of characters.
     * You can specify that case should be ignored.
     *
     * @param s          the string to be inspected
     * @param c          the set of characters
     * @param start      the starting position for the search
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the index of the first character that is not contained in the
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       specified set of characters.
     *         If no such character is found -1 is returned.
     */
    public static int indexOfAnyBut(String s, String c, int start, boolean ignoreCase) {

        int ls = s.length();
        int pos = -1;

        for (int n = start; n < ls; n++) {
            char ch = s.charAt(n);

            if (indexOf(c, ch, ignoreCase) < 0) {
                pos = n;

                break;
            }
        }

        return pos;
    }

    /**
     * Returns the index of the first character that is not contained in the
     * specified set of characters.
     * You can specify that case should be ignored.
     *
     * @param s          the string to be inspected
     * @param c          the set of characters
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the index of the first character that is not contained in the
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       specified set of characters.
     *         If no such character is found -1 is returned.
     */
    public static int indexOfAnyBut(String s, String c, boolean ignoreCase) {
        return indexOfAnyBut(s, c, 0, ignoreCase);
    }

    /**
     * Returns the index of the first character that is not contained in the
     * specified set of characters.
     *
     * @param s     the string to be inspected
     * @param c     the set of characters
     * @param start the starting position for the search
     * @return the index of the first character that is not contained in the
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       specified set of characters.
     *         If no such character is found -1 is returned.
     */
    public static int indexOfAnyBut(String s, String c, int start) {
        return indexOfAnyBut(s, c, start, false);
    }

    /**
     * Returns the index of the first character that is not contained in the
     * specified set of characters.
     *
     * @param s the string to be inspected
     * @param c the set of characters
     * @return the index of the first character that is not contained in the
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       specified set of characters.
     *         If no such character is found -1 is returned.
     */
    public static int indexOfAnyBut(String s, String c) {
        return indexOfAnyBut(s, c, 0, false);
    }

    /**
     * Returns the number of occurrences of the specified string.
     * You can specify that case should be ignored.
     *
     * @param s          the string to be inspected
     * @param c          the string to search for
     * @param start      the starting position for the search
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the number  of occurrences of the specified string.
     *         If the string is not found -1 is returned.
     */
    public static int occurrencesOf(String s, String c, int start, boolean ignoreCase) {

        int lc = c.length();
        int i;
        int n = 0;

        if (start > s.length() - lc) {
            return 0;
        }

        if (lc > 0) {
            while ((i = indexOf(s, c, start, ignoreCase)) >= 0) {
                n++;

                start = i + lc;
            }
        }

        return n;
    }

    /**
     * Returns the number of occurrences of the specified string.
     * You can specify that case should be ignored.
     *
     * @param s          the string to be inspected
     * @param c          the string to search for
     * @param ignoreCase if true ignore case when comparing characters
     * @return the number  of occurrences of the specified string.
     *         If the string is not found -1 is returned.
     */
    public static int occurrencesOf(String s, String c, boolean ignoreCase) {
        return occurrencesOf(s, c, 0, ignoreCase);
    }

    /**
     * Returns the number of occurrences of the specified string.
     *
     * @param s     the string to be inspected
     * @param c     the string to search for
     * @param start the starting position for the search
     * @return the number  of occurrences of the specified string.
     *         If the string is not found -1 is returned.
     */
    public static int occurrencesOf(String s, String c, int start) {
        return occurrencesOf(s, c, start, false);
    }

    /**
     * Returns the number of occurrences of the specified string.
     *
     * @param s     the string to be inspected
     * @param c     the string to search for
     * @param start the starting position for the search
     * @return the number  of occurrences of the specified string.
     *         If the string is not found -1 is returned.
     */
    public static int occurrencesOf(String s, String c) {
        return occurrencesOf(s, c, 0, false);
    }

    /**
     * Returns the index of the last ocurrence of the specified string.
     * The search is performed backwards from the specified starting position.
     * You can specify that case should be ignored.
     *
     * @param s          the string to be inspected
     * @param c          the string to be searched for
     * @param start      the starting position for the search
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the index of the last ocurrence of the specified string.
     *         If the specified string is not found -1 is returned.
     * @see java.lang.string#lastIndexOf(String,int)
     */
    public static int lastIndexOf(String s, String c, int start, boolean ignoreCase) {

        if (ignoreCase) {
            int ls = s.length();
            int lc = c.length();
            int lc1 = lc - 1;
            int le = ls - lc;

            if ((lc == 0) || (start < 0)) {
                return -1;
            }

            int ns = (start >= le)
                    ? le
                    : start;
            char c0 = c.charAt(0);

            while (ns >= 0) {
                if (equals(s.charAt(ns), c0, ignoreCase)) {
                    if (s.regionMatches(ignoreCase, ns + 1, c, 1, lc1)) {
                        return ns;
                    }
                }

                ns--;
            }

            return -1;
        } else {
            return s.lastIndexOf(c, start);
        }
    }

    /**
     * Returns the index of the last ocurrence of the specified string.
     * The search is performed backwards from the end of the string.
     * You can specify that case should be ignored.
     *
     * @param s          the string to be inspected
     * @param c          the string to be searched for
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the index of the last ocurrence of the specified string.
     *         If the specified string is not found -1 is returned.
     */
    public static int lastIndexOf(String s, String c, boolean ignoreCase) {
        return lastIndexOf(s, c, Integer.MAX_VALUE, ignoreCase);
    }

    /**
     * Returns the index of the last ocurrence of the specified string.
     * The search is performed backwards from the specified starting position.
     *
     * @param s     the string to be inspected
     * @param c     the string to be searched for
     * @param start the starting position for the search
     * @return the index of the last ocurrence of the specified string.
     *         If the specified string is not found -1 is returned.
     * @see java.lang.string#lastIndexOf(String,int)
     */
    public static int lastIndexOf(String s, String c, int start) {
        return s.lastIndexOf(c, start);
    }

    /**
     * Returns the index of the last ocurrence of the specified string.
     * The search is performed backwards from the end of the string.
     *
     * @param s the string to be inspected
     * @param c the string to be searched for
     * @return the index of the last ocurrence of the specified string.
     *         If the specified string is not found -1 is returned.
     * @see java.lang.string#lastIndexOf(String)
     */
    public static int lastIndexOf(String s, String c) {
        return s.lastIndexOf(c);
    }

    /**
     * Returns the index of the last ocurrence of the specified character.
     * The search is performed backwards from the specified starting position.
     * You can specify that case should be ignored.
     *
     * @param s          the string to be inspected
     * @param c          the character to be searched for
     * @param start      the starting position for the search
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the index of the last ocurrence of the specified character.
     *         If the specified character is not found -1 is returned.
     * @see java.lang.string#lastIndexOf(char,int)
     */
    public static int lastIndexOf(String s, char c, int start, boolean ignoreCase) {

        if (ignoreCase) {
            int ls = s.length();

            if (start < 0) {
                return -1;
            }

            if (start >= ls) {
                start = ls - 1;
            }

            for (int n = start; n >= 0; n--) {
                if (equals(s.charAt(n), c, ignoreCase)) {
                    return n;
                }
            }

            return -1;
        } else {
            return s.lastIndexOf(c, start);
        }
    }

    /**
     * Returns the index of the last ocurrence of the specified character.
     * The search is performed backwards from the specified starting position.
     *
     * @param s     the string to be inspected
     * @param c     the character to be searched for
     * @param start the starting position for the search
     * @return the index of the last ocurrence of the specified character.
     *         If the specified character is not found -1 is returned.
     * @see java.lang.string#lastIndexOf(char,int)
     */
    public static int lastIndexOf(String s, char c, int start) {
        return s.lastIndexOf(c, start);
    }

    /**
     * Returns the index of the last ocurrence of the specified character.
     * The search is performed backwards from the end of the string.
     *
     * @param s the string to be inspected
     * @param c the string to be searched for
     * @return the index of the last ocurrence of the specified character.
     *         If the specified character is not found -1 is returned.
     * @see java.lang.string#lastIndexOf(char)
     */
    public static int lastIndexOf(String s, char c) {
        return s.lastIndexOf(c);
    }

    /**
     * Returns the index of the last character that is not contained in the
     * specified set of characters.
     * The search is performed backwards from the specified starting position.
     * You can specify that case should be ignored.
     *
     * @param s          the string to be inspected
     * @param c          the set of characters
     * @param start      the starting position for the search
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the index of the last character that is not contained in the
     *         specified set of characters.
     *         If no such character is found -1 is returned.
     */
    public static int lastIndexOfAnyBut(String s, String c, int start, boolean ignoreCase) {

        int ls = s.length() - 1;
        int pos = -1;

        start = (start < ls)
                ? start
                : ls;

        for (int n = start; n >= 0; n--) {
            char ch = s.charAt(n);

            if (indexOf(c, ch, ignoreCase) < 0) {
                pos = n;

                break;
            }
        }

        return pos;
    }

    /**
     * Returns the index of the last character that is not contained in the
     * specified set of characters.
     * The search is performed backwards from the specified starting position.
     *
     * @param s     the string to be inspected
     * @param c     the set of characters
     * @param start the starting position for the search
     * @return the index of the last character that is not contained in the
     *         specified set of characters.
     *         If no such character is found -1 is returned.
     */
    public static int lastIndexOfAnyBut(String s, String c, int start) {
        return lastIndexOfAnyBut(s, c, start, false);
    }

    /**
     * Returns the index of the last character that is not contained in the
     * specified set of characters.
     * The search is performed backwards from the end of the string.
     *
     * @param s the string to be inspected
     * @param c the set of characters
     * @return the index of the last character that is not contained in the
     *         specified set of characters.
     *         If no such character is found -1 is returned.
     */
    public static int lastIndexOfAnyBut(String s, String c) {
        return lastIndexOfAnyBut(s, c, Integer.MAX_VALUE, false);
    }

    /**
     * Returns the index of the last character that is contained in the
     * specified set of characters.
     * The search is performed backwards from the specified starting position.
     * You can specify that case should be ignored.
     *
     * @param s          the string to be inspected
     * @param c          the set of characters
     * @param start      the starting position for the search
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the index of the last character that is contained in the
     *         specified set of characters.
     *         If no character from the string is in the specified set of
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       characters -1 is returned.
     */
    public static int lastIndexOfAnyOf(String s, String c, int start, boolean ignoreCase) {

        int lc = c.length();
        int ls = s.length();
        int pos = -1;

        for (int n = 0; n < lc; n++) {
            char ch = c.charAt(n);
            int i = lastIndexOf(s, ch, start, ignoreCase);

            if ((i >= 0) && (i > pos)) {
                pos = i;
            }
        }

        return pos;
    }

    /**
     * Returns the index of the last character that is contained in the
     * specified set of characters.
     * The search is performed backwards from the specified starting position.
     *
     * @param s     the string to be inspected
     * @param c     the set of characters
     * @param start the starting position for the search
     * @return the index of the last character that is contained in the
     *         specified set of characters.
     *         If no character from the string is in the specified set of
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       characters -1 is returned.
     */
    public static int lastIndexOfAnyOf(String s, String c, int start) {
        return lastIndexOfAnyOf(s, c, start, false);
    }

    /**
     * Returns the index of the last character that is contained in the
     * specified set of characters.
     * The search is performed backwards from the end of the string.
     *
     * @param s the string to be inspected
     * @param c the set of characters
     * @return the index of the last character that is contained in the
     *         specified set of characters.
     *         If no character from the string is in the specified set of
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       characters -1 is returned.
     */
    public static int lastIndexOfAnyOf(String s, String c) {
        return lastIndexOfAnyOf(s, c, Integer.MAX_VALUE, false);
    }

    /**
     * Strips the specified leading characters.
     * You can specify that case should be ignored.
     *
     * @param s          the string of which the leading characters should be
     *                   stripped off
     * @param c          the characters to be stripped off
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the original string with the specified leading characters stripped
     *         off.
     */
    public static String stripLeading(String s, String c, boolean ignoreCase) {

        int ls = s.length();
        int pos = 0;

        while ((pos < ls) && (indexOf(c, s.charAt(pos), ignoreCase) >= 0)) {
            pos++;
        }

        return s.substring(pos);
    }

    /**
     * Strips the specified leading characters.
     *
     * @param s the string of which the leading characters should be
     *          stripped off
     * @param c the characters to be stripped off
     * @return the original string with the specified leading characters stripped
     *         off.
     */
    public static String stripLeading(String s, String c) {
        return stripLeading(s, c, false);
    }

    /**
     * Strips the specified leading character.
     * You can specify that case should be ignored.
     *
     * @param s          the string of which the leading character should be
     *                   stripped off
     * @param c          the character to be stripped off
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the original string with the specified leading character stripped
     *         off.
     */
    public static String stripLeading(String s, char c, boolean ignoreCase) {

        int ls = s.length();
        int pos = 0;

        while ((pos < ls) && equals(s.charAt(pos), c, ignoreCase)) {
            pos++;
        }

        return s.substring(pos);
    }

    /**
     * Strips the specified leading character.
     *
     * @param s the string of which the leading character should be
     *          stripped off
     * @param c the character to be stripped off
     * @return the original string with the specified leading character stripped
     *         off.
     */
    public static String stripLeading(String s, char c) {
        return stripLeading(s, c, false);
    }

    /**
     * Strips leading white space characters.
     * All characters for which Character#isWhitespace() returns true are considered
     * white space characters.
     *
     * @param s the string of which white space is to be stripped off
     * @return the original string with leading white space characters stripped
     *         off.
     * @see Character#isWhitespace()
     */
    public static String stripLeadingBlanks(String s) {

        int ls = s.length();
        int pos = 0;

        while ((pos < ls) && Character.isWhitespace(s.charAt(pos))) {
            pos++;
        }

        return s.substring(pos);
    }

    /**
     * Strips the specified trailing characters.
     * You can specify that case should be ignored.
     *
     * @param s          the string of which trailing characters should be
     *                   stripped off
     * @param c          the characters to be stripped off
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the original string with the specified trailing characters stripped
     *         off.
     */
    public static String stripTrailing(String s, String c, boolean ignoreCase) {

        int pos = s.length() - 1;

        while ((pos >= 0) && (indexOf(c, s.charAt(pos), ignoreCase) >= 0)) {
            pos--;
        }

        return s.substring(0, pos + 1);
    }

    /**
     * Strips the specified trailing characters.
     *
     * @param s the string of which trailing characters should be
     *          stripped off
     * @param c the characters to be stripped off
     * @return the original string with the specified trailing characters stripped
     *         off.
     */
    public static String stripTrailing(String s, String c) {
        return stripTrailing(s, c, false);
    }

    /**
     * Strips the specified trailing character.
     * You can specify that case should be ignored.
     *
     * @param s          the string of which the trailing character should be
     *                   stripped off
     * @param c          the character to be stripped off
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the original string with the specified trailing character stripped
     *         off.
     */
    public static String stripTrailing(String s, char c, boolean ignoreCase) {

        int pos = s.length() - 1;

        while ((pos >= 0) && equals(s.charAt(pos), c, ignoreCase)) {
            pos--;
        }

        return s.substring(0, pos + 1);
    }

    /**
     * Strips the specified trailing character.
     *
     * @param s the string of which trailing characters should be
     *          stripped off
     * @param c the character to be stripped off
     * @return the original string with the specified trailing character stripped
     *         off.
     */
    public static String stripTrailing(String s, char c) {
        return stripTrailing(s, c, false);
    }

    /**
     * Strips trailing white space characters.
     * All characters for which Character#isWhitespace() returns true are considered
     * white space characters.
     *
     * @param s the string of which white space is to be stripped off
     * @return the original string with trailing white space characters stripped
     *         off.
     * @see Character#isWhitespace()
     */
    public static String stripTrailingBlanks(String s) {

        int pos = s.length() - 1;

        while ((pos >= 0) && Character.isWhitespace(s.charAt(pos))) {
            pos--;
        }

        return s.substring(0, pos + 1);
    }

    /**
     * Strips the specified characters from both ends of the string.
     * You can specify that case should be ignored.
     *
     * @param s          the string of which characters should be
     *                   stripped off
     * @param c          the characters to be stripped off
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the original string with the specified characters stripped
     *         off both ends.
     */
    public static String strip(String s, String c, boolean ignoreCase) {

        String s1 = stripLeading(s, c, ignoreCase);

        return stripTrailing(s1, c, ignoreCase);
    }

    /**
     * Strips the specified characters from both ends of the string.
     *
     * @param s the string of which characters should be
     *          stripped off
     * @param c the characters to be stripped off
     * @return the original string with the specified characters stripped
     *         off both ends.
     */
    public static String strip(String s, String c) {
        return strip(s, c, false);
    }

    /**
     * Strips the specified character from both ends of the string.
     * You can specify that case should be ignored.
     *
     * @param s          the string of which the character should be
     *                   stripped off
     * @param c          the character to be stripped off
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the original string with the specified character stripped
     *         off both ends.
     */
    public static String strip(String s, char c, boolean ignoreCase) {

        String s1 = stripLeading(s, c, ignoreCase);

        return stripTrailing(s1, c, ignoreCase);
    }

    /**
     * Strips the specified character from both ends of the string.
     *
     * @param s the string of which the character should be
     *          stripped off
     * @param c the character to be stripped off
     * @return the original string with the specified character stripped
     *         off both ends.
     */
    public static String strip(String s, char c) {
        return strip(s, c, false);
    }

    /**
     * Strips white space characters from both ends of the string.
     * All characters for which Character#isWhitespace() returns true are considered
     * white space characters.
     *
     * @param s the string of which white space is to be stripped off
     * @return the original string with white space stripped
     *         off both ends.
     * @see Character#isWhitespace()
     */
    public static String stripBlanks(String s) {
        return s.trim();
    }

    /**
     * Centers the string within a string of the specified length, padding
     * it with the specified character.
     *
     * @param s   the original string
     * @param len the length of the new string
     * @param pad the pad character
     * @return a string with the original string centered in it.
     */
    public static String center(String s, int len, char pad) {

        int start;
        int ls = s.length();

        if (len < 1) {
            return "";
        }

        char[] buf = new char[len];

        for (int i = 0; i < len; i++) {
            buf[i] = pad;
        }

        if (len > ls) {
            start = (len - ls) / 2;

            s.getChars(0, ls, buf, start);
        } else {
            start = (ls - len) / 2;

            s.getChars(start, start + len, buf, 0);
        }

        return new String(buf);
    }

    /**
     * Centers the string within a string of the specified length, padding
     * it with blanks.
     *
     * @param s   the original string
     * @param len the length of the new string
     * @return a string with the original string centered in it.
     */
    public static String center(String s, int len) {
        return center(s, len, ' ');
    }

    /**
     * Changes occurrences of a specified pattern to a replacement string.
     * The maximum number of changes and a starting position can be specified.
     * You can specify that case should be ignored.
     *
     * @param s          the input string
     * @param oldp       the pattern to be replaced
     * @param newp       the replacement string
     * @param start      the starting position for the search
     * @param maxChanges the maximum number of changes
     * @param ignoreCase if true case is ignored when comparing characters
     * @return a string with the specified patterns changed.
     */
    public static String change(String s, String oldp, String newp, int start, int maxChanges,
                                boolean ignoreCase) {

        int lo = oldp.length();
        int ln = newp.length();
        int ls = s.length();
        int newlen = ls;
        int lastPos;
        int lastDest;
        int fpos;
        int nchg = 0;

        if ((lo == 0) || (start >= ls)) {
            return s;
        }

        if (start < 0) {
            start = 0;
        }

        lastPos = start;

        if (ln > lo) {
            int n = occurrencesOf(s, oldp, start, ignoreCase);

            newlen = ls + n * (ln - lo);
        }

        char[] buf = new char[newlen];

        s.getChars(0, start, buf, 0);

        lastDest = start;

        while ((fpos = indexOf(s, oldp, lastPos, ignoreCase)) >= 0 && (nchg++ < maxChanges)) {
            s.getChars(lastPos, fpos, buf, lastDest);

            lastDest += fpos - lastPos;

            newp.getChars(0, ln, buf, lastDest);

            lastPos = fpos + lo;
            lastDest += ln;
        }

        s.getChars(lastPos, ls, buf, lastDest);

        return new String(buf, 0, lastDest - lastPos + ls);
    }

    /**
     * Changes occurrences of a specified pattern to a replacement string.
     * The starting position can be specified.
     * You can specify that case should be ignored.
     *
     * @param s          the input string
     * @param oldp       the pattern to be replaced
     * @param newp       the replacement string
     * @param start      the starting position for the search
     * @param ignoreCase if true case is ignored when comparing characters
     * @return a string with the specified patterns changed.
     */
    public static String change(String s, String oldp, String newp, int start, boolean ignoreCase) {
        return change(s, oldp, newp, start, Integer.MAX_VALUE, ignoreCase);
    }

    /**
     * Changes occurrences of a specified pattern to a replacement string.
     * The maximum number of changes and a starting position can be specified.
     *
     * @param s          the input string
     * @param oldp       the pattern to be replaced
     * @param newp       the replacement string
     * @param start      the starting position for the search
     * @param maxChanges the maximum number of changes
     * @return a string with the specified patterns changed.
     */
    public static String change(String s, String oldp, String newp, int start, int maxChanges) {
        return change(s, oldp, newp, start, maxChanges, false);
    }

    /**
     * Changes occurrences of a specified pattern to a replacement string.
     * A starting position can be specified.
     *
     * @param s     the input string
     * @param oldp  the pattern to be replaced
     * @param newp  the replacement string
     * @param start the starting position for the search
     * @return a string with the specified patterns changed.
     */
    public static String change(String s, String oldp, String newp, int start) {
        return change(s, oldp, newp, start, Integer.MAX_VALUE);
    }

    /**
     * Changes occurrences of a specified pattern to a replacement string.
     * You can specify that case should be ignored.
     *
     * @param s          the input string
     * @param oldp       the pattern to be replaced
     * @param newp       the replacement string
     * @param ignoreCase if true case is ignored when comparing characters
     * @return a string with the specified patterns changed.
     */
    public static String change(String s, String oldp, String newp, boolean ignoreCase) {
        return change(s, oldp, newp, 0, Integer.MAX_VALUE, ignoreCase);
    }

    /**
     * Changes occurrences of a specified pattern to a replacement string.
     *
     * @param s    the input string
     * @param oldp the pattern to be replaced
     * @param newp the replacement string
     * @return a string with the specified patterns changed.
     */
    public static String change(String s, String oldp, String newp) {
        return change(s, oldp, newp, 0, Integer.MAX_VALUE, false);
    }

    /**
     * Changes occurrences of a specified character to a replacement character.
     * The maximum number of changes and a starting position can be specified.
     * You can specify that case should be ignored.
     *
     * @param s          the input string
     * @param oldc       the pattern to be replaced
     * @param newc       the replacement string
     * @param start      the starting position for the search
     * @param maxChanges the maximum number of changes
     * @param ignoreCase if true case is ignored when comparing characters
     * @return a string with the specified characters changed.
     */
    public static String change(String s, char oldc, char newc, int start, int maxChanges,
                                boolean ignoreCase) {

        int ls = s.length();
        int lastPos;
        int lastDest;
        int fpos;
        int nchg = 0;

        if (start >= ls) {
            return s;
        }

        if (start < 0) {
            start = 0;
        }

        lastPos = start;

        char[] buf = new char[ls];

        s.getChars(0, start, buf, 0);

        lastDest = start;

        while ((fpos = indexOf(s, oldc, lastPos, ignoreCase)) >= 0 && (nchg++ < maxChanges)) {
            s.getChars(lastPos, fpos, buf, lastPos);

            buf[fpos] = newc;
            lastPos = fpos + 1;
        }

        s.getChars(lastPos, ls, buf, lastPos);

        return new String(buf, 0, ls);
    }

    /**
     * Changes occurrences of a specified character to a replacement character.
     * The maximum number of changes and a starting position can be specified.
     *
     * @param s          the input string
     * @param oldc       the pattern to be replaced
     * @param newc       the replacement string
     * @param start      the starting position for the search
     * @param maxChanges the maximum number of changes
     * @return a string with the specified characters changed.
     */
    public static String change(String s, char oldc, char newc, int start, int maxChanges) {
        return change(s, oldc, newc, start, maxChanges, false);
    }

    /**
     * Changes occurrences of a specified character to a replacement character.
     * The starting position can be specified.
     * You can specify that case should be ignored.
     *
     * @param s          the input string
     * @param oldc       the pattern to be replaced
     * @param newc       the replacement string
     * @param start      the starting position for the search
     * @param ignoreCase if true case is ignored when comparing characters
     * @return a string with the specified characters changed.
     */
    public static String change(String s, char oldc, char newc, int start, boolean ignoreCase) {
        return change(s, oldc, newc, start, Integer.MAX_VALUE, ignoreCase);
    }

    /**
     * Changes occurrences of a specified character to a replacement character.
     * A starting position can be specified.
     *
     * @param s     the input string
     * @param oldc  the pattern to be replaced
     * @param newc  the replacement string
     * @param start the starting position for the search
     * @return a string with the specified characters changed.
     */
    public static String change(String s, char oldc, char newc, int start) {
        return change(s, oldc, newc, start, Integer.MAX_VALUE, false);
    }

    /**
     * Changes occurrences of a specified character to a replacement character.
     *
     * @param s    the input string
     * @param oldc the pattern to be replaced
     * @param newc the replacement string
     * @return a string with the specified characters changed.
     * @see java.lang.string#replace(char,char)
     */
    public static String change(String s, char oldc, char newc) {
        return s.replace(oldc, newc);
    }

    /**
     * Returns a string consisting of the specified number of copies of the
     * input string.
     *
     * @param s       the input string
     * @param nCopies the number of copies
     * @return a string consisting of the specified number of copies of the
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       input string.
     * @see java.lang.stringBuffer#append(String)
     */
    public static String copies(String s, int nCopies) {

        int ls = s.length();

        if (nCopies < 1) {
            return "";
        }

        StringBuffer buf = new StringBuffer(ls * nCopies);

        for (int n = 0; n < nCopies; n++) {
            buf.append(s);
        }

        return buf.toString();
    }

    /**
     * Inserts a string after the specified position.
     * If the position is beyond the length of the original string, the result
     * string is padded with the specified pad character.
     *
     * @param s     the input string
     * @param ins   the string to be inserted
     * @param start the starting position for the insert
     * @param pad   the pad character
     * @return a string with the input string inserted after the specified
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       position.
     */
    public static String insert(String s, String ins, int start, char pad) {

        int ls = s.length();
        int li = ins.length();
        int newlen;

        if (start < 0) {
            return "";
        }

        if (start > ls) {
            newlen = li + start;
        } else {
            newlen = ls + li;
        }

        int sPos = (start > ls)
                ? ls
                : start;
        int sRest = sPos;
        char[] buf = new char[newlen];

        s.getChars(0, sPos, buf, 0);

        while (sPos < start) {
            buf[sPos++] = pad;
        }

        ins.getChars(0, li, buf, sPos);
        s.getChars(sRest, ls, buf, sPos + li);

        return new String(buf);
    }

    /**
     * Inserts a string after the specified position.
     * If the position is beyond the length of the original string, the result
     * string is padded with blanks.
     *
     * @param s     the input string
     * @param ins   the string to be inserted
     * @param start the starting position for the insert
     * @return a string with the input string inserted after the specified
     *         *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                               *                                       position.
     */
    public static String insert(String s, String ins, int start) {
        return insert(s, ins, start, ' ');
    }

    /**
     * Left-justifies the string in a string of the specified length.
     * If the specified length is greater than the length of the string, the
     * string is padded with the specified pad character.
     *
     * @param s   the input string
     * @param len the length of the new string
     * @param pad the pad character
     * @return a string with the input string left-justified in it.
     */
    public static String leftJustify(String s, int len, char pad) {

        int ls = s.length();
        int ln = (len < ls)
                ? len
                : ls;

        if (len < 1) {
            return "";
        }

        char[] buf = new char[len];

        s.getChars(0, ln, buf, 0);

        while (ln < len) {
            buf[ln++] = pad;
        }

        return new String(buf);
    }

    /**
     * Left-justifies the string in a string of the specified length.
     * If the specified length is greater than the length of the string, the
     * string is padded with blanks.
     *
     * @param s   the input string
     * @param len the length of the new string
     * @return a string with the input string left-justified in it.
     */
    public static String leftJustify(String s, int len) {
        return leftJustify(s, len, ' ');
    }

    /**
     * Right-justifies the string in a string of the specified length.
     * If the specified length is greater than the length of the string, the
     * string is padded with the specified pad character.
     *
     * @param s   the input string
     * @param len the length of the new string
     * @param pad the pad character
     * @return a string with the input string right-justified in it.
     */
    public static String rightJustify(String s, int len, char pad) {

        int ls = s.length();
        int lp;

        if (len < 1) {
            return "";
        }

        char[] buf = new char[len];

        if (len > ls) {
            lp = len - ls;

            s.getChars(0, ls, buf, lp);

            while (lp > 0) {
                buf[--lp] = pad;
            }
        } else {
            lp = ls - len;

            s.getChars(lp, ls, buf, 0);
        }

        return new String(buf);
    }

    /**
     * Right-justifies the string in a string of the specified length.
     * If the specified length is greater than the length of the string, the
     * string is padded with blanks.
     *
     * @param s   the input string
     * @param len the length of the new string
     * @return a string with the input string right-justified in it.
     */
    public static String rightJustify(String s, int len) {
        return rightJustify(s, len, ' ');
    }

    /**
     * Converts the string to lower case.
     *
     * @param s the input string
     * @return a string with all characters converted to lower case.
     * @see java.lang.string#toLowerCase()
     */
    public static String toLowerCase(String s) {
        return s.toLowerCase();
    }

    /**
     * Converts the string to lower case using the given Locale.
     *
     * @param s the input string
     * @return a string with all characters converted to lower case.
     * @see java.lang.string#toLowerCase(Locale)
     */
    public static String toLowerCase(String s, Locale locale) {
        return s.toLowerCase(locale);
    }

    /**
     * Converts the string to upper case.
     *
     * @param s the input string
     * @return a string with all characters converted to upper case.
     * @see java.lang.string#toUpperCase()
     */
    public static String toUpperCase(String s) {
        return s.toUpperCase();
    }

    /**
     * Converts the string to upper case using the given Locale.
     *
     * @param s the input string
     * @return a string with all characters converted to upper case.
     * @see java.lang.string#toUpperCase(Locale)
     */
    public static String toUpperCase(String s, Locale locale) {
        return s.toUpperCase(locale);
    }

    /**
     * Converts the string to titlecase.
     *
     * @param s the input string
     * @return a string with all characters converted to title case.
     * @see Character#toTitleCase(char)
     */
    public static String toTitleCase(String s) {

        int ls = s.length();
        char[] buf = new char[ls];

        s.getChars(0, ls, buf, 0);

        for (int i = 0; i < ls; i++) {
            Character.toTitleCase(buf[i]);
        }

        return new String(buf);
    }

    /**
     * Replaces part of the string with the specified string, starting at a
     * specified position. If the starting position is beyond the end of the
     * string, it is padded with the specified pad character.
     *
     * @param s     the original string
     * @param o     the string to be overlayed over the original string
     * @param start the starting position for the overlay
     * @param pad   the padding character
     * @return a string with part of it replaced by the overlay string.
     */
    public static String overlayWith(String s, String o, int start, char pad) {

        int ls = s.length();
        int lo = o.length();

        if (start < 0) {
            return "";
        }

        int ln = start + lo;
        int pos = (start >= ls)
                ? ls
                : start;
        int newlen = (ln < ls)
                ? ls
                : ln;
        char[] buf = new char[newlen];

        s.getChars(0, pos, buf, 0);

        for (int i = ls; i < start; i++) {
            buf[i] = pad;
        }

        o.getChars(0, lo, buf, start);

        if (ln < ls) {
            s.getChars(ln, ls, buf, ln);
        }

        return new String(buf);
    }

    /**
     * Replaces part of the string with the specified string, starting at a
     * specified position. If the starting position is beyond the end of the
     * string, it is padded with blanks.
     *
     * @param s     the original string
     * @param o     the string to be overlayed over the original string
     * @param start the starting position for the overlay
     * @return a string with part of it replaced by the overlay string.
     */
    public static String overlayWith(String s, String o, int start) {
        return overlayWith(s, o, start, ' ');
    }

    /**
     * Removes the specified number of characters from the string, starting
     * at the specified position.
     *
     * @param s        the original string
     * @param start    the starting position
     * @param numChars the number of characters to be removed
     * @return a string with the specified number of characters removed.
     */
    public static String remove(String s, int start, int numChars) {

        int ls = s.length();

        if ((start >= ls) || (start < 0) || (numChars < 1)) {
            return s;
        }

        if (start > ls - numChars) {
            numChars = ls - start;
        }

        int newlen = ls - numChars;
        char[] buf = new char[newlen];

        s.getChars(0, start, buf, 0);
        s.getChars(start + numChars, ls, buf, start);

        return new String(buf);
    }

    /**
     * Returns a substring of the specified string, starting at a specified
     * position, with the specified length. If the specified length reaches beyond
     * the length of the string, the result is padded with the specified
     * pad character.
     *
     * @param s     the original string
     * @param start the starting position
     * @param len   the length of the substring
     * @param pad   the padding character
     * @return a substring of the input string.
     * @throws StringIndexOutOfBoundsException
     *          if
     *          <code>start</code> is out of range or <code>len</code> is
     *          less than zero.
     * @see java.lang.string#substring(int,int)
     */
    public static String substring(String s, int start, int len, char pad) {

        int ls = s.length();

        if ((start < 0) || (start >= ls)) {
            throw new StringIndexOutOfBoundsException(start);
        }

        if (len < 0) {
            throw new StringIndexOutOfBoundsException(len);
        }

        if (len > ls - start) {
            char[] buf = new char[len];

            s.getChars(start, ls, buf, 0);

            for (int i = ls - start; i < len; i++) {
                buf[i] = pad;
            }

            return new String(buf);
        } else {
            return s.substring(start, start + len);
        }
    }

    /**
     * Returns a substring of the specified string, starting at a specified
     * position, with the specified length. If the specified length reaches beyond
     * the length of the string, the result is padded with blanks.
     *
     * @param s     the original string
     * @param start the starting position
     * @param len   the length of the substring
     * @return a substring of the input string.
     * @throws StringIndexOutOfBoundsException
     *          if
     *          <code>start</code> is out of range.
     * @see java.lang.string#substring(int,int)
     */
    public static String substring(String s, int start, int len) {
        return substring(s, start, len, ' ');
    }

    /**
     * Returns a substring of the specified string, starting at a specified
     * position, and extending to the end of the string.
     *
     * @param s     the original string
     * @param start the starting position
     * @return a substring of the input string.
     * @throws StringIndexOutOfBoundsException
     *          if
     *          <code>start</code> is out of range.
     * @see java.lang.string#substring(int)
     */
    public static String substring(String s, int start) {
        return s.substring(start);
    }

    /**
     * Reverses the sequence of characters contained in the string.
     *
     * @param s the original string
     * @return a string with the characters from the original string reversed.
     * @see java.lang.stringBuffer#reverse()
     */
    public static String reverse(String s) {

        StringBuffer buf = new StringBuffer(s);

        buf.reverse();

        return buf.toString();
    }

    /**
     * Converts all of the string's characters which are contained in the input
     * set of characters to the corresponding character in the output set of
     * characters.
     * If the output set of characters is shorter than the input set of
     * characters, it is padded with the specified pad character.
     * You can specify that case should be ignored.
     * <p/>
     * The method can be used to reorder characters in a string:
     * <p/>
     * Strings.translate("3421", "1234", "abcd", ' ')  returns "cdba"
     *
     * @param s          the original string
     * @param setin      the input set of characters
     * @param setout     the output set of characters
     * @param pad        the pad character
     * @param ignoreCase if true case is ignored when comparing characters
     * @return a string with the specified characters tramslated.
     */
    public static String translate(String s, String setin, String setout, char pad,
                                   boolean ignoreCase) {

        int ls = s.length();
        int lout = setout.length();
        char[] buf = new char[ls];

        s.getChars(0, ls, buf, 0);

        int pos = 0;
        int n;

        while ((pos = indexOfAnyOf(s, setin, pos, ignoreCase)) >= 0) {
            n = indexOf(setin, buf[pos], ignoreCase);

            if (n >= lout) {
                buf[pos] = pad;
            } else {
                buf[pos] = setout.charAt(n);
            }

            pos++;
        }

        return new String(buf);
    }

    /**
     * Converts all of the string's characters which are contained in the input
     * set of characters to the corresponding character in the output set of
     * characters.
     * If the output set of characters is shorter than the input set of
     * characters, it is padded with the specified pad character.
     * <p/>
     * The method can be used to reorder characters in a string:
     * <p/>
     * Strings.translate("3421", "1234", "abcd", ' ')  returns "cdba"
     *
     * @param s      the original string
     * @param setin  the input set of characters
     * @param setout the output set of characters
     * @param pad    the pad character
     * @return a string with the specified characters tramslated.
     */
    public static String translate(String s, String setin, String setout, char pad) {
        return translate(s, setin, setout, pad, false);
    }

    /**
     * Converts all of the string's characters which are contained in the input
     * set of characters to the corresponding character in the output set of
     * characters.
     * If the output set of characters is shorter than the input set of
     * characters, it is padded with blanks.
     * You can specify that case should be ignored.
     * <p/>
     * The method can be used to reorder characters in a string:
     * <p/>
     * Strings.translate("3421", "1234", "abcd", ' ')  returns "cdba"
     *
     * @param s          the original string
     * @param setin      the input set of characters
     * @param setout     the output set of characters
     * @param ignoreCase if true case is ignored when comparing characters
     * @return a string with the specified characters tramslated.
     */
    public static String translate(String s, String setin, String setout, boolean ignoreCase) {
        return translate(s, setin, setout, ' ', ignoreCase);
    }

    /**
     * Converts all of the string's characters which are contained in the input
     * set of characters to the corresponding character in the output set of
     * characters.
     * If the output set of characters is shorter than the input set of
     * characters, it is padded with blanks.
     * <p/>
     * The method can be used to reorder characters in a string:
     * <p/>
     * Strings.translate("3421", "1234", "abcd")  returns "cdba"
     *
     * @param s      the original string
     * @param setin  the input set of characters
     * @param setout the output set of characters
     * @return a string with the specified characters tramslated.
     */
    public static String translate(String s, String setin, String setout) {
        return translate(s, setin, setout, ' ');
    }

    /**
     * Builds a string consisting of all characters between 'start' and 'end'.
     * Both the start and end characters are included in the string.
     *
     * @param start the starting character
     * @param end   the ending character
     * @return a string consisting of all the specified characters.
     */
    public static String xRange(char start, char end) {

        if (start <= end) {
            int l = end - start + 1;
            char[] buf = new char[l];

            for (int i = 0; i < l; i++) {
                buf[i] = (char) (start + i);
            }

            return new String(buf);
        } else {
            return "";
        }
    }

    /**
     * Returns true if the specified search string is contained in the original
     * string.
     * You can specify that case should be ignored.
     *
     * @param s           the input string
     * @param searchForMe the search string
     * @param ignoreCase  if true case is ignored when comparing characters
     * @return true if the search string is contained in the input string
     * @see java.lang.string#indexOf(String)
     */
    public static boolean contains(String searchIn, String searchForMe, boolean ignoreCase) {
        return indexOf(searchIn, searchForMe, ignoreCase) >= 0;
    }

    /**
     * Returns true if the specified search string is contained in the original
     * string.
     *
     * @param s           the original string
     * @param searchForMe the search string
     * @return true if the search string is contained in the input string
     * @see java.lang.string#indexOf(String)
     */
    public static boolean contains(String searchIn, String searchForMe) {
        return searchIn.indexOf(searchForMe) >= 0;
    }

    /**
     * Returns true if the string is a valid abbreviation of another string.
     * A string <code>s</code> is considered an abbreviation of another string
     * <code>full</code> if
     * the string <code>s</code> is a prefix of string <code>full</code> and its length is at least
     * <code>minLength</code>.
     * You can specify that case should be ignored.
     *
     * @param s          the string to test
     * @param full       the full string
     * @param minLength  the minimum length
     * @param ignoreCase if true case is ignored when comparing characters
     * @return true if the string <code>s</code> is an abbreviation of string <code>full</code>.
     */
    public static boolean isAbbreviationOf(String s, String full, int minLength,
                                           boolean ignoreCase) {

        int ls = s.length();

        if ((minLength > ls) || (minLength < 0)) {
            return false;
        }

        if (minLength == 0) {
            minLength = ls;
        }

        return s.regionMatches(ignoreCase, 0, full, 0, minLength);
    }

    /**
     * Returns true if the string is a valid abbreviation of another string.
     * A string <code>s</code> is considered an abbreviation of another string
     * <code>full</code> if
     * the string <code>s</code> is a prefix of string <code>full</code> and its length is at least
     * <code>minLength</code>.
     *
     * @param s         the string to test
     * @param full      the full string
     * @param minLength the minimum length
     * @return true if the string <code>s</code> is an abbreviation of string <code>full</code>.
     */
    public static boolean isAbbreviationOf(String s, String full, int minLength) {
        return isAbbreviationOf(s, full, minLength, false);
    }

    /**
     * Returns true if the string is a valid abbreviation of another string.
     * A string <code>s</code> is considered an abbreviation of another string
     * <code>full</code> if
     * the string <code>s</code> is a prefix of string <code>full</code>.
     * You can specify that case should be ignored.
     *
     * @param s          the string to test
     * @param full       the full string
     * @param ignoreCase if true case is ignored when comparing characters
     * @return true if the string <code>s</code> is an abbreviation of string <code>full</code>.
     * @see java.lang.string#regionMatches(boolean,int,String,int,int)
     */
    public static boolean isAbbreviationOf(String s, String full, boolean ignoreCase) {
        return s.regionMatches(ignoreCase, 0, full, 0, s.length());
    }

    /**
     * Returns true if the string matches the specified pattern.
     * The pattern may contain two wildcard characters, where the first wildcard
     * character matches any sequence of characters and the second wildcard
     * character matches an arbitrary single character.
     * You can specify that case should be ignored.
     *
     * @param s          the string to test
     * @param pattern    the pattern string
     * @param start      the starting position
     * @param wAnySeq    the wildcard character which matches any sequence of characters
     * @param wAnyChar   the wildcard character which matches any character
     * @param ignoreCase if true case is ignored when comparing characters
     * @return true if the string matches the specified pattern.
     */
    public static boolean isLike(String s, String pattern, int start, char wAnySeq, char wAnyChar,
                                 boolean ignoreCase) {

        int ls = s.length();
        int lp = pattern.length();

        if (lp == 0) {
            return false;
        }

        char p = ' ';
        int ns = start;
        int np = 0;
        int startp = -1;
        int starts = 0;

        while ((np <= lp) && (ns <= ls)) {
            if (np < lp) {
                p = pattern.charAt(np);

                if (p == wAnySeq) {
                    startp = np++;
                    starts = ns;
                } else if (ns < ls) {
                    if ((p == wAnyChar) || equals(s.charAt(ns), pattern.charAt(np), ignoreCase)) {
                        np++;
                        ns++;
                    } else if (startp < 0) {

                        // there was no wAnySeq in the pattern
                        return false;
                    } else {

                        // we have to return to the last wAnySeq
                        np = startp;
                        ns = ++starts;
                    }
                } else {

                    // At end of s and no match found
                    return false;
                }
            } else {
                if (p == wAnySeq) {
                    return true;
                }

                if ((ns < ls) && (startp >= 0)) {

                    // we have to return to the last wAnySeq
                    np = startp + 1;
                    ns = ++starts;
                } else {
                    return (ns >= ls);
                }
            }
        }

        return true;
    }

    /**
     * Returns true if the string matches the specified pattern.
     * The pattern may contain two wildcard characters, where '*'
     * matches any sequence of characters and '?'
     * matches an arbitrary single character.
     * You can specify that case should be ignored.
     *
     * @param s          the string to test
     * @param pattern    the pattern string
     * @param start      the starting position
     * @param ignoreCase if true case is ignored when comparing characters
     * @return true if the string matches the specified pattern.
     */
    public static boolean isLike(String s, String pattern, int start, boolean ignoreCase) {
        return isLike(s, pattern, start, '*', '?', ignoreCase);
    }

    /**
     * Returns true if the string matches the specified pattern.
     * The pattern may contain two wildcard characters, where '*'
     * matches any sequence of characters and '?'
     * matches an arbitrary single character.
     *
     * @param s       the string to test
     * @param pattern the pattern string
     * @param start   the starting position
     * @return true if the string matches the specified pattern.
     */
    public static boolean isLike(String s, String pattern, int start) {
        return isLike(s, pattern, start, '*', '?', false);
    }

    /**
     * Returns true if the string matches the specified pattern.
     * The pattern may contain two wildcard characters, where '*'
     * matches any sequence of characters and '?'
     * matches an arbitrary single character.
     *
     * @param s       the string to test
     * @param pattern the pattern string
     * @return true if the string matches the specified pattern.
     */
    public static boolean isLike(String s, String pattern) {
        return isLike(s, pattern, 0, '*', '?', false);
    }

    /**
     * Returns the index of the first character of the specified nth word within
     * the string. Word numbers start at 0.
     * Words are sequences of characters separated by white space. White space is
     * any character for which java.lang.Character#isWhitespace() returns true.
     *
     * @param s       the input string
     * @param wordNum the number of the word
     * @return the index of the first character of the specified nth word within
     *         the string.
     *         Returns -1 if the word is not found.
     */
    public static int indexOfWord(String s, int wordNum) {

        int ls = s.length();

        if (wordNum < 0) {
            return -1;
        }

        wordNum++;

        int pos = 0;

        while (pos < ls) {

            // search for the beginning of a word
            while ((pos < ls) && Character.isWhitespace(s.charAt(pos))) {
                pos++;
            }

            if (pos < ls) {
                if (--wordNum == 0) {
                    break;
                }

                pos++;

                // search for the end of the word
                while ((pos < ls) && !Character.isWhitespace(s.charAt(pos))) {
                    pos++;
                }

                pos++;
            }
        }

        if (wordNum > 0) {
            pos = -1;
        }

        return pos;
    }

    /**
     * Returns the index of the first occurrence of the specified phrase within
     * the string.
     * Words are sequences of characters separated by white space. White space is
     * any character for which java.lang.Character#isWhitespace() returns true.
     * You can specify that case should be ignored.
     *
     * @param s          the input string
     * @param p          the phrase to be searched for
     * @param startWord  the word number where the search is to start. The
     *                   first word has number 0.
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the index of the first occurrence of the specified phrase within
     *         the string.
     *         Returns -1 if the phrase is not found.
     */
    public static int indexOfPhrase(String s, String p, int startWord, boolean ignoreCase) {

        WordEntry e = wordEntryOfPhrase(s, p, startWord, ignoreCase);

        if (e == null) {
            return -1;
        }

        return e.start;
    }

    /**
     * Returns the index of the first occurrence of the specified phrase within
     * the string.
     * Words are sequences of characters separated by white space. White space is
     * any character for which java.lang.Character#isWhitespace() returns true.
     *
     * @param s         the input string
     * @param p         the phrase to be searched for
     * @param startWord the word number where the search is to start. The
     *                  first word has number 0.
     * @return the index of the first occurrence of the specified phrase within
     *         the string.
     *         Returns -1 if the phrase is not found.
     */
    public static int indexOfPhrase(String s, String p, int startWord) {
        return indexOfPhrase(s, p, startWord, false);
    }

    /**
     * Returns the number of words in the string.
     * Words are sequences of characters separated by white space. White space is
     * any character for which java.lang.Character#isWhitespace() returns true.
     *
     * @param s the input string
     * @return the number of words in the string.
     */
    public static int numWords(String s) {

        int ls = s.length();
        int wordNum = 0;
        int pos = 0;

        while (pos < ls) {

            // search for the beginning of a word
            while ((pos < ls) && Character.isWhitespace(s.charAt(pos))) {
                pos++;
            }

            if (pos++ < ls) {
                wordNum++;
            }

            // search for the end of the word
            while ((pos < ls) && !Character.isWhitespace(s.charAt(pos))) {
                pos++;
            }

            pos++;
        }

        return wordNum;
    }

    /**
     * Removes the specified number of words from the string, starting with the
     * specified word number.
     * Words are sequences of characters separated by white space. White space is
     * any character for which java.lang.Character#isWhitespace() returns true.
     *
     * @param s         the input string
     * @param startWord the number of the first word to be removed
     * @param numWords  the number of words to be removed.
     * @return a string with the specified number of words removed.
     */
    public static String removeWords(String s, int startWord, int numWords) {

        if ((startWord < 0) || (numWords <= 0)) {
            return s;
        }

        int startPos = indexOfWord(s, startWord);

        if (startPos >= 0) {
            int lastPos = indexOfWord(s.substring(startPos), numWords);

            if (lastPos >= 0) {
                return remove(s, startPos, lastPos);
            } else {
                return s.substring(0, startPos);
            }
        }

        return s;
    }

    /**
     * All whitespace between the words of the string is replaced by the
     * specified number of blanks.
     * Words are sequences of characters separated by white space. White space is
     * any character for which java.lang.Character#isWhitespace() returns true.
     *
     * @param s      the input string
     * @param spaces the number of blanks between the words
     * @return a string with the words separated by the specifed number of blanks
     */
    public static String wordSpace(String s, int spaces) {

        int ls = s.length();
        int i;

        if (spaces < 0) {
            return s;
        }

        Vector wordList = buildWordList(s, ls);
        int nWords = wordList.size();

        if (nWords == 0) {
            return "";
        }

        // Calculate length of new string
        int newsize = 0;

        for (i = 0; i < nWords; i++) {
            WordEntry e = (WordEntry) wordList.elementAt(i);

            newsize += e.len;
        }

        char[] buf = new char[newsize + (nWords - 1) * spaces];
        int pos = 0;

        for (i = 0; i < nWords; i++) {
            WordEntry e = (WordEntry) wordList.elementAt(i);

            s.getChars(e.start, e.start + e.len, buf, pos);

            pos += e.len;

            if (i < nWords - 1) {
                for (int j = 0; j < spaces; j++) {
                    buf[pos++] = ' ';
                }
            }
        }

        return new String(buf);
    }

    /**
     * Returns the nth word in the string.
     * Words are sequences of characters separated by white space. White space is
     * any character for which java.lang.Character#isWhitespace() returns true.
     *
     * @param s       the input string
     * @param wordNum the number of the word to be returned
     * @return a string representing the nth word from the input string.
     */
    public static String word(String s, int wordNum) {

        int ls = s.length();
        int i;

        if (wordNum < 0) {
            return "";
        }

        int startPos = indexOfWord(s, wordNum);

        if (startPos >= 0) {

            // search for the end of the word
            int pos = startPos;

            while ((pos < ls) && !Character.isWhitespace(s.charAt(pos))) {
                pos++;
            }

            return s.substring(startPos, pos);
        }

        return "";
    }

    /**
     * Returns the number of the first word of the first occurrence of the
     * specified phrase within the string.
     * Words are sequences of characters separated by white space. White space is
     * any character for which java.lang.Character#isWhitespace() returns true.
     * You can specify that case should be ignored.
     *
     * @param s          the input string
     * @param p          the phrase to be searched for
     * @param startWord  the word number where the search is to start. The
     *                   first word has number 0.
     * @param ignoreCase if true case is ignored when comparing characters
     * @return the number of the first word of the first occurrence of the
     *         specified phrase within the string.
     */
    public static int wordIndexOfPhrase(String s, String p, int startWord, boolean ignoreCase) {

        WordEntry e = wordEntryOfPhrase(s, p, startWord, ignoreCase);

        if (e == null) {
            return -1;
        }

        return e.wordNum;
    }

    /**
     * Returns the number of the first word of the first occurrence of the
     * specified phrase within the string.
     * Words are sequences of characters separated by white space. White space is
     * any character for which java.lang.Character#isWhitespace() returns true.
     *
     * @param s         the input string
     * @param p         the phrase to be searched for
     * @param startWord the word number where the search is to start. The
     *                  first word has number 0.
     * @return the number of the first word of the first occurrence of the
     *         specified phrase within the string.
     */
    public static int wordIndexOfPhrase(String s, String p, int startWord) {
        return wordIndexOfPhrase(s, p, startWord, false);
    }

    /**
     * Returns a substring of the string which starts at a specified word and
     * contains the specified number of words.
     * Words are sequences of characters separated by white space. White space is
     * any character for which java.lang.Character#isWhitespace() returns true.
     *
     * @param s         the input string
     * @param startWord the word number of the first word
     * @param numWords  the number of words to be returned
     * @return a substring of the input string containing the specified number
     *         of words.
     */
    public static String subWords(String s, int startWord, int numWords) {

        int ls = s.length();

        if ((startWord < 0) || (numWords <= 0)) {
            return "";
        }

        int startPos = indexOfWord(s, startWord);

        if (startPos >= 0) {
            int lastPos = indexOfWord(s.substring(startPos), numWords - 1);

            if (lastPos >= 0) {

                // search for the end of the word
                int pos = startPos + lastPos;

                while ((pos < ls) && !Character.isWhitespace(s.charAt(pos))) {
                    pos++;
                }

                return s.substring(startPos, pos);
            } else {
                return stripTrailingBlanks(s.substring(startPos));
            }
        }

        return "";
    }

    /**
     * Return an array of the constituent words of a string.
     * Words are bounded byte sequences of whitespace.
     * Whitespace characters are those for which
     * <code>java.lang.Character#isWhitespace(char)</code> returns
     * <code>true</code>.
     * <p/>
     * (Contributed by Eric Jablow)
     *
     * @param s The string to split into its constituent words.
     * @return Array of the constituent words of <var>s</var>.
     */
    public static String[] split(String s) {
        return split(s, Integer.MAX_VALUE);
    }

    /**
     * Return an array of (at most) the first <var>limit</var>
     * constituent words of a string.
     * Words are bounded byte sequences of whitespace.
     * Whitespace characters are those for which
     * <code>java.lang.Character#isWhitespace(char)</code> returns
     * <code>true</code>.
     * <p/>
     * (Contributed by Eric Jablow)
     *
     * @param s     The string to split into its constituent words.
     * @param limit The maximum number of words to return.
     * @return Array of the constituent words of <var>s</var>.
     */
    public static String[] split(String s, int limit) {

        String[] splitWords = new String[Math.min(limit, numWords(s))];

        for (int i = 0; i < splitWords.length; ++i) {
            splitWords[i] = subWords(s, i, 1);
        }

        return splitWords;
    }

    /**
     * Returns true if the string represents the word "true".
     *
     * @param s the input string
     * @return true if the string contains the characters "true" (ignoring case).
     * @see Boolean#valueOf(String).
     */
    public static boolean toBoolean(String s) {
        return (Boolean.valueOf(s)).booleanValue();
    }

    /**
     * If the string represents a valid byte value, returns this value as a
     * byte.
     *
     * @param s the input string
     * @return the value of the string represented as a byte.
     * @throws NumberFormatException if the string cannot be represented
     *                               as a byte.
     * @see Byte#valueOf(String).
     */
    public static byte toByte(String s) throws NumberFormatException {
        return (Byte.valueOf(s)).byteValue();
    }

    /**
     * If the string represents a valid byte value, returns this value as a
     * byte.
     * You can specify the desired radix.
     *
     * @param s     the input string
     * @param radix the radix of the number
     * @return the value of the string represented as a byte.
     * @throws NumberFormatException if the string cannot be represented
     *                               as a byte.
     * @see Byte#valueOf(String,int).
     */
    public static byte toByte(String s, int radix) throws NumberFormatException {
        return (Byte.valueOf(s, radix)).byteValue();
    }

    /**
     * If the string represents a valid short number, returns this number as a
     * short value.
     * You can specify the desired radix.
     *
     * @param s     the input string
     * @param radix the radix of the number
     * @return the value of the string represented as a short.
     * @throws NumberFormatException if the string cannot be represented
     *                               as a short.
     * @see Short#valueOf(String).
     */
    public static short toShort(String s, int radix) throws NumberFormatException {
        return (Short.valueOf(s, radix)).shortValue();
    }

    /**
     * If the string represents a valid short number, returns this number as a
     * short value.
     *
     * @param s the input string
     * @return the value of the string represented as a short.
     * @throws NumberFormatException if the string cannot be represented
     *                               as a short.
     * @see Short#valueOf(String,int).
     */
    public static short toShort(String s) throws NumberFormatException {
        return (Short.valueOf(s)).shortValue();
    }

    /**
     * If the string represents a valid integer number, returns this number as an
     * int value.
     *
     * @param s the input string
     * @return the value of the string represented as an int.
     * @throws NumberFormatException if the string cannot be represented
     *                               as an int.
     * @see Integer#valueOf(String).
     */
    public static int toInt(String s) throws NumberFormatException {
        return (Integer.valueOf(s)).intValue();
    }

    /**
     * If the string represents a valid integer number, returns this number as an
     * int value.
     * You can specify the desired radix.
     *
     * @param s     the input string
     * @param radix the radix of the number
     * @return the value of the string represented as an int.
     * @throws NumberFormatException if the string cannot be represented
     *                               as an int.
     * @see Integer#valueOf(String,int).
     */
    public static int toInt(String s, int radix) throws NumberFormatException {
        return (Integer.valueOf(s, radix)).intValue();
    }

    /**
     * If the string represents a valid long integer, returns this number as a
     * long value.
     *
     * @param s the input string
     * @return the value of the string represented as a long.
     * @throws NumberFormatException if the string cannot be represented
     *                               as a long.
     * @see Long#valueOf(String).
     */
    public static long toLong(String s) throws NumberFormatException {
        return (Long.valueOf(s)).longValue();
    }

    /**
     * If the string represents a valid long integer, returns this number as a
     * long value.
     * You can specify the desired radix.
     *
     * @param s     the input string
     * @param radix the radix of the number
     * @return the value of the string represented as a long.
     * @throws NumberFormatException if the string cannot be represented
     *                               as a long.
     * @see Long#valueOf(String,int).
     */
    public static long toLong(String s, int radix) throws NumberFormatException {
        return (Long.valueOf(s, radix)).longValue();
    }

    /**
     * If the string represents a valid float number, returns this number as a
     * float value.
     *
     * @param s the input string
     * @return the value of the string represented as a float.
     * @throws NumberFormatException if the string cannot be represented
     *                               as a float.
     * @see Float#valueOf(String).
     */
    public static float toFloat(String s) throws NumberFormatException {
        return (Float.valueOf(s)).floatValue();
    }

    /**
     * If the string represents a valid double number, returns this number as a
     * double value.
     *
     * @param s the input string
     * @return the value of the string represented as a double.
     * @throws NumberFormatException if the string cannot be represented
     *                               as a double.
     * @see Double#valueOf(String).
     */
    public static double toDouble(String s) throws NumberFormatException {
        return (Double.valueOf(s)).doubleValue();
    }

    /**
     * Returns true if the string consists of digits.
     *
     * @param s the input string
     * @return true if the string consists only of digits.
     * @see Character#isDigit(char).
     */
    public static boolean isDigit(String s) {

        int ls = s.length();

        for (int i = 0; i < ls; i++) {
            if (!Character.isDigit(s.charAt(i))) {
                return false;
            }
        }

        return true;
    }

    /**
     * Returns true if the string consists of ignorable characters in a Java
     * identifier or a Unicode identifier.
     *
     * @param s the input string
     * @return true if the string consists only of ignorable characters in a Java
     *         identifier or a Unicode identifier.
     * @see Character#isIdentifierIgnorable(char).
     */
    public static boolean isIdentifierIgnorable(String s) {

        int ls = s.length();

        for (int i = 0; i < ls; i++) {
            if (!Character.isIdentifierIgnorable(s.charAt(i))) {
                return false;
            }
        }

        return true;
    }

    /**
     * Returns true if the string consists of ISO control characters.
     *
     * @param s the input string
     * @return true if the string consists only of ISO control characters.
     * @see Character#isISOControls(char).
     */
    public static boolean isISOControl(String s) {

        int ls = s.length();

        for (int i = 0; i < ls; i++) {
            if (!Character.isISOControl(s.charAt(i))) {
                return false;
            }
        }

        return true;
    }

    /**
     * Returns true if the string is a valid Java identifier.
     *
     * @param s the input string
     * @return true if the string is a valid Java identifier.
     * @see Character#isJavaIdentifierStart(char)
     * @see Character#isJavaIdentifierPart(char).
     */
    public static boolean isJavaIdentifier(String s) {

        int ls = s.length();

        if ((ls == 0) || !Character.isJavaIdentifierStart(s.charAt(0))) {
            return false;
        }

        for (int i = 1; i < ls; i++) {
            if (!Character.isJavaIdentifierPart(s.charAt(i))) {
                return false;
            }
        }

        return true;
    }

    /**
     * Returns true if the string consists of letters.
     *
     * @param s the input string
     * @return true if the string consists only of letters.
     * @see Character#isLetter(char).
     */
    public static boolean isLetter(String s) {

        int ls = s.length();

        for (int i = 0; i < ls; i++) {
            if (!Character.isLetter(s.charAt(i))) {
                return false;
            }
        }

        return true;
    }

    /**
     * Returns true if the string consists of letters or digits.
     *
     * @param s the input string
     * @return true if the string consists only of letters or digits.
     * @see Character#isLetterOrDigit(char).
     */
    public static boolean isLetterOrDigit(String s) {

        int ls = s.length();

        for (int i = 0; i < ls; i++) {
            if (!Character.isLetterOrDigit(s.charAt(i))) {
                return false;
            }
        }

        return true;
    }

    /**
     * Returns true if the string consists of lowercase characters.
     *
     * @param s the input string
     * @return true if the string consists only of lowercase characters.
     * @see Character#isLowercase(char).
     */
    public static boolean isLowerCase(String s) {

        int ls = s.length();

        for (int i = 0; i < ls; i++) {
            if (!Character.isLowerCase(s.charAt(i))) {
                return false;
            }
        }

        return true;
    }

    /**
     * Returns true if the string consists of Unicode space characters.
     *
     * @param s the input string
     * @return true if the string consists only of Unicode space characters.
     * @see Character#isSpaceChar(char).
     */
    public static boolean isSpaceChar(String s) {

        int ls = s.length();

        for (int i = 0; i < ls; i++) {
            if (!Character.isSpaceChar(s.charAt(i))) {
                return false;
            }
        }

        return true;
    }

    /**
     * Returns true if the string consists of title case characters.
     *
     * @param s the input string
     * @return true if the string consists only of Unicode title case characters.
     * @see Character#isTitleCase(char).
     */
    public static boolean isTitleCase(String s) {

        int ls = s.length();

        for (int i = 0; i < ls; i++) {
            if (!Character.isTitleCase(s.charAt(i))) {
                return false;
            }
        }

        return true;
    }

    /**
     * Returns true if the string is a valid Unicode identifier.
     *
     * @param s the input string
     * @return true if the string is a valid Unicode identifier.
     * @see Character#isJavaIdentifierStart(char)
     * @see Character#isJavaIdentifierPart(char)
     */
    public static boolean isUnicodeIdentifier(String s) {

        int ls = s.length();

        if ((ls == 0) || !Character.isUnicodeIdentifierStart(s.charAt(0))) {
            return false;
        }

        for (int i = 1; i < ls; i++) {
            if (!Character.isUnicodeIdentifierPart(s.charAt(i))) {
                return false;
            }
        }

        return true;
    }

    /**
     * Returns true if the string consists of uppercase characters.
     *
     * @param s the input string
     * @return true if the string consists only of uppercase characters.
     * @see Character#isUpperCase(char).
     */
    public static boolean isUpperCase(String s) {

        int ls = s.length();

        for (int i = 0; i < ls; i++) {
            if (!Character.isUpperCase(s.charAt(i))) {
                return false;
            }
        }

        return true;
    }

    /**
     * Returns true if the string consists of whitespace characters.
     *
     * @param s the input string
     * @return true if the string consists only of whitespace characters.
     * @see Character#isWhitespace(char).
     */
    public static boolean isWhitespace(String s) {

        int ls = s.length();

        for (int i = 0; i < ls; i++) {
            if (!Character.isWhitespace(s.charAt(i))) {
                return false;
            }
        }

        return true;
    }

    /*======================== private methods ================================*/
    private static Vector buildWordList(String s, int ls) {

        int start = 0;
        int end = 0;
        int words = 0;
        Vector wordList = new Vector();
        int pos = 0;

        while (pos < ls) {

            // search for the beginning of a word
            while ((pos < ls) && Character.isWhitespace(s.charAt(pos))) {
                pos++;
            }

            start = pos++;

            if (pos < ls) {

                // search for the end of the word
                while ((pos < ls) && !Character.isWhitespace(s.charAt(pos))) {
                    pos++;
                }

                end = pos++;

                WordEntry e = new WordEntry(start, end - start, words++);

                wordList.addElement(e);
            }
        }

        return wordList;
    }

    private static WordEntry wordEntryOfPhrase(String s, String p, int startWord,
                                               boolean ignoreCase) {

        int ls = s.length();
        int lp = p.length();

        if ((ls == 0) || (lp == 0)) {
            return null;
        }

        if (startWord < 0) {
            startWord = 0;
        }

        Vector sList = buildWordList(s, ls);
        Vector pList = buildWordList(p, lp);
        int ns = sList.size();
        int np = pList.size();
        int si = startWord;
        int pi = 0;
        int lastsi = startWord;
        int start = -1;

        while ((si < ns) && (pi < np)) {
            WordEntry es = (WordEntry) sList.elementAt(si);
            WordEntry ep = (WordEntry) pList.elementAt(pi);

            if (s.regionMatches(ignoreCase, es.start, p, ep.start, ep.len)) {
                si++;
                pi++;
            } else {
                si = ++lastsi;
                pi = 0;
            }
        }

        if (pi == np) {
            return (WordEntry) sList.elementAt(lastsi);
        }

        return null;
    }

    //-----------------------------------------------------------------------------//
    //------------------------------------------------------------------------------//
    //------------------------- END VIOLIN STRINGS ---------------------------------//
    //-----------------------------------------------------------------------------//
    //-----------------------------------------------------------------------------//

    /**
     * String related mthods from kiwi etc
     * Line separator for the OS we are operating on.
     * <p/>
     * Line separator for the OS we are operating on.
     * <p/>
     * Line separator for the OS we are operating on.
     * <p/>
     * Line separator for the OS we are operating on.
     * <p/>
     * Line separator for the OS we are operating on.
     * <p/>
     * Line separator for the OS we are operating on.
     * <p/>
     * Line separator for the OS we are operating on.
     */

    /**
     * Line separator for the OS we are operating on.
     */
    private static final String EOL = System.getProperty("line.separator");

    /**
     * Length of the line separator.
     */
    private static final int EOL_LENGTH = EOL.length();

    /**
     * to hex converter
     */
    private static final char[] toHex = {
            '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'
    };

    /**
     * Concatenates a list of objects as a String.
     *
     * @param list The list of objects to concatenate.
     * @return A text representation of the concatenated objects.
     */
    public String concat(List list) {

        StringBuffer sb = new StringBuffer();
        int size = list.size();

        for (int i = 0; i < size; i++) {
            sb.append(list.get(i).toString());
        }

        return sb.toString();
    }

    /**
     * Return a package name as a relative path name
     *
     * @param String package name to convert to a directory.
     * @return String directory path.
     */
    static public String getPackageAsPath(String pckge) {
        return pckge.replace('.', File.separator.charAt(0)) + File.separator;
    }

    /**
     * <p/>
     * Remove underscores from a string and replaces first
     * letters with capitals.  Other letters are changed to lower case.
     * </p>
     * <p/>
     * <p/>
     * For example <code>foo_bar</code> becomes <code>FooBar</code>
     * but <code>foo_barBar</code> becomes <code>FooBarbar</code>.
     * </p>
     *
     * @param data string to remove underscores from.
     * @return String
     * @deprecated Use the org.apache.commons.util.StringUtils class
     *             instead.  Using its firstLetterCaps() method in conjunction
     *             with a StringTokenizer will achieve the same result.
     */
    static public String removeUnderScores(String data) {

        String temp = null;
        StringBuffer out = new StringBuffer();

        temp = data;

        StringTokenizer st = new StringTokenizer(temp, "_");

        while (st.hasMoreTokens()) {
            String element = (String) st.nextElement();

            out.append(firstLetterCaps(element));
        }

        return out.toString();
    }

    /**
     * <p/>
     * 'Camels Hump' replacement of underscores.
     * </p>
     * <p/>
     * <p/>
     * Remove underscores from a string but leave the capitalization of the
     * other letters unchanged.
     * </p>
     * <p/>
     * <p/>
     * For example <code>foo_barBar</code> becomes <code>FooBarBar</code>.
     * </p>
     *
     * @param data string to hump
     * @return String
     */
    static public String removeAndHump(String data) {
        return removeAndHump(data, "_");
    }

    /**
     * <p/>
     * 'Camels Hump' replacement.
     * </p>
     * <p/>
     * <p/>
     * Remove one string from another string but leave the capitalization of the
     * other letters unchanged.
     * </p>
     * <p/>
     * <p/>
     * For example, removing "_" from <code>foo_barBar</code> becomes <code>FooBarBar</code>.
     * </p>
     *
     * @param data        string to hump
     * @param replaceThis string to be replaced
     * @return String
     */
    static public String removeAndHump(String data, String replaceThis) {

        String temp = null;
        StringBuffer out = new StringBuffer();

        temp = data;

        StringTokenizer st = new StringTokenizer(temp, replaceThis);

        while (st.hasMoreTokens()) {
            String element = (String) st.nextElement();

            out.append(capitalizeFirstLetter(element));
        }    //while

        return out.toString();
    }

    /**
     * <p/>
     * Makes the first letter caps and the rest lowercase.
     * </p>
     * <p/>
     * <p/>
     * For example <code>fooBar</code> becomes <code>Foobar</code>.
     * </p>
     *
     * @param data capitalize this
     * @return String
     */
    static public String firstLetterCaps(String data) {

        String firstLetter = data.substring(0, 1).toUpperCase();
        String restLetters = data.substring(1).toLowerCase();

        return firstLetter + restLetters;
    }

    /**
     * <p/>
     * Capitalize the first letter but leave the rest as they are.
     * </p>
     * <p/>
     * <p/>
     * For example <code>fooBar</code> becomes <code>FooBar</code>.
     * </p>
     *
     * @param data capitalize this
     * @return String
     */
    public static String capitalizeFirstLetter(String data) {

        String firstLetter = data.substring(0, 1).toUpperCase();
        String restLetters = data.substring(1);

        return firstLetter + restLetters;
    }

    /**
     * will work properly only for words delimited by a single space
     *
     * @param str
     * @return
     */
    public static String sentenceCase(String str) {

        StringBuffer buf = new StringBuffer();
        StringTokenizer tok = new StringTokenizer(str, " ");

        while (tok.hasMoreElements()) {
            buf.append(capitalizeFirstLetter(tok.nextToken())).append(' ');
        }

        String ret = buf.toString();

        ret = ret.trim();

        return ret;
    }

    /**
     * will work properly only for words delimited by a single space
     *
     * @param str
     * @return
     */
    public static StringBuffer sentenceCase(StringBuffer str) {

        StringBuffer buf = new StringBuffer();
        StringTokenizer tok = new StringTokenizer(str.toString(), " ");
        int cnt = tok.countTokens();

        for (int i = 0; i < cnt; i++) {
            buf.append(capitalizeFirstLetter(tok.nextToken()));

            if (i != cnt - 1) {
                buf.append(' ');
            }
        }

        return buf;
    }

    /**
     * Create a string array from a string separated by delim
     *
     * @param line  the line to split
     * @param delim the delimter to split by
     * @return a string array of the split fields
     */
    public static String[] split(String line, String delim) {

        List list = new ArrayList();
        StringTokenizer t = new StringTokenizer(line, delim);

        while (t.hasMoreTokens()) {
            list.add(t.nextToken());
        }

        return (String[]) list.toArray(new String[list.size()]);
    }

    /**
     * Chop i characters off the end of a string.
     * This method assumes that any EOL characters in String s
     * and the platform EOL will be the same.
     * A 2 character EOL will count as 1 character.
     *
     * @param string String to chop.
     * @param i      Number of characters to chop.
     * @return String with processed answer.
     */
    public static String chop(String s, int i) {
        return chop(s, i, EOL);
    }

    /**
     * Chop i characters off the end of a string.
     * A 2 character EOL will count as 1 character.
     *
     * @param string String to chop.
     * @param i      Number of characters to chop.
     * @param eol    A String representing the EOL (end of line).
     * @return String with processed answer.
     */
    public static String chop(String s, int i, String eol) {

        char[] sa = s.toCharArray();
        int length = sa.length;

        if (eol.length() == 2) {
            char eol1 = eol.charAt(0);
            char eol2 = eol.charAt(1);

            for (; i > 0; i--) {
                if ((sa[length - 1] == eol2) && (sa[length - 2] == eol1)) {
                    length -= 2;
                } else {
                    length--;
                }
            }
        } else {
            length -= i;
        }

        return new String(sa, 0, length);
    }

    /**
     * Perform a series of substitutions. The substitions
     * are performed by replacing $variable in the target
     * string with the value of provided by the key "variable"
     * in the provided hashtable.
     *
     * @param String    target string
     * @param Hashtable name/value pairs used for substitution
     * @return String target string with replacements.
     */
    public static StringBuffer stringSubstitution(String argStr, Hashtable vars) {

        StringBuffer argBuf = new StringBuffer();

        for (int cIdx = 0; cIdx < argStr.length();) {
            char ch = argStr.charAt(cIdx);

            switch (ch) {

                case'$':
                    StringBuffer nameBuf = new StringBuffer();

                    for (++cIdx; cIdx < argStr.length(); ++cIdx) {
                        ch = argStr.charAt(cIdx);

                        if ((ch == '_') || Character.isLetterOrDigit(ch)) {
                            nameBuf.append(ch);
                        } else {
                            break;
                        }
                    }

                    if (nameBuf.length() > 0) {
                        String value = (String) vars.get(nameBuf.toString());

                        if (value != null) {
                            argBuf.append(value);
                        }
                    }
                    break;

                default:
                    argBuf.append(ch);

                    ++cIdx;
                    break;
            }
        }

        return argBuf;
    }

    /**
     * Read the contents of a file and place them in
     * a string object.
     *
     * @param String path to file.
     * @return String contents of the file.
     */
    public static String fileContentsToString(String file) {

        String contents = "";
        File f = new File(file);

        if (f.exists()) {
            try {
                FileReader fr = new FileReader(f);
                char[] template = new char[(int) f.length()];

                fr.read(template);

                contents = new String(template);
            } catch (Exception e) {
                System.out.println(e);
                e.printStackTrace();
            }
        }

        return contents;
    }

    /**
     * Remove/collapse multiple newline characters.
     *
     * @param String string to collapse newlines in.
     * @return String
     */
    public static String collapseNewlines(String argStr) {

        if (argStr == null) {
            return argStr;
        }

        if (argStr.length() == 0) {
            return argStr;
        }

        char last = argStr.charAt(0);
        StringBuffer argBuf = new StringBuffer();

        for (int cIdx = 0; cIdx < argStr.length(); cIdx++) {
            char ch = argStr.charAt(cIdx);

            if ((ch != '\n') || (last != '\n')) {
                argBuf.append(ch);

                last = ch;
            }
        }

        return argBuf.toString();
    }

    /*
    static String noNewLine(String s) {
    char ch[] = s.toCharArray[];
    int n = ch.length, pos = 0;
    for (int i = 0; i < n; i++) {
        switch (ch[i]) {
        default: ch[pos++] = ch[i];
        case '\n': case '\r': break;
        }
    } return new String(ch, 0, pos);
}
*/

    /**
     * Remove/collapse multiple spaces.
     *
     * @param String string to remove multiple spaces from.
     * @return String
     */
    public static String collapseSpaces(String argStr) {

        char last = argStr.charAt(0);
        StringBuffer argBuf = new StringBuffer();

        for (int cIdx = 0; cIdx < argStr.length(); cIdx++) {
            char ch = argStr.charAt(cIdx);

            if ((ch != ' ') || (last != ' ')) {
                argBuf.append(ch);

                last = ch;
            }
        }

        return argBuf.toString();
    }

    /**
     * Replaces all instances of oldString with newString in line.
     * Taken from the Jive forum package.
     *
     * @param String original string.
     * @param String string in line to replace.
     * @param String replace oldString with this.
     * @return String string with replacements.
     */
    public static final String sub(String line, String oldString, String newString) {

        int i = 0;

        if ((i = line.indexOf(oldString, i)) >= 0) {
            char[] line2 = line.toCharArray();
            char[] newString2 = newString.toCharArray();
            int oLength = oldString.length();
            StringBuffer buf = new StringBuffer(line2.length);

            buf.append(line2, 0, i).append(newString2);

            i += oLength;

            int j = i;

            while ((i = line.indexOf(oldString, i)) > 0) {
                buf.append(line2, j, i - j).append(newString2);

                i += oLength;
                j = i;
            }

            buf.append(line2, j, line2.length - j);

            return buf.toString();
        }

        return line;
    }

    /**
     * Returns the output of printStackTrace as a String.
     *
     * @param e A Throwable.
     * @return A String.
     */
    public static final String stackTrace(Throwable e) {

        String foo = null;

        try {

            // And show the Error Screen.
            ByteArrayOutputStream ostr = new ByteArrayOutputStream();

            e.printStackTrace(new PrintWriter(ostr, true));

            foo = ostr.toString();
        } catch (Exception f) {

            // Do nothing.
        }

        return foo;
    }

    /**
     * Return a context-relative path, beginning with a "/", that represents
     * the canonical version of the specified path after ".." and "." elements
     * are resolved out.  If the specified path attempts to go outside the
     * boundaries of the current context (i.e. too many ".." path elements
     * are present), return <code>null</code> instead.
     *
     * @param path Path to be normalized
     * @return String normalized path
     */
    public static final String normalizePath(String path) {

        // Normalize the slashes and add leading slash if necessary
        String normalized = path;

        if (normalized.indexOf('\\') >= 0) {
            normalized = normalized.replace('\\', '/');
        }

        if (!normalized.startsWith("/")) {
            normalized = "/" + normalized;
        }

        // Resolve occurrences of "//" in the normalized path
        while (true) {
            int index = normalized.indexOf("//");

            if (index < 0) {
                break;
            }

            normalized = normalized.substring(0, index) + normalized.substring(index + 1);
        }

        // Resolve occurrences of "%20" in the normalized path
        while (true) {
            int index = normalized.indexOf("%20");

            if (index < 0) {
                break;
            }

            normalized = normalized.substring(0, index) + " " + normalized.substring(index + 3);
        }

        // Resolve occurrences of "/./" in the normalized path
        while (true) {
            int index = normalized.indexOf("/./");

            if (index < 0) {
                break;
            }

            normalized = normalized.substring(0, index) + normalized.substring(index + 2);
        }

        // Resolve occurrences of "/../" in the normalized path
        while (true) {
            int index = normalized.indexOf("/../");

            if (index < 0) {
                break;
            }

            if (index == 0) {
                return (null);    // Trying to go outside our context
            }

            int index2 = normalized.lastIndexOf('/', index - 1);

            normalized = normalized.substring(0, index2) + normalized.substring(index + 3);
        }

        // Return the normalized path that we have completed
        return (normalized);
    }

    /**
     * If state is true then return the trueString, else
     * return the falseString.
     *
     * @param boolean
     * @param String  trueString
     * @param String  falseString
     */
    public String select(boolean state, String trueString, String falseString) {

        if (state) {
            return trueString;
        } else {
            return falseString;
        }
    }

    /**
     * Check to see if all the string objects passed
     * in are empty.
     *
     * @param list A list of {@link String} objects.
     * @return Whether all strings are empty.
     */
    public boolean allEmpty(List list) {

        int size = list.size();

        for (int i = 0; i < size; i++) {
            if ((list.get(i) != null) && (list.get(i).toString().length() > 0)) {
                return false;
            }
        }

        return true;
    }

    /**
     * Pad the beginning of the given String with spaces until
     * the String is of the given length.
     *
     * @param s      String to be padded.
     * @param length desired length of result.
     * @return padded String.
     * @throws NullPointerException if s is null.
     */
    public static String prepad(String s, int length) {
        return prepad(s, length, ' ');
    }

    /**
     * Prepend the given character to the String until
     * the result is the desired length.
     * <p/>
     * If a String is longer than the desired length,
     * it will not be trunkated, however no padding
     * will be added.
     *
     * @param s      String to be padded.
     * @param length desired length of result.
     * @param c      padding character.
     * @return padded String.
     * @throws NullPointerException if s is null.
     */
    public static String prepad(String s, int length, char c) {

        int needed = length - s.length();

        if (needed <= 0) {
            return s;
        }

        StringBuffer sb = new StringBuffer(length);

        for (int i = 0; i < needed; i++) {
            sb.append(c);
        }

        sb.append(s);

        return (sb.toString());
    }

    /**
     * Pad the end of the given String with spaces until
     * the String is of the given length.
     *
     * @param s      String to be padded.
     * @param length desired length of result.
     * @return padded String.
     * @throws NullPointerException if s is null.
     */
    public static String postpad(String s, int length) {
        return postpad(s, length, ' ');
    }

    /**
     * Append the given character to the String until
     * the result is  the desired length.
     * <p/>
     * If a String is longer than the desired length,
     * it will not be trunkated, however no padding
     * will be added.
     *
     * @param s      String to be padded.
     * @param length desired length of result.
     * @param c      padding character.
     * @return padded String.
     * @throws NullPointerException if s is null.
     */
    public static String postpad(String s, int length, char c) {

        int needed = length - s.length();

        if (needed <= 0) {
            return s;
        }

        StringBuffer sb = new StringBuffer(length);

        sb.append(s);

        for (int i = 0; i < needed; i++) {
            sb.append(c);
        }

        return (sb.toString());
    }

    /**
     * Split the given String into tokens.
     * <p/>
     * This method is meant to be similar to the split
     * function in other programming languages but it does
     * not use regular expressions.  Rather the String is
     * split on a single String literal.
     * <p/>
     * Unlike java.util.StringTokenizer which accepts
     * multiple character tokens as delimiters, the delimiter
     * here is a single String literal.
     * <p/>
     * Each null token is returned as an empty String.
     * Delimiters are never returned as tokens.
     * <p/>
     * If there is no delimiter because it is either empty or
     * null, the only element in the result is the original String.
     * <p/>
     * StringUtils.split2("1-2-3", "-");<br>
     * result: {"1", "2", "3"}<br>
     * StringUtils.split2("-1--2-", "-");<br>
     * result: {"", "1", ,"", "2", ""}<br>
     * StringUtils.split2("123", "");<br>
     * result: {"123"}<br>
     * StringUtils.split2("1-2---3----4", "--");<br>
     * result: {"1-2", "-3", "", "4"}<br>
     *
     * @param s         String to be split.
     * @param delimiter String literal on which to split.
     * @return an array of tokens.
     * @throws NullPointerException if s is null.
     */
    public static String[] split2(String s, String delimiter) {

        int delimiterLength;

        // the next statement has the side effect of throwing a null pointer
        // exception if s is null.
        int stringLength = s.length();

        if ((delimiter == null) || (delimiterLength = delimiter.length()) == 0) {

            // it is not inherently clear what to do if there is no delimiter
            // On one hand it would make sense to return each character because
            // the null String can be found between each pair of characters in
            // a String.  However, it can be found many times there and we don't
            // want to be returning multiple null tokens.
            // returning the whole String will be defined as the correct behavior
            // in this instance.
            return new String[]{s};
        }

        // a two pass solution is used because a one pass solution would
        // require the possible resizing and copying of memory structures
        // In the worst case it would have to be resized n times with each
        // resize having a O(n) copy leading to an O(n^2) algorithm.
        int count;
        int start;
        int end;

        // Scan s and count the tokens.
        count = 0;
        start = 0;

        while ((end = s.indexOf(delimiter, start)) != -1) {
            count++;

            start = end + delimiterLength;
        }

        count++;

        // allocate an array to return the tokens,
        // we now know how big it should be
        String[] result = new String[count];

        // Scan s again, but this time pick out the tokens
        count = 0;
        start = 0;

        while ((end = s.indexOf(delimiter, start)) != -1) {
            result[count] = (s.substring(start, end));

            count++;

            start = end + delimiterLength;
        }

        end = stringLength;
        result[count] = s.substring(start, end);

        return (result);
    }

    /**
     * Replace occurances of a substring.
     * <p/>
     * StringUtils.replace("1-2-3", "-", "|");<br>
     * result: "1|2|3"<br>
     * StringUtils.replace("-1--2-", "-", "|");<br>
     * result: "|1||2|"<br>
     * StringUtils.replace("123", "", "|");<br>
     * result: "123"<br>
     * StringUtils.replace("1-2---3----4", "--", "|");<br>
     * result: "1-2|-3||4"<br>
     * StringUtils.replace("1-2---3----4", "--", "---");<br>
     * result: "1-2----3------4"<br>
     *
     * @param s       String to be modified.
     * @param find    String to find.
     * @param replace String to replace.
     * @return a string with all the occurances of the string to find replaced.
     * @throws NullPointerException if s is null.
     */
    public static String replace(String s, String find, String replace) {

        int findLength;

        // the next statement has the side effect of throwing a null pointer
        // exception if s is null.
        int stringLength = s.length();

        if ((find == null) || (findLength = find.length()) == 0) {

            // If there is nothing to find, we won't try and find it.
            return s;
        }

        if (replace == null) {

            // a null string and an empty string are the same
            // for replacement purposes.
            replace = "";
        }

        int replaceLength = replace.length();

        // We need to figure out how long our resulting string will be.
        // This is required because without it, the possible resizing
        // and copying of memory structures could lead to an unacceptable runtime.
        // In the worst case it would have to be resized n times with each
        // resize having a O(n) copy leading to an O(n^2) algorithm.
        int length;

        if (findLength == replaceLength) {

            // special case in which we don't need to count the replacements
            // because the count falls out of the length formula.
            length = stringLength;
        } else {
            int count;
            int start;
            int end;

            // Scan s and count the number of times we find our target.
            count = 0;
            start = 0;

            while ((end = s.indexOf(find, start)) != -1) {
                count++;

                start = end + findLength;
            }

            if (count == 0) {

                // special case in which on first pass, we find there is nothing
                // to be replaced.  No need to do a second pass or create a string buffer.
                return s;
            }

            length = stringLength - (count * (replaceLength - findLength));
        }

        int start = 0;
        int end = s.indexOf(find, start);

        if (end == -1) {

            // nothing was found in the string to replace.
            // we can get this if the find and replace strings
            // are the same length because we didn't check before.
            // in this case, we will return the original string
            return s;
        }

        // it looks like we actually have something to replace
        // *sigh* allocate memory for it.
        StringBuffer sb = new StringBuffer(length);

        // Scan s and do the replacements
        while (end != -1) {
            sb.append(s.substring(start, end));
            sb.append(replace);

            start = end + findLength;
            end = s.indexOf(find, start);
        }

        end = stringLength;

        sb.append(s.substring(start, end));

        return (sb.toString());
    }

    /**
     * Replaces characters that may be confused by a HTML
     * parser with their equivalent character entity references.
     * <p/>
     * Any data that will appear as text on a accessions page should
     * be be escaped.  This is especially important for data
     * that comes from untrusted sources such as internet users.
     * A common mistake in CGI programming is to ask a user for
     * data and then put that data on a accessions page.  For example:<pre>
     * Server: What is your name?
     * User: &lt;b&gt;Joe&lt;b&gt;
     * Server: Hello <b>Joe</b>, Welcome</pre>
     * If the name is put on the page without checking that it doesn't
     * contain HTML code or without sanitizing that HTML code, the user
     * could reformat the page, insert scripts, and control the the
     * content on your webserver.
     * <p/>
     * This method will replace HTML characters such as &gt; with their
     * HTML entity reference (&amp;gt;) so that the html parser will
     * be sure to interpret them as plain text rather than HTML or script.
     * <p/>
     * This method should be used for both data to be displayed in text
     * in the html document, and data put in form elements. For example:<br>
     * <code>&lt;html&gt;&lt;body&gt;<i>This in not a &amp;lt;tag&amp;gt;
     * in HTML</i>&lt;/body&gt;&lt;/html&gt;</code><br>
     * and<br>
     * <code>&lt;form&gt;&lt;input type="hidden" name="date" value="<i>This data could
     * be &amp;quot;malicious&amp;quot;</i>"&gt;&lt;/form&gt;</code><br>
     * In the second example, the form data would be properly be resubmitted
     * to your cgi script in the URLEncoded format:<br>
     * <code><i>This data could be %22malicious%22</i></code>
     *
     * @param s String to be escaped
     * @return escaped String
     * @throws NullPointerException if s is null.
     */
    public static String escapeHTML(String s) {

        int length = s.length();
        int newLength = length;

        // first check for characters that might
        // be dangerous and calculate a length
        // of the string that has escapes.
        for (int i = 0; i < length; i++) {
            char c = s.charAt(i);

            switch (c) {

                case'\"':
                case'\'': {
                    newLength += 5;
                }
                break;

                case'&': {
                    newLength += 4;
                }
                break;

                case'<':
                case'>': {
                    newLength += 3;
                }
                break;
            }
        }

        if (length == newLength) {

            // nothing to escape in the string
            return s;
        }

        StringBuffer sb = new StringBuffer(newLength);

        for (int i = 0; i < length; i++) {
            char c = s.charAt(i);

            switch (c) {

                case'\"': {
                    sb.append("&quot;");
                }
                break;

                case'\'': {
                    sb.append("&apos;");
                }
                break;

                case'&': {
                    sb.append("&amp;");
                }
                break;

                case'<': {
                    sb.append("&lt;");
                }
                break;

                case'>': {
                    sb.append("&gt;");
                }
                break;

                default: {
                    sb.append(c);
                }
            }
        }

        return sb.toString();
    }

    /**
     * Replaces characters that may be confused by an SQL
     * parser with their equivalent escape characters.
     * <p/>
     * Any data that will be put in an SQL query should
     * be be escaped.  This is especially important for data
     * that comes from untrusted sources such as internet users.
     * <p/>
     * For example if you had the following SQL query:<br>
     * <code>"SELECT * FROM adresses WHERE name='" + name + "' AND private='N'"</code><br>
     * Without this function a user could give <code>" OR 1=1 OR ''='"</code>
     * as their name causing the query to be:<br>
     * <code>"SELECT * FROM adresses WHERE name='' OR 1=1 OR ''='' AND private='N'"</code><br>
     * which will give all adresses, including private ones.<br>
     * Correct usage would be:<br>
     * <code>"SELECT * FROM adresses WHERE name='" + StringUtils.escapeSQL(name) + "' AND private='N'"</code><br>
     * <p/>
     * Another way to avoid this problem is to use a PreparedStatement
     * with appropriate placeholders.
     *
     * @param s String to be escaped
     * @return escaped String
     * @throws NullPointerException if s is null.
     */
    public static String escapeSQL(String s) {

        int length = s.length();
        int newLength = length;

        // first check for characters that might
        // be dangerous and calculate a length
        // of the string that has escapes.
        for (int i = 0; i < length; i++) {
            char c = s.charAt(i);

            switch (c) {

                case'\\':
                case'\"':
                case'\'':
                case'0': {
                    newLength += 1;
                }
                break;
            }
        }

        if (length == newLength) {

            // nothing to escape in the string
            return s;
        }

        StringBuffer sb = new StringBuffer(newLength);

        for (int i = 0; i < length; i++) {
            char c = s.charAt(i);

            switch (c) {

                case'\\': {
                    sb.append("\\\"");
                }
                break;

                case'\"': {
                    sb.append("\\\"");
                }
                break;

                case'\'': {
                    sb.append("\\\'");
                }
                break;

                case'0': {
                    sb.append("\\0");
                }
                break;

                default: {
                    sb.append(c);
                }
            }
        }

        return sb.toString();
    }

    /**
     * convert an array of bytes to an hexadecimal string
     * <p/>
     * Code from: package org.w3c.util;
     * // StringUtils.java
     * // $Id: StringUtils.java,v 1.2 2000/08/16 21:37:58 ylafon Exp $
     * // (c) COPYRIGHT MIT, INRIA and Keio, 1999.
     * // Please first read the full copyright statement in file COPYRIGHT.html
     *
     * @param an array of bytes
     * @return a string
     */
    public static String toHexString(byte b[]) {

        int pos = 0;
        char[] c = new char[b.length * 2];

        for (int i = 0; i < b.length; i++) {
            c[pos++] = toHex[(b[i] >> 4) & 0x0F];
            c[pos++] = toHex[b[i] & 0x0f];
        }

        return new String(c);
    }

    public static String wrap(int maxApproxCharsPerLine, String longStr) {
        if (longStr == null || longStr.length() < maxApproxCharsPerLine) {
            return longStr;
        }

        StringTokenizer tok = new StringTokenizer(longStr, " ");
        StringBuffer buf = new StringBuffer();
        int preSize = 0;

        while (tok.hasMoreTokens()) {
            buf.append(tok.nextToken());
            if (buf.length() - preSize > maxApproxCharsPerLine) {
                buf.append('\n');
                preSize = buf.length();
            } else {
                buf.append(' ');
            }
        }

        return buf.toString();
    }

    /**
     * Left justify a string, wrapping words as necessary.
     * Code from The Kiwi Toolkit
     *
     * @param text The text to justify.
     * @param cols The number of columns to use.
     * @author frenzy@ix.netcom.com
     */
    public final static String justify(String text, int cols) {

        StringTokenizer st = new StringTokenizer(text, "\t \f\n", true);
        StringBuffer buf = new StringBuffer(500);
        int ww,
                lw = 0;
        boolean sawLF = false,
                first = true;

        while (st.hasMoreTokens()) {
            String tok = st.nextToken();

            if (tok.equals("\n")) {
                if (cols == 0) {
                    continue;
                }

                if (sawLF) {
                    buf.append('\n');
                    buf.append('\n');

                    lw = 0;
                    first = true;
                    sawLF = false;
                } else {
                    sawLF = true;
                }
            } else if ((tok.equals(" ")) || (tok.equals("\t")) || (tok.equals("\f"))) {
                sawLF = false;

                continue;
            } else {
                sawLF = false;
                ww = tok.length();

                if (!first) {
                    ww++;
                }

                if ((lw + ww) > cols) {
                    buf.append('\n');

                    first = true;
                    lw = 0;
                }

                if (!first) {
                    buf.append(' ');
                }

                buf.append(tok);

                lw += ww;
                first = false;
            }
        }

        String r = buf.toString();

        buf = null;

        return (r);
    }

    /**
     * Determine if a string consists solely of alphanumeric characters.
     * <p/>
     * Code from The Kiwi Toolkit
     *
     * @param text The string to test.
     * @return <b>true</b> if the string contains only alphanumeric characters,
     *         and <b>false</b> otherwise.
     * @author frenzy@ix.netcom.com
     */
    public final static boolean isAlphaNumeric(String text) {

        for (int i = 0; i < text.length(); i++) {
            if (!Character.isLetterOrDigit(text.charAt(i))) {
                return (false);
            }
        }

        return (true);
    }

    /**
     * Join an array of strings into a single string of tokens using the given
     * delimiter.
     * <p/>
     * Code from The Kiwi Toolkit
     *
     * @param array     The tokens to join.
     * @param delimiter A string to insert between adjacent tokens.
     * @return The resulting string
     * @author frenzy@ix.netcom.com
     */
    public final static String join(String array[], String delimiter) {

        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < array.length; i++) {
            if (i > 0) {
                sb.append(delimiter);
            }

            sb.append(array[i]);
        }

        return (sb.toString());
    }

    /**
     * Returns true if the given String contains at least one non alpha-
     * numeric character.
     *
     * @param str The String
     * @return True if one or more alpha-numeric characters are found
     */
    public static boolean hasNonAlphaNumericCharacters(String str) {

        char[] chars = str.toCharArray();

        for (int i = 0; i < chars.length; i++) {
            if (!Character.isLetterOrDigit(chars[i])) {
                return true;
            }
        }

        return false;
    }

    /**
     * Returns true if the given String contains at least one non alpha-
     * numeric character.  Characters in the ignore array are not checked.
     *
     * @param str    The String
     * @param ignore An array of characters to ignore
     * @return True if one or more alpha-numeric characters are found
     */
    public static boolean hasNonAlphaNumericCharacters(String str, char[] ignore) {

        char[] chars = str.toCharArray();

        for (int i = 0; i < chars.length; i++) {
            if (isIgnore(chars[i], ignore)) {
                continue;
            }

            if (!Character.isLetterOrDigit(chars[i])) {
                return true;
            }
        }

        return false;
    }

    /**
     * String all non alpha-numeric characters from the given String.
     *
     * @param str The String
     * @return The new String
     */
    public static String stripNonAlphaNumericCharacters(String str) {

        StringBuffer buffer = new StringBuffer();
        char[] chars = str.toCharArray();

        for (int i = 0; i < chars.length; i++) {
            if (Character.isLetterOrDigit(chars[i])) {
                buffer.append(chars[i]);
            }
        }

        return buffer.toString();
    }

    /**
     * Strip all non alpha-numeric characters from the given String.  Characters
     * contained in the ignore array will be left within the String.
     *
     * @param str    The String
     * @param ignore An array of characters to ignore
     * @return The new String
     */
    public static String stripNonAlphaNumericCharacters(String str, char[] ignore) {

        StringBuffer buffer = new StringBuffer();
        char[] chars = str.toCharArray();

        for (int i = 0; i < chars.length; i++) {
            if (isIgnore(chars[i], ignore)) {
                continue;
            }

            if (Character.isLetterOrDigit(chars[i])) {
                buffer.append(chars[i]);
            }
        }

        return buffer.toString();
    }

    /**
     * Return true if the character is an ASCII character.
     *
     * @param c The character
     * @return True if character is ASCII
     */
    public static boolean isAscii(char c) {
        return (c & 0xff80) == 0;
    }

    /**
     * Return true if the character is an extended ASCII character.
     *
     * @param c The character
     * @return True if character is extended ASCII
     */
    public static boolean isExtendedAscii(char c) {
        return (c & 0xff00) == 0;
    }

    /**
     * Return true if the given character is a member of the ignore array.
     *
     * @param c      The character
     * @param ignore An array of characters to ignore
     * @return True if the character is in the ignore array
     */
    private static boolean isIgnore(char c, char[] ignore) {

        for (int i = 0; i < ignore.length; i++) {
            if (c == ignore[i]) {
                return true;
            }
        }

        return false;
    }

    public static int findNumCommon(String[] a, String[] b) {
        Set set = new HashSet(a.length);
        for (int i = 0; i < a.length; i++) {
            set.add(a[i]);
        }

        int num = 0;

        for (int i = 0; i < b.length; i++) {
            if (set.contains(b[i])) {
                num++;
            }
        }

        return num;

    }

}    // End Strings

/**
 * Class WordEntry
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
class WordEntry {

    public final int start;
    public final int len;
    public final int wordNum;

    public WordEntry(int start, int len, int wordNum) {

        this.start = start;
        this.len = len;
        this.wordNum = wordNum;
    }
}

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
