/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.utils;


import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;


/**
 * Simple timer mechanism to help with performance benchmarking.
 * <p/>
 * Is designed for simple procedural use:
 * <p/>
 * <code>
 * timer.start();
 * // do some task(s)
 * timer.stop()
 * timer.timeTaken() // prints to stdout teh time taken
 * <p/>
 * </code>
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 * @todo See if there is an open-source junit-like class for this.
 * There is one junitperf but its more geared towards stress tests
 * rather than simple timing. The hunt continues.
 */
public class Timer {

    protected static final Logger klog = XLogger.getLogger(Timer.class);
    private long fStart = 0;
    private long fStop = 0;
    private boolean stopped = false;

    public Timer() {
    }

    /**
     * Starts the timer
     */
    public void start() {
        fStart = System.currentTimeMillis();
        stopped = false;
    }

    /**
     * Stops the timer
     */
    public void stop() {
        fStop = System.currentTimeMillis();
        stopped = true;
    }

    public void printTimeTaken() {
        klog.info(_timeTaken(null));
    }

    public void printTimeTakenS() {
        System.out.println(_timeTaken(null));
    }

    public void printTimeTaken_ms() {
        System.out.println(fStop - fStart);
    }

    public void printTimeTaken_ms(String label) {
        System.out.println(label + " " + (fStop - fStart));
    }

    public void printTimeTaken(String label) {
        klog.info(_timeTaken(label));
    }

    public void printTimeTakenS(String label) {
        System.out.println(_timeTaken(label));
    }

    private String _timeTaken(String label) {
        //if (fStop < fStart) log.warn("Looks like the timer was not stop()'ed");
        if (!stopped) {
            stop();
        }

        long t = fStop - fStart;

        if (label != null) {
            if (t > 1000) {
                return ("Time taken for " + label + ": " + t / 1000 + " secs");
            } else {
                return ("Time taken for " + label + ": " + t + " ms");
            }
        } else {
            if (t > 1000) {
                return ("Time taken: " + t / 1000 + " secs");
            } else {
                return ("Time taken: " + t + " ms");
            }
        }
    }

    public long getTimeTaken() {
        return fStop - fStart;
    }

    public long getStartTime() {
        return fStart;
    }

    public long getStopTime() {
        return fStop;
    }

}    // End Timer

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
