/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.utils;


import edu.mit.broad.genome.XLogger;
import org.apache.log4j.Logger;

import java.io.*;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;


/**
 * Recursively list files, and make a zip file.
 *
 * @author Mark Nelson (see credits)
 * @author Aravind Subramanian - for xomics
 * @version %I%, %G%
 * @credits Code from
 * Mark Nelson
 * markn@tiny.com
 * http://web2.airmail.net/markn
 * and
 * Mike Levin
 * http://www.mike-levin.com/learning-java/recursion-zipoutputstream.html
 */
public class Zipper {

    private static final Logger klog = XLogger.getLogger(Zipper.class);
    private ZipOutputStream fZos;
    private File fSource;
    private int fCnt;

    /**
     * Class Constructor.
     * Initializes files and does simple sanity dbc checks.
     * <p/>
     * aratget can be file or dir
     * zipped produces a single zipped file - typicaly give a file
     * withe extension .zip
     * <p/>
     * note that the files are made in the archive relative to the
     * top-level - to-be-zipper dir
     */
    public Zipper() {
    }

    /**
     * target can be file or dir
     * zipped produces a single zipped file - typicaly give a file
     * withe extension .zip
     * <p/>
     * note that the files are made in the archive relative to the
     * top-level - to-be-zipper dir
     */
    public void zipIt(final File source_to_be_zipped, final File zipped_file) {

        try {
            klog.info("Zipping: " + source_to_be_zipped.getName() + " to " + zipped_file.getName());

            this.fSource = source_to_be_zipped;
            this.fCnt = 0;

            // Create the file output streams for both the file and the zip.
            FileOutputStream fos = new FileOutputStream(zipped_file);

            this.fZos = new ZipOutputStream(fos);

            dirFunc(fSource);

            // Close the file output streams for both the file and the zip.
            fZos.flush();
            fZos.close();
            fZos.close();

            StringBuffer sb = new StringBuffer("Zipped folder\n");

            sb.append("Zip source: ").append(source_to_be_zipped.getPath()).append('\n');
            sb.append("Zipped to : ").append(zipped_file.getPath()).append('\n');
            sb.append("Number of files = " + fCnt).append('\n');

        } catch (Throwable t) {
            klog.error("Zipping error", t);
        }
    }

    /**
     * @retryns number of files included in the zipped archive
     */
    private void dirFunc(final File dirObj) throws java.io.IOException {

        if (dirObj.exists() == true) {
            if (dirObj.isDirectory() == true) {
                File[] fileList = dirObj.listFiles();

                for (int i = 0; i < fileList.length; i++) {
                    if (fileList[i].isDirectory()) {
                        dirFunc(fileList[i]);
                    } else if (fileList[i].isFile()) {
                        add(fileList[i]);
                    }
                }
            } else {
                System.out.println(dirObj + " is not a directory.");
            }
        } else {
            klog.warn("Directory " + dirObj + " does not exist.");
        }
    }

    int kcnt = 0;

    private void add(final File file) throws IOException {

        FileInputStream fis = new FileInputStream(file.getPath());
        BufferedInputStream bis = new BufferedInputStream(fis);

        // Create a Zip Entry and put it into the archive (no data yet).
        // note that the files are made in the archive relative to the
        // top-level - to-be-zipper dir
        String name = file.getPath();

        name = name.substring(fSource.getPath().length());

        //log.info("Using name " + name + " for file " + file.getPath());
        ZipEntry fileEntry = new ZipEntry(name);

        //ZipEntry fileEntry = new ZipEntry(file.getName());
        fZos.putNextEntry(fileEntry);

        byte[] data = new byte[1024];
        int byteCount;

        // Create a loop that reads from the buffered input stream and writes
        // to the zip output stream until the bis has been entirely read.
        while ((byteCount = bis.read(data, 0, 1024)) > -1) {
            fZos.write(data, 0, byteCount);
        }

        if (kcnt % 5 == 0) {
            System.out.println("Added to zip archive file " + file + " count: " + kcnt);
        }

        kcnt++;

        //log.info("Added to zip archive file " + file + "\n");
    }
}    // End Zipper

// Alternative method:
/**
 * Adds a single file to a Zip archive, via a ZipOutputStream object.
 * It first has to do a couple of things that java.util.zip ought to do
 * but doesn't.  First, it strips any leading drive or path
 * components from the file, just as Zip or PKZIP will do.
 * Second, it changes any backslash characters to forward
 * slashes.  It then creates a new ZipEntry() object, and
 * calls the putNextEntry() method of ZipOutputStream, which
 * writes out most of the file info to the Zip file.
 * After that, the entire input file is copied to the
 * ZipOutputStream, which compresses it transparently whenever
 * we call the write() method.  While this is going on
 * pacifier strings are written to the screen.  Finally, after
 * the entire file has been written, the closeEntry() method
 * is called for ZipOutputStream.  This will take care of
 * writing the file size, compressed size, and CRC to the
 * Zip file, as well as adding data to the central directory.
 */
/*
protected void add( ZipOutputStream z, String file_name ) throws IOException {
    String entry_name = new String( file_name );

    if ( entry_name.charAt( 0 ) == File.separatorChar ) {
        entry_name = entry_name.substring( 1 );
    }

    ZipEntry entry = new ZipEntry( entry_name.replace( '\\', '/' ) );
    z.putNextEntry( entry );
    FileInputStream f = new FileInputStream( file_name );
    System.out.print( entry_name + " " );
    byte[] buf = new byte[ 10240 ];
    for ( int i = 0 ; ; i++ ) {
        int len = f.read( buf );
        if ( len < 0 )
            break;
        z.write( buf, 0, len );
        switch ( i & 3 ) {
            case 0  : System.out.print( "|\b" ); break;
            case 1  : System.out.print( "/\b" ); break;
            case 2  : System.out.print( "-\b" ); break;
            default : System.out.print( "\\\b" ); break;
        }
    }
    z.closeEntry();
    System.out.println( " " );
}
*/

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
