/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

package edu.mit.broad.genome.utils.containers;


/**
 * A mutable String.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class MutableString {

    /**
     * The current value.
     */
    protected String value;

    /**
     * Construct a new <code>StringHolder</code> with an initial value of
     * <b>null</b>.
     */
    public MutableString() {
        this(null);
    }

    /**
     * Construct a new <code>StringHolder</code> with a specified initial
     * value.
     *
     * @param value The initial value.
     */
    public MutableString(String value) {
        this.value = value;
    }

    /**
     * Set the <code>StringHolder</code>'s value.
     *
     * @param value The new value.
     */
    public synchronized final void setValue(String value) {
        this.value = value;
    }

    /**
     * Get the <code>StringHolder</code>'s value.
     *
     * @return The current value.
     */
    public synchronized final String getValue() {
        return (value);
    }
}    // End MutableString

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
