/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */
package edu.mit.broad.genome.viewers;


import edu.mit.broad.genome.Conf;
import edu.mit.broad.genome.JarResources;
import edu.mit.broad.genome.swing.GuiHelper;
import edu.mit.broad.genome.swing.MemoryMonitor;
import edu.mit.broad.genome.utils.SystemUtils;
import edu.mit.broad.xbench.core.Widget;
import edu.mit.broad.xbench.core.api.Application;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;


/**
 * About Viewer
 * <p/>
 * Simple info on system properties in a text fields
 * Display of memory consumption using the java2d example
 * A button to force GC
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class AboutViewer extends JPanel implements Widget {

    public static final String NAME = "About";
    public static final Icon ICON = JarResources.getIcon("About.gif");
    private JButton bForceGc;
    private MemoryMonitor fMonitor;

    /**
     * Class Constructor.
     */
    public AboutViewer() {
        this(createDefaultXomicsAppInfoPanel());
    }

    public AboutViewer(final JComponent appInfoPanel) {
        init(new String[]{"Application info"}, new JComponent[]{appInfoPanel});
    }

    public AboutViewer(final String[] names, final JComponent[] appInfoPanels) {
        init(names, appInfoPanels);
    }

    private void init(final String[] names, final JComponent[] appInfoPanels) {

        if (appInfoPanels == null) {
            throw new IllegalArgumentException("Param appInfoPanels cannot be null");
        }

        jbInit(names, appInfoPanels);

        bForceGc.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {

                System.gc();
                System.gc();
                System.gc();
                System.gc();
            }
        });
    }

    public String getTitle() {
        return "About";
    }

    private void jbInit(String[] names, JComponent[] appInfoPanels) {

        this.setLayout(new BorderLayout());
        JTabbedPane pane = new JTabbedPane(JTabbedPane.BOTTOM);

        for (int i = 0; i < names.length; i++) {
            pane.addTab(names[i], appInfoPanels[i]);
        }
        pane.addTab("System info", getSystemInfo());
        pane.addTab("Memory usage", getMemoryPanel());
        this.add(pane);
    }

    private static JComponent createDefaultXomicsAppInfoPanel() {

        StringBuffer buf = new StringBuffer("XOMICS development toolkit\n\n");
        buf.append("Debug mode (for development)  : ").append(Conf.isDebugMode()).append('\n');
        buf.append("Application: ").append(Application.getName()).append('\n');
        buf.append("\nMemory available on machine : ").append(Runtime.getRuntime().maxMemory() / 1000000).append("Mb\n");
        buf.append("(At least 256Mb of RAM is recommended. More the better!)\n");

        JTextArea ta = new JTextArea(buf.toString());
        return new JScrollPane(ta);
    }

    private JPanel getMemoryPanel() {
        JPanel memPanel = new JPanel(new BorderLayout());
        fMonitor = new MemoryMonitor();
        memPanel.add(fMonitor, BorderLayout.CENTER);
        fMonitor.surf.start();
        bForceGc = new JButton("Force memory cleanup", GuiHelper.ICON_DELETE24);

        bForceGc.setToolTipText("Make Java perform garbage collection -- use to free up memory when the application is sluggish");
        memPanel.add(bForceGc, BorderLayout.SOUTH);

        return memPanel;
    }

    private JComponent getSystemInfo() {

        JTable table = new JTable(SystemUtils.asTableModel());
        return new JScrollPane(table);
    }

    public boolean allowClose() {
        return true;
    }

    public JComponent getWindowUI() {
        return this;
    }

    public JMenuBar getJMenuBar() {
        return EMPTY_MENU_BAR;
    }

    public void finalize() throws Throwable {
        super.finalize();
        if (fMonitor != null) {
            fMonitor.surf.stop();    // not sure if its really needed?
        }
    }

    public Icon getIcon() {
        return ICON;
    }
}    // End AboutViewer

/*--- Formatted in Sun Java Convention Style on Fri, Sep 27, '02 ---*/
