package edu.mit.broad.genome.viewers;

/**
 * The Broad Institute
 * SOFTWARE COPYRIGHT NOTICE AGREEMENT
 * This software and its documentation are copyright 2004-2005 by the
 * Broad Institute/Massachusetts Institute of Technology.
 * All rights are reserved.
 * This software is supplied without any warranty or guaranteed support
 * whatsoever. Neither the Broad Institute nor MIT can be responsible for
 * its use, misuse, or functionality.
 */

import com.jidesoft.grid.SortableTable;
import edu.mit.broad.genome.JarResources;
import edu.mit.broad.vdb.chip.Chip;
import edu.mit.broad.xbench.actions.ObjectAction;
import edu.mit.broad.xbench.core.Widget;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import java.awt.*;


/**
 * ChipViewer viewer.
 *
 * @author Aravind Subramanian
 * @version %I%, %G%
 */
public class ChipViewer extends AbstractViewer {

    public static final Icon ICON = JarResources.getIcon("Chip16.png");
    public static final String NAME = "ChipViewer";

    private Chip fChip;

    /**
     * @param gset
     */
    public ChipViewer(final Chip chip) {
        super(NAME, ICON, chip);

        this.fChip = chip;

        jbInit();
    }

    private void jbInit() {

        Model model = new Model(fChip);
        SortableTable table = createTable(model, true, true);

        //TableCellRenderer cellrend = new ColoredBgCellRenderer();
        //table.setDefaultRenderer(Object.class, cellrend);

        setColumnSize(100, 1, table, false);
        this.setLayout(new BorderLayout());
        this.add(createExcelScrollPane(table, fChip), BorderLayout.CENTER);
        this.revalidate();

    }

    public JMenuBar getJMenuBar() {
        return EMPTY_MENU_BAR;
    }

    /**
     * Inner class representing an action for the viewer\
     */
    public static class Action extends ObjectAction {

        private Chip fChip;

        /**
         * Class constructor
         */
        public Action() {

            this.putValue(Action.NAME, getName());
            this.putValue(Action.SMALL_ICON, getIcon());
            this.putValue(Action.SHORT_DESCRIPTION, getDescription());
        }

        /**
         * Class constructor
         *
         * @param chip
         */
        public Action(Chip chip) {
            this();
            this.fChip = chip;
        }

        public void setObject(Object obj) {
            this.fChip = (Chip) obj;
        }

        public Widget getWidget() {
            return new ChipViewer(fChip);
        }

        public String getId() {
            return "ChipViewerAction";
        }

        public String getName() {
            return getId();
        }

        public Icon getIcon() {
            return ICON;
        }

        public String getDescription() {
            return "View the chip annotation";
        }

    }    // End Action


    /**
     * An implementation of AbstractTableModel for ChipViewer.
     *
     * @author Aravind Subramanian
     * @version %I%, %G%
     */
    public static class Model extends AbstractTableModel {

        /**
         * The underlying object being modell'ed
         */
        private final Chip fChip;

        private static String[] COL_NAMES = new String[]{"FEATURE", "SYMBOL", "TITLE"};

        /**
         * Class Constructor.
         * Initializes model to specified Template.
         */
        public Model(Chip chip) {
            this.fChip = chip;
        }

        /**
         * Always kColNames.length
         */
        public int getColumnCount() {
            return COL_NAMES.length;
        }

        /**
         * As many rows as there are elements
         */
        public int getRowCount() {
            try {
                return fChip.getNumProbes();
            } catch (Exception e) {
                e.printStackTrace();
                return 0;
            }
        }

        public String getColumnName(int col) {
            return COL_NAMES[col];
        }

        private boolean once = false;

        public Object getValueAt(int row, int col) {

            try {
                String probeName = fChip.getProbeName(row);
                if (col == 0) {
                    return probeName;
                } else if (col == 1) {
                    return fChip.getSymbol(probeName, Chip.OMIT_NULLS);
                } else {

                    return fChip.getTitle(probeName, Chip.OMIT_NULLS);
                }
            } catch (Exception e) {
                if (!once) {
                    e.printStackTrace();
                    once = true;
                }
            }

            return null;
        }

        public Class getColumnClass(int col) {
            return String.class;
        }

        public boolean isEditable() {
            return false;
        }

        public boolean isCellEditable(int row, int col) {
            return false;
        }

    }    // End Inner class Model


}        // End ChipViewer

/*--- Formatted in Sun Java Convention Style on Thu, Oct 17, '02 ---*/
